/*
 * MINIGUI - Harbour Win32 GUI library Demo
 *
 * eztw32.dll is public domain from
 * http://www.dosadi.com/eztwain1.htm
 *
 * Twerp 2.0   27 Aril 2006 by Spike <spike@dosadi.com>
 *
 * Copyright 2005 Walter Formigoni <walter.formigoni@uol.com.br>
 *
 * Copyright 2005-2007 Grigory Filatov <gfilatov@freemail.ru>
*/

ANNOUNCE RDDSYS

#ifdef __XHARBOUR__
  #define __CALLDLL__
#endif

#include "minigui.ch"

#define TW_APP_ICO		1001

#define TWAIN_BW		0x0001	// 1-bit per pixel, B&W		(== TWPT_BW)
#define TWAIN_GRAY		0x0002	// 1,4, or 8-bit grayscale	(== TWPT_GRAY)
#define TWAIN_RGB		0x0004	// 24-bit RGB color		(== TWPT_RGB)
#define TWAIN_PALETTE		0x0008	// 1,4, or 8-bit palette	(== TWPT_PALETTE)
#define TWAIN_ANYTYPE		0x0000	// any of the above

Static hWnd, hpal, hdib, wPixTypes := TWAIN_ANYTYPE, fHideUI := 0

Procedure Main

	SET MULTIPLE OFF

	DEFINE WINDOW Win_1 ;
		AT 0,0 ;
		WIDTH 500 ;
		HEIGHT 350 ;
		TITLE 'Twerp - EZTwain sample app by Spike' ;
		ICON TW_APP_ICO ;
		MAIN ;
		ON INIT hWnd := GetFormHandle ('Win_1') ;
		ON RELEASE DiscardImage() ;
		ON PAINT OnPaint() ;
		BACKCOLOR GRAY

		DEFINE MAIN MENU
			DEFINE POPUP '&File'
				menuitem '&Select Source...' action TWAIN_SelectImageSource( hWnd )
				menuitem '&Acquire...'  action Acquire()
				menuitem 'Acquire to &File...'  action AcquireToFilename()
				menuitem 'Acquire to &Clipboard...' action AcquireToClipboard()
				menuitem 'Save A&s...' action SaveAs() NAME TW_APP_SAVEAS
				menuitem '&Open...' action OnOpen()
				separator
				menuitem 'E&xit' action ThisWindow.Release
			END POPUP
			DEFINE POPUP '&Options'
				MENUITEM "&All PixelTypes" action SetTypes() NAME TW_APP_ANYPIX CHECKED
				MENUITEM "&B&&W" action SetTypes() NAME TW_APP_BW
				MENUITEM "&Grayscale" action SetTypes() NAME TW_APP_GRAYSCALE
				MENUITEM "&RGB Color" action SetTypes() NAME TW_APP_RGB
				MENUITEM "&Palette Color" action SetTypes() NAME TW_APP_PALETTE
				SEPARATOR
				MENUITEM "&Show Source UI" action SetTypes() NAME TW_APP_SHOWUI CHECKED
				MENUITEM "&Hide Source UI" action SetTypes() NAME TW_APP_HIDEUI
			END POPUP
			DEFINE POPUP '&Help'
				menuitem '&About Twerp...' action ;
					MsgInfo ( "MiniGUI Sample Application for EZTwain" + CRLF + ;
					"Based upon a contribution by Walter Formigoni" + CRLF + ;
					"Written by Grigory Filatov (April, 2005)" + CRLF + CRLF + ;
					"EZTwain Dll reports version " + Ltrim(Str(TWAIN_EasyVersion() / 100)) + CRLF + ;
					"TWAIN Services: " + IIf(EMPTY(TWAIN_IsAvailable()), "Not Available", "Available"), "About Twerp", TW_APP_ICO, .f. )
			END POPUP
		END MENU

	END WINDOW

	Win_1.TW_APP_SAVEAS.Enabled := .F.

	CENTER WINDOW Win_1

	ACTIVATE WINDOW Win_1

Return


*************************************
Function Acquire()

	// free up current image and palette if any
	DiscardImage()
	InvalidateRect( _HMG_MainHandle, 1 )

	TWAIN_SetHideUI(fHideUI)

//	if TWAIN_OpenDefaultSource() > 0
		hdib = TWAIN_AcquireNative(hWnd, wPixTypes)
		if !EMPTY(hdib)
			// compute or guess a palette to use for display
			hpal = TWAIN_CreateDibPalette(hdib)

			// size the window to just contain the image
			ResizeWindow()
		endif
//	else
//		MsgStop( "Unable to open default Data Source.", "Error" )
//	endif

return nil
**********

*************************************
Function AcquireToFilename()
Local nResult := 1
Local cFilename := Putfile( { {'Windows Bitmaps (*.bmp)', '*.bmp'}, {'Jpg Files (*.jpg)', '*.jpg'}  }, , ;
			GetStartUpFolder(), , @nResult )

	if !EMPTY(cFilename)
		cFilename := cFilePath(cFilename) + "\" + cFileNoExt(cFilename) + ".bmp"

		// free up current image and palette if any
		DiscardImage()
		InvalidateRect( _HMG_MainHandle, 1 )

		if TWAIN_AcquireToFilename( hWnd, cFilename ) == 0
			hdib = TWAIN_LoadNativeFromFilename( cFilename )
			if !EMPTY(hdib)
				// compute or guess a palette to use for display
				hpal = TWAIN_CreateDibPalette(hdib)

				// size the window to just contain the image
				ResizeWindow()

				if nResult == 2
					Bmp2Jpg( cFilename, cFilePath(cFilename) + "\" + cFileNoExt(cFilename) + ".jpg" )
					Ferase( cFilename )
				endif
			endif
		else
			MsgStop( "No image was acquired or transfer to the file failed.", "Error", , .f. )
		endif
	endif

return nil
**********

**********************************
function AcquireToClipboard()

	if TWAIN_AcquireToClipboard( hWnd, TWAIN_ANYTYPE) == 0
		MsgStop( "No image was acquired or transfer to the clipboard failed.", "Error", , .f. )
	else
		MsgInfo( "The image was transfered to the clipboard.", "Twerp", , .f. )
	endif

return nil
**********

*************************************
Function OnPaint()
Local x := GetDesktopRectWidth(), y := GetDesktopRectHeight()
Local pps, hDC, w, h

	// force repaint of window
	InvalidateRect( _HMG_MainHandle, 1 )

	pps := DefinePaintStru()
	hDC := BeginPaint( _HMG_MainHandle, pps )

if valtype(hpal) == 'N'
	SetPalette(hDC, hpal)
endif
if valtype(hdib) == 'N'
	w := TWAIN_DibWidth(hdib)
	h := TWAIN_DibHeight(hdib)

	// wait cursor (hourglass)
	CursorWait()
	SetCursorPos(x / 2, y / 2)

	TWAIN_DrawDibToDC(hDC, 0, 0, w, h, hdib, 0, 0)

	// delay emulation for power PC
	InKey(.1)
	CursorArrow()

	Win_1.TW_APP_SAVEAS.Enabled := .T.
endif

	EndPaint( _HMG_MainHandle, pps )

return nil
**********

*************************************
Function OnOpen()

	// free up current image and palette if any
	DiscardImage()
	InvalidateRect( _HMG_MainHandle, 1 )

	hdib = TWAIN_LoadNativeFromFilename(0)
	if !EMPTY(hdib)
		// compute or guess a palette to use for display
		hpal = TWAIN_CreateDibPalette(hdib)

		// size the window to just contain the image
		ResizeWindow()
	endif

return nil
**********

****************
Function SaveAs()
Local nResult := 1, aExt := { "bmp" , "jpg" }
Local cFilename := Putfile( { {'Windows Bitmaps (*.bmp)', '*.bmp'}, {'Jpg Files (*.jpg)', '*.jpg'}  }, , ;
			GetStartUpFolder(), , @nResult )

	if !Empty( cFilename )
		cFilename := IF( RAT( ".", cFilename ) == 4, cFilename, cFilename + "." + aExt[nResult] )
		if nResult == 1
			nResult := TWAIN_WriteNativeToFilename(hdib, cFilename)
			//	-1	user cancelled File Save dialog
			//	-2	could not create or open file for writing
			//	-3	(weird) unable to access DIB
			//	-4	writing to .BMP failed, maybe output device is full?
			if nResult < -1
				MsgStop( "Error writing DIB to file - is there room for the image?", "Error" )
			endif
		else
			Save2Jpg( hWnd, cFilename, TWAIN_DibWidth(hdib), TWAIN_DibHeight(hdib) )
		endif
	endif

return nil
**********

****************
Function SetTypes()
Local cItem := This.name, cCurrentType

	if !"UI" $ cItem
		do case
			case wPixTypes = TWAIN_BW
				cCurrentType := "TW_APP_BW"
			case wPixTypes = TWAIN_GRAY
				cCurrentType := "TW_APP_GRAYSCALE"
			case wPixTypes = TWAIN_RGB
				cCurrentType := "TW_APP_RGB"
			case wPixTypes = TWAIN_PALETTE
				cCurrentType := "TW_APP_PALETTE"
			case wPixTypes = TWAIN_ANYTYPE
				cCurrentType := "TW_APP_ANYPIX"
		endcase
		Win_1.&cCurrentType..Checked := .F.
	endif

	do case
            case cItem == "TW_APP_BW"
            	wPixTypes := TWAIN_BW
		case cItem == "TW_APP_GRAYSCALE"
	        	wPixTypes := TWAIN_GRAY
		case cItem == "TW_APP_RGB"
	        	wPixTypes := TWAIN_RGB
		case cItem == "TW_APP_PALETTE"
	        	wPixTypes := TWAIN_PALETTE
		case cItem == "TW_APP_ANYPIX"
	        	wPixTypes := TWAIN_ANYTYPE
		case cItem == "TW_APP_SHOWUI"
			fHideUI = 0
			Win_1.TW_APP_HIDEUI.Checked := .F.
		case cItem == "TW_APP_HIDEUI"
			fHideUI = 1
			Win_1.TW_APP_SHOWUI.Checked := .F.
	endcase
	Win_1.&cItem..Checked := .T.

return nil
**********

****************
Procedure ResizeWindow()
Local w := GetDesktopRectWidth(), h := GetDesktopRectHeight()

	Win_1.Hide

	Win_1.Width := Max( Min( w, TWAIN_DibWidth(hdib) + 2 * GetBorderWidth() ), GetMinWidth(hWnd) )
	Win_1.Height := Min( h, TWAIN_DibHeight(hdib) + GetTitleHeight() + 2 * GetBorderHeight() + GetMenuBarHeight() )

	if GetProperty( "Win_1", "Width" ) >= w .OR. GetProperty( "Win_1", "Height" ) >= h
		Win_1.Row := 0
		Win_1.Col := 0
	else
		Win_1.Center
	endif

	Win_1.Show

return
**********

*************************************
Function DiscardImage()
	// delete/free global palette, and dib, as necessary.
	if hpal # NIL
		DeleteObject(hpal)
		hpal = NIL
	endif
	if hdib # NIL
		TWAIN_FreeNative(hdib)
		hdib = NIL
		Win_1.TW_APP_SAVEAS.Enabled := .F.
	endif

return nil
**********


*-----------------------------------------------------------------------------*
Function Putfile ( aFilter, title, cIniFolder, nochangedir, nFilterIndex )
*-----------------------------------------------------------------------------*
local c:='' , n
 
	IF aFilter == Nil
		aFilter := {}
	EndIf
	IF nFilterIndex == Nil
		nFilterIndex := 1
	EndIf
 
	FOR n := 1 TO Len ( aFilter )
		c += aFilter [n] [1] + chr(0) + aFilter [n] [2] + chr(0)
	NEXT
 
Return C_PutFile ( c, title, cIniFolder, nochangedir, @nFilterIndex )


#ifdef __XHARBOUR__   // Declaration of DLLs using syntax in CallDll.Lib

   *-----------------------------------------------------------------------------*
   DECLARE SaveToJpgEx(hWnd, cFileName, nWidth, nHeight) IN JPG.DLL ALIAS SAVE2JPG

   DECLARE BmpToJpg(BmpFile, JpgFile) IN JPG.DLL ALIAS BMP2JPG
   *-----------------------------------------------------------------------------*

   *-----------------------------------------------------------------------------*
   * EZTwain wrappers for xHarbour
   *
   * Copyright 2005 Walter Formigoni <walter.formigoni@uol.com.br>
   *
   * Copyright 2005 Grigory Filatov <gfilatov@freemail.ru>
   *-----------------------------------------------------------------------------*
   declare TWAIN_IsAvailable() in EZTW32.DLL

   declare TWAIN_SelectImageSource( hWnd ) in EZTW32.DLL

   declare TWAIN_AcquireToFilename( hWnd, lpString ) in EZTW32.DLL

   declare TWAIN_AcquireToClipboard( hwndApp, DwPixTypes ) in EZTW32.DLL

   declare TWAIN_EasyVersion() in EZTW32.DLL

   declare TWAIN_AcquireNative( hWnd, wPixTypes ) in EZTW32.DLL

   declare TWAIN_LoadNativeFromFilename( pszFile ) in EZTW32.DLL
   declare TWAIN_FreeNative( hdib ) in EZTW32.DLL

   declare TWAIN_WriteNativeToFilename( hdib, pszFile ) in EZTW32.DLL

   declare TWAIN_CreateDibPalette( hdib ) in EZTW32.DLL

   declare TWAIN_DibWidth( hdib ) in EZTW32.DLL
   declare TWAIN_DibHeight( hdib ) in EZTW32.DLL

   declare TWAIN_DrawDibToDC( hDC, dx, dy, w, h, hdib, sx, sy ) in EZTW32.DLL

   declare TWAIN_SetHideUI( fHide ) in EZTW32.DLL
   *-----------------------------------------------------------------------------*

#else

   *-----------------------------------------------------------------------------*
   DECLARE DLL_TYPE_VOID SaveToJpgEx(DLL_TYPE_LONG hWnd, DLL_TYPE_LPCSTR cFileName, ;
	DLL_TYPE_INT nWidth, DLL_TYPE_INT nHeight) IN JPG.DLL ALIAS SAVE2JPG

   DECLARE DLL_TYPE_VOID BmpToJpg(DLL_TYPE_LPCSTR BmpFile, DLL_TYPE_LPCSTR JpgFile) ;
        IN JPG.DLL ALIAS BMP2JPG
   *-----------------------------------------------------------------------------*

   *-----------------------------------------------------------------------------*
   * EZTwain wrappers for Harbour
   *
   * Copyright 2005 Walter Formigoni <walter.formigoni@uol.com.br>
   *
   * Copyright 2005 Grigory Filatov <gfilatov@freemail.ru>
   *-----------------------------------------------------------------------------*
   declare DLL_TYPE_INT TWAIN_IsAvailable() in EZTW32.DLL

   declare DLL_TYPE_INT TWAIN_SelectImageSource( DLL_TYPE_HWND hWnd ) in EZTW32.DLL

   declare DLL_TYPE_INT TWAIN_AcquireToFilename( DLL_TYPE_HWND hWnd, DLL_TYPE_LPCSTR lpString ) in EZTW32.DLL

   declare DLL_TYPE_INT TWAIN_AcquireToClipboard( DLL_TYPE_HWND hwndApp, DLL_TYPE_LONG wPixTypes ) in EZTW32.DLL

   declare DLL_TYPE_INT TWAIN_EasyVersion() in EZTW32.DLL

   //declare DLL_TYPE_INT TWAIN_OpenDefaultSource() in EZTW32.DLL

   declare DLL_TYPE_HANDLE TWAIN_AcquireNative( DLL_TYPE_HWND hWnd, DLL_TYPE_LONG wPixTypes ) in EZTW32.DLL

   declare DLL_TYPE_HANDLE TWAIN_LoadNativeFromFilename( DLL_TYPE_LPCSTR pszFile ) in EZTW32.DLL
   declare DLL_TYPE_VOID TWAIN_FreeNative( DLL_TYPE_HANDLE hdib ) in EZTW32.DLL

   declare DLL_TYPE_INT TWAIN_WriteNativeToFilename( DLL_TYPE_HANDLE hdib, DLL_TYPE_LPCSTR pszFile ) in EZTW32.DLL

   declare DLL_TYPE_HANDLE TWAIN_CreateDibPalette( DLL_TYPE_HANDLE hdib ) in EZTW32.DLL

   declare DLL_TYPE_INT TWAIN_DibWidth( DLL_TYPE_HANDLE hdib ) in EZTW32.DLL
   declare DLL_TYPE_INT TWAIN_DibHeight( DLL_TYPE_HANDLE hdib ) in EZTW32.DLL

   declare DLL_TYPE_VOID TWAIN_DrawDibToDC( DLL_TYPE_HDC hDC, ;
      DLL_TYPE_INT dx, DLL_TYPE_INT dy, DLL_TYPE_INT w, DLL_TYPE_INT h, ;
      DLL_TYPE_HANDLE hdib, DLL_TYPE_INT sx, DLL_TYPE_INT sy ) in EZTW32.DLL

   declare DLL_TYPE_VOID TWAIN_SetHideUI( DLL_TYPE_INT fHide ) in EZTW32.DLL
   *-----------------------------------------------------------------------------*

#endif


#pragma BEGINDUMP

#define HB_OS_WIN_32_USED
#define _WIN32_WINNT   0x0400
#include <windows.h>
#include "hbapi.h"
#include "hbapiitm.h"

HB_FUNC( GETDESKTOPRECTWIDTH )
{
	RECT rect;
	SystemParametersInfo( SPI_GETWORKAREA, 0, &rect, 0 );
	hb_retni(rect.right - rect.left);
}

HB_FUNC( GETDESKTOPRECTHEIGHT )
{
	RECT rect;
	SystemParametersInfo( SPI_GETWORKAREA, 0, &rect, 0 );
	hb_retni(rect.bottom - rect.top);
}

HB_FUNC( C_CENTER )
{
	HWND hwnd = (HWND) hb_parnl(1);
	RECT rect;
	int w, h, x, y;
	GetWindowRect( hwnd, &rect );
	w  = rect.right  - rect.left;
	h = rect.bottom - rect.top;
	SystemParametersInfo( SPI_GETWORKAREA, 1, &rect, 0 );
	x = rect.right - rect.left;
	y = rect.bottom - rect.top;
	SetWindowPos( hwnd, HWND_TOP, (x - w) / 2, (y - h) / 2, 0, 0, SWP_NOSIZE | SWP_NOACTIVATE );
}

HB_FUNC( SETPALETTE )
{
	HDC hDC = (HDC) hb_parnl(1);
	HPALETTE hPal = (HPALETTE) hb_parnl(2);

	SelectPalette (hDC, hPal, FALSE);
	RealizePalette (hDC);
}


HB_FUNC( DEFINEPAINTSTRU )
{
   PAINTSTRUCT *pps = (PAINTSTRUCT*) hb_xgrab( sizeof( PAINTSTRUCT ) );
   hb_retnl( (LONG) pps );
}

HB_FUNC( BEGINPAINT )
{
   PAINTSTRUCT *pps = (PAINTSTRUCT*) hb_parnl( 2 );
   HDC hDC = BeginPaint( (HWND) hb_parnl( 1 ), pps );
   hb_retnl( (LONG) hDC );
}

HB_FUNC( ENDPAINT )
{
   hb_retl( EndPaint( (HWND) hb_parnl( 1 ), (PAINTSTRUCT*) hb_parcx( 2 ) ) );
}

HB_FUNC( GETMINWIDTH )
{
	HWND  hwnd = (HWND) hb_parnl(1);
	HDC	hDC = GetDC(hwnd);
	int	xMin;
	if (hDC) {
		xMin = LOWORD(GetTabbedTextExtent(hDC, "Twerp - EZTwain sample app by Spike", 36, 0, NULL));
		ReleaseDC(hwnd, hDC);
		hb_retni(xMin);
    }
}

HB_FUNC( C_PUTFILE )
{
    OPENFILENAME ofn;
    char buffer[512];
 
    int flags = OFN_FILEMUSTEXIST | OFN_EXPLORER ;
 
    if ( hb_parl(4) )
    {
        flags = flags | OFN_NOCHANGEDIR ;
    }
 
    strcpy( buffer, "" );
 
    memset( (void*) &ofn, 0, sizeof( OPENFILENAME ) );
    ofn.lStructSize = sizeof(ofn);
    ofn.hwndOwner = GetActiveWindow() ;
    ofn.lpstrFilter = hb_parc(1) ;
    ofn.lpstrFile = buffer;
    ofn.nMaxFile = 512;
    ofn.lpstrInitialDir = hb_parc(3);
    ofn.nFilterIndex    = hb_parni(5);
    ofn.lpstrTitle = hb_parc(2) ;
    ofn.Flags = flags;
 
    if( GetSaveFileName( &ofn ) )
    {
        hb_stornl(ofn.nFilterIndex, 5 );
        hb_retc( ofn.lpstrFile );
    }
    else
    {
        hb_retc( "" );
    }
}

#pragma ENDDUMP
