/*----------------------------------------------------------------------------
 MINIGUI - Harbour Win32 GUI library source code

 Copyright 2002 Roberto Lopez <harbourminigui@gmail.com>
 http://harbourminigui.googlepages.com/
-----------------------------------------------------------------------------
*/
#include "minigui.ch"
#include "i_xml.ch"

#define NOFOUND  -1     //Indicates a duplicate entry was not found
#define NOMRUS    0     //Indicates no MRUs are currently defined

*--------------------------------------------------
Function  MRU_Load()
*--------------------------------------------------
    // Initialize the count of MRUs
    MRUCount := NOMRUS
    // Call sub to retrieve the MRU filenames
    GetMRUFileList()
Return NIL

*--------------------------------------------------
Function MRU_Unload()
*--------------------------------------------------
   // Call to save the MRU filenames
   SaveMRUFileList() 
Return Nil

*--------------------------------------------------
Function mnuMRU_Click( NewItem )
*--------------------------------------------------
    Local result
    // Call to reorder the MRU list
    Read_file( NewItem )  

Return Nil

*--------------------------------------------------
Function mnuOpen_Click( NewItem )
*--------------------------------------------------
  
    // Call to add this file as an MRU
    AddMenuElement( NewItem  )
Return Nil

*--------------------------------------------------
Function AddMRUItem( NewItem )
*--------------------------------------------------
    Local result
    result = CheckForDuplicateMRU( NewItem )
    If result <> NOFOUND 
        ReorderMRUList( NewItem, result )
    Endif
    if result != 1 .or. MRUCount == 0 
        AddMenuElement( NewItem )
    endif
Return Nil

*--------------------------------------------------
Function CheckForDuplicateMRU( NewItem ) 
*--------------------------------------------------
    Local i , DuplicateMRU := -1
    
    if !empty(NewItem)
        // Uppercase newitem for string comparisons
        NewItem := Upper(NewItem)
        // Check all existing MRUs for duplicate
        i := ascan( aMRU_File , {|y| UPPER(y[2])==NewItem})
        if i != 0
            DuplicateMRU = i
        Endif
    endif
Return DuplicateMRU 

*--------------------------------------------------
Function AddMenuElement( NewItem  )
*--------------------------------------------------
    Local i , x:=1 , n , cx
    Local action , cAction , Caption, xCaption, cyMRU_Id, cxMRU_Id

    if len(NewItem) < 40
        Caption := NewItem
    else
        Caption := substr(NewItem,1,3)+'...'+ substr(NewItem,len(NewItem)-34)
    endif
    cAction := '{|| mnuMRU_Click( "'+NewItem + '" ) }'
    action := &cAction
     
        // Check if this is the first item
        If MRUCount == 0 
            // Modify a first element the menu
            cxMRU_Id := cMRU_Id
            _ModifyMenuItem ( cxMRU_Id , 'Form_1' ,'&1 '+caption , action  )
            aadd( aMRU_File , { caption, NewItem, cxMRU_Id, action, 1 })

        Else
            // Add a new element to the menu
            for n:=1 to Len( aMRU_File )+1
                X := ascan( aMRU_File , {|y| y[5]==n})
                if X == 0
                    x :=n
                    exit
                ENDIF
            next   

            cx:=  alltrim(str( x ))   //strzero(x,2)
            cyMRU_Id := cMRU_Id +'_'+ cx          
            cxMRU_Id := aMRU_File[1,3]
            _InsertMenuItem ( cxMRU_Id , 'Form_1' ,'&1 '+caption , action, cyMRU_Id  )
            ASIZE(aMRU_File, Len(aMRU_File)+1) 
            AINS( aMRU_File, 1 ) 
            aMRU_File [ 1 ] := {caption,NewItem,cyMRU_Id,action,x}
            for n:= 1 to len(aMRU_File) 
                cx:= alltrim(str(n))
                cxMRU_Id := aMRU_File[n,3]
                xCaption := '&'+cx+' '+aMRU_File[n,1]
                _ModifyMenuItem ( cxMRU_Id , 'Form_1' , xCaption , aMRU_File[n,4] ) 
            next
            if len(aMRU_File) > maxMRU_Files
                cxMRU_Id := aMRU_File[ Len(aMRU_File) , 3 ] 
                ASIZE( aMRU_File , maxMRU_Files ) 
                _RemoveMenuItem( cxMRU_Id , 'Form_1')
            endif

        Endif
        //Increment the menu count
        MRUCount++

Return Nil

*--------------------------------------------------
Function ReorderMRUList(DuplicateMRU , DuplicateLocation )
*--------------------------------------------------
    Local i, cxMRU_Id
   
    // Move entries previously "more recent" than the
    // duplicate down one in the MRU list
    if DuplicateLocation > 1
        cxMRU_Id := aMRU_File[ DuplicateLocation , 3 ] 
        _RemoveMenuItem( cxMRU_Id , 'Form_1')
        ADEL( aMRU_File, DuplicateLocation ) 
        ASIZE(aMRU_File, Len(aMRU_File)-1) 
    endif
   
Return Nil

*--------------------------------------------------
Function ClearMRUList( )
*--------------------------------------------------
    Local n, cxMRU_Id, xCaption
   
    for n:= len(aMRU_File) to 1 step -1
        if n > 1
            cxMRU_Id := aMRU_File[ n , 3 ] 
            _RemoveMenuItem( cxMRU_Id , 'Form_1')
        else	
	        cxMRU_Id := aMRU_File[n,3]
            cMRU_Id := cxMRU_Id 
            xCaption := ' (Empty) '
            _ModifyMenuItem ( cMRU_Id , 'Form_1' , xCaption , {|| Nil } ) 
            aMRU_File :={} 
            MRUCount := 0 
	        SetProperty('Form_1',cMRU_Id,'Enabled',.f.)
        endif
    next
Return Nil

*--------------------------------------------------
Function GetMRUFileList()
*--------------------------------------------------
    Local i := 1, aTmp:={}
    Local result       

    BEGIN INI FILE cFileIni
   
    Do while i <= maxMRU_Files 
      // Retrieve entry from registry (XML)
       
      result = GetSetting("file", i)
      // Check if a value was returned
      If !empty(result) 
         Aadd(aTmp, result)
      else
         exit
      Endif
      i++
    enddo
    END INI	
    for i := len(aTmp) to 1 step -1
         AddMRUItem(aTmp[i] )
    next
Return Nil

*--------------------------------------------------
Function SaveMRUFileList()
*--------------------------------------------------
    Local oXmlNode
    Local i, j, cFile
   
    // Loop through all MRU
    For i = 1 To maxMRU_Files

      // Write MRU to registry with key as it's position in list
        if i <= len(aMRU_File)
            cFile := aMru_File[ i , 2 ]
        else
            cFile := ""
        endif

        oXmlNode := oXmlDoc:aItems[1]:aItems[i]
        oXmlNode:SetAttribute( "file", cFile )
        lIniChanged := .T.

         FOR j := 1 TO Len( oXmlNode:aItems )
            IF Valtype( oXmlNode:aItems[j] ) == "C"
               oXmlNode:aItems[j] := cFileNoPath( cFile )
            ELSEIF oXmlNode:aItems[j]:title == "info"
               if file( cFile )
                  oXMLNode:aItems[j] := Attr2XML( cFile )
               endif
            ENDIF
         NEXT

    Next

Return Nil

*--------------------------------------------------
Function Attr2XML( cFile )
*--------------------------------------------------
Local aAttr := {}, aFile := Directory( cFile ), cItem
   cItem := aFile[1][2]
   Aadd( aAttr, { "size", Ltrim(str(cItem)) } )
   cItem := aFile[1][3]
   Aadd( aAttr, { "date", DtoC(cItem) } )
   cItem := aFile[1][4]
   Aadd( aAttr, { "time", cItem } )
   cItem := aFile[1][5]
   Aadd( aAttr, { "attr", cItem } )
Return HXMLNode():New( "info", HBXML_TYPE_SINGLE, aAttr )

*--------------------------------------------------
Function GetSetting (cFile, nItem)
*--------------------------------------------------
Local oXmlNode, cName := ''
    oXmlNode := oXmlDoc:aItems[1]:aItems[nItem]
    cName := oXmlNode:GetAttribute( cFile )
Return cName
