/*
 * Author: P.Chornyj <myorg63@mail.ru>
 *
 * Last revision : 2007.06.18
 *
*/

#include "common.ch"
#include "unrar.ch"

/*
*/
FUNCTION Hb_RarGetFilesCount( ArchiveName, cPassWord, lIncludeDirectory )
LOCAL nResult := 0, nI
LOCAL aFL
	DEFAULT lIncludeDirectory TO TRUE

	IF ( lIncludeDirectory )
		nResult := Len( Hb_RarGetFilesList( ArchiveName, cPassWord ) )
	ELSE
		aFL := Hb_RarGetFilesList( ArchiveName, cPassWord )
                nResult := Len( aFL )
		IF nResult <> 0
			FOR nI := 1 TO nResult
				IF ( "D" $ aFL[ nI ][ RAR_FILEATTR ] )
					nResult --
				ENDIF 
			NEXT
		END
	END
		
RETURN nResult

/*
*/
FUNCTION Hb_RarGetFilesList( ArchiveName, cPassWord, lFileTimeAsDate, lIncludeTime, lIncludeSeconds )
LOCAL aResult 
	
	DEFAULT lFileTimeAsDate TO FALSE
	DEFAULT lIncludeTime    TO TRUE
	DEFAULT lIncludeSeconds TO FALSE

	IF File( ArchiveName )
		aResult := Hb_RGetFilesList( ArchiveName, cPassWord, lFileTimeAsDate, lIncludeTime, lIncludeSeconds )
	ENDIF

RETURN iif( ValType( aResult ) == "A", aResult, {} )

/*
*/
FUNCTION Hb_RarSetCmtBufSize( CmtBuffSize )
LOCAL lSuccess := .F.
	IF ValType( CmtBuffSize ) == "N"
		IF CmtBuffSize >= 64 .AND. CmtBuffSize <= 56535
			Hb_RSetCmtBufSize( CmtBuffSize )
			lSuccess := .T.
		ENDIF
	ELSE
		Hb_RSetCmtBufSize( 16384 )
	ENDIF
RETURN lSuccess

/*
*/
FUNCTION Hb_UnrarFiles( ArchiveName, cPassWord, cPath, File )
RETURN Hb_ProcessFile( ArchiveName, cPassWord, cPath, File, RAR_EXTRACT )

/*
*/
FUNCTION Hb_RarTestFiles( ArchiveName, cPassWord, File )
RETURN Hb_ProcessFile( ArchiveName, cPassWord, , File, RAR_TEST )

/*
*/
STATIC FUNCTION Hb_ProcessFile( ArchiveName, cPassWord, cPath, File, Operation )
LOCAL lSuccess := .F.
LOCAL sw, sw1, i, j
LOCAL aFileList := {}, aFile := {}

	IF File( ArchiveName )
		IF ( ISNIL( File ) )		
			lSuccess := Hb_RProcessFiles( Operation, ArchiveName, cPassWord, cPath )	
		ELSE
			aFileList := Hb_RGetFilesList( ArchiveName, cPassWord )
			sw := ValType( File )
			SWITCH sw
				CASE "N"
					IF ( File >= 1 .AND. File <= Len( aFileList ) )
						AAdd( aFile, aFileList[ File ] [ RAR_FILENAME ] ) 
					ENDIF
					EXIT
				CASE "C"
					IF ( AScan( aFileList, {|e| e[ RAR_FILENAME ] == File  } ) > 0  )
						AAdd( aFile, File ) 
					ENDIF
					EXIT
				CASE "A"
					FOR i := 1 TO Len( File )
						sw1 := ValType( File[i] )
						SWITCH sw1
							CASE "N"
								IF ( File[i] >= 1 .AND. File[i] <= Len( aFileList ) )
									AAdd( aFile, aFileList[ File[ i ] ][ RAR_FILENAME ] )
								ENDIF
								EXIT
							CASE "C"
								j := AScan( aFileList, {|e| e[ RAR_FILENAME ] == File[i] } )
								IF ( j > 0 ) 
									AAdd( aFile, aFileList[ j ][ RAR_FILENAME ] ) 
								ENDIF
								EXIT
						END
					NEXT
					EXIT
			END		

			IF .NOT. Empty ( aFile )
				lSuccess := Hb_RProcessFiles( Operation, ArchiveName, cPassWord, cPath, aFile )	
			ENDIF
		ENDIF
	ENDIF

RETURN lSuccess


#pragma BEGINDUMP
 
#include <windows.h>
#include "unrar.h"
#include "hbapi.h"
#include "hbvm.h"
#include "hbstack.h"
#include "hbapiitm.h"
#include "hbapifs.h"

#define RAR_ARCNAME        1
#define RAR_FILENAME       2
#define RAR_FLAGS          3
#define RAR_PACKSIZE       4
#define RAR_UNPSIZE        5
#define RAR_HOSTOS         6
#define RAR_FILECRC        7
#define RAR_FILETIME       8
#define RAR_UNPVER         9
#define RAR_METHOD         10
#define RAR_FILEATTR       11

#define RAR_ST_SUCCESS     0
#define RAR_ST_OPEN        1
#define RAR_ST_OUT         2
#define RAR_ST_OPEN_OUT    3
#define RAR_ST_HBROKEN     4

static void OpenProcessFileError( int Error );
static void OutProcessFileError( int Error, const char* Filename );
static void ClearError( void );

static PHB_ITEM RGetDate( UINT FileTime );
static PHB_ITEM RConvertFileTimeToString( UINT FileTime, BOOL includeTime, BOOL includeSeconds );

typedef UINT (CALLBACK* RGetApiDllVersion)( void );

static UINT CmtBufSize = 16384;
static int  iOpenError = 0;	
static char msgOpenError[ 64 ];	
static int  iOutError = 0;	
static char msgOutError[ 64 ];	
static char FName[ _MAX_PATH ];	
static BOOL bHeaderBroken;
/*
	Get Unrar.DLL API version
*/
HB_FUNC ( HB_RARGETDLLVERSION )
{
	int iResult = 0;
	HINSTANCE m_hDLL;
	RGetApiDllVersion m_UnRarDllApiVersion;

	m_hDLL = LoadLibrary( "unrar.dll" );

	if ( m_hDLL != NULL )
	{
		m_UnRarDllApiVersion = ( (RGetApiDllVersion) GetProcAddress( m_hDLL, "RARGetDllVersion" ) ) ;
		if ( m_UnRarDllApiVersion != NULL )  
			iResult = m_UnRarDllApiVersion();

		FreeLibrary( m_hDLL );
	}
	else
		iResult = -1;

	hb_retni( iResult );
}

/*
	Archive comments operations	
*/
static HB_FUNC ( HB_RSETCMTBUFSIZE )
{
	CmtBufSize = hb_parni( 1 );
}

HB_FUNC ( HB_RARGETCMTBUFSIZE )
{
	hb_retni( CmtBufSize );
}

HB_FUNC ( HB_RARGETCOMMENT )
{
	PHB_ITEM pCmtState = hb_param( 2, HB_IT_INTEGER );
	HANDLE hArcData;
	char* CmtBuf = ( char* ) hb_xgrab( CmtBufSize );
	struct RAROpenArchiveDataEx OpenArchiveData;

	hb_xmemset( &OpenArchiveData, 0, sizeof( OpenArchiveData ) );
	OpenArchiveData.ArcName = hb_parc( 1 );
	OpenArchiveData.CmtBuf  = CmtBuf;
	OpenArchiveData.CmtBufSize = CmtBufSize;
	OpenArchiveData.OpenMode   = RAR_OM_LIST;

	ClearError();
	hArcData = RAROpenArchiveEx( &OpenArchiveData );

	if ( OpenArchiveData.OpenResult == 0 )
	{
		if ( OpenArchiveData.CmtState == 1)
		{
			char* szTempR = (char*) hb_xgrab( OpenArchiveData.CmtSize + 1 );

			hb_xmemset( szTempR, 0, OpenArchiveData.CmtSize + 1 );
			strcpy( szTempR, OpenArchiveData.CmtBuf );

			hb_retcAdopt( szTempR );
		}
		else
			hb_retc( "" );

		if ( pCmtState != NULL ) 
			hb_storni( OpenArchiveData.CmtState, 2 );
	}       
	else
	{
		OpenProcessFileError( OpenArchiveData.OpenResult );
		hb_retc( "" );
	}

	RARCloseArchive( hArcData );
	hb_xfree( CmtBuf );
}

/*
	Get archive info 	
*/
HB_FUNC ( HB_RARGETARCHIVEINFO )
{
	HANDLE hArcData;
	int iResult = 0;
	char* CmtBuf = ( char* ) hb_xgrab( CmtBufSize );
	struct RAROpenArchiveDataEx OpenArchiveData;

	hb_xmemset( &OpenArchiveData, 0, sizeof( OpenArchiveData ) );
	OpenArchiveData.ArcName  = hb_parc( 1 );
	OpenArchiveData.CmtBuf   = CmtBuf;
	OpenArchiveData.CmtBufSize = CmtBufSize;
	OpenArchiveData.OpenMode = RAR_OM_LIST;

	ClearError();
	hArcData = RAROpenArchiveEx( &OpenArchiveData );
	if ( OpenArchiveData.OpenResult == 0 )
		iResult = OpenArchiveData.Flags;
	else
		OpenProcessFileError( OpenArchiveData.OpenResult );
		
	RARCloseArchive(hArcData);

	hb_xfree( CmtBuf );

	hb_retni( iResult );
}

/*
	Get file list from archive
*/
static HB_FUNC ( HB_RGETFILESLIST )
{
	PHB_ITEM pFileList = hb_itemArrayNew( 0 );
	HANDLE hArcData;
	int RHCode;
	struct RAROpenArchiveDataEx OpenArchiveData;

	hb_xmemset( &OpenArchiveData, 0, sizeof( OpenArchiveData ) );
	OpenArchiveData.ArcName  = hb_parc( 1 );
	OpenArchiveData.CmtBuf   = NULL;
	OpenArchiveData.OpenMode = RAR_OM_LIST;

	ClearError();
	hArcData = RAROpenArchiveEx( &OpenArchiveData );
	if ( OpenArchiveData.OpenResult == 0 )
	{
		struct RARHeaderDataEx HeaderData;
		int PFCode;

		PHB_ITEM pPassword = hb_param( 2, HB_IT_STRING );

		BOOL fFileTimeAsDate = hb_parl( 3 );
		BOOL fIncludeTime    = hb_parl( 4 );
		BOOL fIncludeSeconds = hb_parl( 5 );

		PHB_ITEM pArcName  = hb_itemNew( NULL );
		PHB_ITEM pFileName = hb_itemNew( NULL );
		PHB_ITEM pFlags    = hb_itemNew( NULL );
		PHB_ITEM pUnpSize  = hb_itemNew( NULL );
		PHB_ITEM pPackSize = hb_itemNew( NULL );
		PHB_ITEM pHostOS   = hb_itemNew( NULL );
		PHB_ITEM pFileCRC  = hb_itemNew( NULL );
		PHB_ITEM pUnpVer   = hb_itemNew( NULL );
		PHB_ITEM pMethod   = hb_itemNew( NULL );
		PHB_ITEM pFileAttr = hb_itemNew( NULL );

		if ( pPassword != NULL )
			RARSetPassword( hArcData, (char *) hb_itemGetCPtr( pPassword ) );
	
//  RARSetCallback( hArcData, CallbackProc, 0 );

		HeaderData.CmtBuf = NULL; 
		while ( ( RHCode = RARReadHeaderEx( hArcData, &HeaderData ) ) == 0 )
		{
		        PHB_ITEM pSubarray = hb_itemArrayNew( 11 );
			__int64 PackSize = HeaderData.PackSize +( ( (__int64) HeaderData.PackSizeHigh ) << 32 );
			__int64 UnpSize = HeaderData.UnpSize +( ( (__int64) HeaderData.UnpSizeHigh ) << 32 );
			char buff[32];

		        hb_arraySet( pSubarray, RAR_ARCNAME,  hb_itemPutC( pArcName,  HeaderData.ArcName ) );
		        hb_arraySet( pSubarray, RAR_FILENAME, hb_itemPutC( pFileName, HeaderData.FileName ) );
		        hb_arraySet( pSubarray, RAR_FLAGS,    hb_itemPutNI( pFlags,   HeaderData.Flags ) );
		        hb_arraySet( pSubarray, RAR_PACKSIZE, hb_itemPutND( pPackSize, PackSize ) );
		        hb_arraySet( pSubarray, RAR_UNPSIZE,  hb_itemPutND( pUnpSize, UnpSize ) );
		        hb_arraySet( pSubarray, RAR_HOSTOS,   hb_itemPutNI( pHostOS,   HeaderData.HostOS ) );
		        hb_arraySet( pSubarray, RAR_FILECRC,  hb_itemPutNI( pFileCRC,  HeaderData.FileCRC ) );

			if ( fFileTimeAsDate )
		         	hb_arraySet( pSubarray, RAR_FILETIME, RGetDate( HeaderData.FileTime ) );
			else
		      	   	hb_arraySet( pSubarray, RAR_FILETIME, RConvertFileTimeToString( HeaderData.FileTime, fIncludeTime, fIncludeSeconds ) );

		        hb_arraySet( pSubarray, RAR_UNPVER,   hb_itemPutNI( pUnpVer,   HeaderData.UnpVer ) );
		        hb_arraySet( pSubarray, RAR_METHOD,   hb_itemPutNI( pMethod,  HeaderData.Method ) );
		        hb_arraySet( pSubarray, RAR_FILEATTR, hb_itemPutC( pFileAttr,  hb_fsAttrDecode( HeaderData.FileAttr, buff ) ) );

		        hb_arrayAdd( pFileList, pSubarray );
		        hb_itemRelease( pSubarray );

			if ( ( PFCode = RARProcessFile( hArcData, RAR_SKIP, NULL, NULL ) ) != 0 )
			{
				OutProcessFileError( PFCode, HeaderData.FileName );
				break;
			}
		}
		hb_itemRelease( pArcName );
		hb_itemRelease( pFileName );
		hb_itemRelease( pFlags );
		hb_itemRelease( pUnpSize );
		hb_itemRelease( pPackSize );
		hb_itemRelease( pHostOS );
		hb_itemRelease( pFileCRC );
		hb_itemRelease( pUnpVer );
		hb_itemRelease( pMethod );
		hb_itemRelease( pFileAttr );

		if ( RHCode == ERAR_BAD_DATA )
			bHeaderBroken = TRUE;
//			MessageBox( GetActiveWindow(), "nFile header broken", "Unrar error", MB_OK | MB_ICONINFORMATION | MB_SYSTEMMODAL );
	}
	else
		OpenProcessFileError( OpenArchiveData.OpenResult );

#ifdef __XHARBOUR__	
	hb_itemRelease( hb_itemReturn( pFileList ) );
#else
	hb_itemReturnRelease( pFileList );
#endif
	RARCloseArchive(hArcData);
}

/*
*/
static HB_FUNC ( HB_RPROCESSFILES )
{
	BOOL fResult = TRUE;
	HANDLE hArcData;
	int RHCode;
	struct RAROpenArchiveDataEx OpenArchiveData;

	int Operation   = hb_parni( 1 );
	PHB_ITEM pArray = hb_param( 5, HB_IT_ARRAY );

	hb_xmemset( &OpenArchiveData, 0, sizeof( OpenArchiveData ) );
	OpenArchiveData.ArcName  = hb_parc( 2 );
	OpenArchiveData.CmtBuf   = NULL;
	OpenArchiveData.OpenMode = RAR_OM_EXTRACT;

	ClearError();
	hArcData = RAROpenArchiveEx( &OpenArchiveData );
	if ( OpenArchiveData.OpenResult == 0 )
	{
		struct RARHeaderDataEx HeaderData;
		int PFCode;

		PHB_ITEM pPassword = hb_param( 3, HB_IT_STRING );
		PHB_ITEM pPath = hb_param( 4, HB_IT_STRING );

		DWORD ulLen = hb_itemGetCLen( pPath );
		char * pszDst = ( char * ) hb_xgrab( ulLen + 1 );

		hb_xmemset( pszDst, 0, ulLen + 1 );
		// Convert path from ANSI to OEM. Is need ?
		CharToOemBuff( ( LPCSTR ) hb_itemGetCPtr( pPath ), ( LPSTR ) pszDst, ulLen );

		if ( pPassword != NULL )
			RARSetPassword( hArcData, (char *) hb_itemGetCPtr( pPassword ) );

//RARSetCallback( hArcData, CallbackProc, 0 );

		HeaderData.CmtBuf = NULL; 

		while ( ( RHCode = RARReadHeaderEx( hArcData, &HeaderData ) ) == 0 )
		{
			PHB_ITEM pValue = hb_itemNew( NULL );
		        hb_itemPutC( pValue, HeaderData.FileName );

		        if ( pArray != NULL )
			{
#ifdef __XHARBOUR__
				if ( ( PFCode = RARProcessFile( hArcData, ( hb_arrayScan( pArray, pValue, NULL, NULL, FALSE, FALSE ) > 0 ) ? Operation : RAR_SKIP, pszDst, NULL ) ) != 0 )
#else
				if ( ( PFCode = RARProcessFile( hArcData, ( hb_arrayScan( pArray, pValue, NULL, NULL, FALSE ) > 0 ) ? Operation : RAR_SKIP, pszDst, NULL ) ) != 0 )
#endif
				{
					OutProcessFileError( PFCode, HeaderData.FileName );
					fResult = FALSE;
					break;
				}
			}
			else
			{
				if ( ( PFCode = RARProcessFile( hArcData, Operation, pszDst, NULL ) ) != 0 )
				{
					OutProcessFileError( PFCode, HeaderData.FileName );
					fResult = FALSE;
					break;
				}
			}
			hb_itemRelease( pValue );
		}

		if ( RHCode == ERAR_BAD_DATA )
		{
			fResult = FALSE;
			bHeaderBroken = TRUE;
//			MessageBox( GetActiveWindow(), "File header broken", "Unrar error", MB_OK | MB_ICONINFORMATION | MB_SYSTEMMODAL );
		}
		hb_xfree( pszDst );
	}       
	else
	{
		fResult = FALSE;
		OpenProcessFileError( OpenArchiveData.OpenResult );
	}

	RARCloseArchive(hArcData);
	
	hb_retl( fResult );
}

/*
	Error processing
*/
static void OutProcessFileError( int Error, const char* FileName )
{
	switch( Error )
	{
		case ERAR_UNKNOWN_FORMAT:
			strcpy( msgOutError, "Unknown archive format" );
			break;
		case ERAR_BAD_ARCHIVE:
			strcpy( msgOutError, "Bad volume" );
			break;
		case ERAR_ECREATE:
			strcpy( msgOutError, "File create error" );
			break;
		case ERAR_EOPEN:
			strcpy( msgOutError, "Volume open error" );
			break;
		case ERAR_ECLOSE:
			strcpy( msgOutError, "File close error" );
			break;
		case ERAR_EREAD:
			strcpy( msgOutError, "Read error" );
			break;
		case ERAR_EWRITE:
			strcpy( msgOutError, "Write error" );
			break;
		case ERAR_BAD_DATA:
			strcpy( msgOutError, "CRC error" );
			break;
		case ERAR_UNKNOWN:
			strcpy( msgOutError, "Unknown error" );
			break;
		case ERAR_MISSING_PASSWORD:
			strcpy( msgOutError, "Password for encrypted file is not specified" );
			break;
	}
	iOutError = Error;
	strcpy( FName, FileName );
//	MessageBox( GetActiveWindow(), msgOutError, FileName, MB_OK | MB_ICONINFORMATION | MB_SYSTEMMODAL );
}

static void OpenProcessFileError( int Error )
{
	switch( Error )
	{
		case ERAR_NO_MEMORY:
			strcpy( msgOpenError, "Not enough memory to initialize data structures" );
			break;
		case ERAR_BAD_DATA:
			strcpy( msgOpenError, "Archive header broken" );
			break;
		case ERAR_BAD_ARCHIVE:
			strcpy( msgOpenError, "File is not valid RAR archive" );
			break;
		case ERAR_UNKNOWN_FORMAT:
			strcpy( msgOpenError, "Unknown encryption used for archive headers" );
			break;
		case ERAR_EOPEN:
			strcpy( msgOpenError, "File open error" );
			break;
	}
	iOpenError = Error;
	// MessageBox( GetActiveWindow(), msgOpenError, "Unrar error", MB_OK | MB_ICONINFORMATION | MB_SYSTEMMODAL );
}

static void ClearError( void )
{
	iOpenError = 0;	
	msgOpenError[ 0 ] = '\0';	

	iOutError = 0;	
	msgOutError[ 64 ] = '\0';	
	FName[ 0 ] = '\0';	

	bHeaderBroken = FALSE;
}

HB_FUNC( HB_RARGETPROCSTATUS )
{
	INT iResult;
	
	if ( ( iOpenError == 0 ) && ( iOutError == 0 ) )	
		iResult = RAR_ST_SUCCESS;
	else if ( ( iOpenError != 0 ) && ( iOutError != 0 ) )
		iResult = RAR_ST_OPEN_OUT;
	else if ( iOpenError != 0 )
		iResult = RAR_ST_OPEN;
	else
		iResult = RAR_ST_OUT;

	if ( iResult == RAR_ST_SUCCESS )
		iResult = ( bHeaderBroken ) ? RAR_ST_HBROKEN : iResult;

	hb_retni( iResult );
}

HB_FUNC( HB_RARGETERRORINFO )
{
	PHB_ITEM pResult = hb_itemArrayNew( 5 );
	PHB_ITEM tmp;

	tmp = hb_itemPutNI( NULL, iOpenError );
	hb_arraySet( pResult, 1, tmp );

	tmp = hb_itemPutC( NULL, msgOpenError );
	hb_arraySet( pResult, 2, tmp );

	tmp = hb_itemPutNI( NULL, iOutError );
	hb_arraySet( pResult, 3, tmp );

	tmp = hb_itemPutC( NULL, msgOutError );
	hb_arraySet( pResult, 4, tmp );

	tmp = hb_itemPutC( NULL, FName );
	hb_arraySet( pResult, 5, tmp );

	hb_itemRelease( tmp );

	ClearError();

#ifdef __XHARBOUR__	
	hb_itemRelease( hb_itemReturn( pResult ) );
#else
	hb_itemReturnRelease( pResult );
#endif
}

/*
	Convert FileTime to date or string
*/
static PHB_ITEM RGetDate( UINT FileTime )
{
	FILETIME ft;
	SYSTEMTIME st;
	PHB_ITEM pResult = hb_itemNew( NULL );

	DosDateTimeToFileTime( HIWORD( FileTime ), LOWORD( FileTime ), &ft );
	FileTimeToSystemTime( &ft, &st );

	return ( hb_itemPutD( pResult, st.wYear, st.wMonth, st.wDay ) );
}

static PHB_ITEM RConvertFileTimeToString( UINT FileTime, BOOL includeTime, BOOL includeSeconds )
{
	PHB_ITEM pResult = hb_itemNew( NULL );
	FILETIME ft;
	SYSTEMTIME st;
	char s[64];

	DosDateTimeToFileTime( HIWORD( FileTime ), LOWORD( FileTime ), &ft );
	FileTimeToSystemTime( &ft, &st );

	s[0] = '\0';
	sprintf( s, "%04d-%02d-%02d", st.wYear, st.wMonth, st.wDay );
	if ( includeTime )
	{
		sprintf( s + strlen(s), " %02d:%02d", st.wHour, st.wMinute );
		if ( includeSeconds )
			sprintf( s + strlen(s), ":%02d", st.wSecond );
/*
		if ( includeMilliseconds )
			sprintf( s + strlen(s), ":%02d", st.wMilliseconds );
*/
	}
	return ( hb_itemPutC( pResult, s ) );
}

#pragma ENDDUMP
