/*----------------------------------------------------------------------------
 MINIGUI - Harbour Win32 GUI library source code

 Copyright 2002 Roberto Lopez <harbourminigui@gmail.com>
 http://harbourminigui.googlepages.com/
 -----------------------------------------------------------------------------
*/
#define __SYSDATA__
#include "minigui.ch"

#define NOFOUND  -1     //Indicates a duplicate entry was not found
#define NOMRUS    0     //Indicates no MRUs are currently defined

*--------------------------------------------------
Function  MRU_Load()
*--------------------------------------------------
   // Initialize the count of MRUs
   MRUCount := NOMRUS
   // Call sub to retrieve the MRU filenames
   GetMRUFileList()
Return NIL

*--------------------------------------------------
Function MRU_Unload()
*--------------------------------------------------
   // Call to save the MRU filenames
   SaveMRUFileList() 
Return Nil

*--------------------------------------------------
Function AddMRUItem( NewItem )
*--------------------------------------------------
   Local result

   result := CheckForDuplicateMRU( NewItem )
   If result <> NOFOUND 
	ReorderMRUList( result )
   Endif
   if result != 1 .or. MRUCount == 0 
	AddMenuElement( NewItem )
   endif
Return Nil

*--------------------------------------------------
Static Function CheckForDuplicateMRU( NewItem ) 
*--------------------------------------------------
    Local i , DuplicateMRU := NOFOUND
    
    if !empty(NewItem)
        // Uppercase newitem for string comparisons
        NewItem := Upper(NewItem)
        // Check all existing MRUs for duplicate
        i := ascan( aMRU_File , {|y| UPPER(y[2]) == NewItem} )
        if i != 0
            DuplicateMRU := i
        Endif
    endif
Return DuplicateMRU 

*--------------------------------------------------
Static Function AddMenuElement( NewItem  )
*--------------------------------------------------
    Local i , x:=1 , n , cx
    Local action , cAction , Caption, xCaption, cyMRU_Id, cxMRU_Id

    if len(NewItem) < 40
        Caption := NewItem
    else
        Caption := substr(NewItem, 1, 3) + '...' + substr(NewItem, len(NewItem)-34)
    endif
    cAction := '{|| OpenDBF( "'+NewItem + '" ) }'
    action := &cAction
  
    // Check if this is the first item
    If MRUCount == 0 
        // Modify a first element the menu
        cxMRU_Id := cMRU_Id
        _ModifyMenuItem ( cxMRU_Id , 'Form_Main' , '&1 '+caption , action )
        aadd( aMRU_File , { caption, NewItem, cxMRU_Id, action, 1 } )

    Else
        // Add a new element to the menu
        for n := 1 to Len( aMRU_File )+1
            x := ascan( aMRU_File , {|y| y[5] == n} )
            if x == 0
                x := n
                exit
            ENDIF
        next   

        cx := ltrim(str( x ))
        cyMRU_Id := cMRU_Id +'_'+ cx          
        cxMRU_Id := aMRU_File[1,3]
        _InsertMenuItem ( cxMRU_Id , 'Form_Main' , '&1 '+caption , action, cyMRU_Id )
        ASIZE(aMRU_File, Len(aMRU_File)+1) 
        AINS( aMRU_File, 1 ) 
        aMRU_File[ 1 ] := {caption, NewItem, cyMRU_Id, action, x}
        for n:= 1 to len(aMRU_File) 
            cx:= alltrim(str( n ))
            cxMRU_Id := aMRU_File[n, 3]
            xCaption := '&'+cx+' '+aMRU_File[n, 1]
            _ModifyMenuItem ( cxMRU_Id , 'Form_Main' , xCaption , aMRU_File[n, 4] ) 
        next
        if len(aMRU_File) > maxMRU_Files
            cxMRU_Id := aMRU_File[ Len(aMRU_File) , 3 ]
            ASIZE( aMRU_File , maxMRU_Files ) 
            _RemoveMenuItem( cxMRU_Id , 'Form_Main' )
        endif

    Endif
    //Increment the menu count
    MRUCount++
      
Return Nil

*--------------------------------------------------
Static Function ReorderMRUList( DuplicateLocation )
*--------------------------------------------------
    Local cxMRU_Id

    // Move entries previously "more recent" than the
    // duplicate down one in the MRU list
    if DuplicateLocation > 1
        cxMRU_Id := aMRU_File[ DuplicateLocation , 3 ] 
        _RemoveMenuItem( cxMRU_Id , 'Form_Main')
        ADEL( aMRU_File, DuplicateLocation ) 
        ASIZE( aMRU_File, Len(aMRU_File) - 1 )
    endif
   
Return Nil

*--------------------------------------------------
Static Function GetMRUFileList()
*--------------------------------------------------
    Local i := 1, aTmp:={}
    Local result       

    BEGIN INI FILE cFileIni
   
    Do while i <= MaxMRU_Files
      // Retrieve entry from INI
       
      result := GetSetting("Recent File List", LTrim(Str(i)))
      // Check if a value was returned
      If !empty(result) 
         Aadd(aTmp, result)
      else
         exit
      Endif
      i++
    enddo
    END INI	
    for i := Len(aTmp) to 1 step -1
         AddMRUItem( aTmp[i] )
    next
Return Nil

*--------------------------------------------------
Static Function SaveMRUFileList()
*--------------------------------------------------
    Local i , cFile     
   
    BEGIN INI FILE cFileIni
    // Loop through all MRU
    For i := 1 To maxMRU_Files

	// Write MRU to INI with key as it's position in list
        if i <= Len(aMRU_File)
            cFile := aMru_File[ i , 2 ]
        else
            cFile := ""
        endif
        SET SECTION "Recent File List" ENTRY LTrim(Str(i)) TO cFile
    Next 
    END INI	
Return Nil

*--------------------------------------------------
Static Function GetSetting (cSec, cEnt)
*--------------------------------------------------
    Local cVal := ''

    GET cVal SECTION cSec ENTRY cEnt DEFAULT ""
 
Return cVal
