/*
 * MINIGUI - Harbour Win32 GUI library Demo
 *
 * Copyright 2002-06 Roberto Lopez <harbourminigui@gmail.com>
 * http://harbourminigui.googlepages.com/
 *
 * Copyright 2003-07 Grigory Filatov <gfilatov@freemail.ru>
*/

ANNOUNCE RDDSYS

#include "minigui.ch"

#define PROGRAM 'BackUp'
#define VERSION ' ver.1.2'
#define COPYRIGHT ' 2003-2007 by Grigory Filatov'

#define CRLF HB_OsNewLine()
#define NTRIM( n ) LTrim( Str( n ) )

#define MsgInfo( c ) MsgInfo( c, "Information", , .f. )
#define MsgAlert( c ) MsgEXCLAMATION( c, "Attention", , .f. )
#define MsgStop( c ) MsgStop( c, "Stop!", , .f. )

STATIC lConfirm := "", nCompress := 6, cSkin := "", cTarget := "", cZip := ""
STATIC cMsgCancel := "", cMsgSkin := "", cMsgConf := "", cMsgStart := "", ;
       cMsgDir := "", cMsgEmpty := "", cMsgEnd := "", cMsgFailed := "", cMsgCreate := ""

MEMVAR aInfo, nTotLen
MEMVAR nActualTot
*--------------------------------------------------------*
PROCEDURE Main()
*--------------------------------------------------------*
LOCAL aMask := AppInit()

   IF Len(aMask) > 0

	IF !lConfirm

		do_backup(aMask)

	ELSEIF lConfirm .and. MsgYesNo(cMsgStart, cMsgConf, , , .f.)

		do_backup(aMask)

	ENDIF

   ENDIF

RETURN

*--------------------------------------------------------*
FUNCTION AppInit()
*--------------------------------------------------------*
LOCAL nI, cMask := "", aMask := {}

   BEGIN INI FILE DirPrg()+"\backup.ini"

	GET lConfirm SECTION "BACKUP" ;
		ENTRY "Confirm start" default "Yes"
	lConfirm := (lConfirm == "Yes")

	GET nCompress SECTION "BACKUP" ;
		ENTRY "Compression ratio" default nCompress

	GET cTarget SECTION "BACKUP" ;
		ENTRY "Target" default "BACKUP"

	GET cSkin SECTION "BACKUP" ;
		ENTRY "Skin" default "Backup.bmp"

	For nI := 1 To 99

		GET cMask SECTION "SOURCE" + Str(nI, 2) + "/99" ;
			ENTRY "Files" default ""

		IF !Empty(cMask)
			Aadd(aMask, cMask)
		ELSE
			Exit
		ENDIF
	Next

	GET cMsgCancel SECTION "LANGUAGE" ;
		ENTRY "1" default "Cancel this backup?"

	GET cMsgSkin SECTION "LANGUAGE" ;
		ENTRY "2" default "Cannot find skin %1..."

	GET cMsgConf SECTION "LANGUAGE" ;
		ENTRY "4" default "Confirm start"

	GET cMsgStart SECTION "LANGUAGE" ;
		ENTRY "9" default "Start backup now?"

	GET cMsgDir SECTION "LANGUAGE" ;
		ENTRY "10" default "Cannot find directory %1..."

	GET cMsgEmpty SECTION "LANGUAGE" ;
		ENTRY "85" default "No files to back-up..."

	GET cMsgEnd SECTION "LANGUAGE" ;
		ENTRY "86" default "Back-up ready!"

	GET cMsgFailed SECTION "LANGUAGE" ;
		ENTRY "89" default "Back-up to %1 failed..."

	GET cMsgCreate SECTION "LANGUAGE" ;
		ENTRY "90" default "Cannot create %1..."

   END INI

return aMask

*--------------------------------------------------------*
FUNCTION do_backup(aMask)
*--------------------------------------------------------*
LOCAL aFiles := {}, aDir := {}, nVar, cMack, cPath
PRIVATE aInfo := {}, nTotLen := 0

   IF AT("%D", cTarget) > 0
	cTarget := STRTRAN(cTarget, "%D", StrZero(Day(Date()), 2))
   ENDIF
   IF AT("%M", cTarget)>0
	cTarget := STRTRAN(cTarget,"%M", StrZero(Month(Date()), 2))
   ENDIF
   IF AT("%Y", cTarget)>0
	cTarget := STRTRAN(cTarget, "%Y", Right(Str(Year(Date())), 2))
   ENDIF
   IF AT("%H", cTarget)>0
	cTarget := STRTRAN(cTarget, "%H", CurDrive() + ":\" + CurDir())
	cTarget := STRTRAN(cTarget, "\\", "\")
   ENDIF
   IF AT("%C", cTarget)>0
	cTarget := STRTRAN(cTarget, "%C", CurDrive() + ":")
   ENDIF
   IF AT("%W", cTarget)>0
	cTarget := STRTRAN(cTarget, "%W", GetWindowsFolder())
   ENDIF
   IF AT("%S", cTarget)>0
	cTarget := STRTRAN(cTarget, "%S", GetSystemFolder())
   ENDIF
   cZip := if(At(".", cFileNoPath(cTarget)) = 0, cTarget + ".ZIP", cTarget)

   cPath := cFilePath( cZip )
   IF !lIsDir( cPath )
	IF ( MAKEDIR( cPath ) # 0 )
		MsgStop( STRTRAN(cMsgCreate, "%1", cPath) )
		return nil
	EndIF
   EndIF

   FOR nVar := 1 TO LEN(aMask)
	cMack := aMask[nVar]
	IF AT("%H", cMack) > 0
		cMack := STRTRAN(cMack, "%H", CurDrive() + ":\" + CurDir())
	ENDIF
	IF AT("%C", cMack) > 0
		cMack := STRTRAN(cMack, "%C", CurDrive() + ":")
	ENDIF
	IF AT("%W", cMack) > 0
		cMack := STRTRAN(cMack, "%W", GetWindowsFolder())
	ENDIF
	IF AT("%S", cMack) > 0
		cMack := STRTRAN(cMack, "%S", GetSystemFolder())
	ENDIF

	cPath := cFilePath( cMack )

	If !lIsDir( cPath )
		MsgAlert( STRTRAN(cMsgDir, "%1", cPath) )
	EndIf
	aDir := Directory( cMack )
	IF LEN(aDir) > 0
		AEVAL(aDir, {|ele| AADD(aFiles, cPath + "\" + ele[1]), ;
			AADD(aInfo, {cPath+"\"+ele[1], ele[2], ele[3], ele[4]}), ;
			nTotLen += ele[2]})
	ENDIF
   NEXT

   IF LEN(aFiles)==0
	MsgStop( cMsgEmpty )
	return nil
   ENDIF

   DEFINE WINDOW Form_1 ; 
      AT 0,0 ; 
      WIDTH 588 ; 
      HEIGHT IF(IsXPThemeActive(), 187, 180) ; 
      TITLE PROGRAM + VERSION ; 
      ICON 'ICON_1' ;
      MAIN ;
      NOMINIMIZE NOMAXIMIZE ;
      NOSIZE ;
      ON INIT ZipFiles(aFiles) ;
      FONT "MS Sans Serif" ;
      SIZE 8

   if FILE( DirPrg() + "\SKINS\" + cSkin )

      @ -2,-2 IMAGE Image_1 ; 
            PICTURE DirPrg()+"\SKINS\"+cSkin ; 
            WIDTH Form_1.Width - 2; 
            HEIGHT Form_1.Height - IF(IsXPThemeActive(), 27, 20)
   endif

      @ 34,53 LABEL Label_1 ; 
            WIDTH 475 HEIGHT 14 ;
            FONTCOLOR BLUE CENTERALIGN

      @ 72,52 IMAGE Image_2 ; 
            PICTURE "ANIM1" ; 
            WIDTH 15 HEIGHT 20

	@ 72, 78 PROGRESSBAR Progress_1	;
            RANGE 0, 100 			;
            WIDTH 425 HEIGHT 20 SMOOTH

      @ Form_1.Height - IF(IsXPThemeActive(), 47, 40),6 LABEL Label_2 ; 
            VALUE "Copyright " + Chr(169) + COPYRIGHT ;
            WIDTH 220 HEIGHT 14 ;
            TRANSPARENT

   END WINDOW

   if !FILE( DirPrg() + "\SKINS\" + cSkin )

	MsgAlert( STRTRAN(cMsgSkin, "%1", cSkin) )

   endif

   CENTER WINDOW Form_1

   ACTIVATE WINDOW Form_1

RETURN NIL

*--------------------------------------------------------*
FUNCTION ZipFiles(aFiles)
*--------------------------------------------------------*
LOCAL cBmp := "ANIM", nInterval := 50, nCount := 0, nMax := 17
PRIVATE nActualTot := 0

	nCompress := IF(nCompress < 0, 0, IF(nCompress > 9, 9, nCompress))

	DEFINE TIMER Timer_1 OF Form_1 ;
            INTERVAL nInterval ;
            ACTION ( nCount++, nCount := IIf(nCount > nMax, 1, nCount), ;
		Form_1.Image_2.Picture := cBmp + NTRIM(nCount) )

	COMPRESS aFiles 					;
            TO cZip 						;
            BLOCK {|cFile, nPos| ProgressUpdate(nPos, cFile) }	;
            LEVEL nCompress					;
            OVERWRITE STOREPATH

	Form_1.Timer_1.Enabled := .f.
	Form_1.Progress_1.Value := 100

	IF FILE(cZip) .AND. DIRECTORY(cZip)[1][2] > 22
		Form_1.Label_1.Value := cMsgEnd
		Form_1.Image_2.Hide
		Form_1.Image_2.Picture := "ZIP_OK"
		Form_1.Image_2.Show
		PlayOK()
	ELSE
		Form_1.Label_1.Hide
		Form_1.Label_1.Value := STRTRAN(cMsgFailed, "%1", cZip)
		Form_1.Label_1.FontColor := RED
		Form_1.Label_1.Show
		Form_1.Image_2.Hide
		Form_1.Image_2.Picture := "ZIP_NOK"
		Form_1.Image_2.Show
		PlayExclamation()
	ENDIF

	INKEY(3)
	Form_1.Release

RETURN NIL

*------------------------------------------------------------------------------*
STATIC FUNCTION ProgressUpdate(nPos, cFile)
*------------------------------------------------------------------------------*
LOCAL nFileSize := aInfo[nPos][2]

	nActualTot += nFileSize
	Form_1.Progress_1.Value := Int( ( nActualTot / nTotLen ) * 100 )

	Form_1.Label_1.Value := Lower(cFile + " ==> " + cZip)
	DO EVENTS

Return Nil

*--------------------------------------------------------*
STATIC FUNCTION lIsDir( cDir )
*--------------------------------------------------------*
LOCAL lExist := .T.

   IF DIRCHANGE( cDir ) > 0
	lExist := .F.
   ELSE
	DIRCHANGE( DirPrg() )
   ENDIF

RETURN lExist

*--------------------------------------------------------*
STATIC FUNCTION DirPrg()
*--------------------------------------------------------*
Local cFolder := GetStartupFolder()
RETURN IF(Right(cFolder, 1) == "\", Left(cFolder, Len(cFolder) - 1), cFolder)
