/*
File:         DualBrowse_DBC.prg
Author:       Dr Joe Fanucchi <drjoe@meditrax.com>, <meditrax_drjoe@yahoo.com>
Description:  Opens a file in two browse windows at once. The second window contains only
              items selected from the first window. Shows use of DYNAMICBACKCOLOR and
              DYNAMICFORECOLOR in browse.
Status:       Public Domain
*/
* This function enables the user to select items from a master "Catalog" browse,
* and display the selected items in a separate "Wanted" list. Items which have
* been selected are displayed on a yellow background in the "Catalog" list.
* Items may be removed from the "Wanted" list, and the quantity of items in the
* "Wanted" list may be edited.
*
* Items in the "Catalog" list may be displayed alphabetically, or in numeric code
* order.
*--------------------------------------------------------------------------------*

#include "minigui.ch"

DECLARE WINDOW SelectItems
DECLARE WINDOW Form_1

* #DEFINE cs2UnderScores "__"
#DEFINE cs4sp          SPACE( 4 )

*------------------------------------------------------------*
FUNCTION MAIN
*------------------------------------------------------------*
LOCAL bColor1    := { || IIF( PickedIt(), RGB(255,255,0), RGB(255,255,255)) }
LOCAL bColor2    := { || RGB(0,0,0) }
LOCAL bColor3    := { || IIF( MoreThanOne(), RGB(208,68,68), RGB(0,0,0)) }

PRIVATE aChosenRecs := { 0 }
PRIVATE aQuantity   := { 0 }
PRIVATE nPicked     := 0

SET BROWSESYNC ON

DEFINE WINDOW SelectItems ;
  AT 0,0 ;
  WIDTH 910 HEIGHT 543 ;
  MAIN NOSIZE ;
  TITLE ' MiniGUI Dual Browse with DynamicBackColor - ' + CHR( 169) + ' Dr Joe Fanucchi 2005' ;
  ON INIT { || OpenTables() } ;
  ON RELEASE { || DBCLOSEALL() }

  *----------------------------------------------------------*
  * Frame the browse windows:
  @ 13,15 FRAME Frame_Catalog ;
    WIDTH 355 HEIGHT 369 ;
    FONT "Arial" BOLD SIZE 10 ;
    CAPTION "Items in Catalog:"

  @ 13,509 FRAME Frame_Wanted ;
    WIDTH 381 HEIGHT 369 ;
    FONT "Arial" BOLD SIZE 10 ;
    CAPTION "Items Wanted:"

  *----------------------------------------------------------*
  * Display the browse windows:
  @ 38,18 BROWSE Brw_Catalog ;
    WIDTH 349 ;
    HEIGHT 340 ;
    HEADERS { 'Item', 'Code', 'Amount' } ;
    WIDTHS { 218, 50, 60 } ;
    WORKAREA CATALOG ;
    FIELDS { 'CATALOG->itemname', 'CATALOG->itemcode', 'CATALOG->amount' } ;
    DYNAMICBACKCOLOR { bColor1, bColor1, bColor1 } ;
    JUSTIFY { , , BROWSE_JTFY_RIGHT } ;
    ON GOTFOCUS ( SelectItems.Btn_Remove.Enabled := .F., ;
    SelectItems.Btn_Modify.Enabled := .F., ;
    SelectItems.Spin_Qty.Enabled  := .F., ;
    SelectItems.Btn_Add.Enabled := .T., ;
    lIncremental := .T. ) ;
    ON HEADCLICK { {|| ShowByName() }, { || ShowByCode() } } ;
    ON DBLCLICK PutItOn()

  @ 38,512 BROWSE Brw_Wanted ;
    WIDTH 375 ;
    HEIGHT 340 ;
    HEADERS { 'Service', 'Code', '#', 'Amount' } ;
    WIDTHS { 218, 50, 26, 60 } ;
    WORKAREA WANTED ;
    FIELDS { 'WANTED->itemname', 'WANTED->itemcode', 'LTRIM( STR( aQuantity[ ASCAN( aChosenRecs, RECNO()) ], 3 ))', ;
    'STR( WANTED->amount * aQuantity[ ASCAN( aChosenRecs, RECNO()) ], 8, 2 )' } ;
    DYNAMICFORECOLOR { bColor2, bColor2, bColor3, bColor3 } ;
    JUSTIFY { , , BROWSE_JTFY_RIGHT, BROWSE_JTFY_RIGHT } ;
    ON GOTFOCUS ShowSvcButtons()

  *----------------------------------------------------------*
  * Buttons for adding, removing, and editing quantity:
  @ 114,379 BUTTON Btn_Add ;
    CAPTION "Add >>" ;
    ACTION { || PutItOn() } ;
    WIDTH 120 HEIGHT 28

  @ 172,379 BUTTON Btn_Remove ;
    CAPTION "<< Remove" ;
    ACTION { || TakeItOff() } ;
    WIDTH 120 HEIGHT 28

  @ 206,379 BUTTON Btn_Modify ;
    CAPTION "Edit Quantity" ;
    ACTION { || EditQuantity() } ;
    WIDTH 120 HEIGHT 28

  @ 242,409 SPINNER Spin_Qty ;
    RANGE 0,20 ;
    WIDTH 60

  @ 274,379 BUTTON Btn_QtyOK ;
    CAPTION "Accept Quantity" ;
    ACTION { || UpdateQty(), UpdateChgTotals()} ;
    WIDTH 120 HEIGHT 28

  *----------------------------------------------------------*
  * This frame will be used for an incremental search function and display of total cost
  * The incremental search is NOT implemented in this demo
  @ 392,15 FRAME Frame_Navig ;
    WIDTH 875 HEIGHT 50

  @ 410,30 LABEL AlfaSearch ;
    WIDTH 127 HEIGHT 18 ;
    VALUE 'Alphabetic Listing' ;
    FONT "Arial" BOLD SIZE 10

  @ 410,30 LABEL CodeSearch ;
    WIDTH 127 HEIGHT 18 ;
    VALUE 'Listing by Code' ;
    FONT "Arial" BOLD SIZE 10

*   @ 410,152 LABEL SearchText ;
*     WIDTH 220 HEIGHT 18 ;
*     VALUE cs2UnderScores ;
*     FONT "Arial" BOLD SIZE 10

  @ 410,605 LABEL Lbl_ChgExpl ;
    WIDTH 200 HEIGHT 18 ;
    VALUE "Total Charges: $" ;
    FONT "Arial" BOLD SIZE 10 ;
    RIGHTALIGN

  @ 410,805 LABEL Lbl_ChgTotal ;
    WIDTH 58 HEIGHT 18 ;
    VALUE '0.00' ;
    FONT "Arial" SIZE 10 ;
    RIGHTALIGN

  *----------------------------------------------------------*
  @ 450,15 FRAME Frame_Save ;
    WIDTH 875 HEIGHT 50

  @ 462,320 BUTTON Btn_Save ;
    CAPTION "Accept" ;
    ACTION { || ShowSelections(), nPicked := 0, StartOver() } ;
    WIDTH 120 HEIGHT 28

  @ 462, 470 BUTTON Btn_Cancel ;
    CAPTION "Cancel" ;
    ACTION { || nPicked := 0, StartOver() } ;
    WIDTH 120 HEIGHT 28

  SelectItems.Btn_Remove.Enabled := .F.
  SelectItems.Spin_Qty.Visible   := .F.
  SelectItems.Btn_QtyOK.Visible  := .F.
  SelectItems.CodeSearch.Visible := .F.

END WINDOW

CENTER WINDOW SelectItems

ACTIVATE WINDOW SelectItems

RETURN NIL
*
*------------------------------------------------------------*
STATIC FUNCTION EditQuantity
*------------------------------------------------------------*
* Enables the user to update the quantity of an individual item
* by making the "quantity" SPINNER and the "accept" BUTTON visible

LOCAL nElement := ASCAN( aChosenRecs, WANTED->( RECNO()))

SelectItems.Spin_Qty.Value := aQuantity[ nElement ]

SelectItems.Btn_Remove.Enabled := .F.
SelectItems.Btn_Modify.Enabled := .F.
SelectItems.Spin_Qty.Visible   := .T.
SelectItems.Spin_Qty.Enabled   := .T.
SelectItems.Btn_QtyOK.Visible  := .T.

RETURN NIL
*
*------------------------------------------------------------*
STATIC FUNCTION MoneyComma( cMoneyString, nLength )
*------------------------------------------------------------*
* Formats money amounts in US $, adding commas before thousands
* e.g.: 12337.89 -> 12,337.89
LOCAL cDollars   := cMoneyString
LOCAL cPennies := ""
LOCAL i
LOCAL lInTheHole := .F.

IF ! VALTYPE( nLength ) == "N"
  nLength := LEN( cMoneyString )
ENDIF

IF VAL( cMoneyString ) < 0.00
  lInTheHole := .T.
  cMoneyString := CharRem( { "-", " " }, cMoneyString )
ENDIF

cMoneyString := LTRIM( TRIM( cMoneyString ))

IF "." $ cMoneyString                                // cents are shown
  cDollars   := LEFT( cMoneyString, AT( ".", cMoneyString ) - 1 )
  cPennies := RIGHT( cMoneyString, 2 )
ELSE
  cDollars := cMoneyString
ENDIF

FOR i := 3 TO 15 STEP 4                              // add commas to money display: 1,234,567.89
  IF LEN( cDollars ) > i
    PutCashComma( @cDollars, i )                     // 000,000
  ENDIF
NEXT i

IF lInTheHole                                        // we started out with a negative amount
  cDollars := "-" + cDollars
ENDIF

IF "." $ cMoneyString                                // cents are shown
  IF LEN( cDollars + "." + cPennies ) > nLength
    nLength := LEN( cDollars ) + 3
  ENDIF
  RETU PADL( cDollars + "." + cPennies, nLength )
ENDIF

IF LEN( cDollars ) > nLength
  nLength := LEN( cDollars )
ENDIF

RETURN PADL( cDollars, nLength )
*
*------------------------------------------------------------*
STATIC FUNCTION MoreThanOne
*------------------------------------------------------------*
* Determines whether a multiple quantity of a WANTED item has
* been ordered. Called by the DYNAMICFORECOLOR control.
LOCAL nElement := ASCAN( aChosenRecs, WANTED->( RECNO()))

RETURN ( aQuantity[ nElement ] >= 2 )
*
*------------------------------------------------------------*
STATIC FUNCTION OpenTables
*------------------------------------------------------------*
* Open the same data table in two windows
SELECT 0
USE SUPPLIES
IF ! FILE( "FALFA001.NTX" )
  INDEX ON itemname TO FALFA001
ENDIF
IF ! FILE( "FCODE001.NTX" )
  INDEX ON itemcode TO FCODE001
ENDIF
CLOSE

SELECT 0
USE SUPPLIES ALIAS "CATALOG" SHARED
SET INDEX TO FALFA001, FCODE001

SELECT 0
USE SUPPLIES ALIAS "WANTED" SHARED
SET INDEX TO FALFA001, FCODE001
SET FILTER TO ASCAN( aChosenRecs, WANTED->( RECNO())) > 0

RETURN NIL
*
*------------------------------------------------------------*
STATIC FUNCTION PickedIt
*------------------------------------------------------------*
* Determines whether a CATALOG item has been selected. Called by
* the DYNAMICBACKCOLOR control.

RETURN ASCAN( aChosenRecs, CATALOG->( RECNO())) > 0
*
*------------------------------------------------------------*
FUNCTION PutCashComma( cMoneyString, nCommaHere )
*------------------------------------------------------------*
* Inserts a comma into the string
cMoneyString := LEFT( cMoneyString, LEN( cMoneyString ) - nCommaHere ) ;
  + "," + RIGHT( cMoneyString, nCommaHere )

RETURN NIL
*
*------------------------------------------------------------*
STATIC FUNCTION PutItOn()
*------------------------------------------------------------*
* Adds an item from the "Catalog" browse to the "Wanted" browse,
* by adding the RECNO() to the array of chosen items. Prevents
* duplicate selection of the same item.

LOCAL nThisRecNo := CATALOG->( RECNO())

IF ASCAN( aChosenRecs, nThisRecNo ) == 0
  ASIZE( aChosenRecs, ++nPicked )
  ASIZE( aQuantity, nPicked )
  aChosenRecs[ nPicked ] := nThisRecNo
  aQuantity[ nPicked ] := 1
  SelectItems.Brw_Wanted.Value := nThisRecNo
  SelectItems.Brw_Wanted.Refresh
  UpdateChgTotals()
ELSE
  MsgExclamation( "This item has already been selected.", " Duplicate Selection !" )
ENDIF

SelectItems.Brw_Catalog.Refresh
SelectItems.Brw_Catalog.SetFocus

RETURN NIL
*
*------------------------------------------------------------*
STATIC FUNCTION ShowByCode
*------------------------------------------------------------*
* Changes the display of items in both browse windows so they
* appear in order of a numeric code.
LOCAL nThisRecNo := WANTED->( RECNO())
SELECT WANTED
SET ORDER TO 2
GO nThisRecNo
SelectItems.Brw_Wanted.Refresh

nThisRecNo := CATALOG->( RECNO())
SELECT CATALOG
SET ORDER TO 2
GO nThisRecNo
SelectItems.Brw_Catalog.Refresh

SelectItems.AlfaSearch.Visible := .F.
SelectItems.CodeSearch.Visible := .T.
* SelectItems.SearchText.Col     := 136

RETURN NIL
*
*------------------------------------------------------------*
STATIC FUNCTION ShowByName
*------------------------------------------------------------*
* Changes the display of items in both browse windows so they
* appear in alphabetic order.
LOCAL nThisRecNo := WANTED->( RECNO())

SELECT WANTED
SET ORDER TO 1
GO nThisRecNo
SelectItems.Brw_Wanted.Refresh

nThisRecNo := CATALOG->( RECNO())
SELECT CATALOG
SET ORDER TO 1
GO nThisRecNo
SelectItems.Brw_Catalog.Refresh

* SelectItems.CodeSearch.Visible := .F.
* SelectItems.AlfaSearch.Visible := .T.
* SelectItems.SearchText.Col     := 152

RETURN NIL
*
*------------------------------------------------------------*
STATIC FUNCTION ShowSelections()
*------------------------------------------------------------*
* Displays the items which have been selected. In my production code, this
* message is not displayed, but the array is transferred to a printout.
LOCAL cMessage := "You selected:"

IF nPicked < 1
  MsgInfo( "You did not select any items." + cs4sp, " No Selections Made !" )
  RETU NIL
ELSE
  SELECT WANTED
  SET ORDER TO 1
  GO TOP
  WHILE ! EOF()
    IF ASCAN( aChosenRecs, RECNO()) > 0
      cMessage += CRLF + "  " + TRIM( WANTED->itemname ) ;
        + " [ " + LTRIM( STR( aQuantity[ ASCAN( aChosenRecs, RECNO()) ], 3 )) ;
        + " @ " + LTRIM( STR( WANTED->amount, 8, 2 )) + " ]" ;
        + cs4sp                                      // padding at the end of each line helps improve readability
    ENDIF
    SKIP
  ENDDO
  MsgInfo( cMessage, " Selected Items" )
ENDIF
RETURN NIL
*
*------------------------------------------------------------*
STATIC FUNCTION ShowSvcButtons()
*------------------------------------------------------------*
* Enables the buttons which modify or remove items from the WANTED browse.
IF nPicked > 0
  SelectItems.Btn_Remove.Enabled := .T.
  SelectItems.Btn_Modify.Enabled := .T.
  SelectItems.Btn_Add.Enabled    := .F.
  lIncremental := .F.
ENDIF

RETURN NIL
*
*------------------------------------------------------------*
STATIC FUNCTION StartOver
*------------------------------------------------------------*
aChosenRecs := { 0 }
aQuantity := { 0 }
SelectItems.Lbl_ChgTotal.Value := "0.00"

SELECT WANTED
GO TOP
SelectItems.Brw_Wanted.Value := RECNO()
SelectItems.Brw_Wanted.Refresh

SELECT CATALOG
GO TOP
_PushKey( VK_HOME )
SelectItems.Brw_Catalog.Refresh
SelectItems.Brw_Catalog.SetFocus

RETURN NIL
*
*------------------------------------------------------------*
STATIC FUNCTION TakeItOff()
*------------------------------------------------------------*
* Removes a selected item from the WANTED browse
LOCAL nThisRecNo := RECNO()
LOCAL nElement   := ASCAN( aChosenRecs, WANTED->( RECNO()))

IF nElement > 0
  ADEL( aChosenRecs, nElement )
  ADEL( aQuantity, nElement )
  --nPicked
  UpdateChgTotals()
ENDIF

SELECT WANTED
GO TOP
SelectItems.Brw_Wanted.Value := RECNO()
SelectItems.Brw_Wanted.Refresh

SELECT CATALOG
SelectItems.Brw_Catalog.SetFocus
SelectItems.Brw_Catalog.Value := nThisRecNo
SelectItems.Brw_Catalog.Refresh

RETURN NIL
*
*------------------------------------------------------------*
STATIC FUNCTION UpdateChgTotals()
*------------------------------------------------------------*
* Displays the total cost of selected items
LOCAL i
LOCAL cAlias := ALIAS()
LOCAL nRecNo := RECNO()
LOCAL nTotal := 0

IF nPicked > 0
  SELECT CATALOG
  FOR i := 1 TO nPicked
    GO aChosenRecs[ i ]
    nTotal += CATALOG->amount * aQuantity[ i ]
  NEXT i
ENDIF

SelectItems.Lbl_ChgTotal.Value := LTRIM( MoneyComma( STR( nTotal, 8, 2 ), 9 ))

SelectItems.Btn_Save.Enabled   := .T.
SelectItems.Btn_Cancel.Enabled := .T.

IF cAlias == "CATALOG"
  SELECT CATALOG
  GO nRecNo
  SelectItems.Brw_Catalog.Refresh
  SelectItems.Brw_Catalog.SetFocus
ELSE
  SELECT WANTED
  GO nRecNo
  SelectItems.Brw_Wanted.Refresh
  SelectItems.Brw_Wanted.SetFocus
ENDIF

RETURN NIL
*
*------------------------------------------------------------*
STATIC FUNCTION UpdateQty
*------------------------------------------------------------*
* Enables the user to modify the quantity of WANTED items.
LOCAL nElement

IF SELECT( "WANTED" ) > 0
  nElement := ASCAN( aChosenRecs, WANTED->( RECNO()))
  aQuantity[ nElement ] := SelectItems.Spin_Qty.Value
  SelectItems.Btn_QtyOK.Visible := .F.
  SelectItems.Spin_Qty.Visible  := .F.
  SelectItems.Brw_Wanted.Refresh
  SelectItems.Brw_Wanted.SetFocus
ENDIF

RETURN NIL
*
*------------------------------------------------------------*
* EOF()
*------------------------------------------------------------*