/*----------------------------------------------------------------------------
MINIGUI - Harbour Win32 GUI library source code

Copyright 2002-2008 Roberto Lopez <harbourminigui@gmail.com>
http://harbourminigui.googlepages.com/

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along with
   this software; see the file COPYING. If not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA (or
   visit the web site http://www.gnu.org/).

   As a special exception, you have permission for additional uses of the text
   contained in this release of Harbour Minigui.

   The exception is that, if you link the Harbour Minigui library with other
   files to produce an executable, this does not by itself cause the resulting
   executable to be covered by the GNU General Public License.
   Your use of that executable is in no way restricted on account of linking the
   Harbour-Minigui library code into it.

   Parts of this project are based upon:

   "Harbour GUI framework for Win32"
   Copyright 2001 Alexander S.Kresin <alex@belacy.belgorod.su>
   Copyright 2001 Antonio Linares <alinares@fivetech.com>
   www - http://www.harbour-project.org

   "Harbour Project"
   Copyright 1999-2008, http://www.harbour-project.org/

   "WHAT32"
   Copyright 2002 AJ Wos <andrwos@aust1.net>

   "HWGUI"
   Copyright 2001-2007 Alexander S.Kresin <alex@belacy.belgorod.su>

---------------------------------------------------------------------------*/

#include "minigui.ch"
#include "error.ch"

#ifdef _TSBROWSE_
   MEMVAR _TSB_aControlhWnd
#endif
*-----------------------------------------------------------------------------*
PROCEDURE ErrorSys
*-----------------------------------------------------------------------------*

   ErrorBlock( { | oError | DefError( oError ) } )
#ifndef __XHARBOUR__
   Set( _SET_HBOUTLOG, GetStartUpFolder() + "\error.log" )
   Set( _SET_HBOUTLOGINFO, MiniGUIVersion() )
#endif
   Init()

RETURN

*-----------------------------------------------------------------------------*
STATIC FUNCTION DefError( oError )
*-----------------------------------------------------------------------------*
   LOCAL n := 2, cText
   LOCAL HtmArch, HtmText

   // By default, division by zero results in zero
   IF oError:genCode == EG_ZERODIV
      RETURN 0
   ENDIF

   // Set NetErr() if there was a database open error
   IF oError:genCode == EG_OPEN .AND.;
      oError:osCode == 32 .AND.;
      oError:canDefault
      NetErr( .T. )
      RETURN .F.
   ENDIF

   // Set NetErr() if there was a lock error on dbAppend()
   IF oError:genCode == EG_APPENDLOCK .AND.;
      oError:canDefault
      NetErr( .T. )
      RETURN .F.
   ENDIF

   HtmArch := Html_ErrorLog()
   cText := ErrorMessage( oError )

   Html_LineText( HtmArch, '<p class="updated">Date: ' + Dtoc(Date()) + "  " + "Time: " + Time() )
   Html_LineText( HtmArch, cText + "</p>" )

   cText += CRLF + CRLF
   WHILE ! Empty( ProcName( n ) )
      HtmText := "Called from " + ProcName( n ) + "(" + AllTrim( Str( ProcLine( n++ ) ) ) + ")"  + CRLF
      cText += HtmText
      Html_LineText( HtmArch, HtmText )
   ENDDO
   Html_Line(HtmArch)
   Html_End(HtmArch)

   ShowError( cText, oError )

RETURN .F.

*-----------------------------------------------------------------------------*
STATIC FUNCTION ErrorMessage( oError )
*-----------------------------------------------------------------------------*
   LOCAL cMessage

   // start error message
   cMessage := iif( oError:severity > ES_WARNING, "Error", "Warning" ) + " "

   // add subsystem name if available
   IF ISCHARACTER( oError:subsystem )
      cMessage += oError:subsystem()
   ELSE
      cMessage += "???"
   ENDIF

   // add subsystem's error code if available
   IF ISNUMBER( oError:subCode )
      cMessage += "/" + LTrim( Str( oError:subCode ) )
   ELSE
      cMessage += "/???"
   ENDIF

   // add error description if available
   IF ISCHARACTER( oError:description )
      cMessage += "  " + oError:description
   ENDIF

   // add either filename or operation
   DO CASE
   CASE !Empty( oError:filename )
      cMessage += ": " + oError:filename
   CASE !Empty( oError:operation )
      cMessage += ": " + oError:operation
   ENDCASE

   // add OS error code if available
   IF !Empty( oError:osCode )
      cMessage += " (DOS Error " + LTrim( Str( oError:osCode ) ) + ")"
   ENDIF

RETURN cMessage

*-----------------------------------------------------------------------------*
PROCEDURE ShowError( ErrorMessage, oError )
*-----------------------------------------------------------------------------*
   STATIC _lShowError := .T.
   LOCAL ix, ControlCount

   If _lShowError

      _lShowError := .F.

      ControlCount := Len ( _HMG_aControlHandles )

      For ix := 1 To ControlCount
         If _HMG_aControlType [ix] == 'HOTKEY'
            ReleaseHotKey ( _HMG_aControlParentHandles [ix] , _HMG_aControlIds [ix] )
         ElseIf _HMG_aControlType [ix] == 'TIMER'
            KillTimer ( _HMG_aControlParentHandles [ix] , _HMG_aControlIds [ix] )
         ElseIf _HMG_aControlType [ix] == 'FONT'
            _EraseFontDef (ix)
         EndIf
      Next
#ifdef _TSBROWSE_
      _TSB_aControlhWnd := {}
#endif
      dbcloseall()

      MsgStop( IIf( _lShowDetailError() .OR. oError==NIL, ErrorMessage,;
         ErrorMessage( oError ) ), 'Program Error', NIL, .F. )

   Endif

   ExitProcess(0)

RETURN

/*
   Date Created: 14/11/2005
   Author: Antonio Novo antonionovo@gmail.com
   Enable/Disable Error Detail
*/
*-----------------------------------------------------------------------------*
FUNCTION _lShowDetailError( lNewValue )
*-----------------------------------------------------------------------------*
   STATIC _lShowDetailError := .T.
   LOCAL lOldValue := _lShowDetailError

   If lNewValue <> Nil
      _lShowDetailError := lNewValue
   Endif

RETURN lOldValue

*-01-01-2003
*-AUTHOR: Antonio Novo
*-Create/Open the ErrorLog.Htm file
*-Note: Is used in: errorsys.prg and h_error.prg
*-----------------------------------------------------------------------------*
FUNCTION HTML_ERRORLOG
*-----------------------------------------------------------------------------*
   LOCAL HtmArch := -1
   LOCAL cErrorLogFile := _GetErrorlogFile()

   IF IsErrorLogActive()
      If .Not. File(cErrorLogFile)
         HtmArch := Html_Ini(cErrorLogFile, "Harbour MiniGUI Errorlog File")
         if HtmArch > 0
            Html_Line(HtmArch)
         endif
      Else
         HtmArch := FOPEN(cErrorLogFile, 2)
         if HtmArch > 0
            FSeek(HtmArch, 0, 2)    //End Of File
         endif
      EndIf
   ENDIF
RETURN (HtmArch)

*-30-12-2002
*-AUTHOR: Antonio Novo
*-HTML Page Head
*-----------------------------------------------------------------------------*
FUNCTION HTML_INI( ARCH, TITLE )
*-----------------------------------------------------------------------------*
   LOCAL HtmArch := -1
   LOCAL cStyle  := "<style> "                     +;
      "body{ "                       +;
      "font-family: sans-serif;"   +;
      "background-color: #ffffff;" +;
      "font-size: 75%;"            +;
      "color: #000000;"            +;
      "}"                          +;
      "h1{"                          +;
      "font-family: sans-serif;"   +;
      "font-size: 150%;"           +;
      "color: #0000cc;"            +;
      "font-weight: bold;"         +;
      "background-color: #f0f0f0;" +;
      "}"                          +;
      ".updated{"                    +;
      "font-family: sans-serif;"   +;
      "color: #cc0000;"            +;
      "font-size: 110%;"           +;
      "}"                          +;
      ".normaltext{"                 +;
      "font-family: sans-serif;"    +;
      "font-size: 100%;"            +;
      "color: #000000;"             +;
      "font-weight: normal;"        +;
      "text-transform: none;"       +;
      "text-decoration: none;"      +;
      "}"                            +;
      "</style>"
   IF IsErrorLogActive()
      HtmArch := FCREATE(ARCH)
      IF FERROR() != 0
         MsgStop("Can`t open errorlog file "+ARCH,"Error")
         RETURN (-1)
      ENDIF
      FWRITE(HtmArch,"<HTML><HEAD><TITLE>"+TITLE+"</TITLE></HEAD>"+cStyle+"<BODY>"+CHR(13)+CHR(10))
      FWRITE(HtmArch,'<H1 Align=Center>'+TITLE+'<br>'+MiniGUIVersion()+'</H1><BR>'+CHR(13)+CHR(10))
   ENDIF
RETURN (HtmArch)

*-30-12-2002
*-AUTHOR: Antonio Novo
*-HTM Page Line
*-----------------------------------------------------------------------------*
PROCEDURE HTML_LINETEXT( HTMARCH, LINEA )
*-----------------------------------------------------------------------------*
   IF HTMARCH > 0 .AND. IsErrorLogActive()
      FWRITE(HTMARCH, RTRIM( LINEA ) + "<BR>"+CHR(13)+CHR(10))
   ENDIF
RETURN

*-30-12-2002
*-AUTHOR: Antonio Novo
*-HTM Line
*-----------------------------------------------------------------------------*
PROCEDURE HTML_LINE( HTMARCH )
*-----------------------------------------------------------------------------*
   IF HTMARCH > 0 .AND. IsErrorLogActive()
      FWRITE(HTMARCH,"<HR>"+CHR(13)+CHR(10))
   ENDIF
RETURN

*-----------------------------------------------------------------------------*
PROCEDURE HTML_END( HTMARCH )
*-----------------------------------------------------------------------------*
   IF HTMARCH > 0 .AND. IsErrorLogActive()
      FWRITE(HTMARCH, "</BODY></HTML>")
      FCLOSE(HTMARCH)
   ENDIF
RETURN

// (JK) HMG 1.0 Build 6
*-----------------------------------------------------------------------------*
PROCEDURE _SetErrorLogFile( cFile )
*-----------------------------------------------------------------------------*
   _HMG_ErrorLogFile := IFEMPTY(cFile, GetStartUpFolder() + "\ErrorLog.htm", cFile)
RETURN