/*
 * Harbour Project source code:
 * gtnull.prg
 * source of GTnull.lib
 *
 * Created 2006 by Jacek Kubica <kubica@wssk.wroc.pl>
 * MiniGUI Team *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA (or visit the web site http://www.gnu.org/).
 *
 * As a special exception, the Harbour Project gives permission for
 * additional uses of the text contained in its release of Harbour.
 *
 * The exception is that, if you link the Harbour libraries with other
 * files to produce an executable, this does not by itself cause the
 * resulting executable to be covered by the GNU General Public License.
 * Your use of that executable is in no way restricted on account of
 * linking the Harbour library code into it.
 *
 * This exception does not however invalidate any other reasons why
 * the executable file might be covered by the GNU General Public License.
 *
 * This exception applies only to the code released by the Harbour
 * Project under the name Harbour.  If you copy code from other
 * Harbour Project or Free Software Foundation releases into a copy of
 * Harbour, as the General Public License permits, the exception does
 * not apply to the code that you add in this way.  To avoid misleading
 * anyone as to the status of such modified files, you must delete
 * this exception notice from them.
 *
 * If you write modifications of your own for Harbour, it is your choice
 * whether to permit this exception to apply to your modifications.
 * If you do not wish that, delete this exception notice.
 *
 */
ANNOUNCE GTSYS

PROCEDURE HB_GT_DEFAULT_NUL()
RETURN

/*
 * Tone support for Win32 GUI applications
 * Copyright 2006 Roberto Lopez <roblez@ciudad.com.ar>
 * Corrected by Grigory Filatov <gfilatov@freemail.ru>
 */

PROCEDURE HB_GT_WIN()
// I've added this because gt_win is automatically requested by gtsys.c
// So, if gtwin is not linked, an 'undefined reference' error is generated.
RETURN


#pragma BEGINDUMP

#define HB_OS_WIN_32_USED
#include <windows.h>
#include "hbapi.h"

static void hb_Tone( double dFrequency, double dDuration ) ;
static void beep9x( double dFreq, double dDurat ) ;
static void beepNt( double dFreq, double dDurat ) ;
static int hb_Outp9x( USHORT usPort, USHORT usVal ) ;
static int hb_Inp9x( USHORT usPort ) ;

HB_FUNC( TONE )
{
	hb_Tone( (DWORD) hb_parnl( 1 ), (DWORD) hb_parnl( 2 ) );
}

static void hb_Tone( double dFrequency, double dDuration )
{
	OSVERSIONINFOEX osvi;

	/*
	* According to the Clipper NG, the duration in 'ticks' is truncated to the
	* interger portion  ... Depending on the platform, Harbour allows a finer
	* resolution, but the minimum is 1 tick (for compatibility)
	*/
	/* Convert from ticks to seconds */
	dDuration  = ( HB_MIN( HB_MAX( 1.0, dDuration ), ULONG_MAX ) ) / 18.2 * 1000;

	/* keep the frequency in an acceptable range */
	dFrequency = HB_MIN( HB_MAX( 0.0, dFrequency ), 32767.0 );

	ZeroMemory( &osvi, sizeof(OSVERSIONINFOEX) );
	osvi.dwOSVersionInfoSize = sizeof(OSVERSIONINFOEX);
	GetVersionEx( (OSVERSIONINFO*) &osvi );

	if ( osvi.dwPlatformId == VER_PLATFORM_WIN32_WINDOWS )
	{
		beep9x( dFrequency, dDuration );
	}
	else /* ( osvi.dwPlatformId == VER_PLATFORM_WIN32_NT ) */
	{
		beepNt( dFrequency, dDuration );
	}
}

static void beepNt( double dFreq, double dDurat )
{
	if( dFreq >= 20.0 )
	{
		Beep( dFreq , dDurat );
	}
	else
	{
		Sleep( dDurat );
	}
}

static void beep9x( double dFreq, double dDurat )
{

   INT uLSB,uMSB;
   ULONG lAdjFreq;

   /* Clipper ignores Tone() requests (but delays anyway) if Frequency is
      less than < 20 hz (and so should we) to maintain compatibility */

   if( dFreq >= 20.0 )
   {
      /* Setup Sound Control Port Registers and timer channel 2 */
      hb_Outp9x(67, 182) ;

      lAdjFreq = (ULONG)( 1193180 / dFreq ) ;

      if( (LONG) lAdjFreq < 0 )
         uLSB = lAdjFreq + 65536;
      else
         uLSB = lAdjFreq % 256;

      if( (LONG) lAdjFreq < 0 )
         uMSB = lAdjFreq + 65536;
      else
         uMSB = lAdjFreq / 256;

      /* set the frequency (LSB,MSB) */

      hb_Outp9x(66, uLSB);
      hb_Outp9x(66, uMSB);

      /* Get current Port setting */
      /* enable Speaker Data & Timer gate bits */
      /* (00000011B is bitmask to enable sound) */
      /* Turn on Speaker - sound Tone for duration.. */

      hb_Outp9x(97, hb_Inp9x( 97 ) | 3);

      Sleep( dDurat );

      /* Read back current Port value for Reset */
      /* disable Speaker Data & Timer gate bits */
      /* (11111100B is bitmask to disable sound) */
      /* Turn off the Speaker ! */

      hb_Outp9x(97, hb_Inp9x( 97 ) & 0xFC);

   }
   else
   {
      Sleep( dDurat );
   }

}

static int hb_Inp9x( USHORT usPort )
{
   USHORT usVal;

   HB_TRACE(HB_TR_DEBUG, ("hb_Inp9x(%hu)", usPort));

   #if defined( __BORLANDC__ ) || defined(__DMC__)

      _DX = usPort;
      __emit__(0xEC);         /* ASM  IN AL, DX */
      __emit__(0x32,0xE4);    /* ASM XOR AH, AH */
      usVal = _AX;

   #elif defined( __XCC__ )

      __asm {
               mov   dx, usPort
               xor   ax, ax
               in    al, dx
               mov   usVal, ax
            }

   #elif defined( __MINGW32__ )
      __asm__ __volatile__ ("inb %w1,%b0":"=a" (usVal):"Nd" (usPort));

   #elif defined( __WATCOMC__ )

      usVal = inp( usPort );

   #else

      usVal = _inp( usPort );

   #endif

   return usVal;
}

static int hb_Outp9x( USHORT usPort, USHORT usVal )
{
   HB_TRACE(HB_TR_DEBUG, ("hb_Outp9x(%hu, %hu)", usPort, usVal));

   #if defined( __BORLANDC__ ) || defined(__DMC__)

      _DX = usPort;
      _AL = usVal;
      __emit__(0xEE);        /* ASM OUT DX, AL */

   #elif defined( __XCC__ )

      __asm {
               mov   dx, usPort
               mov   ax, usVal
               out   dx, al
            }

   #elif defined( __MINGW32__ )

      __asm__ __volatile__ ("outb %b0,%w1": :"a" (usVal), "Nd" (usPort));

   #elif defined( __WATCOMC__ )

       outp( usPort, usVal );

   #else

      _outp( usPort, usVal );

   #endif

   return usVal;
}

#pragma ENDDUMP