// -----------------------------------------------------------------------------
// HBPRINTER - Harbour Win32 Printing library source code
//
// Copyright 2002-2005 Richard Rylko <rrylko@poczta.onet.pl>
// -----------------------------------------------------------------------------

#include "HBClass.ch"
#include "minigui.ch"
#include "winprint.ch"

/* Background Modes */

#define TRANSPARENT	1
#define OPAQUE		2
#define BKMODE_LAST	2

MEMVAR page,scale,azoom,ahs,npages
MEMVAR dx,dy,ngroup,ath,Iloscstron
MEMVAR aopisy

CLASS HBPrinter

   DATA    hDC INIT 0
   PROTECT hDCRef INIT 0
   DATA    PrinterName INIT ""

   DATA    lEscaped  INIT .F.
   DATA    nFromPage INIT 1
   DATA    nToPage   INIT 0
   PROTECT CurPage   INIT 1
   DATA    nCopies   INIT 1
   DATA    nWhatToPrint INIT 0
   PROTECT PrintOpt  INIT 1

   DATA    PrinterDefault INIT ""
   DATA    Error INIT 0 READONLY
   DATA    PaperNames INIT {}
   DATA    BINNAMES INIT {}
   DATA    DOCNAME INIT "HBPRINTER"
   DATA    TextColor INIT 0
   DATA    BkColor INIT 0xFFFFFF
   DATA    BkMode INIT 1      && TRANSPARENT
   DATA    PolyFillMode INIT 1
   DATA    Cargo INIT {0,0,0,0,0,0,0,0,0,0}
   DATA    FONTS INIT {{},{},0,{}}
   DATA    BRUSHES INIT {{},{}}
   DATA    PENS INIT {{},{}}
   DATA    REGIONS INIT {{},{}}
   DATA    IMAGELISTS INIT {{},{}}
   DATA    UNITS INIT 0
   DATA    DEVCAPS INIT {0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0}
   DATA    MAXROW INIT 0
   DATA    MAXCOL INIT 0
   PROTECT METAFILES INIT {}
   DATA    PREVIEWMODE INIT .F.
   DATA    THUMBNAILS INIT .F.
   DATA    CLSPREVIEW INIT .T.
   DATA    VIEWPORTORG INIT {0,0}
   DATA    PREVIEWRECT INIT {0,0,0,0}
   PROTECT PRINTINGEMF INIT .F.
   DATA    PRINTING INIT .F.
   DATA    PREVIEWSCALE INIT 1
   DATA    Printers INIT {}
   DATA    Ports INIT {}
   PROTECT Version INIT 2.17

   METHOD New()
   METHOD SelectPrinter(cPrinter ,lPrev)
   METHOD SetDevMode(what,newvalue)
   METHOD SetUserMode(what,value,value2)
   METHOD Startdoc(ldocname)
   METHOD SetPage(orient,size,fontname)
   METHOD Startpage()
   METHOD Endpage()
   METHOD Enddoc()
   METHOD SetTextColor(clr)
   METHOD GetTextColor()      INLINE ::TextColor
   METHOD SetBkColor(clr)
   METHOD GetBkColor()        INLINE ::BkColor
   METHOD SetBkMode(nmode)
   METHOD GetBkMode()         INLINE ::BkMode
   METHOD DefineImageList(defname,cpicture,nicons)
   METHOD DrawImageList(defname,nicon,row,col,torow,tocol,lstyle,color)
   METHOD DefineBrush(defname,lstyle,lcolor,lhatch)
   METHOD ModifyBrush(defname,lstyle,lcolor,lhatch)
   METHOD SelectBrush(defname)
   METHOD DefinePen(defname,lstyle,lwidth,lcolor)
   METHOD ModifyPen(defname,lstyle,lwidth,lcolor)
   METHOD SelectPen(defname)
   METHOD DefineFont(defname,lfontname,lfontsize,lfontwidth,langle,lweight,litalic,lunderline,lstrikeout)
   METHOD ModifyFont(defname,lfontname,lfontsize,lfontwidth,langle,lweight,lnweight,litalic,lnitalic,lunderline,lnunderline,lstrikeout,lnstrikeout)
   METHOD SelectFont(defname)
   METHOD GetObjByName(defname,what,retpos)
   METHOD DrawText(row,col,torow,tocol,txt,style,defname)
   METHOD TextOut(row,col,txt,defname)
   METHOD Say(row,col,txt,defname,lcolor,lalign)
   METHOD SetCharset(charset) INLINE rr_setcharset(charset)
   METHOD Rectangle(row,col,torow,tocol,defpen,defbrush)
   METHOD RoundRect(row,col,torow,tocol,widthellipse,heightellipse,defpen,defbrush)
   METHOD FillRect(row,col,torow,tocol,defbrush)
   METHOD FrameRect(row,col,torow,tocol,defbrush)
   METHOD InvertRect(row,col,torow,tocol)
   METHOD Ellipse(row,col,torow,tocol,defpen,defbrush)
   METHOD Arc(row,col,torow,tocol,rowsarc,colsarc,rowearc,colearc,defpen)
   METHOD ArcTo(row,col,torow,tocol,rowrad1,colrad1,rowrad2,colrad2,defpen)
   METHOD Chord(row,col,torow,tocol,rowrad1,colrad1,rowrad2,colrad2,defpen,defbrush)
   METHOD Pie(row,col,torow,tocol,rowrad1,colrad1,rowrad2,colrad2,defpen,defbrush)
   METHOD Polygon(apoints,defpen,defbrush,style)
   METHOD PolyBezier(apoints,defpen)
   METHOD PolyBezierTo(apoints,defpen)
   METHOD SetUnits(newvalue,r,c)
   METHOD Convert(arr,lsize)
   METHOD DefineRectRgn(defname,row,col,torow,tocol)
   METHOD DefinePolygonRgn(defname,apoints,style)
   METHOD DefineEllipticRgn(defname,row,col,torow,tocol)
   METHOD DefineRoundRectRgn(defname,row,col,torow,tocol,widthellipse,heightellipse)
   METHOD CombineRgn(defname,reg1,reg2,style)
   METHOD SelectClipRgn(defname)
   METHOD DeleteClipRgn()
   METHOD SetPolyFillMode(style)
   METHOD GetPolyFillMode()   INLINE ::PolyFillMode
   METHOD SetViewPortOrg(row,col)
   METHOD GetViewPortOrg()
   METHOD DxColors(par)
   METHOD SetRGB(red,green,blue)
   METHOD SetTextCharExtra(col)
   METHOD GetTextCharExtra()
   METHOD SetTextJustification(col)
   METHOD GetTextJustification()
   METHOD SetTextAlign(style)
   METHOD GetTextAlign()
   METHOD Picture(row,col,torow,tocol,cpicture,extrow,extcol)
   METHOD Line(row,col,torow,tocol,defpen)
   METHOD LineTo(row,col,defpen)
   METHOD SaveMetaFiles(nr,fn)
   METHOD GetTextExtent(ctext,apoint,deffont)
   METHOD GetTextExtent_mm(ctext,apoint,deffont)
   METHOD End()
   METHOD ReportData(l_x1,l_x2,l_x3,l_x4,l_x5,l_x6)
   METHOD Preview(x,y,width,height)
   METHOD PrevPrint()
   METHOD PrevShow()
   METHOD PrevThumb()
   METHOD PrevClose(lEsc)
   METHOD PrintOption()
   METHOD GetVersion()        INLINE ::Version

ENDCLASS


METHOD New() CLASS HBPrinter
local aprnport
   aprnport:=rr_getprinters()
   IF aprnport<>",,"
      aprnport:=str2arr(aprnport,",,")
      aeval(aprnport,{|x,xi| aprnport[xi]:=str2arr(x,',')})
      aeval(aprnport,{|x| aadd(::Printers,x[1]),  aadd(::ports,x[2]) })
      ::PrinterDefault:=RR_GETDEFAULTPRINTER()
   ELSE
      ::error:=1
   ENDIF
return self

METHOD SelectPrinter( cPrinter ,lPrev) CLASS HBPrinter
local txtp:="",txtb:="",t:={0,0,1,.t.}
   IF cPrinter == Nil
      ::hDCRef := rr_getdc(::PrinterDefault)
      ::hDC:=::hDCRef
      ::PrinterName:=::PrinterDefault
   ELSEIF Empty( cPrinter )
      ::hDCRef := rr_printdialog(t)
      ::nfrompage:=t[1]
      ::ntopage:=t[2]
      ::ncopies:=t[3]
      ::nwhattoprint:=t[4]
      ::hDC:=::hDCRef
      ::PrinterName := rr_PrinterName()
   ELSE
      ::hDCRef := rr_getdc(cPrinter)
      ::hDC:=::hDCRef
      ::PrinterName:=cPrinter
   ENDIF
   IF valtype(lPrev)=="L"
     if lprev
      ::PreviewMode:=.t.
     endif
   ENDIF
   IF ::hDC==0
      ::error:=1
      ::PrinterName:=""
   ELSE
      rr_devicecapabilities(@txtp,@txtb)
      ::PaperNames:=str2arr(txtp,",,")
      ::BinNames:=str2arr(txtb,",,")
      aeval(::BinNames,{|x,xi| ::BinNames[xi]:=str2arr(x,',')})
      aeval(::PaperNames,{|x,xi| ::PaperNames[xi]:=str2arr(x,',')})
      aadd(::Fonts[1],rr_getcurrentobject(1)) ; aadd(::Fonts[2],"*") ; aadd(::Fonts[4],{})
      aadd(::Fonts[1],rr_getcurrentobject(1)) ; aadd(::Fonts[2],"DEFAULT") ; aadd(::Fonts[4],{})
      aadd(::Brushes[1],rr_getcurrentobject(2)) ; aadd(::Brushes[2],"*")
      aadd(::Brushes[1],rr_getcurrentobject(2)) ; aadd(::Brushes[2],"DEFAULT")
      aadd(::Pens[1],rr_getcurrentobject(3)) ; aadd(::Pens[2],"*")
      aadd(::Pens[1],rr_getcurrentobject(3)) ; aadd(::Pens[2],"DEFAULT")
      aadd(::Regions[1],0) ; aadd(::Regions[2],"*")
      aadd(::Regions[1],0) ; aadd(::Regions[2],"DEFAULT")
      ::Fonts[3]:=::Fonts[1,1]
      rr_getdevicecaps(::DEVCAPS,::Fonts[3])
      ::setunits(::units)
   ENDIF
Return Nil

METHOD SetDevMode(what,newvalue) CLASS HBPrinter
  ::hDCRef:=rr_setdevmode(what,newvalue)
  rr_getdevicecaps(::DEVCAPS,::Fonts[3])
  ::setunits(::units)
Return Self

METHOD SetUserMode(what,value,value2) CLASS HBPrinter
  ::hDCRef:=rr_setusermode(what,value,value2)
  rr_getdevicecaps(::DEVCAPS,::Fonts[3])
  ::setunits(::units)
Return Self

METHOD StartDoc(ldocname) CLASS HBPrinter
  ::Printing:=.t.
  if ldocname<>NIL
     ::DOCNAME:=ldocname
  endif
  if !::PreviewMode
      rr_startdoc(::DOCNAME)
  endif
return self

METHOD SetPage(orient,size,fontname) CLASS HBPrinter
local lhand:=::getobjbyname(fontname,"F")
  if size<>NIL
     ::SetDevMode(DM_PAPERSIZE,size)
  endif
  if orient<>NIL
     ::SetDevMode(DM_ORIENTATION,orient)
  endif
  if lhand<>0
     ::Fonts[3]:=lhand
  endif
  rr_getdevicecaps(::DEVCAPS,::Fonts[3])
  ::setunits(::units)
return Self

METHOD Startpage() CLASS HBPrinter
  if ::PreviewMode
     ::hDC:=rr_createmfile()
  else
    rr_Startpage()
  endif
  if !::Printingemf
    rr_selectcliprgn(::Regions[1,1])
    rr_setviewportorg(::ViewPortOrg)
    rr_settextcolor(::textcolor)
    rr_setbkcolor(::bkcolor)
    rr_setbkmode(::bkmode)
    rr_selectbrush(::Brushes[1,1])
    rr_selectpen(::Pens[1,1])
    rr_selectfont(::Fonts[1,1])
  endif
return self

METHOD Endpage() CLASS HBPrinter
  if ::PreviewMode
     aadd(::MetaFiles,{rr_closemfile(),::DEVCAPS[1],::DEVCAPS[2],::DEVCAPS[3],::DEVCAPS[4],::DEVCAPS[15],::DEVCAPS[17]})
  else
     rr_endpage()
  endif
return self

METHOD SaveMetaFiles(number, filename) CLASS HBPrinter
 if ::PreviewMode
    default filename := "page"
    if number==NIL
       aeval(::METAFILES,{|x,xi| str2file(x[1],filename+ltrim(str(xi))+".emf") })
    else
       str2file(::METAFILES[number,1],if(at(".",filename)>0,filename,filename+ltrim(str(number))+".emf"))
    endif
 endif
return self

***********************************
METHOD EndDoc() CLASS HBPrinter
***********************************
 if ::PreviewMode
    ::preview()
 else
   rr_enddoc()
 endif
 ::Printing:=.f.
return self

******************************************
METHOD SetTextColor(clr) CLASS HBPrinter
******************************************
local lret:=::Textcolor
 if clr<>NIL
    // BEGIN RL 2003-08-03
    IF VALTYPE (clr) == 'N'
       ::TextColor:=rr_settextcolor(clr)
    ELSEIF VALTYPE (clr) == 'A'
       ::TextColor:=rr_settextcolor( RGB ( clr [1] , clr [2] , clr [3] ) )
    ENDIF
    // END RL
 endif
return lret

METHOD SetPolyFillMode(style) CLASS HBPrinter
local lret:=::PolyFillMode
  ::PolyFillMode:=rr_setpolyfillmode(style)
return lret

METHOD SetBkColor(clr) CLASS HBPrinter
local lret:=::BkColor
 // BEGIN RL 2003-08-03
 IF VALTYPE (clr) == 'N'
    ::BkColor:=rr_setbkcolor(clr)
 ELSEIF VALTYPE (clr) == 'A'
    ::BkColor:=rr_setbkcolor( RGB ( clr [1] , clr [2] , clr [3] ) )
 ENDIF
 // END RL
return lret

METHOD SetBkMode(nmode) CLASS HBPrinter
local lret:=::Bkmode
 ::BkMode:=nmode
 rr_setbkmode(nmode)
return lret

METHOD DefineBrush(defname,lstyle,lcolor,lhatch) CLASS HBPrinter
local lhand:=::getobjbyname(defname,"B")
 if lhand<>0
    return self
 endif
 // BEGIN RL 2003-08-03
 IF VALTYPE (lcolor) == 'A'
    lcolor := RGB ( lcolor [1] , lcolor [2] , lcolor [3] )
 ENDIF
 // END RL
 lstyle:=if(lstyle==NIL,BS_NULL,lstyle)
 lcolor:=if(lcolor==NIL,0xFFFFFF,lcolor)
 lhatch:=if(lhatch==NIL,HS_HORIZONTAL,lhatch)
 aadd(::Brushes[1],rr_createbrush(lstyle,lcolor,lhatch))
 aadd(::Brushes[2],upper(alltrim(defname)))
return self

METHOD SelectBrush(defname) CLASS HBPrinter
local lhand:=::getobjbyname(defname,"B")
 if lhand<>0
    rr_selectbrush(lhand)
    ::Brushes[1,1]:=lhand
 endif
return self

METHOD ModifyBrush(defname,lstyle,lcolor,lhatch) CLASS HBPrinter
local lhand:=0,lpos
 if defname=="*"
    lpos:=ascan(::Brushes[1],::Brushes[1,1],2)
    if lpos>1
       lhand:=::Brushes[1,lpos]
    endif
 else
    lhand:=::getobjbyname(defname,"B")
    lpos:=::getobjbyname(defname,"B",.t.)
 endif
 if lhand==0 .or. lpos==0
    ::error:=1
    return self
 endif
 lstyle:=if(lstyle==NIL,-1,lstyle)
 // BEGIN RL 2003-08-03
 IF VALTYPE (lcolor) == 'A'
    lcolor := RGB ( lcolor [1] , lcolor [2] , lcolor [3] )
 ENDIF
 // END RL
 lcolor:=if(lcolor==NIL,-1,lcolor)
 lhatch:=if(lhatch==NIL,-1,lhatch)
 ::Brushes[1,lpos]:=rr_modifybrush(lhand,lstyle,lcolor,lhatch)
 if lhand==::Brushes[1,1]
    ::selectbrush(::Brushes[2,lpos])
 endif
return self

***************************************************************
METHOD DefinePen(defname,lstyle,lwidth,lcolor) CLASS HBPrinter
***************************************************************
local lhand:=::getobjbyname(defname,"P")
 if lhand<>0
    return self
 endif
 // BEGIN RL 2003-08-03
 IF VALTYPE (lcolor) == 'A'
    lcolor := RGB ( lcolor [1] , lcolor [2] , lcolor [3] )
 ENDIF
 // END RL
 lstyle:=if(lstyle==NIL,PS_SOLID,lstyle)
 lcolor:=if(lcolor==NIL,0xFFFFFF,lcolor)
 lwidth:=if(lwidth==NIL,0,lwidth)
 aadd(::Pens[1],rr_createpen(lstyle,lwidth,lcolor))
 aadd(::Pens[2],upper(alltrim(defname)))
return self

***************************************************************
METHOD ModifyPen(defname,lstyle,lwidth,lcolor) CLASS HBPrinter
***************************************************************
local lhand:=0,lpos
 if defname=="*"
    lpos:=ascan(::Pens[1],::Pens[1,1],2)
    if lpos>1
       lhand:=::Pens[1,lpos]
    endif
 else
    lhand:=::getobjbyname(defname,"P")
    lpos:=::getobjbyname(defname,"P",.t.)
 endif
 if lhand==0 .or. lpos<=1
    ::error:=1
    return self
 endif

 lstyle:=if(lstyle==NIL,-1,lstyle)
 // BEGIN RL 2003-08-03
 IF VALTYPE (lcolor) == 'A'
    lcolor := RGB ( lcolor [1] , lcolor [2] , lcolor [3] )
 ENDIF
 // END RL
 lcolor:=if(lcolor==NIL,-1,lcolor)
 lwidth:=if(lwidth==NIL,-1,lwidth)
 ::Pens[1,lpos]:=rr_modifypen(lhand,lstyle,lwidth,lcolor)
 if lhand==::Pens[1,1]
    ::selectpen(::Pens[2,lpos])
 endif
return self

*********************************************
METHOD SelectPen(defname) CLASS HBPrinter
*********************************************
local lhand:=::getobjbyname(defname,"P")
 if lhand<>0
    rr_selectpen(lhand)
    ::Pens[1,1]:=lhand
 endif
return self

*********************************************************
METHOD DefineFont(defname,lfontname,lfontsize,lfontwidth,langle,lweight,litalic,lunderline,lstrikeout) CLASS HBPrinter
*********************************************************
local lhand:=::getobjbyname(lfontname,"F")
 if lhand<>0
    return self
 endif
 lfontname:=if(lfontname==NIL,"",upper(alltrim(lfontname)))
 if(lfontsize==NIL,lfontsize:=-1,NIL)
 if(lfontwidth==nil,lfontwidth:=0,NIL)
 if(langle==NIL,langle:=-1,NIL)
 lweight:=if(empty(lweight),0,1)
 litalic:=if(empty(litalic),0,1)
 lunderline:=if(empty(lunderline),0,1)
 lstrikeout:=if(empty(lstrikeout),0,1)
 aadd(::Fonts[1],rr_createfont(lfontname,lfontsize,-lfontwidth,langle*10,lweight,litalic,lunderline,lstrikeout))
 aadd(::Fonts[2],upper(alltrim(defname)))
 aadd(::Fonts[4],{lfontname,lfontsize,lfontwidth,langle,lweight,litalic,lunderline,lstrikeout})
return self

*********************************************************
METHOD ModifyFont(defname,lfontname,lfontsize,lfontwidth,langle,lweight,lnweight,litalic,lnitalic,lunderline,lnunderline,lstrikeout,lnstrikeout) CLASS HBPrinter
*********************************************************
local lhand:=0,lpos
 if defname=="*"
    lpos:=ascan(::Fonts[1],::Fonts[1,1],2)
    if lpos>1
       lhand:=::Fonts[1,lpos]
    endif
 else
    lhand:=::getobjbyname(defname,"F")
    lpos:=::getobjbyname(defname,"F",.t.)
 endif
 if lhand==0 .or. lpos<=1
    ::error:=1
    return self
 endif

 if(lfontname<>NIL ,::Fonts[4,lpos,1]:=upper(alltrim(lfontname)),NIL)
 if(lfontsize<>NIL ,::Fonts[4,lpos,2]:=lfontsize,NIL)
 if(lfontwidth<>NIL,::Fonts[4,lpos,3]:=lfontwidth,NIL)
 if(langle<>NIL    ,::Fonts[4,lpos,4]:=langle,NIL)
 if(lweight ,::Fonts[4,lpos,5]:=1,NIL)
 if(lnweight,::Fonts[4,lpos,5]:=0,NIL)
 if(litalic ,::Fonts[4,lpos,6]:=1,NIL)
 if(lnitalic,::Fonts[4,lpos,6]:=0,NIL)
 if(lunderline ,::Fonts[4,lpos,7]:=1,NIL)
 if(lnunderline,::Fonts[4,lpos,7]:=0,NIL)
 if(lstrikeout ,::Fonts[4,lpos,8]:=1,NIL)
 if(lnstrikeout,::Fonts[4,lpos,8]:=0,NIL)

 ::Fonts[1,lpos]:=rr_createfont(::Fonts[4,lpos,1],::Fonts[4,lpos,2],-::Fonts[4,lpos,3],::Fonts[4,lpos,4]*10,::Fonts[4,lpos,5],::Fonts[4,lpos,6],::Fonts[4,lpos,7],::Fonts[4,lpos,8])

 if lhand==::Fonts[1,1]
    ::selectfont(::Fonts[2,lpos])
 endif
 rr_deleteobjects({0,lhand})
return self

METHOD SelectFont(defname) CLASS HBPrinter
local lhand:=::getobjbyname(defname,"F")
 if lhand<>0
    rr_selectfont(lhand)
    ::Fonts[1,1]:=lhand
 endif
return self

METHOD SetUnits(newvalue,r,c) CLASS HBPrinter
local oldvalue:=::UNITS
 newvalue:=if(valtype(newvalue)=="N",newvalue,0)
 ::UNITS:=if(newvalue<0 .or. newvalue>4,0,newvalue)
 do case
      case ::Units==0   && ROWCOL
           ::MaxRow:=::DevCaps[13]-1
           ::MaxCol:=::DevCaps[14]-1
      case ::Units==1   && MM
           ::MaxRow:=::DevCaps[1]-1
           ::MaxCol:=::DevCaps[2]-1
      case ::Units==2   && INCHES
           ::MaxRow:=(::DevCaps[1]/25.4)-1
           ::MaxCol:=(::DevCaps[2]/25.4)-1
      case ::Units==3   && PIXEL
           ::MaxRow:=::DevCaps[3]
           ::MaxCol:=::DevCaps[4]
      case ::Units==4   && ROWS   COLS
           if(valtype(r)=="N",::MaxRow:=r-1,NIL)
           if(valtype(c)=="N",::MaxCol:=c-1,NIL)
 endcase
return oldvalue

METHOD Convert(arr,lsize) CLASS HBPrinter
local aret:=aclone(arr)
 do case
    case ::UNITS==0        && ROWCOL
         aret[1]:=(arr[1])*::DEVCAPS[11]
         aret[2]:=(arr[2])*::DEVCAPS[12]
    case ::UNITS==3        && PIXEL
    case ::UNITS==4        && ROWS   COLS
         aret[1]:=(arr[1])*::DEVCAPS[3]/(::maxrow+1)
         aret[2]:=(arr[2])*::DEVCAPS[4]/(::maxcol+1)
    case ::UNITS==1        && MM
         aret[1]:=(arr[1])*::DEVCAPS[5]/25.4-if(lsize==NIL,::DEVCAPS[9 ],0)
         aret[2]:=(arr[2])*::DEVCAPS[6]/25.4-if(lsize==NIL,::DEVCAPS[10],0)
    case ::UNITS==2        && INCHES
         aret[1]:=(arr[1])*::DEVCAPS[5]-if(lsize==NIL,::DEVCAPS[9 ],0)
         aret[2]:=(arr[2])*::DEVCAPS[6]-if(lsize==NIL,::DEVCAPS[10],0)
    otherwise
         aret[1]:=(arr[1])*::DEVCAPS[11]
         aret[2]:=(arr[2])*::DEVCAPS[12]
 endcase
return aret

METHOD DrawText(row,col,torow,tocol,txt,style,defname) CLASS HBPrinter
local lhf:=::getobjbyname(defname,"F")
 if(torow==NIL,torow:=::maxrow,NIL)
 if(tocol==NIL,tocol:=::maxcol,NIL)
 rr_drawtext(::Convert({row,col}),::Convert({torow,tocol}),txt,style,lhf)
return self

METHOD TEXTOUT(row,col,txt,defname) CLASS HBPrinter
local lhf:=::getobjbyname(defname,"F")
 rr_textout(txt,::Convert({row,col}),lhf,numat(" ",txt))
return self

METHOD Say(row,col,txt,defname,lcolor,lalign) CLASS HBPrinter
local atxt:={},i,lhf:=::getobjbyname(defname,"F"),oldalign
local apos
 do case
    case valtype(txt)=="N"    ;  aadd(atxt,str(txt))
    case valtype(txt)=="D"    ;  aadd(atxt,dtoc(txt))
    case valtype(txt)=="L"    ;  aadd(atxt,if(txt,".T.",".F."))
    case valtype(txt)=="U"    ;  aadd(atxt,"NIL")
    case valtype(txt)$"BO"    ;  aadd(atxt,"")
    case valtype(txt)=="A"    ;  aeval(txt,{|x| aadd(atxt,sayconvert(x)) })
    case valtype(txt)$"MC"    ;  atxt:=str2arr(txt,hb_osnewline())
 endcase
 apos:=::convert({row,col})
 if lcolor<>NIL
    // BEGIN RL 2003-08-03
    IF VALTYPE (lcolor) == 'N'
       rr_settextcolor(lcolor)
    ELSEIF VALTYPE (lcolor) == 'A'
       rr_settextcolor( RGB ( lcolor [1] , lcolor [2] , lcolor [3] ) )
    ENDIF
        // END RL
 endif
 if lalign<>NIL
    oldalign:=rr_gettextalign()
    rr_settextalign(lalign)
 endif
 for i:=1 to len(atxt)
    rr_textout(atxt[i],apos,lhf,numat(" ",atxt[i]))
    apos[1]+=::DEVCAPS[11]
 next
 if lalign<>NIL
    rr_settextalign(oldalign)
 endif
 if lcolor<>NIL
    rr_settextcolor(::textcolor)
 endif
return self

METHOD DefineImageList(defname,cpicture,nicons) CLASS HBPrinter
local lhi:=::getobjbyname(defname,"I"),w:=0,h:=0,hand
 if lhi<>0
    return self
 endif
 hand:=rr_createimagelist(cpicture,nicons,@w,@h)
 if hand<>0 .and. w>0 .and. h>0
   aadd(::imagelists[1],{hand,nicons,w,h})
   aadd(::imagelists[2],upper(alltrim(defname)))
 endif
return self

METHOD DRAWIMAGELIST(defname,nicon,row,col,torow,tocol,lstyle,color) CLASS HBPrinter
local lhi:=::getobjbyname(defname,"I")
 if empty(lhi)
    return self
 endif
 if(color==NIL,color:=-1,NIL)
 if(torow==NIL,torow:=::maxrow,NIL)
 if(tocol==NIL,tocol:=::maxcol,NIL)
 ::error:=rr_drawimagelist(lhi[1],nicon,::convert({row,col}),::convert({torow-row,tocol-col}),lhi[3],lhi[4],lstyle,color)
return self

METHOD Rectangle(row,col,torow,tocol,defpen,defbrush) CLASS HBPrinter
local lhp:=::getobjbyname(defpen,"P"),lhb:=::getobjbyname(defbrush,"B")
 if(torow==NIL,torow:=::maxrow,NIL)
 if(tocol==NIL,tocol:=::maxcol,NIL)
 ::error=rr_rectangle(::convert({row,col}),::convert({torow,tocol}),lhp,lhb)
return self

METHOD FrameRect(row,col,torow,tocol,defbrush) CLASS HBPrinter
local lhb:=::getobjbyname(defbrush,"B")
 if(torow==NIL,torow:=::maxrow,NIL)
 if(tocol==NIL,tocol:=::maxcol,NIL)
 ::error=rr_framerect(::convert({row,col}),::convert({torow,tocol}),lhb)
return self

METHOD RoundRect(row,col,torow,tocol,widthellipse,heightellipse,defpen,defbrush) CLASS HBPrinter
local lhp:=::getobjbyname(defpen,"P"),lhb:=::getobjbyname(defbrush,"B")
 if(torow==NIL,torow:=::maxrow,NIL)
 if(tocol==NIL,tocol:=::maxcol,NIL)
 ::error=rr_roundrect(::convert({row,col}),::convert({torow,tocol}),::convert({widthellipse,heightellipse}),lhp,lhb)
return self

METHOD FillRect(row,col,torow,tocol,defbrush) CLASS HBPrinter
local lhb:=::getobjbyname(defbrush,"B")
 if(torow==NIL,torow:=::maxrow,NIL)
 if(tocol==NIL,tocol:=::maxcol,NIL)
 ::error=rr_fillrect(::convert({row,col}),::convert({torow,tocol}),lhb)
return self

METHOD InvertRect(row,col,torow,tocol) CLASS HBPrinter
 if(torow==NIL,torow:=::maxrow,NIL)
 if(tocol==NIL,tocol:=::maxcol,NIL)
 ::error=rr_invertrect(::convert({row,col}),::convert({torow,tocol}))
return self

METHOD Ellipse(row,col,torow,tocol,defpen,defbrush) CLASS HBPrinter
local lhp:=::getobjbyname(defpen,"P"),lhb:=::getobjbyname(defbrush,"B")
 if(torow==NIL,torow:=::maxrow,NIL)
 if(tocol==NIL,tocol:=::maxcol,NIL)
 ::error=rr_ellipse(::convert({row,col}),::convert({torow,tocol}),lhp,lhb)
return self

METHOD Arc(row,col,torow,tocol,rowsarc,colsarc,rowearc,colearc,defpen) CLASS HBPrinter
local lhp:=::getobjbyname(defpen,"P")
 if(torow==NIL,torow:=::maxrow,NIL)
 if(tocol==NIL,tocol:=::maxcol,NIL)
 ::error=rr_arc(::convert({row,col}),::convert({torow,tocol}),::convert({rowsarc,colsarc}),::convert({rowearc,colearc}),lhp)
return self

METHOD ArcTo(row,col,torow,tocol,rowrad1,colrad1,rowrad2,colrad2,defpen) CLASS HBPrinter
local lhp:=::getobjbyname(defpen,"P")
 if(torow==NIL,torow:=::maxrow,NIL)
 if(tocol==NIL,tocol:=::maxcol,NIL)
 ::error=rr_arcto(::convert({row,col}),::convert({torow,tocol}),::convert({rowrad1,colrad1}),::convert({rowrad2,colrad2}),lhp)
return self


METHOD Chord(row,col,torow,tocol,rowrad1,colrad1,rowrad2,colrad2,defpen,defbrush) CLASS HBPrinter
local lhp:=::getobjbyname(defpen,"P"),lhb:=::getobjbyname(defbrush,"B")
 if(torow==NIL,torow:=::maxrow,NIL)
 if(tocol==NIL,tocol:=::maxcol,NIL)
 ::error=rr_chord(::convert({row,col}),::convert({torow,tocol}),::convert({rowrad1,colrad1}),::convert({rowrad2,colrad2}),lhp,lhb)
return self

METHOD Pie(row,col,torow,tocol,rowrad1,colrad1,rowrad2,colrad2,defpen,defbrush) CLASS HBPrinter
local lhp:=::getobjbyname(defpen,"P"),lhb:=::getobjbyname(defbrush,"B")
 if(torow==NIL,torow:=::maxrow,NIL)
 if(tocol==NIL,tocol:=::maxcol,NIL)
 ::error=rr_pie(::convert({row,col}),::convert({torow,tocol}),::convert({rowrad1,colrad1}),::convert({rowrad2,colrad2}),lhp,lhb)
return self

METHOD Polygon(apoints,defpen,defbrush,style) CLASS HBPrinter
local apx:={},apy:={},temp
local lhp:=::getobjbyname(defpen,"P"),lhb:=::getobjbyname(defbrush,"B")
 aeval(apoints,{|x| temp:=::convert(x),aadd(apx,temp[2]), aadd(apy,temp[1])})
 ::error:=rr_polygon(apx,apy,lhp,lhb,style)
return self

METHOD PolyBezier(apoints,defpen) CLASS HBPrinter
local apx:={},apy:={},temp
local lhp:=::getobjbyname(defpen,"P")
 aeval(apoints,{|x| temp:=::convert(x),aadd(apx,temp[2]), aadd(apy,temp[1])})
 ::error:=rr_polybezier(apx,apy,lhp)
return self

METHOD PolyBezierTo(apoints,defpen) CLASS HBPrinter
local apx:={},apy:={},temp
local lhp:=::getobjbyname(defpen,"P")
 aeval(apoints,{|x| temp:=::convert(x),aadd(apx,temp[2]), aadd(apy,temp[1])})
 ::error:=rr_polybezierto(apx,apy,lhp)
return self

METHOD Line(row,col,torow,tocol,defpen) CLASS HBPrinter
local lhp:=::getobjbyname(defpen,"P")
 if(torow==NIL,torow:=::maxrow,NIL)
 if(tocol==NIL,tocol:=::maxcol,NIL)
 ::error=rr_line(::convert({row,col}),::convert({torow,tocol}),lhp)
return self

METHOD LineTo(row,col,defpen) CLASS HBPrinter
local lhp:=::getobjbyname(defpen,"P")
 ::error=rr_lineto(::convert({row,col}),lhp)
return self

METHOD GetTextExtent(ctext,apoint,deffont) CLASS HBPrinter
local lhf:=::getobjbyname(deffont,"F")
 ::error=rr_gettextextent(ctext,apoint,lhf)
return self

METHOD GetTextExtent_mm(ctext,apoint,deffont) CLASS HBPrinter
local lhf:=::getobjbyname(deffont,"F")
 ::error=rr_gettextextent(ctext,apoint,lhf)
 apoint[1]:= 25.4*apoint[1]/::DevCaps[5]
 apoint[2]:= 25.4*apoint[2]/::DevCaps[6]
return self

METHOD GetObjByName(defname,what,retpos) CLASS HBPrinter
local lfound,lret:=0,aref,ahref
 if valtype(defname)=="C"
    do case
       case what=="F" ; aref:=::Fonts[2]      ; ahref:=::Fonts[1]
       case what=="B" ; aref:=::Brushes[2]    ; ahref:=::Brushes[1]
       case what=="P" ; aref:=::Pens[2]       ; ahref:=::Pens[1]
       case what=="R" ; aref:=::Regions[2]    ; ahref:=::Regions[1]
       case what=="I" ; aref:=::ImageLists[2] ; ahref:=::ImageLists[1]
    endcase
    lfound:=ascan(aref,upper(alltrim(defname)))
    if lfound>0
       if aref[lfound]==upper(alltrim(defname))
          if retpos<>NIL
            lret:=lfound
          else
            lret:=ahref[lfound]
          endif
       endif
    endif
 endif
return lret

METHOD DefineRectRgn(defname,row,col,torow,tocol) CLASS HBPrinter
local lhand:=::getobjbyname(defname,"R")
 if lhand<>0
    return self
 endif
 if(torow==NIL,torow:=::maxrow,NIL)
 if(tocol==NIL,tocol:=::maxcol,NIL)
 aadd(::Regions[1],rr_creatergn(::convert({row,col}),::convert({torow,tocol}),1))
 aadd(::Regions[2],upper(alltrim(defname)))
return self

METHOD DefineEllipticRgn(defname,row,col,torow,tocol) CLASS HBPrinter
local lhand:=::getobjbyname(defname,"R")
 if lhand<>0
    return self
 endif
 if(torow==NIL,torow:=::maxrow,NIL)
 if(tocol==NIL,tocol:=::maxcol,NIL)
 aadd(::Regions[1],rr_creatergn(::convert({row,col}),::convert({torow,tocol}),2))
 aadd(::Regions[2],upper(alltrim(defname)))
return self

METHOD DefineRoundRectRgn(defname,row,col,torow,tocol,widthellipse,heightellipse) CLASS HBPrinter
local lhand:=::getobjbyname(defname,"R")
 if lhand<>0
    return self
 endif
 if(torow==NIL,torow:=::maxrow,NIL)
 if(tocol==NIL,tocol:=::maxcol,NIL)
 aadd(::Regions[1],rr_creatergn(::convert({row,col}),::convert({torow,tocol}),3,::convert({widthellipse,heightellipse})))
 aadd(::Regions[2],upper(alltrim(defname)))
return self

METHOD DefinePolygonRgn(defname,apoints,style) CLASS HBPrinter
local apx:={},apy:={},temp
local lhand:=::getobjbyname(defname,"R")
 if lhand<>0
    return self
 endif
 aeval(apoints,{|x| temp:=::convert(x),aadd(apx,temp[2]), aadd(apy,temp[1])})
 aadd(::Regions[1],rr_createPolygonrgn(apx,apy,style))
 aadd(::Regions[2],upper(alltrim(defname)))
return self

METHOD CombineRgn(defname,reg1,reg2,style) CLASS HBPrinter
local lr1:=::getobjbyname(reg1,"R"),lr2:=::getobjbyname(reg2,"R")
local lhand:=::getobjbyname(defname,"R")
 if lhand<>0 .or. lr1==0 .or. lr2==0
    return self
 endif
 aadd(::Regions[1],rr_combinergn(lr1,lr2,style))
 aadd(::Regions[2],upper(alltrim(defname)))
return self

METHOD SelectClipRgn(defname) CLASS HBPrinter
local lhand:=::getobjbyname(defname,"R")
  if lhand<>0
        rr_selectcliprgn(lhand)
        ::Regions[1,1]:=lhand
  endif
return self

METHOD DeleteClipRgn() CLASS HBPrinter
  ::Regions[1,1]:=0
  rr_deletecliprgn()
return self

METHOD SetViewPortOrg(row,col) CLASS HBPrinter
  row:=if(row<>NIL,row,0)
  col:=if(col<>NIL,col,0)
       ::VIEWPORTORG:=::convert({row,col})
       rr_setviewportorg(::ViewportOrg)
return self

METHOD GetViewPortOrg() CLASS HBPrinter
       rr_getviewportorg(::VIEWPORTORG)
return self

METHOD End() CLASS HBPrinter
  if ::PreviewMode
       ::Metafiles:={}
  endif
  if ::HDCRef!=0
      ef_resetprinter()
      rr_deletedc(::HDCRef)
  endif
  rr_deleteobjects(::Fonts[1])
  rr_deleteobjects(::Brushes[1])
  rr_deleteobjects(::Pens[1])
  rr_deleteobjects(::Regions[1])
  rr_deleteimagelists(::ImageLists[1])
  rr_finish()
return NIL

METHOD DXCOLORS(par) CLASS HBPrinter
static rgbColorNames
local ltemp:=0
rgbColorNames := ARRAY(140, 2)
rgbColorNames[1]:={"aliceblue",0xfffff8f0}
rgbColorNames[2]:={"antiquewhite",0xffd7ebfa}
rgbColorNames[3]:={"aqua",0xffffff00}
rgbColorNames[4]:={"aquamarine",0xffd4ff7f}
rgbColorNames[5]:={"azure",0xfffffff0}
rgbColorNames[6]:={"beige",0xffdcf5f5}
rgbColorNames[7]:={"bisque",0xffc4e4ff}
rgbColorNames[8]:={"black",0xff000000}
rgbColorNames[9]:={"blanchedalmond",0xffcdebff}
rgbColorNames[10]:={"blue",0xffff0000}
rgbColorNames[11]:={"blueviolet",0xffe22b8a}
rgbColorNames[12]:={"brown",0xff2a2aa5}
rgbColorNames[13]:={"burlywood",0xff87b8de}
rgbColorNames[14]:={"cadetblue",0xffa09e5f}
rgbColorNames[15]:={"chartreuse",0xff00ff7f}
rgbColorNames[16]:={"chocolate",0xff1e69d2}
rgbColorNames[17]:={"coral",0xff507fff}
rgbColorNames[18]:={"cornflowerblue",0xffed9564}
rgbColorNames[19]:={"cornsilk",0xffdcf8ff}
rgbColorNames[20]:={"crimson",0xff3c14dc}
rgbColorNames[21]:={"cyan",0xffffff00}
rgbColorNames[22]:={"darkblue",0xff8b0000}
rgbColorNames[23]:={"darkcyan",0xff8b8b00}
rgbColorNames[24]:={"darkgoldenrod",0xff0b86b8}
rgbColorNames[25]:={"darkgray",0xffa9a9a9}
rgbColorNames[26]:={"darkgreen",0xff006400}
rgbColorNames[27]:={"darkkhaki",0xff6bb7bd}
rgbColorNames[28]:={"darkmagenta",0xff8b008b}
rgbColorNames[29]:={"darkolivegreen",0xff2f6b55}
rgbColorNames[30]:={"darkorange",0xff008cff}
rgbColorNames[31]:={"darkorchid",0xffcc3299}
rgbColorNames[32]:={"darkred",0xff00008b}
rgbColorNames[33]:={"darksalmon",0xff7a96e9}
rgbColorNames[34]:={"darkseagreen",0xff8fbc8f}
rgbColorNames[35]:={"darkslateblue",0xff8b3d48}
rgbColorNames[36]:={"darkslategray",0xff4f4f2f}
rgbColorNames[37]:={"darkturquoise",0xffd1ce00}
rgbColorNames[38]:={"darkviolet",0xffd30094}
rgbColorNames[39]:={"deeppink",0xff9314ff}
rgbColorNames[40]:={"deepskyblue",0xffffbf00}
rgbColorNames[41]:={"dimgray",0xff696969}
rgbColorNames[42]:={"dodgerblue",0xffff901e}
rgbColorNames[43]:={"firebrick",0xff2222b2}
rgbColorNames[44]:={"floralwhite",0xfff0faff}
rgbColorNames[45]:={"forestgreen",0xff228b22}
rgbColorNames[46]:={"fuchsia",0xffff00ff}
rgbColorNames[47]:={"gainsboro",0xffdcdcdc}
rgbColorNames[48]:={"ghostwhite",0xfffff8f8}
rgbColorNames[49]:={"gold",0xff00d7ff}
rgbColorNames[50]:={"goldenrod",0xff20a5da}
rgbColorNames[51]:={"gray",0xff808080}
rgbColorNames[52]:={"green",0xff008000}
rgbColorNames[53]:={"greenyellow",0xff2fffad}
rgbColorNames[54]:={"honeydew",0xfff0fff0}
rgbColorNames[55]:={"hotpink",0xffb469ff}
rgbColorNames[56]:={"indianred",0xff5c5ccd}
rgbColorNames[57]:={"indigo",0xff82004b}
rgbColorNames[58]:={"ivory",0xfff0ffff}
rgbColorNames[59]:={"khaki",0xff8ce6f0}
rgbColorNames[60]:={"lavender",0xfffae6e6}
rgbColorNames[61]:={"lavenderblush",0xfff5f0ff}
rgbColorNames[62]:={"lawngreen",0xff00fc7c}
rgbColorNames[63]:={"lemonchiffon",0xffcdfaff}
rgbColorNames[64]:={"lightblue",0xffe6d8ad}
rgbColorNames[65]:={"lightcoral",0xff8080f0}
rgbColorNames[66]:={"lightcyan",0xffffffe0}
rgbColorNames[67]:={"lightgoldenrodyellow",  0xffd2fafa}
rgbColorNames[68]:={"lightgreen",0xff90ee90}
rgbColorNames[69]:={"lightgrey",0xffd3d3d3}
rgbColorNames[70]:={"lightpink",0xffc1b6ff}
rgbColorNames[71]:={"lightsalmon",0xff7aa0ff}
rgbColorNames[72]:={"lightseagreen",0xffaab220}
rgbColorNames[73]:={"lightskyblue",0xffface87}
rgbColorNames[74]:={"lightslategray",0xff998877}
rgbColorNames[75]:={"lightsteelblue",0xffdec4b0}
rgbColorNames[76]:={"lightyellow",0xffe0ffff}
rgbColorNames[77]:={"lime",0xff00ff00}
rgbColorNames[78]:={"limegreen",0xff32cd32}
rgbColorNames[79]:={"linen",0xffe6f0fa}
rgbColorNames[80]:={"magenta",0xffff00ff}
rgbColorNames[81]:={"maroon",0xff000080}
rgbColorNames[82]:={"mediumaquamarine",0xffaacd66}
rgbColorNames[83]:={"mediumblue",0xffcd0000}
rgbColorNames[84]:={"mediumorchid",0xffd355ba}
rgbColorNames[85]:={"mediumpurple",0xffdb7093}
rgbColorNames[86]:={"mediumseagreen",0xff71b33c}
rgbColorNames[87]:={"mediumslateblue",0xffee687b}
rgbColorNames[88]:={"mediumspringgreen",0xff9afa00}
rgbColorNames[89]:={"mediumturquoise",0xffccd148}
rgbColorNames[90]:={"mediumvioletred",0xff8515c7}
rgbColorNames[91]:={"midnightblue",0xff701919}
rgbColorNames[92]:={"mintcream",0xfffafff5}
rgbColorNames[93]:={"mistyrose",0xffe1e4ff}
rgbColorNames[94]:={"moccasin",0xffb5e4ff}
rgbColorNames[95]:={"navajowhite",0xffaddeff}
rgbColorNames[96]:={"navy",0xff800000}
rgbColorNames[97]:={"oldlace",0xffe6f5fd}
rgbColorNames[98]:={"olive",0xff008080}
rgbColorNames[99]:={"olivedrab",0xff238e6b}
rgbColorNames[100]:={"orange",0xff00a5ff}
rgbColorNames[101]:={"orangered",0xff0045ff}
rgbColorNames[102]:={"orchid",0xffd670da}
rgbColorNames[103]:={"palegoldenrod",0xffaae8ee}
rgbColorNames[104]:={"palegreen",0xff98fb98}
rgbColorNames[105]:={"paleturquoise",0xffeeeeaf}
rgbColorNames[106]:={"palevioletred",0xff9370db}
rgbColorNames[107]:={"papayawhip",0xffd5efff}
rgbColorNames[108]:={"peachpuff",0xffb9daff}
rgbColorNames[109]:={"peru",0xff3f85cd}
rgbColorNames[110]:={"pink",0xffcbc0ff}
rgbColorNames[111]:={"plum",0xffdda0dd}
rgbColorNames[112]:={"powderblue",0xffe6e0b0}
rgbColorNames[113]:={"purple",0xff800080}
rgbColorNames[114]:={"red",0xff0000ff}
rgbColorNames[115]:={"rosybrown",0xff8f8fbc}
rgbColorNames[116]:={"royalblue",0xffe16941}
rgbColorNames[117]:={"saddlebrown",0xff13458b}
rgbColorNames[118]:={"salmon",0xff7280fa}
rgbColorNames[119]:={"sandybrown",0xff60a4f4}
rgbColorNames[120]:={"seagreen",0xff578b2e}
rgbColorNames[121]:={"seashell",0xffeef5ff}
rgbColorNames[122]:={"sienna",0xff2d52a0}
rgbColorNames[123]:={"silver",0xffc0c0c0}
rgbColorNames[124]:={"skyblue",0xffebce87}
rgbColorNames[125]:={"slateblue",0xffcd5a6a}
rgbColorNames[126]:={"slategray",0xff908070}
rgbColorNames[127]:={"snow",0xfffafaff}
rgbColorNames[128]:={"springgreen",0xff7fff00}
rgbColorNames[129]:={"steelblue",0xffb48246}
rgbColorNames[130]:={"tan",0xff8cb4d2}
rgbColorNames[131]:={"teal",0xff808000}
rgbColorNames[132]:={"thistle",0xffd8bfd8}
rgbColorNames[133]:={"tomato",0xff4763ff}
rgbColorNames[134]:={"turquoise",0xffd0e040}
rgbColorNames[135]:={"violet",0xffee82ee}
rgbColorNames[136]:={"wheat",0xffb3def5}
rgbColorNames[137]:={"white",0xffffffff}
rgbColorNames[138]:={"whitesmoke",0xfff5f5f5}
rgbColorNames[139]:={"yellow",0xff00ffff}
rgbColorNames[140]:={"yellowgreen",0xff32cd9a}

if valtype(par)=="C"
   par:=lower(alltrim(par))
   aeval(rgbcolornames,{|x| if(x[1]==par,ltemp:=x[2],'')})
elseif valtype(par)=="N"
   ltemp := if(par<=len(rgbcolornames),rgbcolornames[par,2],0)
endif
return ltemp

METHOD SetRGB(red,green,blue) CLASS HBPrinter
return rr_setrgb(red,green,blue)

METHOD SetTextCharExtra(col) CLASS HBPrinter
local p1:=::convert({0,0}),p2:=::convert({0,col})
return rr_SetTextCharExtra(p2[2]-p1[2])

METHOD GetTextCharExtra() CLASS HBPrinter
return rr_GetTextCharExtra()

METHOD SetTextJustification(col) CLASS HBPrinter
local p1:=::convert({0,0}),p2:=::convert({0,col})
return rr_SetTextJustification(p2[2]-p1[2])

METHOD GetTextJustification() CLASS HBPrinter
return rr_GetTextJustification()

METHOD SetTextAlign(style) CLASS HBPrinter
return rr_settextalign(style)

METHOD GetTextAlign() CLASS HBPrinter
return rr_gettextalign()

METHOD Picture(row,col,torow,tocol,cpicture,extrow,extcol) CLASS HBPrinter
local lp1:=::convert({row,col}),lp2,lp3
 if(torow==NIL,torow:=::maxrow,NIL)
 if(tocol==NIL,tocol:=::maxcol,NIL)
 lp2:=::convert({torow,tocol},1)
 if(extrow==NIL,extrow:=0,NIL)
 if(extcol==NIL,extcol:=0,NIL)
 lp3:=::convert({extrow,extcol})
 rr_drawpicture(cpicture,lp1,lp2,lp3)
return self

static function str2file(ctxt,cfile)
local hand,lrec
 hand:=fcreate(cfile)
 if hand<0
    return 0
 endif
 lrec:=fwrite(hand,ctxt)
 fclose(hand)
return lrec

static function sayconvert(ltxt)
 do case
    case valtype(ltxt)$"MC"    ;  return ltxt
    case valtype(ltxt)=="N"    ;  return str(ltxt)
    case valtype(ltxt)=="D"    ;  return dtoc(ltxt)
    case valtype(ltxt)=="L"    ;  return if(ltxt,".T.",".F.")
 endcase
return ""

static function str2arr( cList, cDelimiter )
LOCAL nPos
LOCAL aList := {}
LOCAL nlencd:=0
LOCAL asub
DO CASE
 CASE VALTYPE(cDelimiter)=='C'
   cDelimiter:=if(cDelimiter==NIL,",",cDelimiter)
   nlencd:=len(cdelimiter)
   DO WHILE ( nPos := AT( cDelimiter, cList )) != 0
      AADD( aList, SUBSTR( cList, 1, nPos - 1 ))
      cList := SUBSTR( cList, nPos + nlencd )
   ENDDO
   AADD( aList, cList )
 CASE VALTYPE(cDelimiter)=='N'
   DO WHILE len((nPos:=left(cList,cDelimiter)))==cDelimiter
      aadd(aList,nPos)
      cList:=substr(cList,cDelimiter+1)
   ENDDO
 CASE VALTYPE(cDelimiter)=='A'
   AEVAL(cDelimiter,{|x| nlencd+=x})
   DO WHILE len((nPos:=left(cList,nlencd)))==nlencd
      asub:={}
      aeval(cDelimiter,{|x| aadd(asub,left(nPos,x)),nPos:=substr(nPos,x+1)})
      aadd(aList,asub)
      cList:=substr(cList,nlencd+1)
   ENDDO
 ENDCASE
RETURN ( aList )

**********************************************
METHOD PrevThumb(nclick) CLASS HBPrinter
**********************************************
local i,spage
 if iloscstron==1
    return self
 endif
 if nclick<>NIL
    page:=ngroup*15+nclick
    ::prevshow()
*    hbpreview.combo_1.value:=page
    SetProperty ( 'hbpreview' , 'combo_1' , 'value' , Page )
    return self
 endif
 if int((page-1)/15)<>ngroup
    ngroup:=int((page-1)/15)
 else
    return self
 endif
 spage:=ngroup*15

 for i:=1 to 15
  if i+spage>iloscstron
      HideWindow(ath[i,5])
  else
     if ::Metafiles[i+spage,2]>=::Metafiles[i+spage,3]
       ath[i,3]:=dy-5
       ath[i,4]:=dx*::Metafiles[i+spage,3]/::Metafiles[i+spage,2]-5
     else
       ath[i,4]:=dx-5
       ath[i,3]:=dy*::Metafiles[i+spage,2]/::Metafiles[i+spage,3]-5
     endif
     rr_playthumb(ath[i],::Metafiles[i+spage],alltrim(str(i+spage)),i)
     CShowControl(ath[i,5])
  endif
 next
return self

**************************************
METHOD PrevShow() CLASS HBPrinter
**************************************
local spos:={0,0}

 if ::Thumbnails
    ::Prevthumb()
 endif
 if !empty(azoom[3]).and.!empty(azoom[4])
    spos[1]:=GetScrollpos(ahs[5,7],SB_HORZ)/azoom[4]
    spos[2]:=GetScrollpos(ahs[5,7],SB_VERT)/(azoom[3])
 endif
 if ::MetaFiles[page,2]>=::MetaFiles[page,3]
         azoom[3]:=(ahs[5,3])*scale-60
         azoom[4]:=(ahs[5,3]*::MetaFiles[page,3]/::MetaFiles[page,2])*scale-60
 else
         azoom[3]:=(ahs[5,4]*::MetaFiles[page,2]/::MetaFiles[page,3])*scale-60
         azoom[4]:=(ahs[5,4])*scale-60
 endif
 _SetItem ("StatusBar","hbpreview",1,aopisy[15]+" "+alltrim(str(page)))

 if azoom[3]<30
    scale:=scale*1.25
    ::prevshow()
    msgstop(aopisy[18],aopisy[1], , .F., .F.)
 endif
 HideWindow(ahs[6,7])
 _SetControlHeight("i1","hbpreview1",azoom[3]+20)
 _SetControlWidth ("i1","hbpreview1",azoom[4])
 SetScrollRange(ahs[5,7],SB_VERT,0,azoom[3]+20,1)
 SetScrollRange(ahs[5,7],SB_HORZ,0,azoom[4],1)

 if !rr_previewplay(ahs[6,7],::METAFILES[page],azoom)
      scale:=scale/1.25
      ::PrevShow()
      msgstop(aopisy[18],aopisy[1], , .F., .F.)
 endif
 rr_scrollwindow(ahs[5,7],-spos[1]*azoom[4],-spos[2]*azoom[3])
 CShowControl(ahs[6,7])
return self

**************************************
METHOD PrevPrint(n1) CLASS HBPrinter
**************************************
local i,ilkop,toprint:=.t.
 ::Previewmode:=.f.
 ::Printingemf:=.t.
 rr_lalabye(1)
 if n1<>NIL
       ::startdoc()
       ::setpage(::MetaFiles[n1,6],::MetaFiles[n1,7])
       ::startpage()
       rr_PlayEnhMetaFile(::MetaFiles[n1],::hDCRef)
       ::endpage()
       ::enddoc()
 else
       for ilkop = 1 to ::nCopies
        ::startdoc()
          for i:=max(1,::nFromPage) to min(iloscstron,::nToPage)
           do case
             case ::PrintOpt==1                    ; toprint:=.t.
             case ::PrintOpt==2 .or. ::PrintOpt==4 ; toprint:=!(i%2==0)
             case ::PrintOpt==3 .or. ::PrintOpt==5 ; toprint:=(i%2==0)
           endcase
           if toprint
              toprint:=.f.
              ::setpage(::MetaFiles[i,6],::MetaFiles[i,7])
              ::startpage()
              rr_PlayEnhMetaFile(::MetaFiles[i],::hDCRef)
              ::endpage()
           endif
          next i
          ::enddoc()

          if ::PrintOpt==4 .or. ::PrintOpt==5
             MsgBox(aopisy[30],aopisy[29], .F., .F.)
             ::startdoc()
             for i:=max(1,::nFromPage) to min(iloscstron,::nToPage)
               do case
                  case ::PrintOpt==4 ; toprint:=(i%2==0)
                  case ::PrintOpt==5 ; toprint:=!(i%2==0)
               endcase
               if toprint
                  toprint:=.f.
                  ::setpage(::MetaFiles[i,6],::MetaFiles[i,7])
                  ::startpage()
                  rr_PlayEnhMetaFile(::MetaFiles[i],::hDCRef)
                  ::endpage()
               endif
            next i
            ::enddoc()
          endif
       next ilkop
 endif
 rr_lalabye(0)
 ::printingemf:=.f.
 ::Previewmode:=.t.
return self

**********************************
Static Function LangInit
**********************************
#ifdef _MULTILINGUAL_
Local cLang := Set ( _SET_LANGUAGE )
#endif
Local aLang := { 'Preview'                    , ;
                 '&Cancel'                    , ;
                 '&Print'                     , ;
                 '&Save'                      , ;
                 '&First'                     , ;
                 'P&revious'                  , ;
                 '&Next'                      , ;
                 '&Last'                      , ;
                 'Zoom &In'                   , ;
                 '&Zoom Out'                  , ;
                 '&Options'                   , ;
                 'Go To Page:'                , ;
                 'Page preview '              , ;
                 'Thumbnails preview'         , ;
                 'Page'                       , ;
                 'Print only current page'    , ;
                 'Pages:'                     , ;
                 'No more zoom !'             , ;
                 'Print options'              , ;
                 'Print from'                 , ;
                 'to'                         , ;
                 'Copies'                     , ;
                 'Print Range'                , ;
                 'All from range'             , ;
                 'Odd only'                   , ;
                 'Even only'                  , ;
                 'All but odd first'          , ;
                 'All but even first'         , ;
                 'Printing ....'              , ;
                 'Waiting for paper change...', ;
                 'Sa&ve as...'                , ; //31
                 'Save &All'                  , ; //32
                 'EMF Files'                  , ; //33
                 'All Files'                    ; //34
               }

#ifdef _MULTILINGUAL_

// LANGUAGE IS NOT SUPPORTED BY hb_langSelect() FUNCTION
If ( _HMG_LANG_ID == 'FI' )    // FINNISH
   cLang := 'FI'		
Endif

Do case
   // Russian
   Case ( ( cLang == 'RUWIN'  ) .or. ;
          ( cLang == 'RU866'  ) .or. ;
          ( cLang == 'RUKOI8' )      ;
        )
      aLang := { '', ;
                 '', ;
                 '', ;
                 '', ;
                 '', ;
                 '', ;
                 '', ;
                 '', ;
                 '', ;
                 '', ;
                 '', ;
                 ':', ;
                 '  ', ;
                 '', ;
                 '', ;
                 ' ', ;
                 ':', ;
                 '  !', ;
                 ' ', ;
                 ' ', ;
                 '', ;
                 '', ;
                 '', ;
                 ' ', ;
                 '', ;
                 '׸', ;
                 ',   ', ;
                 ',   ', ;
                 ' ....', ;
                 ' ...', ;
                 ' ...', ;
                 ' ', ;
                 ' EMF', ;
                 ' '  ;
               }
        
   // Ukrainian 
   Case ( ( cLang == 'UAWIN' ) .or. ;
          ( cLang == 'UA866' )      ;
        )
      aLang := { '', ;
                 'i', ;
                 '', ;
                 '', ;
                 '', ;
                 '', ;
                 '', ;
                 'i', ;
                 'i', ;
                 '', ;
                 'i', ;
                 'i:', ;
                 ' i ', ;
                 'ii', ;
                 'i', ;
                 ' ', ;
                 'i:', ;
                 '  !', ;
                 ' ', ;
                  'i ', ;
                 '', ;
                 'i', ;
                 '', ;
                 'i i', ;
                 'i', ;
                 'i', ;
                 'i,   i', ;
                 'i,   i', ;
                 ' ....', ;
                 'ii i...', ;
                 ' ...', ;
                 ' ', ;
                 ' EMF', ;
                 'i ' ;
               }
        
   // Italian
   Case ( cLang == 'IT' )
      aLang := { 'Anteprima', ;
                 '&Cancella', ;
                 'S&tampa', ;
                 '&Salva', ;
                 '&Primo', ;
                 '&Indietro', ;
                 '&Avanti', ;
                 '&Ultimo', ;
                 'Zoom In', ;
                 'Zoom Out', ;
                 '&Opzioni', ;
                 'Pagina:', ;
                 'Pagina anteprima ', ;
                 'Miniatura Anteprima', ;
                 'Pagina', ;
                 'Stampa solo pagina attuale', ;
                 'Pagine:', ;
                 'Limite zoom !', ;
                 'Opzioni Stampa', ;
                 'Stampa da', ;
                 'a', ;
                 'Copie', ;
                 'Range Stampa', ;
                 'Tutte', ;
                 'Solo dispari', ;
                 'Solo pari', ;
                 'Tutte iniziando dispari', ;
                 'Tutte iniziando pari', ;
                 'Stampa in corso ....', ;
                 'Attendere cambio carta...', ;
                 'Sa&ve as...', ;
                 'Save &All', ;
                 'EMF Files', ;
                 'All Files'  ;
               }

   // Spanish
   Case ( ( cLang == 'ES'    ) .or. ;
          ( cLang == 'ESWIN' )      ;
        )
      aLang := { 'Vista Previa', ;
                 '&Cancelar', ;
                 '&Imprimir', ;
                 '&Guardar', ;
                 '&Primera', ;
                 '&Anterior', ;
                 '&Siguiente', ;
                 '&Ultima', ;
                 'Zoom +', ;
                 'Zoom -', ;
                 '&Opciones', ;
                 'Pag.:', ;
                 'Pagina ', ;
                 'Miniaturas', ;
                 'Pag.', ;
                 'Imprimir solo pag. actual', ;
                 'Paginas:', ;
                 'Zoom Maximo/Minimo', ;
                 'Opciones de Impresion', ;
                 'Imprimir Desde', ;
                 'Hasta', ;
                 'Copias', ;
                 'Imprimir rango', ;
                 'Todo a partir de desde', ;
                 'Solo impares', ;
                 'Solo pares', ;
                 'Todo (impares primero)', ;
                 'Todo (pares primero)', ;
                 'Imprimiendo ....', ;
                 'Esperando cambio de papel...', ;
                 'Sa&ve as...', ;
                 'Save &All', ;
                 'EMF Files', ;
                 'All Files'  ;
               }

   // Polish
   Case ( ( cLang == 'PLWIN' ) .or. ;
          ( cLang == 'PL852' ) .or. ;
          ( cLang == 'PLISO' ) .or. ;
          ( cLang == ''      ) .or. ;
          ( cLang == 'PLMAZ' )      ;
        ) 
      aLang := { 'Podgld', ;
                 '&Rezygnuj', ;
                 '&Drukuj', ;
                 'Zapisz', ;
                 'Pierwsza', ;
                 'Poprzednia', ;
                 'Nastpna', ;
                 'Ostatnia', ;
                 'Powiksz', ;
                 'Pomniejsz', ;
                 'Opc&je', ;
                 'Id do strony:', ;
                 'Podgld strony', ;
                 'Podgld miniaturek', ;
                 'Strona', ;
                 'Drukuj aktualn stron', ;
                 'Stron:', ;
                 'Nie mozna wicej !', ;
                 'Opcje drukowania', ;
                 'Drukuj od', ;
                 'do', ;
                 'Kopii', ;
                 'Zakres', ;
                 'Wszystkie z zakresu', ;
                 'Tylko nieparzyste', ;
                 'Tylko parzyste', ;
                 'Najpierw nieparzyste', ;
                 'Najpierw parzyste', ;
                 'Drukowanie ....', ;
                 'Czekam na zmiane papieru...', ;
                 'Sa&ve as...', ;
                 'Save &All', ;
                 'EMF Files', ;
                 'All Files'  ;
               }

   // Portuguese
   Case ( cLang == 'PT' )
      aLang := { 'Prvisualizao', ;
                 '&Cancelar', ;
                 '&Imprimir', ;
                 '&Salvar', ;
                 '&Primeira', ;
                 '&Anterior', ;
                 'Prximo', ;
                 '&ltimo', ;
                 'Zoom +', ;
                 'Zoom -', ;
                 '&Opes', ;
                 'Pag.:', ;
                 'Pgina ', ;
                 'Miniaturas', ;
                 'Pag.', ;
                 'Imprimir somente a pag. atual', ;
                 'Pginas:', ;
                 'Zoom Mximo/Minimo', ;
                 'Opes de Impresso', ;
                 'Imprimir de', ;
                 'a', ;
                 'Cpias', ;
                 'Imprimir rango', ;
                 'Tudo a partir desta', ;
                 'S mpares', ;
                 'S Pares', ;
                 'Todas as mpares Primeiro', ;
                 'Todas Pares primeiro', ;
                 'Imprimindo ....', ;
                 'Esperando por papel...', ;
                 'Sa&lvar Como...', ;
                 'Salvar &Tudo', ;
                 'Arquivos EMF', ;
                 'Todos Arquivos' ;
               }

   // German
   Case ( ( cLang == 'DEWIN' ) .or. ;
          ( cLang == 'DE'    )      ;
        )   
      aLang := { 'Vorschau', ;
                 '&Abbruch', ;
                 '&Drucken', ;
                 '&Speichern', ;
                 '&Erste', ;
                 '&Vorige', ;
                 '&Nchste', ;
                 '&Letzte', ;
                 'Ver&grern', ;
                 'Ver&kleinern', ;
                 '&Optionen', ;
                 'Seite:', ;
                 'Seitenvorschau', ;
                 'berblick', ;
                 'Seite', ;
                 'Aktuelle Seite drucken', ;
                 'Seiten:', ;
                 'Maximum erreicht!', ;
                 'Druckeroptionen', ;
                 'Drucke von', ;
                 'bis', ;
                 'Anzahl', ;
                 'Bereich', ;
                 'Alle Seiten', ;
                 'Ungerade Seiten', ;
                 'Gerade Seiten', ;
                 'Alles, ungerade Seiten zuerst', ;
                 'Alles, gerade Seiten zuerst', ;
                 'Druckt ....', ;
                 'Bitte Papier nachlegen...', ;
                 'Sa&ve as...', ;
                 'Save &All', ;
                 'EMF Files', ;
                 'All Files' ;
               }

   // French
   Case ( cLang == 'FR' )
      aLang := { 'Prvisualisation', ;
                 '&Abandonner', ;
                 '&Imprimer', ;
                 '&Sauver', ;
                 '&Premier', ;
                 'P&rcdent', ;
                 '&Suivant', ;
                 '&Dernier', ;
                 'Zoom +', ;
                 'Zoom -', ;
                 '&Options', ;
                 'Aller  la page:', ;
                 'Aperu de la page', ;
                 'Aperu affichettes', ;
                 'Page', ;
                 'Imprimer la page en cours', ;
                 'Pages:', ;
                 'Plus de zoom !', ;
                 "Options d'impression", ;
                 'Imprimer de', ;
                 '', ;
                 'Copies', ;
                 "Intervalle d'impression", ;
                 "Tout dans l'intervalle", ;
                 'Impair seulement', ;
                 'Pair seulement', ;
                 "Tout mais impair d'abord", ;
                 "Tout mais pair d'abord", ;
                 'Impression ....', ;
                 'Attente de changement de papier...', ;
                 'Sa&ve as...', ;
                 'Save &All', ;
                 'EMF Files', ;
                 'All Files' ;
               }

   // Finnish 
   Case ( cLang == 'FI' )
      aLang := { 'Esikatsele', ;
                 '&Keskeyt', ;
                 '&Tulosta', ;
                 'T&allenna', ;
                 '&Ensimminen', ;
                 'E&dellinen', ;
                 '&Seuraava', ;
                 '&Viimeinen', ;
                 'Suurenna', ;
                 'Pienenn', ;
                 '&Optiot', ;
                 'Mene sivulle:', ;
                 'Esikatsele sivu ', ;
                 'Esikatsele miniatyyrit', ;
                 'Sivu', ;
                 'Tulosta tm sivu', ;
                 'Sivuja:', ;
                 'Ei voi suurentaa !', ;
                 'Tulostus optiot', ;
                 'Alkaen', ;
                 '->', ;
                 'Kopiot', ;
                 'Tulostus alue', ;
                 'Kaikki alueelta', ;
                 'Vain parittomat', ;
                 'Vain parilleset', ;
                 'Kaikki paitsi ensim. pariton', ;
                 'Kaikki paitsi ensim. parillinen', ;
                 'Tulostan ....', ;
                 'Odotan paperin vaihtoa...', ;
                 'Tallenna nimell...', ;
                 'Tallenna kaikki', ;
                 'EMF Tiedostot', ;
                 'Kaikki Tiedostot' ;
               }

   // Dutch
   Case ( cLang == 'NL' )
      aLang := { 'Afdrukvoorbeeld', ;
                 'Annuleer', ;
                 'Print', ;
                 'Opslaan', ;
                 'Eerste', ;
                 'Vorige', ;
                 'Volgende', ;
                 'Laatste', ;
                 'Inzoomen', ;
                 'Uitzoomen', ;
                 'Opties', ;
                 'Ga naar pagina:', ;
                 'Pagina voorbeeld ', ;
                 'Thumbnails voorbeeld', ;
                 'Pagina', ;
                 'Print alleen huidige pagina', ;
                 "Pagina's:", ;
                 'Geen zoom meer !', ;
                 'Print opties', ;
                 'Print van', ;
                 'tot', ;
                 'Aantal exemplaren', ;
                 "Pagina's", ;
                 "Alle pagina's", ;
                 'Alleen oneven', ;
                 'Alleen even', ;
                 'Alles maar oneven eerst', ;
                 'Alles maar even eerst', ;
                 'Printen ....', ;
                 'Wacht op papier wissel...', ;
                 'Be&waar als...', ;
                 'Bewaar &Alles', ;
                 'EMF-bestanden', ;
                 'Alle bestanden' ;
               }

   // Czech
   Case ( ( cLang == 'CS'    ) .or. ;
          ( cLang == 'CSWIN' )      ;
        )
      aLang := { "Nhled", ;
                 "&Storno", ;
                 "&Tisk", ;
                 "&Uloit", ;
                 "&Prvn", ;
                 "P&edchoz", ;
                 "&Dal", ;
                 "P&osledn", ;
                 "Z&vtit", ;
                 "&Zmenit", ;
                 "&Monosti", ;
                 "Uk stranu:", ;
                 "Nhled strany ", ;
                 "Nhled vc strn", ;
                 "Strana", ;
                 "Tisk aktulnej strany", ;
                 "Strn:", ;
                 "Nemono dle zmnit velikost!", ;
                 "Monosti tisku", ;
                 "Tisk od", ;
                 "po", ;
                 "Kpi", ;
                 "Tisk strn", ;
                 "Vech strn", ;
                 "Len lich", ;
                 "Len sud", ;
                 "Vechny okrem prvej lichej", ;
                 "Vechny okrem prvej sudej", ;
                 "Tisknem ...", ;
                 "ekm na papr ...", ;
                 "Uloit &jako...", ;
                 "Uloit &vetko", ;
                 "EMF soubor", ;
                 "Vechen soubor", ;
               }

   // Slovak
   Case ( ( cLang == 'SK'    ) .or. ;
          ( cLang == 'SKWIN' )      ;
        )
      aLang := { "Nhad", ;
                 "&Storno", ;
                 "&Tla", ;
                 "&Uloit", ;
                 "&Prv", ;
                 "P&redchzajca", ;
                 "&alia", ;
                 "Po&sledn", ;
                 "Z&vi", ;
                 "&Zmeni", ;
                 "&Monosti", ;
                 "Uk stranu:", ;
                 "Nhad strany ", ;
                 "Nhad viac strnok", ;
                 "Strana", ;
                 "Tla aktulnej strany", ;
                 "Stran:", ;
                 "Vekos alej nie je mon zmeni!", ;
                 "Monosti tlae", ;
                 "Tla od", ;
                 "po", ;
                 "Kpi", ;
                 "Tla strn", ;
                 "Vetky strany", ;
                 "Len neprne", ;
                 "Len prne", ;
                 "Vetko okrem prvej neprnej", ;
                 "Vetko okrem prvej prnej", ;
                 "Tlam ...", ;
                 "akm na papier ...", ;
                 "Uloi &ako...", ;
                 "Uloi &vetko", ;
                 "EMF sbor", ;
                 "Vetky sbory", ;
               }

   // Slovenian
   Case ( ( cLang == 'SL'    ) .or. ;
          ( cLang == 'SLWIN' )      ;
        )
      aLang := { 'Predgled', ;
                 'Prekini', ;
                 'Natisni', ;
                 'Shrani', ;
                 'Prva', ;
                 'Prejnja', ;
                 'Naslednja', ;
                 'Zadnja', ;
                 'Poveaj', ;
                 'Pomanjaj', ;
                 'Monosti', ;
                 'Skok na stran:', ;
                 'Predgled', ;
                 'Mini predgled', ;
                 'Stran', ;
                 'Samo trenutna stran', ;
                 'Strani:' , ;
                 'Ni ve poveave!', ;
                 'Monosti tiskanja', ;
                 'Tiskaj od', ;
                 'do', ;
                 'Kopij', ;
                 'Tiskanje', ;
                 'Vse iz izbora', ;
                 'Samo neparne strani', ;
                 'Samo parne strani', ;
                 'Vse - le brez prve neparne strani', ;
                 'Vse - le brez prve parne strani', ;
                 'Tiskanje ....'              , ;
                 'akanje na zamenjavo papirja...', ;
                 'Shrani kot...', ;
                 'Shrani vse', ;
                 'EMF datoteke', ;
                 'Vse datoteke' ;
               }

   // Hungarian
   Case ( ( cLang == 'HU'    ) .or. ;
          ( cLang == 'HUWIN' )      ;
        )
      aLang := { "Elnzet", ;
                 "&Mgse", ;
                 "Nyo&mtats", ;
                 "&Ments", ;
                 "&Els", ;
                 "E&lz", ;
                 "&Kvetkez", ;
                 "&Utols", ;
                 "&Nagyts", ;
                 "K&icsinyts", ;
                 "&Opcik", ;
                 "Oldalt mutasd:", ;
                 "Oldal elnzete ", ;
                 "Tbb oldal elnzete", ;
                 "Oldal", ;
                 "Aktulis oldal nyomtatsa", ;
                 "Oldal:", ;
                 "A nagysg tovbb nem vltoztathat!", ;
                 "Nyomtatsi lehetsgek", ;
                 "Nyomtats ettl", ;
                 "eddig", ;
                 "Msolat", ;
                 "Oldalak nyomtatsa", ;
                 "Minden oldalt", ;
                 "Csak a pratlan", ;
                 "Csak a pros", ;
                 "Mindet kivve az els pratlant", ;
                 "Mindet kivve az els prost", ;
                 "Nyomtatom ...", ;
                 "Paprra vrok ...", ;
                 "Ments msknt ...", ;
                 "Mindet mentsd", ;
                 "EMF llomny", ;
                 "Minden llomny", ;
               }

   // Greek - Ellinika
   Case ( ( cLang == 'EL'    ) .or. ;
          ( cLang == 'ELWIN' )      ;
        )
      aLang := { '', ;
                 '&', ;
                 '&', ;
                 '&', ;
                 '&1', ;
                 '&/', ;
                 '&', ;
                 '&/', ;
                 'Zoom +', ;
                 'Zoom -', ;
                 '&', ;
                 ' :', ;
                 ' ', ;
                 '', ;
                 '.', ;
                 '   ', ;
                 ':', ;
                 '  zoom !', ;
                 '', ;
                 ' ', ;
                 '', ;
                 '', ;
                 ' ', ;
                 ' ', ;
                 '  ', ;
                 ' ', ;
                 '    1 ', ;
                 '    1 ', ;
                 ' ....', ;
                 '   ...', ;
                 ' ..', ;
                 ' ', ;
                 ' EMF', ;
                 '  ' ;
               }

   // Bulgarian
   Case ( ( cLang == 'BGWIN'  ) .or. ;
          ( cLang == 'BG866'  )      ;
        )
      aLang := { '', ;
                 '', ;
                 '', ;
                 '', ;
                 '', ;
                 '', ;
                 '', ;
                 '', ;
                 '', ;
                 '', ;
                 '', ;
                 ':', ;
                 '   ', ;
                 '', ;
                 '', ;
                 '  ', ;
                 ':', ;
                 ' e   !', ;
                 '  ', ;
                 ' ', ;
                 '', ;
                 '', ;
                 '', ;
                 ' ', ;
                 '', ;
                 '', ;
                 ',   ', ;
                 ',   ', ;
                 ' ....', ;
                 ' ...', ;
                 ' ...', ;
                 ' ', ;
                 ' EMF', ;
                 ' '  ;
               }
Endcase

#endif
                 
Return aLang

**********************************
METHOD Preview() CLASS HBPrinter
**********************************
local i,pi:=1
private page:=1,scale:=::PREVIEWSCALE,azoom:={0,0,0,0},ahs:={},npages:={}
private dx,dy,ngroup:=-1,ath:={},Iloscstron:=len(::metafiles)
private aOpisy := LangInit()

  if ::nwhattoprint<2
   ::ntopage:=iloscstron
  endif

  if !::PreviewMode .or. empty(::metafiles)
    return self
  endif

  for pi=1 to iloscstron
    AADD(npages,padl(pi,4))
  next pi

  aadd(ahs,{0,0,0,0,0,0,0})
  //rr_getwindowrect(ahs[1])

  if ::Previewrect[3]==-1 .and. ::Previewrect[4]==-1
    ::Previewrect:=rr_getdesktoparea()
  endif
  if ::PreviewRect[3]>0 .and. ::PreviewRect[4]>0
    ahs[1,1]:=::Previewrect[1]
    ahs[1,2]:=::Previewrect[2]
    ahs[1,3]:=::Previewrect[3]
    ahs[1,4]:=::Previewrect[4]
    ahs[1,5]:=::Previewrect[3]-::Previewrect[1]+1
    ahs[1,6]:=::Previewrect[4]-::Previewrect[2]+1
  else
    pi:=rr_getdesktoparea()
    ahs[1,1]:=pi[1]+10
    ahs[1,2]:=pi[2]+10
    ahs[1,3]:=pi[3]-10
    ahs[1,4]:=pi[4]-10
    ahs[1,5]:=ahs[1,3]-ahs[1,1]+1
    ahs[1,6]:=ahs[1,4]-ahs[1,2]+1
  endif

  DEFINE WINDOW HBPREVIEW ;
         AT  ahs[1,1] , ahs[1,2] ;
         WIDTH ahs[1,6] HEIGHT ahs[1,5] ;
         TITLE aopisy[1] ICON 'zzz_Printicon' ;
         MODAL NOSIZE ;
         FONT 'Arial' SIZE 9

         _definehotkey("HBPREVIEW", 0,  27, {|| ::PrevClose(.T.) }) //Escape
         _definehotkey("HBPREVIEW", 0, 107, {|| scale:=scale*1.25, ::PrevShow() }) //zoom in
         _definehotkey("HBPREVIEW", 0, 109, {|| scale:=scale/1.25, ::PrevShow() }) //zoom out
         _definehotkey("HBPREVIEW", MOD_CONTROL, VK_P, {|| ::prevprint(), if(::CLSPREVIEW, ::PrevClose(.F.), nil) }) //Print
         _definehotkey("HBPREVIEW", 0,  33, {|| page:=::CurPage:=if(page==1,1,page-1), HBPREVIEW.combo_1.value:=page, ::PrevShow() }) //back
         _definehotkey("HBPREVIEW", 0,  34, {|| page:=::CurPage:=if(page==iloscstron,page,page+1), HBPREVIEW.combo_1.value:=page, ::PrevShow() }) //next

         DEFINE STATUSBAR

                STATUSITEM aopisy[15]+" "+alltrim(str(page)) WIDTH 100
                STATUSITEM aopisy[16] WIDTH 200 ICON 'zzz_Printicon'  ACTION ::PREVPRINT(PAGE) RAISED
                STATUSITEM aopisy[17]+" "+alltrim(str(iloscstron)) WIDTH 100

         END STATUSBAR

         @ 17, ahs[1,6]-184 LABEL prl VALUE aopisy[12] WIDTH 100 HEIGHT 18 FONT 'Arial' SIZE 8 RIGHTALIGN
         @ 15, ahs[1,6]-77  COMBOBOX combo_1  ITEMS npages VALUE 1 WIDTH 58 FONT 'Arial' SIZE  8 ON CHANGE {||page:=::CurPage:=HBPREVIEW.combo_1.value,::PrevShow() }

         DEFINE SPLITBOX
                DEFINE TOOLBAR TB1 BUTTONSIZE 50, 37 FONT 'Arial Narrow' SIZE 8 FLAT BREAK
                       BUTTON B3 CAPTION  aopisy[4]  PICTURE 'hbprint_save'  WHOLEDROPDOWN 
                       DEFINE DROPDOWN MENU BUTTON B3
                              ITEM aopisy[4]  ACTION {|| ::savemetafiles(::CurPage)}
                              ITEM aopisy[31] ACTION {|| pi := Putfile ( { {aopisy[33],'*.emf'}, {aopisy[34],'*.*'} }, , GetStartUpFolder(), .t. ), if(empty(pi), ,::savemetafiles(::CurPage, pi))}
                              ITEM aopisy[32] ACTION {|| ::savemetafiles()}
                       END MENU
                       BUTTON B2 CAPTION  aopisy[3]  PICTURE 'hbprint_print' ACTION {|| ::prevprint(), if(::CLSPREVIEW, ::PrevClose(.F.), nil) }
                       BUTTON B10 CAPTION aopisy[11] PICTURE 'hbprint_option'  ACTION {|| ::PrintOption() } Separator
                       if iloscstron>1
                          BUTTON B4 CAPTION  aopisy[5] PICTURE 'hbprint_top'   ACTION {|| page:=::CurPage:=1, HBPREVIEW.combo_1.value:=page, ::PrevShow() }
                          BUTTON B5 CAPTION  aopisy[6] PICTURE 'hbprint_back'  ACTION {|| page:=::CurPage:=if(page==1, 1, page-1), HBPREVIEW.combo_1.value:=page, ::PrevShow() }
                          BUTTON B6 CAPTION  aopisy[7] PICTURE 'hbprint_next'  ACTION {|| page:=::CurPage:=if(page==iloscstron, page, page+1), HBPREVIEW.combo_1.value:=page, ::PrevShow() }
                          BUTTON B7 CAPTION  aopisy[8] PICTURE 'hbprint_end'   ACTION {|| page:=::CurPage:=iloscstron, HBPREVIEW.combo_1.value:=page, ::PrevShow() } Separator
                       endif
                       BUTTON B8 CAPTION  aopisy[9]  PICTURE 'hbprint_zoomin'  ACTION {|| scale:=scale*1.25, ::PrevShow() }
                       BUTTON B9 CAPTION  aopisy[10] PICTURE 'hbprint_zoomout' ACTION {|| scale:=scale/1.25, ::PrevShow() } Separator
                       BUTTON B1 CAPTION  aopisy[2]  PICTURE 'hbprint_close' ACTION {|| ::PrevClose(.T.) } Separator
                END TOOLBAR

                aadd(ahs,{0,0,0,0,0,0,GetFormHandle("hbpreview")})
                rr_getclientrect(ahs[2])
                aadd(ahs,{0,0,0,0,0,0,GetControlHandle("Tb1","hbpreview")})
                rr_getclientrect(ahs[3])
                aadd(ahs,{0,0,0,0,0,0,GetControlHandle("StatusBar","hbpreview")})
                rr_getclientrect(ahs[4])

                DEFINE WINDOW HBPREVIEW1  ;
                       WIDTH ahs[2,6]-15  HEIGHT ahs[2,5]-ahs[3,5]-ahs[4,5]-10 ;
                       VIRTUAL WIDTH ahs[2,6] -5;
                       VIRTUAL HEIGHT ahs[2,5]-ahs[3,5]-ahs[4,5] ;
                       TITLE aopisy[13] SPLITCHILD GRIPPERTEXT "P"

                       aadd(ahs,{0,0,0,0,0,0,GetFormHandle("hbpreview1")})
                       rr_getclientrect(ahs[5])
                       @ ahs[5,2]+10,ahs[5,1]+10 IMAGE I1  PICTURE "" WIDTH ahs[5,6]-10 HEIGHT ahs[5,5]-10
                       aadd(ahs,{0,0,0,0,0,0,GetControlHandle("i1","hbpreview1")})
                       rr_getclientrect(ahs[6])

                       _definehotkey("HBPREVIEW1", 0,  27,{|| _ReleaseWindow("HBPREVIEW") })
                       _definehotkey("HBPREVIEW1", 0, 107,{|| scale:=scale*1.25,:: PrevShow () })
                       _definehotkey("HBPREVIEW1", 0, 109,{|| scale:=scale/1.25,:: PrevShow () })
                       _definehotkey("HBPREVIEW1", MOD_CONTROL, VK_P, {|| ::prevprint(),if ( ::CLSPREVIEW, _ReleaseWindow ("HBPREVIEW" ), nil ) }) //Print

                       if iloscstron > 1
                          _definehotkey("HBPREVIEW1", 0, 33, {|| page:=::CurPage:=if(page==1,1,page-1),HBPREVIEW.combo_1.value:=page, ::PrevShow() })   //back
                          _definehotkey("HBPREVIEW1", 0, 34, {|| page:=::CurPage:=if(page==iloscstron,page,page+1) , HBPREVIEW.combo_1.value:=page,::PrevShow() }) //next
                          _definehotkey("HBPREVIEW1", 0, 35, {|| page:=::CurPage:=iloscstron,HBPREVIEW.combo_1.value:=page,::PrevShow() })  //end
                          _definehotkey("HBPREVIEW1", 0, 36, {|| page:=::CurPage:=1,HBPREVIEW.combo_1.value:=page, ::PrevShow() })  //home
                          _definehotkey("HBPREVIEW1", 0, 37, {|| page:=::CurPage:=if(page==1,1,page-1),HBPREVIEW.combo_1.value:=page, ::PrevShow() })   //Left
                          _definehotkey("HBPREVIEW1", 0, 38, {|| page:=::CurPage:=if(page==1,1,page-1),HBPREVIEW.combo_1.value:=page, ::PrevShow() })   //up
                          _definehotkey("HBPREVIEW1", 0, 39, {|| page:=::CurPage:=if(page==iloscstron,page,page+1) , HBPREVIEW.combo_1.value:=page,::PrevShow() }) //right
                          _definehotkey("HBPREVIEW1", 0, 40, {|| page:=::CurPage:=if(page==iloscstron,page,page+1) , HBPREVIEW.combo_1.value:=page,::PrevShow() }) //down 
                       endif
                END WINDOW

                IF ::thumbnails .and. iloscstron>1
                   DEFINE WINDOW HBPREVIEW2  ;
                       WIDTH ahs[2,6]-15  HEIGHT ahs[2,5]-ahs[3,5]-ahs[4,5]-10 ;
                       TITLE aopisy[14] SPLITCHILD GRIPPERTEXT "T"
                       aadd(ahs,{0,0,0,0,0,0,GetFormHandle("hbpreview2")})
                       rr_getClientRect(ahs[7])
                       dx:=(ahs[5,6]-20)/5-5
                       dy:=ahs[5,5]/3-5
                       for i:=1 to 15
                          aadd(ath,{0,0,0,0,0})
                          if ::Metafiles[1,2]>=::Metafiles[1,3]
                              ath[i,3]:=dy-5
                              ath[i,4]:=dx*::Metafiles[1,3]/::Metafiles[1,2]-5
                          else
                              ath[i,4]:=dx-5
                              ath[i,3]:=dy*::Metafiles[1,2]/::Metafiles[1,3]-5
                          endif
                          ath[i,1]:=int((i-1)/5)*dy+5
                          ath[i,2]:=((i-1)%5)*dx+5
                        next
                        @ath[1 ,1],ath[1 ,2]  image it1  of hbpreview2 picture "" action {|| ::Prevthumb(1 ) } width ath[1 ,4] height ath[1 ,3]
                        @ath[2 ,1],ath[2 ,2]  image it2  of hbpreview2 picture "" action {|| ::Prevthumb(2 ) } width ath[2 ,4] height ath[2 ,3]
                        @ath[3 ,1],ath[3 ,2]  image it3  of hbpreview2 picture "" action {|| ::Prevthumb(3 ) } width ath[3 ,4] height ath[3 ,3]
                        @ath[4 ,1],ath[4 ,2]  image it4  of hbpreview2 picture "" action {|| ::Prevthumb(4 ) } width ath[4 ,4] height ath[4 ,3]
                        @ath[5 ,1],ath[5 ,2]  image it5  of hbpreview2 picture "" action {|| ::Prevthumb(5 ) } width ath[5 ,4] height ath[5 ,3]
                        @ath[6 ,1],ath[6 ,2]  image it6  of hbpreview2 picture "" action {|| ::Prevthumb(6 ) } width ath[6 ,4] height ath[6 ,3]
                        @ath[7 ,1],ath[7 ,2]  image it7  of hbpreview2 picture "" action {|| ::Prevthumb(7 ) } width ath[7 ,4] height ath[7 ,3]
                        @ath[8 ,1],ath[8 ,2]  image it8  of hbpreview2 picture "" action {|| ::Prevthumb(8 ) } width ath[8 ,4] height ath[8 ,3]
                        @ath[9 ,1],ath[9 ,2]  image it9  of hbpreview2 picture "" action {|| ::Prevthumb(9 ) } width ath[9 ,4] height ath[9 ,3]
                        @ath[10,1],ath[10,2]  image it10 of hbpreview2 picture "" action {|| ::Prevthumb(10) } width ath[10,4] height ath[10,3]
                        @ath[11,1],ath[11,2]  image it11 of hbpreview2 picture "" action {|| ::Prevthumb(11) } width ath[11,4] height ath[11,3]
                        @ath[12,1],ath[12,2]  image it12 of hbpreview2 picture "" action {|| ::Prevthumb(12) } width ath[12,4] height ath[12,3]
                        @ath[13,1],ath[13,2]  image it13 of hbpreview2 picture "" action {|| ::Prevthumb(13) } width ath[13,4] height ath[13,3]
                        @ath[14,1],ath[14,2]  image it14 of hbpreview2 picture "" action {|| ::Prevthumb(14) } width ath[14,4] height ath[14,3]
                        @ath[15,1],ath[15,2]  image it15 of hbpreview2 picture "" action {|| ::Prevthumb(15) } width ath[15,4] height ath[15,3]

                        for i:=1 to 15
                           ath[i,5]:=GetControlHandle("it"+alltrim(str(i)),"hbpreview2")
                           rr_playthumb(ath[i],::Metafiles[i],alltrim(str(i)),i)
                           if i>=iloscstron
                             exit
                           endif
                        next
                   END WINDOW
                ENDIF
           END SPLITBOX
  END WINDOW
  ::PrevShow()
  ACTIVATE WINDOW HBPREVIEW
return nil

*******************************************
METHOD PrevClose(lEsc) CLASS HBPrinter
*******************************************
  ::lEscaped := lEsc
  _ReleaseWindow ( "HBPREVIEW" )
return self

*******************************************
METHOD PrintOption() CLASS HBPrinter
*******************************************
Local OKprint:=.f.

If IsWindowDefined(PrOpt)==.f.

  DEFINE WINDOW PrOpt AT 0,0 WIDTH 298 HEIGHT 146+IF(IsXPThemeActive(), 7, 0) ;
	TITLE aopisy[19] ICON 'zzz_Printicon' ;
	MODAL ;
	NOSIZE FONT 'Arial' SIZE 9

  @ 2,1    FRAME   PrOptFrame  WIDTH 291 HEIGHT 117
  @ 19,9   LABEL   label_11  VALUE aopisy[20] WIDTH 87 HEIGHT 16 FONT 'Arial' SIZE 9 BOLD
  @ 18,90  TEXTBOX textFrom  HEIGHT 21 WIDTH 33 NUMERIC Font 'Arial' size 09 MAXLENGTH 4 RIGHTALIGN ON ENTER PrOpt.TextTo.setfocus
  @ 19,134 LABEL   label_12  VALUE aopisy[21] WIDTH 14 HEIGHT 19 FONT 'Arial' SIZE 09 BOLD
  @ 18,156 TEXTBOX textTo HEIGHT 21 WIDTH 33 NUMERIC Font 'Arial' size 09 MAXLENGTH 4 RIGHTALIGN ON ENTER PrOpt.TextCopies.setfocus
  @ 19,200 LABEL   label_18  VALUE aopisy[22]  WIDTH 40 HEIGHT 19 FONT 'Arial' SIZE 09 BOLD
  @ 18,252 TEXTBOX textCopies HEIGHT 21 WIDTH 30 NUMERIC Font 'Arial' size 09 MAXLENGTH 4 RIGHTALIGN ON ENTER PrOpt.prnCombo.setfocus
  @ 55,9   LABEL label_13 VALUE aopisy[23]  WIDTH 71 HEIGHT 17 FONT 'Arial' SIZE 9 BOLD
  @ 50,90  COMBOBOX prnCombo VALUE ::PRINTOPT ITEMS {aopisy[24],aopisy[25],aopisy[26],aopisy[27],aopisy[28]} ;
	WIDTH 195 FONT 'Arial' SIZE 9 ON ENTER PrOpt.Button_14.setfocus

  @ 82,90 BUTTON button_14 CAPTION "&OK";
    ACTION {|| OKprint:=.t. , ::nFromPage:=PrOpt.textFrom.Value , ::nToPage:=PrOpt.textTo.Value , ;
	::nCopies:=max(PrOpt.textCopies.Value,1) , ::PrintOpt:=PrOpt.prnCombo.Value , PrOpt.Release} ;
    WIDTH 93 HEIGHT 24 ;
    FONT 'Arial' SIZE 9

  @ 82,190 BUTTON button_15 CAPTION aopisy[2];
    ACTION PrOpt.Release ;
    WIDTH 93 HEIGHT 24 ;
    FONT 'Arial' SIZE 9

  END WINDOW

  _definehotkey("PrOpt", 0, 27, {||_ReleaseWindow("PrOpt") })
  PrOpt.textCopies.Value := ::nCopies
  PrOpt.textFrom.Value := max(::nfrompage,1)
  PrOpt.textTo.Value := if(::nwhattoprint<2,iloscstron,::ntopage)
  PrOpt.Center
  PrOpt.Activate

endif

Return OKPrint

METHOD ReportData(l_x1,l_x2,l_x3,l_x4,l_x5,l_x6) CLASS HBPrinter
set printer to "hbprinter.rep" ADDITIVE
set device to print
set printer on
set console off
 ? '-----------------',date(),time()
 ?
 ?? if(valtype(l_x1)<>"U",l_x1,",")
 ?? if(valtype(l_x2)<>"U",l_x2,",")
 ?? if(valtype(l_x3)<>"U",l_x3,",")
 ?? if(valtype(l_x4)<>"U",l_x4,",")
 ?? if(valtype(l_x5)<>"U",l_x5,",")
 ?? if(valtype(l_x6)<>"U",l_x6,",")
 ? 'HDC            :',::HDC
 ? 'HDCREF         :',::HDCREF
 ? 'PRINTERNAME    :',::PRINTERNAME
 ? 'PRINTEDEFAULT  :',::PRINTERDEFAULT
 ? 'VERT X HORZ SIZE         :',::DEVCAPS[1],"x",::DEVCAPS[2]
 ? 'VERT X HORZ RES          :',::DEVCAPS[3],"x",::DEVCAPS[4]
 ? 'VERT X HORZ LOGPIX       :',::DEVCAPS[5],"x",::DEVCAPS[6]
 ? 'VERT X HORZ PHYS. SIZE   :',::DEVCAPS[7],"x",::DEVCAPS[8]
 ? 'VERT X HORZ PHYS. OFFSET :',::DEVCAPS[9],"x",::DEVCAPS[10]
 ? 'VERT X HORZ FONT SIZE    :',::DEVCAPS[11],"x",::DEVCAPS[12]
 ? 'VERT X HORZ ROWS COLS    :',::DEVCAPS[13],"x",::DEVCAPS[14]
 ? 'ORIENTATION              :',::DEVCAPS[15]
 ? 'PAPER SIZE               :',::DEVCAPS[17]
set printer off
set printer to
set console on
set device to screen
return self


#pragma BEGINDUMP

#define _WIN32_IE    0x0500
#define HB_OS_WIN_32_USED
#define _WIN32_WINNT 0x0400
#define WINVER       0x0400

#include <windows.h>
#include <winuser.h>
#include <wingdi.h>
#include "hbapi.h"
#include "hbvm.h"
#include "hbstack.h"
#include "hbapiitm.h"
#include <olectl.h>
#include <ocidl.h>
#include <commctrl.h>

static HDC              hDC = NULL;
static HDC              hDCRef = NULL;
static HDC              hDCtemp;
static DEVMODE          *pDevMode = NULL;
static DEVMODE          *pDevMode2 = NULL;
static DEVNAMES         *pDevNames = NULL;
static HANDLE           hPrinter = NULL;
static PRINTER_INFO_2   *pi2 = NULL;
static PRINTER_INFO_2   *pi22 = NULL;  // to restore printer dev mode after print.
static PRINTER_DEFAULTS pd;
static PRINTDLG         pdlg;
static DOCINFO          di;
static int              nFromPage = 0;
static int              nToPage = 0;
static char             PrinterName[128];
static char             PrinterDefault[128];
static DWORD            charset = DEFAULT_CHARSET;
static HFONT            hfont;
static HPEN             hpen;
static HBRUSH           hbrush;
static int              textjust = 0;
static int              devcaps[] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0 };
static int              preview = 0;
static int              polyfillmode = 1;
static HRGN             hrgn = NULL;
static HBITMAP          himgbmp;
static HBITMAP          hbmp[] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
static OSVERSIONINFO    osvi;

void                    rr_getdevmode( void );

HB_FUNC( RR_FINISH )
{
   pDevMode = NULL;
   pDevMode2 = NULL;
   pDevNames = NULL;
   ClosePrinter( hPrinter );
   hPrinter = NULL;
   pi2 = NULL;
   memset( &pd, 0, sizeof(pd) );
   memset( &pdlg, 0, sizeof(pdlg) );
   memset( &di, 0, sizeof(di) );
   nFromPage = 0;
   nToPage = 0;
   hfont = NULL;
   hpen = NULL;
   hbrush = NULL;
   textjust = 0;
   memset( &devcaps, 0, sizeof(devcaps) );
   devcaps[15] = 1;
   preview = 0;
   polyfillmode = 1;
   hrgn = NULL;
   himgbmp = NULL;
   memset( &hbmp, 0, sizeof(hbmp) );
}

HB_FUNC( RR_PRINTERNAME )
{
   hb_retc( PrinterName );
}

HB_FUNC( RR_PRINTDIALOG )
{
   LPCTSTR  pDevice;
   memset( &pdlg, 0, sizeof(pdlg) );
   pdlg.lStructSize = sizeof( pdlg );
   pdlg.Flags = PD_RETURNDC | PD_ALLPAGES;
   pdlg.hwndOwner = GetActiveWindow(); // Identifies the window that owns the dialog box.
   pdlg.nFromPage = 1;
   pdlg.nToPage = 1;

   //  pdlg.nMinPage=1;
   //  pdlg.nMaxPage=999999;

   if( PrintDlg(&pdlg) )
   {
      hDC = pdlg.hDC;
      pDevMode = ( LPDEVMODE ) GlobalLock( pdlg.hDevMode );
      pDevNames = ( LPDEVNAMES ) GlobalLock( pdlg.hDevNames );

      // Note: pDevMode->dmDeviceName is limited to 32 characters.
      // if the printer name is greater than 32, like network printers,
      // the rr_getdc() function return a null handle. So, I'm using
      // pDevNames instead pDevMode. (E.F.)
      //strcpy(PrinterName,pDevMode->dmDeviceName);

      pDevice = ( LPCTSTR ) pDevNames + pDevNames->wDeviceOffset;
      strcpy( PrinterName, ( char * ) pDevice );

      if( hDC == NULL )
      {
         strcpy( PrinterName, "" );
         GlobalUnlock( pdlg.hDevMode );
         GlobalUnlock( pdlg.hDevNames );
      }
      else
      {
         hb_stornl( (LONG) pdlg.nFromPage, 1, 1 );
         hb_stornl( (LONG) pdlg.nToPage, 1, 2 );
         hb_stornl( (LONG) pdlg.nCopies, 1, 3 );
         if( (pdlg.Flags & PD_PAGENUMS) == PD_PAGENUMS )
         {
            hb_stornl( 2, 1, 4 );
         }
         else if( (pdlg.Flags & PD_SELECTION) == PD_SELECTION )
         {
            hb_stornl( 1, 1, 4 );
         }
         else
         {
            hb_stornl( 0, 1, 4 );
         }

         rr_getdevmode();
      }
   }
   else
   {
      hDC = 0;
   }

   hDCRef = hDC;

   hb_retnl( (LONG) hDC );
}

HB_FUNC( RR_GETDC )
{
   if( osvi.dwPlatformId == VER_PLATFORM_WIN32_NT )
   {
      hDC = CreateDC( "WINSPOOL", hb_parc(1), NULL, NULL );
   }
   else
   {
      hDC = CreateDC( NULL, hb_parc(1), NULL, NULL );
   }

   if( hDC )
   {
      strcpy( PrinterName, hb_parc(1) );
      rr_getdevmode();
   }

   hDCRef = hDC;
   hb_retnl( (LONG) hDC );
}

void rr_getdevmode()
{
   DWORD dwNeeded = 0;
   memset( &pd, 0, sizeof(pd) );
   pd.DesiredAccess = PRINTER_ALL_ACCESS;
   OpenPrinter( PrinterName, &hPrinter, NULL );
   GetPrinter( hPrinter, 2, 0, 0, &dwNeeded );
   pi2 = ( PRINTER_INFO_2 * ) GlobalAlloc( GPTR, dwNeeded );
   GetPrinter( hPrinter, 2, (LPBYTE) pi2, dwNeeded, &dwNeeded );
   pi22 = ( PRINTER_INFO_2 * ) GlobalAlloc( GPTR, dwNeeded );
   GetPrinter( hPrinter, 2, (LPBYTE) pi22, dwNeeded, &dwNeeded );
   if( pDevMode )
   {
      pi2->pDevMode = pDevMode;
   }
   else
   {
      if( pi2->pDevMode == NULL )
      {
         dwNeeded = DocumentProperties( NULL, hPrinter, PrinterName, NULL, NULL, 0 );
         pDevMode2 = ( DEVMODE * ) GlobalAlloc( GPTR, dwNeeded );
         DocumentProperties( NULL, hPrinter, PrinterName, pDevMode2, NULL, DM_OUT_BUFFER );
         pi2->pDevMode = pDevMode2;
      }
   }

   hfont = ( HFONT ) GetCurrentObject( hDC, OBJ_FONT );
   hbrush = ( HBRUSH ) GetCurrentObject( hDC, OBJ_BRUSH );
   hpen = ( HPEN ) GetCurrentObject( hDC, OBJ_PEN );
}

HB_FUNC( EF_RESETPRINTER )
{
   if( pi22 )
   {
      SetPrinter( hPrinter, 2, (LPBYTE) pi22, 0 );
   }

   GlobalFree( pi22 );
   pi22 = NULL;
}

HB_FUNC( RR_DELETEDC )
{
   if( pdlg.hDevMode )
   {
      GlobalUnlock( pdlg.hDevMode );
   }

   if( pDevMode )
   {
      GlobalFree( pDevMode );
   }

   if( pDevMode2 )
   {
      GlobalFree( pDevMode2 );
   }

   if( pDevNames )
   {
      GlobalFree( pDevNames );
   }

   if( pi2 )
   {
      GlobalFree( pi2 );
   }

   DeleteDC( (HDC) hb_parnl(1) );
}

HB_FUNC( RR_GETDEVICECAPS )
{
   TEXTMETRIC  tm;
   UINT        i;
   HFONT       xfont = ( HFONT ) hb_parnl( 2 );
   if( xfont != 0 )
   {
      SelectObject( hDCRef, xfont );
   }

   GetTextMetrics( hDCRef, &tm );
   devcaps[1] = GetDeviceCaps( hDCRef, VERTSIZE );
   devcaps[2] = GetDeviceCaps( hDCRef, HORZSIZE );
   devcaps[3] = GetDeviceCaps( hDCRef, VERTRES );
   devcaps[4] = GetDeviceCaps( hDCRef, HORZRES );
   devcaps[5] = GetDeviceCaps( hDCRef, LOGPIXELSY );
   devcaps[6] = GetDeviceCaps( hDCRef, LOGPIXELSX );
   devcaps[7] = GetDeviceCaps( hDCRef, PHYSICALHEIGHT );
   devcaps[8] = GetDeviceCaps( hDCRef, PHYSICALWIDTH );
   devcaps[9] = GetDeviceCaps( hDCRef, PHYSICALOFFSETY );
   devcaps[10] = GetDeviceCaps( hDCRef, PHYSICALOFFSETX );

   devcaps[11] = tm.tmHeight;
   devcaps[12] = tm.tmAveCharWidth;
   devcaps[13] = ( int ) ( (devcaps[3] - tm.tmAscent) / tm.tmHeight );
   devcaps[14] = ( int ) ( devcaps[4] / tm.tmAveCharWidth );
   devcaps[15] = pi2->pDevMode->dmOrientation;
   devcaps[16] = ( int ) tm.tmAscent;
   devcaps[17] = ( int ) pi2->pDevMode->dmPaperSize;
   for( i = 1; i <= hb_parinfa(1, 0); i++ )
   {
      hb_storni( devcaps[i], 1, i );
   }

   if( xfont != 0 )
   {
      SelectObject( hDCRef, hfont );
   }
}

HB_FUNC( RR_SETDEVMODE )
{
   DWORD what = hb_parnl( 1 );
   if( what == (pi2->pDevMode->dmFields & what) )
   {
      if( what == DM_ORIENTATION )
      {
         pi2->pDevMode->dmOrientation = ( short ) hb_parni( 2 );
      }

      if( what == DM_PAPERSIZE )
      {
         pi2->pDevMode->dmPaperSize = ( short ) hb_parni( 2 );
      }

      if( what == DM_SCALE )
      {
         pi2->pDevMode->dmScale = ( short ) hb_parni( 2 );
      }

      if( what == DM_COPIES )
      {
         pi2->pDevMode->dmCopies = ( short ) hb_parni( 2 );
      }

      if( what == DM_DEFAULTSOURCE )
      {
         pi2->pDevMode->dmDefaultSource = ( short ) hb_parni( 2 );
      }

      if( what == DM_PRINTQUALITY )
      {
         pi2->pDevMode->dmPrintQuality = ( short ) hb_parni( 2 );
      }

      if( what == DM_COLOR )
      {
         pi2->pDevMode->dmColor = ( short ) hb_parni( 2 );
      }

      if( what == DM_DUPLEX )
      {
         pi2->pDevMode->dmDuplex = ( short ) hb_parni( 2 );
      }
   }

   DocumentProperties( NULL, hPrinter, PrinterName, pi2->pDevMode, pi2->pDevMode, DM_IN_BUFFER | DM_OUT_BUFFER );
   SetPrinter( hPrinter, 2, (LPBYTE) pi2, 0 );
   ResetDC( hDCRef, pi2->pDevMode );
   hb_retnl( (LONG) hDCRef );
}

HB_FUNC( RR_SETUSERMODE )
{
   DWORD what = hb_parnl( 1 );
   if( what == (pi2->pDevMode->dmFields & what) )
   {
      pi2->pDevMode->dmFields = pi2->pDevMode->dmFields | DM_PAPERSIZE | DM_PAPERWIDTH | DM_PAPERLENGTH;
      pi2->pDevMode->dmPaperSize   = DMPAPER_USER;
      pi2->pDevMode->dmPaperWidth  = ( short ) hb_parnl( 2 );
      pi2->pDevMode->dmPaperLength = ( short ) hb_parnl( 3 );
   }

   DocumentProperties( NULL, hPrinter, PrinterName, pi2->pDevMode, pi2->pDevMode, DM_IN_BUFFER | DM_OUT_BUFFER );
   SetPrinter( hPrinter, 2, (LPBYTE) pi2, 0 );
   ResetDC( hDCRef, pi2->pDevMode );
   hb_retnl( (LONG) hDCRef );
}

HB_FUNC( RR_GETDEFAULTPRINTER )
{
   DWORD             Needed, Returned;
   DWORD             BuffSize = 256;
   LPPRINTER_INFO_5  PrinterInfo;

   if( osvi.dwPlatformId == VER_PLATFORM_WIN32_WINDOWS )
   {
      EnumPrinters( PRINTER_ENUM_DEFAULT, NULL, 5, NULL, 0, &Needed, &Returned );
      PrinterInfo = ( LPPRINTER_INFO_5 ) LocalAlloc( LPTR, Needed );
      EnumPrinters( PRINTER_ENUM_DEFAULT, NULL, 5, (LPBYTE) PrinterInfo, Needed, &Needed, &Returned );
      strcpy( PrinterDefault, PrinterInfo->pPrinterName );
      LocalFree( PrinterInfo );
   }
   else if( osvi.dwPlatformId == VER_PLATFORM_WIN32_NT )
   {
      if( osvi.dwMajorVersion >= 5 )   /* Windows 2000 or later */
      {
         GetDefaultPrinter( PrinterDefault, &BuffSize );
      }
      else  /* Windows NT 4.0 or earlier */
      {
         GetProfileString( "windows", "device", "", PrinterDefault, BuffSize );
         strtok( PrinterDefault, "," );
      }
   }

   hb_retc( PrinterDefault );
   return;
}

HB_FUNC( RR_GETPRINTERS )
{
   DWORD          dwSize = 0;
   DWORD          dwPrinters = 0;
   DWORD          i;
   LPBYTE         pBuffer;
   LPBYTE         cBuffer;
   PRINTER_INFO_4 *pInfo4;
   PRINTER_INFO_5 *pInfo5;
   DWORD          level;
   DWORD          flags;

   osvi.dwOSVersionInfoSize = sizeof( OSVERSIONINFO );
   GetVersionEx( &osvi );
   if( osvi.dwPlatformId == VER_PLATFORM_WIN32_NT )
   {
      level = 4;
      flags = PRINTER_ENUM_CONNECTIONS | PRINTER_ENUM_LOCAL;
   }
   else
   {
      level = 5;
      flags = PRINTER_ENUM_LOCAL;
   }

   EnumPrinters( flags, NULL, level, NULL, 0, &dwSize, &dwPrinters );

   pBuffer = (char *) GlobalAlloc( GPTR, dwSize );
   if( pBuffer == NULL )
   {
      hb_retc( ",," );
      return;
   }

   EnumPrinters( flags, NULL, level, pBuffer, dwSize, &dwSize, &dwPrinters );

   if( dwPrinters == 0 )
   {
      hb_retc( ",," );
      return;
   }

   cBuffer = (char *) GlobalAlloc( GPTR, dwPrinters * 256 );

   if( osvi.dwPlatformId == VER_PLATFORM_WIN32_NT )
   {
      pInfo4 = ( PRINTER_INFO_4 * ) pBuffer;
   }
   else
   {
      pInfo5 = ( PRINTER_INFO_5 * ) pBuffer;
   }

   for( i = 0; i < dwPrinters; i++ )
   {
      if( osvi.dwPlatformId == VER_PLATFORM_WIN32_NT )
      {
         strcat( cBuffer, pInfo4->pPrinterName );
         strcat( cBuffer, "," );
         if( pInfo4->Attributes == PRINTER_ATTRIBUTE_LOCAL )
         {
            strcat( cBuffer, "local printer" );
         }
         else
         {
            strcat( cBuffer, "network printer" );
         }

         pInfo4++;
      }
      else
      {
         strcat( cBuffer, pInfo5->pPrinterName );
         strcat( cBuffer, "," );
         strcat( cBuffer, pInfo5->pPortName );
         pInfo5++;
      }

      if( i < dwPrinters - 1 )
      {
         strcat( cBuffer, ",," );
      }
   }

   hb_retc( cBuffer );
   GlobalFree( pBuffer );
   GlobalFree( cBuffer );
   return;
}

HB_FUNC( RR_STARTDOC )
{
   memset( &di, 0, sizeof(di) );
   di.cbSize = sizeof( di );
   di.lpszDocName = hb_parc( 1 );
   StartDoc( hDC, &di );
}

HB_FUNC( RR_STARTPAGE )
{
   StartPage( hDC );
   SetTextAlign( hDC, TA_BASELINE );
}

HB_FUNC( RR_ENDPAGE )
{
   EndPage( hDC );
}

HB_FUNC( RR_ENDDOC )
{
   EndDoc( hDC );
}

HB_FUNC( RR_ABORTDOC )
{
   AbortDoc( hDC );
   DeleteDC( hDC );
}

HB_FUNC( RR_DEVICECAPABILITIES )
{
   char *cBuffer, *pBuffer, *nBuffer, *sBuffer, *bnBuffer, *bwBuffer, *bcBuffer;
   DWORD  numpapers,numbins,i;
   LPPOINT lp;
   char buffer [sizeof(long)*8+1];

   numbins=DeviceCapabilities(pi2->pPrinterName,pi2->pPortName,DC_BINNAMES,NULL,NULL);
   numpapers=DeviceCapabilities(pi2->pPrinterName,pi2->pPortName,DC_PAPERNAMES,NULL,NULL);
   if( numpapers > 0 )
   {
      pBuffer = (char *) GlobalAlloc( GPTR, numpapers * 64 );
      nBuffer = (char *) GlobalAlloc( GPTR, numpapers * sizeof( WORD ) );
      sBuffer = (char *) GlobalAlloc( GPTR, numpapers * sizeof( POINT ) );
      cBuffer = (char *) GlobalAlloc( GPTR, numpapers * 128 );
      DeviceCapabilities( pi2->pPrinterName, pi2->pPortName, DC_PAPERNAMES, pBuffer, pi2->pDevMode );
      DeviceCapabilities( pi2->pPrinterName, pi2->pPortName, DC_PAPERS, nBuffer, pi2->pDevMode );
      DeviceCapabilities( pi2->pPrinterName, pi2->pPortName, DC_PAPERSIZE, sBuffer, pi2->pDevMode );
      cBuffer[ 0 ] = 0;
      for( i = 0; i < numpapers; i++ )
      {
         strcat( cBuffer, pBuffer );
         strcat( cBuffer, "," );
         strcat( cBuffer, itoa( *nBuffer, buffer, 10 ) );
         strcat( cBuffer, "," );

         lp = ( LPPOINT ) sBuffer;
         strcat( cBuffer, ltoa( lp->x, buffer, 10 ) );
         strcat( cBuffer, "," );
         strcat( cBuffer, ltoa( lp->y, buffer, 10 ) );
         if( i < numpapers - 1 )
         {
            strcat(cBuffer,",,");
         }
         pBuffer += 64;
         nBuffer += sizeof( WORD );
         sBuffer += sizeof( POINT );
      }

      hb_storc( cBuffer, 1 );

      GlobalFree( cBuffer );
      GlobalFree( pBuffer );
      GlobalFree( nBuffer );
      GlobalFree( sBuffer );
   }
   else
   {
      hb_storc( "", 1 );
   }

   if( numbins > 0 )
   {
      bnBuffer = (char *) GlobalAlloc(GPTR,numbins*24);
      bwBuffer = (char *) GlobalAlloc(GPTR,numbins*sizeof(WORD));
      bcBuffer = (char *) GlobalAlloc(GPTR,numbins*64);
      DeviceCapabilities( pi2->pPrinterName, pi2->pPortName, DC_BINNAMES, bnBuffer, pi2->pDevMode );
      DeviceCapabilities( pi2->pPrinterName, pi2->pPortName, DC_BINS, bwBuffer, pi2->pDevMode );
      bcBuffer[ 0 ] = 0;
      for( i = 0; i < numbins; i++ )
      {
         strcat( bcBuffer, bnBuffer );
         strcat( bcBuffer, "," );
         strcat( bcBuffer, itoa( *bwBuffer, buffer, 10 ) );

         if( i < numbins - 1 )
         {
            strcat( bcBuffer, ",," );
         }
         bnBuffer += 24;
         bwBuffer += sizeof( WORD );
      }

      hb_storc( bcBuffer, 2 );

      GlobalFree( bnBuffer );
      GlobalFree( bwBuffer );
      GlobalFree( bcBuffer );
   }
   else
   {
      hb_storc( "", 2 );
   }
}

HB_FUNC( RR_SETPOLYFILLMODE )
{
   if( SetPolyFillMode(hDC, (COLORREF) hb_parnl(1)) != 0 )
   {
      hb_retnl( hb_parnl(1) );
   }
   else
   {
      hb_retnl( (LONG) GetPolyFillMode(hDC) );
   }
}

HB_FUNC( RR_SETTEXTCOLOR )
{
   if( SetTextColor(hDC, (COLORREF) hb_parnl(1)) != CLR_INVALID )
   {
      hb_retnl( hb_parnl(1) );
   }
   else
   {
      hb_retnl( (LONG) GetTextColor(hDC) );
   }
}

HB_FUNC( RR_SETBKCOLOR )
{
   if( SetBkColor(hDC, (COLORREF) hb_parnl(1)) != CLR_INVALID )
   {
      hb_retnl( hb_parnl(1) );
   }
   else
   {
      hb_retnl( (LONG) GetBkColor(hDC) );
   }
}

HB_FUNC( RR_SETBKMODE )
{
   if( hb_parni(1) == 1 )
   {
      SetBkMode( hDC, TRANSPARENT );
   }
   else
   {
      SetBkMode( hDC, OPAQUE );
   }
}

HB_FUNC( RR_DELETEOBJECTS )
{
   UINT  i;
   for( i = 2; i <= hb_parinfa(1, 0); i++ )
   {
      DeleteObject( (HGDIOBJ) hb_parnl(1, i) );
   }
}

HB_FUNC( RR_DELETEIMAGELISTS )
{
   UINT  i;
   for( i = 1; i <= hb_parinfa(1, 0); i++ )
   {
      ImageList_Destroy( (HIMAGELIST) hb_parnl(1, i, 1) );
   }
}

HB_FUNC( RR_DELETEMFILES )
{
   UINT  i;
   for( i = 1; i <= hb_parinfa(1, 0); i++ )
   {
      DeleteEnhMetaFile( (HENHMETAFILE) hb_parnl(1, i, 1) );
   }

   if( himgbmp != NULL )
   {
      DeleteObject( himgbmp );
   }

   for( i = 1; i <= 15; i++ )
   {
      if( hbmp[i] != NULL )
      {
         DeleteObject( hbmp[i] );
      }
   }
}

HB_FUNC( RR_SAVEMETAFILE )
{
   CopyEnhMetaFile( (HENHMETAFILE) hb_parnl(1), hb_parc(2) );
}

HB_FUNC( RR_GETCURRENTOBJECT )
{
   int      what = hb_parni( 1 );
   HGDIOBJ  hand;
   if( what == 1 )
   {
      hand = GetCurrentObject( hDC, OBJ_FONT );
   }
   else if( what == 2 )
   {
      hand = GetCurrentObject( hDC, OBJ_BRUSH );
   }
   else
   {
      hand = GetCurrentObject( hDC, OBJ_PEN );
   }

   hb_retnl( (LONG) hand );
}

HB_FUNC( RR_GETSTOCKOBJECT )
{
   hb_retnl( (LONG) GetStockObject(hb_parni(1)) );
}

HB_FUNC( RR_CREATEPEN )
{
   hb_retnl( (LONG) CreatePen(hb_parni(1), hb_parni(2), (COLORREF) hb_parnl(3)) );
}

HB_FUNC( RR_MODIFYPEN )
{
   LOGPEN   ppn;
   int      i;
   HPEN     hp;
   memset( &ppn, 0, sizeof(LOGPEN) );
   i = GetObject( (HPEN) hb_parnl(1), sizeof(LOGPEN), &ppn );
   if( i > 0 )
   {
      if( hb_parni(2) >= 0 )
      {
         ppn.lopnStyle = ( UINT ) hb_parni( 2 );
      }

      if( hb_parnl(3) >= 0 )
      {
         ppn.lopnWidth.x = hb_parnl( 3 );
      }

      if( hb_parnl(4) >= 0 )
      {
         ppn.lopnColor = ( COLORREF ) hb_parnl( 4 );
      }

      hp = CreatePenIndirect( &ppn );
      if( hp != NULL )
      {
         DeleteObject( (HPEN) hb_parnl(1) );
         hb_retnl( (LONG) hp );
      }
      else
      {
         hb_retnl( (LONG) hb_parnl(1) );
      }
   }
   else
   {
      hb_retnl( (LONG) hb_parnl(1) );
   }
}

HB_FUNC( RR_SELECTPEN )
{
   SelectObject( hDC, (HPEN) hb_parnl(1) );
   hpen = ( HPEN ) hb_parnl( 1 );
}

HB_FUNC( RR_CREATEBRUSH )
{
   LOGBRUSH pbr;
   pbr.lbStyle = hb_parni( 1 );
   pbr.lbColor = ( COLORREF ) hb_parnl( 2 );
   pbr.lbHatch = ( LONG ) hb_parnl( 3 );
   hb_retnl( (LONG) CreateBrushIndirect(&pbr) );
}

HB_FUNC( RR_MODIFYBRUSH )
{
   LOGBRUSH ppn;
   int      i;
   HBRUSH   hb;
   memset( &ppn, 0, sizeof(LOGBRUSH) );
   i = GetObject( (HBRUSH) hb_parnl(1), sizeof(LOGBRUSH), &ppn );
   if( i > 0 )
   {
      if( hb_parni(2) >= 0 )
      {
         ppn.lbStyle = ( UINT ) hb_parni( 2 );
      }

      if( hb_parnl(3) >= 0 )
      {
         ppn.lbColor = ( COLORREF ) hb_parnl( 3 );
      }

      if( hb_parnl(4) >= 0 )
      {
         ppn.lbHatch = hb_parnl( 4 );
      }

      hb = CreateBrushIndirect( &ppn );
      if( hb != NULL )
      {
         DeleteObject( (HBRUSH) hb_parnl(1) );
         hb_retnl( (LONG) hb );
      }
      else
      {
         hb_retnl( (LONG) hb_parnl(1) );
      }
   }
   else
   {
      hb_retnl( (LONG) hb_parnl(1) );
   }
}

HB_FUNC( RR_SELECTBRUSH )
{
   SelectObject( hDC, (HBRUSH) hb_parnl(1) );
   hbrush = ( HBRUSH ) hb_parnl( 1 );
}

HB_FUNC( RR_CREATEFONT )
{
   char        *FontName = hb_parc( 1 );
   int         FontSize = hb_parni( 2 );
   LONG        FontWidth = hb_parnl( 3 );
   LONG        Orient = hb_parnl( 4 );
   LONG        Weight = hb_parnl( 5 );
   int         Italic = hb_parni( 6 );
   int         Underline = hb_parni( 7 );
   int         Strikeout = hb_parni( 8 );
   HFONT       oldfont, hxfont;
   LONG        newWidth, FontHeight;
   TEXTMETRIC  tm;
   BYTE        bItalic, bUnderline, bStrikeOut;

   newWidth = ( LONG ) FontWidth;
   if( FontSize <= 0 )
   {
      FontSize = 10;
   }

   if( FontWidth < 0 )
   {
      newWidth = 0;
   }

   if( Orient <= 0 )
   {
      Orient = 0;
   }

   if( Weight <= 0 )
   {
      Weight = FW_NORMAL;
   }
   else
   {
      Weight = FW_BOLD;
   }

   if( Italic <= 0 )
   {
      bItalic = 0;
   }
   else
   {
      bItalic = 1;
   }

   if( Underline <= 0 )
   {
      bUnderline = 0;
   }
   else
   {
      bUnderline = 1;
   }

   if( Strikeout <= 0 )
   {
      bStrikeOut = 0;
   }
   else
   {
      bStrikeOut = 1;
   }

   FontHeight = -MulDiv( FontSize, GetDeviceCaps(hDCRef, LOGPIXELSY), 72 );
   hxfont = CreateFont
      (
         FontHeight,
         newWidth,
         Orient,
         Orient,
         Weight,
         bItalic,
         bUnderline,
         bStrikeOut,
         charset,
         OUT_TT_PRECIS,
         CLIP_DEFAULT_PRECIS,
         DEFAULT_QUALITY,
         FF_DONTCARE,
         FontName
      );
   if( FontWidth < 0 )
   {
      oldfont = ( HFONT ) SelectObject( hDC, hxfont );
      GetTextMetrics( hDC, &tm );
      SelectObject( hDC, oldfont );
      DeleteObject( hxfont );
      newWidth = ( int ) ( ( float ) -(tm.tmAveCharWidth + tm.tmOverhang) * FontWidth / 100 );
      hxfont = CreateFont
         (
            FontHeight,
            newWidth,
            Orient,
            Orient,
            Weight,
            bItalic,
            bUnderline,
            bStrikeOut,
            charset,
            OUT_TT_PRECIS,
            CLIP_DEFAULT_PRECIS,
            DEFAULT_QUALITY,
            FF_DONTCARE,
            FontName
         );
   }

   hb_retnl( (LONG) hxfont );
}

HB_FUNC( RR_MODIFYFONT )
{
   LOGFONT  ppn;
   int      i;
   HFONT    hf;
   LONG     nHeight;
   memset( &ppn, 0, sizeof(LOGFONT) );
   i = GetObject( (HFONT) hb_parnl(1), sizeof(LOGFONT), &ppn );
   if( i > 0 )
   {
      //     if (hb_parc(2)!="")
      //       ppn.lfFaceName = hb_parc(2);

      if( hb_parni(3) > 0 )
      {
         nHeight = -MulDiv( hb_parni(3), GetDeviceCaps(hDC, LOGPIXELSY), 72 );
         ppn.lfHeight = nHeight;
      }

      if( hb_parnl(4) >= 0 )
      {
         ppn.lfWidth = ( LONG ) hb_parnl( 4 ) * ppn.lfWidth / 100;
      }

      if( hb_parnl(5) >= 0 )
      {
         ppn.lfOrientation = hb_parnl( 5 );
         ppn.lfEscapement = hb_parnl( 5 );
      }

      if( hb_parnl(6) >= 0 )
      {
         if( hb_parnl(6) == 0 )
         {
            ppn.lfWeight = FW_NORMAL;
         }
         else
         {
            ppn.lfWeight = FW_BOLD;
         }
      }

      if( hb_parni(7) >= 0 )
      {
         ppn.lfItalic = ( BYTE ) hb_parni( 7 );
      }

      if( hb_parni(8) >= 0 )
      {
         ppn.lfUnderline = ( BYTE ) hb_parni( 8 );
      }

      if( hb_parni(9) >= 0 )
      {
         ppn.lfStrikeOut = ( BYTE ) hb_parni( 9 );
      }

      hf = CreateFontIndirect( &ppn );
      if( hf != NULL )
      {
         DeleteObject( (HFONT) hb_parnl(1) );
         hb_retnl( (LONG) hf );
      }
      else
      {
         hb_retnl( (LONG) hb_parnl(1) );
      }
   }
   else
   {
      hb_retnl( (LONG) hb_parnl(1) );
   }
}

HB_FUNC( RR_SELECTFONT )
{
   SelectObject( hDC, (HFONT) hb_parnl(1) );
   hfont = ( HFONT ) hb_parnl( 1 );
}

HB_FUNC( RR_SETCHARSET )
{
   charset = ( DWORD ) hb_parnl( 1 );
}

HB_FUNC( RR_TEXTOUT )
{
   LONG  xfont = hb_parnl( 3 );
   HFONT prevfont;
   SIZE  szMetric;
   int   lspace = hb_parni( 4 );

   if( xfont != 0 )
   {
      prevfont = ( HFONT ) SelectObject( hDC, (HFONT) xfont );
   }

   if( textjust > 0 )
   {
      GetTextExtentPoint32( hDC, hb_parc(1), hb_parclen(1), &szMetric );
      if( szMetric.cx < textjust )
      {     // or can be for better look (szMetric.cx>(int) textjust*2/3)
         if( lspace > 0 )
         {
            SetTextJustification( hDC, ( int ) textjust - szMetric.cx, lspace );
         }
      }
   }

   hb_retl( TextOut(hDC, hb_parni(2, 2), hb_parni(2, 1) + devcaps[16], hb_parc(1), hb_parclen(1)) );
   if( xfont != 0 )
   {
      SelectObject( hDC, prevfont );
   }

   if( textjust > 0 )
   {
      SetTextJustification( hDC, 0, 0 );
   }
}

HB_FUNC( RR_DRAWTEXT )
{
   LONG  xfont = hb_parnl( 5 );
   HFONT prevfont;
   RECT  rect;
   SetRect( &rect, hb_parnl(1, 2), hb_parnl(1, 1), hb_parnl(2, 2), hb_parnl(2, 1) );
   if( xfont != 0 )
   {
      prevfont = ( HFONT ) SelectObject( hDC, (HFONT) xfont );
   }

   hb_retni( DrawText(hDC, hb_parc(3), -1, &rect, hb_parni(4)) );
   if( xfont != 0 )
   {
      SelectObject( hDC, prevfont );
   }
}

HB_FUNC( RR_RECTANGLE )
{
   LONG  xpen = hb_parnl( 3 );
   LONG  xbrush = hb_parnl( 4 );
   if( xpen != 0 )
   {
      SelectObject( hDC, (HPEN) xpen );
   }

   if( xbrush != 0 )
   {
      SelectObject( hDC, (HBRUSH) xbrush );
   }

   hb_retni( Rectangle(hDC, hb_parnl(1, 2), hb_parnl(1, 1), hb_parnl(2, 2), hb_parnl(2, 1)) );
   if( xpen != 0 )
   {
      SelectObject( hDC, hpen );
   }

   if( xbrush != 0 )
   {
      SelectObject( hDC, hbrush );
   }
}

HB_FUNC( RR_CLOSEMFILE )
{
   UINT              size;
   HENHMETAFILE      hh;
   LPBYTE            eBuffer;
   LPENHMETAHEADER   eHeader;
   hh = CloseEnhMetaFile( hDC );
   size = GetEnhMetaFileBits( hh, NULL, NULL );
   eBuffer = (char *) GlobalAlloc( GPTR, (DWORD) size );
   GetEnhMetaFileBits( hh, size, eBuffer );
   eHeader = ( LPENHMETAHEADER ) eBuffer;
   eHeader->szlDevice.cx = devcaps[4];
   eHeader->szlDevice.cy = devcaps[3];
   eHeader->szlMillimeters.cx = devcaps[2];
   eHeader->szlMillimeters.cy = devcaps[1];

   hb_retclen( eBuffer, (ULONG) size );
   DeleteEnhMetaFile( hh );
   GlobalFree( eBuffer );
}

HB_FUNC( RR_CREATEMFILE )
{
   RECT  emfrect;
   SetRect( &emfrect, 0, 0, GetDeviceCaps(hDCRef, HORZSIZE) * 100, GetDeviceCaps(hDCRef, VERTSIZE) * 100 );
   hDC = CreateEnhMetaFile( hDCRef, NULL, &emfrect, "hbprinter\0emf file\0\0" );
   SetTextAlign( hDC, TA_BASELINE );
   preview = 1;
   hb_retnl( (LONG) hDC );
}

HB_FUNC( RR_DELETECLIPRGN )
{
   SelectClipRgn( hDC, NULL );
}

HB_FUNC( RR_CREATERGN )
{
   POINT lpp;
   GetViewportOrgEx( hDC, &lpp );
   if( hb_parni(3) == 2 )
   {
      hb_retnl( (LONG) CreateEllipticRgn(hb_parni(1, 2) + lpp.x, hb_parni(1, 1) + lpp.y, hb_parni(2, 2) + lpp.x, hb_parni(2, 1) + lpp.y) );
   }
   else if( hb_parni(3) == 3 )
   {
      hb_retnl
      (
         (LONG) CreateRoundRectRgn
            (
               hb_parni(1, 2) + lpp.x,
               hb_parni(1, 1) + lpp.y,
               hb_parni(2, 2) + lpp.x,
               hb_parni(2, 1) + lpp.y,
               hb_parni(4, 2) + lpp.x,
               hb_parni(4, 1) + lpp.y
            )
      );
   }
   else
   {
      hb_retnl( (LONG) CreateRectRgn(hb_parni(1, 2) + lpp.x, hb_parni(1, 1) + lpp.y, hb_parni(2, 2) + lpp.x, hb_parni(2, 1) + lpp.y) );
   }
}

HB_FUNC( RR_CREATEPOLYGONRGN )
{
   int   number = hb_parinfa( 1, 0 );
   int   i;
   POINT apoints[1024];
   for( i = 0; i <= number - 1; i++ )
   {
      apoints[i].x = hb_parni( 1, i + 1 );
      apoints[i].y = hb_parni( 2, i + 1 );
   }

   hb_retnl( (LONG) CreatePolygonRgn(apoints, number, hb_parni(3)) );
}

HB_FUNC( RR_COMBINERGN )
{
   HRGN  rgnnew = CreateRectRgn( 0, 0, 1, 1 );
   CombineRgn( rgnnew, (HRGN) hb_parnl(1), (HRGN) hb_parnl(2), hb_parni(3) );
   hb_retnl( (LONG) rgnnew );
}

HB_FUNC( RR_SELECTCLIPRGN )
{
   SelectClipRgn( hDC, (HRGN) hb_parnl(1) );
   hrgn = ( HRGN ) hb_parnl( 1 );
}

HB_FUNC( RR_SETVIEWPORTORG )
{
   hb_retl( SetViewportOrgEx(hDC, hb_parni(1, 2), hb_parni(1, 1), NULL) );
}

HB_FUNC( RR_GETVIEWPORTORG )
{
   POINT lpp;
   hb_retl( GetViewportOrgEx(hDC, &lpp) );
   hb_stornl( lpp.x, 1, 2 );
   hb_stornl( lpp.y, 1, 1 );
}

HB_FUNC( RR_SETRGB )
{
   hb_retnl( RGB(hb_parni(1), hb_parni(2), hb_parni(3)) );
}

HB_FUNC( RR_SETTEXTCHAREXTRA )
{
   hb_retni( SetTextCharacterExtra(hDC, hb_parni(1)) );
}

HB_FUNC( RR_GETTEXTCHAREXTRA )
{
   hb_retni( GetTextCharacterExtra(hDC) );
}

HB_FUNC( RR_SETTEXTJUSTIFICATION )
{
   textjust = hb_parni( 1 );
}

HB_FUNC( RR_GETTEXTJUSTIFICATION )
{
   hb_retni( textjust );
}

HB_FUNC( RR_GETTEXTALIGN )
{
   hb_retni( GetTextAlign(hDC) );
}

HB_FUNC( RR_SETTEXTALIGN )
{
   hb_retni( SetTextAlign(hDC, TA_BASELINE | hb_parni(1)) );
}

HB_FUNC( RR_PICTURE )
{
   IStream  *iStream;
   IPicture *iPicture;
   HGLOBAL  hGlobal;
   void     *pGlobal;
   HANDLE   hFile;
   DWORD    nFileSize;
   DWORD    nReadByte;
   long     lWidth, lHeight;
   int      x, y, xe, ye;
   int      r = hb_parni( 2, 1 );
   int      c = hb_parni( 2, 2 );
   int      dr = hb_parni( 3, 1 );
   int      dc = hb_parni( 3, 2 );
   int      tor = hb_parni( 4, 1 );
   int      toc = hb_parni( 4, 2 );
   HRGN     hrgn1;
   POINT    lpp;

   hFile = CreateFile( hb_parc(1), GENERIC_READ, 0, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL );
   if( hFile == INVALID_HANDLE_VALUE )
   {
      return;
   }

   nFileSize = GetFileSize( hFile, NULL );
   hGlobal = GlobalAlloc( GMEM_MOVEABLE, nFileSize );
   pGlobal = GlobalLock( hGlobal );
   ReadFile( hFile, pGlobal, nFileSize, &nReadByte, NULL );
   CloseHandle( hFile );
   GlobalUnlock( hGlobal );
   CreateStreamOnHGlobal( hGlobal, TRUE, &iStream );
   OleLoadPicture( iStream, nFileSize, TRUE, &IID_IPicture, ( LPVOID * ) &iPicture );
   GlobalFree( hGlobal );
   iStream->lpVtbl->Release( iStream );
   if( iPicture == NULL )
   {
      return;
   }

   iPicture->lpVtbl->get_Width( iPicture, &lWidth );
   iPicture->lpVtbl->get_Height( iPicture, &lHeight );
   if( dc == 0 )
   {
      dc = ( int ) ( ( float ) dr * lWidth / lHeight );
   }

   if( dr == 0 )
   {
      dr = ( int ) ( ( float ) dc * lHeight / lWidth );
   }

   if( tor <= 0 )
   {
      tor = dr;
   }

   if( toc <= 0 )
   {
      toc = dc;
   }

   x = c;
   y = r;
   xe = c + toc - 1;
   ye = r + tor - 1;
   GetViewportOrgEx( hDC, &lpp );
   hrgn1 = CreateRectRgn( c + lpp.x, r + lpp.y, xe + lpp.x, ye + lpp.y );
   if( hrgn == NULL )
   {
      SelectClipRgn( hDC, hrgn1 );
   }
   else
   {
      ExtSelectClipRgn( hDC, hrgn1, RGN_AND );
   }

   while( x < xe )
   {
      while( y < ye )
      {
         iPicture->lpVtbl->Render( iPicture, hDC, x, y, dc, dr, 0, lHeight, lWidth, -lHeight, NULL );
         y += dr;
      }

      y = r;
      x += dc;
   }

   iPicture->lpVtbl->Release( iPicture );
   SelectClipRgn( hDC, hrgn );
   DeleteObject( hrgn1 );
   hb_retni( 0 );
}

LPVOID rr_loadpicturefromresource( char *resname, LONG *lwidth, LONG *lheight )
{
   HBITMAP     hbmpx;
   IPicture    *iPicture = NULL;
   IStream     *iStream = NULL;
   PICTDESC    picd;
   HGLOBAL     hGlobalres;
   HGLOBAL     hGlobal;
   HRSRC       hSource;
   LPVOID      lpVoid;
   HINSTANCE   hinstance = GetModuleHandle( NULL );
   int         nSize;
   hbmpx = (HBITMAP) LoadImage( GetModuleHandle(NULL), resname, IMAGE_BITMAP, 0, 0, LR_CREATEDIBSECTION );
   if( hbmpx != NULL )
   {
      picd.cbSizeofstruct = sizeof( PICTDESC );
      picd.picType = PICTYPE_BITMAP;
      picd.bmp.hbitmap = hbmpx;
      OleCreatePictureIndirect( &picd, &IID_IPicture, TRUE, ( LPVOID * ) &iPicture );
   }
   else
   {
      hSource = FindResource( hinstance, resname, "HMGPICTURE" );
      if( hSource == NULL )
      {
         return NULL;
      }

      hGlobalres = LoadResource( hinstance, hSource );
      if( hGlobalres == NULL )
      {
         return NULL;
      }

      lpVoid = LockResource( hGlobalres );
      if( lpVoid == NULL )
      {
         return NULL;
      }

      nSize = SizeofResource( hinstance, hSource );
      hGlobal = GlobalAlloc( GPTR, nSize );
      if( hGlobal == NULL )
      {
         return NULL;
      }

      memcpy( hGlobal, lpVoid, nSize );
      FreeResource( hGlobalres );
      CreateStreamOnHGlobal( hGlobal, TRUE, &iStream );
      if( iStream == NULL )
      {
         GlobalFree( hGlobal );
         return NULL;
      }

      OleLoadPicture( iStream, nSize, TRUE, &IID_IPicture, ( LPVOID * ) &iPicture );
      iStream->lpVtbl->Release( iStream );
      GlobalFree( hGlobal );
   }

   if( iPicture != NULL )
   {
      iPicture->lpVtbl->get_Width( iPicture, lwidth );
      iPicture->lpVtbl->get_Height( iPicture, lheight );
   }

   return iPicture;
}

LPVOID rr_loadpicture( char *filename, LONG *lwidth, LONG *lheight )
{
   IStream  *iStream = NULL;
   IPicture *iPicture = NULL;
   HGLOBAL  hGlobal;
   void     *pGlobal;
   HANDLE   hFile;
   DWORD    nFileSize, nReadByte;

   hFile = CreateFile( filename, GENERIC_READ, 0, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL );
   if( hFile == INVALID_HANDLE_VALUE )
   {
      return NULL;
   }

   nFileSize = GetFileSize( hFile, NULL );
   hGlobal = GlobalAlloc( GMEM_MOVEABLE, nFileSize + 4096 );
   pGlobal = GlobalLock( hGlobal );
   ReadFile( hFile, pGlobal, nFileSize, &nReadByte, NULL );
   CloseHandle( hFile );
   CreateStreamOnHGlobal( hGlobal, TRUE, &iStream );
   if( iStream == NULL )
   {
      GlobalUnlock( hGlobal );
      GlobalFree( hGlobal );
      return NULL;
   }

   OleLoadPicture( iStream, nFileSize, TRUE, &IID_IPicture, ( LPVOID * ) &iPicture );
   GlobalUnlock( hGlobal );
   GlobalFree( hGlobal );
   iStream->lpVtbl->Release( iStream );
   iStream = NULL;
   if( iPicture != NULL )
   {
      iPicture->lpVtbl->get_Width( iPicture, lwidth );
      iPicture->lpVtbl->get_Height( iPicture, lheight );
   }

   return iPicture;
}

HB_FUNC( RR_DRAWPICTURE )
{
   IPicture *ipic;
   int      x, y, xe, ye;
   int      r = hb_parni( 2, 1 );
   int      c = hb_parni( 2, 2 );
   int      dr = hb_parni( 3, 1 );
   int      dc = hb_parni( 3, 2 );
   int      tor = hb_parni( 4, 1 );
   int      toc = hb_parni( 4, 2 );
   long     lwidth = 0;
   long     lheight = 0;
   RECT     lrect;
   HRGN     hrgn1;
   POINT    lpp;
   int      lw, lh;

   ipic = rr_loadpicture( hb_parc(1), &lwidth, &lheight );
   if( ipic == NULL )
   {
      ipic = rr_loadpicturefromresource( hb_parc(1), &lwidth, &lheight );
   }

   if( ipic == NULL )
   {
      return;
   }

   lw = MulDiv( lwidth, devcaps[6], 2540 );
   lh = MulDiv( lheight, devcaps[5], 2540 );
   if( dc == 0 )
   {
      dc = ( int ) ( ( float ) dr * lw / lh );
   }

   if( dr == 0 )
   {
      dr = ( int ) ( ( float ) dc * lh / lw );
   }

   if( tor <= 0 )
   {
      tor = dr;
   }

   if( toc <= 0 )
   {
      toc = dc;
   }

   x = c;
   y = r;
   xe = c + toc - 1;
   ye = r + tor - 1;
   GetViewportOrgEx( hDC, &lpp );
   hrgn1 = CreateRectRgn( c + lpp.x, r + lpp.y, xe + lpp.x, ye + lpp.y );
   if( hrgn == NULL )
   {
      SelectClipRgn( hDC, hrgn1 );
   }
   else
   {
      ExtSelectClipRgn( hDC, hrgn1, RGN_AND );
   }

   while( x < xe )
   {
      while( y < ye )
      {
         SetRect( &lrect, x, y, dc + x, dr + y );
         ipic->lpVtbl->Render( ipic, hDC, x, y, dc, dr, 0, lheight, lwidth, -lheight, &lrect );
         y += dr;
      }

      y = r;
      x += dc;
   }

   ipic->lpVtbl->Release( ipic );
   SelectClipRgn( hDC, hrgn );
   DeleteObject( hrgn1 );
   hb_retni( 0 );
}

HB_FUNC( RR_CREATEIMAGELIST )
{
   HBITMAP     hbmpx;
   BITMAP      bm;
   HIMAGELIST  himl;
   int         dx, number;

   hbmpx = ( HBITMAP ) LoadImage( 0, hb_parc(1), IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE | LR_CREATEDIBSECTION );
   if( hbmpx == NULL )
   {
      hbmpx = ( HBITMAP ) LoadImage( GetModuleHandle(NULL), hb_parc(1), IMAGE_BITMAP, 0, 0, LR_CREATEDIBSECTION );
   }

   if( hbmpx == NULL )
   {
      return;
   }

   GetObject( hbmpx, sizeof(BITMAP), &bm );
   number = hb_parni( 2 );
   if( number == NULL )
   {
      number = ( int ) bm.bmWidth / bm.bmHeight;
      dx = bm.bmHeight;
   }
   else
   {
      dx = ( int ) bm.bmWidth / number;
   }

   himl = ImageList_Create( dx, bm.bmHeight, ILC_COLOR24 | ILC_MASK, number, 0 );
   ImageList_AddMasked( himl, hbmpx, CLR_DEFAULT );
   hb_storni( dx, 3 );
   hb_storni( bm.bmHeight, 4 );
   DeleteObject( hbmpx );
   hb_retnl( (LONG) himl );
}

HB_FUNC( RR_DRAWIMAGELIST )
{
   HIMAGELIST  himl = ( HIMAGELIST ) hb_parnl( 1 );
   HDC         tempdc, temp2dc;
   HBITMAP     hbmpx;
   RECT        rect;
   HWND        hwnd = GetActiveWindow();
   rect.left = hb_parni( 3, 2 );
   rect.top = hb_parni( 3, 1 );
   rect.right = hb_parni( 4, 2 );
   rect.bottom = hb_parni( 4, 1 );
   temp2dc = GetWindowDC( hwnd );
   tempdc = CreateCompatibleDC( temp2dc );
   hbmpx = CreateCompatibleBitmap( temp2dc, hb_parni(5), hb_parni(6) );
   ReleaseDC( hwnd, temp2dc );
   SelectObject( tempdc, hbmpx );
   BitBlt( tempdc, 0, 0, hb_parni(5), hb_parni(6), tempdc, 0, 0, WHITENESS );
   if( hb_parnl(8) >= 0 )
   {
      ImageList_SetBkColor( himl, (COLORREF) hb_parnl(8) );
   }

   ImageList_Draw( himl, hb_parni(2) - 1, tempdc, 0, 0, hb_parni(7) );
   if( hb_parnl(8) >= 0 )
   {
      ImageList_SetBkColor( himl, CLR_NONE );
   }

   hb_retl( StretchBlt(hDC, rect.left, rect.top, rect.right, rect.bottom, tempdc, 0, 0, hb_parni(5), hb_parni(6), SRCCOPY) );
   DeleteDC( tempdc );
   DeleteObject( hbmpx );
}

HB_FUNC( RR_POLYGON )
{
   int   number = hb_parinfa( 1, 0 );
   int   i;
   int   styl = GetPolyFillMode( hDC );
   POINT apoints[1024];
   LONG  xpen = hb_parnl( 3 );
   LONG  xbrush = hb_parnl( 4 );

   for( i = 0; i <= number - 1; i++ )
   {
      apoints[i].x = hb_parni( 1, i + 1 );
      apoints[i].y = hb_parni( 2, i + 1 );
   }

   if( xpen != 0 )
   {
      SelectObject( hDC, (HPEN) xpen );
   }

   if( xbrush != 0 )
   {
      SelectObject( hDC, (HBRUSH) xbrush );
   }

   SetPolyFillMode( hDC, hb_parni(5) );

   hb_retnl( (LONG) Polygon(hDC, apoints, number) );

   if( xpen != 0 )
   {
      SelectObject( hDC, hpen );
   }

   if( xbrush != 0 )
   {
      SelectObject( hDC, hbrush );
   }

   SetPolyFillMode( hDC, styl );
}

HB_FUNC( RR_POLYBEZIER )
{
   DWORD number = ( DWORD ) hb_parinfa( 1, 0 );
   DWORD i;
   POINT apoints[1024];
   LONG  xpen = hb_parnl( 3 );

   for( i = 0; i <= number - 1; i++ )
   {
      apoints[i].x = hb_parni( 1, i + 1 );
      apoints[i].y = hb_parni( 2, i + 1 );
   }

   if( xpen != 0 )
   {
      SelectObject( hDC, (HPEN) xpen );
   }

   hb_retnl( (LONG) PolyBezier(hDC, apoints, number) );

   if( xpen != 0 )
   {
      SelectObject( hDC, hpen );
   }
}

HB_FUNC( RR_POLYBEZIERTO )
{
   DWORD number = ( DWORD ) hb_parinfa( 1, 0 );
   DWORD i;
   POINT apoints[1024];
   LONG  xpen = hb_parnl( 3 );

   for( i = 0; i <= number - 1; i++ )
   {
      apoints[i].x = hb_parni( 1, i + 1 );
      apoints[i].y = hb_parni( 2, i + 1 );
   }

   if( xpen != 0 )
   {
      SelectObject( hDC, (HPEN) xpen );
   }

   hb_retnl( (LONG) PolyBezierTo(hDC, apoints, number) );

   if( xpen != 0 )
   {
      SelectObject( hDC, hpen );
   }
}

HB_FUNC( RR_GETTEXTEXTENT )
{
   LONG  xfont = hb_parnl( 3 );
   SIZE  szMetric;

   if( xfont != 0 )
   {
      SelectObject( hDC, (HPEN) xfont );
   }

   hb_retni( GetTextExtentPoint32(hDC, hb_parc(1), hb_parclen(1), &szMetric) );
   hb_storni( szMetric.cy, 2, 1 );
   hb_storni( szMetric.cx, 2, 2 );
   if( xfont != 0 )
   {
      SelectObject( hDC, hfont );
   }
}

HB_FUNC( RR_ROUNDRECT )
{
   LONG  xpen = hb_parnl( 3 );
   LONG  xbrush = hb_parnl( 4 );
   if( xpen != 0 )
   {
      SelectObject( hDC, (HPEN) xpen );
   }

   if( xbrush != 0 )
   {
      SelectObject( hDC, (HBRUSH) xbrush );
   }

   hb_retni( RoundRect(hDC, hb_parni(1, 2), hb_parni(1, 1), hb_parni(2, 2), hb_parni(2, 1), hb_parni(3, 2), hb_parni(3, 1)) );

   if( xbrush != 0 )
   {
      SelectObject( hDC, (HBRUSH) hbrush );
   }

   if( xpen != 0 )
   {
      SelectObject( hDC, (HPEN) hpen );
   }
}

HB_FUNC( RR_ELLIPSE )
{
   LONG  xpen = hb_parnl( 3 );
   LONG  xbrush = hb_parnl( 4 );
   if( xpen != 0 )
   {
      SelectObject( hDC, (HPEN) xpen );
   }

   if( xbrush != 0 )
   {
      SelectObject( hDC, (HBRUSH) xbrush );
   }

   hb_retni( Ellipse(hDC, hb_parni(1, 2), hb_parni(1, 1), hb_parni(2, 2), hb_parni(2, 1)) );

   if( xpen != 0 )
   {
      SelectObject( hDC, hpen );
   }

   if( xbrush != 0 )
   {
      SelectObject( hDC, hbrush );
   }
}

HB_FUNC( RR_CHORD )
{
   LONG  xpen = hb_parnl( 5 );
   LONG  xbrush = hb_parnl( 6 );
   if( xpen != 0 )
   {
      SelectObject( hDC, (HPEN) xpen );
   }

   if( xbrush != 0 )
   {
      SelectObject( hDC, (HBRUSH) xbrush );
   }

   hb_retni( Chord(hDC, hb_parni(1, 2), hb_parni(1, 1), hb_parni(2, 2), hb_parni(2, 1), hb_parni(3, 2), hb_parni(3, 1), hb_parni(4, 2), hb_parni(4, 1)) );

   if( xpen != 0 )
   {
      SelectObject( hDC, hpen );
   }

   if( xbrush != 0 )
   {
      SelectObject( hDC, hbrush );
   }
}

HB_FUNC( RR_ARCTO )
{
   LONG  xpen = hb_parnl( 5 );
   if( xpen != 0 )
   {
      SelectObject( hDC, (HPEN) xpen );
   }

   hb_retni( ArcTo(hDC, hb_parni(1, 2), hb_parni(1, 1), hb_parni(2, 2), hb_parni(2, 1), hb_parni(3, 2), hb_parni(3, 1), hb_parni(4, 2), hb_parni(4, 1)) );

   if( xpen != 0 )
   {
      SelectObject( hDC, hpen );
   }
}

HB_FUNC( RR_ARC )
{
   LONG  xpen = hb_parnl( 5 );
   if( xpen != 0 )
   {
      SelectObject( hDC, (HPEN) xpen );
   }

   hb_retni( Arc(hDC, hb_parni(1, 2), hb_parni(1, 1), hb_parni(2, 2), hb_parni(2, 1), hb_parni(3, 2), hb_parni(3, 1), hb_parni(4, 2), hb_parni(4, 1)) );
   if( xpen != 0 )
   {
      SelectObject( hDC, hpen );
   }
}

HB_FUNC( RR_PIE )
{
   LONG  xpen = hb_parnl( 5 );
   LONG  xbrush = hb_parnl( 6 );
   if( xpen != 0 )
   {
      SelectObject( hDC, (HPEN) xpen );
   }

   if( xbrush != 0 )
   {
      SelectObject( hDC, (HBRUSH) xbrush );
   }

   hb_retni( Pie(hDC, hb_parni(1, 2), hb_parni(1, 1), hb_parni(2, 2), hb_parni(2, 1), hb_parni(3, 2), hb_parni(3, 1), hb_parni(4, 2), hb_parni(4, 1)) );

   if( xpen != 0 )
   {
      SelectObject( hDC, hpen );
   }

   if( xbrush != 0 )
   {
      SelectObject( hDC, hbrush );
   }
}

HB_FUNC( RR_FILLRECT )
{
   RECT  rect;
   rect.left = hb_parni( 1, 2 );
   rect.top = hb_parni( 1, 1 );
   rect.right = hb_parni( 2, 2 );
   rect.bottom = hb_parni( 2, 1 );
   hb_retni( FillRect(hDC, &rect, (HBRUSH) hb_parnl(3)) );
}

HB_FUNC( RR_FRAMERECT )
{
   RECT  rect;
   rect.left = hb_parni( 1, 2 );
   rect.top = hb_parni( 1, 1 );
   rect.right = hb_parni( 2, 2 );
   rect.bottom = hb_parni( 2, 1 );
   hb_retni( FrameRect(hDC, &rect, (HBRUSH) hb_parnl(3)) );
}

HB_FUNC( RR_LINE )
{
   LONG  xpen = hb_parnl( 3 );
   if( xpen != 0 )
   {
      SelectObject( hDC, (HPEN) xpen );
   }

   MoveToEx( hDC, hb_parni(1, 2), hb_parni(1, 1), NULL );
   hb_retni( LineTo(hDC, hb_parni(2, 2), hb_parni(2, 1)) );
   if( xpen != 0 )
   {
      SelectObject( hDC, hpen );
   }
}

HB_FUNC( RR_LINETO )
{
   LONG  xpen = hb_parnl( 2 );
   if( xpen != 0 )
   {
      SelectObject( hDC, (HPEN) xpen );
   }

   hb_retni( LineTo(hDC, hb_parni(1, 2), hb_parni(1, 1)) );
   if( xpen != 0 )
   {
      SelectObject( hDC, hpen );
   }
}

HB_FUNC( RR_INVERTRECT )
{
   RECT  rect;
   rect.left = hb_parni( 1, 2 );
   rect.top = hb_parni( 1, 1 );
   rect.right = hb_parni( 2, 2 );
   rect.bottom = hb_parni( 2, 1 );
   hb_retni( InvertRect(hDC, &rect) );
}

HB_FUNC ( RR_GETDESKTOPAREA )
{
	RECT rect;
	SystemParametersInfo( SPI_GETWORKAREA, 1, &rect, 0 );

	hb_reta(4);
	hb_storni((INT) rect.top, -1, 1);
	hb_storni((INT) rect.left, -1, 2);
	hb_storni((INT) rect.bottom - rect.top, -1, 3);
	hb_storni((INT) rect.right - rect.left, -1, 4);
}
/*
HB_FUNC( RR_GETWINDOWRECT )
{
   RECT  rect;
   HWND  hwnd = ( HWND ) hb_parnl( 1, 7 );
   if( hwnd == 0 )
   {
      hwnd = GetDesktopWindow();
   }

   GetWindowRect( hwnd, &rect );
   hb_storni( rect.top, 1, 1 );
   hb_storni( rect.left, 1, 2 );
   hb_storni( rect.bottom, 1, 3 );
   hb_storni( rect.right, 1, 4 );
   hb_storni( rect.bottom - rect.top + 1, 1, 5 );
   hb_storni( rect.right - rect.left + 1, 1, 6 );
}
*/
HB_FUNC( RR_GETCLIENTRECT )
{
   RECT  rect;
   GetClientRect( (HWND) hb_parnl(1, 7), &rect );
   hb_storni( rect.top, 1, 1 );
   hb_storni( rect.left, 1, 2 );
   hb_storni( rect.bottom, 1, 3 );
   hb_storni( rect.right, 1, 4 );
   hb_storni( rect.bottom - rect.top + 1, 1, 5 );
   hb_storni( rect.right - rect.left + 1, 1, 6 );
}

HB_FUNC( RR_SCROLLWINDOW )
{
   ScrollWindow( (HWND) hb_parnl(1), hb_parni(2), hb_parni(3), NULL, NULL );
}

HB_FUNC( RR_PREVIEWPLAY )
{
   RECT           rect;
   HDC            imgDC = GetWindowDC( (HWND) hb_parnl(1) );
   HDC            tmpDC = CreateCompatibleDC( imgDC );
   HENHMETAFILE   hh = SetEnhMetaFileBits( (UINT) hb_parclen(2, 1), hb_parc(2, 1) );
   if( tmpDC == NULL )
   {
      ReleaseDC( (HWND) hb_parnl(1), imgDC );
      hb_retl( 0 );
   }

   if( himgbmp != 0 )
   {
      DeleteObject( himgbmp );
   }

   SetRect( &rect, 0, 0, hb_parnl(3, 4), hb_parnl(3, 3) );
   himgbmp = CreateCompatibleBitmap( imgDC, rect.right, rect.bottom );
   DeleteObject( SelectObject(tmpDC, (HBITMAP) himgbmp) );
   FillRect( tmpDC, &rect, (HBRUSH) GetStockObject(WHITE_BRUSH) );
   PlayEnhMetaFile( tmpDC, hh, &rect );
   DeleteEnhMetaFile( hh );
   SendMessage( (HWND) hb_parnl(1), (UINT) STM_SETIMAGE, (WPARAM) IMAGE_BITMAP, (LPARAM) himgbmp );
   ReleaseDC( (HWND) hb_parnl(1), imgDC );
   DeleteDC( tmpDC );
   if( himgbmp == 0 )
   {
      hb_retl( 0 );
   }
   else
   {
      hb_retl( 1 );
   }
}

HB_FUNC( RR_PLAYTHUMB )
{
   RECT           rect;
   HDC            tmpDC;
   HDC            imgDC = GetWindowDC( (HWND) hb_parnl(1, 5) );
   HENHMETAFILE   hh = SetEnhMetaFileBits( (UINT) hb_parclen(2, 1), hb_parc(2, 1) );
   int            i;
   i = hb_parni( 4 ) - 1;
   tmpDC = CreateCompatibleDC( imgDC );
   SetRect( &rect, 0, 0, hb_parni(1, 4), hb_parni(1, 3) );
   hbmp[i] = CreateCompatibleBitmap( imgDC, rect.right, rect.bottom );
   DeleteObject( SelectObject(tmpDC, hbmp[i]) );
   FillRect( tmpDC, &rect, (HBRUSH) GetStockObject(WHITE_BRUSH) );
   PlayEnhMetaFile( tmpDC, hh, &rect );
   DeleteEnhMetaFile( hh );
   TextOut( tmpDC, ( int ) rect.right / 2 - 5, ( int ) rect.bottom / 2 - 5, hb_parc(3), hb_parclen(3) );
   SendMessage( (HWND) hb_parnl(1, 5), (UINT) STM_SETIMAGE, (WPARAM) IMAGE_BITMAP, (LPARAM) hbmp[i] );
   ReleaseDC( (HWND) hb_parnl(1, 5), imgDC );
   DeleteDC( tmpDC );
}

HB_FUNC( RR_PLAYENHMETAFILE )
{
   RECT           rect;
   HENHMETAFILE   hh = SetEnhMetaFileBits( (UINT) hb_parclen(1, 1), hb_parc(1, 1) );
   SetRect( &rect, 0, 0, hb_parnl(1, 5), hb_parnl(1, 4) );
   PlayEnhMetaFile( (HDC) hb_parnl(2), hh, &rect );
   DeleteEnhMetaFile( hh );
}

HB_FUNC( RR_LALABYE )
{
   if( hb_parni(1) == 1 )
   {
      hDCtemp = hDC;
      hDC = hDCRef;
   }
   else
   {
      hDC = hDCtemp;
   }
}

#pragma ENDDUMP
