/*----------------------------------------------------------------------------
 MINIGUI - Harbour Win32 GUI library source code

 Copyright 2002-2008 Roberto Lopez <harbourminigui@gmail.com>
 http://harbourminigui.googlepages.com/

 This program is free software; you can redistribute it and/or modify it under 
 the terms of the GNU General Public License as published by the Free Software 
 Foundation; either version 2 of the License, or (at your option) any later 
 version. 

 This program is distributed in the hope that it will be useful, but WITHOUT 
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS 
 FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

 You should have received a copy of the GNU General Public License along with 
 this software; see the file COPYING. If not, write to the Free Software 
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA (or 
 visit the web site http://www.gnu.org/).

 As a special exception, you have permission for additional uses of the text 
 contained in this release of Harbour Minigui.

 The exception is that, if you link the Harbour Minigui library with other 
 files to produce an executable, this does not by itself cause the resulting 
 executable to be covered by the GNU General Public License.
 Your use of that executable is in no way restricted on account of linking the 
 Harbour-Minigui library code into it.

 Parts of this project are based upon:

	"Harbour GUI framework for Win32"
 	Copyright 2001 Alexander S.Kresin <alex@belacy.belgorod.su>
 	Copyright 2001 Antonio Linares <alinares@fivetech.com>
	www - http://www.harbour-project.org

	"Harbour Project"
	Copyright 1999-2008, http://www.harbour-project.org/

 Parts of this module are based upon:

	"HBPRINT" 
	Copyright 2002 Richard Rylko <rrylko@poczta.onet.pl>
	http://rrylko.republika.pl

	"HBPRINTER"
	Copyright 2002 Richard Rylko <rrylko@poczta.onet.pl>
	http://rrylko.republika.pl

---------------------------------------------------------------------------*/

///////////////////////////////////////////////////////////////////////////////
// LOW LEVEL C PRINT ROUTINES
///////////////////////////////////////////////////////////////////////////////


#include <windows.h>
#include <stdio.h>
#include "hbapi.h"
#include "hbvm.h"
#include "hbstack.h"
#include "hbapiitm.h"
#include <winuser.h>
#include <wingdi.h>
#include "commctrl.h"
#include "olectl.h"

HB_FUNC ( _HMG_PRINTER_ABORTDOC )
{
	HDC hdcPrint = (HDC) hb_parnl(1) ;
	AbortDoc(hdcPrint);
}

HB_FUNC ( _HMG_PRINTER_STARTDOC )
{

	DOCINFO docInfo;

	HDC hdcPrint = (HDC) hb_parnl(1) ;

	if ( hdcPrint != 0 )
	{
		ZeroMemory(&docInfo, sizeof(docInfo));
		docInfo.cbSize = sizeof(docInfo);
		docInfo.lpszDocName = hb_parc(2);

		StartDoc(hdcPrint, &docInfo);
	}
}

HB_FUNC ( _HMG_PRINTER_STARTPAGE )
{

	HDC hdcPrint = (HDC) hb_parnl(1) ;

	if ( hdcPrint != 0 )
	{
		StartPage(hdcPrint);
	}

}

HB_FUNC ( _HMG_PRINTER_C_PRINT )
{

	// 1:  Hdc
	// 2:  y
	// 3:  x
	// 4:  FontName
	// 5:  FontSize
	// 6:  R Color
	// 7:  G Color
	// 8:  B Color
	// 9:  Text
	// 10: Bold
	// 11: Italic
	// 12: Underline
	// 13: StrikeOut
	// 14: Color Flag
	// 15: FontName Flag
	// 16: FontSize Flag

	HGDIOBJ hgdiobj ;

	char FontName [32] ;
	int FontSize ;

	DWORD fdwItalic ;
	DWORD fdwUnderline ;
	DWORD fdwStrikeOut ;

	int fnWeight ;
	int r ;
	int g ;
	int b ;

	int x = hb_parni(3) ;
	int y = hb_parni(2) ;

	HFONT hfont ;

	HDC hdcPrint = (HDC) hb_parnl(1) ;

	int FontHeight ;

	if ( hdcPrint != 0 )
	{

		// Bold

		if ( hb_parl(10) )
		{
			fnWeight = FW_BOLD ;
		}
		else
		{
			fnWeight = FW_NORMAL ;
		}

		// Italic 

		if ( hb_parl(11) )
		{
			fdwItalic = TRUE ;
		}
		else
		{
			fdwItalic = FALSE ;
		}

		// UnderLine

		if ( hb_parl(12) )
		{
			fdwUnderline = TRUE ;
		}
		else
		{
			fdwUnderline = FALSE ;
		}

		// StrikeOut

		if ( hb_parl(13) )
		{
			fdwStrikeOut = TRUE ;
		}
		else
		{
			fdwStrikeOut = FALSE ;
		}

		// Color

		if ( hb_parl(14) )
		{
			r = hb_parni(6) ;
			g = hb_parni(7) ;
			b = hb_parni(8) ;
		}
		else
		{
			r = 0 ;
			g = 0 ;
			b = 0 ;
		}

		// Fontname

		if ( hb_parl(15) )
		{
			strcpy ( FontName , hb_parc(4) ) ;
		}
		else
		{
			strcpy( FontName , "Arial" ) ;
		}

		// FontSize

		if ( hb_parl(16) )
		{
			FontSize = hb_parni(5) ;
		}
		else
		{
			FontSize = 10 ;
		}

		FontHeight = -MulDiv ( FontSize , GetDeviceCaps ( hdcPrint , LOGPIXELSY ) , 72 ) ;

		hfont = CreateFont 
			( 
			FontHeight, 
			0, 
			0, 
			0, 
			fnWeight , 
			fdwItalic , 
			fdwUnderline , 
			fdwStrikeOut , 
			DEFAULT_CHARSET, 
			OUT_TT_PRECIS, 
			CLIP_DEFAULT_PRECIS, 
			DEFAULT_QUALITY, 
			FF_DONTCARE, 
			FontName 
			);

		hgdiobj = SelectObject ( hdcPrint , hfont ) ;

		SetTextColor( hdcPrint , RGB ( r , g , b ) ) ;
		SetBkMode( hdcPrint , TRANSPARENT );
 
		TextOut( hdcPrint , 
			( x * GetDeviceCaps ( hdcPrint , LOGPIXELSX ) / 1000 ) - GetDeviceCaps ( hdcPrint , PHYSICALOFFSETX ) ,
			( y * GetDeviceCaps ( hdcPrint , LOGPIXELSY ) / 1000 ) - GetDeviceCaps ( hdcPrint , PHYSICALOFFSETY ) ,
			hb_parc(9),
			strlen(hb_parc(9)) ) ; 

		SelectObject ( hdcPrint , hgdiobj ) ;

		DeleteObject ( hfont ) ;

	}

}

HB_FUNC ( _HMG_PRINTER_C_MULTILINE_PRINT )
{

	// 1:  Hdc
	// 2:  y
	// 3:  x
	// 4:  FontName
	// 5:  FontSize
	// 6:  R Color
	// 7:  G Color
	// 8:  B Color
	// 9:  Text
	// 10: Bold
	// 11: Italic
	// 12: Underline
	// 13: StrikeOut
	// 14: Color Flag
	// 15: FontName Flag
	// 16: FontSize Flag
	// 17: ToRow
	// 18: ToCol
	// 19: Alignment

	UINT uFormat = 0 ;

	HGDIOBJ hgdiobj ;

	char FontName [32] ;
	int FontSize ;

	DWORD fdwItalic ;
	DWORD fdwUnderline ;
	DWORD fdwStrikeOut ;

	RECT rect ;

	int fnWeight ;
	int r ;
	int g ;
	int b ;

	int x = hb_parni(3) ;
	int y = hb_parni(2) ;
	int toy = hb_parni(17) ;
	int tox = hb_parni(18) ;

	HFONT hfont ;

	HDC hdcPrint = (HDC) hb_parnl(1) ;

	int FontHeight ;

	if ( hdcPrint != 0 )
	{

		// Bold

		if ( hb_parl(10) )
		{
			fnWeight = FW_BOLD ;
		}
		else
		{
			fnWeight = FW_NORMAL ;
		}

		// Italic 

		if ( hb_parl(11) )
		{
			fdwItalic = TRUE ;
		}
		else
		{
			fdwItalic = FALSE ;
		}

		// UnderLine

		if ( hb_parl(12) )
		{
			fdwUnderline = TRUE ;
		}
		else
		{
			fdwUnderline = FALSE ;
		}

		// StrikeOut

		if ( hb_parl(13) )
		{
			fdwStrikeOut = TRUE ;
		}
		else
		{
			fdwStrikeOut = FALSE ;
		}

		// Color

		if ( hb_parl(14) )
		{
			r = hb_parni(6) ;
			g = hb_parni(7) ;
			b = hb_parni(8) ;
		}
		else
		{
			r = 0 ;
			g = 0 ;
			b = 0 ;
		}

		// Fontname

		if ( hb_parl(15) )
		{
			strcpy ( FontName , hb_parc(4) ) ;
		}
		else
		{
			strcpy( FontName , "Arial" ) ;
		}

		// FontSize

		if ( hb_parl(16) )
		{
			FontSize = hb_parni(5) ;
		}
		else
		{
			FontSize = 10 ;
		}

		FontHeight = -MulDiv ( FontSize , GetDeviceCaps ( hdcPrint , LOGPIXELSY ) , 72 ) ;

		hfont = CreateFont 
			( 
			FontHeight, 
			0, 
			0, 
			0, 
			fnWeight , 
			fdwItalic , 
			fdwUnderline , 
			fdwStrikeOut , 
			DEFAULT_CHARSET, 
			OUT_TT_PRECIS, 
			CLIP_DEFAULT_PRECIS, 
			DEFAULT_QUALITY, 
			FF_DONTCARE, 
			FontName 
			);

		if ( hb_parni(19) == 0 )
		{
			uFormat = DT_END_ELLIPSIS | DT_NOPREFIX | DT_WORDBREAK | DT_LEFT ; 
		}
		else if ( hb_parni(19) == 2 )
		{
			uFormat = DT_END_ELLIPSIS | DT_NOPREFIX | DT_WORDBREAK | DT_RIGHT ; 
		}
		else if ( hb_parni(19) == 6 )
		{
			uFormat = DT_END_ELLIPSIS | DT_NOPREFIX | DT_WORDBREAK | DT_CENTER ; 
		}

		hgdiobj = SelectObject ( hdcPrint , hfont ) ;

		SetTextColor( hdcPrint , RGB ( r , g , b ) ) ;
		SetBkMode( hdcPrint , TRANSPARENT );

		rect.left	= ( x * GetDeviceCaps ( hdcPrint , LOGPIXELSX ) / 1000 ) - GetDeviceCaps ( hdcPrint , PHYSICALOFFSETX ) ;
		rect.top	= ( y * GetDeviceCaps ( hdcPrint , LOGPIXELSY ) / 1000 ) - GetDeviceCaps ( hdcPrint , PHYSICALOFFSETY )  ;
		rect.right	= ( tox * GetDeviceCaps ( hdcPrint , LOGPIXELSX ) / 1000 ) - GetDeviceCaps ( hdcPrint , PHYSICALOFFSETX ) ;
		rect.bottom	= ( toy * GetDeviceCaps ( hdcPrint , LOGPIXELSY ) / 1000 ) - GetDeviceCaps ( hdcPrint , PHYSICALOFFSETY ) ;
 
		DrawText ( hdcPrint , 
			hb_parc(9),
			strlen(hb_parc(9)), 
			&rect ,
			uFormat  
			) ; 

		SelectObject ( hdcPrint , hgdiobj ) ;

		DeleteObject ( hfont ) ;

	}

}

HB_FUNC ( _HMG_PRINTER_ENDPAGE )
{
	HDC hdcPrint = (HDC) hb_parnl(1) ;

	if ( hdcPrint != 0 )
	{
		EndPage(hdcPrint);
	}

}

HB_FUNC ( _HMG_PRINTER_ENDDOC )
{
	HDC hdcPrint = (HDC) hb_parnl(1) ;

	if ( hdcPrint != 0 )	
	{
		EndDoc(hdcPrint);
	}

}

HB_FUNC ( _HMG_PRINTER_DELETEDC )
{
	HDC hdcPrint = (HDC) hb_parnl(1) ;

	DeleteDC(hdcPrint);

}

HB_FUNC ( _HMG_PRINTER_PRINTDIALOG )
{

	PRINTDLG pd ;

	LPDEVMODE   pDevMode;

	pd.lStructSize = sizeof(PRINTDLG); 
	pd.hDevMode = (HANDLE) NULL; 
	pd.hDevNames = (HANDLE) NULL; 
	pd.Flags = PD_RETURNDC | PD_PRINTSETUP ; 
	pd.hwndOwner = NULL ; 
	pd.hDC = (HDC) NULL; 
	pd.nFromPage = 1; 
	pd.nToPage = 1; 
	pd.nMinPage = 0; 
	pd.nMaxPage = 0; 
	pd.nCopies = 1; 
	pd.hInstance = (HINSTANCE) NULL; 
	pd.lCustData = 0L; 
	pd.lpfnPrintHook = (LPPRINTHOOKPROC) NULL; 
	pd.lpfnSetupHook = (LPSETUPHOOKPROC) NULL; 
	pd.lpPrintTemplateName = (LPSTR) NULL; 
	pd.lpSetupTemplateName = (LPSTR)  NULL; 
	pd.hPrintTemplate = (HANDLE) NULL; 
	pd.hSetupTemplate = (HANDLE) NULL; 
 
	if ( PrintDlg(&pd) ) 
	{
		pDevMode = (LPDEVMODE) GlobalLock(pd.hDevMode);

		hb_reta( 4 );
		hb_stornl	( (LONG) pd.hDC		, -1, 1 ); 
		hb_storc	( pDevMode->dmDeviceName, -1, 2 ); 
		hb_storni	( pDevMode->dmCopies	, -1, 3 ); 
		hb_storni	( pDevMode->dmCollate	, -1, 4 ); 

		GlobalUnlock(pd.hDevMode);
	}
	else
	{
		hb_reta( 4 );
		hb_stornl	( 0	, -1, 1 ); 
		hb_storc	( ""	, -1, 2 ); 
		hb_storni	( 0	, -1, 3 ); 
		hb_storni	( 0	, -1, 4 ); 
	}

}

HB_FUNC (APRINTERS)
{

	OSVERSIONINFO osvi ;

	LPBYTE cBuffer ;
	LPBYTE pBuffer ;

	DWORD dwSize = 0;
	DWORD dwPrinters = 0;
	DWORD i;

	PRINTER_INFO_4* pInfo4 = NULL ;
	PRINTER_INFO_5* pInfo = NULL ;

	osvi.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);

	GetVersionEx(&osvi);

	if (osvi.dwPlatformId == VER_PLATFORM_WIN32_NT)
	{
		EnumPrinters(PRINTER_ENUM_LOCAL | PRINTER_ENUM_CONNECTIONS , NULL,4, NULL, 0, &dwSize, &dwPrinters);
	}
	else
	{
		EnumPrinters(PRINTER_ENUM_LOCAL , NULL,5, NULL, 0, &dwSize, &dwPrinters);
	}

	pBuffer = (char *) GlobalAlloc(GPTR, dwSize);

	if (pBuffer == NULL)
	{
		hb_reta(0);
		GlobalFree(pBuffer);
		return;
	}

	if (osvi.dwPlatformId == VER_PLATFORM_WIN32_NT)
	{
		EnumPrinters( PRINTER_ENUM_LOCAL | PRINTER_ENUM_CONNECTIONS , NULL,4, pBuffer, dwSize, &dwSize, &dwPrinters);
	}
	else
	{
		EnumPrinters( PRINTER_ENUM_LOCAL , NULL,5, pBuffer, dwSize, &dwSize, &dwPrinters);
	}

	if (dwPrinters == 0)
	{
		hb_reta(0);
		GlobalFree(pBuffer);
		return;
	}

	if (osvi.dwPlatformId == VER_PLATFORM_WIN32_NT)
	{
		pInfo4 = (PRINTER_INFO_4*)pBuffer;
	}
	else
	{
		pInfo = (PRINTER_INFO_5*)pBuffer;
	}

	hb_reta( dwPrinters );

	if (osvi.dwPlatformId == VER_PLATFORM_WIN32_NT)
	{
		for ( i = 0; i < dwPrinters; i++, pInfo4++)
		{
			cBuffer = (char *) GlobalAlloc(GPTR, 256);
			strcat( cBuffer, pInfo4->pPrinterName );
			hb_storc( cBuffer , -1 , i+1 );
			GlobalFree(cBuffer);
		}
	}
	else
	{
		for ( i = 0; i < dwPrinters; i++, pInfo++)
		{
			cBuffer = (char *) GlobalAlloc(GPTR, 256);
			strcat( cBuffer, pInfo->pPrinterName );
			hb_storc( cBuffer , -1 , i+1 );
			GlobalFree(cBuffer);
		}
	}

	GlobalFree(pBuffer);

}

HB_FUNC ( _HMG_PRINTER_C_RECTANGLE )
{

	// 1: hDC
	// 2: y
	// 3: x
	// 4: toy
	// 5: tox
	// 6: width
	// 7: R Color
	// 8: G Color
	// 9: B Color
	// 10: lWindth
	// 11: lColor 

	int r ;
	int g ;
	int b ;

	int x = hb_parni(3) ;
	int y = hb_parni(2) ;

	int tox = hb_parni(5) ;
	int toy = hb_parni(4) ;

	int width ;

	HDC hdcPrint = (HDC) hb_parnl(1) ;
	HGDIOBJ hgdiobj;
	HPEN hpen;

	if ( hdcPrint != 0 )
	{

		// Width

		if ( hb_parl(10) )
		{
			width = hb_parni(6) ;
		}
		else
		{
			width = 1 * 10000 / 254 ;
		}

		// Color

		if ( hb_parl(11) )
		{
			r = hb_parni(7) ;
			g = hb_parni(8) ;
			b = hb_parni(9) ;
		}
		else
		{
			r = 0 ;
			g = 0 ;
			b = 0 ;
		}

		hpen = CreatePen( (int) PS_SOLID, ( width * GetDeviceCaps ( hdcPrint , LOGPIXELSX ) / 1000 ), (COLORREF) RGB( r , g , b ) );

		hgdiobj = SelectObject( (HDC) hdcPrint , hpen );

		Rectangle( (HDC) hdcPrint , 
			( x * GetDeviceCaps ( hdcPrint , LOGPIXELSX ) / 1000 ) - GetDeviceCaps ( hdcPrint , PHYSICALOFFSETX ) ,
			( y * GetDeviceCaps ( hdcPrint , LOGPIXELSY ) / 1000 ) - GetDeviceCaps ( hdcPrint , PHYSICALOFFSETY ) ,	 
			( tox * GetDeviceCaps ( hdcPrint , LOGPIXELSX ) / 1000 ) - GetDeviceCaps ( hdcPrint , PHYSICALOFFSETX ) ,
			( toy * GetDeviceCaps ( hdcPrint , LOGPIXELSY ) / 1000 ) - GetDeviceCaps ( hdcPrint , PHYSICALOFFSETY ) 
			);

		SelectObject( (HDC) hdcPrint , (HGDIOBJ) hgdiobj );

		DeleteObject( hpen );

	}

}

HB_FUNC ( _HMG_PRINTER_C_ROUNDRECTANGLE )
{

	// 1: hDC
	// 2: y
	// 3: x
	// 4: toy
	// 5: tox
	// 6: width
	// 7: R Color
	// 8: G Color
	// 9: B Color
	// 10: lWindth
	// 11: lColor 

	int r ;
	int g ;
	int b ;

	int x = hb_parni(3) ;
	int y = hb_parni(2) ;

	int tox = hb_parni(5) ;
	int toy = hb_parni(4) ;

	int width ;

	int w , h , p ;

	HDC hdcPrint = (HDC) hb_parnl(1) ;
	HGDIOBJ hgdiobj;
	HPEN hpen;

	if ( hdcPrint != 0 )
	{

		// Width

		if ( hb_parl(10) )
		{
			width = hb_parni(6) ;
		}
		else
		{
			width = 1 * 10000 / 254 ;
		}

		// Color

		if ( hb_parl(11) )
		{
			r = hb_parni(7) ;
			g = hb_parni(8) ;
			b = hb_parni(9) ;
		}
		else
		{
			r = 0 ;
			g = 0 ;
			b = 0 ;
		}

		hpen = CreatePen( (int) PS_SOLID, ( width * GetDeviceCaps ( hdcPrint , LOGPIXELSX ) / 1000 ), (COLORREF) RGB( r , g , b ) );

		hgdiobj = SelectObject( (HDC) hdcPrint , hpen );

		w = ( tox * GetDeviceCaps ( hdcPrint , LOGPIXELSX ) / 1000 ) - ( x * GetDeviceCaps ( hdcPrint , LOGPIXELSX ) / 1000 ) ;
		h = ( toy * GetDeviceCaps ( hdcPrint , LOGPIXELSY ) / 1000 ) - ( y * GetDeviceCaps ( hdcPrint , LOGPIXELSY ) / 1000 ) ;
		p = ( w + h ) / 2 ;
		p = p / 10 ;

		RoundRect( (HDC) hdcPrint , 
			( x * GetDeviceCaps ( hdcPrint , LOGPIXELSX ) / 1000 ) - GetDeviceCaps ( hdcPrint , PHYSICALOFFSETX ) ,
			( y * GetDeviceCaps ( hdcPrint , LOGPIXELSY ) / 1000 ) - GetDeviceCaps ( hdcPrint , PHYSICALOFFSETY ) ,	 
			( tox * GetDeviceCaps ( hdcPrint , LOGPIXELSX ) / 1000 ) - GetDeviceCaps ( hdcPrint , PHYSICALOFFSETX ) ,
			( toy * GetDeviceCaps ( hdcPrint , LOGPIXELSY ) / 1000 ) - GetDeviceCaps ( hdcPrint , PHYSICALOFFSETY ) , 
			p ,
			p 
			) ;

		SelectObject( (HDC) hdcPrint , (HGDIOBJ) hgdiobj );

		DeleteObject( hpen );

	}

}

HB_FUNC ( _HMG_PRINTER_C_LINE )
{

	// 1: hDC
	// 2: y
	// 3: x
	// 4: toy
	// 5: tox
	// 6: width
	// 7: R Color
	// 8: G Color
	// 9: B Color
	// 10: lWindth
	// 11: lColor 

	int r ;
	int g ;
	int b ;

	int x = hb_parni(3) ;
	int y = hb_parni(2) ;

	int tox = hb_parni(5) ;
	int toy = hb_parni(4) ;

	int width ;

	HDC hdcPrint = (HDC) hb_parnl(1) ;
	HGDIOBJ hgdiobj;
	HPEN hpen;

	if ( hdcPrint != 0 )
	{

		// Width

		if ( hb_parl(10) )
		{
			width = hb_parni(6) ;
		}
		else
		{
			width = 1 * 10000 / 254 ;
		}

		// Color

		if ( hb_parl(11) )
		{
			r = hb_parni(7) ;
			g = hb_parni(8) ;
			b = hb_parni(9) ;
		}
		else
		{
			r = 0 ;
			g = 0 ;
			b = 0 ;
		}

		hpen = CreatePen( (int) PS_SOLID, ( width * GetDeviceCaps ( hdcPrint , LOGPIXELSX ) / 1000 ), (COLORREF) RGB( r , g , b ) );

		hgdiobj = SelectObject( (HDC) hdcPrint , hpen );

		MoveToEx( (HDC) hdcPrint , 
			( x * GetDeviceCaps ( hdcPrint , LOGPIXELSX ) / 1000 ) - GetDeviceCaps ( hdcPrint , PHYSICALOFFSETX ) ,
			( y * GetDeviceCaps ( hdcPrint , LOGPIXELSY ) / 1000 ) - GetDeviceCaps ( hdcPrint , PHYSICALOFFSETY ) ,	 
			NULL
			);

		LineTo ( (HDC) hdcPrint , 
			( tox * GetDeviceCaps ( hdcPrint , LOGPIXELSX ) / 1000 ) - GetDeviceCaps ( hdcPrint , PHYSICALOFFSETX ) ,
			( toy * GetDeviceCaps ( hdcPrint , LOGPIXELSY ) / 1000 ) - GetDeviceCaps ( hdcPrint , PHYSICALOFFSETY ) 
			);

		SelectObject( (HDC) hdcPrint , (HGDIOBJ) hgdiobj );

		DeleteObject( hpen );

	}

}

HB_FUNC ( _HMG_PRINTER_SETPRINTERPROPERTIES )
{
	HANDLE hPrinter = NULL;
	DWORD dwNeeded = 0;
	PRINTER_INFO_2 *pi2 ;
	DEVMODE *pDevMode = NULL;
	BOOL bFlag;
	LONG lFlag;

	HDC hdcPrint ;
	
	int fields = 0 ;

	bFlag = OpenPrinter( hb_parc(1) , &hPrinter, NULL);

	if (!bFlag || (hPrinter == NULL))
	{
		MessageBox(0, "Printer Configuration Failed! (001)", "Error!",MB_ICONEXCLAMATION | MB_OK | MB_SYSTEMMODAL);

		hb_reta( 4 );
		hb_stornl	( 0 	, -1, 1 ); 
		hb_storc	( ""	, -1, 2 ); 
		hb_storni	( 0	, -1, 3 ); 
		hb_storni	( 0	, -1, 4 ); 

		return;
	}

	SetLastError(0);

	bFlag = GetPrinter(hPrinter, 2, 0, 0, &dwNeeded);

	if ((!bFlag) && (GetLastError() != ERROR_INSUFFICIENT_BUFFER) || (dwNeeded == 0))
	{
		ClosePrinter(hPrinter);
		MessageBox(0, "Printer Configuration Failed! (002)", "Error!",MB_ICONEXCLAMATION | MB_OK | MB_SYSTEMMODAL);

		hb_reta( 4 );
		hb_stornl	( 0 	, -1, 1 ); 
		hb_storc	( ""	, -1, 2 ); 
		hb_storni	( 0	, -1, 3 ); 
		hb_storni	( 0	, -1, 4 ); 

		return;
	}

	pi2 = (PRINTER_INFO_2 *)GlobalAlloc(GPTR, dwNeeded);

	if (pi2 == NULL)
	{
		ClosePrinter(hPrinter);
		MessageBox(0, "Printer Configuration Failed! (003)", "Error!",MB_ICONEXCLAMATION | MB_OK | MB_SYSTEMMODAL);

		hb_reta( 4 );
		hb_stornl	( 0 	, -1, 1 ); 
		hb_storc	( ""	, -1, 2 ); 
		hb_storni	( 0	, -1, 3 ); 
		hb_storni	( 0	, -1, 4 ); 

		return;
	}

	bFlag = GetPrinter(hPrinter, 2, (LPBYTE)pi2, dwNeeded, &dwNeeded);

	if (!bFlag)
	{
		GlobalFree(pi2);
		ClosePrinter(hPrinter);
		MessageBox(0, "Printer Configuration Failed! (004)", "Error!",MB_ICONEXCLAMATION | MB_OK | MB_SYSTEMMODAL);

		hb_reta( 4 );
		hb_stornl	( 0 	, -1, 1 ); 
		hb_storc	( ""	, -1, 2 ); 
		hb_storni	( 0	, -1, 3 ); 
		hb_storni	( 0	, -1, 4 ); 

		return;
	}

	if (pi2->pDevMode == NULL)
	{
		dwNeeded = DocumentProperties(NULL, hPrinter, hb_parc(1), NULL, NULL, 0);
		if (dwNeeded <= 0)
		{
			GlobalFree(pi2);
			ClosePrinter(hPrinter);
			MessageBox(0, "Printer Configuration Failed! (005)", "Error!",MB_ICONEXCLAMATION | MB_OK | MB_SYSTEMMODAL);

			hb_reta( 4 );
			hb_stornl	( 0 	, -1, 1 ); 
			hb_storc	( ""	, -1, 2 ); 
			hb_storni	( 0	, -1, 3 ); 
			hb_storni	( 0	, -1, 4 ); 

			return;
		}

		pDevMode = (DEVMODE *)GlobalAlloc(GPTR, dwNeeded);
		if (pDevMode == NULL)
		{
			GlobalFree(pi2);
			ClosePrinter(hPrinter);
			MessageBox(0, "Printer Configuration Failed! (006)", "Error! (006)",MB_ICONEXCLAMATION | MB_OK | MB_SYSTEMMODAL);

			hb_reta( 4 );
			hb_stornl	( 0 	, -1, 1 ); 
			hb_storc	( ""	, -1, 2 ); 
			hb_storni	( 0	, -1, 3 ); 
			hb_storni	( 0	, -1, 4 ); 

			return;
		}

		lFlag = DocumentProperties(NULL, hPrinter, hb_parc(1), pDevMode, NULL,DM_OUT_BUFFER);
		if (lFlag != IDOK || pDevMode == NULL)
		{
			GlobalFree(pDevMode);
			GlobalFree(pi2);
			ClosePrinter(hPrinter);
			MessageBox(0, "Printer Configuration Failed! (007)", "Error!",MB_ICONEXCLAMATION | MB_OK | MB_SYSTEMMODAL);

			hb_reta( 4 );
			hb_stornl	( 0 	, -1, 1 ); 
			hb_storc	( ""	, -1, 2 ); 
			hb_storni	( 0	, -1, 3 ); 
			hb_storni	( 0	, -1, 4 ); 

			return;
		}

		pi2->pDevMode = pDevMode;
	}

	///////////////////////////////////////////////////////////////////////
	// Specify Fields
	//////////////////////////////////////////////////////////////////////
	// Orientation
	if ( hb_parni(2) != -999 )
	{
		fields = fields | DM_ORIENTATION ;
	}

	// PaperSize
	if ( hb_parni(3) != -999 )
	{
		fields = fields | DM_PAPERSIZE ;
	}

	// PaperLength
	if ( hb_parni(4) != -999 )
	{
		fields = fields | DM_PAPERLENGTH ;
	}

	// PaperWidth
	if ( hb_parni(5) != -999 )
	{
		fields = fields | DM_PAPERWIDTH	 ;
	}

	// Copies
	if ( hb_parni(6) != -999 )
	{
		fields = fields | DM_COPIES ;
	}

	// Default Source
	if ( hb_parni(7) != -999 )
	{
		fields = fields | DM_DEFAULTSOURCE ;
	}

	// Print Quality
	if ( hb_parni(8) != -999 )
	{
		fields = fields | DM_PRINTQUALITY ;
	}

	// Print Color
	if ( hb_parni(9) != -999 )
	{
		fields = fields | DM_COLOR ;
	}

	// Print Duplex Mode
	if ( hb_parni(10) != -999 )
	{
		fields = fields | DM_DUPLEX ;
	}

	// Print Collate
	if ( hb_parni(11) != -999 )
	{
		fields = fields | DM_COLLATE ;
	}

	pi2->pDevMode->dmFields = fields ;

	///////////////////////////////////////////////////////////////////////
	// Load Fields
	//////////////////////////////////////////////////////////////////////
	// Orientation
	if ( hb_parni(2) != -999 )
	{
		if (!(pi2->pDevMode->dmFields & DM_ORIENTATION))
		{
			MessageBox(0, "Printer Configuration Failed: ORIENTATION Property Not Supported By Selected Printer", "Error!",MB_ICONEXCLAMATION | MB_OK | MB_SYSTEMMODAL);

			hb_reta( 4 );
			hb_stornl	( 0 	, -1, 1 ); 
			hb_storc	( ""	, -1, 2 ); 
			hb_storni	( 0	, -1, 3 ); 
			hb_storni	( 0	, -1, 4 ); 

			return;
		}

		pi2->pDevMode->dmOrientation = (short) hb_parni(2);
	}

	// PaperSize
	if ( hb_parni(3) != -999 )
	{
		if (!(pi2->pDevMode->dmFields & DM_PAPERSIZE ))
		{
			MessageBox(0, "Printer Configuration Failed: PAPERSIZE Property Not Supported By Selected Printer", "Error!",MB_ICONEXCLAMATION | MB_OK | MB_SYSTEMMODAL);

			hb_reta( 4 );
			hb_stornl	( 0 	, -1, 1 ); 
			hb_storc	( ""	, -1, 2 ); 
			hb_storni	( 0	, -1, 3 ); 
			hb_storni	( 0	, -1, 4 ); 

			return;
		}

		pi2->pDevMode->dmPaperSize = (short) hb_parni(3);
	}

	// PaperLength
	if ( hb_parni(4) != -999 )
	{
		if (!(pi2->pDevMode->dmFields & DM_PAPERLENGTH ))
		{
			MessageBox(0, "Printer Configuration Failed: PAPERLENGTH Property Not Supported By Selected Printer", "Error!",MB_ICONEXCLAMATION | MB_OK | MB_SYSTEMMODAL);

			hb_reta( 4 );
			hb_stornl	( 0 	, -1, 1 ); 
			hb_storc	( ""	, -1, 2 ); 
			hb_storni	( 0	, -1, 3 ); 
			hb_storni	( 0	, -1, 4 ); 

			return;
		}

		pi2->pDevMode->dmPaperLength = (short) hb_parni(4) * 10 ;
	}

	// PaperWidth
	if ( hb_parni(5) != -999 )
	{
		if (!(pi2->pDevMode->dmFields & DM_PAPERWIDTH))
		{
			MessageBox(0, "Printer Configuration Failed: PAPERWIDTH Property Not Supported By Selected Printer", "Error!",MB_ICONEXCLAMATION | MB_OK | MB_SYSTEMMODAL);

			hb_reta( 4 );
			hb_stornl	( 0 	, -1, 1 ); 
			hb_storc	( ""	, -1, 2 ); 
			hb_storni	( 0	, -1, 3 ); 
			hb_storni	( 0	, -1, 4 ); 

			return;
		}

		pi2->pDevMode->dmPaperWidth = (short) hb_parni(5) * 10 ;
	}

	// Copies
	if ( hb_parni(6) != -999 )
	{
		if (!(pi2->pDevMode->dmFields & DM_COPIES ))
		{
			MessageBox(0, "Printer Configuration Failed: COPIES Property Not Supported By Selected Printer", "Error!",MB_ICONEXCLAMATION | MB_OK | MB_SYSTEMMODAL);

			hb_reta( 4 );
			hb_stornl	( 0 	, -1, 1 ); 
			hb_storc	( ""	, -1, 2 ); 
			hb_storni	( 0	, -1, 3 ); 
			hb_storni	( 0	, -1, 4 ); 

			return;
		}

		pi2->pDevMode->dmCopies = (short) hb_parni(6);
	}

	// Default Source
	if ( hb_parni(7) != -999 )
	{
		if (!(pi2->pDevMode->dmFields & DM_DEFAULTSOURCE ))
		{
			MessageBox(0, "Printer Configuration Failed: DEFAULTSOURCE Property Not Supported By Selected Printer", "Error!",MB_ICONEXCLAMATION | MB_OK | MB_SYSTEMMODAL);

			hb_reta( 4 );
			hb_stornl	( 0 	, -1, 1 ); 
			hb_storc	( ""	, -1, 2 ); 
			hb_storni	( 0	, -1, 3 ); 
			hb_storni	( 0	, -1, 4 ); 

			return;
		}

		pi2->pDevMode->dmDefaultSource = (short) hb_parni(7);
	}

	// Print Quality
	if ( hb_parni(8) != -999 )
	{
		if (!(pi2->pDevMode->dmFields & DM_PRINTQUALITY ))
		{
			MessageBox(0, "Printer Configuration Failed: QUALITY Property Not Supported By Selected Printer", "Error!",MB_ICONEXCLAMATION | MB_OK | MB_SYSTEMMODAL);

			hb_reta( 4 );
			hb_stornl	( 0 	, -1, 1 ); 
			hb_storc	( ""	, -1, 2 ); 
			hb_storni	( 0	, -1, 3 ); 
			hb_storni	( 0	, -1, 4 ); 

			return;
		}

		pi2->pDevMode->dmPrintQuality = (short) hb_parni(8);
	}

	// Print Color
	if ( hb_parni(9) != -999 )
	{
		if (!(pi2->pDevMode->dmFields & DM_COLOR ))
		{
			MessageBox(0, "Printer Configuration Failed: COLOR Property Not Supported By Selected Printer", "Error!",MB_ICONEXCLAMATION | MB_OK | MB_SYSTEMMODAL);

			hb_reta( 4 );
			hb_stornl	( 0 	, -1, 1 ); 
			hb_storc	( ""	, -1, 2 ); 
			hb_storni	( 0	, -1, 3 ); 
			hb_storni	( 0	, -1, 4 ); 

			return;
		}

		pi2->pDevMode->dmColor = (short) hb_parni(9);
	}

	// Print Duplex
	if ( hb_parni(10) != -999 )
	{
		if (!(pi2->pDevMode->dmFields & DM_DUPLEX ))
		{
			MessageBox(0, "Printer Configuration Failed: DUPLEX Property Not Supported By Selected Printer", "Error!",MB_ICONEXCLAMATION | MB_OK | MB_SYSTEMMODAL);

			hb_reta( 4 );
			hb_stornl	( 0 	, -1, 1 ); 
			hb_storc	( ""	, -1, 2 ); 
			hb_storni	( 0	, -1, 3 ); 
			hb_storni	( 0	, -1, 4 ); 

			return;
		}

		pi2->pDevMode->dmDuplex = (short) hb_parni(10);
	}

	// Print Collate
	if ( hb_parni(11) != -999 )
	{
		if (!(pi2->pDevMode->dmFields & DM_COLLATE ))
		{
			MessageBox(0, "Printer Configuration Failed: COLLATE Property Not Supported By Selected Printer", "Error!",MB_ICONEXCLAMATION | MB_OK | MB_SYSTEMMODAL);

			hb_reta( 4 );
			hb_stornl	( 0 	, -1, 1 ); 
			hb_storc	( ""	, -1, 2 ); 
			hb_storni	( 0	, -1, 3 ); 
			hb_storni	( 0	, -1, 4 ); 

			return;
		}

		pi2->pDevMode->dmCollate = (short) hb_parni(11);
	}

	//////////////////////////////////////////////////////////////////////

	pi2->pSecurityDescriptor = NULL;

	lFlag = DocumentProperties(NULL, hPrinter, hb_parc(1), pi2->pDevMode, pi2->pDevMode,DM_IN_BUFFER | DM_OUT_BUFFER);

	if (lFlag != IDOK)
	{
		GlobalFree(pi2);
		ClosePrinter(hPrinter);
		if (pDevMode)
		{
			GlobalFree(pDevMode);
		}
		MessageBox(0, "Printer Configuration Failed! (008)", "Error!",MB_ICONEXCLAMATION | MB_OK | MB_SYSTEMMODAL);

		hb_reta( 4 );
		hb_stornl	( 0 	, -1, 1 ); 
		hb_storc	( ""	, -1, 2 ); 
		hb_storni	( 0	, -1, 3 ); 
		hb_storni	( 0	, -1, 4 ); 

		return;
	}

	hdcPrint = CreateDC(NULL, TEXT( hb_parc(1) ), NULL, pi2->pDevMode);

	if ( hdcPrint != NULL )
	{
		hb_reta( 4 );
		hb_stornl	( (LONG) hdcPrint		, -1, 1 ); 
		hb_storc	( hb_parc(1)			, -1, 2 ); 
		hb_storni	( (INT) pi2->pDevMode->dmCopies	, -1, 3 ); 
		hb_storni	( (INT) pi2->pDevMode->dmCollate	, -1, 4 ); 
	}
	else
	{
		hb_reta( 4 );
		hb_stornl	( 0 	, -1, 1 ); 
		hb_storc	( ""	, -1, 2 ); 
		hb_storni	( 0	, -1, 3 ); 
		hb_storni	( 0	, -1, 4 ); 
	}

	if (pi2)
	{
		GlobalFree(pi2);
	}

	if (hPrinter)
	{
		ClosePrinter(hPrinter);
	}

	if (pDevMode)
	{
		GlobalFree(pDevMode);
	}

}


HB_FUNC (GETDEFAULTPRINTER)
{
        
	OSVERSIONINFO osvi;
	LPPRINTER_INFO_5 PrinterInfo;
	DWORD Needed , Returned ;
	DWORD BufferSize = 254;

	char DefaultPrinter [254] ;

	osvi.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);

	GetVersionEx(&osvi);

	if (osvi.dwPlatformId == VER_PLATFORM_WIN32_WINDOWS)
	{

		EnumPrinters(PRINTER_ENUM_DEFAULT,NULL,5,NULL,0,&Needed,&Returned);
		PrinterInfo = (LPPRINTER_INFO_5) LocalAlloc(LPTR,Needed);
		EnumPrinters(PRINTER_ENUM_DEFAULT,NULL,5,(LPBYTE) PrinterInfo,Needed,&Needed,&Returned);
		strcpy(DefaultPrinter,PrinterInfo->pPrinterName);
		LocalFree(PrinterInfo);

	}
	else if (osvi.dwPlatformId == VER_PLATFORM_WIN32_NT)
	{

		GetProfileString("windows","device","",DefaultPrinter,BufferSize);
		strtok(DefaultPrinter, ",");

	}

	hb_retc(DefaultPrinter);

}

HB_FUNC ( _HMG_PRINTER_STARTPAGE_PREVIEW )
{

	HDC tmpDC ;
	RECT emfrect ;

	SetRect(&emfrect,0,0,GetDeviceCaps( (HDC) hb_parnl(1) , HORZSIZE)*100 , GetDeviceCaps( (HDC) hb_parnl(1) , VERTSIZE)*100 ) ;

	tmpDC = CreateEnhMetaFile ( (HDC) hb_parnl(1) , hb_parc(2) , &emfrect , "" ) ; 

	hb_retnl ( (LONG) tmpDC ) ; 

}

HB_FUNC ( _HMG_PRINTER_ENDPAGE_PREVIEW )
{
	DeleteEnhMetaFile(CloseEnhMetaFile( (HDC) hb_parnl (1) ) );
}

HB_FUNC ( _HMG_PRINTER_SHOWPAGE )
{

	HENHMETAFILE hemf ;
	HWND hWnd = (HWND) hb_parnl(2) ;
	HDC hDC ;
	RECT rct ;
	RECT aux ;
	int zw ;
	int zh ;
	int ClientWidth ;
	int ClientHeight ;
	int xOffset ;
	int yOffset ;

	hemf = GetEnhMetaFile( hb_parc(1) ) ; 

	hDC = GetDC(hWnd); 

	GetClientRect(hWnd,&rct);

	ClientWidth = rct.right - rct.left ;
	ClientHeight = rct.bottom - rct.left ;

	zw = hb_parni(5) * GetDeviceCaps( (HDC) hb_parnl(3) , HORZSIZE) / 750 ;
	zh = hb_parni(5) * GetDeviceCaps( (HDC) hb_parnl(3) , VERTSIZE) / 750 ;

	xOffset = ( ClientWidth - ( GetDeviceCaps( (HDC) hb_parnl(3) , HORZSIZE)  * hb_parni(4) / 10000 ) ) / 2 ;
	yOffset = ( ClientHeight - ( GetDeviceCaps( (HDC) hb_parnl(3) , VERTSIZE)  * hb_parni(4) / 10000 ) ) / 2 ;	

	SetRect(&rct,
	xOffset + hb_parni(6) - zw ,
	yOffset + hb_parni(7) - zh ,
	xOffset + ( GetDeviceCaps( (HDC) hb_parnl(3) , HORZSIZE) * hb_parni(4) / 10000 ) + hb_parni(6) + zw , 
	yOffset + ( GetDeviceCaps( (HDC) hb_parnl(3) , VERTSIZE) * hb_parni(4) / 10000 ) + hb_parni(7) + zh  
	) ;

	FillRect( (HDC) hDC , &rct , (HBRUSH) ( RGB(255,255,255) ) ) ;

	PlayEnhMetaFile(hDC, hemf, &rct); 

	// Remove prints outside printable area

	// Right
	aux.top = 0 ;
	aux.left = rct.right ;	
	aux.right =  ClientWidth ;
	aux.bottom = ClientHeight ;
	FillRect( (HDC) hDC , &aux , (HBRUSH) GetStockObject(GRAY_BRUSH	) ) ;

	// Bottom
	aux.top = rct.bottom ;
	aux.left = 0 ;	
	aux.right =  ClientWidth ;
	aux.bottom = ClientHeight ;
	FillRect( (HDC) hDC , &aux , (HBRUSH) GetStockObject(GRAY_BRUSH	) ) ;

	// Top
	aux.top = 0 ;
	aux.left = 0 ;	
	aux.right =  ClientWidth ;
	aux.bottom = yOffset + hb_parni(7) - zh ;
	FillRect( (HDC) hDC , &aux , (HBRUSH) GetStockObject(GRAY_BRUSH	) ) ;

	// Left
	aux.top = 0 ;
	aux.left = 0 ;	
	aux.right = xOffset + hb_parni(6) - zw ;
	aux.bottom = ClientHeight ;
	FillRect( (HDC) hDC , &aux , (HBRUSH) GetStockObject(GRAY_BRUSH	) ) ;

	//
 
	DeleteEnhMetaFile(hemf); 

	ReleaseDC(hWnd, hDC); 

} 

HB_FUNC ( _HMG_PRINTER_GETPAGEWIDTH )
{
	hb_retni ( GetDeviceCaps( (HDC) hb_parnl(1) , HORZSIZE ) ) ;
} 

HB_FUNC ( _HMG_PRINTER_GETPAGEHEIGHT )
{
	hb_retni ( GetDeviceCaps( (HDC) hb_parnl(1) , VERTSIZE ) ) ;
} 

HB_FUNC ( _HMG_PRINTER_PRINTPAGE )
{
	
	HENHMETAFILE hemf ;

	RECT rect ;

	hemf = GetEnhMetaFile( hb_parc(2) ) ; 

	SetRect(&rect,0,0,GetDeviceCaps((HDC) hb_parnl(1), HORZRES),GetDeviceCaps((HDC) hb_parnl(1), VERTRES));

	StartPage((HDC) hb_parnl(1));

	PlayEnhMetaFile((HDC) hb_parnl(1) , (HENHMETAFILE) hemf , &rect ) ;

	EndPage((HDC) hb_parnl(1));

	DeleteEnhMetaFile(hemf); 

}


HB_FUNC ( _HMG_PRINTER_PREVIEW_ENABLESCROLLBARS )
{
	EnableScrollBar( (HWND) hb_parnl(1) , SB_BOTH , ESB_ENABLE_BOTH	 ) ;
}

HB_FUNC ( _HMG_PRINTER_PREVIEW_DISABLESCROLLBARS )
{
	EnableScrollBar( (HWND) hb_parnl(1) , SB_BOTH , ESB_DISABLE_BOTH ) ;
}

HB_FUNC ( _HMG_PRINTER_PREVIEW_DISABLEHSCROLLBAR )
{
	EnableScrollBar( (HWND) hb_parnl(1) , SB_HORZ , ESB_DISABLE_BOTH ) ;
}

HB_FUNC ( _HMG_PRINTER_SETVSCROLLVALUE )
{

	SendMessage ( (HWND) hb_parnl(1) , WM_VSCROLL , MAKEWPARAM (SB_THUMBPOSITION , hb_parni(2) ) , 0 ) ;

}

HB_FUNC ( _HMG_PRINTER_GETPRINTERWIDTH )
{
	HDC hdc = (HDC) hb_parnl(1) ;
	hb_retnl ( GetDeviceCaps ( hdc , HORZSIZE ) ) ;
}

HB_FUNC ( _HMG_PRINTER_GETPRINTERHEIGHT )
{
	HDC hdc = (HDC) hb_parnl(1) ;
	hb_retnl ( GetDeviceCaps ( hdc , VERTSIZE ) ) ;
}

HB_FUNC ( _HMG_PRINTER_GETPRINTABLEAREAPHYSICALOFFSETX )
{
	HDC hdc = (HDC) hb_parni(1) ;
	hb_retnl ( GetDeviceCaps ( hdc , PHYSICALOFFSETX ) ) ;
}

HB_FUNC ( _HMG_PRINTER_GETPRINTABLEAREALOGPIXELSX )
{
	HDC hdc = (HDC) hb_parni(1) ;
	hb_retnl ( GetDeviceCaps ( hdc , LOGPIXELSX ) ) ;
}

HB_FUNC ( _HMG_PRINTER_GETPRINTABLEAREAPHYSICALOFFSETY )
{
	HDC hdc = (HDC) hb_parni(1) ;
	hb_retnl ( GetDeviceCaps ( hdc , PHYSICALOFFSETY ) ) ;
}

HB_FUNC ( _HMG_PRINTER_GETPRINTABLEAREALOGPIXELSY )
{
	HDC hdc = (HDC) hb_parni(1) ;
	hb_retnl ( GetDeviceCaps ( hdc , LOGPIXELSY ) ) ;
}

HB_FUNC ( _HMG_PRINTER_C_IMAGE )
{

	// 1: hDC
	// 2: Image File
	// 3: Row
	// 4: Col
	// 5: Height
	// 6: Width
	// 7: Stretch

	HRGN hrgn ;

	HDC hdcPrint = (HDC) hb_parnl(1) ;
	IStream *iStream ;
	IPicture *iPicture ;
	HGLOBAL hGlobal ;
	HANDLE hFile ;
	DWORD nFileSize ;
	DWORD nReadByte ;
	long lWidth ;
	long lHeight ;
	POINT lpp ;
	HRSRC hSource ;
	HGLOBAL hGlobalres ;
	LPVOID lpVoid ;
	int nSize ;
	HINSTANCE hinstance = GetModuleHandle ( NULL ) ;
	HBITMAP hbmp ;
	PICTDESC picd;

	int r = hb_parni(3) ; 
	int c = hb_parni(4) ; 
	int odr = hb_parni(5) ; // Height
	int odc = hb_parni(6) ; // Width
	int dr ;
	int dc ;

	if ( hdcPrint != 0 )
	{

		c = ( c * GetDeviceCaps ( hdcPrint , LOGPIXELSX ) / 1000 ) - GetDeviceCaps ( hdcPrint , PHYSICALOFFSETX ) ;
		r = ( r * GetDeviceCaps ( hdcPrint , LOGPIXELSY ) / 1000 ) - GetDeviceCaps ( hdcPrint , PHYSICALOFFSETY ) ;
		dc = ( odc * GetDeviceCaps ( hdcPrint , LOGPIXELSX ) / 1000 ) ; 
		dr = ( odr * GetDeviceCaps ( hdcPrint , LOGPIXELSY ) / 1000 ) ; 

		hFile = CreateFile(hb_parc(2), GENERIC_READ, 0, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);

		if (hFile == INVALID_HANDLE_VALUE)
		{

			hbmp = (HBITMAP) LoadImage(GetModuleHandle(NULL),hb_parc(2),IMAGE_BITMAP,0,0,LR_CREATEDIBSECTION);
			if (hbmp!=NULL)
			{
				picd.cbSizeofstruct=sizeof(PICTDESC);
				picd.picType=PICTYPE_BITMAP;
				picd.bmp.hbitmap=hbmp;
				OleCreatePictureIndirect(&picd,&IID_IPicture,TRUE,(LPVOID *)&iPicture);
			}
			else
			{
				hSource = FindResource(hinstance,hb_parc(2),"GIF");
				if ( hSource == 0 )
				{
					hSource = FindResource( hinstance , hb_parc(2) , "JPG" ) ;
				}
				if ( hSource == 0 ) 
				{
					return ;
				}
        
				hGlobalres = LoadResource(hinstance, hSource);

				if ( hGlobalres == 0 ) 
				{
					return ;
				}

				lpVoid = LockResource ( hGlobalres ) ;

				if ( lpVoid == 0 ) 
				{
					return ;
				}

				nSize = SizeofResource ( hinstance , hSource ) ;

				hGlobal = GlobalAlloc ( GPTR, nSize ) ;

				if ( hGlobal == 0 ) 
				{
					return ;
				}

				memcpy ( hGlobal , lpVoid , nSize ) ;

				FreeResource ( hGlobalres ) ;

				CreateStreamOnHGlobal ( hGlobal , TRUE, &iStream ) ;

				if ( iStream == 0 )
				{
					GlobalFree ( hGlobal ) ;
					return ;
				}

				OleLoadPicture ( iStream , nSize , TRUE , &IID_IPicture , (LPVOID *)&iPicture ) ;

				iStream->lpVtbl->Release(iStream);

			}
		}
		else
		{
			nFileSize = GetFileSize ( hFile , NULL ) ;
			hGlobal = GlobalAlloc ( GPTR , nFileSize ) ;
			ReadFile ( hFile , hGlobal , nFileSize , &nReadByte, NULL ) ;
			CloseHandle ( hFile ) ;
			CreateStreamOnHGlobal ( hGlobal , TRUE , &iStream ) ;
			OleLoadPicture ( iStream , nFileSize , TRUE , &IID_IPicture , (LPVOID*)&iPicture ) ;

			if ( iPicture == 0 )
			{
				return;
			}
		}

		iPicture->lpVtbl->get_Width(iPicture,&lWidth);
		iPicture->lpVtbl->get_Height(iPicture,&lHeight);

		if ( ! hb_parl(7) ) // Scale
		{

			if ( odr * lHeight / lWidth <= odr )
			{
				dr = odc * GetDeviceCaps ( hdcPrint , LOGPIXELSY ) / 1000 * lHeight / lWidth ;
			}
			else
			{
				dc = odr * GetDeviceCaps ( hdcPrint , LOGPIXELSX ) / 1000 * lWidth / lHeight ;
			}

		}

		GetViewportOrgEx ( hdcPrint , &lpp ) ;

		hrgn = CreateRectRgn(	c + lpp.x , 
					r + lpp.y ,
					c + dc + lpp.x - 1 , 
					r + dr + lpp.y - 1 
					) ;

		SelectClipRgn( hdcPrint , hrgn ) ;

		iPicture->lpVtbl->Render( 	iPicture , 
						hdcPrint , 
						c , 
						r , 
						dc , 
						dr , 
						0 , 
						lHeight , 
						lWidth , 
						-lHeight , 
						NULL ) ;

		SelectClipRgn(hdcPrint,NULL);

		iPicture->lpVtbl->Release(iPicture);

		GlobalFree(hGlobal);

	}

}

HB_FUNC( GETTEXTALIGN )
{
   hb_retni( GetTextAlign( (HDC) hb_parnl( 1 ) ) ) ;
}

HB_FUNC( SETTEXTALIGN )
{
   hb_retni( SetTextAlign( (HDC) hb_parnl( 1 ), (UINT) hb_parni( 2 ) ) ) ;
}
