/*----------------------------------------------------------------------------
 MINIGUI - Harbour Win32 GUI library source code

 Copyright 2002-2008 Roberto Lopez <harbourminigui@gmail.com>
 http://harbourminigui.googlepages.com/

 Property Grid control source code
 (C)2007 Janusz Pora <januszpora@onet.eu>

 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
 Foundation; either version 2 of the License, or (at your option) any later
 version.

 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

 You should have received a copy of the GNU General Public License along with
 this software; see the file COPYING. If not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA (or
 visit the web site http://www.gnu.org/).

 As a special exception, you have permission for additional uses of the text
 contained in this release of Harbour Minigui.

 The exception is that, if you link the Harbour Minigui library with other
 files to produce an executable, this does not by itself cause the resulting
 executable to be covered by the GNU General Public License.
 Your use of that executable is in no way restricted on account of linking the
 Harbour-Minigui library code into it.

 Parts of this project are based upon:

   "Harbour GUI framework for Win32"
    Copyright 2001 Alexander S.Kresin <alex@belacy.belgorod.su>
    Copyright 2001 Antonio Linares <alinares@fivetech.com>
   www - http://www.harbour-project.org

   "Harbour Project"
   Copyright 1999-2008, http://www.harbour-project.org/

   "WHAT32"
   Copyright 2002 AJ Wos <andrwos@aust1.net>

   "HWGUI"
     Copyright 2001-2007 Alexander S.Kresin <alex@belacy.belgorod.su>

---------------------------------------------------------------------------*/

#include "minigui.ch"
#include "i_PropGrid.ch"
#include "i_xml.ch"

#define EM_GETSEL   176
#define EM_SETSEL   177
#define WM_CLEAR    771
#define WM_CHAR     258
//#define WM_SETFOCUS   7
#define WM_KILLFOCUS  8
#define WM_NOTIFY     78
#define WM_KEYDOWN    256
#define WM_COMMAND      0x0111
#define WM_LBUTTONDOWN    513    // 0x0201
#define WM_LBUTTONUP      514    // 0x0202
#define WM_LBUTTONDBLCLK  515    // 0x203

#define BN_CLICKED      0
#define EN_CHANGE       768
/*
 * Combo Box Notification Codes
 */
#define CBN_SELCHANGE       1
#define CBN_KILLFOCUS       4
#define CBN_EDITCHANGE      5

#define TVM_EXPAND   4354
#define TVE_COLLAPSE   1
#define TVE_EXPAND   2
#define TVE_TOGGLE   3


#define TVN_SELCHANGED TVN_SELCHANGEDA
#define TVN_SELCHANGEDA   (-402)

#define CLR_NONE                0xFFFFFFFF

#define COLOR_SCROLLBAR         0
#define COLOR_BACKGROUND        1
#define COLOR_ACTIVECAPTION     2
#define COLOR_INACTIVECAPTION   3
#define COLOR_MENU              4
#define COLOR_WINDOW            5
#define COLOR_WINDOWFRAME       6
#define COLOR_MENUTEXT          7
#define COLOR_WINDOWTEXT        8
#define COLOR_CAPTIONTEXT       9
#define COLOR_ACTIVEBORDER      10
#define COLOR_INACTIVEBORDER    11
#define COLOR_APPWORKSPACE      12
#define COLOR_HIGHLIGHT         13
#define COLOR_HIGHLIGHTTEXT     14
#define COLOR_BTNFACE           15
#define COLOR_BTNSHADOW         16
#define COLOR_GRAYTEXT          17
#define COLOR_BTNTEXT           18
#define COLOR_INACTIVECAPTIONTEXT 19
#define COLOR_BTNHIGHLIGHT      20
#define COLOR_3DDKSHADOW        21
#define COLOR_3DLIGHT           22
#define COLOR_INFOTEXT          23
#define COLOR_INFOBK            24

#define APG_TYPE     1
#define APG_NAME     2
#define APG_VALUE    3
#define APG_DATA     4
#define APG_DIS      5
#define APG_CHG      6
#define APG_ID       7
#define APG_INFO     8
#define APG_VALNAME  9

//INFO aRowItem  => (Type,PropertyName,Value,Data,Disabled,changed,ItemID, ItemInfo, ValueName
//                     1        2        3     4     5       6       7       8          9
STATIC hIListSys   := 0
STATIC aFontName   := {'Font Name','Font Size','Bold','Italic','Underline','Strikeout'}
STATIC aFontType   := {'enum','numeric','logic','logic','logic','logic'}
STATIC nItemId     := 0

*------------------------------------------------------------------------------*
Function _DefinePropGrid ( ControlName, ParentFormName, row, col, width, height,;
   change, cFile, lXml, tooltip, fontname, fontsize, gotfocus, lostfocus, onclose,;
   break, aRowItem, HelpId, bold, italic, underline, strikeout, itemexpand,;
   backcolor, fontcolor, indent, itemheight, datawidth, ImgList, readonly, lInfo,;
   infoHeight, changevalue, aheadname, singleexpand )
*------------------------------------------------------------------------------*

Local i , ParentFormHandle , aControlhandle , mVar
Local FontHandle , aFont , k , Style := 0 , hColorIL

   DEFAULT ImgList      := 0
   DEFAULT Width        := 240
   DEFAULT Height       := 200
   DEFAULT dataWidth    := 120
   DEFAULT indent       := 12
   DEFAULT change       := ""
   DEFAULT changevalue  := ""
   DEFAULT gotfocus     := ""
   DEFAULT lostfocus    := ""
   DEFAULT onclose      := ""
   DEFAULT cFile        := ""
   DEFAULT aRowItem     := {}
   DEFAULT lInfo        := .f.
   DEFAULT infoHeight   := 80

   _HMG_ActivePropGridHandle := 0
   _HMG_ActiveCategoryHandle := 0

   IF dataWidth > 0.8 * width
      dataWidth :=  0.8 * width
   ENDIF

   if ( FontHandle := GetFontHandle( FontName ) ) != 0
           aFont := GetFontParam(FontHandle)
           FontName      := aFont[1]
           FontSize      := aFont[2]
           bold          := aFont[3]
           italic        := aFont[4]
           underline     := aFont[5]
           strikeout     := aFont[6]
   endif

   if _HMG_BeginWindowActive = TRUE
      ParentFormName := _HMG_ActiveFormName
      if .Not. Empty (_HMG_ActiveFontName) .And. ValType(FontName) == "U"
         FontName := _HMG_ActiveFontName
      EndIf
      if .Not. Empty (_HMG_ActiveFontSize) .And. ValType(FontSize) == "U"
         FontSize := _HMG_ActiveFontSize
      EndIf
   endif
   if _HMG_FrameLevel > 0
      col    := col + _HMG_ActiveFrameCol [_HMG_FrameLevel]
      row    := row + _HMG_ActiveFrameRow [_HMG_FrameLevel]
      ParentFormName := _HMG_ActiveFrameParentFormName [_HMG_FrameLevel]
   EndIf

   If .Not. _IsWindowDefined (ParentFormName)
      MsgMiniGuiError("Window: "+ ParentFormName + " is not defined. Program terminated" )
   Endif

   If _IsControlDefined (ControlName,ParentFormName)
      MsgMiniGuiError ("Control: " + ControlName + " Of " + ParentFormName + " Already defined. Program Terminated" )
   endif

   If aheadname != Nil .And. ValType(aheadname ) == 'A'
     IF Len(aheadname) > 0 .And. ValType( aheadname[1] ) == 'A'
        aheadname := aheadname[1]
     ENDIF
   EndIf

   mVar := '_' + ParentFormName + '_' + ControlName
   k := _GetControlFree()

   ParentFormHandle = GetFormHandle (ParentFormName)

   if valtype(Row) == "U" .or. valtype(Col) == "U"

      If _HMG_SplitLastControl == 'TOOLBAR'
          Break := .T.
      EndIf

      i := GetFormIndex ( ParentFormName )

      if i > 0

         aControlHandle := InitPropGrid ( _HMG_aFormReBarHandle [i] , col , row , width , height , indent , datawidth, readonly, lInfo, infoHeight, aheadname, singleexpand )

         If FontHandle != 0
            _SetFontHandle(aControlHandle[1],FontHandle)
         Else
            if valtype(fontname) == "U"
               FontName := _HMG_DefaultFontName
            endif
            if valtype(fontsize) == "U"
               FontSize := _HMG_DefaultFontSize
            endif
            FontHandle := _SetFont (aControlHandle[1],FontName,FontSize)
         EndIf

         AddSplitBoxItem ( aControlhandle[1] , _HMG_aFormReBarHandle [i] , Width , break , , , , _HMG_ActiveSplitBoxInverted )

         _HMG_SplitLastControl := 'PROPGRID'

      EndIf

   Else

      aControlHandle := InitPropGrid ( ParentFormHandle , col , row , width , height , indent , datawidth, readonly, lInfo, infoHeight, aheadname, singleexpand )

   endif

   _HMG_ActivePropGridHandle := aControlHandle[1]

   If FontHandle != 0
      _SetFontHandle(aControlHandle[1],FontHandle)
      _SetFontHandle(aControlHandle[2],FontHandle)
   Else
      if valtype(fontname) == "U"
         fontname := _HMG_DefaultFontName
      endif
      if valtype(fontsize) == "U"
         fontsize := _HMG_DefaultFontSize
      endif
      _SetFont (aControlHandle[2],fontname,_HMG_DefaultFontSize,.t.,italic,underline,strikeout)
      _SetFont (aControlHandle[3],fontname,_HMG_DefaultFontSize,bold,italic,underline,strikeout)
      if aControlHandle[3] != 0
         _SetFont (aControlHandle[5],fontname,_HMG_DefaultFontSize,.t.,italic,underline,strikeout)
      endif
      FontHandle := _SetFont (aControlHandle[1],fontname,_HMG_DefaultFontSize,bold,italic,underline,strikeout)
   EndIf

   If _HMG_BeginTabActive = .T.
      aAdd ( _HMG_ActiveTabCurrentPageMap , aControlHandle[1] )
   EndIf

   if valtype(tooltip) != "U"
      SetToolTip ( aControlHandle[1] , tooltip , GetFormToolTipHandle (ParentFormName) )
   endif

   if valtype(backcolor) == "A"
      TreeView_SetBkColor( aControlHandle[1], backcolor)
   endif

   if valtype(fontcolor) == "A"
      TreeView_SetTextColor( aControlHandle[1], fontcolor)
   endif

   indent := TreeView_GetIndent( aControlHandle[1])

   if valtype(itemheight) == "N"
      TreeView_SetItemHeight( aControlHandle[1], itemheight)
   else
      itemheight := TreeView_GetItemHeight( aControlHandle[1])
   endif

   hColorIL :=InitImageList ((itemheight-4)*1.4,itemheight-4,.t.)

   InitPropGridImageList( aControlHandle[1], hColorIL )  // Init Color Image List

   _HMG_ActivePropGridIndex := k
   Public &mVar. := k

   _HMG_aControlType [k] := "PROPGRID"
   _HMG_aControlNames [k] :=   ControlName
   _HMG_aControlHandles [k] :=   aControlHandle[1]
   _HMG_aControlParentHandles [k] :=   ParentFormHandle
   _HMG_aControlIds [k] :=   0
   _HMG_aControlProcedures [k] :=   ""
   _HMG_aControlPageMap  [k] :=  {}
   _HMG_aControlValue  [k] :=  Nil
   _HMG_aControlInputMask  [k] :=  itemexpand
   _HMG_aControllostFocusProcedure [k] :=   lostfocus
   _HMG_aControlGotFocusProcedure  [k] :=  gotfocus
   _HMG_aControlChangeProcedure  [k] :=  change
   _HMG_aControlDeleted  [k] :=  .F.
   _HMG_aControlBkColor  [k] :=  backcolor
   _HMG_aControlFontColor  [k] :=  fontcolor
   _HMG_aControlDblClick   [k] := onclose
   _HMG_aControlHeadClick   [k] := changevalue
   _HMG_aControlRow   [k] := Row
   _HMG_aControlCol   [k] := Col
   _HMG_aControlWidth   [k] := Width
   _HMG_aControlHeight   [k] := Height - IF(lInfo, infoHeight,0)
   _HMG_aControlSpacing   [k] := hColorIL
   _HMG_aControlContainerRow  [k] :=  iif ( _HMG_FrameLevel > 0 ,_HMG_ActiveFrameRow [_HMG_FrameLevel] , -1 )
   _HMG_aControlContainerCol  [k] :=  iif ( _HMG_FrameLevel > 0 ,_HMG_ActiveFrameCol [_HMG_FrameLevel] , -1 )
   _HMG_aControlPicture  [k] :=  {}
   _HMG_aControlContainerHandle  [k] :=  0
   _HMG_aControlFontName  [k] :=  fontname
   _HMG_aControlFontSize  [k] :=  fontsize
   _HMG_aControlFontAttributes   [k] := {bold,italic,underline,strikeout}
   _HMG_aControlToolTip   [k] :=  tooltip
   _HMG_aControlRangeMin   [k] :=  itemheight
   _HMG_aControlRangeMax   [k] :=  datawidth
   _HMG_aControlCaption   [k] :=  cFile
   _HMG_aControlVisible  [k] :=   .t.
   _HMG_aControlHelpId   [k] :=   HelpId
   _HMG_aControlFontHandle   [k] :=  FontHandle
   _HMG_aControlBrushHandle  [k] :=   0
   _HMG_aControlEnabled  [k] :=   .T.
   _HMG_aControlMiscData1 [k] := { indent, ImgList, aControlHandle, lInfo, infoHeight }

   IF  !Empty(cFile) .and. File(cFile)
      IF lXml
         InitItemPropertyXml(cFile, k)
         aRowItem := {}
      ELSE
         aRowItem := _LoadProperty(cFile)
      endif
   ENDIF
   IF Len(aRowItem) > 0
      InitItemProperty(aRowItem, k)
   ENDIF
Return Nil

*------------------------------------------------------------------------------*
FUNCTION _DefinePropertyItem ( cType, cName, cValue, aData, disabled, id, cInfo, cValName, cValNameDef )
*------------------------------------------------------------------------------*
   LOCAL n, aCol, nValue, cData, cToken,aItem
   LOCAL typePg := IdentTypePG(cType)
   DEFAULT cValue := "", aData := "", disabled := .f., cInfo :="", cValNameDef :="", cValName := cValNameDef
   IF ValType(aData) == 'N'
      aData:= LTrim(Str(adata))
   endif
   IF typePG == PG_ERROR
      MsgMiniGuiError ("Property Item type: "+cType+" wrong defined. Program Terminated" )
   endif
   IF ValType(id) == 'U'
      IF Len(_HMG_ActivePropGridArray) == 0
         nItemId := 100
      ENDIF
      Id := nItemId++
   ENDIF
   IF id !=0
      IF AScan(_HMG_ActivePropGridArray,{|x| x[7] == Id }) > 0
         MsgMiniGuiError ("Property Item ID double defined. Program Terminated" )
      ENDIF
   ENDIF
   CheckDataPG(typePG,@cValue,@aData,0)
   AAdd( _HMG_ActivePropGridArray,{ cType, cName, cValue, aData, disabled,.f.,id,cInfo,cValName})
RETURN Nil

*------------------------------------------------------------------------------*
Function CheckDataPG( typePG, cValue, aData, mod )
*------------------------------------------------------------------------------*
   LOCAL n, aCol, nValue, cData, aDat, cToken,aItem, cErr:="", ret:=.t.
   DO CASE
   CASE typePG == PG_STRING
      IF ValType(cValue) != 'C'
         cErr := "Property Value for STRING wrong defined."
         ret:=.f.
      endif
   CASE typePG == PG_INTEGER
      cErr := "Property Value for INTEGER wrong defined."
      IF ValType(cValue) == 'N'
         cValue := LTrim(Str(Int(cValue)))
      ELSEif ValType(cValue) == 'C'
         IF ISDIGIT( cValue)
            cValue := LTrim(Str(Int(Val(cValue))))
         ELSE
            ret:=.f.
         endif
      ELSE
         ret:=.f.
      ENDIF
   CASE typePG == PG_DOUBLE
      cErr := "Property Value for DOUBLE wrong defined."
      IF ValType(cValue) == 'N'
         cValue := LTrim(REMRIGHT(Str(cValue,16,8),'0'))
      ELSEif ValType(cValue) == 'C'
         IF ISDIGIT( cValue)
            cValue := LTrim(REMRIGHT(Str(Val(cValue),16,8),'0'))
         ELSE
            ret:=.f.
         endif
      ELSE
         ret:=.f.
      ENDIF
   CASE typePG == PG_SYSCOLOR
      IF IdentColor(2,cValue)==0
         cErr := "Property Value for SYSCOLOR wrong defined."
         ret:=.f.
      Endif
   CASE typePG == PG_COLOR
      aData := If( ValType( aData ) == "B", ;
                   Eval( aData ), aData )
      aCol :=IdentDataColor(aData)
      TOKENINIT (cValue,' ,()')
      n:=0
      do while !TOKENEND() .and. n++ <= 3
         nValue := Val( TOKENNEXT (cValue) )
         IF nValue != aCol[n]
            aCol[n] := nValue
         endif
      enddo
      TOKENEXIT()
      cValue := aCol2Str(aCol)
      aData :=  aVal2Str(aCol)
   CASE typePG == PG_USERFUN
      IF SubStr(alltrim(aData),1,1) != '{'
         aData := "{|x| "+aData +" }"
         IF ValType(&aData) != 'B'
            cErr := "Property Data for USERFUN wrong defined."
            ret:=.f.
         endif
      endif
   CASE typePG == PG_LOGIC
      IF At( cValue, 'true false' ) == 0
         cErr := "Property Value for LOGIC wrong defined."
         ret:=.f.
      Endif
   CASE typePG == PG_DATE
      IF !Empty(cValue)
         IF Empty(CToD(cValue))
            cErr := "Property Value for DATE wrong defined."
            ret:=.f.
         ENDIF
      endif
   CASE typePG == PG_FONT
   CASE typePG == PG_ARRAY
   CASE typePG == PG_ENUM
      IF At(cValue,aData ) == 0
         cErr := "Property Value for ENUM wrong defined."
         ret:=.f.
      Endif
   CASE typePG ==  PG_LIST
      IF At(cValue,aData ) == 0
         cErr := "Property Value for LIST wrong defined."
         ret:=.f.
      Endif
   CASE typePG == PG_FLAG
      cData := CHARREM ("[]", cValue)
      TOKENINIT (cData,',')
      do while !TOKENEND()
         cToken := Alltrim( TOKENNEXT (cData) )
         IF At(cToken,aData) == 0
            cErr := "Property Value for FLAG wrong defined."
            ret:=.f.
         endif
      enddo
      TOKENEXIT()
   CASE typePG == PG_SYSINFO
      IF ValType( aData ) != "B"
         IF At("SYSTEM",aData) == 0
            IF At("USERHOME",aData) == 0
               IF At("USERID",aData) == 0
                  IF At("USERNAME",aData) == 0
                     cErr := "Property Value for SYSINFO wrong defined."
                     ret:=.f.
                  ENDIF
               ENDIF
            ENDIF
         ENDIF
      ENDIF

   CASE typePG == PG_IMAGE
   CASE typePG == PG_FILE
   CASE typePG == PG_FOLDER
   CASE typePG == PG_CHECK
      aDat := IdentData(aData)
      cErr := "Property Value for CHECK wrong defined."
      IF Len(aDat)== 1
         IF At( aDat[1], 'true false' ) == 0
            ret:=.f.
         Endif
      ELSE
         if AScan(aDat,cValue)==0
            ret:=.f.
         ENDIF
      endif
   CASE typePG == PG_SIZE
      cData := CHARREM ("()", cValue)
      TOKENINIT (cData,',')
      do while !TOKENEND()
         cToken := Alltrim( TOKENNEXT (cData) )
         IF !ISDIGIT( cToken )
            cErr := "Property Value for SIZE wrong defined."
            ret:=.f.
         endif
      enddo
      TOKENEXIT()

   ENDCASE
   IF !ret
      IF Mod == 0
         MsgMiniGuiError (cErr+" Program Terminated")
      ELSE
         MsgExclamation(cErr,'Error', ,.f.)
      ENDIF
   ENDIF
RETURN ret

*------------------------------------------------------------------------------*
PROCEDURE _EndCategory()
*------------------------------------------------------------------------------*
   AAdd(_HMG_ActivePropGridArray,{'category','end','','',.f.,.f.,0})
RETURN

*------------------------------------------------------------------------------*
PROCEDURE _EndPropGrid()
*------------------------------------------------------------------------------*
   IF Len(_HMG_ActivePropGridArray) > 0
      InitItemProperty(_HMG_ActivePropGridArray, _HMG_ActivePropGridIndex)
   ENDIF
   _HMG_ActivePropGridHandle := 0
   _HMG_ActiveCategoryHandle := 0
   _HMG_ActivePropGridIndex  := 0
   _HMG_ActivePropGridArray  := {}
RETURN

*------------------------------------------------------------------------------*
FUNCTION _ShowInfoItem  (ParentForm, ControlName )
*------------------------------------------------------------------------------*
   LOCAL k, aControlHandle
   k := GetControlIndex ( ControlName, ParentForm )
   IF k > 0
      aControlHandle := _HMG_aControlMiscData1 [k,3]
      Pg_ToggleInfo( aControlHandle[1] )
   ENDIF
RETURN Nil

*------------------------------------------------------------------------------*
FUNCTION _AddPropertyItem  (ControlName , ParentForm, cCategory, cType, cName, cValue, aData, disabled, id ,Info, cValName, cValNameDef, mod )
*------------------------------------------------------------------------------*
   LOCAL aRowItem, hWndPG, nIndex, hItem, aNodeHandle :={}
   LOCAL typePg := IdentTypePG(cType)
   DEFAULT cCategory := "", cValue := "", aData := "", disabled := .f. ,Id := 0, Info :="", cValNameDef :="", cValName := cValNameDef, Mod:= 0

   if ValType(ParentForm) == 'U'
      IF _HMG_BeginWindowActive = TRUE
         ParentForm := _HMG_ActiveFormName
      ELSE
         MsgMiniGuiError("Parent Window is not defined. Program terminated" )
      ENDIF
   endif
   hWndPG := GetControlHandle (ControlName,ParentForm)
   nIndex := GetControlIndex (ControlName,ParentForm)
   IF Empty(cCategory)
      hItem := TreeView_GetSelection( hWndPG )
   ELSE
      hItem := PG_SearchCategory(hWndPG,cCategory)
   ENDIF

   IF id == 0 .or. PG_SearchID(hWndPG,Id) == 0

      IF CheckDataPG(typePG,@cValue,@aData,Mod)

         aRowItem := { cType, cName, cValue, aData, disabled, .f., Id, Info, cValName  }
         IF hItem != 0
            IF PG_GETITEM(hWndPG,hItem,PGI_TYPE)== 1
               AAdd(aNodeHandle,hItem)
               AddPropItem(hWndPG, aRowItem, 1, aNodeHandle,nIndex,.t.)
            ELSEIF Empty(cCategory)
               hItem:=TREEVIEW_GETPARENT(hWndPG,hItem)
               AAdd(aNodeHandle,hItem)
               AddPropItem(hWndPG, aRowItem, 1, aNodeHandle,nIndex,.t.)
            ELSE
               MsgInfo(" Item is not type of Category!", "Error")
            endif
         ELSE
            MsgInfo(" Category "+cCategory+" not found!", "Error")
         ENDIF
      endif
   else
      MsgInfo ("Property Item ID "+AllTrim(Str(Id))+" double defined."+CRLF+" Not added Item","Error" )
   ENDIF

RETURN Nil

*------------------------------------------------------------------------------*
FUNCTION _AddPropertyCategory  (ControlName , ParentForm, cCategory, cName, id ,Info )
*------------------------------------------------------------------------------*
   LOCAL aRowItem, hWndPG, nIndex, hItem, nLev, aNodeHandle :={}
   LOCAL typePg := IdentTypePG("category")
   DEFAULT cCategory := "", Id := 0, Info :=""

   if ValType(ParentForm) == 'U'
      IF _HMG_BeginWindowActive = TRUE
         ParentForm := _HMG_ActiveFormName
      ELSE
         MsgMiniGuiError("Parent Window is not defined. Program terminated" )
      ENDIF
   endif
   hWndPG := GetControlHandle (ControlName,ParentForm)
   nIndex := GetControlIndex (ControlName,ParentForm)
   IF Empty(cCategory) .or. Upper(cCategory)  == "ROOT"
      hItem := 0    //PG_GetRoot( hWndPG )
      nLev  := 0
   ELSE
      hItem := PG_SearchCategory(hWndPG,cCategory)
      nLev  := 1
   ENDIF
   IF id == 0 .or. PG_SearchID(hWndPG,Id) == 0

      aRowItem := { "category", cName, "", "", .t., .f., Id, Info, "" }
      IF hItem != 0
         IF PG_GETITEM(hWndPG,hItem,PGI_TYPE)== PG_CATEG
            AAdd(aNodeHandle,hItem)
            AddPropItem(hWndPG, aRowItem, nLev, aNodeHandle,nIndex,.t.)
         ELSEIF Empty(cCategory)
            hItem:=TREEVIEW_GETPARENT(hWndPG,hItem)
            AAdd(aNodeHandle,hItem)
            AddPropItem(hWndPG, aRowItem, nLev, aNodeHandle,nIndex,.t.)
         ELSE
            MsgInfo(" Item is not type of Category!", "Error")
         endif
      ELSE
         AddPropItem(hWndPG, aRowItem, nLev, aNodeHandle,nIndex,.f.)
      ENDIF
   else
      MsgInfo ("Property Item ID "+AllTrim(Str(Id))+" double defined."+CRLF+" Not added Item","Error" )
   ENDIF
RETURN Nil

*------------------------------------------------------------------------------*
FUNCTION InitItemProperty(aRowItem, k)
*------------------------------------------------------------------------------*
   LOCAL nLev:=0, n, nRow ,hItem
   LOCAL ControlHandle :=  _HMG_aControlHandles [k], aNodeHandle:={0}
//INFO aRowItem  => (Type,PropertyName,Value,Data,Disabled,changed,ItemID, ItemInfo, ValueName
//                     1        2        3     4     5       6       7       8          9
   TreeView_DeleteAllItems ( ControlHandle  )
   FOR n:=1 TO Len(aRowItem)
      nLev:= addPropItem(ControlHandle, aRowItem[n], nLev, aNodeHandle,k)
   NEXT n
   IF _HMG_aControlInputMask [k]
      ExpandPG(ControlHandle, 1 )
   endif
   TreeView_SelectItem ( ControlHandle , PG_GetRoot(  ControlHandle ) )
   SetFocus( ControlHandle )
RETURN Nil

*------------------------------------------------------------------------------*
FUNCTION InitItemPropertyXml(cFile, k)
*------------------------------------------------------------------------------*
LOCAL nLev, n, i, nRow ,hItem
LOCAL ControlHandle :=  _HMG_aControlHandles [k], aNodeHandle:={0}
LOCAL aRowItem, oXmlDoc, oXmlNode, oXmlSubNode

   oXmlDoc :=HXMLDoc():Read( cFile )
   TreeView_DeleteAllItems ( ControlHandle  )
   nLev := 0
//INFO aRowItem  => (Type,PropertyName,Value,Data,Disabled,changed,ItemID, ItemInfo, ValueName
//                     1        2        3     4     5       6       7       8          9
   IF !Empty( oXmlDoc:aItems )
      FOR n := 1 TO Len( oXmlDoc:aItems[1]:aItems )
         oXmlNode := oXmlDoc:aItems[1]:aItems[n]
         IF oXmlNode:Title =="category"  .OR. oXmlNode:Title =="font"
            aRowItem := SetItemArray(oXmlNode)
            addPropItem(ControlHandle, aRowItem, nLev, aNodeHandle,k)
            IF !Empty( oXmlNode:aItems )
               FOR i := 1 TO Len(oXmlNode:aItems)
                  oXmlSubNode := oXmlNode:aItems[i]
                  GetNextLevel( oXmlSubNode, ControlHandle, nLev, aNodeHandle,k)
               next
            ENDIF
         ELSE
            aRowItem := SetItemArray(oXmlNode)
            addPropItem(ControlHandle, aRowItem, nLev, aNodeHandle,k)
         endif
      NEXT n
   ENDIF
   IF _HMG_aControlInputMask [k]
      ExpandPG(ControlHandle, 1 )
   endif
   TreeView_SelectItem ( ControlHandle , PG_GetRoot(  ControlHandle ) )
   SetFocus( ControlHandle )

RETURN Nil

*------------------------------------------------------------------------------*
FUNCTION GetNextLevel( oXmlSubNode, ControlHandle, nLev, aNodeHandle, k )
*------------------------------------------------------------------------------*
   LOCAL n,i,oXmlSubNode1, aRowItem
   nLev++
   IF oXmlSubNode:Title =="category"  .OR. oXmlSubNode:Title =="font"
      aRowItem := SetItemArray(oXmlSubNode)
      addPropItem(ControlHandle, aRowItem, nLev, aNodeHandle,k)
      IF !Empty( oXmlSubNode:aItems )
         FOR i:=1 TO Len(oXmlSubNode:aItems)
            oXmlSubNode1 := oXmlSubNode:aItems[i]
            GetNextLevel( oXmlSubNode1, ControlHandle, nLev, aNodeHandle,k)
         NEXT
      endif
   else
      aRowItem := SetItemArray(oXmlsubNode)
      addPropItem(ControlHandle, aRowItem, nLev, aNodeHandle,k)
   ENDIF
RETURN nlev

*------------------------------------------------------------------------------*
FUNCTION SetItemArray(oXmlNode)
*------------------------------------------------------------------------------*
   Local aItem := {}
   AAdd (aItem,oXmlNode:Title)
   AAdd (aItem,GetAttr(oXmlNode,"Name"))
   AAdd (aItem,GetAttr(oXmlNode,"Value"))
   AAdd (aItem,GetAttr(oXmlNode,"cData"))
   AAdd (aItem,(GetAttr(oXmlNode,"disabled")=="true"))
   AAdd (aItem,(GetAttr(oXmlNode,"changed")=="true"))
   AAdd (aItem,Val(GetAttr(oXmlNode,"ItemID")))
   AAdd (aItem,GetAttr(oXmlNode,"Info"))
   AAdd (aItem,GetAttr(oXmlNode,"VarName"))
RETURN aItem

*------------------------------------------------------------------------------*
FUNCTION GetAttr(oXmlNode, cAttr )
*------------------------------------------------------------------------------*
   LOCAL xAttr:=oXmlNode:GetAttribute(cAttr)
RETURN IF(xAttr==Nil,"",xAttr)

*------------------------------------------------------------------------------*
FUNCTION ExpandPG( hWndPG, typ )
*------------------------------------------------------------------------------*
   LOCAL hItem
   SetFocus( hWndPG )
   hItem := PG_GetRoot( hWndPG )
   WHILE hItem != 0
      IF PG_GetItem( hWndPG, hItem, PGI_TYPE) ==  PG_CATEG
         DO case
         CASE typ ==0
            SendMessage ( hWndPG , TVM_EXPAND, TVE_TOGGLE, hItem )
         CASE typ ==1
            SendMessage ( hWndPG , TVM_EXPAND, TVE_EXPAND, hItem )
         CASE typ ==2
            SendMessage ( hWndPG , TVM_EXPAND, TVE_COLLAPSE, hItem )
         ENDCASE
      ENDIF
      hItem := PG_GetNextItem( hWndPG, hItem )
   ENDDO
RETURN Nil

*------------------------------------------------------------------------------*
FUNCTION ExpandCategPG( ParentForm, ControlName, cCategory, typ )
*------------------------------------------------------------------------------*
   LOCAL k, hItem, hWndPG
   k := GetControlIndex ( ControlName, ParentForm )
   IF k > 0
      hWndPG := _HMG_aControlHandles  [k]
      SetFocus( hWndPG )
      hItem := PG_SearchCategory(hWndPG,cCategory)

      IF hItem != 0
         IF PG_GetItem( hWndPG, hItem, PGI_TYPE) ==  PG_CATEG
            DO case
            CASE typ ==0
               SendMessage ( hWndPG , TVM_EXPAND, TVE_TOGGLE, hItem )
            CASE typ ==1
               SendMessage ( hWndPG , TVM_EXPAND, TVE_EXPAND, hItem )
            CASE typ ==2
               SendMessage ( hWndPG , TVM_EXPAND, TVE_COLLAPSE, hItem )
            ENDCASE
         ENDIF
      ENDIF
   endif
RETURN Nil

*------------------------------------------------------------------------------*
FUNCTION AddPropItem( ControlHandle, aRowIt, nLev, aNodeHandle, nIndex, lSelect )
*------------------------------------------------------------------------------*
   LOCAL n,nNodeH:=0 ,id :=0, PropType, nNodePrevH ,ImgId, ImgIdSel,nDataH,cInfo,cValName
   LOCAL aData,aDataFont, aDataFlag,nCheck,ItemType,ItemID
   DEFAULT lSelect :=.f.
   PropType := Lower(aRowIt[APG_TYPE])
   ItemType   := IdentTypePG(aRowIt[APG_TYPE])
   ItemId:= IF(Len(aRowIt) < APG_ID,0,aRowIt[APG_ID])
   cInfo := IF(Len(aRowIt) < APG_INFO,"",aRowIt[APG_INFO])
   cValName := IF(Len(aRowIt) < APG_VALNAME,"",aRowIt[APG_VALNAME])
   DO CASE
   CASE PropType == 'category'
      IF Lower(aRowIt[2]) == 'end'
         nLev--
         RETURN nLev
      ELSE
         nNodePrevH := IF(nLev > 0,aNodeHandle [nLev] ,0)
         nNodeH := AddPGItem ( ControlHandle , nNodePrevH , "", 0, 0 ,0,aRowIt[2],aRowIt[3],aRowIt[4],aRowIt[5],aRowIt[6],ItemType,ItemID, cInfo, cValName )
         nLev++
         IF Len( aNodeHandle ) >= nLev
            aNodeHandle[nLev] := nNodeH
         ELSE
            AAdd(aNodeHandle, nNodeH)
         ENDIF
         TreeView_SetBoldItem( ControlHandle,  nNodeH , .t. )
      ENDIF
   CASE PropType == 'font'
      IF nLev > 0
         nNodeH := AddPGItem ( ControlHandle , aNodeHandle [nLev] , "", 0, 0, 0,aRowIt[2],aRowIt[3],aRowIt[4],aRowIt[5],aRowIt[6],ItemType,ItemID, cInfo, cValName )
         aData :=IdentDataFont( aRowIt[4])
         nLev++
         IF Len( aNodeHandle ) >= nLev
            aNodeHandle[nLev] := nNodeH
         else
            AAdd(aNodeHandle,nNodeH)
         endif
         FOR n:=1 TO Len(aData)
            nDataH := AddPGItem ( ControlHandle , aNodeHandle [nLev] , "", 0, 0, 0,aData[n,2],aData[n,3],"FONT",.f.,.f.,IdentTypePG(aData[n,1]),0,"","" )
         NEXT
         nLev--
      ENDIF
   CASE PropType == 'flag'
      IF nLev > 0
         nNodeH := AddPGItem ( ControlHandle , aNodeHandle [nLev] , "", 0, 0, 0,aRowIt[2],aRowIt[3],aRowIt[4],aRowIt[5],aRowIt[6],ItemType,ItemID, cInfo, cValName )
         aData := IdentDataFlag(aRowIt[3],aRowIt[4])
         nLev++
         IF Len( aNodeHandle ) >= nLev
            aNodeHandle[nLev] := nNodeH
         ELSE
            AAdd(aNodeHandle,nNodeH)
         ENDIF
         FOR n:=1 TO Len(aData)
            nDataH := AddPGItem ( ControlHandle , aNodeHandle [nLev] , "", 0, 0, 0,aData[n,2],aData[n,3],"FLAG",.f.,.f.,IdentTypePG(aData[n,1]),0,"","" )
         NEXT
         nLev--
      ENDIF
   CASE PropType == 'size'
      IF nLev > 0
         nNodeH := AddPGItem ( ControlHandle , aNodeHandle [nLev] , "", 0, 0, 0,aRowIt[2],aRowIt[3],aRowIt[4],aRowIt[5],aRowIt[6],ItemType,ItemID, cInfo, cValName )
         aData := IdentDataSize(aRowIt[3],aRowIt[4])
         nLev++
         IF Len( aNodeHandle ) >= nLev
            aNodeHandle[nLev] := nNodeH
         else
            AAdd(aNodeHandle,nNodeH)
         endif
         FOR n:=1 TO Len(aData)
            nDataH := AddPGItem ( ControlHandle , aNodeHandle [nLev] , "", 0, 0, 0,aData[n,1],aData[n,2],"SIZE",.f.,.f.,PG_INTEGER,0,"","" )
         NEXT
         nLev--
      ENDIF
   OTHERWISE
      ImgId := 0
      ImgIdSel := 0
      nCheck := 0
      IF PropType == 'color'
         ImgId  := AddColorIL(aRowIt[3],nIndex)+1
         ImgIdSel := ImgId
      endif
      IF PropType == 'syscolor'
         ImgId  := AddColorIL(aRowIt[3],nIndex)+1
         ImgIdSel := ImgId
      endif
      IF  PropType == 'sysinfo'
         aRowIt[3] := GetSysInfo(aRowIt)
      endif
      IF  PropType == 'image'
         ImgId  := LoadImagePg(aRowIt[3],nIndex)+1
         ImgIdSel := ImgId
      endif
      IF  PropType == 'check'
         aData := IdentData(aRowIt[APG_DATA])
         IF Len(aData)== 1
            nCheck:= IF(aData[1] == 'true',2,1)
         ELSE
            if (nCheck := AScan(aData,aRowIt[APG_VALUE]))==0
               nCheck := 1
            ENDIF
         ENDIF
      endif
      IF nLev > 0
         nNodeH := AddPGItem ( ControlHandle , aNodeHandle [nLev] , "", ImgId, ImgIdSel, nCheck,aRowIt[2],aRowIt[3],aRowIt[4],aRowIt[5],aRowIt[6],ItemType,ItemID,cInfo,cValName  )
         IF nLev == 1
            TreeView_SetBoldItem( ControlHandle,  nNodeH )
         ENDIF
      ENDIF
  ENDCASE
  IF lSelect
     TreeView_SelectItem ( ControlHandle , nNodeH )
  ENDIF

RETURN nLev

*------------------------------------------------------------------------------*
FUNCTION GetSysInfo(aRowIt)
*------------------------------------------------------------------------------*
   LOCAL aDan, cDan := aRowIt[APG_VALUE]
   LOCAL typ := Lower(aRowIt[APG_DATA])
   IF  ValType(aRowIt[APG_DATA] ) == "B"
      cDan := Eval( aRowIt[APG_DATA] )
      IF ValType(cDan) != 'C'
         cDan := aRowIt[APG_VALUE]
      ENDIF
   ELSE
      typ := Lower(aRowIt[APG_DATA])
      DO CASE
      CASE typ == "system"
         aDan := WindowsVersion()
         cDan := aDan[1]+"("+  aDan[2]+','+ aDan[3]+")"
      CASE typ == "username"
         cDan := GetComputerName ( )
      CASE typ == "userid"
         cDan :=   GetUserName()
      CASE typ == "userhome"
         cDan :=  GetMyDocumentsFolder ( )
      ENDCASE
   ENDIF
RETURN cDan

*------------------------------------------------------------------------------*
FUNCTION AddColorIL(cColor,k)
*------------------------------------------------------------------------------*
   LOCAL hImageLst, nColor, hImage, ItHeight, ItWidth
   hImageLst := _HMG_aControlSpacing[k]
   ItHeight := _HMG_aControlRangeMin [k]-4
   ItWidth := ItHeight *1.4
   nColor := IdentColor(0,cColor)
   hImage := CREATECOLORBMP(_HMG_aControlParentHandles [k],nColor,ItWidth,ItHeight)

RETURN IL_AddIndirect( hImageLst , hImage , ItWidth , ItHeight )

*------------------------------------------------------------------------------*
FUNCTION LoadImagePG(cFile,k,hItem)
*------------------------------------------------------------------------------*
   LOCAL hImageLst, hImage, ItHeight, ItWidth
   DEFAULT hItem :=0
   hImageLst :=  _HMG_aControlSpacing[k]
   ItHeight:=  _HMG_aControlRangeMin [k]-4
   ItWidth := ItHeight *1.4
   hImage :=PG_SetPicture(_HMG_aControlParentHandles [k],cFile, ItWidth,ItHeight)
   IF hItem == 0
      RETURN IL_AddIndirect( hImageLst , hImage , ItWidth , ItHeight )
   ENDIF
   ResetPropGridImageList(_HMG_aControlHandles [k], hItem,hImage)
RETURN Nil

*------------------------------------------------------------------------------*
Function IdentDataColor(cColor,sep)
*------------------------------------------------------------------------------*
   LOCAL aData := {0,0,0},cToken,n:=1
   DEFAULT sep := ';'
   cColor := REMALL (cColor, '()')
   TOKENINIT (cColor,sep)
   do while !TOKENEND()
      cToken := Alltrim(TOKENNEXT (cColor) )
      aData[n++] :=Val(cToken)
      IF n > 3
         EXIT
      endif
   enddo
   TOKENEXIT()
RETURN aData

*------------------------------------------------------------------------------*
Function IdentDataFont(cFont,sep)
*------------------------------------------------------------------------------*
   LOCAL aDataFont := {},cToken,pos,n
   DEFAULT sep := ';'
   IF sep==','
      aDataFont :={"","","false","false","false","false"}
   ENDIF
   n:=1
   TOKENINIT (cFont,sep)
   do while !TOKENEND()
      cToken := Alltrim(TOKENNEXT (cFont) )
      IF sep ==','
         IF n < 3
            aDataFont[n]:= cToken
         ELSE
            IF (pos := AScan(aFontName,cToken))>0
               aDataFont[pos] := "true"
            ENDIF
         ENDIF
      ELSE
         AAdd(aDataFont,{aFontType[n],aFontName[n],cToken})
      endif
      n++
   enddo
   TOKENEXIT()
RETURN aDataFont

*------------------------------------------------------------------------------*
Function IdentDataFlag(cData,cFlag)
*------------------------------------------------------------------------------*
   LOCAL aData := {},cToken,cLogic
   cData := CHARREM ("[]", cData)
   TOKENINIT (cFlag,';')
   do while !TOKENEND()
      cToken := Alltrim(TOKENNEXT (cFlag) )
      cLogic := IF(At(cToken,cData) != 0,'true','false')
      AAdd(aData,{'logic',cToken,cLogic})
      IF cLogic == 'false'
         aData[1,3] := cLogic
      endif
   enddo
   TOKENEXIT()
RETURN aData

*------------------------------------------------------------------------------*
Function IdentDataSize(cValue,cData)
*------------------------------------------------------------------------------*
   LOCAL aData := {},cToken,n
   TOKENINIT (cData,';')
   do while !TOKENEND()
      cToken := Alltrim(TOKENNEXT (cData) )
      AAdd(aData,{cToken,""})
   enddo
   TOKENEXIT()
   cValue := CHARREM ("()", cValue)
   TOKENINIT (cValue,',')
   n:=1
   do while !TOKENEND() .and. n <= Len(aData)
      cToken := Alltrim(TOKENNEXT (cValue) )
      aData[n,2] :=cToken
      n++
   enddo
   TOKENEXIT()
RETURN aData

*------------------------------------------------------------------------------*
Function IdentData(cData,sep)
*------------------------------------------------------------------------------*
   LOCAL aData := {},cToken
   DEFAULT sep := ';'
   TOKENINIT (cData,sep)
   do while !TOKENEND()
      cToken := Alltrim(TOKENNEXT (cData) )
      AAdd(aData,cToken)
   enddo
   TOKENEXIT()
RETURN aData

*------------------------------------------------------------------------------*
Function IdentFileFlt(cData)
*------------------------------------------------------------------------------*
   LOCAL aData := {},cToken
   TOKENINIT (cData,';')
   do while !TOKENEND()
      cToken := Alltrim(TOKENNEXT (cData) )
      AAdd(aData,{'File ('+cToken+')',cToken})
   enddo
   TOKENEXIT()
RETURN aData

*------------------------------------------------------------------------------*
FUNCTION IdentTypePG(cType)
*------------------------------------------------------------------------------*
LOCAL ItemType := PG_DEFAULT
   IF ValType(cType)=='C'
      cType := AllTrim(Lower(cType))
      DO CASE
      CASE cType == 'category'
         ItemType := PG_CATEG
      CASE cType == 'string'
         ItemType := PG_STRING
      CASE cType == 'numeric' .or. cType == 'integer'
         ItemType := PG_INTEGER
      CASE cType == 'double'
         ItemType := PG_DOUBLE
      CASE cType == 'logic'
         ItemType := PG_LOGIC
      CASE cType == 'font'
         ItemType := PG_FONT
      CASE cType == 'color'
         ItemType := PG_COLOR
      CASE cType == 'syscolor'
         ItemType := PG_SYSCOLOR
      CASE cType == 'array'
         ItemType := PG_ARRAY
      CASE cType == 'enum'
         ItemType := PG_ENUM
      CASE cType == 'date'
         ItemType := PG_DATE
      CASE cType == 'image'
         ItemType := PG_IMAGE
      CASE cType == 'sysinfo'
         ItemType := PG_SYSINFO
      CASE cType == 'flag'
         ItemType := PG_FLAG
      CASE cType == 'check'
         ItemType := PG_CHECK
      CASE cType == 'size'
         ItemType := PG_SIZE
      CASE cType == 'file'
         ItemType := PG_FILE
      CASE cType == 'folder'
         ItemType := PG_FOLDER
      CASE cType == 'list'
         ItemType := PG_LIST
      CASE cType == 'userfun'
         ItemType := PG_USERFUN
      OTHERWISE
         ItemType := PG_ERROR
      ENDCASE
   ELSE
      DO CASE
      CASE cType == PG_CATEG
         ItemType := 'category'
      CASE cType == PG_STRING .or. cType == PG_DEFAULT
         ItemType := 'string'
      CASE cType == PG_INTEGER
         ItemType := 'numeric'
      CASE cType == PG_DOUBLE
         ItemType := 'double'
      CASE cType == PG_LOGIC
         ItemType := 'logic'
      CASE cType == PG_FONT
         ItemType := 'font'
      CASE cType == PG_COLOR
         ItemType := 'color'
      CASE cType == PG_SYSCOLOR
         ItemType := 'syscolor'
      CASE cType == PG_ARRAY
         ItemType := 'array'
      CASE cType == PG_ENUM
         ItemType := 'enum'
      CASE cType == PG_DATE
         ItemType := 'date'
      CASE cType == PG_IMAGE
         ItemType := 'image'
      CASE cType == PG_SYSINFO
         ItemType := 'sysinfo'
      CASE cType == PG_FLAG
         ItemType := 'flag'
      CASE cType == PG_CHECK
         ItemType := 'check'
      CASE cType == PG_SIZE
         ItemType := 'size'
      CASE cType == PG_FILE
         ItemType := 'file'
      CASE cType == PG_FOLDER
         ItemType := 'folder'
      CASE cType == PG_LIST
         ItemType := 'list'
      CASE cType == PG_USERFUN
         ItemType := 'userfun'
      ENDCASE
   ENDIF
RETURN ItemType

*------------------------------------------------------------------------------*
FUNCTION IdentColor(met,cColor)
*------------------------------------------------------------------------------*
   LOCAL nColor := CLR_NONE, pos, aCol, cToken, result
   LOCAL aSysColor := ;
      {{COLOR_3DDKSHADOW,   "ControlDark"},;
       {COLOR_BTNFACE,      "ButtonFace"},;
       {COLOR_BTNHIGHLIGHT, "ButtonHighlight"},;
       {COLOR_3DLIGHT,      "ControlLight"},;
       {COLOR_BTNSHADOW,    "ButtonShadow"},;
       {COLOR_ACTIVEBORDER, "ActiveBorder"},;
       {COLOR_ACTIVECAPTION,"ActiveCaption"},;
       {COLOR_APPWORKSPACE, "AppWorkspace"},;
       {COLOR_BACKGROUND,   "Desktop"},;
       {COLOR_BTNTEXT,      "ButtonText"},;
       {COLOR_CAPTIONTEXT,  "CaptionText"},;
       {COLOR_GRAYTEXT,     "GrayText"},;
       {COLOR_HIGHLIGHT,    "Highlight"},;
       {COLOR_HIGHLIGHTTEXT,"HighlightText"},;
       {COLOR_INACTIVEBORDER,"InactiveBorder"},;
       {COLOR_INACTIVECAPTION,"InactiveCaption"},;
       {COLOR_INACTIVECAPTIONTEXT,"InactiveCaptionText"},;
       {COLOR_INFOBK,       "Tooltip"},;
       {COLOR_INFOTEXT,     "TooltipText"},;
       {COLOR_MENU,         "Menu"},;
       {COLOR_MENUTEXT,     "MenuText"},;
       {COLOR_SCROLLBAR,    "Scroll"},;
       {COLOR_WINDOW,       "Window"},;
       {COLOR_WINDOWFRAME,  "WindowFrame"},;
       {COLOR_WINDOWTEXT,   "WindowText"}}

LOCAL aColor := {{YELLOW,   "YELLOW"},;
                 {PINK,     "PINK"},;
                 {RED,      "RED"},;
                 {FUCHSIA,  "FUCHSIA"},;
                 {BROWN,    "BROWN"},;
                 {ORANGE,   "ORANGE"},;
                 {GREEN,    "GREEN"},;
                 {PURPLE,   "PURPLE"},;
                 {BLACK,    "BLACK"},;
                 {WHITE,    "WHITE"},;
                 {GRAY,     "GRAY"},;
                 {BLUE,     "BLUE"}}
   DEFAULT met := 0
   DO CASE
   CASE met == 0
      cColor := AllTrim(cColor)
      IF (pos := Ascan ( aSysColor , {|x|  Upper(x[2]) == Upper(cColor) })) > 0
         nColor := GetSysColor ( aSysColor[pos,1])
      ELSEIF (pos := Ascan ( aColor , {|x|  Upper(x[2]) == Upper(cColor) })) > 0
         nColor := RGB( aColor [pos,1], aColor [ pos,2 ], aColor [ pos,3 ] )
      ELSE
         IF substr(cColor,1,1) == '(' .and. RAt(')',cColor)==Len(cColor)
            aCol:={}
            TOKENINIT (cColor,' ,()')
            do while !TOKENEND()
               cToken := Alltrim( TOKENNEXT (cColor) )
               AAdd( aCol, Val(cToken) )
            enddo
            TOKENEXIT()
            nColor := RGB( aCol [ 1 ], aCol [ 2 ], aCol [ 3 ] )
         ENDIF
      ENDIF
      result := nColor
   CASE met == 1
      result := aSysColor
   CASE met == 2
      result := Ascan ( aSysColor , {|x|  Upper( x[2] ) == Upper( AllTrim(cColor) ) })
   ENDCASE
RETURN result

*------------------------------------------------------------------------------*
FUNCTION SavePropFileXml(ParentForm,ControlName,cFileXml)
*------------------------------------------------------------------------------*
   LOCAL oXmlDoc
   oXmlDoc := CreatePropXml(ParentForm,ControlName)
   IF File(cFileXml)
      IF MsgYesNo ( "Overwrite File :"+cFileXml+" ?" , "Warning" )
         oXmlDoc:Save( cFileXml )
      ENDIF
   ELSE
      oXmlDoc:Save( cFileXml )
   ENDIF
RETURN Nil

*------------------------------------------------------------------------------*
FUNCTION CreatePropXml(ParentForm,ControlName)
*------------------------------------------------------------------------------*
   LOCAL aLev:={}, aNode:={}, aItemRt, hItem, hParentItem
   LOCAL oXmlDoc      := HXMLDoc():new()
   LOCAL oXmlNode, oXmlSubNode, oXmlNodeAkt, aAttr
   LOCAL hWndPG := GetControlHandle (ControlName,ParentForm)

   hItem := PG_GetRoot( hWndPG )
   hParentItem := TreeView_GetParent( hWndPG, hItem )      // Parent Item

   oXmlNode := HXMLNode():New( "ROOT" )
   oXmlNode:SetAttribute( "Title","PropertyGrid" )
   oXmlDoc:add( oXmlNode )
   AAdd(aNode,oXmlNode )
   AAdd(aLev,hParentItem)
   oXmlNodeAkt := oXmlNode
   WHILE hItem != 0
      hParentItem := TreeView_GetParent( hWndPG, hItem )   // Parent Item
      aItemRt := PG_GetItem( hWndPG, hItem, PGI_ALLDATA)
      DO while ATail(aLev) != hParentItem
        aLev := ASize(aLev,Len(aLev)-1)
        aNode := ASize(aNode,Len(aLev))
      ENDDO
      oXmlNodeAkt := ATail(aNode)

      IF IdentTypePG(aItemRt[6]) == "category"
         oXmlSubNode := HXMLNode():New(IdentTypePG(aItemRt[6]), , AttrCreate(aItemRt) )
         oXmlNodeAkt:add( oXmlSubNode )
         oXmlNodeAkt := oXmlSubNode
         AAdd(aNode,oXmlSubNode )
         AAdd(aLev,hItem )
      ELSE
         oXmlSubNode := HXMLNode():New( IdentTypePG(aItemRt[6]), HBXML_TYPE_SINGLE, AttrCreate(aItemRt) )
         oXmlNodeAkt:add( oXmlSubNode )
      ENDIF
      hItem := GetNextItemPG( hWndPG, hItem )
   ENDDO
RETURN oXmlDoc

*------------------------------------------------------------------------------*
FUNCTION GetNextItemPG(hWndPG, hItem)
*------------------------------------------------------------------------------*
   LOCAL TypePG, hItemParent
   TypePG := PG_GetItem( hWndPG, hItem, PGI_TYPE )
   hItemParent := hItem
   hItem := PG_GetNextItem( hWndPG, hItem )
   IF TypePG == PG_FONT .or. TypePG == PG_FLAG .or. TypePG == PG_SIZE
      DO While hItemParent == TreeView_GetParent( hWndPG, hItem )
         hItem := PG_GetNextItem( hWndPG, hItem )
     ENDDO
  ENDIF
RETURN hItem

*------------------------------------------------------------------------------*
FUNCTION  AttrCreate(aItProperty)
*------------------------------------------------------------------------------*
   Local aAttr := {}, lDis, lChd
   Aadd( aAttr, { "Name", AttrTran(aItProperty[1],'C') } )
   Aadd( aAttr, { "Value", AttrTran(aItProperty[2],'C') } )
   Aadd( aAttr, { "cData", AttrTran(aItProperty[3],'C') } )
   Aadd( aAttr, { "disabled", AttrTran(aItProperty[4],'L' ) } )
   Aadd( aAttr, { "changed", AttrTran(aItProperty[5],'L') } )
   Aadd( aAttr, { "ItemID", AttrTran(aItProperty[7],'N') } )
   Aadd( aAttr, { "Info", AttrTran(aItProperty[8],'C') } )
   Aadd( aAttr, { "VarName", AttrTran(aItProperty[9],'C') } )
RETURN aAttr

*------------------------------------------------------------------------------*
FUNCTION AttrTran(xData,type)
*------------------------------------------------------------------------------*
   LOCAL n, cData
   DO CASE
   CASE ValType(xData)== 'U'
      RETURN IF(Type=='L',"false","")
   CASE Type== 'C'
      IF ValType(xData)=='C'
         RETURN xData
      endif
   CASE Type== 'N'
      IF ValType(xData)=='N'
         RETURN AllTrim(Str(xData))
      ENDIF
   CASE Type== 'A'
      IF ValType(xData)=='A'
         cData:=""
         FOR n:=1 TO Len(xData)
            IF valtype(xdata[n]) =='N'
               cData+=AllTrim(Str(xData[n]))+IF(n<Len(xData),';','')
            endif
            IF valtype(xdata[n]) =='C'
               cData+=xData[n]+IF(n<Len(xData),';','')
            endif
           IF valtype(xdata[n]) =='L'
              cData+=IF(xData[n],"true","false")+IF(n<Len(xData),';','')
           endif
         NEXT
         RETURN cData
      ENDIF
   CASE Type== 'L'
      IF ValType(xData)=='L'
         RETURN IF(xData,"true","false")
      endif
   ENDCASE
RETURN ""

*------------------------------------------------------------------------------*
FUNCTION CreatePropFile(ParentForm,ControlName,cFile)
*------------------------------------------------------------------------------*
   LOCAL aLev:={}, aItemRt, hItem, hParentItem
   LOCAL hand, lret := .f.
   LOCAL hWndPG := GetControlHandle (ControlName,ParentForm)

   hand := FCreate( cFile, 0 )
   if FError() == 0
      lret := .t.
      hItem := PG_GetRoot(  hWndPG )
      hParentItem := TreeView_GetParent( hWndPG, hItem )      // Parent Item
      AAdd(aLev,hParentItem)
      WHILE hItem != 0
         aItemRt := PG_GetItem( hWndPG, hItem, PGI_ALLDATA)
         IF IdentTypePG(aItemRt[6]) == "category"
            AAdd(aLev,hItem)
            lret := ItemRt2File(hand, aItemRt)
         ELSE
            lret := ItemRt2File(hand, aItemRt)
         ENDIF
         hItem := GetNextItemPG( hWndPG, hItem )
         hParentItem := TreeView_GetParent( hWndPG, hItem )   // Parent Item
         DO while ATail(aLev) != hParentItem
            aLev := ASize(aLev,Len(aLev)-1)
            lret := ItemRt2File(hand, {"end","","","","",1,0,"",""})
         ENDDO
      ENDDO
      FClose(hand)
   ENDIF
RETURN lret

*------------------------------------------------------------------------------*
FUNCTION CreateIniFile(ParentForm,ControlName,cFile)
*------------------------------------------------------------------------------*
   LOCAL cSection:="", aItemRt, hItem, hParentItem
   LOCAL iItem:=.f., lret := .f.
   LOCAL hWndPG := GetControlHandle (ControlName,ParentForm)
   IF _BeginIni( cFile ) == 0
      lRet := .t.
      hItem := PG_GetRoot(  hWndPG )
      WHILE hItem != 0
         aItemRt := PG_GetItem( hWndPG, hItem, PGI_ALLDATA)
         IF IdentTypePG(aItemRt[6]) == "category"
            cSection := aItemRt[1]
         ELSE
            SET  SECTION cSection ENTRY aItemRt[1]  TO aItemRt[2]
         ENDIF
         hItem := PG_GetNextItem( hWndPG, hItem )
      ENDDO
      _EndIni()
    endif
RETURN lret

*------------------------------------------------------------------------------*
FUNCTION ItemRt2File(hand, aItemRt)
*------------------------------------------------------------------------------*
   LOCAL n, aAttr := AttrCreate(aItemRt)
   LOCAL lin := '"'+IdentTypePG(aItemRt[6])+'" '
   FOR n :=1 TO Len(aAttr)
      IF n==4
         IF aAttr[n,2]=="true"
            lin += '"'+aAttr[n,1]+'" '
         else
            lin += '"" '
         ENDIF
      ELSEIF  n==5
         lin += '"" '
      ELSE
         lin += '"'+aAttr[n,2]+'" '
      ENDIF
   NEXT
   lin += CRLF
   FWrite(hand,lin,len(lin))

RETURN ( FError() == 0 )

*------------------------------------------------------------------------------*
FUNCTION SavePropFile(ParentForm,ControlName,cFile,met)
*------------------------------------------------------------------------------*
LOCAL oXmlDoc, lSave := .t.,cExt
   cExt:= Lower(SubStr(cFile,RAt('.',cFile)+1))
   IF File(cFile)
      IF !MsgYesNo ( "Overwrite File :"+cFile+" ?" , "Warning" )
         lSave := .f.
      ENDIF
   ENDIF
   IF lSave
      DO case
      CASE cExt == 'xml'
         oXmlDoc := CreatePropXml(ParentForm,ControlName)
         oXmlDoc:Save( cFile )
      CASE cExt == 'txt'
         lSave := CreatePropFile(ParentForm,ControlName,cFile)
      CASE cExt == 'ini'
         lSave := CreateIniFile(ParentForm,ControlName,cFile)
      ENDCASE
   ENDIF
RETURN lSave


#define NM_SETFOCUS      -7
#define NM_KILLFOCUS   (-8)
#define LVN_ITEMCHANGED   (-101)
#define NM_DBLCLK   (-3)
#define NM_CUSTOMDRAW   (-12)

*-----------------------------------------------------------------------------*
Function OPROPGRIDEVENTS( hWnd, nMsg, wParam, lParam, hItem, hEdit )
*-----------------------------------------------------------------------------*
   Local i, ItemType, ItHeight, hWndItem, iCheck, cData, aData, cValue
   DO CASE
   CASE nMsg == WM_CHAR
      if wParam == 27
         _PGInitData( hWnd, hEdit, hItem, PG_GETITEM( hWnd, hItem, PGI_TYPE ) )
      endif
   CASE nMsg == WM_LBUTTONUP
     RETURN 0
   CASE nMsg == WM_LBUTTONDOWN
      RETURN 0
   CASE nMsg == WM_LBUTTONDBLCLK
      ItemType := PG_GetItem( hWnd, hItem, PGI_TYPE)
      IF ItemType == PG_CATEG .or. ItemType == PG_FONT .or. ItemType == PG_FLAG .or. ItemType == PG_SIZE
         SendMessage ( hWnd , TVM_EXPAND, TVE_TOGGLE, hItem )
         RETURN 1
      endif
      RETURN 0
   CASE nMsg == WM_COMMAND
      IF  HIWORD(wParam) == EN_CHANGE .and. lParam == hItem
         i := Ascan ( _HMG_aControlHandles , hWnd  )
         If i > 0
            _DoControlEventProcedure ( _HMG_aControlHeadClick [i] , i )
         endif
      endif
      IF  HIWORD(wParam) == BN_CLICKED
         IF (ItemType := PG_GetItem( hWnd, hItem, PGI_TYPE)) == PG_CHECK
            iCheck :=LOWORD(wParam)
            cData  := PG_GETITEM(hWnd,hItem,PGI_DATA)
            IF !Empty(cData)
               aData := IdentData(cData)
               IF Len(aData) >= 2
                  cValue := aData[iCheck]
                  PG_SETDATAITEM(hWnd, hItem, cValue, cData, .f.)
               ELSE
                  cValue := PG_GETITEM(hWnd,hItem,PGI_VALUE)
                  cData := IF(iCheck==2,'true','false')
                  PG_SETDATAITEM(hWnd, hItem, cValue, cData, .t.)
               ENDIF
            ENDIF
         endif
      ENDIF
      IF  HIWORD(wParam) == CBN_KILLFOCUS
         IF (ItemType := PG_GetItem( hWnd, hItem, PGI_TYPE)) == PG_LIST
            cValue := GetWindowText (hEdit )
            cData  := PG_GETITEM(hWnd,hItem,PGI_DATA)
            IF !Empty(cData)
               aData := IdentData(cData)
               IF AScan(aData,cValue) == 0
                  cData := cData+";"+cValue
                  PG_SETDATAITEM(hWnd, hItem, cValue, cData, .t.)
               ENDIF
            ELSE
               PG_SETDATAITEM(hWnd, hItem, cValue, cValue, .t.)
            ENDIF
         endif
      ENDIF
   CASE nMsg == WM_NOTIFY
      i := Ascan ( _HMG_aControlHandles , GetHwndFrom (lParam) )
      If i > 0
            If GetNotifyCode ( lParam ) = TVN_SELCHANGED   //Tree
               _DoControlEventProcedure ( _HMG_aControlChangeProcedure [i] , i )
               Return 0
            EndIf

            * PropGrid Double Click .........................

            If GetNotifyCode ( lParam ) == NM_DBLCLK
               _DoControlEventProcedure ( _HMG_aControlDblClick [i] , i )
               Return 0
            EndIf

            If GetNotifyCode ( lParam ) = -181
               ReDrawWindow ( _hmg_acontrolhandles [i,1] )
            Endif
      Endif
   ENDCASE
Return 1

*------------------------------------------------------------------------------*
FUNCTION aCol2Str(aColor)
*------------------------------------------------------------------------------*
   LOCAL n, cColor:="("
   FOR n:=1 TO 3
      cColor+=AllTrim(Str(aColor[n])+IF(n<3,',',')'))
   NEXT
RETURN cColor

*------------------------------------------------------------------------------*
FUNCTION aFont2Str(aFont)
*------------------------------------------------------------------------------*
   LOCAL n, cValue :="" ,sep:=""
   FOR n:=1 TO Len(aFont)
      IF At(aFont[n,3],'true false' ) != 0
         IF aFont[n,3]  == 'true'
            cValue += ','+aFontName [n]
         ENDIF
      ELSE
         cValue += sep+aFont[n,3]
      ENDIF
      sep:=','
   NEXT
RETURN cValue

*------------------------------------------------------------------------------*
FUNCTION aVal2Str(aData,sep)
*------------------------------------------------------------------------------*
   LOCAL n, cData:=""
   DEFAULT sep := ';'
   IF ValType(aData)=='A'
      FOR n:=1 TO Len(aData)
        IF valtype(adata[n]) =='N'
           cData+=AllTrim(Str(aData[n]))+IF(n<Len(aData),sep,'')
        endif
        IF valtype(adata[n]) =='C'
           cData+=aData[n]+IF(n<Len(aData),sep,'')
        endif
        IF valtype(adata[n]) =='L'
           cData+=IF(aData[n],"true","false")+IF(n<Len(aData),sep,'')
        endif
      NEXT
   ENDIF
RETURN cData

*------------------------------------------------------------------------------*
FUNCTION SetPropGridValue ( ParentForm,ControlName,nID,cValue,cData )
*------------------------------------------------------------------------------*
   LOCAL hItem, RetVal:="",ItemType, lData:=.t.
   LOCAL hWndPG
   DEFAULT nID := 0
   if ValType(ParentForm) == 'U'
      IF _HMG_BeginWindowActive = TRUE
         ParentForm := _HMG_ActiveFormName
      else
         MsgMiniGuiError("Parent Window is not defined. Program terminated" )
      ENDIF
   endif
   hWndPG := GetControlHandle (ControlName,ParentForm)
   IF ValType(cData)=='U'
      lData:=.f.
   endif
   if hWndPG > 0
      IF nId == 0
         hItem := TreeView_GetSelection( hWndPG )
      ELSE
         hItem := PG_SearchID(hWndPG,nID)
      ENDIF
      IF hItem != 0  .and. !PG_GETITEM(hWndPG,hItem,PGI_ENAB)
         ItemType := PG_GETITEM(hWndPG,hItem,PGI_TYPE)
         IF !lData
            cData := PG_GETITEM(hWndPG,hItem,PGI_DATA)
         endif
         IF CheckDataPG(ItemType,@cValue,@cData,1)
            PG_SETDATAITEM(hWndPG, hItem, cValue, cData, lData)
            PG_REDRAWITEM(hWndPG, hItem)
            PostMessage(hWndPG,WM_KEYDOWN,VK_ESCAPE,0)
         endif
      endif
   ENDIF
RETURN retVal

*------------------------------------------------------------------------------*
FUNCTION EnablePropGridItem ( ParentForm,ControlName,nID,lEnabled )
*------------------------------------------------------------------------------*
   LOCAL hItem, RetVal:="",ItemType, lData:=.t.
   LOCAL hWndPG
   DEFAULT nID := 0
   if ValType(ParentForm) == 'U'
      IF _HMG_BeginWindowActive = TRUE
         ParentForm := _HMG_ActiveFormName
      ELSE
         MsgMiniGuiError("Parent Window is not defined. Program terminated" )
      ENDIF
   endif
   hWndPG := GetControlHandle (ControlName,ParentForm)
   IF hWndPG > 0
      IF nId == 0
         hItem := TreeView_GetSelection( hWndPG )
      ELSE
         hItem := PG_SearchID( hWndPG, nID )
      ENDIF
      IF hItem != 0
         IF ( !PG_GETITEM(hWndPG,hItem,PGI_ENAB) .and. lEnabled ) .or. ;
            ( PG_GETITEM(hWndPG,hItem,PGI_ENAB) .and. !lEnabled )
               PG_ENABLEITEM(hWndPG, hItem, lEnabled)
               PG_REDRAWITEM(hWndPG, hItem)
               PostMessage(hWndPG, WM_KEYDOWN, VK_ESCAPE, 0)
         ENDIF
      ENDIF
   ENDIF
RETURN retVal

*------------------------------------------------------------------------------*
FUNCTION RedrawPropGridItem ( ParentForm,ControlName,nID )
*------------------------------------------------------------------------------*
   LOCAL hItem, RetVal:="",ItemType, lData:=.t.
   LOCAL hWndPG
   DEFAULT nID := 0
   if ValType(ParentForm) == 'U'
      IF _HMG_BeginWindowActive = TRUE
         ParentForm := _HMG_ActiveFormName
      ELSE
         MsgMiniGuiError("Parent Window is not defined. Program terminated" )
      ENDIF
   endif
   hWndPG := GetControlHandle (ControlName,ParentForm)
   if hWndPG > 0
      IF nId == 0
         hItem := TreeView_GetSelection( hWndPG )
      ELSE
         hItem := PG_SearchID(hWndPG,nID)
      ENDIF
      IF hItem != 0
         PG_REDRAWITEM(hWndPG, hItem)
      endif
   ENDIF
RETURN Nil

*------------------------------------------------------------------------------*
FUNCTION GetPropGridValue ( ParentForm, ControlName, nID, lAllData, nSubItem )
*------------------------------------------------------------------------------*
   LOCAL hItem, RetVal, ItemType, aData
   LOCAL hWndPG
   DEFAULT nID := 0
   if ValType(ParentForm) == 'U'
      IF _HMG_BeginWindowActive = TRUE
         ParentForm := _HMG_ActiveFormName
      ELSE
         MsgMiniGuiError("Parent Window is not defined. Program terminated" )
      ENDIF
   endif
   hWndPG := GetControlHandle (ControlName,ParentForm)
   IF nId == 0
      hItem := TreeView_GetSelection( hWndPG )
   ELSE
      hItem := PG_SearchID(hWndPG,nID)
   ENDIF
   IF hItem != 0
      IF lAllData
         RetVal := PG_GETITEM(hWndPG,hItem,PGI_ALLDATA)
      ELSE
         ItemType := PG_GETITEM(hWndPG,hItem,PGI_TYPE)
         RetVal := PG_GETITEM(hWndPG,hItem,PGI_VALUE)
         aData := PG_GETITEM(hWndPG,hItem,PGI_DATA)
         RetVal := ValueTran(RetVal,ItemType,aData,nSubItem)
      ENDIF
   ENDIF
RETURN retVal

*------------------------------------------------------------------------------*
FUNCTION ValueTran( cValue, ItType, cData, nSubIt )
*------------------------------------------------------------------------------*
LOCAL n, xData, aData
   DEFAULT nSubIt := 0
   IF ValType(cValue)=='C'
      DO CASE
      CASE ItType == PG_DEFAULT .or. ItType == PG_CATEG .or. ItType == PG_STRING .or. ItType == PG_SYSINFO ;
         .or. ItType == PG_IMAGE .or. ItType == PG_FLAG .or. ItType == PG_ENUM ;
         .or. ItType == PG_FILE .or. ItType == PG_FOLDER .or. ItType == PG_LIST .or. ItType == PG_USERFUN
         xData := cValue
      CASE ItType == PG_INTEGER
         xData := Int(Val(cValue))
      CASE ItType == PG_DOUBLE
         xData := Val(cValue)
      CASE ItType == PG_SYSCOLOR .or. ItType == PG_COLOR
         xData:= IdentColor(0,cValue)
         xData := {GetRed (xData),GetGreen (xData), GetBlue (xData)}
      CASE ItType ==  PG_LOGIC
         xData := IF(RTrim(cValue)=="true",.t.,.f.)
      CASE ItType ==  PG_DATE
         xData := CToD(cValue)
      CASE ItType == PG_FONT
         xData :=IdentDataFont(cValue,',')
         IF nSubIt > 0 .and. nSubIt <= Len(xData)
            xdata:= Val(xData[nSubIt])
         else
            AEval(xData,{|x| x:=IF(x=='true',.t.,.f.)},3)
            ASize(xData,8)
            xData:=AIns(xData,5)
            xData[5]:={0,0,0}
            xData[8]:=0
         endif
      CASE ItType == PG_ARRAY
        xData := IdentData(cValue,',')
      CASE ItType == PG_CHECK
         aData := IdentData(cData)
         IF Len(aData)== 1
            xData := IF(aData[1] == 'true',.t.,.f.)
         else
            IF AScan(aData,cValue)== 1
               xData := .F.
            ELSE
               xData := .T.
            ENDIF
         endif
      CASE ItType == PG_SIZE
         aData :=IdentDataSize(cValue,cData)
         IF nSubIt > 0 .and. nSubIt <= Len(aData)
            xdata:= Val(aData[nSubIt,2])
         endif
      ENDCASE
   ENDIF
RETURN xData

*------------------------------------------------------------------------------*
FUNCTION _LoadProperty(cFile)
*------------------------------------------------------------------------------*
   LOCAL oFile, cLine, aLine, cExt := 'txt'
   LOCAL aProperty := {}
   nItemId := 100
   cExt := Lower(SubStr(cFile, RAt('.', cFile)+1))
   oFile := TFileRead():New( cFile )
   oFile:Open()
   IF !oFile:Error()
      WHILE oFile:MoreToRead()
         cLine := oFile:ReadLine()
         IF ! Empty(cLine)
            IF cExt == 'txt'
               aLine := FormatPropertyLine(cLine)
            ELSE
               aLine := FormatIniLine(cLine)
               IF Len(aProperty) > 0
                  IF aLine[1] == 'category'
                     AAdd(aProperty, {'category', 'end', '', '', .f., .f., 0, ''})
                  ENDIF
               ENDIF
            ENDIF
            AAdd(aProperty, aLine)
         ENDIF
      END WHILE
      oFile:Close()
   ENDIF
RETURN aProperty

*------------------------------------------------------------------------------*
FUNCTION FormatPropertyLine(cString)
*------------------------------------------------------------------------------*
   Local cToken, mod, nPos, lToken, cStr:='',n
   LOCAL aLine:={}
   LOCAL aRowDef := {'string', '', '', '', .f., .f., 0, ''}
   Mod := IF(At('"', cString) > 1, 1, 0)

   TOKENINIT (cString,' '+chr(9)+CHR(13))
   do while (!TOKENEND())
      cToken := TOKENNEXT(cString)
      IF RAt('"', cToken) < Len(cToken)
         lToken := .f.
         cStr += cToken+" "
      ELSE
         lToken := .t.
         cStr += cToken
      ENDIF
      IF lToken
         cStr := REMALL (cStr, '"')
         AAdd(aLine, cStr)
         cStr:=""
      ENDIF
   enddo
   TOKENEXIT()
   IF Len(aLine)< 8
      aLine := ASize(aLine, 8)
   endif
   FOR n :=1 TO 8
      IF aLine[n] == Nil
         IF n == 7
            aLine[n] :=  nItemId++
         else
            aLine[n] := aRowDef[n]
         endif
      ELSE
         DO case
         CASE n == 5 .and. Lower(aLine[5]) == "disabled"
            aLine[5] := .t.
         CASE n == 7 .and. ValType(aLine[7])== 'C'
            aLine[7] := Val(aLine[7])
         ENDCASE
      ENDIF
   NEXT
RETURN aLine

*------------------------------------------------------------------------------*
FUNCTION FormatIniLine(cString)
*------------------------------------------------------------------------------*
   Local cToken, n
   LOCAL aLine:={''}
   LOCAL aRowDef := {'string', '', '', '', .f., .f., 0, ''}
   cString:=LTrim(cString)
   TOKENINIT (cString,'='+chr(9)+CHR(13))
   IF At('[', cString) == 1
      aLine := {'category'}
   ELSE
      aLine := {'string'}
   ENDIF
   cString := CHARREM ("[]", cString)
   DO WHILE (!TOKENEND())
      cToken := TOKENNEXT(cString)
      AAdd(aLine, cToken)
   ENDDO
   TOKENEXIT()
   IF Len(aLine)< 8
      aLine := ASize(aLine,8)
   ENDIF
   FOR n :=1 TO 8
      IF aLine[n] == Nil
         IF n == 7
            aLine[n] := nItemId++
         ELSE
            aLine[n] := aRowDef[n]
            IF n == 5 .and. aLine[1] == "category"
               aLine[n] := .t.
            ENDIF
         ENDIF
      ENDIF
   NEXT
RETURN aLine

*------------------------------------------------------------------------------*
Function OPGEDITEVENTS( hWnd, nMsg, wParam, lParam, hWndPG, hItem )
*------------------------------------------------------------------------------*
LOCAL icp := 0, icpe := 0, i, n, x, ItemType, nColor, Pos
LOCAL hImage, ItHeight, hEdit, hParentItem, hChildItem, ParentItemType, cParentName
LOCAL aData, aDataNew, cData, cDataNew, cValue, cVal, cFltr, lAll, cFold, bData
lParam := Nil //unused parameter

   i := Ascan ( _HMG_aControlhandles , hWndPG )
   IF i == 0
      RETURN 0
   ENDIF
   x := ascan ( _HMG_aFormHandles ,_HMG_aControlParentHandles[i] )
   if x > 0
      cParentName :=  _HMG_aFormNames [ x ]
   endif
   ItHeight := _HMG_aControlRangeMin [i] - 4
   DO CASE
   CASE nMsg == WM_CHAR
      if wParam == 27
         _PGInitData( hWndPG, hWnd, hItem, PG_GETITEM( hWndPG, hItem, PGI_TYPE ) )
         SetFocus(hWndPG)
      Endif
      icp :=  HiWord ( SendMessage( hWnd , EM_GETSEL , 0 , 0 ) )
      icpe := LoWord ( SendMessage( hWnd , EM_GETSEL , 0 , 0 ) )
      cValue := GetWindowText (hWnd )
      // simulate overwrite mode
      if ! IsInsertActive() .and. wParam <> 13 .and. wParam <> 8 //.and. substr( inBuffer, icp + 1, 1 ) <> chr( 13 )

         if IsAlpha( chr( wParam ) ) .or. IsDigit( chr( wParam ) )
            if icp <> icpe
               SendMessage( hWnd , WM_CLEAR , 0 , 0 )
               SendMessage( hWnd , EM_SETSEL , icpe , icpe )
            else
               SendMessage( hWnd , EM_SETSEL , icp , icp + 1 )
               SendMessage( hWnd , WM_CLEAR , 0 , 0 )
               SendMessage( hWnd , EM_SETSEL , icp , icp )
            endif

         else

            if wParam == 1
               SendMessage( hWnd , EM_SETSEL , 0 , -1 )
            endif

         endif

      else
           if wParam == 1
               SendMessage( hWnd , EM_SETSEL , 0 , -1 )
           endif
      endif

   CASE nMsg == WM_COMMAND
      hParentItem := TreeView_GetParent(hWndPG,hItem)      // Parent Item
      hChildItem  := TreeView_GetChild(hWndPG,hParentItem) // First Child Item
      ParentItemType := PG_GETITEM(hWndPG,hParentItem,PGI_TYPE)

      DO CASE
      CASE  HIWORD(wParam) == BN_CLICKED
         DO CASE
         CASE LOWORD(wParam) == PG_COLOR
            cData := PG_GETITEM(hWndPG,hItem,PGI_DATA)
            aData := IdentDataColor( cData )
            aDataNew := GetColor ( aData )
            IF aDataNew[1] != NIL
               cData:= AttrTran(aDataNew,'A')
               cValue  := aCol2Str(aDataNew)
               SetWindowText (hWnd, cValue )
               nColor := IdentColor(0, cValue )
               hImage := CREATECOLORBMP(hWndPG,nColor,ItHeight*1.4 , ItHeight)
               ResetPropGridImageList(hWndPG, hItem,hImage)
            ENDIF
            SetFocus( hEdit )
         CASE LOWORD(wParam) == PG_USERFUN
            cValue := GetWindowText ( hWnd )
            cData := PG_GETITEM(hWndPG,hItem,PGI_DATA)
            bData := &(cData)
            cValue := Eval(bData,cValue)
            IF cValue != NIL .and. ValType(cValue) == 'C'
               SetWindowText (hWnd, cValue )
            ENDIF
            SetFocus( hEdit )
         CASE LOWORD(wParam) == PG_FONT
            cData := PG_GETITEM(hWndPG,hItem,PGI_DATA)
            aData := IdentDataFont( cData )
            aDataNew := GetFont (aData[1,3], Val(aData[2,3]) ,aData[3,3]=="true" , aData[4,3]=="true" , , aData[5,3]=="true" , aData[6,3]=="true" )
            IF !Empty(aDataNew[1])
               ADel(aDataNew,5)
               ASize(aDataNew,6)
               cData:= AttrTran(aDataNew,'A')
               aData :=IdentDataFont(cData)
               cValue  := aFont2Str(aData)
               SetWindowText (hWnd, cValue )
            ENDIF

         CASE LOWORD(wParam) == PG_IMAGE
            cData := GetWindowText ( hWnd )
            cFltr := IdentFileFlt( PG_GETITEM(hWndPG,hItem,PGI_DATA))
            cDataNew := GetFile ( cFltr , "Image File", cData , .f. , .t.)
            IF !Empty( cDataNew )
               cValue := cDataNew
               SetWindowText (hWnd, cValue )
               LoadImagePG(cValue,i,hItem)
            ENDIF
         CASE LOWORD(wParam) == PG_FILE
            cData := GetWindowText ( hWnd )
            cFltr := IdentFileFlt( PG_GETITEM(hWndPG,hItem,PGI_DATA))
            cDataNew := GetFile ( cFltr , "File", cData , .f. , .t.)
            IF !Empty( cDataNew )
               cValue := cDataNew
               SetWindowText (hWnd, cValue )
            ENDIF
         CASE LOWORD(wParam) == PG_FOLDER
            cData := GetWindowText ( hWnd )
            cFold := PG_GETITEM(hWndPG,hItem,PGI_DATA)
            cDataNew := GetFolder ( cFold, cData )
            IF !Empty( cDataNew )
               cValue := cDataNew
               SetWindowText (hWnd, cValue )
            ENDIF
         CASE LOWORD(wParam) == PG_ARRAY
            cValue :=  GetWindowText (hWnd)
            cDataNew :=ArrayDlg( cValue, cParentName )
            IF !Empty( cDataNew )
               SetWindowText (hWnd, cDataNew )
            ENDif

         ENDCASE

      CASE  HIWORD(wParam) == CBN_SELCHANGE
         cValue := ComboGetString(hWnd, ComboGetCursel (hWnd))
      ENDCASE

   CASE  nMsg == WM_KILLFOCUS
      ItemType   := PG_GETITEM(hWndPG,hItem,PGI_TYPE)
      hParentItem := TreeView_GetParent(hWndPG,hItem)      // Parent Item
      hChildItem  := TreeView_GetChild(hWndPG,hParentItem) // First Child Item
      ParentItemType := PG_GETITEM(hWndPG,hParentItem,PGI_TYPE)

      DO CASE
      CASE ItemType == PG_DEFAULT .or. ItemType == PG_CATEG .OR.ItemType == PG_LOGIC .OR. ItemType == PG_ARRAY .or. ItemType == PG_LIST .or.;
         ItemType == PG_STRING .or. ItemType == PG_INTEGER .or. ItemType == PG_DOUBLE .or. ItemType == PG_DATE .or. ItemType == PG_ENUM
         cValue := GetWindowText ( hWnd )
         PG_SETDATAITEM(hWndPG, hItem, cValue, "", .f.)
         DO case
         CASE ParentItemType == PG_FONT
            cValue:=''
            cValue := PG_GETITEM(hWndPG, hChildItem, PGI_VALUE)
            cData  := cValue
            n:=1
            DO WHILE ( hChildItem := TreeView_GetNextSibling(hWndPG, hChildItem) ) > 0
               IF TreeView_GetParent(hWndPG, hChildItem) == hParentItem
                  n++
                  cVal := PG_GETITEM(hWndPG, hChildItem, PGI_VALUE)
                  cData += ';'+cVal
                  IF At(cVal,'true false' ) != 0
                     IF cVal  == 'true'
                        cValue += ','+aFontName [n]
                     ENDIF
                  ELSE
                     cValue += ','+cVal
                  ENDIF
               ENDIF
            ENDDO
            IF !Empty(cValue)
               PG_SETDATAITEM(hWndPG, hParentItem, cValue, cData, .t.)
               PG_REDRAWITEM(hWndPG, hParentItem)
            ENDIF

         CASE ParentItemType == PG_FLAG
            cData := PG_GETITEM(hWndPG, hParentItem, PGI_DATA)
            lAll :=.f.
            IF hItem == hChildItem
               lAll := PG_GETITEM(hWndPG, hItem, PGI_VALUE) == 'true'
               cValue :="["+ PG_GETITEM(hWndPG, hItem, PGI_NAME)
            ELSE
               PG_SETDATAITEM(hWndPG, hChildItem, 'false', "", .f.)
               cValue :="["
            ENDIF
            DO WHILE ( hChildItem := TreeView_GetNextSibling(hWndPG,hChildItem) ) > 0
               IF TreeView_GetParent(hWndPG,hChildItem) == hParentItem
                  IF lAll
                     PG_SETDATAITEM(hWndPG, hChildItem, 'false', "", .f.)
                  ENDIF
                  cVal:=PG_GETITEM(hWndPG, hChildItem, PGI_VALUE)
                  IF cVal  == 'true'
                     cValue += IF(Len(cValue) > 1, ',' ,'')+PG_GETITEM(hWndPG, hChildItem, PGI_NAME)
                  ENDIF
               ENDIF
            ENDDO
            cValue +="]"
            PG_SETDATAITEM(hWndPG, hParentItem, cValue, cData, .t.)
            ReDrawWindow( hWndPG, hParentItem )

         CASE ParentItemType == PG_SIZE
            hParentItem := TreeView_GetParent(hWndPG,hItem)
            hChildItem  := TreeView_GetChild(hWndPG,hParentItem)
            cData  := PG_GETITEM(hWndPG, hParentItem, PGI_DATA)
            cValue := '('+PG_GETITEM(hWndPG, hChildItem, PGI_VALUE)
            DO WHILE ( hChildItem := TreeView_GetNextSibling(hWndPG,hChildItem) ) > 0
               IF TreeView_GetParent(hWndPG,hChildItem) == hParentItem
                  cValue += ','+PG_GETITEM(hWndPG, hChildItem, PGI_VALUE)
               ENDIF
            ENDDO
            cValue += ')'
            IF !Empty(cValue)
               PG_SETDATAITEM(hWndPG, hParentItem, cValue, cData, .t.)
               PG_REDRAWITEM(hWndPG, hParentItem)
            ENDIF
         ENDCASE

      CASE ItemType == PG_FONT
         cValue := GetWindowText( hWnd )
         aData:=IdentDataFont(cValue,',')
         cData:=AttrTran(aData,'A')
         PG_SETDATAITEM(hWndPG, hItem, cValue, cData, .t.)
         IF PG_GETITEM(hWndPG,hItem,PGI_CHG)
            aData:=IdentDataFont(cData,';')
            hChildItem  := TreeView_GetChild(hWndPG,hItem)
            PG_SETDATAITEM(hWndPG, hChildItem, aData[1,3], "FONT", .t.)
            PG_REDRAWITEM(hWndPG, hChildItem)
            DO WHILE ( hChildItem := TreeView_GetNextSibling(hWndPG,hChildItem) ) > 0
               IF TreeView_GetParent(hWndPG,hChildItem) == hItem
                  IF  (pos:= AScan(aData,{|fIt| fIt[2] == PG_GETITEM(hWndPG,hChildItem,PGI_NAME)}))> 0
                     PG_SETDATAITEM(hWndPG, hChildItem, aData[pos,3], "FONT", .t.)
                     PG_REDRAWITEM(hWndPG, hChildItem)
                  ENDIF
               ENDIF
            ENDDO
         endif

      CASE ItemType == PG_LOGIC
         cValue := ComboGetString(hWnd, ComboGetCursel (hWnd))
         PG_SETDATAITEM(hWndPG, hItem, cValue, "", .f.)

      CASE ItemType == PG_IMAGE
         cValue := GetWindowText ( hWnd )
         cData  := PG_GETITEM(hWndPG,hItem,PGI_DATA)
         PG_SETDATAITEM(hWndPG, hItem, cValue, cData, .t.)
         LoadImagePG(cValue,i,hItem)

      CASE ItemType == PG_FILE .or. ItemType == PG_ENUM .or. ItemType == PG_FOLDER
         cValue := GetWindowText ( hWnd )
         cData  := PG_GETITEM(hWndPG,hItem,PGI_DATA)
         PG_SETDATAITEM(hWndPG, hItem, cValue, cData, .t.)

      CASE ItemType == PG_USERFUN
         cValue := GetWindowText ( hWnd )
         cData  := PG_GETITEM(hWndPG,hItem,PGI_DATA)
         PG_SETDATAITEM(hWndPG, hItem, cValue, cData, .f.)

      CASE ItemType == PG_LIST
         cValue := GetWindowText ( hWnd )
         cData  := PG_GETITEM(hWndPG,hItem,PGI_DATA)
         aData := IdentData(cData)
         IF AScan(aData,cValue) == 0
            cData := cData+";"+cValue
            PG_SETDATAITEM(hWnd, hItem, cValue, cData, .t.)
         ENDIF

      CASE ItemType == PG_SYSCOLOR
         cValue := ComboGetString(hWnd, ComboGetCursel ( hWnd ))
         nColor := IdentColor(0, cValue)
         hImage := CREATECOLORBMP(hWndPG, nColor, ItHeight*1.4, ItHeight)
         ResetPropGridImageList(hWndPG, hItem, hImage)
         PG_SETDATAITEM(hWndPG, hItem, cValue, "", .f.)

      CASE ItemType == PG_COLOR
         cValue := GetWindowText ( hWnd )
         aData  := IdentDataColor(cValue,',')
         cValue := aCol2Str(aData)
         cData  := aVal2Str(aData)
         nColor := IdentColor(0, cValue )
         hImage := CREATECOLORBMP(hWndPG, nColor, ItHeight*1.4, ItHeight)
         ResetPropGridImageList(hWndPG, hItem, hImage)
         PG_SETDATAITEM(hWndPG, hItem, cValue, cData, .t.)
      ENDCASE

   ENDCASE

Return 0

*------------------------------------------------------------------------------*
Function _PGInitData( hWnd, hEdit, hWndItem, ItemType )
*------------------------------------------------------------------------------*
LOCAL i, x, n, aSysColor, nColor, hIlist, hImage, aISize, ItHeight, aData, hParentItem
   i := Ascan ( _HMG_aControlhandles , hWnd )
   IF i > 0
      ItHeight := _HMG_aControlRangeMin [i] - 4
      DO CASE
      CASE ItemType == PG_DEFAULT .or. ItemType == PG_CATEG .or. ItemType == PG_ARRAY .or.;
                   ItemType == PG_STRING .or. ItemType == PG_INTEGER .or. ItemType == PG_DOUBLE .or.;
                   ItemType == PG_SYSINFO .or. ItemType == PG_USERFUN
         SetWindowText (hEdit, PG_GETITEM(hWnd,hWndItem,PGI_VALUE))
         SetFocus( hEdit )
      CASE ItemType == PG_LOGIC
         ComboBoxReset(hEdit)
         ComboAddString (hEdit, "true" )
         ComboAddString (hEdit, "false" )
         ComboSetCurSel (hEdit, IF( Lower(PG_GETITEM(hWnd,hWndItem,PGI_VALUE))=="true",1,2))
      CASE ItemType == PG_ENUM .or. ItemType == PG_LIST
         hParentItem := TreeView_GetParent(hWnd,hWndItem)      // Parent Item
         SetWindowText (hEdit, PG_GETITEM(hWnd,hWndItem,PGI_VALUE) )
         ComboBoxReset(hEdit)
         IF PG_GETITEM(hWnd,hParentItem,PGI_TYPE) == PG_FONT
            PG_GetFonts(hEdit)
            n:=FindComboInexact(hEdit,PG_GETITEM(hWnd,hWndItem,PGI_VALUE))
            ComboSetCurSel (hEdit,n)
         ELSE
            aData := IdentData(PG_GETITEM(hWnd,hWndItem,PGI_DATA))
            FOR n:=1 TO Len(aData)
               PGCOMBOADDSTRING(hEdit,aData[n],0 )
            NEXT
            ComboSetCurSel (hEdit, AScan(aData,PG_GETITEM(hWnd,hWndItem,PGI_VALUE)))
         ENDIF
      CASE ItemType == PG_FONT .or. ItemType == PG_FLAG .or. ItemType == PG_SIZE
         SetWindowText (hEdit, PG_GETITEM(hWnd,hWndItem,PGI_VALUE) )
      CASE ItemType == PG_COLOR  .or. ItemType == PG_IMAGE .or. ItemType == PG_FILE .or. ItemType == PG_FOLDER
         SetWindowText (hEdit, PG_GETITEM(hWnd,hWndItem,PGI_VALUE) )
      CASE ItemType == PG_SYSCOLOR
         aSysColor := IdentColor(1)
         ComboBoxReset(hEdit)
         IF hIListSys == 0
            hIListSys := InitImageList (ItHeight*1.4 , ItHeight, .f., 0 )
            FOR n:=1 TO Len(aSysColor)
               nColor := GetSysColor ( aSysColor[n,1])
               hImage := CREATECOLORBMP(hWnd,nColor, ItHeight*1.4 , ItHeight)
               IL_AddIndirect( hIListSys , hImage , ItHeight*1.4 , ItHeight , 1 )
            NEXT
         ENDIF
         FOR n:=1 TO Len( aSysColor )
            PGCOMBOADDSTRING( hEdit,aSysColor[n,2 ],hIListSys )
         NEXT
         ComboSetCurSel (hEdit, AScan(aSysColor, {|colIt| colIt[2] == PG_GETITEM(hWnd, hWndItem, PGI_VALUE)}))
      ENDCASE
   ENDIF
RETURN Nil

Function ArrayDlg( cArr, FormName )

   Local oDlg, oFont,aItem,  aItemOld, aPos, nRow, nCol, ;
         cData := "", ;
         cNewArr:="",;
         lOk     := .F.
   IF _IsWIndowDefined ( FormName )
      aPos := GetCursorPos()
      nRow := IF( aPos[1]+300 >  getdesktopheight(), aPos[1]-300, aPos[1])
      nCol := IF( aPos[2]+270 > getdesktopwidth(), aPos[2]-270, aPos[2])

      aItem:= IdentData(cArr,',')
      aItemOld:=AClone(aItem)
      DEFINE FONT Font_7 FONTNAME "Arial" SIZE 9

      DEFINE DIALOG Dlg_1 OF &FormName  AT nRow, nCol WIDTH 270 HEIGHT 300 FONT "Font_7" ;
         CAPTION "Array Property" MODAL;
         DIALOGPROC  DialogFun(@lOk, @aItem, aItemOld ) ;
         ON INIT SetInitItem(aItem,0)

         @ 10, 10 TEXTBOX tbox_1 VALUE cData HEIGHT 24 WIDTH 150   ID 101
         @ 10, 170 BUTTON BtnAdd ID 110 CAPTION "&Add"  WIDTH 90 HEIGHT 24

         @ 40, 10 FRAME Frm   ID 100  WIDTH 250  HEIGHT 0

         @ 50 ,10  LISTBOX Lst_1 ID 102 WIDTH 150 HEIGHT 170  ITEMS aItem

         @ 50, 170 BUTTON BtnRem ID 111 CAPTION "&Remove" WIDTH 90 HEIGHT 24
         @ 80, 170 BUTTON BtnUp  ID 112 CAPTION "&Up"     WIDTH 90 HEIGHT 24
         @ 110,170 BUTTON BtnDwn ID 113 CAPTION "&Down"   WIDTH 90 HEIGHT 24


         @ 230,  10 BUTTON Btn1 ID 105 CAPTION "&Accept" WIDTH 70 HEIGHT 24 DEFAULT
         @ 230,  90 BUTTON Btn2 ID 106 CAPTION "&Cancel" WIDTH 70 HEIGHT 24
         @ 230, 170 BUTTON Btn3 ID 107 CAPTION "&Exit"   WIDTH 70 HEIGHT 24

      END DIALOG

      RELEASE FONT Font_7
      IF lOk
         cNewArr := aVal2Str(aItem,',')
      ENDIF
   ENDIF
Return cNewArr

Static Function DialogFun(lOk, aItem,aItemOld)
   Local   ret := 0, cValue, pos, hListBox
   if DLG_ID != Nil
        hListBox:=Getdialogitemhandle(DLG_HWND, 102 )
        do case
        case DLG_ID == 101 .and. DLG_NOT ==1024
            cValue := GetEditText (DLG_HWND, 101 )
            if !empty(cValue)
                EnableDialogItem (DLG_HWND, 110)
            else
                DisableDialogItem (DLG_HWND, 110)
            endif
         case DLG_ID == 110 .and. DLG_NOT == 0
            cValue := GetEditText (DLG_HWND, 101 )
            EnableDialogItem (DLG_HWND, 105)
            SetDialogItemText(DLG_HWND, 101,"" )
            AAdd( aItem,cValue)
            ListboxAddString (hListBox,cValue)
            DisableDialogItem (DLG_HWND, 110)

         case DLG_ID == 111 .and. DLG_NOT == 0
            pos:= ListBoxGetCurSel( hListBox )
            IF Pos > 0 .and. pos <= Len(aItem)
               ADel(aItem,pos)
               ASize(aItem,Len(aItem)-1 )
               ListBoxReset( hListBox )
               SetInitItem(aItem,1)
               EnableDialogItem (DLG_HWND, 105)
            endif
         case DLG_ID == 112 .and. DLG_NOT == 0
            pos:= ListBoxGetCurSel( hListBox )
            IF Pos > 1 .and. pos <= Len(aItem)
               cValue:=aItem[pos]
               ADel(aItem,pos)
               AIns(aItem,pos-1)
               aItem[pos-1]:=cValue
               ListBoxReset( hListBox )
               SetInitItem(aItem,1)
               ListBoxSetCurSel( hListBox,pos-1 )
               EnableDialogItem (DLG_HWND, 105)
            endif
         case DLG_ID == 113 .and. DLG_NOT == 0
            pos:= ListBoxGetCurSel( hListBox )
            IF Pos > 0 .and. pos <= Len(aItem)-1
               cValue:=aItem[pos]
               ADel(aItem,pos)
               AIns(aItem,pos+1)
               aItem[pos+1]:=cValue
               ListBoxReset( hListBox )
               SetInitItem(aItem,1)
               ListBoxSetCurSel( hListBox,pos+1 )
               EnableDialogItem (DLG_HWND, 105)
            endif
        case DLG_ID == 105 .and. DLG_NOT == 0
            ret := GetEditText (DLG_HWND, 101 )
            lOk := .t.
            _ReleaseDialog ( )
        case DLG_ID == 106 .and. DLG_NOT == 0
             ListBoxReset( hListBox )
             aItem :=AClone(aItemOld)
             SetInitItem(aItem,1)
//             DisableDialogItem (DLG_HWND, 105)
             SetDialogItemText(DLG_HWND, 101,"" )
        case DLG_ID == 107 .and. DLG_NOT == 0
            _ReleaseDialog ( )
        endcase
    endif
Return ret

Static Function SetInitItem(aItem, met)
   LOCAL hListBox, i
   IF met == 0
//      DisableDialogItem (DLG_HWND, 105)
      DisableDialogItem (DLG_HWND, 110)
   endif
   hListBox:=Getdialogitemhandle(DLG_HWND, 102 )
   for i = 1 to len (aItem)
      ListboxAddString(hListBox, aItem[i])
   next i
Return Nil

