/*----------------------------------------------------------------------------
 MINIGUI - Harbour Win32 GUI library source code

 Copyright 2002-2008 Roberto Lopez <harbourminigui@gmail.com>
 http://harbourminigui.googlepages.com/

 BTNTEXTBOX control source code
 (C)2006 Janusz Pora <januszpora@onet.eu>

 This program is free software; you can redistribute it and/or modify it under 
 the terms of the GNU General Public License as published by the Free Software 
 Foundation; either version 2 of the License, or (at your option) any later 
 version. 

 This program is distributed in the hope that it will be useful, but WITHOUT 
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS 
 FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

 You should have received a copy of the GNU General Public License along with 
 this software; see the file COPYING. If not, write to the Free Software 
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA (or 
 visit the web site http://www.gnu.org/).

 As a special exception, you have permission for additional uses of the text 
 contained in this release of Harbour Minigui.

 The exception is that, if you link the Harbour Minigui library with other 
 files to produce an executable, this does not by itself cause the resulting 
 executable to be covered by the GNU General Public License.
 Your use of that executable is in no way restricted on account of linking the 
 Harbour-Minigui library code into it.

 Parts of this project are based upon:

	"Harbour GUI framework for Win32"
 	Copyright 2001 Alexander S.Kresin <alex@belacy.belgorod.su>
 	Copyright 2001 Antonio Linares <alinares@fivetech.com>
	www - http://www.harbour-project.org

	"Harbour Project"
	Copyright 1999-2008, http://www.harbour-project.org/

	"WHAT32"
	Copyright 2002 AJ Wos <andrwos@aust1.net> 

	"HWGUI"
  	Copyright 2001-2007 Alexander S.Kresin <alex@belacy.belgorod.su>

---------------------------------------------------------------------------*/

#define _WIN32_IE 0x0500
#define HB_OS_WIN_32_USED
#define _WIN32_WINNT 0x0400
#include <shlobj.h>

#include <windows.h>
#include <commctrl.h>
#include "hbapi.h"
#include "hbvm.h"
#include "hbstack.h"
#include "hbapiitm.h"

#ifdef MAKELONG
#undef MAKELONG
#endif
#define MAKELONG( a, b )   ( (LONG) (((WORD) ((DWORD_PTR) (a) & 0xffff)) | (((DWORD) ((WORD) ((DWORD_PTR) (b) & 0xffff))) << 16)) )

LRESULT CALLBACK  OwnBtnTextProc( HWND hbutton, UINT msg, WPARAM wParam, LPARAM lParam );

typedef struct
{
   UINT     uCmdId;                    // sent in a WM_COMMAND message
   UINT     uCmdId2;                   // sent in a WM_COMMAND message
   UINT     fButtonDown;               // is the button2 up/down?
   UINT     fButtonDown2;              // is the button up/down?
   BOOL     fButton2;                  // is the button2 ?
   BOOL     fMouseDown;                // is the mouse activating the button?
   BOOL     fMouseDown2;               // is the mouse activating the button2?
   WNDPROC  oldproc;                   // need to remember the old window procedure
   int      cxLeftEdge, cxRightEdge;   // size of the current window borders.
   int      cyTopEdge, cyBottomEdge;   // given these, we know where to insert our button
   int      cxLeftEdge2, cxRightEdge2; // size of the current window borders.
   int      cyTopEdge2, cyBottomEdge2; // given these, we know where to insert our button2
   int      uState;
   int      uState2;
   int      cxButton;
   int      cxButton2;
   int      nButton;
   BOOL     fMouseActive;
   HWND     himage;
   HWND     himage2;
} INSBTN, *PINSBTN;

BOOL InsertButton( HWND hwnd, HWND image, int BtnWidth, HWND image2, int BtnWidth2, BOOL fBtn2 )
{
   INSBTN   *pbtn;

   pbtn = HeapAlloc( GetProcessHeap(), 0, sizeof(INSBTN) );

   if( !pbtn )
   {
      return FALSE;
   }

   pbtn->uCmdId = 0;
   pbtn->uCmdId2 = 1;
   pbtn->fButtonDown = FALSE;
   pbtn->fButtonDown2 = FALSE;
   pbtn->fButton2 = fBtn2;
   pbtn->himage = image;
   pbtn->himage2 = image2;
   pbtn->cxButton = ( BtnWidth >= GetSystemMetrics(SM_CXVSCROLL) ? BtnWidth : GetSystemMetrics(SM_CXVSCROLL) );
   pbtn->cxButton2 = ( BtnWidth2 >= GetSystemMetrics(SM_CXVSCROLL) ? BtnWidth2 : GetSystemMetrics(SM_CXVSCROLL) );

   // replace the old window procedure with our new one

   pbtn->oldproc = ( WNDPROC ) SetWindowLong( hwnd, GWL_WNDPROC, (LONG) OwnBtnTextProc );

   // associate our button state structure with the window

   SetWindowLong( hwnd, GWL_USERDATA, (LONG) pbtn );

   // force the edit control to update its non-client area

   SetWindowPos( hwnd, 0, 0, 0, 0, 0, SWP_FRAMECHANGED | SWP_NOMOVE | SWP_NOSIZE | SWP_NOACTIVATE | SWP_NOZORDER );

   return TRUE;
}

// retrieve the coordinates of an inserted button, given the
// specified window rectangle.

void GetButtonRect( INSBTN *pbtn, RECT *rect, INT nBtn )
{
   rect->right -= ( nBtn == 1 ? pbtn->cxRightEdge : pbtn->cxRightEdge2 );
   rect->top += ( nBtn == 1 ? pbtn->cyTopEdge : pbtn->cyTopEdge2 );
   rect->bottom -= ( nBtn == 1 ? pbtn->cyBottomEdge : pbtn->cyBottomEdge2 );
   rect->left = rect->right - ( nBtn == 1 ? pbtn->cxButton : pbtn->cxButton2 );

   // take into account any scrollbars in the edit control

   if( nBtn == 1 )
   {
      if( pbtn->cxRightEdge > pbtn->cxLeftEdge )
      {
         OffsetRect( rect, pbtn->cxRightEdge - pbtn->cxLeftEdge, 0 );
      }
   }
   else
   {
      if( pbtn->cxRightEdge2 > pbtn->cxLeftEdge2 )
      {
         OffsetRect( rect, pbtn->cxRightEdge2 - pbtn->cxLeftEdge2, 0 );
      }
   }
}

HB_FUNC( INITBTNTEXTBOX )
{
   HWND  hwnd;          // Handle of the parent window/form.
   HWND  hedit;         // Handle of the child window/control.
   int   iStyle;        // TEXTBOX window base style.
   HWND  himage, himage2;
   BOOL  fBtn2;
   int   BtnWidth = ( ISNIL(18) ? 0 : ( int ) hb_parni(18) );

   // Get the handle of the parent window/form.

   hwnd = ( HWND ) hb_parnl( 1 );

   iStyle = WS_CHILD | ES_AUTOHSCROLL | BS_FLAT;

   if( hb_parl(12) )    // if <lNumeric> is TRUE, then ES_NUMBER style is added.
   {
      iStyle = iStyle | ES_NUMBER;

      // Set to a numeric TEXTBOX, so don't worry about other "textual" styles.

   }
   else
   {
      if( hb_parl(10) ) // if <lUpper> is TRUE, then ES_UPPERCASE style is added.
      {
         iStyle = iStyle | ES_UPPERCASE;
      }

      if( hb_parl(11) ) // if <lLower> is TRUE, then ES_LOWERCASE style is added.
      {
         iStyle = iStyle | ES_LOWERCASE;
      }
   }

   if( hb_parl(13) )    // if <lPassword> is TRUE, then ES_PASSWORD style is added.
   {
      iStyle = iStyle | ES_PASSWORD;
   }

   if( hb_parl(14) )
   {
      iStyle = iStyle | ES_RIGHT;
   }

   if( !hb_parl(15) )
   {
      iStyle = iStyle | WS_VISIBLE;
   }

   if( !hb_parl(16) )
   {
      iStyle = iStyle | WS_TABSTOP;
   }

   // Creates the child control.

   hedit = CreateWindowEx
      (
         WS_EX_CLIENTEDGE,
         "EDIT",
         "",
         iStyle,
         hb_parni(3),
         hb_parni(4),
         hb_parni(5),
         hb_parni(6),
         hwnd,
         (HMENU) hb_parni(2),
         GetModuleHandle(NULL),
         NULL
      );

   SendMessage( hedit, (UINT) EM_LIMITTEXT, (WPARAM) hb_parni(9), (LPARAM) 0 );
   if( !(hb_parc(17) == NULL) )
   {
      himage = ( HWND ) LoadImage( GetModuleHandle(0), hb_parc(17), IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE | LR_LOADMAP3DCOLORS | LR_LOADTRANSPARENT );

      if( himage == NULL )
      {
         himage = ( HWND ) LoadImage( GetModuleHandle(NULL), hb_parc(17), IMAGE_BITMAP, 0, 0, LR_LOADMAP3DCOLORS | LR_LOADTRANSPARENT );
      }
   }
   else
   {
      himage = NULL;
   }

   if( !(hb_parc(19) == NULL) )
   {
      himage2 = ( HWND ) LoadImage( GetModuleHandle(0), hb_parc(19), IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE | LR_LOADMAP3DCOLORS | LR_LOADTRANSPARENT );

      if( himage2 == NULL )
      {
         himage2 = ( HWND ) LoadImage( GetModuleHandle(NULL), hb_parc(19), IMAGE_BITMAP, 0, 0, LR_LOADMAP3DCOLORS | LR_LOADTRANSPARENT );
      }
   }
   else
   {
      himage2 = NULL;
   }

   fBtn2 = hb_parl( 20 );

   InsertButton( hedit, himage, BtnWidth, himage2, BtnWidth, fBtn2 );

   hb_retnl( (LONG) hedit );
}

HB_FUNC( REDEFBTNTEXTBOX )
{
   HWND  himage, himage2;
   int   BtnWidth = ( ISNIL(3) ? 0 : ( int ) hb_parni(3) );

   if( !(hb_parc(2) == NULL) )
   {
      himage = ( HWND ) LoadImage( GetModuleHandle(0), hb_parc(2), IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE | LR_LOADMAP3DCOLORS | LR_LOADTRANSPARENT );

      if( himage == NULL )
      {
         himage = ( HWND ) LoadImage( GetModuleHandle(NULL), hb_parc(2), IMAGE_BITMAP, 0, 0, LR_LOADMAP3DCOLORS | LR_LOADTRANSPARENT );
      }
   }
   else
   {
      himage = NULL;
   }

   if( !(hb_parc(4) == NULL) )
   {
      himage2 = ( HWND ) LoadImage( GetModuleHandle(0), hb_parc(4), IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE | LR_LOADMAP3DCOLORS | LR_LOADTRANSPARENT );

      if( himage2 == NULL )
      {
         himage2 = ( HWND ) LoadImage( GetModuleHandle(NULL), hb_parc(4), IMAGE_BITMAP, 0, 0, LR_LOADMAP3DCOLORS | LR_LOADTRANSPARENT );
      }
   }
   else
   {
      himage2 = NULL;
   }

   InsertButton( (HWND) hb_parnl(1), himage, BtnWidth, himage2, BtnWidth, hb_parl(5) );
}

void DrawInsertedButton( HWND hwnd, INSBTN *pbtn, RECT *prect, INT nBtn )
{
   HDC   hdc;
   HWND  hBitmap = ( nBtn == 1 ? pbtn->himage : pbtn->himage2 );
   BOOL  fBtnDown = ( nBtn == 1 ? pbtn->fButtonDown : pbtn->fButtonDown2 );
   hdc = GetWindowDC( hwnd );

   // now draw our inserted button:

   if( fBtnDown == TRUE )
   {
      // draw a 3d-edge around the button.

      DrawEdge( hdc, prect, EDGE_RAISED, BF_RECT | BF_FLAT | BF_ADJUST );

      // fill the inside of the button

      FillRect( hdc, prect, GetSysColorBrush(COLOR_BTNFACE) );

      OffsetRect( prect, 1, 1 );
   }
   else
   {
      DrawEdge( hdc, prect, EDGE_RAISED, BF_RECT | BF_ADJUST );

      // fill the inside of the button

      FillRect( hdc, prect, GetSysColorBrush(COLOR_BTNFACE) );
   }

   if( hBitmap == NULL )
   {
      SetBkMode( hdc, TRANSPARENT );
      DrawText( hdc, "...", 3, prect, DT_CENTER | DT_VCENTER | DT_SINGLELINE );
   }
   else
   {
      LONG     wRow = prect->top;
      LONG     wCol = prect->left;
      LONG     wWidth = prect->right - prect->left;
      LONG     wHeight = prect->bottom - prect->top;

      HDC      hDCmem = CreateCompatibleDC( hdc );
      BITMAP   bitmap;
      DWORD    dwRaster = SRCCOPY;

      SelectObject( hDCmem, hBitmap );
      GetObject( hBitmap, sizeof(BITMAP), (LPVOID) & bitmap );
      if( wWidth && (wWidth != bitmap.bmWidth || wHeight != bitmap.bmHeight) )
      {
         StretchBlt( hdc, wCol, wRow, wWidth, wHeight, hDCmem, 0, 0, bitmap.bmWidth, bitmap.bmHeight, dwRaster );
      }
      else
      {
         BitBlt( hdc, wCol, wRow, bitmap.bmWidth, bitmap.bmHeight, hDCmem, 0, 0, dwRaster );
      }

      DeleteDC( hDCmem );
   }

   ReleaseDC( hwnd, hdc );
}

void DrawInsertedButton2( HWND hwnd, INSBTN *pbtn, RECT *prect )
{
   HDC   hdc;
   HWND  hBitmap = pbtn->himage2;

   hdc = GetWindowDC( hwnd );

   // now draw our inserted button:

   if( pbtn->fButtonDown2 == TRUE )
   {
      // draw a 3d-edge around the button.

      DrawEdge( hdc, prect, EDGE_RAISED, BF_RECT | BF_FLAT | BF_ADJUST );

      // fill the inside of the button

      FillRect( hdc, prect, GetSysColorBrush(COLOR_BTNFACE) );

      OffsetRect( prect, 1, 1 );
   }
   else
   {
      DrawEdge( hdc, prect, EDGE_RAISED, BF_RECT | BF_ADJUST );

      // fill the inside of the button

      FillRect( hdc, prect, GetSysColorBrush(COLOR_BTNFACE) );
   }

   if( hBitmap == NULL )
   {
      SetBkMode( hdc, TRANSPARENT );
      DrawText( hdc, "...", 3, prect, DT_CENTER | DT_VCENTER | DT_SINGLELINE );
   }
   else
   {
      LONG     wRow = prect->top;
      LONG     wCol = prect->left;
      LONG     wWidth = prect->right - prect->left;
      LONG     wHeight = prect->bottom - prect->top;

      HDC      hDCmem = CreateCompatibleDC( hdc );
      BITMAP   bitmap;
      DWORD    dwRaster = SRCCOPY;

      SelectObject( hDCmem, hBitmap );
      GetObject( hBitmap, sizeof(BITMAP), (LPVOID) & bitmap );
      if( wWidth && (wWidth != bitmap.bmWidth || wHeight != bitmap.bmHeight) )
      {
         StretchBlt( hdc, wCol, wRow, wWidth, wHeight, hDCmem, 0, 0, bitmap.bmWidth, bitmap.bmHeight, dwRaster );
      }
      else
      {
         BitBlt( hdc, wCol, wRow, bitmap.bmWidth, bitmap.bmHeight, hDCmem, 0, 0, dwRaster );
      }

      DeleteDC( hDCmem );
   }

   ReleaseDC( hwnd, hdc );
}

LRESULT CALLBACK OwnBtnTextProc( HWND hwnd, UINT Msg, WPARAM wParam, LPARAM lParam )
{
   WNDPROC  OldWndProc;
   RECT     *prect;
   RECT     oldrect;
   RECT     rect;
   POINT    pt;
   UINT     oldstate, oldstate2;

   // get the button state structure

   INSBTN   *pbtn = ( INSBTN * ) GetWindowLong( hwnd, GWL_USERDATA );
   OldWndProc = pbtn->oldproc;

   switch( Msg )
   {
      case WM_NCDESTROY:
         OldWndProc = pbtn->oldproc;
         HeapFree( GetProcessHeap(), 0, pbtn );
         return CallWindowProc( OldWndProc, hwnd, Msg, wParam, lParam );

      case WM_NCCALCSIZE:
         prect = ( RECT * ) lParam;
         oldrect = *prect;

         // let the old wndproc allocate space for the borders,
         // or any other non-client space.

         CallWindowProc( pbtn->oldproc, hwnd, Msg, wParam, lParam );

         // calculate what the size of each window border is,
         // we need to know where the button is going to live.

         pbtn->cxLeftEdge = prect->left - oldrect.left;
         pbtn->cxRightEdge = oldrect.right - prect->right;
         pbtn->cyTopEdge = prect->top - oldrect.top;
         pbtn->cyBottomEdge = oldrect.bottom - prect->bottom;

         pbtn->cxLeftEdge2 = pbtn->cxLeftEdge + pbtn->cxButton2;
         pbtn->cxRightEdge2 = pbtn->cxRightEdge + pbtn->cxButton2;
         pbtn->cyTopEdge2 = pbtn->cyTopEdge;
         pbtn->cyBottomEdge2 = pbtn->cyBottomEdge;

         // now we can allocate additional space by deflating the
         // rectangle even further. Our button will go on the right-hand side,
         // and will be the same width as a scrollbar button

         prect->right -= pbtn->cxButton;
         if( pbtn->fButton2 )
         {
            prect->right -= pbtn->cxButton2;
         }

         return 0;

      case WM_NCPAINT:
         // let the old window procedure draw the borders / other non-client
         // bits-and-pieces for us.

         CallWindowProc( pbtn->oldproc, hwnd, Msg, wParam, lParam );

         // get the screen coordinates of the window.
         // adjust the coordinates so they start from 0,0

         GetWindowRect( hwnd, &rect );
         OffsetRect( &rect, -rect.left, -rect.top );

         // work out where to draw the button

         GetButtonRect( pbtn, &rect, 1 );

         DrawInsertedButton( hwnd, pbtn, &rect, 1 );

         if( pbtn->fButton2 )
         {
            GetWindowRect( hwnd, &rect );
            OffsetRect( &rect, -rect.left, -rect.top );

            GetButtonRect( pbtn, &rect, 2 );
            DrawInsertedButton( hwnd, pbtn, &rect, 2 );
         }

         // that's it! This is too easy!

         return 0;

      case WM_NCHITTEST:
         // get the screen coordinates of the mouse

         pt.x = LOWORD( lParam );
         pt.y = HIWORD( lParam );

         // get the position of the inserted button

         GetWindowRect( hwnd, &rect );
         GetButtonRect( pbtn, &rect, 1 );

         // check that the mouse is within the inserted button

         if( PtInRect(&rect, pt) )
         {
            return HTBORDER;
         }
         else
         {
            if( pbtn->fButton2 )
            {
               GetButtonRect( pbtn, &rect, 2 );

               // check that the mouse is within the inserted button

               if( PtInRect(&rect, pt) )
               {
                  return HTBORDER;
               }
               else
               {
                  break;
               }
            }
            else
            {
               break;
            }
         }

      case WM_NCLBUTTONDBLCLK:
      case WM_NCLBUTTONDOWN:
         // get the screen coordinates of the mouse

         pt.x = LOWORD( lParam );
         pt.y = HIWORD( lParam );

         // get the position of the inserted button

         GetWindowRect( hwnd, &rect );
         pt.x -= rect.left;
         pt.y -= rect.top;
         OffsetRect( &rect, -rect.left, -rect.top );
         GetButtonRect( pbtn, &rect, 1 );

         // check that the mouse is within the inserted button

         if( PtInRect(&rect, pt) )
         {
            SetCapture( hwnd );

            pbtn->fButtonDown = TRUE;
            pbtn->fMouseDown = TRUE;

            //redraw the non-client area to reflect the change

            DrawInsertedButton( hwnd, pbtn, &rect, 1 );
         }

         if( pbtn->fButton2 )
         {
            pt.x = LOWORD( lParam );
            pt.y = HIWORD( lParam );

            GetWindowRect( hwnd, &rect );
            pt.x -= rect.left;
            pt.y -= rect.top;
            OffsetRect( &rect, -rect.left, -rect.top );
            GetButtonRect( pbtn, &rect, 2 );

            // check that the mouse is within the inserted button

            if( PtInRect(&rect, pt) )
            {
               SetCapture( hwnd );

               pbtn->fButtonDown2 = TRUE;
               pbtn->fMouseDown2 = TRUE;

               //redraw the non-client area to reflect the change

               DrawInsertedButton( hwnd, pbtn, &rect, 2 );
            }
         }
         break;

      case WM_MOUSEMOVE:
         if( pbtn->fMouseDown == TRUE )
         {
            // get the SCREEN coordinates of the mouse

            pt.x = LOWORD( lParam );
            pt.y = HIWORD( lParam );
            ClientToScreen( hwnd, &pt );

            // get the position of the inserted button

            GetWindowRect( hwnd, &rect );

            pt.x -= rect.left;
            pt.y -= rect.top;
            OffsetRect( &rect, -rect.left, -rect.top );

            GetButtonRect( pbtn, &rect, 1 );

            oldstate = pbtn->fButtonDown;

            // check that the mouse is within the inserted button

            if( PtInRect(&rect, pt) )
            {
               pbtn->fButtonDown = 1;
            }
            else
            {
               pbtn->fButtonDown = 0;
            }

            // redraw the non-client area to reflect the change.
            // to prevent flicker, we only redraw the button if its state
            // has changed

            if( oldstate != pbtn->fButtonDown )
            {
               DrawInsertedButton( hwnd, pbtn, &rect, 1 );
            }
         }

         if( pbtn->fButton2 )
         {
            // get the SCREEN coordinates of the mouse

            pt.x = LOWORD( lParam );
            pt.y = HIWORD( lParam );
            ClientToScreen( hwnd, &pt );

            // get the position of the inserted button

            GetWindowRect( hwnd, &rect );

            pt.x -= rect.left;
            pt.y -= rect.top;
            OffsetRect( &rect, -rect.left, -rect.top );
            if( pbtn->fMouseDown2 == TRUE )
            {
               GetButtonRect( pbtn, &rect, 2 );

               oldstate2 = pbtn->fButtonDown2;

               // check that the mouse is within the inserted button

               if( PtInRect(&rect, pt) )
               {
                  pbtn->fButtonDown2 = 1;
               }
               else
               {
                  pbtn->fButtonDown2 = 0;
               }

               // redraw the non-client area to reflect the change.
               // to prevent flicker, we only redraw the button if its state
               // has changed

               if( oldstate2 != pbtn->fButtonDown2 )
               {
                  DrawInsertedButton( hwnd, pbtn, &rect, 2 );
               }
            }
         }
         break;

      case WM_LBUTTONUP:
         if( pbtn->fMouseDown == TRUE )
         {
            // get the SCREEN coordinates of the mouse

            pt.x = LOWORD( lParam );
            pt.y = HIWORD( lParam );
            ClientToScreen( hwnd, &pt );

            // get the position of the inserted button

            GetWindowRect( hwnd, &rect );

            pt.x -= rect.left;
            pt.y -= rect.top;
            OffsetRect( &rect, -rect.left, -rect.top );

            GetButtonRect( pbtn, &rect, 1 );

            // check that the mouse is within the inserted button

            if( PtInRect(&rect, pt) )
            {
               PostMessage( GetParent(hwnd), WM_COMMAND, MAKEWPARAM(pbtn->uCmdId, BN_CLICKED), (LPARAM) hwnd );
               SetFocus( hwnd );
            }

            ReleaseCapture();
            pbtn->fButtonDown = FALSE;
            pbtn->fMouseDown = FALSE;

            // redraw the non-client area to reflect the change.

            DrawInsertedButton( hwnd, pbtn, &rect, 1 );
         }

         if( pbtn->fButton2 )
         {
            if( pbtn->fMouseDown2 == TRUE )
            {
               // get the SCREEN coordinates of the mouse

               pt.x = LOWORD( lParam );
               pt.y = HIWORD( lParam );
               ClientToScreen( hwnd, &pt );
               GetWindowRect( hwnd, &rect );

               pt.x -= rect.left;
               pt.y -= rect.top;
               OffsetRect( &rect, -rect.left, -rect.top );

               GetButtonRect( pbtn, &rect, 2 );

               // check that the mouse is within the inserted button

               if( PtInRect(&rect, pt) )
               {
                  PostMessage( GetParent(hwnd), WM_COMMAND, MAKEWPARAM(pbtn->uCmdId2, BN_CLICKED), (LPARAM) hwnd );
                  SetFocus( hwnd );
               }

               ReleaseCapture();
               pbtn->fButtonDown2 = FALSE;
               pbtn->fMouseDown2 = FALSE;

               // redraw the non-client area to reflect the change.

               DrawInsertedButton( hwnd, pbtn, &rect, 2 );
            }
         }
         break;
   }

   return( CallWindowProc(OldWndProc, hwnd, Msg, wParam, lParam) );
}
