/*----------------------------------------------------------------------------
 MINIGUI - Harbour Win32 GUI library source code

 Copyright 2002-2008 Roberto Lopez <harbourminigui@gmail.com>
 http://harbourminigui.googlepages.com/

 This program is free software; you can redistribute it and/or modify it under 
 the terms of the GNU General Public License as published by the Free Software 
 Foundation; either version 2 of the License, or (at your option) any later 
 version. 

 This program is distributed in the hope that it will be useful, but WITHOUT 
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS 
 FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

 You should have received a copy of the GNU General Public License along with 
 this software; see the file COPYING. If not, write to the Free Software 
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA (or 
 visit the web site http://www.gnu.org/).

 As a special exception, you have permission for additional uses of the text 
 contained in this release of Harbour Minigui.

 The exception is that, if you link the Harbour Minigui library with other 
 files to produce an executable, this does not by itself cause the resulting 
 executable to be covered by the GNU General Public License.
 Your use of that executable is in no way restricted on account of linking the 
 Harbour-Minigui library code into it.

 Parts of this project are based upon:

	"Harbour GUI framework for Win32"
 	Copyright 2001 Alexander S.Kresin <alex@belacy.belgorod.su>
 	Copyright 2001 Antonio Linares <alinares@fivetech.com>
	www - http://www.harbour-project.org

	"Harbour Project"
	Copyright 1999-2008, http://www.harbour-project.org/

	"WHAT32"
	Copyright 2002 AJ Wos <andrwos@aust1.net> 

	"HWGUI"
  	Copyright 2001-2007 Alexander S.Kresin <alex@belacy.belgorod.su>

---------------------------------------------------------------------------*/

/*
File:		c_cursor.c
Contributors:	Jacek Kubica <kubica@wssk.wroc.pl>
                Grigory Filatov <gfilatov@freemail.ru>
Description:	Mouse Cursor Shapes handling for MiniGUI
Status:		Public Domain
*/

//#define __WIN98__
//#define __WINNT__

#define _WIN32_IE 0x0500
#define HB_OS_WIN_32_USED
#ifdef __POCC__ 
#define _WIN32_WINNT 0x0500
#else
#define _WIN32_WINNT 0x0400
#endif
#include <shlobj.h>

#include <windows.h>
#include <commctrl.h>
#include "hbapi.h"
#include "hbvm.h"
#include "hbstack.h"
#include "hbapiitm.h"

HB_FUNC( LOADCURSOR )
{
   HINSTANCE   hInstance = ( ISNIL(1) ? NULL : (HINSTANCE) hb_parnl(1) );
   LPCTSTR     lpCursorName = ( hb_parinfo(2) == HB_IT_STRING ? hb_parc(2) : MAKEINTRESOURCE(hb_parnl(2)) );
   hb_retnl( ( long ) LoadCursor(hInstance, lpCursorName) );
}

HB_FUNC( LOADCURSORFROMFILE )
{
   LPCTSTR  lpFileName = ( LPCTSTR ) hb_parc( 1 ); // file with cursor picture or anims (.cur .ani)
   hb_retnl( ( long ) LoadCursorFromFile(lpFileName) );
}

HB_FUNC( SETRESCURSOR )
{
   hb_retnl( (LONG) SetCursor((HCURSOR) hb_parnl(1)) );
}

HB_FUNC( FILECURSOR )
{
   hb_retnl( ( long ) SetCursor(LoadCursorFromFile(hb_parc(1))) );
}

#ifdef __WINNT__
HB_FUNC( CURSORHAND )
{
   hb_retnl( ( long ) SetCursor(LoadCursor(GetModuleHandle(NULL), "MINIGUI_FINGER")) );
}

#else
#ifndef __WIN98__
HB_FUNC( CURSORHAND )
{
   hb_retnl( ( long ) SetCursor(LoadCursor(0, IDC_HAND)) );
}
#endif
#endif
HB_FUNC( CURSORARROW )
{
   hb_retnl( ( long ) SetCursor(LoadCursor(0, IDC_ARROW)) );
}

HB_FUNC( CURSORHELP )
{
   hb_retnl( ( long ) SetCursor(LoadCursor(0, IDC_HELP)) );
}

#ifndef __WIN98__
HB_FUNC( CURSORWAIT )
{
   hb_retnl( ( long ) SetCursor(LoadCursor(0, IDC_WAIT)) );
}
#endif
HB_FUNC( CURSORCROSS )
{
   hb_retnl( ( long ) SetCursor(LoadCursor(0, IDC_CROSS)) );
}

HB_FUNC( CURSORIBEAM )
{
   hb_retnl( ( long ) SetCursor(LoadCursor(0, IDC_IBEAM)) );
}

HB_FUNC( CURSORAPPSTARTING )
{
   hb_retnl( ( long ) SetCursor(LoadCursor(0, IDC_APPSTARTING)) );
}

HB_FUNC( CURSORNO )
{
   hb_retnl( ( long ) SetCursor(LoadCursor(0, IDC_NO)) );
}

HB_FUNC( CURSORSIZEALL )
{
   hb_retnl( ( long ) SetCursor(LoadCursor(0, IDC_SIZEALL)) );
}

HB_FUNC( CURSORSIZENESW )
{
   hb_retnl( ( long ) SetCursor(LoadCursor(0, IDC_SIZENESW)) );
}

HB_FUNC( CURSORSIZENWSE )
{
   hb_retnl( ( long ) SetCursor(LoadCursor(0, IDC_SIZENWSE)) );
}

HB_FUNC( CURSORSIZENS )
{
   hb_retnl( ( long ) SetCursor(LoadCursor(0, IDC_SIZENS)) );
}

HB_FUNC( CURSORSIZEWE )
{
   hb_retnl( ( long ) SetCursor(LoadCursor(0, IDC_SIZEWE)) );
}

HB_FUNC( CURSORUPARROW )
{
   hb_retnl( ( long ) SetCursor(LoadCursor(0, IDC_UPARROW)) );
}

HB_FUNC( SETWINDOWCURSOR )
{
   HCURSOR  ch;

   if( ISCHAR(2) )
   {
      ch = LoadCursor( GetModuleHandle(NULL), hb_parc(2) );

      if( ch == NULL )
      {
         ch = LoadCursorFromFile( (LPCTSTR) hb_parc(2) );
      }
   }
   else
   {
      ch = LoadCursor( NULL, MAKEINTRESOURCE(hb_parnl(2)) );
   }

   SetClassLong( (HWND) hb_parnl(1), // window handle
   GCL_HCURSOR, // change cursor
   (LONG) ch );   // new cursor
}

HB_FUNC( SETHANDCURSOR )
{
   SetClassLong( (HWND) hb_parnl(1), // window handle
   GCL_HCURSOR, // change cursor
   (LONG) LoadCursor(NULL, IDC_HAND) );   // new cursor
}

HB_FUNC( SETWAITCURSOR )
{
   SetClassLong( (HWND) hb_parnl(1), // window handle
   GCL_HCURSOR, // change cursor
   (LONG) LoadCursor(NULL, IDC_WAIT) );   // new cursor
}

HB_FUNC( SETARROWCURSOR )
{
   SetClassLong( (HWND) hb_parnl(1), // window handle
   GCL_HCURSOR, // change cursor
   (LONG) LoadCursor(NULL, IDC_ARROW) );  // new cursor
}
