/*----------------------------------------------------------------------------
 MINIGUI - Harbour Win32 GUI library source code

 Copyright 2002-2008 Roberto Lopez <harbourminigui@gmail.com>
 http://harbourminigui.googlepages.com/

 This program is free software; you can redistribute it and/or modify it under 
 the terms of the GNU General Public License as published by the Free Software 
 Foundation; either version 2 of the License, or (at your option) any later 
 version. 

 This program is distributed in the hope that it will be useful, but WITHOUT 
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS 
 FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

 You should have received a copy of the GNU General Public License along with 
 this software; see the file COPYING. If not, write to the Free Software 
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA (or 
 visit the web site http://www.gnu.org/).

 As a special exception, you have permission for additional uses of the text 
 contained in this release of Harbour Minigui.

 The exception is that, if you link the Harbour Minigui library with other 
 files to produce an executable, this does not by itself cause the resulting 
 executable to be covered by the GNU General Public License.
 Your use of that executable is in no way restricted on account of linking the 
 Harbour-Minigui library code into it.

 Parts of this project are based upon:

	"Harbour GUI framework for Win32"
 	Copyright 2001 Alexander S.Kresin <alex@belacy.belgorod.su>
 	Copyright 2001 Antonio Linares <alinares@fivetech.com>
	www - http://www.harbour-project.org

	"Harbour Project"
	Copyright 1999-2008, http://www.harbour-project.org/

	"WHAT32"
	Copyright 2002 AJ Wos <andrwos@aust1.net> 

	"HWGUI"
  	Copyright 2001-2007 Alexander S.Kresin <alex@belacy.belgorod.su>

---------------------------------------------------------------------------*/

#define _WIN32_IE 0x0500
#define HB_OS_WIN_32_USED
#define _WIN32_WINNT 0x0400
#include <shlobj.h>

#if !defined( __POCC__ )
#if defined( __MINGW32__ )
#define UDM_GETPOS32 0x0472
#define UDM_SETPOS32 0x0471
#endif
#if defined( _MSC_VER )
#define UDM_SETPOS32 ( WM_USER + 113 )
#define UDM_GETPOS32 ( WM_USER + 114 )
#endif
#endif
#include <windows.h>
#include <commctrl.h>
#include "hbapi.h"
#include "hbvm.h"
#include "hbstack.h"
#include "hbapiitm.h"

HFONT PrepareFont( char *Fontname, int FontSize, int Weight, int Italic, int Underline, int StrikeOut, int Angle )
{
   HDC   hDC;
   int   cyp;

   memset( &cyp, 0, sizeof(cyp) );
   memset( &hDC, 0, sizeof(hDC) );

   hDC = GetDC( HWND_DESKTOP );

   cyp = GetDeviceCaps( hDC, LOGPIXELSY );

   ReleaseDC( HWND_DESKTOP, hDC );

   FontSize = ( FontSize * cyp ) / 72;

   return CreateFont
      (
         0 - FontSize,
         0,
         Angle,
         0,
         Weight,
         Italic,
         Underline,
         StrikeOut,
         DEFAULT_CHARSET,
         OUT_TT_PRECIS,
         CLIP_DEFAULT_PRECIS,
         DEFAULT_QUALITY,
         FF_DONTCARE,
         Fontname
      );
}

HB_FUNC( INITFONT )
{
   HFONT font;
   int   bold = FW_NORMAL;
   int   italic = 0;
   int   underline = 0;
   int   strikeout = 0;
   int   angle = 0;

   if( hb_parl(3) )
   {
      bold = FW_BOLD;
   }

   if( hb_parl(4) )
   {
      italic = 1;
   }

   if( hb_parl(5) )
   {
      underline = 1;
   }

   if( hb_parl(6) )
   {
      strikeout = 1;
   }

   if( hb_parni(7) )
   {
      angle = hb_parni( 7 );
   }

   font = PrepareFont( hb_parc(1), hb_parni(2), bold, italic, underline, strikeout, angle );

   hb_retnl( (LONG) font );
}

HB_FUNC( _SETFONT )
{
   HFONT font;
   int   bold = FW_NORMAL;
   int   italic = 0;
   int   underline = 0;
   int   strikeout = 0;
   int   angle = 0;

   if( hb_parl(4) )
   {
      bold = FW_BOLD;
   }

   if( hb_parl(5) )
   {
      italic = 1;
   }

   if( hb_parl(6) )
   {
      underline = 1;
   }

   if( hb_parl(7) )
   {
      strikeout = 1;
   }

   if( hb_parni(8) )
   {
      angle = hb_parni( 8 );
   }

   font = PrepareFont( hb_parc(2), (LPARAM) hb_parni(3), bold, italic, underline, strikeout, angle );
   SendMessage( (HWND) hb_parnl(1), (UINT) WM_SETFONT, (WPARAM) font, 1 );
   hb_retnl( (LONG) font );
}

HB_FUNC( SETFONTNAMESIZE )
{
   HFONT font;
   int   bold = FW_NORMAL;
   int   italic = 0;
   int   underline = 0;
   int   strikeout = 0;
   int   angle = 0;

   if( hb_parl(4) )
   {
      bold = FW_BOLD;
   }

   if( hb_parl(5) )
   {
      italic = 1;
   }

   if( hb_parl(6) )
   {
      underline = 1;
   }

   if( hb_parl(7) )
   {
      strikeout = 1;
   }

   if( hb_parni(8) )
   {
      angle = hb_parni( 8 );
   }

   font = PrepareFont( hb_parc(2), (LPARAM) hb_parni(3), bold, italic, underline, strikeout, angle );
   SendMessage( (HWND) hb_parnl(1), (UINT) WM_SETFONT, (WPARAM) font, 1 );
   hb_retnl( (LONG) font );
}

HB_FUNC( _SETFONTHANDLE )
{
   SendMessage( (HWND) hb_parnl(1), (UINT) WM_SETFONT, (WPARAM) (HFONT) hb_parnl(2), 1 );
}
