/*----------------------------------------------------------------------------
 MINIGUI - Harbour Win32 GUI library source code

 Copyright 2002-2008 Roberto Lopez <harbourminigui@gmail.com>
 http://harbourminigui.googlepages.com/

 This program is free software; you can redistribute it and/or modify it under 
 the terms of the GNU General Public License as published by the Free Software 
 Foundation; either version 2 of the License, or (at your option) any later 
 version. 

 This program is distributed in the hope that it will be useful, but WITHOUT 
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS 
 FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

 You should have received a copy of the GNU General Public License along with 
 this software; see the file COPYING. If not, write to the Free Software 
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA (or 
 visit the web site http://www.gnu.org/).

 As a special exception, you have permission for additional uses of the text 
 contained in this release of Harbour Minigui.

 The exception is that, if you link the Harbour Minigui library with other 
 files to produce an executable, this does not by itself cause the resulting 
 executable to be covered by the GNU General Public License.
 Your use of that executable is in no way restricted on account of linking the 
 Harbour-Minigui library code into it.

 Parts of this project are based upon:

	"Harbour GUI framework for Win32"
 	Copyright 2001 Alexander S.Kresin <alex@belacy.belgorod.su>
 	Copyright 2001 Antonio Linares <alinares@fivetech.com>
	www - http://www.harbour-project.org

	"Harbour Project"
	Copyright 1999-2008, http://www.harbour-project.org/

	"WHAT32"
	Copyright 2002 AJ Wos <andrwos@aust1.net> 

	"HWGUI"
  	Copyright 2001-2007 Alexander S.Kresin <alex@belacy.belgorod.su>

---------------------------------------------------------------------------*/

#include <windows.h>
#include <commctrl.h>
#include "hbapi.h"
#include <wingdi.h>
#include <winuser.h>
#include "item.api"

HFONT PrepareFont( char *, int, int, int, int, int, int );

HB_FUNC( TEXTDRAW )
{
   HWND     hWnd1;
   HDC      hdc1;
   RECT     rect1;
   HFONT    font;
   HGDIOBJ  hgdiobj1;
   int      bold = FW_NORMAL;
   int      italic = 0;
   int      underline = 0;
   int      strikeout = 0;
   int      angle = 0;

   hWnd1 = ( HWND ) hb_parnl( 1 );
   hdc1 = GetDC( (HWND) hWnd1 );

   if( hb_parl(11) )
   {
      bold = FW_BOLD;
   }

   if( hb_parl(12) )
   {
      italic = 1;
   }

   if( hb_parl(13) )
   {
      underline = 1;
   }

   if( hb_parl(14) )
   {
      strikeout = 1;
   }

   font = PrepareFont( hb_parc(9), (LPARAM) hb_parni(10), bold, italic, underline, strikeout, angle );

   hgdiobj1 = SelectObject( (HDC) hdc1, font );

   if( hb_parl(15) )
   {
      SetBkMode( hdc1, TRANSPARENT );
   }
   else
   {
      SetBkColor( hdc1, RGB(hb_parni(8, 1), hb_parni(8, 2), hb_parni(8, 3)) );
   }

   SetTextColor( hdc1, RGB(hb_parni(7, 1), hb_parni(7, 2), hb_parni(7, 3)) );

   SetRect( &rect1, hb_parni(3), hb_parni(2), hb_parni(6), hb_parni(5) );
   ExtTextOut( hdc1, hb_parni(3), hb_parni(2), ETO_OPAQUE, &rect1, hb_parc(4), hb_parclen(4), NULL );

   SelectObject( (HDC) hdc1, (HGDIOBJ) hgdiobj1 );
   DeleteObject( hgdiobj1 );

   if( hb_parl(15) )
   {
      SetBkMode( hdc1, OPAQUE );
   }

   ReleaseDC( hWnd1, hdc1 );
}

HB_FUNC( LINEDRAW )
{
   HWND     hWnd1;
   HDC      hdc1;
   HGDIOBJ  hgdiobj1;
   HPEN     hpen;
   hWnd1 = ( HWND ) hb_parnl( 1 );
   hdc1 = GetDC( (HWND) hWnd1 );
   hpen = CreatePen( ( int ) PS_SOLID, ( int ) hb_parni(7), (COLORREF) RGB(( int ) hb_parni(6, 1), ( int ) hb_parni(6, 2), ( int ) hb_parni(6, 3)) );
   hgdiobj1 = SelectObject( (HDC) hdc1, hpen );
   MoveToEx( (HDC) hdc1, ( int ) hb_parni(3), ( int ) hb_parni(2), NULL );
   LineTo( (HDC) hdc1, ( int ) hb_parni(5), ( int ) hb_parni(4) );
   SelectObject( (HDC) hdc1, (HGDIOBJ) hgdiobj1 );
   DeleteObject( hpen );
   ReleaseDC( hWnd1, hdc1 );
}

HB_FUNC( RECTDRAW )
{
   HWND     hWnd1;
   HDC      hdc1;
   HGDIOBJ  hgdiobj1, hgdiobj2;
   HPEN     hpen;
   HBRUSH   hbrush;
   LOGBRUSH br;
   hWnd1 = ( HWND ) hb_parnl( 1 );
   hdc1 = GetDC( (HWND) hWnd1 );
   hpen = CreatePen( ( int ) PS_SOLID, ( int ) hb_parni(7), (COLORREF) RGB(( int ) hb_parni(6, 1), ( int ) hb_parni(6, 2), ( int ) hb_parni(6, 3)) );

   hgdiobj1 = SelectObject( (HDC) hdc1, hpen );
   if( hb_parl(9) )
   {
      hbrush = CreateSolidBrush( (COLORREF) RGB(( int ) hb_parni(8, 1), ( int ) hb_parni(8, 2), ( int ) hb_parni(8, 3)) );
      hgdiobj2 = SelectObject( (HDC) hdc1, hbrush );
   }
   else
   {
      br.lbStyle = BS_HOLLOW;
      hbrush = CreateBrushIndirect( &br );
      hgdiobj2 = SelectObject( (HDC) hdc1, hbrush );
   }

   Rectangle( (HDC) hdc1, ( int ) hb_parni(3), ( int ) hb_parni(2), ( int ) hb_parni(5), ( int ) hb_parni(4) );
   SelectObject( (HDC) hdc1, (HGDIOBJ) hgdiobj1 );
   SelectObject( (HDC) hdc1, (HGDIOBJ) hgdiobj2 );
   DeleteObject( hpen );
   DeleteObject( hbrush );
   ReleaseDC( hWnd1, hdc1 );
}

HB_FUNC( ROUNDRECTDRAW )
{
   HWND     hWnd1;
   HDC      hdc1;
   HGDIOBJ  hgdiobj1, hgdiobj2;
   HPEN     hpen;
   HBRUSH   hbrush;
   LOGBRUSH br;
   hWnd1 = ( HWND ) hb_parnl( 1 );
   hdc1 = GetDC( (HWND) hWnd1 );
   hpen = CreatePen( ( int ) PS_SOLID, ( int ) hb_parni(9), (COLORREF) RGB(( int ) hb_parni(8, 1), ( int ) hb_parni(8, 2), ( int ) hb_parni(8, 3)) );
   hgdiobj1 = SelectObject( (HDC) hdc1, hpen );
   if( hb_parl(11) )
   {
      hbrush = CreateSolidBrush( (COLORREF) RGB(( int ) hb_parni(10, 1), ( int ) hb_parni(10, 2), ( int ) hb_parni(10, 3)) );
      hgdiobj2 = SelectObject( (HDC) hdc1, hbrush );
   }
   else
   {
      br.lbStyle = BS_HOLLOW;
      hbrush = CreateBrushIndirect( &br );
      hgdiobj2 = SelectObject( (HDC) hdc1, hbrush );
   }

   RoundRect
   (
      (HDC) hdc1,
      ( int ) hb_parni(3),
      ( int ) hb_parni(2),
      ( int ) hb_parni(5),
      ( int ) hb_parni(4),
      ( int ) hb_parni(6),
      ( int ) hb_parni(7)
   );
   SelectObject( (HDC) hdc1, (HGDIOBJ) hgdiobj1 );
   SelectObject( (HDC) hdc1, (HGDIOBJ) hgdiobj2 );
   DeleteObject( hpen );
   DeleteObject( hbrush );
   ReleaseDC( hWnd1, hdc1 );
}

HB_FUNC( ELLIPSEDRAW )
{
   HWND     hWnd1;
   HDC      hdc1;
   HGDIOBJ  hgdiobj1, hgdiobj2;
   HPEN     hpen;
   HBRUSH   hbrush;
   LOGBRUSH br;
   hWnd1 = ( HWND ) hb_parnl( 1 );
   hdc1 = GetDC( (HWND) hWnd1 );
   hpen = CreatePen( ( int ) PS_SOLID, ( int ) hb_parni(7), (COLORREF) RGB(( int ) hb_parni(6, 1), ( int ) hb_parni(6, 2), ( int ) hb_parni(6, 3)) );
   hgdiobj1 = SelectObject( (HDC) hdc1, hpen );
   if( hb_parl(9) )
   {
      hbrush = CreateSolidBrush( (COLORREF) RGB(( int ) hb_parni(8, 1), ( int ) hb_parni(8, 2), ( int ) hb_parni(8, 3)) );
      hgdiobj2 = SelectObject( (HDC) hdc1, hbrush );
   }
   else
   {
      br.lbStyle = BS_HOLLOW;
      hbrush = CreateBrushIndirect( &br );
      hgdiobj2 = SelectObject( (HDC) hdc1, hbrush );
   }

   Ellipse( (HDC) hdc1, ( int ) hb_parni(3), ( int ) hb_parni(2), ( int ) hb_parni(5), ( int ) hb_parni(4) );
   SelectObject( (HDC) hdc1, (HGDIOBJ) hgdiobj1 );
   SelectObject( (HDC) hdc1, (HGDIOBJ) hgdiobj2 );
   DeleteObject( hpen );
   DeleteObject( hbrush );
   ReleaseDC( hWnd1, hdc1 );
}

HB_FUNC( ARCDRAW )
{
   HWND     hWnd1;
   HDC      hdc1;
   HGDIOBJ  hgdiobj1;
   HPEN     hpen;
   hWnd1 = ( HWND ) hb_parnl( 1 );
   hdc1 = GetDC( (HWND) hWnd1 );
   hpen = CreatePen
      (
         ( int ) PS_SOLID,
         ( int ) hb_parni(11),
         (COLORREF) RGB(( int ) hb_parni(10, 1), ( int ) hb_parni(10, 2), ( int ) hb_parni(10, 3))
      );
   hgdiobj1 = SelectObject( (HDC) hdc1, hpen );
   Arc
   (
      (HDC) hdc1,
      ( int ) hb_parni(3),
      ( int ) hb_parni(2),
      ( int ) hb_parni(5),
      ( int ) hb_parni(4),
      ( int ) hb_parni(7),
      ( int ) hb_parni(6),
      ( int ) hb_parni(9),
      ( int ) hb_parni(8)
   );
   SelectObject( (HDC) hdc1, (HGDIOBJ) hgdiobj1 );
   DeleteObject( hpen );
   ReleaseDC( hWnd1, hdc1 );
}

HB_FUNC( PIEDRAW )
{
   HWND     hWnd1;
   HDC      hdc1;
   HGDIOBJ  hgdiobj1, hgdiobj2;
   HPEN     hpen;
   HBRUSH   hbrush;
   LOGBRUSH br;
   hWnd1 = ( HWND ) hb_parnl( 1 );
   hdc1 = GetDC( (HWND) hWnd1 );
   hpen = CreatePen
      (
         ( int ) PS_SOLID,
         ( int ) hb_parni(11),
         (COLORREF) RGB(( int ) hb_parni(10, 1), ( int ) hb_parni(10, 2), ( int ) hb_parni(10, 3))
      );
   hgdiobj1 = SelectObject( (HDC) hdc1, hpen );
   if( hb_parl(13) )
   {
      hbrush = CreateSolidBrush( (COLORREF) RGB(( int ) hb_parni(12, 1), ( int ) hb_parni(12, 2), ( int ) hb_parni(12, 3)) );
      hgdiobj2 = SelectObject( (HDC) hdc1, hbrush );
   }
   else
   {
      br.lbStyle = BS_HOLLOW;
      hbrush = CreateBrushIndirect( &br );
      hgdiobj2 = SelectObject( (HDC) hdc1, hbrush );
   }

   Pie
   (
      (HDC) hdc1,
      ( int ) hb_parni(3),
      ( int ) hb_parni(2),
      ( int ) hb_parni(5),
      ( int ) hb_parni(4),
      ( int ) hb_parni(7),
      ( int ) hb_parni(6),
      ( int ) hb_parni(9),
      ( int ) hb_parni(8)
   );
   SelectObject( (HDC) hdc1, (HGDIOBJ) hgdiobj1 );
   SelectObject( (HDC) hdc1, (HGDIOBJ) hgdiobj2 );
   DeleteObject( hpen );
   DeleteObject( hbrush );
   ReleaseDC( hWnd1, hdc1 );
}

HB_FUNC( POLYGONDRAW )
{
   HWND     hWnd1;
   HDC      hdc1;
   HGDIOBJ  hgdiobj1, hgdiobj2;
   HPEN     hpen;
   HBRUSH   hbrush;
   LOGBRUSH br;
   POINT    apoints[1024];
   int      number = hb_parinfa( 2, 0 );
   int      i;
   hWnd1 = ( HWND ) hb_parnl( 1 );
   hdc1 = GetDC( (HWND) hWnd1 );
   hpen = CreatePen( ( int ) PS_SOLID, ( int ) hb_parni(5), (COLORREF) RGB(( int ) hb_parni(4, 1), ( int ) hb_parni(4, 2), ( int ) hb_parni(4, 3)) );
   hgdiobj1 = SelectObject( (HDC) hdc1, hpen );
   if( hb_parl(7) )
   {
      hbrush = CreateSolidBrush( (COLORREF) RGB(( int ) hb_parni(6, 1), ( int ) hb_parni(6, 2), ( int ) hb_parni(6, 3)) );
      hgdiobj2 = SelectObject( (HDC) hdc1, hbrush );
   }
   else
   {
      br.lbStyle = BS_HOLLOW;
      hbrush = CreateBrushIndirect( &br );
      hgdiobj2 = SelectObject( (HDC) hdc1, hbrush );
   }

   for( i = 0; i <= number - 1; i++ )
   {
      apoints[i].x = hb_parni( 2, i + 1 );
      apoints[i].y = hb_parni( 3, i + 1 );
   }

   Polygon( (HDC) hdc1, apoints, number );
   SelectObject( (HDC) hdc1, (HGDIOBJ) hgdiobj1 );
   SelectObject( (HDC) hdc1, (HGDIOBJ) hgdiobj2 );
   DeleteObject( hpen );
   DeleteObject( hbrush );
   ReleaseDC( hWnd1, hdc1 );
}

HB_FUNC( POLYBEZIERDRAW )
{
   HWND     hWnd1;
   HDC      hdc1;
   HGDIOBJ  hgdiobj1;
   HPEN     hpen;
   POINT    apoints[1024];
   DWORD    number = ( DWORD ) hb_parinfa( 2, 0 );
   DWORD    i;
   hWnd1 = ( HWND ) hb_parnl( 1 );
   hdc1 = GetDC( (HWND) hWnd1 );
   hpen = CreatePen( ( int ) PS_SOLID, ( int ) hb_parni(5), (COLORREF) RGB(( int ) hb_parni(4, 1), ( int ) hb_parni(4, 2), ( int ) hb_parni(4, 3)) );
   hgdiobj1 = SelectObject( (HDC) hdc1, hpen );
   for( i = 0; i <= number - 1; i++ )
   {
      apoints[i].x = hb_parni( 2, i + 1 );
      apoints[i].y = hb_parni( 3, i + 1 );
   }

   PolyBezier( (HDC) hdc1, apoints, number );
   SelectObject( (HDC) hdc1, (HGDIOBJ) hgdiobj1 );
   DeleteObject( hpen );
   ReleaseDC( hWnd1, hdc1 );
}

void WndDrawBox( HDC hDC, RECT *rct, HPEN hPUpLeft, HPEN hPBotRit )
{
   HPEN  hOldPen = ( HPEN ) SelectObject( hDC, hPUpLeft );
   POINT pt;

   MoveToEx( hDC, rct->left, rct->bottom, &pt );

   LineTo( hDC, rct->left, rct->top );
   LineTo( hDC, rct->right, rct->top );
   SelectObject( hDC, hPBotRit );

   MoveToEx( hDC, rct->left, rct->bottom, &pt );

   LineTo( hDC, rct->right, rct->bottom );
   LineTo( hDC, rct->right, rct->top - 1 );
   SelectObject( hDC, hOldPen );
}

void WindowBoxIn( HDC hDC, RECT *pRect )
{
   HPEN  hWhite = CreatePen( PS_SOLID, 1, GetSysColor(COLOR_BTNHIGHLIGHT) );
   HPEN  hGray = CreatePen( PS_SOLID, 1, GetSysColor(COLOR_BTNSHADOW) );

   WndDrawBox( hDC, pRect, hGray, hWhite );

   DeleteObject( hGray );
   DeleteObject( hWhite );
}

void WindowRaised( HDC hDC, RECT *pRect )
{
   HPEN  hGray = CreatePen( PS_SOLID, 1, GetSysColor(COLOR_BTNSHADOW) );
   HPEN  hWhite = CreatePen( PS_SOLID, 1, GetSysColor(COLOR_BTNHIGHLIGHT) );

   WndDrawBox( hDC, pRect, hWhite, hGray );

   DeleteObject( hGray );
   DeleteObject( hWhite );
}

HB_FUNC( WNDBOXIN )
{
   RECT  rct;

   rct.top = hb_parni( 2 );
   rct.left = hb_parni( 3 );
   rct.bottom = hb_parni( 4 );
   rct.right = hb_parni( 5 );

   WindowBoxIn( (HDC) hb_parnl(1), &rct );
}

HB_FUNC( WNDBOXRAISED )
{
   RECT  rct;

   rct.top = hb_parni( 2 );
   rct.left = hb_parni( 3 );
   rct.bottom = hb_parni( 4 );
   rct.right = hb_parni( 5 );

   WindowRaised( (HDC) hb_parnl(1), &rct );
}
