/*----------------------------------------------------------------------------
 MINIGUI - Harbour Win32 GUI library source code

 Copyright 2002-2008 Roberto Lopez <harbourminigui@gmail.com>
 http://harbourminigui.googlepages.com/

 This program is free software; you can redistribute it and/or modify it under 
 the terms of the GNU General Public License as published by the Free Software 
 Foundation; either version 2 of the License, or (at your option) any later 
 version. 

 This program is distributed in the hope that it will be useful, but WITHOUT 
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS 
 FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

 You should have received a copy of the GNU General Public License along with 
 this software; see the file COPYING. If not, write to the Free Software 
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA (or 
 visit the web site http://www.gnu.org/).

 As a special exception, you have permission for additional uses of the text 
 contained in this release of Harbour Minigui.

 The exception is that, if you link the Harbour Minigui library with other 
 files to produce an executable, this does not by itself cause the resulting 
 executable to be covered by the GNU General Public License.
 Your use of that executable is in no way restricted on account of linking the 
 Harbour-Minigui library code into it.

 Parts of this project are based upon:

	"Harbour GUI framework for Win32"
 	Copyright 2001 Alexander S.Kresin <alex@belacy.belgorod.su>
 	Copyright 2001 Antonio Linares <alinares@fivetech.com>
	www - http://www.harbour-project.org

	"Harbour Project"
	Copyright 1999-2008, http://www.harbour-project.org/

	"WHAT32"
	Copyright 2002 AJ Wos <andrwos@aust1.net> 

	"HWGUI"
  	Copyright 2001-2007 Alexander S.Kresin <alex@belacy.belgorod.su>

---------------------------------------------------------------------------*/

#ifndef CINTERFACE
#define CINTERFACE
#endif
#define _WIN32_IE 0x0500
#define HB_OS_WIN_32_USED
#define _WIN32_WINNT 0x0400
#include <shlobj.h>

#include <windows.h>
#include <commctrl.h>
#include "hbapi.h"
#include "hbvm.h"
#include "hbstack.h"
#include "hbapiitm.h"
#include <winuser.h>
#include <wingdi.h>
#include "olectl.h"

HBITMAP  loadolepicture( char *filename, int width, int height, HWND handle, int scalestrech, int whitebackground );

HB_FUNC( INITIMAGE )
{
   HWND  h;
   HWND  hwnd;
   int   Style;

   hwnd = ( HWND ) hb_parnl( 1 );

   Style = WS_CHILD | SS_BITMAP;

   if( !hb_parl(5) )
   {
      Style = Style | WS_VISIBLE;
   }

   if( hb_parl(6) )
   {
      Style = Style | SS_NOTIFY;
   }

   h = CreateWindowEx( 0, "static", NULL, Style, hb_parni(3), hb_parni(4), 0, 0, hwnd, (HMENU) hb_parni(2), GetModuleHandle(NULL), NULL );

   hb_retnl( (LONG) h );
}

HB_FUNC( C_SETPICTURE )
{
   HBITMAP  hBitmap;

   hBitmap = loadolepicture( hb_parc(2), hb_parni(3), hb_parni(4), (HWND) hb_parnl(1), hb_parni(5), hb_parni(6) );

   if( hBitmap != NULL )
   {
      SendMessage( (HWND) hb_parnl(1), (UINT) STM_SETIMAGE, (WPARAM) IMAGE_BITMAP, (LPARAM) hBitmap );
   }

   hb_retnl( (LONG) hBitmap );
}

HBITMAP loadolepicture( char *filename, int width, int height, HWND handle, int scalestrech, int whitebackground )
{
   IStream     *iStream;
   IPicture    *iPicture = NULL;
   HGLOBAL     hGlobal;
   HGLOBAL     hGlobalres;
   HRSRC       hSource;
   LPVOID      lpVoid;
   HANDLE      hFile;
   DWORD       nFileSize;
   DWORD       nReadByte;
   RECT        rect, rect2;
   HBITMAP     hpic, hpic2;
   BITMAP      bm;
   int         nSize;
   long        lWidth, lHeight;

   HINSTANCE   hinstance = GetModuleHandle( NULL );
   HDC         imgDC = GetDC( handle );
   HDC         tmpDC = CreateCompatibleDC( imgDC );
   HDC         tmp2DC;

   if( (width == 0) & (height == 0) )
   {
      GetClientRect( handle, &rect );
   }
   else
   {
      SetRect( &rect, 0, 0, width, height );
   }

   SetRect( &rect2, 0, 0, rect.right, rect.bottom );

   hpic2 = ( HBITMAP ) LoadImage( 0, filename, IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE | LR_CREATEDIBSECTION );

   if( hpic2 == NULL )
   {
      hpic2 = ( HBITMAP ) LoadImage( GetModuleHandle(NULL), filename, IMAGE_BITMAP, 0, 0, LR_CREATEDIBSECTION );
   }

   if( hpic2 != NULL )
   {
      tmp2DC = CreateCompatibleDC( imgDC );
      GetObject( hpic2, sizeof(BITMAP), &bm );
      lWidth = bm.bmWidth;
      lHeight = bm.bmHeight;
      SelectObject( tmp2DC, hpic2 );
   }
   else
   {
      hSource = FindResource( hinstance, filename, "JPG" );
      if( hSource == NULL )
      {
         hSource = FindResource( hinstance, filename, "GIF" );
      }

      if( hSource == NULL )
      {
         hFile = CreateFile( filename, GENERIC_READ, 0, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL );
         if( hFile == INVALID_HANDLE_VALUE )
         {
            return NULL;
         }

         nFileSize = GetFileSize( hFile, NULL );
         hGlobal = GlobalAlloc( GPTR, nFileSize );
         lpVoid = GlobalLock( hGlobal );
         ReadFile( hFile, lpVoid, nFileSize, &nReadByte, NULL );
         GlobalUnlock( hGlobal );
         CloseHandle( hFile );
      }
      else
      {
         hGlobalres = LoadResource( hinstance, hSource );
         if( hGlobalres == NULL )
         {
            return NULL;
         }

         lpVoid = LockResource( hGlobalres );
         if( lpVoid == NULL )
         {
            return NULL;
         }

         nSize = SizeofResource( hinstance, hSource );
         hGlobal = GlobalAlloc( GPTR, nSize );
         if( hGlobal == NULL )
         {
            return NULL;
         }

         memcpy( hGlobal, lpVoid, nSize );
         FreeResource( hGlobalres );
      }

      CreateStreamOnHGlobal( hGlobal, TRUE, &iStream );
      OleLoadPicture( iStream, nFileSize, TRUE, &IID_IPicture, ( LPVOID * ) &iPicture );
      iStream->lpVtbl->Release( iStream );
      if( iPicture == NULL )
      {
         return NULL;
      }

      iPicture->lpVtbl->get_Width( iPicture, &lWidth );
      iPicture->lpVtbl->get_Height( iPicture, &lHeight );
   }

   if( scalestrech == 0 )
   {
      if( ( int ) lWidth * rect.bottom / lHeight <= rect.right )
      {
         rect.right = ( int ) lWidth * rect.bottom / lHeight;
      }
      else
      {
         rect.bottom = ( int ) lHeight * rect.right / lWidth;
      }
   }

   rect.left = ( int ) ( width - rect.right ) / 2;
   rect.top = ( int ) ( height - rect.bottom ) / 2;

   hpic = CreateCompatibleBitmap( imgDC, width, height );
   SelectObject( tmpDC, hpic );

   if( whitebackground == 1 )
   {
      FillRect( tmpDC, &rect2, (HBRUSH) GetStockObject(WHITE_BRUSH) );
   }
   else
   {
      FillRect( tmpDC, &rect2, (HBRUSH) GetSysColorBrush(COLOR_BTNFACE) );
   }

   if( iPicture != NULL )
   {
      iPicture->lpVtbl->Render( iPicture, tmpDC, rect.left, rect.top, rect.right, rect.bottom, 0, lHeight, lWidth, -lHeight, NULL );
      iPicture->lpVtbl->Release( iPicture );
      GlobalFree( hGlobal );
   }
   else
   {
      StretchBlt( tmpDC, rect.left, rect.top, rect.right, rect.bottom, tmp2DC, 0, 0, lWidth, lHeight, SRCCOPY );
      DeleteDC( tmp2DC );
      DeleteObject( hpic2 );
   }

   DeleteDC( imgDC );
   DeleteDC( tmpDC );

   return hpic;
}
