/*----------------------------------------------------------------------------
 MINIGUI - Harbour Win32 GUI library source code

 Copyright 2002-2008 Roberto Lopez <harbourminigui@gmail.com>
 http://harbourminigui.googlepages.com/

 This program is free software; you can redistribute it and/or modify it under 
 the terms of the GNU General Public License as published by the Free Software 
 Foundation; either version 2 of the License, or (at your option) any later 
 version. 

 This program is distributed in the hope that it will be useful, but WITHOUT 
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS 
 FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

 You should have received a copy of the GNU General Public License along with 
 this software; see the file COPYING. If not, write to the Free Software 
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA (or 
 visit the web site http://www.gnu.org/).

 As a special exception, you have permission for additional uses of the text 
 contained in this release of Harbour Minigui.

 The exception is that, if you link the Harbour Minigui library with other 
 files to produce an executable, this does not by itself cause the resulting 
 executable to be covered by the GNU General Public License.
 Your use of that executable is in no way restricted on account of linking the 
 Harbour-Minigui library code into it.

 Parts of this project are based upon:

	"Harbour GUI framework for Win32"
 	Copyright 2001 Alexander S.Kresin <alex@belacy.belgorod.su>
 	Copyright 2001 Antonio Linares <alinares@fivetech.com>
	www - http://www.harbour-project.org

	"Harbour Project"
	Copyright 1999-2008, http://www.harbour-project.org/

	"WHAT32"
	Copyright 2002 AJ Wos <andrwos@aust1.net> 

	"HWGUI"
  	Copyright 2001-2007 Alexander S.Kresin <alex@belacy.belgorod.su>

 Parts of this code is contributed and used here under permission of his author:
       Copyright 2007 (C) P.Chornyj <myorg63@mail.ru>
---------------------------------------------------------------------------*/

#define __WINNT__

#define _WIN32_IE 0x0500
#define HB_OS_WIN_32_USED
#define _WIN32_WINNT 0x0400
#include <shlobj.h>

#include <windows.h>
#include <commctrl.h>
#include "hbapi.h"
#include "hbvm.h"
#include "hbstack.h"
#include "hbapiitm.h"
#include "item.api"

static INT  bm_size = 16;
static INT  min_width = 20;
static INT  min_height = 20;
static INT  cx_delta = 4;
static INT  cy_delta = 4;

static BOOL gradientVertical = TRUE;

typedef enum
{
   Full,
   Short
} MENUCURSORTYPE;
typedef enum
{
   Single,
   Double
} SEPARATORTYPE;
typedef enum
{
   Left,
   Middle,
   Right
} SEPARATORPOSITION;

static MENUCURSORTYPE eMenuCursorType = Full;

static SEPARATORTYPE eSeparatorType = Double;
static SEPARATORPOSITION eSeparatorPosition = Left;
static BOOL bSelectedItemBorder3d = TRUE;

static COLORREF clrMenuBar1 = RGB( 236, 233, 216 );
static COLORREF clrMenuBar2 = RGB( 236, 233, 216 );

static COLORREF clrMenuBarText = RGB( 0, 0, 0 );
static COLORREF clrMenuBarSelectedText = RGB( 0, 0, 0 );
static COLORREF clrMenuBarGrayedText = RGB( 192, 192, 192 );

static COLORREF clrSelectedMenuBarItem1 = RGB( 255, 252, 248 );
static COLORREF clrSelectedMenuBarItem2 = RGB( 136, 133, 116 );

static COLORREF clrText1 = RGB( 0, 0, 0 );
static COLORREF clrSelectedText1 = RGB( 0, 0, 0 );
static COLORREF clrGrayedText1 = RGB( 192, 192, 192 );

static COLORREF clrBk1 = RGB( 255, 255, 255 );
static COLORREF clrBk2 = RGB( 255, 255, 255 );

static COLORREF clrSelectedBk1 = RGB( 182, 189, 210 );
static COLORREF clrSelectedBk2 = RGB( 182, 189, 210 );

static COLORREF clrGrayedBk1 = RGB( 255, 255, 255 );
static COLORREF clrGrayedBk2 = RGB( 255, 255, 255 );

static COLORREF clrImageBk1 = RGB( 246, 245, 244 );
static COLORREF clrImageBk2 = RGB( 207, 210, 200 );

static COLORREF clrSeparator1 = RGB( 168, 169, 163 );
static COLORREF clrSeparator2 = RGB( 255, 255, 255 );

static COLORREF clrSelectedItemBorder1 = RGB( 10, 36, 106 );
static COLORREF clrSelectedItemBorder2 = RGB( 10, 36, 106 );
static COLORREF clrSelectedItemBorder3 = RGB( 10, 36, 106 );
static COLORREF clrSelectedItemBorder4 = RGB( 10, 36, 106 );

static COLORREF clrCheckMark = RGB( 0, 0, 0 );
static COLORREF clrCheckMarkBk = RGB( 216, 220, 224 );
static COLORREF clrCheckMarkSq = RGB( 64, 116, 200 );
static COLORREF clrCheckMarkGr = RGB( 128, 128, 128 );

static BOOL newStyle = FALSE;

typedef struct _MYMENUITEM
{
   UINT     uiID;
   LPSTR    caption;
   UINT     cch;
   HBITMAP  hBitmap;
   HFONT    hFont;
   UINT     uiItemType;
} MYMENUITEM;

VOID           DrawCheck( HDC hdc, SIZE size, RECT rect, BOOL enabled, BOOL selected, HBITMAP hbitmap );
VOID           DrawGlyph( HDC hDC, int x, int y, int dx, int dy, HBITMAP hBmp, COLORREF rgbTransparent, BOOL disabled, BOOL stretched );
VOID           DrawSeparator( HDC hDC, RECT r );
VOID           DrawBitmapBK( HDC hDC, RECT r );
VOID           DrawItemBk( HDC hDC, RECT r, BOOL fSelected, BOOL fGrayed, UINT itemType, BOOL clear );
VOID           DrawSelectedItemBorder( HDC hDC, RECT r /*, BOOL fSelected, BOOL fGrayed*/, UINT itemType, BOOL clear );
#ifndef __WINNT__
VOID           SetMenuBarColor( HMENU hMenu, COLORREF clrBk, BOOL fSubMenu );
#endif
static BOOL    IsColorEqual( COLORREF clr1, COLORREF clr2 );

extern BOOL    EnabledGradient( void );
extern BOOL    FillGradient( HDC hDC, RECT *rect, BOOL vertical, COLORREF crFrom, COLORREF crTo );
extern HBRUSH  LinearGradientBrush( HDC pDC, long cx, long cy, COLORREF cFrom, COLORREF cTo, BOOL bVert );

/*
 From old c_menu.c
*/

HB_FUNC( TRACKPOPUPMENU )
{
   HWND  hwnd;
   hwnd = ( HWND ) hb_parnl( 4 );

   SetForegroundWindow( hwnd );

   TrackPopupMenu( (HMENU) hb_parnl(1), 0, hb_parni(2), hb_parni(3), 0, hwnd, 0 );
}

HB_FUNC( SETMENU )
{
   SetMenu( (HWND) hb_parnl(1), (HMENU) hb_parnl(2) );
}

HB_FUNC( XCHECKMENUITEM )
{
   CheckMenuItem( (HMENU) hb_parnl(1), hb_parni(2), MF_CHECKED );
}

HB_FUNC( XUNCHECKMENUITEM )
{
   CheckMenuItem( (HMENU) hb_parnl(1), hb_parni(2), MF_UNCHECKED );
}

HB_FUNC( XENABLEMENUITEM )
{
   EnableMenuItem( (HMENU) hb_parnl(1), hb_parni(2), MF_ENABLED );
}

HB_FUNC( XDISABLEMENUITEM )
{
   EnableMenuItem( (HMENU) hb_parnl(1), hb_parni(2), MF_GRAYED );
}

/*
 * From old c_menu.c - modified	
 * HMG 1.3 Ex build 34
*/

HB_FUNC( CREATEMENU )
{
   HMENU hMenu = CreateMenu();
   #ifndef __WINNT__
   if( newStyle )
   {
      SetMenuBarColor( hMenu, clrMenuBar1, TRUE );
   }
   #endif
   hb_retnl( (LONG) hMenu );
}

HB_FUNC( CREATEPOPUPMENU )
{
   HMENU menu = CreatePopupMenu();
   hb_retnl( (LONG) menu );
}

HB_FUNC( APPENDMENUSTRING )
{
   if( newStyle )
   {
      MYMENUITEM  *lpMenuItem;
      UINT        cch = hb_strnlen( hb_parc(3), 255 );

      lpMenuItem = ( MYMENUITEM * ) hb_xgrab( (sizeof(MYMENUITEM)) );
      ZeroMemory( lpMenuItem, sizeof(MYMENUITEM) );

      lpMenuItem->uiID = hb_parni( 2 );
      lpMenuItem->caption = hb_strndup( hb_parc(3), cch );
      lpMenuItem->cch = cch;
      lpMenuItem->hBitmap = ( HBITMAP ) NULL;
      lpMenuItem->hFont = ( HFONT ) NULL;
      lpMenuItem->uiItemType = hb_parnl( 4 );
      hb_retnl( AppendMenu((HMENU) hb_parnl(1), MF_OWNERDRAW, hb_parni(2), (LPTSTR) lpMenuItem) );
   }
   else
   {
      hb_retnl( AppendMenu((HMENU) hb_parnl(1), MF_STRING, hb_parni(2), hb_parc(3)) );
   }
}

HB_FUNC( APPENDMENUPOPUP )
{
   if( newStyle )
   {
      MYMENUITEM  *lpMenuItem;
      UINT        cch = hb_strnlen( hb_parc(3), 255 );

      lpMenuItem = ( MYMENUITEM * ) hb_xgrab( (sizeof(MYMENUITEM)) );
      ZeroMemory( lpMenuItem, sizeof(MYMENUITEM) );

      lpMenuItem->uiID = hb_parni( 2 );
      lpMenuItem->caption = hb_strndup( hb_parc(3), cch );
      lpMenuItem->cch = cch;
      lpMenuItem->hBitmap = ( HBITMAP ) NULL;
      lpMenuItem->hFont = ( HFONT ) NULL;
      lpMenuItem->uiItemType = hb_parnl( 4 );
      hb_retnl( AppendMenu((HMENU) hb_parnl(1), MF_POPUP | MF_OWNERDRAW, hb_parni(2), (LPTSTR) lpMenuItem) );
   }
   else
   {
      hb_retnl( AppendMenu((HMENU) hb_parnl(1), MF_POPUP | MF_STRING, hb_parni(2), hb_parc(3)) );
   }
}

HB_FUNC( APPENDMENUSEPARATOR )
{
   if( newStyle )
   {
      MYMENUITEM  *lpMenuItem;

      lpMenuItem = ( MYMENUITEM * ) hb_xgrab( (sizeof(MYMENUITEM)) );
      ZeroMemory( lpMenuItem, sizeof(MYMENUITEM) );
      lpMenuItem->uiItemType = 1000;
      hb_retnl( AppendMenu((HMENU) hb_parnl(1), MF_SEPARATOR | MF_OWNERDRAW, 0, (LPTSTR) lpMenuItem) );
   }
   else
   {
      hb_retnl( AppendMenu((HMENU) hb_parnl(1), MF_SEPARATOR, 0, NULL) );
   }
}

HB_FUNC( MENUITEM_SETBITMAPS )
{
   HBITMAP  himage1;
   HBITMAP  himage2;

   himage1 = ( HBITMAP ) LoadImage
      (
         GetModuleHandle(NULL),
         hb_parc(3),
         IMAGE_BITMAP,
         0,
         0,
         LR_DEFAULTCOLOR      /* LR_LOADMAP3DCOLORS | LR_LOADTRANSPARENT */
      );
   if( himage1 == NULL )
   {
      himage1 = ( HBITMAP ) LoadImage( 0, hb_parc(3), IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE | LR_DEFAULTCOLOR );
   }

   if( !newStyle )
   {
      himage2 = ( HBITMAP ) LoadImage( GetModuleHandle(NULL), hb_parc(4), IMAGE_BITMAP, 0, 0, LR_DEFAULTCOLOR );
      if( himage2 == NULL )
      {
         himage2 = ( HBITMAP ) LoadImage( 0, hb_parc(4), IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE | LR_DEFAULTCOLOR );
      }
   }

   if( !newStyle )
   {
      SetMenuItemBitmaps( (HMENU) hb_parnl(1), hb_parni(2), MF_BYCOMMAND, himage1, himage2 );
   }
   else
   {
      MENUITEMINFO   MenuItemInfo;
      MYMENUITEM     *pmyMenuItem;

      MenuItemInfo.cbSize = sizeof( MENUITEMINFO );
      MenuItemInfo.fMask = MIIM_DATA;
      GetMenuItemInfo( (HMENU) hb_parnl(1), hb_parni(2), FALSE, &MenuItemInfo );

      pmyMenuItem = ( MYMENUITEM * ) MenuItemInfo.dwItemData;
      if( pmyMenuItem->hBitmap != NULL )
      {
         DeleteObject( pmyMenuItem->hBitmap );
      }

      pmyMenuItem->hBitmap = himage1;
   }
}

HB_FUNC( MENUITEM_SETCHECKMARKS )
{
   if( newStyle )
   {
      MENUITEMINFO   MenuItemInfo;
      HBITMAP        himage1;
      HBITMAP        himage2;

      himage1 = ( HBITMAP ) LoadImage
         (
            GetModuleHandle(NULL),
            hb_parc(3),
            IMAGE_BITMAP,
            0,
            0,
            LR_DEFAULTCOLOR   /* LR_LOADMAP3DCOLORS | LR_LOADTRANSPARENT */
         );
      if( himage1 == NULL )
      {
         himage1 = ( HBITMAP ) LoadImage( 0, hb_parc(3), IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE | LR_DEFAULTCOLOR );
      }
      {
         himage2 = ( HBITMAP ) LoadImage( GetModuleHandle(NULL), hb_parc(4), IMAGE_BITMAP, 0, 0, LR_DEFAULTCOLOR );
         if( himage2 == NULL )
         {
            himage2 = ( HBITMAP ) LoadImage( 0, hb_parc(4), IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE | LR_DEFAULTCOLOR );
         }
      }

      MenuItemInfo.cbSize = sizeof( MENUITEMINFO );
      MenuItemInfo.fMask = MIIM_CHECKMARKS;
      GetMenuItemInfo( (HMENU) hb_parnl(1), hb_parni(2), FALSE, &MenuItemInfo );

      if( MenuItemInfo.hbmpChecked != NULL )
      {
         DeleteObject( MenuItemInfo.hbmpChecked );
      }

      MenuItemInfo.hbmpChecked = himage1;

      if( MenuItemInfo.hbmpUnchecked != NULL )
      {
         DeleteObject( MenuItemInfo.hbmpUnchecked );
      }

      MenuItemInfo.hbmpUnchecked = himage2;

      SetMenuItemInfo( (HMENU) hb_parnl(1), hb_parni(2), FALSE, &MenuItemInfo );
   }
}

HB_FUNC( MENUITEM_SETFONT )
{
   if( newStyle )
   {
      MENUITEMINFO   MenuItemInfo;
      MYMENUITEM     *pmyMenuItem;

      MenuItemInfo.cbSize = sizeof( MENUITEMINFO );
      MenuItemInfo.fMask = MIIM_DATA;
      GetMenuItemInfo( (HMENU) hb_parnl(1), hb_parni(2), FALSE, &MenuItemInfo );

      pmyMenuItem = ( MYMENUITEM * ) MenuItemInfo.dwItemData;

      if( GetObjectType((HGDIOBJ) hb_parnl(3)) == OBJ_FONT )
      {
         if( pmyMenuItem->hFont != NULL )
         {
            DeleteObject( pmyMenuItem->hFont );
         }

         pmyMenuItem->hFont = ( HFONT ) hb_parnl( 3 );
      }
   }
}

HB_FUNC( XGETMENUCAPTION )
{
   if( newStyle )
   {
      MENUITEMINFO   MenuItemInfo;
      MYMENUITEM     *pmyMenuItem;

      MenuItemInfo.cbSize = sizeof( MENUITEMINFO );
      MenuItemInfo.fMask = MIIM_DATA;
      GetMenuItemInfo( (HMENU) hb_parnl(1), hb_parni(2), FALSE, &MenuItemInfo );

      pmyMenuItem = ( MYMENUITEM * ) MenuItemInfo.dwItemData;

      if( pmyMenuItem->caption != NULL )
      {
         hb_retclen( pmyMenuItem->caption, pmyMenuItem->cch );
      }
      else
      {
         hb_retc( "" );
      }
   }
}

HB_FUNC( XSETMENUCAPTION )
{
   if( newStyle )
   {
      MENUITEMINFO   MenuItemInfo;
      MYMENUITEM     *pmyMenuItem;

      MenuItemInfo.cbSize = sizeof( MENUITEMINFO );
      MenuItemInfo.fMask = MIIM_DATA;
      GetMenuItemInfo( (HMENU) hb_parnl(1), hb_parni(2), FALSE, &MenuItemInfo );

      pmyMenuItem = ( MYMENUITEM * ) MenuItemInfo.dwItemData;

      if( pmyMenuItem->caption != NULL )
      {
         UINT  cch = hb_strnlen( hb_parc(3), 255 );

         hb_retclen( pmyMenuItem->caption, pmyMenuItem->cch );

         hb_xfree( pmyMenuItem->caption );

         pmyMenuItem->cch = cch;
         pmyMenuItem->caption = hb_strndup( hb_parc(3), cch );
      }
      else
      {
         hb_retc( "" );
      }
   }
}

HB_FUNC( XGETMENUCHECKSTATE )
{
   UINT  state;
   state = GetMenuState( (HMENU) hb_parnl(1), hb_parni(2), MF_BYCOMMAND );

   if( state != 0xFFFFFFFF )
   {
      hb_retl( (state & MF_CHECKED) ? TRUE : FALSE );
   }
   else
   {
      hb_retl( FALSE );
   }
}

HB_FUNC( XGETMENUENABLEDSTATE )
{
   UINT  state;
   state = GetMenuState( (HMENU) hb_parnl(1), hb_parni(2), MF_BYCOMMAND );

   if( state != 0xFFFFFFFF )
   {
      hb_retl( ((state & MF_GRAYED) || (state & MF_DISABLED)) ? TRUE : FALSE );
   }
   else
   {
      hb_retl( FALSE );
   }
}

/*
 *  Added in HMG Ex. 1.3 build 34
 *  Author: P.Chornyj <myorg63@mail.ru>
*/

HB_FUNC( _NEWMENUSTYLE )
{
   if( hb_pcount() > 0 )
   {
      hb_retl( (newStyle = hb_parl(1)) );
   }
   else
   {
      hb_retl( newStyle );
   }
}

HB_FUNC( ISMENU )
{
   hb_retl( IsMenu((HMENU) hb_parnl(1)) );
}

HB_FUNC( GETMENU )
{
   hb_retnl( (LONG) GetMenu((HWND) hb_parnl(1)) );
}

HB_FUNC( GETMENUITEMCOUNT )
{
   hb_retnl( GetMenuItemCount((HMENU) hb_parnl(1)) );
}

HB_FUNC( _ONMEASUREMENUITEM )
{
   HDC                  hdc;
   LPMEASUREITEMSTRUCT  lpmis;
   MYMENUITEM           *pmyMenuItem;
   UINT                 uiLen;
   SIZE                 size;

   HFONT                oldfont;

   hdc = GetDC( (HWND) hb_parnl(1) );
   lpmis = ( LPMEASUREITEMSTRUCT ) hb_parnl( 4 );
   pmyMenuItem = ( MYMENUITEM * ) lpmis->itemData;

   if( pmyMenuItem->hFont != NULL )
   {
      oldfont = ( HFONT ) SelectObject( hdc, pmyMenuItem->hFont );
   }
   else
   {
      oldfont = ( HFONT ) SelectObject( hdc, GetStockObject(DEFAULT_GUI_FONT) );
   }

   if( pmyMenuItem->uiItemType == 1000 )
   {
      lpmis->itemHeight = 2 * cy_delta;
      lpmis->itemWidth = 0;
   }
   else
   {
      uiLen = hb_strnlen( pmyMenuItem->caption, 255 );
      GetTextExtentPoint32( hdc, pmyMenuItem->caption, uiLen, &size );
   }

   if( pmyMenuItem->uiItemType == 1 )
   {
      lpmis->itemWidth = size.cx;
   }
   else if( lpmis->itemID > 0 )
   {
      lpmis->itemWidth = min_width + cx_delta + size.cx + 8;
   }

   if( pmyMenuItem->uiItemType != 1000 )
   {
      lpmis->itemHeight = ( size.cy > min_height ? size.cy : min_height );
      lpmis->itemHeight += cy_delta;
   }

   SelectObject( hdc, oldfont );

   ReleaseDC( (HWND) hb_parnl(1), hdc );
}

/*
 * Owner draw actions
 *
 * ODA_DRAWENTIRE - This bit is set when the entire control needs to be drawn.
 * ODA_FOCUS - This bit is set when the control gains or loses input focus. The itemState member should be checked to determine whether the control has focus.
 * ODA_SELECT - This bit is set when only the selection status has changed. The itemState member should be checked to determine the new selection state. 

 * Owner draw state
 *
 * ODS_CHECKED - This bit is set if the menu item is to be checked. This bit is used only in a menu.
 * ODS_DISABLED - This bit is set if the item is to be drawn as disabled.
 * ODS_FOCUS - This bit is set if the item has input focus.
 * ODS_GRAYED - This bit is set if the item is to be dimmed. This bit is used only in a menu.
 * ODS_SELECTED - This bit is set if the item's status is selected.
 * ODS_COMBOBOXEDIT - The drawing takes place in the selection field (edit control) of an ownerdrawn combo box.
 * ODS_DEFAULT - The item is the default item. 
*/

HB_FUNC( _ONDRAWMENUITEM )
{
   LPDRAWITEMSTRUCT  lpdis;
   MYMENUITEM        *pmyMenuItem;
   INT               iLen;
   COLORREF          clrText, clrBackground;
   UINT              bkMode;
   BOOL              fSelected = FALSE;
   BOOL              fGrayed = FALSE;
   BOOL              fChecked = FALSE;

   HFONT             oldfont;

   lpdis = ( LPDRAWITEMSTRUCT ) hb_parnl( 1 );
   pmyMenuItem = ( MYMENUITEM * ) lpdis->itemData;

   if( lpdis->CtlType != ODT_MENU )
   {
      return;
   }

   // draw SEPARATOR

   if( lpdis->itemID == 0 )
   {
      DrawSeparator( lpdis->hDC, lpdis->rcItem );
      return;
   }

   if( pmyMenuItem->hFont != NULL )
   {
      oldfont = ( HFONT ) SelectObject( lpdis->hDC, pmyMenuItem->hFont );
   }
   else
   {
      oldfont = ( HFONT ) SelectObject( lpdis->hDC, GetStockObject(DEFAULT_GUI_FONT) );
   }

   // save prev. colours state

   clrText = SetTextColor( lpdis->hDC, clrText1 );
   clrBackground = SetBkColor( lpdis->hDC, clrBk1 );
   bkMode = SetBkMode( lpdis->hDC, TRANSPARENT );

   // set colours and flags ( fSelected etc. )

   if( ((lpdis->itemAction & ODA_SELECT) || (lpdis->itemAction & ODA_DRAWENTIRE)) && (!(lpdis->itemState & ODS_GRAYED)) )
   {
      if( lpdis->itemState & ODS_SELECTED )
      {
         clrText = SetTextColor( lpdis->hDC, (pmyMenuItem->uiItemType != 1) ? clrSelectedText1 : clrMenuBarSelectedText );
         clrBackground = SetBkColor( lpdis->hDC, (pmyMenuItem->uiItemType != 1) ? clrSelectedBk1 : clrMenuBar1 );
         fSelected = TRUE;
      }
      else
      {
         clrText = SetTextColor( lpdis->hDC, (pmyMenuItem->uiItemType != 1) ? clrText1 : clrMenuBarText );
         clrBackground = SetBkColor( lpdis->hDC, (pmyMenuItem->uiItemType != 1) ? clrBk2 : clrMenuBar2 );
         fSelected = FALSE;
      }
   }

   if( lpdis->itemState & ODS_GRAYED )
   {
      clrText = SetTextColor( lpdis->hDC, (pmyMenuItem->uiItemType != 1) ? clrGrayedText1 : clrMenuBarGrayedText );
      fGrayed = TRUE;
   }

   if( lpdis->itemState & ODS_CHECKED )
   {
      fChecked = TRUE;
   }

   // draw menu item BITMAP BACKGROUND

   if( pmyMenuItem->uiItemType != 1 )
   {
      DrawBitmapBK( lpdis->hDC, lpdis->rcItem );
   }

   // draw menu ITEM BACKGROUND

   DrawItemBk( lpdis->hDC, lpdis->rcItem, fSelected, fGrayed, pmyMenuItem->uiItemType, ((pmyMenuItem->hBitmap == NULL) && (!fChecked)) );

   // draw menu ITEM BORDER

   if( fSelected && (!fGrayed) )
   {
      DrawSelectedItemBorder
      (
         lpdis->hDC,
         lpdis->rcItem,       /* fSelected, fGrayed,*/
         pmyMenuItem->uiItemType,
         ((pmyMenuItem->hBitmap == NULL) && (!fChecked))
      );
   }

   // draw BITMAP

   if( (pmyMenuItem->hBitmap != NULL) && (!fChecked) )
   {
      DrawGlyph
      (
         lpdis->hDC,
         lpdis->rcItem.left + cx_delta - 2,
         lpdis->rcItem.top + cy_delta,
         bm_size,
         bm_size,
         pmyMenuItem->hBitmap,
         (COLORREF) RGB(125, 125, 125),
         ((fGrayed) ? TRUE : FALSE),
         TRUE
      );

      if( fSelected && (pmyMenuItem->uiItemType != 1) && (eMenuCursorType == Short) && bSelectedItemBorder3d )
      {
         HPEN  pen, pen1, oldPen;
         RECT  rect;

         pen = CreatePen( 0, (UINT) 1, clrSelectedItemBorder2 );
         pen1 = CreatePen( 0, (UINT) 1, clrSelectedItemBorder4 );

         oldPen = SelectObject( lpdis->hDC, pen1 );

         CopyRect( &rect, &lpdis->rcItem );
         rect.left += ( cx_delta / 2 - 2 );
         rect.top += ( cy_delta / 2 );
         rect.right = rect.left + bm_size + cx_delta;
         rect.bottom = rect.top + bm_size + cy_delta;

         MoveToEx( lpdis->hDC, rect.left, rect.top, NULL );

         LineTo( lpdis->hDC, rect.right, rect.top );
         SelectObject( lpdis->hDC, pen );
         LineTo( lpdis->hDC, rect.right, rect.bottom );
         LineTo( lpdis->hDC, rect.left, rect.bottom );
         SelectObject( lpdis->hDC, pen1 );
         LineTo( lpdis->hDC, rect.left, rect.top );

         SelectObject( lpdis->hDC, oldPen );

         DeleteObject( pen );
         DeleteObject( pen1 );
      }
   }

   //  draw menu ITEM TEXT

   iLen = hb_strnlen( pmyMenuItem->caption, 255 );

   if( pmyMenuItem->uiItemType == 1 )
   {
      DrawText( lpdis->hDC, pmyMenuItem->caption, iLen, &lpdis->rcItem, DT_CENTER | DT_VCENTER | DT_SINGLELINE | DT_END_ELLIPSIS | DT_EXPANDTABS );
   }
   else
   {
      lpdis->rcItem.left += ( min_width + cx_delta + 2 );
      DrawText( lpdis->hDC, pmyMenuItem->caption, iLen, &lpdis->rcItem, DT_LEFT | DT_VCENTER | DT_SINGLELINE | DT_END_ELLIPSIS | DT_EXPANDTABS );
      lpdis->rcItem.left -= ( min_width + cx_delta + 2 );
   }

   //  draw menu item checked mark

   if( fChecked )
   {
      MENUITEMINFO   MenuItemInfo;
      SIZE           size;

      MenuItemInfo.cbSize = sizeof( MENUITEMINFO );
      MenuItemInfo.fMask = MIIM_CHECKMARKS;
      GetMenuItemInfo( (HMENU) lpdis->hwndItem, lpdis->itemID, FALSE, &MenuItemInfo );

      size.cx = bm_size;      //GetSystemMetrics(SM_CXMENUCHECK);
      size.cy = bm_size;      //GetSystemMetrics(SM_CYMENUCHECK);
      DrawCheck( lpdis->hDC, size, lpdis->rcItem, fGrayed, fSelected, MenuItemInfo.hbmpChecked );
   }

   SelectObject( lpdis->hDC, oldfont );

   // restore prev. colours state

   SetTextColor( lpdis->hDC, clrText );
   SetBkColor( lpdis->hDC, clrBackground );
   SetBkMode( lpdis->hDC, bkMode );
}

VOID DrawSeparator( HDC hDC, RECT r )
{
   HPEN  pen, oldPen;
   RECT  rect;

   CopyRect( &rect, &r );
   rect.right = rect.left + min_width + cx_delta / 2;

   if( (EnabledGradient()) && (!IsColorEqual(clrImageBk1, clrImageBk2)) )
   {
      FillGradient( hDC, &rect, FALSE, clrImageBk1, clrImageBk2 );
   }
   else
   {
      HBRUSH   brush = CreateSolidBrush( clrImageBk1 );
      FillRect( hDC, &rect, brush );
      DeleteObject( brush );
   }

   CopyRect( &rect, &r );
   rect.left += ( min_width + cx_delta / 2 );

   if( (EnabledGradient()) && (!IsColorEqual(clrBk1, clrBk2)) )
   {
      FillGradient( hDC, &rect, FALSE, clrBk1, clrBk2 );
   }
   else
   {
      HBRUSH   brush = CreateSolidBrush( clrBk1 );
      FillRect( hDC, &rect, brush );
      DeleteObject( brush );
   }

   CopyRect( &rect, &r );

   pen = CreatePen( 0, (UINT) 1, clrSeparator1 );
   oldPen = SelectObject( hDC, pen );

   if( eSeparatorPosition == Right )
   {
      rect.left += ( min_width + cx_delta + 2 );
   }
   else if( eSeparatorPosition == Middle )
   {
      rect.left += ( min_width - cx_delta );
      rect.right -= ( min_width - cx_delta );
   }

   rect.top += ( rect.bottom - rect.top ) / 2;
   MoveToEx( hDC, rect.left, rect.top, NULL );
   LineTo( hDC, rect.right, rect.top );

   if( eSeparatorType == Double )
   {
      HPEN  pen1, oldPen1;
      pen1 = CreatePen( 0, (UINT) 1, clrSeparator2 );
      oldPen1 = SelectObject( hDC, pen1 );

      rect.top += 1;
      MoveToEx( hDC, rect.left, rect.top, NULL );
      LineTo( hDC, rect.right, rect.top );

      SelectObject( hDC, oldPen1 );
      DeleteObject( pen1 );
   }

   SelectObject( hDC, oldPen );
   DeleteObject( pen );
}

VOID DrawBitmapBK( HDC hDC, RECT r )
{
   RECT  rect;

   CopyRect( &rect, &r );
   rect.right = rect.left + min_width + cx_delta / 2;

   if( (EnabledGradient()) && (!IsColorEqual(clrImageBk1, clrImageBk2)) )
   {
      FillGradient( hDC, &rect, FALSE, clrImageBk1, clrImageBk2 );
   }
   else
   {
      HBRUSH   brush = CreateSolidBrush( clrImageBk1 );
      FillRect( hDC, &rect, brush );
      DeleteObject( brush );
   }
}

VOID DrawItemBk( HDC hDC, RECT r, BOOL Selected, BOOL Grayed, UINT itemType, BOOL clear )
{
   RECT  rect;

   CopyRect( &rect, &r );
   if( (!Selected) && (itemType != 1) )
   {
      rect.left += ( min_width + cx_delta / 2 );
   }

   if( (Selected) && (itemType != 1) && (eMenuCursorType == Short) && (!clear) )
   {
      rect.left += ( min_width + cx_delta / 2 );
   }

   if( !Grayed )
   {
      if( Selected )
      {
         if( EnabledGradient() )
         {
            FillGradient
            (
               hDC,
               &rect,
               (itemType == 1) ? TRUE : gradientVertical,
               (itemType == 1) ? clrSelectedMenuBarItem1 : clrSelectedBk1,
               (itemType == 1) ? clrSelectedMenuBarItem2 : clrSelectedBk2
            );
         }
         else
         {
            HBRUSH   brush = CreateSolidBrush( (itemType == 1) ? clrSelectedMenuBarItem1 : clrSelectedBk1 );
            FillRect( hDC, &rect, brush );
            DeleteObject( brush );
         }
      }
      else
      {
         if( (EnabledGradient()) && ((!IsColorEqual(clrMenuBar1, clrMenuBar2)) || (!IsColorEqual(clrBk1, clrBk2)) && (itemType != 1)) )
         {
            FillGradient
            (
               hDC,
               &rect,
               ((itemType == 1) ? TRUE : FALSE),
               ((itemType == 1) ? clrMenuBar1 : clrBk1),
               ((itemType == 1) ? clrMenuBar2 : clrBk2)
            );
         }
         else
         {
            HBRUSH   brush = CreateSolidBrush( (itemType == 1) ? clrMenuBar1 : clrBk1 );
            FillRect( hDC, &rect, brush );
            DeleteObject( brush );
         }
      }
   }
   else
   {
      if( EnabledGradient() )
      {
         FillGradient( hDC, &rect, FALSE, clrGrayedBk1, clrGrayedBk2 );
      }
      else
      {
         HBRUSH   brush = CreateSolidBrush( clrGrayedBk1 );
         FillRect( hDC, &rect, brush );
         DeleteObject( brush );
      }
   }
}

VOID DrawSelectedItemBorder( HDC hDC, RECT r,

/*BOOL fSelected, BOOL fGrayed,*/ UINT itemType, BOOL clear )
{
   HPEN  pen, pen1, oldPen;
   RECT  rect;

   if( itemType != 1 )
   {
      pen = CreatePen( 0, (UINT) 1, clrSelectedItemBorder1 );
      pen1 = CreatePen( 0, (UINT) 1, clrSelectedItemBorder3 );
   }
   else
   {
      pen = CreatePen( 0, (UINT) 1, clrSelectedItemBorder2 );
      pen1 = CreatePen( 0, (UINT) 1, clrSelectedItemBorder4 );
   }

   oldPen = SelectObject( hDC, pen );

   CopyRect( &rect, &r );
   if( (eMenuCursorType == Short) && (itemType != 1) && (!clear) )
   {
      rect.left += ( min_width + cx_delta / 2 );
   }

   InflateRect( &rect, -1, -1 );

   MoveToEx( hDC, rect.left, rect.top, NULL );

   if( (itemType == 1) && bSelectedItemBorder3d )
   {
      LineTo( hDC, rect.right, rect.top );
      SelectObject( hDC, pen1 );
      LineTo( hDC, rect.right, rect.bottom );
      LineTo( hDC, rect.left, rect.bottom );
      SelectObject( hDC, pen );
      LineTo( hDC, rect.left, rect.top );
   }
   else
   {
      LineTo( hDC, rect.right, rect.top );
      LineTo( hDC, rect.right, rect.bottom );
      LineTo( hDC, rect.left, rect.bottom );
      LineTo( hDC, rect.left, rect.top );
   }

   SelectObject( hDC, oldPen );
   DeleteObject( pen );
   DeleteObject( pen1 );
}

VOID DrawCheck( HDC hdc, SIZE size, RECT rect, BOOL disabled, BOOL selected, HBITMAP hbitmap )
{
   if( hbitmap != 0 )
   {
      DrawGlyph
      (
         hdc,
         rect.left + cx_delta / 2,
         rect.top + cy_delta / 2 + 2,
         size.cx,
         size.cy,
         hbitmap,
         (COLORREF) RGB(125, 125, 125),
         ((disabled) ? TRUE : FALSE),
         TRUE
      );
   }
   else
   {
      HPEN     pen, oldPen;
      HBRUSH   brush, oldBrush;
      UINT     x, y, w, h;

      if( (selected) && (eMenuCursorType != Short) )
      {
         brush = CreateSolidBrush( clrSelectedBk1 );
      }
      else
      {
         brush = CreateSolidBrush( clrCheckMarkBk );
      }

      oldBrush = SelectObject( hdc, brush );

      pen = CreatePen( 0, (UINT) 1, clrCheckMarkSq );
      oldPen = SelectObject( hdc, pen );

      w = ( size.cx > min_width ? min_width : size.cx );
      h = w;
      x = rect.left + ( min_width - w ) / 2;
      y = rect.top + ( min_height + cy_delta - h ) / 2;

      Rectangle( hdc, x, y, x + w, y + h );

      DeleteObject( pen );

      if( disabled )
      {
         pen = CreatePen( 0, (UINT) 1, clrCheckMarkGr );
      }
      else
      {
         pen = CreatePen( 0, (UINT) 1, clrCheckMark );
      }

      SelectObject( hdc, pen );

      MoveToEx( hdc, x + 1, y + 5, NULL );
      LineTo( hdc, x + 4, y + h - 2 );
      MoveToEx( hdc, x + 2, y + 5, NULL );
      LineTo( hdc, x + 4, y + h - 3 );
      MoveToEx( hdc, x + 2, y + 4, NULL );
      LineTo( hdc, x + 5, y + h - 3 );
      MoveToEx( hdc, x + 4, y + h - 3, NULL );
      LineTo( hdc, x + w + 2, y - 1 );
      MoveToEx( hdc, x + 4, y + h - 2, NULL );
      LineTo( hdc, x + w - 2, y + 3 );

      SelectObject( hdc, oldPen );
      SelectObject( hdc, oldBrush );

      DeleteObject( pen );
      DeleteObject( brush );
   }
}

static BOOL IsColorEqual( COLORREF clr1, COLORREF clr2 )
{
   return( ((GetRValue(clr1) == GetRValue(clr2)) && (GetGValue(clr1) == GetGValue(clr2)) && (GetBValue(clr1) == GetBValue(clr2))) ? TRUE : FALSE );
}

#ifndef __WINNT__
VOID SetMenuBarColor( HMENU hMenu, COLORREF clrBk, BOOL fSubMenu )
{
   MENUINFO MenuInfo;

   MenuInfo.cbSize = sizeof( MENUINFO );
   GetMenuInfo( hMenu, &MenuInfo );

   MenuInfo.fMask = MIM_BACKGROUND;
   if( fSubMenu )
   {
      MenuInfo.fMask |= MIM_APPLYTOSUBMENUS;
   }

   MenuInfo.hbrBack = CreateSolidBrush( clrBk );
   SetMenuInfo( hMenu, &MenuInfo );
}
#endif
HB_FUNC( SETMENUBITMAPHEIGHT )
{
   bm_size = hb_parni( 1 );
   min_height = min_width = bm_size + 4;
   hb_retni( bm_size );
}

HB_FUNC( GETMENUBITMAPHEIGHT )
{
   hb_retni( bm_size );
}

HB_FUNC( GETMENUCOLORS )
{
   PHB_ITEM aResult = _itemArrayNew( 28 );
   PHB_ITEM tmp;

   tmp = _itemPutNL( NULL, clrMenuBar1 );
   hb_arraySet( aResult, 1, tmp );

   tmp = _itemPutNL( NULL, clrMenuBar2 );
   hb_arraySet( aResult, 2, tmp );

   tmp = _itemPutNL( NULL, clrMenuBarText );
   hb_arraySet( aResult, 3, tmp );

   tmp = _itemPutNL( NULL, clrMenuBarSelectedText );
   hb_arraySet( aResult, 4, tmp );

   tmp = _itemPutNL( NULL, clrMenuBarGrayedText );
   hb_arraySet( aResult, 5, tmp );

   tmp = _itemPutNL( NULL, clrSelectedMenuBarItem1 );
   hb_arraySet( aResult, 6, tmp );

   tmp = _itemPutNL( NULL, clrSelectedMenuBarItem2 );
   hb_arraySet( aResult, 7, tmp );

   tmp = _itemPutNL( NULL, clrText1 );
   hb_arraySet( aResult, 8, tmp );

   tmp = _itemPutNL( NULL, clrSelectedText1 );
   hb_arraySet( aResult, 9, tmp );

   tmp = _itemPutNL( NULL, clrGrayedText1 );
   hb_arraySet( aResult, 10, tmp );

   tmp = _itemPutNL( NULL, clrBk1 );
   hb_arraySet( aResult, 11, tmp );

   tmp = _itemPutNL( NULL, clrBk2 );
   hb_arraySet( aResult, 12, tmp );

   tmp = _itemPutNL( NULL, clrSelectedBk1 );
   hb_arraySet( aResult, 13, tmp );

   tmp = _itemPutNL( NULL, clrSelectedBk2 );
   hb_arraySet( aResult, 14, tmp );

   tmp = _itemPutNL( NULL, clrGrayedBk1 );
   hb_arraySet( aResult, 15, tmp );

   tmp = _itemPutNL( NULL, clrGrayedBk2 );
   hb_arraySet( aResult, 16, tmp );

   tmp = _itemPutNL( NULL, clrImageBk1 );
   hb_arraySet( aResult, 17, tmp );

   tmp = _itemPutNL( NULL, clrImageBk2 );
   hb_arraySet( aResult, 18, tmp );

   tmp = _itemPutNL( NULL, clrSeparator1 );
   hb_arraySet( aResult, 19, tmp );

   tmp = _itemPutNL( NULL, clrSeparator2 );
   hb_arraySet( aResult, 20, tmp );

   tmp = _itemPutNL( NULL, clrSelectedItemBorder1 );
   hb_arraySet( aResult, 21, tmp );

   tmp = _itemPutNL( NULL, clrSelectedItemBorder2 );
   hb_arraySet( aResult, 22, tmp );

   tmp = _itemPutNL( NULL, clrSelectedItemBorder3 );
   hb_arraySet( aResult, 23, tmp );

   tmp = _itemPutNL( NULL, clrSelectedItemBorder4 );
   hb_arraySet( aResult, 24, tmp );

   tmp = _itemPutNL( NULL, clrCheckMark );
   hb_arraySet( aResult, 25, tmp );

   tmp = _itemPutNL( NULL, clrCheckMarkBk );
   hb_arraySet( aResult, 26, tmp );

   tmp = _itemPutNL( NULL, clrCheckMarkSq );
   hb_arraySet( aResult, 27, tmp );

   tmp = _itemPutNL( NULL, clrCheckMarkGr );
   hb_arraySet( aResult, 28, tmp );

   _itemRelease( tmp );
   _itemReturn( aResult );
   _itemRelease( aResult );
}

HB_FUNC( SETMENUCOLORS )
{
   PHB_ITEM pArray = hb_param( 1, HB_IT_ARRAY );

   if( (pArray != NULL) && (hb_arrayLen(pArray) >= 28) )
   {
      clrMenuBar1 = ( COLORREF ) hb_parnl( 1, 1 );
      clrMenuBar2 = ( COLORREF ) hb_parnl( 1, 2 );
      clrMenuBarText = ( COLORREF ) hb_parnl( 1, 3 );
      clrMenuBarSelectedText = ( COLORREF ) hb_parnl( 1, 4 );
      clrMenuBarGrayedText = ( COLORREF ) hb_parnl( 1, 5 );
      clrSelectedMenuBarItem1 = ( COLORREF ) hb_parnl( 1, 6 );
      clrSelectedMenuBarItem2 = ( COLORREF ) hb_parnl( 1, 7 );
      clrText1 = ( COLORREF ) hb_parnl( 1, 8 );
      clrSelectedText1 = ( COLORREF ) hb_parnl( 1, 9 );
      clrGrayedText1 = ( COLORREF ) hb_parnl( 1, 10 );
      clrBk1 = ( COLORREF ) hb_parnl( 1, 11 );
      clrBk2 = ( COLORREF ) hb_parnl( 1, 12 );
      clrSelectedBk1 = ( COLORREF ) hb_parnl( 1, 13 );
      clrSelectedBk2 = ( COLORREF ) hb_parnl( 1, 14 );
      clrGrayedBk1 = ( COLORREF ) hb_parnl( 1, 15 );
      clrGrayedBk2 = ( COLORREF ) hb_parnl( 1, 16 );
      clrImageBk1 = ( COLORREF ) hb_parnl( 1, 17 );
      clrImageBk2 = ( COLORREF ) hb_parnl( 1, 18 );
      clrSeparator1 = ( COLORREF ) hb_parnl( 1, 19 );
      clrSeparator2 = ( COLORREF ) hb_parnl( 1, 20 );
      clrSelectedItemBorder1 = ( COLORREF ) hb_parnl( 1, 21 );
      clrSelectedItemBorder2 = ( COLORREF ) hb_parnl( 1, 22 );
      clrSelectedItemBorder3 = ( COLORREF ) hb_parnl( 1, 23 );
      clrSelectedItemBorder4 = ( COLORREF ) hb_parnl( 1, 24 );
      clrCheckMark = ( COLORREF ) hb_parnl( 1, 25 );
      clrCheckMarkBk = ( COLORREF ) hb_parnl( 1, 26 );
      clrCheckMarkSq = ( COLORREF ) hb_parnl( 1, 27 );
      clrCheckMarkGr = ( COLORREF ) hb_parnl( 1, 28 );
   }
}

HB_FUNC( SETMENUSEPARATORTYPE )
{
   eSeparatorType = ( SEPARATORTYPE ) hb_parni( 1 );
   eSeparatorPosition = ( SEPARATORPOSITION ) hb_parni( 2 );
}

HB_FUNC( SETMENUSELECTEDITEM3D )
{
   bSelectedItemBorder3d = ( BOOL ) hb_parl( 1 );
}

HB_FUNC( SETMENUCURSORTYPE )
{
   eMenuCursorType = ( MENUCURSORTYPE ) hb_parni( 1 );
}

HB_FUNC( _ONDESTROYMENU )
{
   #ifndef __POCC__

   HMENU menu = ( HMENU ) hb_parnl( 1 );

   if( IsMenu(menu) )
   {
      hb_retl( DestroyMenu(menu) );
   }
   else
      #endif
      hb_retl( FALSE );
}

#ifndef __POCC__
BOOL WINAPI DestroyMenu( HMENU menu )
{
   INT   i;
   for( i = 0; i < GetMenuItemCount(menu); i++ )
   {
      MENUITEMINFO   MenuItemInfo;
      HMENU          pSubMenu;

      MenuItemInfo.cbSize = sizeof( MENUITEMINFO );
      MenuItemInfo.fMask = MIIM_CHECKMARKS | MIIM_DATA;

      GetMenuItemInfo( menu, i, TRUE, &MenuItemInfo );

      if( MenuItemInfo.hbmpChecked != NULL )
      {
         DeleteObject( MenuItemInfo.hbmpChecked );
      }

      if( MenuItemInfo.hbmpUnchecked != NULL )
      {
         DeleteObject( MenuItemInfo.hbmpUnchecked );
      }

      if( newStyle )
      {
         MYMENUITEM  *pmyMenuItem;
         pmyMenuItem = ( MYMENUITEM * ) MenuItemInfo.dwItemData;

         if( pmyMenuItem->caption != NULL )
         {
            hb_xfree( pmyMenuItem->caption );
         }

         if( pmyMenuItem->hBitmap != NULL )
         {
            DeleteObject( pmyMenuItem->hBitmap );
         }

         if( pmyMenuItem->hFont != NULL )
         {
            DeleteObject( pmyMenuItem->hFont );
         }
      }

      pSubMenu = GetSubMenu( menu, i );

      if( pSubMenu != NULL )
      {
         DestroyMenu( pSubMenu );
      }
   }

   return TRUE;
}
#endif
