/*----------------------------------------------------------------------------
 MINIGUI - Harbour Win32 GUI library source code

 Copyright 2002-2008 Roberto Lopez <harbourminigui@gmail.com>
 http://harbourminigui.googlepages.com/

 This program is free software; you can redistribute it and/or modify it under 
 the terms of the GNU General Public License as published by the Free Software 
 Foundation; either version 2 of the License, or (at your option) any later 
 version. 

 This program is distributed in the hope that it will be useful, but WITHOUT 
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS 
 FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

 You should have received a copy of the GNU General Public License along with 
 this software; see the file COPYING. If not, write to the Free Software 
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA (or 
 visit the web site http://www.gnu.org/).

 As a special exception, you have permission for additional uses of the text 
 contained in this release of Harbour Minigui.

 The exception is that, if you link the Harbour Minigui library with other 
 files to produce an executable, this does not by itself cause the resulting 
 executable to be covered by the GNU General Public License.
 Your use of that executable is in no way restricted on account of linking the 
 Harbour-Minigui library code into it.

 Parts of this project are based upon:

	"Harbour GUI framework for Win32"
 	Copyright 2001 Alexander S.Kresin <alex@belacy.belgorod.su>
 	Copyright 2001 Antonio Linares <alinares@fivetech.com>
	www - http://www.harbour-project.org

	"Harbour Project"
	Copyright 1999-2008, http://www.harbour-project.org/

	"WHAT32"
	Copyright 2002 AJ Wos <andrwos@aust1.net> 

	"HWGUI"
  	Copyright 2001-2007 Alexander S.Kresin <alex@belacy.belgorod.su>

---------------------------------------------------------------------------*/

#define _WIN32_IE 0x0500
#define HB_OS_WIN_32_USED
#define _WIN32_WINNT 0x0400
#include <shlobj.h>

#include <windows.h>
#include <commctrl.h>
#include <richedit.h>
#include "hbapi.h"
#include "hbvm.h"
#include "hbstack.h"
#include "hbapiitm.h"

#include "Winuser.h"
#include <wingdi.h>

HB_FUNC( INITRICHEDITBOX )
{
   HWND  hwnd;
   HWND  hwndRE;
   int   Style;

   hwnd = ( HWND ) hb_parnl( 1 );

   Style = ES_MULTILINE | ES_WANTRETURN | WS_CHILD | ES_NOHIDESEL;

   if( hb_parl(10) )
   {
      Style = Style | ES_READONLY;
   }

   if( !hb_parl(11) )
   {
      Style = Style | WS_VISIBLE;
   }

   if( !hb_parl(12) )
   {
      Style = Style | WS_TABSTOP;
   }

   if( !hb_parl(13) )
   {
      Style = Style | WS_HSCROLL;
   }

   if( !hb_parl(14) )
   {
      Style = Style | WS_VSCROLL;
   }
   else
   {
      Style = Style | ES_AUTOVSCROLL;
   }

   InitCommonControls();
   if( LoadLibrary("RichEd20.dll") )
   {
      hwndRE = CreateWindowEx
         (
            WS_EX_CLIENTEDGE,
            RICHEDIT_CLASS,
            (LPSTR) NULL,
            Style,
            hb_parni(3),
            hb_parni(4),
            hb_parni(5),
            hb_parni(6),
            hwnd,
            (HMENU) hb_parni(2),
            GetModuleHandle(NULL),
            NULL
         );

      SendMessage( hwndRE, (UINT) EM_LIMITTEXT, (WPARAM) hb_parni(9), (LPARAM) 0 );
      SendMessage( hwndRE, (UINT) EM_SETEVENTMASK, (WPARAM) 0, (LPARAM) ENM_SELCHANGE | ENM_DRAGDROPDONE | ENM_CHANGE | ENM_SCROLL );
   }

   hb_retnl( (LONG) hwndRE );
}

DWORD CALLBACK EditStreamCallbackR( DWORD dwCookie, LPBYTE lpbBuff, LONG cb, LONG FAR *pcb )
{
   HANDLE   hFile = ( HANDLE ) dwCookie;

   if( !ReadFile(hFile, (LPVOID) lpbBuff, cb, (LPDWORD) pcb, NULL) )
   {
      return( (DWORD) - 1 );
   }

   return( 0 );
}

DWORD CALLBACK EditStreamCallbackW( DWORD dwCookie, LPBYTE lpbBuff, LONG cb, LONG FAR *pcb )
{
   HANDLE   hFile = ( HANDLE ) dwCookie;

   if( !WriteFile(hFile, (LPVOID) lpbBuff, cb, (LPDWORD) pcb, NULL) )
   {
      return( (DWORD) - 1 );
   }

   return( 0 );
}

HB_FUNC( STREAMIN )        //StreamIn(HWND hwndCtrl, LPCTSTR lpszPath, int typ )
{
   HWND        hwnd;
   HANDLE      hFile;
   EDITSTREAM  es;
   long        Flag, Mode;

   hwnd = ( HWND ) hb_parnl( 1 );
   switch( hb_parni(3) )
   {
      case 1:  { Flag = SF_TEXT; Mode = TM_PLAINTEXT; break; }
      case 2:  { Flag = SF_RTF; Mode = TM_RICHTEXT; break; }
      case 3:  { Flag = SF_TEXT | SF_UNICODE; Mode = TM_PLAINTEXT; break; }
      case 4:  { Flag = ( CP_UTF8 << 16 ) | SF_USECODEPAGE | SF_TEXT; Mode = TM_PLAINTEXT; break; }
      case 5:  { Flag = ( CP_UTF8 << 16 ) | SF_USECODEPAGE | SF_RTF; Mode = TM_RICHTEXT; break; }
      case 6:  { Flag = ( CP_UTF7 << 16 ) | SF_USECODEPAGE | SF_TEXT; Mode = TM_PLAINTEXT; break; }
      default: { Flag = SF_TEXT; Mode = TM_PLAINTEXT; }
   }

   /* open the source file. */

   if( (hFile = CreateFile(hb_parc(2), GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL)) == INVALID_HANDLE_VALUE )
   {
      hb_retl( FALSE );
   }

   es.dwCookie = ( DWORD ) hFile;
   es.dwError = 0;
   es.pfnCallback = EditStreamCallbackR;

   /* send EM_STREAMIN message to the Rich Edit Control. */

   SendMessage( hwnd, (UINT) EM_STREAMIN, (WPARAM) Flag, (LPARAM) & es );
   SendMessage( hwnd, (UINT) EM_SETTEXTMODE, (WPARAM) Mode, 0 );

   CloseHandle( hFile );

   if( es.dwError )
   {
      hb_retl( FALSE );
   }
   else
   {
      hb_retl( TRUE );
   }
}

HB_FUNC( STREAMOUT )       //StreamOut(HWND hwndCtrl, LPCTSTR lpszPath, int Typ )
{
   HWND        hwnd;
   HANDLE      hFile;
   EDITSTREAM  es;
   long        Flag;

   hwnd = ( HWND ) hb_parnl( 1 );
   switch( hb_parni(3) )
   {
      case 1:  Flag = SF_TEXT; break;
      case 2:  Flag = SF_RTF; break;
      case 3:  Flag = SF_TEXT | SF_UNICODE; break;
      case 4:  Flag = ( CP_UTF8 << 16 ) | SF_USECODEPAGE | SF_TEXT; break;
      case 5:  Flag = ( CP_UTF8 << 16 ) | SF_USECODEPAGE | SF_RTF; break;
      case 6:  Flag = ( CP_UTF7 << 16 ) | SF_USECODEPAGE | SF_TEXT; break;
      default: Flag = SF_TEXT;
   }

   /* open the destination file. */

   if( (hFile = CreateFile(hb_parc(2), GENERIC_WRITE, FILE_SHARE_WRITE, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL)) == INVALID_HANDLE_VALUE )
   {
      hb_retl( FALSE );
   }

   es.dwCookie = ( DWORD ) hFile;
   es.dwError = 0;
   es.pfnCallback = EditStreamCallbackW;

   /* send EM_STREAMOUT message to the Rich Edit Control. */

   SendMessage( hwnd, EM_STREAMOUT, (WPARAM) Flag, (LPARAM) & es );

   CloseHandle( hFile );

   if( es.dwError )
   {
      hb_retl( FALSE );
   }
   else
   {
      hb_retl( TRUE );
   }
}

HB_FUNC( GETAUTOFONTRTF )  // GetAutoFont(HWND hwnd)
{
   HWND     hwnd;
   LRESULT  lAuto;

   hwnd = ( HWND ) hb_parnl( 1 );
   lAuto = SendMessage( hwnd, EM_GETLANGOPTIONS, 0, 0 ) & IMF_AUTOFONT;

   if( lAuto )
   {
      hb_retl( TRUE );
   }
   else
   {
      hb_retl( FALSE );
   }
}

HB_FUNC( SETAUTOFONTRTF )  // SetAutoFont(HWND hwnd, lAutoFont)
{
   HWND     hwnd;
   LRESULT  lOpt, lResult;

   hwnd = ( HWND ) hb_parnl( 1 );
   lOpt = SendMessage( hwnd, EM_GETLANGOPTIONS, 0, 0 );

   if( hb_parl(2) )
   {
      lOpt &= IMF_AUTOFONT;
   }
   else
   {
      lOpt &= ~IMF_AUTOFONT;
   }

   lResult = SendMessage( hwnd, EM_SETLANGOPTIONS, 0, lOpt );

   if( lResult )
   {
      hb_retl( TRUE );
   }
   else
   {
      hb_retl( FALSE );
   }
}

HB_FUNC( SETBKGNDCOLOR )   // SetBkgndColor(HWND hwnd, lSyscol, nRed, nGreen, nBlue)
{
   LRESULT  lResult;
   COLORREF bkgcolor;
   INT      syscol = 1;

   bkgcolor = ( COLORREF ) RGB( hb_parni(3), hb_parni(4), hb_parni(5) );
   if( hb_parl(2) )
   {
      syscol = 0;
   }

   lResult = SendMessage( (HWND) hb_parnl(1), (UINT) EM_SETBKGNDCOLOR, (WPARAM) syscol, (LPARAM) bkgcolor );

   hb_retnl( lResult );
}

HB_FUNC( GETFONTRTF )
{
   CHARFORMAT  cF;
   long        PointSize;
   int         bold;
   int         Italic;
   int         Underline;
   int         StrikeOut;
   int         SelText;

   cF.cbSize = sizeof( CHARFORMAT );
   cF.dwMask = CFM_BOLD | CFM_ITALIC | CFM_UNDERLINE | CFM_SIZE;
   if( hb_parni(2) > 0 )
   {
      SelText = SCF_SELECTION;
   }
   else
   {
      SelText = SCF_DEFAULT;
   }

   SendMessage( (HWND) hb_parnl(1), EM_GETCHARFORMAT, (WPARAM) SelText, (LPARAM) & cF );

   PointSize = cF.yHeight / 20;

   bold = ( cF.dwEffects & CFE_BOLD ) ? 1 : 0;
   Italic = ( cF.dwEffects & CFE_ITALIC ) ? 1 : 0;
   Underline = ( cF.dwEffects & CFE_UNDERLINE ) ? 1 : 0;
   StrikeOut = ( cF.dwEffects & CFE_STRIKEOUT ) ? 1 : 0;

   hb_reta( 8 );
   hb_storc( cF.szFaceName, -1, 1 );
   hb_stornl( (LONG) PointSize, -1, 2 );
   hb_storl( bold, -1, 3 );
   hb_storl( Italic, -1, 4 );
   hb_stornl( cF.crTextColor, -1, 5 );
   hb_storl( Underline, -1, 6 );
   hb_storl( StrikeOut, -1, 7 );
   hb_storni( cF.bCharSet, -1, 8 );
}

HB_FUNC( SETFONTRTF )
{
   LRESULT     lResult;
   CHARFORMAT  cF;
   DWORD       Mask;
   DWORD       Effects = 0;
   int         SelText = SCF_SELECTION;

   cF.cbSize = sizeof( CHARFORMAT );
   Mask = SendMessage( (HWND) hb_parnl(1), EM_GETCHARFORMAT, (WPARAM) SelText, (LPARAM) & cF );

   if( hb_parni(10) > 0 )
   {
      Mask = hb_parni( 10 );
   }

   if( hb_parni(2) > 0 )
   {
      SelText = SCF_SELECTION | SCF_WORD;
   }

   if( hb_parni(2) < 0 )
   {
      SelText = SCF_ALL;
   }

   if( hb_parl(5) )
   {
      Effects = Effects | CFE_BOLD;
   }

   if( hb_parl(6) )
   {
      Effects = Effects | CFE_ITALIC;
   }

   if( hb_parl(8) )
   {
      Effects = Effects | CFE_UNDERLINE;
   }

   if( hb_parl(9) )
   {
      Effects = Effects | CFE_STRIKEOUT;
   }

   cF.dwMask = Mask;
   cF.dwEffects = Effects;
   if( hb_parnl(4) )
   {
      cF.yHeight = hb_parnl( 4 ) * 20;
   }

   cF.crTextColor = hb_parnl( 7 );

   if( strlen(hb_parc(3)) )
   {
      lstrcpy( cF.szFaceName, hb_parc(3) );
   }

   lResult = SendMessage( (HWND) hb_parnl(1), EM_SETCHARFORMAT, (WPARAM) SelText, (LPARAM) & cF );

   if( lResult )
   {
      hb_retl( TRUE );
   }
   else
   {
      hb_retl( FALSE );
   }
}
