/*----------------------------------------------------------------------------
 MINIGUI - Harbour Win32 GUI library source code

 Copyright 2002-2008 Roberto Lopez <harbourminigui@gmail.com>
 http://harbourminigui.googlepages.com/

 This program is free software; you can redistribute it and/or modify it under 
 the terms of the GNU General Public License as published by the Free Software 
 Foundation; either version 2 of the License, or (at your option) any later 
 version. 

 This program is distributed in the hope that it will be useful, but WITHOUT 
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS 
 FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

 You should have received a copy of the GNU General Public License along with 
 this software; see the file COPYING. If not, write to the Free Software 
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA (or 
 visit the web site http://www.gnu.org/).

 As a special exception, you have permission for additional uses of the text 
 contained in this release of Harbour Minigui.

 The exception is that, if you link the Harbour Minigui library with other 
 files to produce an executable, this does not by itself cause the resulting 
 executable to be covered by the GNU General Public License.
 Your use of that executable is in no way restricted on account of linking the 
 Harbour-Minigui library code into it.

 Parts of this project are based upon:

	"Harbour GUI framework for Win32"
 	Copyright 2001 Alexander S.Kresin <alex@belacy.belgorod.su>
 	Copyright 2001 Antonio Linares <alinares@fivetech.com>
	www - http://www.harbour-project.org

	"Harbour Project"
	Copyright 1999-2008, http://www.harbour-project.org/

	"WHAT32"
	Copyright 2002 AJ Wos <andrwos@aust1.net> 

	"HWGUI"
  	Copyright 2001-2007 Alexander S.Kresin <alex@belacy.belgorod.su>

---------------------------------------------------------------------------*/

#define _WIN32_IE 0x0500
#define HB_OS_WIN_32_USED
#define _WIN32_WINNT 0x0400
#include <shlobj.h>

#include <windows.h>
#include <commctrl.h>
#include "hbapi.h"
#include "hbvm.h"
#include "hbstack.h"
#include "hbapiitm.h"

typedef BOOL ( WINAPI *VERIFYSCREENSAVEPWD ) ( HWND hwnd );
typedef VOID ( WINAPI *PWDCHANGEPASSWORD ) ( LPCSTR lpcRegkeyname, HWND hwnd, UINT uiReserved1, UINT uiReserved2 );

HB_FUNC( VERIFYPASSWORD )
{
   // Under NT, we return TRUE immediately. This lets the saver quit,
   // and the system manages passwords. Under '95, we call VerifyScreenSavePwd.
   // This checks the appropriate registry key and, if necessary,
   // pops up a verify dialog

   HWND                 hwnd;
   HINSTANCE            hpwdcpl;
   VERIFYSCREENSAVEPWD  VerifyScreenSavePwd;
   BOOL                 bres;

   if( GetVersion() < 0x80000000 )
   {
      hb_retl( TRUE );
   }

   hpwdcpl = LoadLibrary( "PASSWORD.CPL" );

   if( hpwdcpl == NULL )
   {
      hb_retl( FALSE );
   }

   VerifyScreenSavePwd = ( VERIFYSCREENSAVEPWD ) GetProcAddress( hpwdcpl, "VerifyScreenSavePwd" );
   if( VerifyScreenSavePwd == NULL )
   {
      FreeLibrary( hpwdcpl );
      hb_retl( FALSE );
   }

   hwnd = ( HWND ) hb_parnl( 1 );

   bres = VerifyScreenSavePwd( hwnd );
   FreeLibrary( hpwdcpl );

   hb_retl( bres );
}

HB_FUNC( CHANGEPASSWORD )
{
   // This only ever gets called under '95, when started with the /a option.

   HWND              hwnd;

   HINSTANCE         hmpr = LoadLibrary( "MPR.DLL" );
   PWDCHANGEPASSWORD PwdChangePassword;

   if( hmpr == NULL )
   {
      hb_retl( FALSE );
   }

   PwdChangePassword = ( PWDCHANGEPASSWORD ) GetProcAddress( hmpr, "PwdChangePasswordA" );

   if( PwdChangePassword == NULL )
   {
      FreeLibrary( hmpr );
      hb_retl( FALSE );
   }

   hwnd = ( HWND ) hb_parnl( 1 );
   PwdChangePassword( "SCRSAVE", hwnd, 0, 0 );
   FreeLibrary( hmpr );

   hb_retl( TRUE );
}
