/*----------------------------------------------------------------------------
 MINIGUI - Harbour Win32 GUI library source code

 Copyright 2002-2008 Roberto Lopez <harbourminigui@gmail.com>
 http://harbourminigui.googlepages.com/

 This program is free software; you can redistribute it and/or modify it under 
 the terms of the GNU General Public License as published by the Free Software 
 Foundation; either version 2 of the License, or (at your option) any later 
 version. 

 This program is distributed in the hope that it will be useful, but WITHOUT 
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS 
 FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

 You should have received a copy of the GNU General Public License along with 
 this software; see the file COPYING. If not, write to the Free Software 
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA (or 
 visit the web site http://www.gnu.org/).

 As a special exception, you have permission for additional uses of the text 
 contained in this release of Harbour Minigui.

 The exception is that, if you link the Harbour Minigui library with other 
 files to produce an executable, this does not by itself cause the resulting 
 executable to be covered by the GNU General Public License.
 Your use of that executable is in no way restricted on account of linking the 
 Harbour-Minigui library code into it.

 Parts of this project are based upon:

	"Harbour GUI framework for Win32"
 	Copyright 2001 Alexander S.Kresin <alex@belacy.belgorod.su>
 	Copyright 2001 Antonio Linares <alinares@fivetech.com>
	www - http://www.harbour-project.org

	"Harbour Project"
	Copyright 1999-2008, http://www.harbour-project.org/

	"WHAT32"
	Copyright 2002 AJ Wos <andrwos@aust1.net> 

	"HWGUI"
  	Copyright 2001-2007 Alexander S.Kresin <alex@belacy.belgorod.su>

---------------------------------------------------------------------------*/

#define _WIN32_IE 0x0500
#define HB_OS_WIN_32_USED
#define _WIN32_WINNT 0x0400
#include <shlobj.h>

#include <windows.h>
#include <commctrl.h>
#include "richedit.h"
#include "hbapi.h"
#include "hbvm.h"
#include "hbstack.h"
#include "hbapiitm.h"

LRESULT CALLBACK  OwnEditProc( HWND hbutton, UINT msg, WPARAM wParam, LPARAM lParam );

HB_FUNC( INITMASKEDTEXTBOX )
{
   HWND  hwnd;
   HWND  hbutton;

   int   Style;

   hwnd = ( HWND ) hb_parnl( 1 );

   Style = WS_CHILD | ES_AUTOHSCROLL;

   if( hb_parl(9) )
   {
      Style = Style | ES_UPPERCASE;
   }

   if( hb_parl(10) )
   {
      Style = Style | ES_LOWERCASE;
   }

   if( hb_parl(12) )
   {
      Style = Style | ES_RIGHT;
   }

   if( hb_parl(13) )
   {
      Style = Style | ES_READONLY;
   }

   if( !hb_parl(14) )
   {
      Style = Style | WS_VISIBLE;
   }

   if( !hb_parl(15) )
   {
      Style = Style | WS_TABSTOP;
   }

   hbutton = CreateWindowEx
      (
         WS_EX_CLIENTEDGE,
         "EDIT",
         "",
         Style,
         hb_parni(3),
         hb_parni(4),
         hb_parni(5),
         hb_parni(11),
         hwnd,
         (HMENU) hb_parni(2),
         GetModuleHandle(NULL),
         NULL
      );

   hb_retnl( (LONG) hbutton );
}

HB_FUNC( INITTEXTBOX )
{
   HWND  hwnd;          // Handle of the parent window/form.
   HWND  hedit;         // Handle of the child window/control.
   int   iStyle;        // TEXTBOX window base style.

   // Get the handle of the parent window/form.

   hwnd = ( HWND ) hb_parnl( 1 );

   iStyle = WS_CHILD | ES_AUTOHSCROLL | BS_FLAT;

   if( hb_parl(12) )    // if <lNumeric> is TRUE, then ES_NUMBER style is added.
   {
      iStyle = iStyle | ES_NUMBER;

      // Set to a numeric TEXTBOX, so don't worry about other "textual" styles.

   }
   else
   {
      if( hb_parl(10) ) // if <lUpper> is TRUE, then ES_UPPERCASE style is added.
      {
         iStyle = iStyle | ES_UPPERCASE;
      }

      if( hb_parl(11) ) // if <lLower> is TRUE, then ES_LOWERCASE style is added.
      {
         iStyle = iStyle | ES_LOWERCASE;
      }
   }

   if( hb_parl(13) )    // if <lPassword> is TRUE, then ES_PASSWORD style is added.
   {
      iStyle = iStyle | ES_PASSWORD;
   }

   if( hb_parl(14) )
   {
      iStyle = iStyle | ES_RIGHT;
   }

   if( hb_parl(15) )
   {
      iStyle = iStyle | ES_READONLY;
   }

   if( !hb_parl(16) )
   {
      iStyle = iStyle | WS_VISIBLE;
   }

   if( !hb_parl(17) )
   {
      iStyle = iStyle | WS_TABSTOP;
   }

   // Creates the child control.

   hedit = CreateWindowEx
      (
         WS_EX_CLIENTEDGE,
         "EDIT",
         "",
         iStyle,
         hb_parni(3),
         hb_parni(4),
         hb_parni(5),
         hb_parni(6),
         hwnd,
         (HMENU) hb_parni(2),
         GetModuleHandle(NULL),
         NULL
      );

   SendMessage( hedit, (UINT) EM_LIMITTEXT, (WPARAM) hb_parni(9), (LPARAM) 0 );

   SetProp( (HWND) hedit, "oldeditproc", (HWND) GetWindowLong((HWND) hedit, GWL_WNDPROC) );
   SetWindowLong( hedit, GWL_WNDPROC, (LONG) (WNDPROC) OwnEditProc );

   hb_retnl( (LONG) hedit );
}

HB_FUNC( INITCHARMASKTEXTBOX )
{
   HWND  hwnd;
   HWND  hbutton;

   int   Style;

   hwnd = ( HWND ) hb_parnl( 1 );

   Style = WS_CHILD | ES_AUTOHSCROLL;

   if( hb_parl(9) )
   {
      Style = Style | ES_UPPERCASE;
   }

   if( hb_parl(10) )
   {
      Style = Style | ES_LOWERCASE;
   }

   if( hb_parl(12) )
   {
      Style = Style | ES_RIGHT;
   }

   if( hb_parl(13) )
   {
      Style = Style | ES_READONLY;
   }

   if( !hb_parl(14) )
   {
      Style = Style | WS_VISIBLE;
   }

   if( !hb_parl(15) )
   {
      Style = Style | WS_TABSTOP;
   }

   hbutton = CreateWindowEx
      (
         WS_EX_CLIENTEDGE,
         "EDIT",
         "",
         Style,
         hb_parni(3),
         hb_parni(4),
         hb_parni(5),
         hb_parni(11),
         hwnd,
         (HMENU) hb_parni(2),
         GetModuleHandle(NULL),
         NULL
      );

   hb_retnl( (LONG) hbutton );
}

HB_FUNC( SETTEXTEDITREADONLY )
{
   SendMessage( (HWND) hb_parnl(1), (UINT) EM_SETREADONLY, (WPARAM) hb_parl(2), (LPARAM) 0 );
}

LRESULT CALLBACK OwnEditProc( HWND hButton, UINT Msg, WPARAM wParam, LPARAM lParam )
{
   static PHB_SYMB   pSymbol = NULL;
   long int          r;
   WNDPROC           OldWndProc;

   OldWndProc = ( WNDPROC ) GetProp( hButton, "oldeditproc" );

   switch( Msg )
   {
      case WM_DESTROY:
         SetWindowLong( hButton, GWL_WNDPROC, (DWORD) OldWndProc );
         RemoveProp( hButton, "oldeditproc" );
         break;

      case WM_CONTEXTMENU:
      case WM_CHAR:
         {
            if( !pSymbol )
            {
               pSymbol = hb_dynsymSymbol( hb_dynsymGet("OEDITEVENTS") );
            }

            if( pSymbol )
            {
               hb_vmPushSymbol( pSymbol );
               hb_vmPushNil();
               hb_vmPushLong( (LONG) hButton );
               hb_vmPushLong( Msg );
               hb_vmPushLong( wParam );
               hb_vmPushLong( lParam );
               hb_vmDo( 4 );
            }

            r = hb_parnl( -1 );

            if( r != 0 )
            {
               return r;
            }
            else
            {
               return( CallWindowProc(OldWndProc, hButton, Msg, wParam, lParam) );
            }
         }
   }

   return( CallWindowProc(OldWndProc, hButton, Msg, wParam, lParam) );
}
