/*----------------------------------------------------------------------------
 MINIGUI - Harbour Win32 GUI library source code

 Copyright 2002-2008 Roberto Lopez <harbourminigui@gmail.com>
 http://harbourminigui.googlepages.com/

 This program is free software; you can redistribute it and/or modify it under 
 the terms of the GNU General Public License as published by the Free Software 
 Foundation; either version 2 of the License, or (at your option) any later 
 version. 

 This program is distributed in the hope that it will be useful, but WITHOUT 
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS 
 FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

 You should have received a copy of the GNU General Public License along with 
 this software; see the file COPYING. If not, write to the Free Software 
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA (or 
 visit the web site http://www.gnu.org/).

 As a special exception, you have permission for additional uses of the text 
 contained in this release of Harbour Minigui.

 The exception is that, if you link the Harbour Minigui library with other 
 files to produce an executable, this does not by itself cause the resulting 
 executable to be covered by the GNU General Public License.
 Your use of that executable is in no way restricted on account of linking the 
 Harbour-Minigui library code into it.

 Parts of this project are based upon:

	"Harbour GUI framework for Win32"
 	Copyright 2001 Alexander S.Kresin <alex@belacy.belgorod.su>
 	Copyright 2001 Antonio Linares <alinares@fivetech.com>
	www - http://www.harbour-project.org

	"Harbour Project"
	Copyright 1999-2008, http://www.harbour-project.org/

	"WHAT32"
	Copyright 2002 AJ Wos <andrwos@aust1.net> 

	"HWGUI"
  	Copyright 2001-2007 Alexander S.Kresin <alex@belacy.belgorod.su>

---------------------------------------------------------------------------*/

#define _WIN32_IE 0x0500
#define HB_OS_WIN_32_USED
#define _WIN32_WINNT 0x0400

#include <shlobj.h>
#include <shlwapi.h>
#include <winerror.h>

#include <windows.h>
#include <lmcons.h>
#include <commctrl.h>
#include "hbapi.h"
#include "hbvm.h"
#include "hbstack.h"
#include "hbapiitm.h"
#include "hbapifs.h"

#include "tchar.h"
#include "item.api"

#if defined( __XHARBOUR__ )
extern HB_EXPORT void   hb_evalBlock0( PHB_ITEM pCodeBlock );
#else
#ifndef HB_LEGACY_LEVEL
#define FHANDLE HB_FHANDLE
#endif
#endif
BOOL                    SysRefresh( void );

/*WaitRun function For minigui With Pipe redirection
Author Luiz Rafael Culik Guimaraes: culikr@uol.com.br
Parameters WaitRunPipe(cCommand,nShowWindow,cFile)
*/

HB_FUNC( WAITRUNPIPE )
{
   STARTUPINFO          StartupInfo = { 0 };
   PROCESS_INFORMATION  ProcessInfo = { 0 };
   HANDLE               ReadPipeHandle;
   HANDLE               WritePipeHandle;  // not used here
   char                 Data[1024];
   BYTE                 *szFile = ( BYTE * ) hb_parc( 3 );
   FHANDLE              nHandle;
   SECURITY_ATTRIBUTES  sa;
   ZeroMemory( &sa, sizeof(SECURITY_ATTRIBUTES) );
   sa.nLength = sizeof( SECURITY_ATTRIBUTES );
   sa.bInheritHandle = 1;
   sa.lpSecurityDescriptor = NULL;

   if( !hb_fsFile(szFile) )
   {
      nHandle = hb_fsCreate( szFile, 0 );
   }
   else
   {
      nHandle = hb_fsOpen( szFile, 2 );
      hb_fsSeek( nHandle, 0, 2 );
   }

   if( !CreatePipe(&ReadPipeHandle, &WritePipeHandle, &sa, 0) )
   {
      hb_retnl( -1 );
   }

   ProcessInfo.hProcess = INVALID_HANDLE_VALUE;
   ProcessInfo.hThread = INVALID_HANDLE_VALUE;
   StartupInfo.dwFlags = STARTF_USESHOWWINDOW | STARTF_USESTDHANDLES;
   StartupInfo.wShowWindow = ( WORD ) hb_parni( 2 );
   StartupInfo.hStdOutput = WritePipeHandle;
   StartupInfo.hStdError = WritePipeHandle;

   if( !CreateProcess(0, hb_parc(1), 0, 0, FALSE, CREATE_NEW_CONSOLE | NORMAL_PRIORITY_CLASS, 0, 0, &StartupInfo, &ProcessInfo) )
   {
      hb_retnl( -1 );
   }

   for( ;; )
   {
      DWORD BytesRead;
      DWORD TotalBytes;
      DWORD BytesLeft;

      //Check for the presence of data in the pipe

      if( !PeekNamedPipe(ReadPipeHandle, Data, sizeof(Data), &BytesRead, &TotalBytes, &BytesLeft) )
      {
         hb_retnl( -1 );
      }

      //If there is bytes, read them

      if( BytesRead )
      {
         if( !ReadFile(ReadPipeHandle, Data, sizeof(Data) - 1, &BytesRead, NULL) )
         {
            hb_retnl( -1 );
         }

         Data[BytesRead] = '\0';
         hb_fsWriteLarge( nHandle, ( BYTE * ) Data, BytesRead );
      }
      else
      {
         //Is the console app terminated?

         if( WaitForSingleObject(ProcessInfo.hProcess, 0) == WAIT_OBJECT_0 )
         {
            break;
         }
      }
   }

   CloseHandle( ProcessInfo.hThread );
   CloseHandle( ProcessInfo.hProcess );
   CloseHandle( ReadPipeHandle );
   CloseHandle( WritePipeHandle );
   hb_fsClose( nHandle );
}

// CopyRtfToClipboard(cRtfText) store cRTFText in Windows clipboard

HB_FUNC( COPYRTFTOCLIPBOARD )
{
   HGLOBAL  hglbCopy;
   char     *lptstrCopy;
   UINT     cf;
   char     *cStr = hb_parc( 1 );
   int      nLen = strlen( cStr );

   if( !OpenClipboard(GetActiveWindow()) )
   {
      return;
   }

   // Get Clipboard format id for RTF.

   cf = RegisterClipboardFormat( "Rich Text Format" );

   EmptyClipboard();

   hglbCopy = GlobalAlloc( GMEM_MOVEABLE | GMEM_DDESHARE, (nLen + 4) * sizeof(TCHAR) );
   if( hglbCopy == NULL )
   {
      CloseClipboard();
      return;
   }

   lptstrCopy = ( char * ) GlobalLock( hglbCopy );
   memcpy( lptstrCopy, cStr, nLen * sizeof(TCHAR) );
   lptstrCopy[nLen] = ( TCHAR ) 0;        // null character
   GlobalUnlock( hglbCopy );

   SetClipboardData( cf, hglbCopy );
   CloseClipboard();
}

// CopyToClipboard(cText) store cText in Windows clipboard

HB_FUNC( COPYTOCLIPBOARD )
{
   HGLOBAL  hglbCopy;
   char     *lptstrCopy;

   char     *cStr = hb_parc( 1 );
   int      nLen = strlen( cStr );

   if( !OpenClipboard(GetActiveWindow()) )
   {
      return;
   }

   EmptyClipboard();

   hglbCopy = GlobalAlloc( GMEM_DDESHARE, (nLen + 1) * sizeof(TCHAR) );
   if( hglbCopy == NULL )
   {
      CloseClipboard();
      return;
   }

   lptstrCopy = ( char * ) GlobalLock( hglbCopy );
   memcpy( lptstrCopy, cStr, nLen * sizeof(TCHAR) );
   lptstrCopy[nLen] = ( TCHAR ) 0;        // null character
   GlobalUnlock( hglbCopy );

   SetClipboardData( ISNUM(2) ? (UINT) hb_parni(2) : CF_TEXT, hglbCopy );
   CloseClipboard();
}

HB_FUNC( RETRIVETEXTFROMCLIPBOARD )
{
   if( !OpenClipboard(GetActiveWindow()) )
   {
      return;
   }

   hb_retc( GetClipboardData(CF_TEXT) );
   CloseClipboard();
}

HB_FUNC( GETCLIPBOARDDATA )
{
   HANDLE   hClipMem;
   LPSTR    lpClip;

   hClipMem = GetClipboardData( (UINT) hb_parni(1) );

   if( hClipMem )
   {
      lpClip = ( LPSTR ) GlobalLock( hClipMem );
      hb_retclen( lpClip, GlobalSize(hClipMem) );
      GlobalUnlock( hClipMem );
   }
}

HB_FUNC( GETBLUE )
{
   hb_retnl( GetBValue(hb_parnl(1)) );
}

HB_FUNC( GETRED )
{
   hb_retnl( GetRValue(hb_parnl(1)) );
}

HB_FUNC( GETGREEN )
{
   hb_retnl( GetGValue(hb_parnl(1)) );
}

HB_FUNC( GETKEYSTATE )
{
   hb_retni( GetKeyState(hb_parni(1)) );
}

HB_FUNC( GETDC )
{
   hb_retnl( (ULONG) GetDC((HWND) hb_parnl(1)) );
}

HB_FUNC( RELEASEDC )
{
   hb_retl( ReleaseDC((HWND) hb_parnl(1), (HDC) hb_parnl(2)) );
}

HB_FUNC( HIWORD )
{
   hb_retnl( HIWORD(hb_parnl(1)) );
}

HB_FUNC( LOWORD )
{
   hb_retnl( LOWORD(hb_parnl(1)) );
}

HB_FUNC( C_GETSPECIALFOLDER ) // Contributed By Ryszard Ryko
{
   char           *lpBuffer = ( char * ) hb_xgrab( MAX_PATH + 1 );
   LPITEMIDLIST   pidlBrowse; // PIDL selected by user
   SHGetSpecialFolderLocation( GetActiveWindow(), hb_parni(1), &pidlBrowse );
   SHGetPathFromIDList( pidlBrowse, lpBuffer );
   hb_retc( lpBuffer );
   hb_xfree( lpBuffer );
}

HB_FUNC( C_GETDLLSPECIALFOLDER ) // Based Upon Code Contributed By Jacek Kubica <kubica@wssl.wroc.pl>
{
   TCHAR       szPath[MAX_PATH];
   HRESULT     hResult;
   HINSTANCE   hLib = LoadLibrary( "SHFolder.dll" );

   hResult = SHGetFolderPath( NULL, hb_parni(1), NULL, 0, szPath );

   if( hResult == S_OK )
   {
      hb_retc( szPath );
   }
   else
   {
      hb_retc( "" );
   }

   FreeLibrary( hLib );
}

HB_FUNC( MEMORYSTATUS )
{
   MEMORYSTATUS   mst;
   long           n = hb_parnl( 1 );

   mst.dwLength = sizeof( MEMORYSTATUS );
   GlobalMemoryStatus( &mst );

   switch( n )
   {
      case 1:  hb_retnl( mst.dwTotalPhys / (1024 * 1024) ); break;
      case 2:  hb_retnl( mst.dwAvailPhys / (1024 * 1024) ); break;
      case 3:  hb_retnl( mst.dwTotalPageFile / (1024 * 1024) ); break;
      case 4:  hb_retnl( mst.dwAvailPageFile / (1024 * 1024) ); break;
      case 5:  hb_retnl( mst.dwTotalVirtual / (1024 * 1024) ); break;
      case 6:  hb_retnl( mst.dwAvailVirtual / (1024 * 1024) ); break;
      default: hb_retnl( 0 );
   }
}

HB_FUNC( SHELLABOUT )
{
   ShellAbout( 0, hb_parc(1), hb_parc(2), (HICON) hb_parnl(3) );
}

HB_FUNC( PAINTBKGND )
{
   HWND     hwnd;
   HBRUSH   brush;
   RECT     recClie;
   HDC      hdc;

   hwnd = ( HWND ) hb_parnl( 1 );
   hdc = GetDC( hwnd );

   GetClientRect( hwnd, &recClie );

   if( (hb_pcount() > 1) && (!ISNIL(2)) )
   {
      brush = CreateSolidBrush( RGB(hb_parni(2, 1), hb_parni(2, 2), hb_parni(2, 3)) );
      FillRect( hdc, &recClie, brush );
   }
   else
   {
      brush = ( HBRUSH ) ( COLOR_BTNFACE + 1 );
      FillRect( hdc, &recClie, brush );
   }

   ReleaseDC( hwnd, hdc );

   hb_retnl( (LONG) brush );
}

/* Functions Contributed  By Luiz Rafael Culik Guimaraes( culikr@uol.com.br) */

HB_FUNC( GETWINDOWSDIR )
{
   char  szBuffer[MAX_PATH + 1] = { 0 };
   GetWindowsDirectory( szBuffer, MAX_PATH );
   hb_retc( szBuffer );
}

HB_FUNC( GETSYSTEMDIR )
{
   char  szBuffer[MAX_PATH + 1] = { 0 };
   GetSystemDirectory( szBuffer, MAX_PATH );
   hb_retc( szBuffer );
}

HB_FUNC( GETTEMPDIR )
{
   char  szBuffer[MAX_PATH + 1] = { 0 };
   GetTempPath( MAX_PATH, szBuffer );
   hb_retc( szBuffer );
}

HB_FUNC( _PROCESSMESS )
{
   MSG   Msg;

   if( PeekMessage((LPMSG) & Msg, 0, 0, 0, PM_REMOVE) )
   {
      TranslateMessage( &Msg );
      DispatchMessage( &Msg );
      hb_retl( TRUE );
   }
   else
   {
      hb_retl( FALSE );
   }
}

HB_FUNC( POSTMESSAGE )
{
   hb_retnl( (LONG) PostMessage((HWND) hb_parnl(1), (UINT) hb_parni(2), (WPARAM) hb_parnl(3), (LPARAM) hb_parnl(4)) );
}

HB_FUNC( DEFWINDOWPROC )
{
   hb_retnl( (LONG) DefWindowProc((HWND) hb_parnl(1), (UINT) hb_parni(2), (WPARAM) hb_parnl(3), (LPARAM) hb_parnl(4)) );
}

HB_FUNC( GETSTOCKOBJECT )
{
   hb_retnl( (LONG) GetStockObject(hb_parni(1)) );
}

HB_FUNC( SETBKMODE )
{
   hb_retni( SetBkMode((HDC) hb_parnl(1), hb_parni(2)) );
}

HB_FUNC( GETNEXTDLGTABITEM )
{
   hb_retnl( (LONG) GetNextDlgTabItem((HWND) hb_parnl(1), (HWND) hb_parnl(2), hb_parl(3)) );
}

HB_FUNC( SHELLEXECUTE )
{
   hb_retnl
   (
      (LONG) ShellExecute
         (
            (HWND) hb_parnl(1),
            ISNIL(2) ? NULL : (LPCSTR) hb_parc(2),
            (LPCSTR) hb_parc(3),
            ISNIL(4) ? NULL : (LPCSTR) hb_parc(4),
            ISNIL(5) ? NULL : (LPCSTR) hb_parc(5),
            hb_parni(6)
         )
   );
}

HB_FUNC( WAITRUN )
{
   DWORD                dwExitCode;

   STARTUPINFO          stInfo;
   PROCESS_INFORMATION  prInfo;
   BOOL                 bResult;

   ZeroMemory( &stInfo, sizeof(stInfo) );

   stInfo.cb = sizeof( stInfo );

   stInfo.dwFlags = STARTF_USESHOWWINDOW;

   stInfo.wShowWindow = ( WORD ) hb_parni( 2 );

   bResult = CreateProcess( NULL, hb_parc(1), NULL, NULL, TRUE, CREATE_NEW_CONSOLE | NORMAL_PRIORITY_CLASS, NULL, NULL, &stInfo, &prInfo );

   if( !bResult )
   {
      hb_retl( -1 );
   }

   WaitForSingleObject( prInfo.hProcess, INFINITE );

   GetExitCodeProcess( prInfo.hProcess, &dwExitCode );

   CloseHandle( prInfo.hThread );
   CloseHandle( prInfo.hProcess );

   hb_retnl( dwExitCode );
}

/* WaitRunTerm contributed by Kevin Carmody (i@kevincarmody.com) 2007.11.16 */

HB_FUNC( WAITRUNTERM )
{
   PHB_ITEM             pWaitProc = hb_param( 4, HB_IT_BLOCK );
   ULONG                ulWaitMsec = ( ISNIL(5) ? 2000 : hb_parnl(5) );
   BOOL                 bTerm = FALSE;
   BOOL                 bWait;
   ULONG                ulNoSignal;
   DWORD                dwExitCode;
   STARTUPINFO          stInfo;
   PROCESS_INFORMATION  prInfo;
   BOOL                 bResult;

   ZeroMemory( &stInfo, sizeof(stInfo) );
   stInfo.cb = sizeof( stInfo );
   stInfo.dwFlags = STARTF_USESHOWWINDOW;
   stInfo.wShowWindow = ( WORD ) ( ISNIL(3) ? 5 : hb_parni(3) );

   bResult = CreateProcess
      (
         NULL,
         hb_parc(1),
         NULL,
         NULL,
         TRUE,
         CREATE_NEW_CONSOLE | NORMAL_PRIORITY_CLASS,
         NULL,
         ISNIL(2) ? NULL : hb_parc(2),
         &stInfo,
         &prInfo
      );

   if( !bResult )
   {
      hb_retnl( -2 );
   }

   if( pWaitProc )
   {
      do
      {
         ulNoSignal = WaitForSingleObject( prInfo.hProcess, ulWaitMsec );
         if( ulNoSignal )
         {
            hb_evalBlock0( pWaitProc );
            bWait = hb_parl( -1 );
            if( !bWait )
            {
               if( TerminateProcess(prInfo.hProcess, 0) != 0 )
               {
                  bTerm = TRUE;
               }
               else
               {
                  bWait = TRUE;
               }
            }
         }
         else
         {
            bWait = FALSE;
         }
      }
      while( bWait );
   }
   else
   {
      WaitForSingleObject( prInfo.hProcess, INFINITE );
   }

   if( bTerm )
   {
      dwExitCode = -1;
   }
   else
   {
      GetExitCodeProcess( prInfo.hProcess, &dwExitCode );
   }

   CloseHandle( prInfo.hThread );
   CloseHandle( prInfo.hProcess );
   hb_retnl( dwExitCode );
}

HB_FUNC( ISEXERUNNING )          // ( cExeNameCaseSensitive ) --> lResult
{
   HANDLE   hMutex = CreateMutex( NULL, TRUE, (LPTSTR) hb_parc(1) );

   hb_retl( GetLastError() == ERROR_ALREADY_EXISTS );

   ReleaseMutex( hMutex );
}

HB_FUNC( SETSCROLLPOS )
{
   hb_retni( SetScrollPos((HWND) hb_parnl(1), hb_parni(2), hb_parni(3), hb_parl(4)) );
}

HB_FUNC( CREATEFOLDER )
{
   CreateDirectory( (LPCTSTR) hb_parc(1), NULL );
}

HB_FUNC( SETCURRENTFOLDER )
{
   SetCurrentDirectory( (LPCTSTR) hb_parc(1) );
}

HB_FUNC( REMOVEFOLDER )
{
   hb_retl( RemoveDirectory((LPCSTR) hb_parc(1)) );
}

HB_FUNC( GETCURRENTFOLDER )
{
   char  Path[MAX_PATH + 1] = { 0 };
   GetCurrentDirectory( MAX_PATH, (LPSTR) Path );
   hb_retc( Path );
}

HB_FUNC( CREATESOLIDBRUSH )
{
   hb_retnl( (LONG) CreateSolidBrush((COLORREF) RGB(hb_parni(1), hb_parni(2), hb_parni(3))) );
}

HB_FUNC( SETTEXTCOLOR )
{
   hb_retnl( (ULONG) SetTextColor((HDC) hb_parnl(1), (COLORREF) RGB(hb_parni(2), hb_parni(3), hb_parni(4))) );
}

HB_FUNC( SETBKCOLOR )
{
   hb_retnl( (ULONG) SetBkColor((HDC) hb_parnl(1), (COLORREF) RGB(hb_parni(2), hb_parni(3), hb_parni(4))) );
}

HB_FUNC( GETSYSCOLOR )
{
   hb_retnl( GetSysColor(hb_parni(1)) );
}

HB_FUNC( SETWINDOWLONG )
{
   hb_retnl( SetWindowLong((HWND) hb_parnl(1), hb_parni(2), hb_parnl(3)) );
}

/**************************************************************************************/
/*                                                                                    */
/*  This function returns the Windows Version on which the app calling the function   */
/*  is running.                                                                       */
/*                                                                                    */
/*  The return value is an three dimensinal array containing the OS in the first,     */
/*  the servicepack or the system release number in the second and the build number   */
/*  in the third array element.                                                       */
/*                                                                                    */
/**************************************************************************************/

HB_FUNC( WINVERSION )
{
   #define VER_SUITE_PERSONAL 0x00000200
   #define VER_SUITE_BLADE    0x00000400

   OSVERSIONINFOEX   osvi;
   BOOL              bOsVersionInfoEx;
   CHAR              *szVersion = NULL;
   CHAR              *szServicePack = NULL;
   CHAR              *szBuild = NULL;
   CHAR              buffer[5];

   CHAR              *szVersionEx = NULL;

   ZeroMemory( &osvi, sizeof(OSVERSIONINFOEX) );
   osvi.dwOSVersionInfoSize = sizeof( OSVERSIONINFOEX );

   bOsVersionInfoEx = GetVersionEx( ( OSVERSIONINFO * ) &osvi );
   if( !bOsVersionInfoEx )
   {
      osvi.dwOSVersionInfoSize = sizeof( OSVERSIONINFO );
      if( !GetVersionEx(( OSVERSIONINFO * ) &osvi) )
      {
         szVersion = "Unknown Operating System";
      }
   }

   if( szVersion == NULL )
   {
      switch( osvi.dwPlatformId )
      {
         case VER_PLATFORM_WIN32_NT:
            if( osvi.dwMajorVersion == 5 && osvi.dwMinorVersion == 2 )
            {
               szVersion = "Windows Server 2003 family ";
            }

            if( osvi.dwMajorVersion == 5 && osvi.dwMinorVersion == 1 )
            {
               szVersion = "Windows XP ";
            }

            if( osvi.dwMajorVersion == 5 && osvi.dwMinorVersion == 0 )
            {
               szVersion = "Windows 2000 ";
            }

            if( osvi.dwMajorVersion <= 4 )
            {
               szVersion = "Windows NT ";
            }

            if( bOsVersionInfoEx )
            {
               if( osvi.wProductType == VER_NT_WORKSTATION )
               {
                  if( osvi.dwMajorVersion == 6 )
                  {
                     szVersion = "Windows Vista ";
                  }
                  else if( osvi.dwMajorVersion == 4 )
                  {
                     szVersionEx = "Workstation 4.0 ";
                  }
                  else if( osvi.wSuiteMask & VER_SUITE_PERSONAL )
                  {
                     szVersionEx = "Home Edition ";
                  }
                  else
                  {
                     szVersionEx = "Professional ";
                  }
               }
               else if( osvi.wProductType == VER_NT_SERVER )
               {
                  if( osvi.dwMajorVersion == 6 )
                  {
                     szVersion = "Windows Server 2008 ";
                  }
                  else if( osvi.dwMajorVersion == 5 && osvi.dwMinorVersion == 2 )
                  {
                     if( osvi.wSuiteMask & VER_SUITE_DATACENTER )
                     {
                        szVersionEx = "Datacenter Edition ";
                     }
                     else if( osvi.wSuiteMask & VER_SUITE_ENTERPRISE )
                     {
                        szVersionEx = "Enterprise Edition ";
                     }
                     else if( osvi.wSuiteMask & VER_SUITE_BLADE )
                     {
                        szVersionEx = "Web Edition ";
                     }
                     else
                     {
                        szVersionEx = "Standard Edition ";
                     }
                  }
                  else if( osvi.dwMajorVersion == 5 && osvi.dwMinorVersion == 0 )
                  {
                     if( osvi.wSuiteMask & VER_SUITE_DATACENTER )
                     {
                        szVersionEx = "Datacenter Server ";
                     }
                     else if( osvi.wSuiteMask & VER_SUITE_ENTERPRISE )
                     {
                        szVersionEx = "Advanced Server ";
                     }
                     else
                     {
                        szVersionEx = "Server ";
                     }
                  }
                  else
                  {
                     if( osvi.wSuiteMask & VER_SUITE_ENTERPRISE )
                     {
                        szVersionEx = "Server 4.0, Enterprise Edition ";
                     }
                     else
                     {
                        szVersionEx = "Server 4.0 ";
                     }
                  }
               }
            }
            else
            {
               HKEY  hKey;
               char  szProductType[80];
               DWORD dwBufLen = 80;
               LONG  lRetVal;

               lRetVal = RegOpenKeyEx( HKEY_LOCAL_MACHINE, "SYSTEM\\CurrentControlSet\\Control\\ProductOptions", 0, KEY_QUERY_VALUE, &hKey );

               if( lRetVal != ERROR_SUCCESS )
               {
                  szVersion = "Unknown Operating System";
               }
               else
               {
                  lRetVal = RegQueryValueEx( hKey, "ProductType", NULL, NULL, (LPBYTE) szProductType, &dwBufLen );
                  if( (lRetVal != ERROR_SUCCESS) || (dwBufLen > 80) )
                  {
                     szVersion = "Unknown Operating System";
                  }
               }

               RegCloseKey( hKey );

               if( szVersion != "Unknown Operating System" )
               {
                  if( lstrcmpi("WINNT", szProductType) == 0 )
                  {
                     szVersionEx = "Workstation ";
                  }

                  if( lstrcmpi("LANMANNT", szProductType) == 0 )
                  {
                     szVersionEx = "Server ";
                  }

                  if( lstrcmpi("SERVERNT", szProductType) == 0 )
                  {
                     szVersionEx = "Advanced Server ";
                  }

                  szVersion = strcat( szVersion, itoa(osvi.dwMajorVersion, buffer, 10) );
                  szVersion = strcat( szVersion, "." );
                  szVersion = strcat( szVersion, itoa(osvi.dwMinorVersion, buffer, 10) );
               }
            }

            if( osvi.dwMajorVersion == 4 && lstrcmpi(osvi.szCSDVersion, "Service Pack 6") == 0 )
            {
               HKEY  hKey;
               LONG  lRetVal;

               lRetVal = RegOpenKeyEx
                  (
                     HKEY_LOCAL_MACHINE,
                     "SOFTWARE\\Microsoft\\Windows NT\\CurrentVersion\\Hotfix\\Q246009",
                     0,
                     KEY_QUERY_VALUE,
                     &hKey
                  );
               if( lRetVal == ERROR_SUCCESS )
               {
                  szServicePack = "Service Pack 6a";
                  szBuild = itoa( osvi.dwBuildNumber & 0xFFFF, buffer, 10 );
               }
               else
               {
                  szServicePack = osvi.szCSDVersion;
                  szBuild = itoa( osvi.dwBuildNumber & 0xFFFF, buffer, 10 );
               }

               RegCloseKey( hKey );
            }
            else
            {
               szServicePack = osvi.szCSDVersion;
               szBuild = itoa( osvi.dwBuildNumber & 0xFFFF, buffer, 10 );
            }
            break;

         case VER_PLATFORM_WIN32_WINDOWS:
            if( (osvi.dwMajorVersion == 4) && (osvi.dwMinorVersion == 0) )
            {
               if( osvi.szCSDVersion[1] == 'B' )
               {
                  szVersion = "Windows 95 B";
                  szServicePack = "OSR2";
               }
               else
               {
                  if( osvi.szCSDVersion[1] == 'C' )
                  {
                     szVersion = "Windows 95 C";
                     szServicePack = "OSR2";
                  }
                  else
                  {
                     szVersion = "Windows 95";
                     szServicePack = "OSR1";
                  }
               }

               szBuild = itoa( osvi.dwBuildNumber & 0x0000FFFF, buffer, 10 );
            }

            if( (osvi.dwMajorVersion == 4) && (osvi.dwMinorVersion == 10) )
            {
               if( osvi.szCSDVersion[1] == 'A' )
               {
                  szVersion = "Windows 98 A";
                  szServicePack = "Second Edition";
               }
               else
               {
                  szVersion = "Windows 98";
                  szServicePack = "First Edition";
               }

               szBuild = itoa( osvi.dwBuildNumber & 0x0000FFFF, buffer, 10 );
            }

            if( (osvi.dwMajorVersion == 4) && (osvi.dwMinorVersion == 90) )
            {
               szVersion = "Windows ME";
               szBuild = itoa( osvi.dwBuildNumber & 0x0000FFFF, buffer, 10 );
            }
            break;
      }
   }

   hb_reta( 4 );
   hb_storc( szVersion, -1, 1 );
   hb_storc( szServicePack, -1, 2 );
   hb_storc( szBuild, -1, 3 );
   hb_storc( szVersionEx, -1, 4 );
}

HB_FUNC( ISWINXPORLATER )
{
   OSVERSIONINFO  osvi;

   osvi.dwOSVersionInfoSize = sizeof( OSVERSIONINFO );
   GetVersionEx( ( OSVERSIONINFO * ) &osvi );

   hb_retl( osvi.dwPlatformId == VER_PLATFORM_WIN32_NT && ((osvi.dwMajorVersion == 5 && osvi.dwMinorVersion > 0) || (osvi.dwMajorVersion > 5)) );
}

static DLLGETVERSIONPROC   pDLLGETVERSION = NULL;

HB_FUNC( GETDLLVERSION )
{
   typedef HRESULT ( CALLBACK *DLLGETVERSIONPROC ) ( DLLVERSIONINFO * );

   HMODULE        hModComCtl;
   char           cLibName[MAX_PATH];

   DLLVERSIONINFO dvi = { 0 };

   strcpy( cLibName, hb_parc(1) );

   hModComCtl = LoadLibrary( cLibName );
   if( hModComCtl )
   {
      pDLLGETVERSION = ( DLLGETVERSIONPROC ) GetProcAddress( hModComCtl, "DllGetVersion" );
      if( pDLLGETVERSION )
      {
         memset( &dvi, 0, sizeof(dvi) );              // clear
         dvi.cbSize = sizeof( dvi );                  // set size for Windows
         if( pDLLGETVERSION(&dvi) == NOERROR )
         {
            hb_reta( 3 );
            hb_stornl( dvi.dwMajorVersion, -1, 1 );   // Major version
            hb_stornl( dvi.dwMinorVersion, -1, 2 );   // Minor version
            hb_stornl( dvi.dwBuildNumber, -1, 3 );    // Build number
         }
         else
         {
            hb_reta( 3 );
            hb_stornl( 0, -1, 1 );
            hb_stornl( 0, -1, 2 );
            hb_stornl( 0, -1, 3 );
         }
      }
      else
      {
         hb_reta( 3 );
         hb_stornl( 0, -1, 1 );
         hb_stornl( 0, -1, 2 );
         hb_stornl( 0, -1, 3 );
      }

      FreeLibrary( hModComCtl );
   }
   else
   {
      hb_reta( 3 );
      hb_stornl( 0, -1, 1 );
      hb_stornl( 0, -1, 2 );
      hb_stornl( 0, -1, 3 );
   }
}

// Jacek Kubica <kubica@wssk.wroc.pl> HMG 1.0 Experimental Build 9a

HB_FUNC( SELECTOBJECT )
{
   hb_retnl( (LONG) SelectObject((HDC) hb_parnl(1), // handle of device context
   (HGDIOBJ) hb_parnl(2) // handle of object
   ) );
}

HB_FUNC( INVALIDATERECT )
{
   RECT  rc;

   if( hb_pcount() > 2 )
   {
      rc.left = hb_parni( 3 );
      rc.top = hb_parni( 4 );
      rc.right = hb_parni( 5 );
      rc.bottom = hb_parni( 6 );
   }

   InvalidateRect
   (
      (HWND) hb_parnl(1),              // handle of window with changed update region
      (hb_pcount() > 2) ? &rc : NULL,  // address of rectangle coordinates
      hb_parni(2)          // erase-background flag
   );
}

HB_FUNC( FILLRECT )
{
   RECT  rc;

   rc.left = hb_parni( 2 );
   rc.top = hb_parni( 3 );
   rc.right = hb_parni( 4 );
   rc.bottom = hb_parni( 5 );

   FillRect( (HDC) hb_parnl(1), // handle to device context
   &rc, // pointer to structure with rectangle
   (HBRUSH) hb_parnl(6) // handle to brush
   );
}

// BOOL GetComputerName( LPTSTR lpBuffer, LPDWORD nSize );
// http://msdn.microsoft.com/library/default.asp?url=/library/en-us/sysinfo/base/getcomputername.asp

HB_FUNC( GETCOMPUTERNAME )
{
   TCHAR lpBuffer[129];
   DWORD nSize = 128;

   GetComputerName( lpBuffer, &nSize );

   hb_retc( lpBuffer );
}

// BOOL GetUserName( LPTSTR lpBuffer, LPDWORD nSize );
// http://msdn.microsoft.com/library/default.asp?url=/library/en-us/sysinfo/base/getusername.asp

HB_FUNC( GETUSERNAME )
{
   TCHAR lpBuffer[UNLEN + 1];
   DWORD nSize = UNLEN;

   GetUserName( lpBuffer, &nSize );

   hb_retc( lpBuffer );
}

// Grigory Filatov <gfilatov@freemail.ru> HMG 1.1 Experimental Build 10d

HB_FUNC( CLEANPROGRAMMEMORY )
{
   hb_retl( SetProcessWorkingSetSize(GetCurrentProcess(), -1, -1) );
}

// Grigory Filatov <gfilatov@freemail.ru> HMG 1.1 Experimental Build 11a

typedef INT ( WINAPI *_GETCOMPACTPATH ) ( LPTSTR pszOut, LPTSTR pszSrc, INT cchMax, DWORD dwFlags );

HB_FUNC( GETCOMPACTPATH )
{
   HINSTANCE   handle = LoadLibrary( "shlwapi.dll" );

   if( handle )
   {
      _GETCOMPACTPATH   pFunc;
      pFunc = ( _GETCOMPACTPATH ) GetProcAddress( handle, "PathCompactPathExA" );
      hb_retni( pFunc((LPTSTR) hb_parc(1), (LPTSTR) hb_parc(2), (INT) hb_parni(3), (DWORD) hb_parc(4)) );
      FreeLibrary( handle );
   }
}

// Jacek Kubica <kubica@wssk.wroc.pl> HMG 1.1 Experimental Build 11a

HB_FUNC( GETSHORTPATHNAME )
{
   char  buffer[MAX_PATH + 1] = { 0 };
   DWORD iRet;

   iRet = GetShortPathName( hb_parc(1), buffer, MAX_PATH );
   if( iRet < MAX_PATH )
   {
      hb_storclen( buffer, iRet, 2 );
   }
   else
   {
      hb_storc( "", 2 );
   }

   hb_retnl( iRet );
}

HB_FUNC( DRAWTEXT )
{
   char  *cText = hb_parc( 2 );
   RECT  rc;

   rc.left = hb_parni( 3 );
   rc.top = hb_parni( 4 );
   rc.right = hb_parni( 5 );
   rc.bottom = hb_parni( 6 );

   DrawText
   (
      (HDC) hb_parnl(1),   // device context
      (LPCTSTR) cText,     // pointer to string
      strlen(cText),       // length of  string
      &rc,                 // rectangle
      hb_parni(7)          // draw style
   );
}

HB_FUNC( GETTEXTMETRIC )
{
   TEXTMETRIC  tm;
   PHB_ITEM    aMetr = _itemArrayNew( 6 );
   PHB_ITEM    temp;

   GetTextMetrics( (HDC) hb_parnl(1), // handle of device context
   &tm // address of text metrics structure
   );

   //tmHeight
   //Specifies the height (ascent + descent) of characters.

   temp = _itemPutNL( NULL, tm.tmHeight );
   _itemArrayPut( aMetr, 1, temp );
   _itemRelease( temp );

   // .tmAveCharWidth Specifies the average width of characters in the font
   //  (generally defined as the width of the letter x).
   //  This value does not include the overhang required for bold or italic characters.

   temp = _itemPutNL( NULL, tm.tmAveCharWidth );
   _itemArrayPut( aMetr, 2, temp );
   _itemRelease( temp );

   // tmMaxCharWidth
   //Specifies the width of the widest character in the font.

   temp = _itemPutNL( NULL, tm.tmMaxCharWidth );
   _itemArrayPut( aMetr, 3, temp );
   _itemRelease( temp );

   //tmAscent
   //Specifies the ascent (units above the base line) of characters.

   temp = _itemPutNL( NULL, tm.tmAscent );
   _itemArrayPut( aMetr, 4, temp );
   _itemRelease( temp );

   // tmDescent
   // Specifies the descent (units below the base line) of characters.

   temp = _itemPutNL( NULL, tm.tmDescent );
   _itemArrayPut( aMetr, 5, temp );
   _itemRelease( temp );

   //tmInternalLeading
   //Specifies the amount of leading (space) inside the bounds set by the tmHeight member.
   //Accent marks and other diacritical characters may occur in this area.
   //The designer may set this member to zero.

   temp = _itemPutNL( NULL, tm.tmInternalLeading );
   _itemArrayPut( aMetr, 6, temp );
   _itemRelease( temp );

   _itemReturn( aMetr );
   _itemRelease( aMetr );
}

// Grigory Filatov <gfilatov@freemail.ru> HMG 1.1 Experimental Build 17d

HB_FUNC( ISOEMTEXT )
{
   LPBYTE   pString = ( LPBYTE ) hb_parc( 1 );
   WORD     w = 0, wLen = ( WORD ) hb_parclen( 1 );
   BOOL     bOem = FALSE;

   while( w < wLen && !bOem )
   {
      bOem = pString[w] >= 128 && pString[w] <= 168;
      w++;
   }

   hb_retl( bOem );
}

/*
 Harbour MiniGUI 1.3 Extended (Build 33)
 added by P.Chornyj

 Function GetObjectType()
 ------------------------
 The GetObjectType identifies the type of the specified object.

 Syntax
     GetObjectType( nObject ) --> nType

 Arguments
     nObject is identifies the object

 Returns
     If the function succeeds, the return value identifies the object. 
 This value can be one of the following:
     OBJ_PEN         1
     OBJ_BRUSH       2
     OBJ_DC          3
     OBJ_METADC	     4
     OBJ_PAL	     5	
     OBJ_FONT	     6
     OBJ_BITMAP	     7  
     OBJ_REGION      8
     OBJ_METAFILE    9
     OBJ_MEMDC       10
     OBJ_EXTPEN      11
     OBJ_ENHMETADC   12
     OBJ_ENHMETAFILE 13
     OBJ_COLORSPACE  14
*/

HB_FUNC( GETOBJECTTYPE )
{
   hb_retnl( (LONG) GetObjectType((HGDIOBJ) hb_parnl(1)) );
}

/*
 Harbour MiniGUI 1.3 Extended (Build 33)
 added by P.Chornyj

 The WindowFromDC function returns a handle to the window 
 associated with the specified display device context (DC).
*/

HB_FUNC( GETWINDOWFROMDC )
{
   hb_retnl( (LONG) WindowFromDC((HDC) hb_parnl(1)) );
}

/*
 Harbour MiniGUI 1.4 Extended (Build 47)
 added by Grigory Filatov
*/

HB_FUNC( DRAGACCEPTFILES )
{
   DragAcceptFiles( (HWND) hb_parnl(1), hb_parl(2) );
}

HB_FUNC( DRAGQUERYFILES )
{
   HDROP hDrop = ( HDROP ) hb_parnl( 1 );
   WORD  wFiles = ( WORD ) DragQueryFile( hDrop, -1, 0, 0 );
   INT   i;
   BYTE  bBuffer[250];

   hb_reta( wFiles );

   for( i = 0; i < wFiles; i++ )
   {
      DragQueryFile( hDrop, i, ( char * ) bBuffer, 249 );
      hb_storc( ( char * ) bBuffer, -1, i + 1 );
   }
}

HB_FUNC( DRAGFINISH )
{
   DragFinish( (HDROP) hb_parnl(1) );
}
