/*----------------------------------------------------------------------------
 MINIGUI - Harbour Win32 GUI library source code

 Copyright 2002-2008 Roberto Lopez <harbourminigui@gmail.com>
 http://harbourminigui.googlepages.com/

 This program is free software; you can redistribute it and/or modify it under 
 the terms of the GNU General Public License as published by the Free Software 
 Foundation; either version 2 of the License, or (at your option) any later 
 version. 

 This program is distributed in the hope that it will be useful, but WITHOUT 
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS 
 FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

 You should have received a copy of the GNU General Public License along with 
 this software; see the file COPYING. If not, write to the Free Software 
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA (or 
 visit the web site http://www.gnu.org/).

 As a special exception, you have permission for additional uses of the text 
 contained in this release of Harbour Minigui.

 The exception is that, if you link the Harbour Minigui library with other 
 files to produce an executable, this does not by itself cause the resulting 
 executable to be covered by the GNU General Public License.
 Your use of that executable is in no way restricted on account of linking the 
 Harbour-Minigui library code into it.

 Parts of this project are based upon:

	"Harbour GUI framework for Win32"
 	Copyright 2001 Alexander S.Kresin <alex@belacy.belgorod.su>
 	Copyright 2001 Antonio Linares <alinares@fivetech.com>
	www - http://www.harbour-project.org

	"Harbour Project"
	Copyright 1999-2008, http://www.harbour-project.org/

	"WHAT32"
	Copyright 2002 AJ Wos <andrwos@aust1.net> 

	"HWGUI"
  	Copyright 2001-2007 Alexander S.Kresin <alex@belacy.belgorod.su>

---------------------------------------------------------------------------*/

#define WINVER    0x0500
#define _WIN32_IE 0x0500
#define HB_OS_WIN_32_USED
#define _WIN32_WINNT 0x0400

#define WM_TASKBAR   WM_USER + 1043

#include <shlobj.h>
#include <windows.h>
#include <commctrl.h>
#include "richedit.h"
#include "hbapi.h"
#include "hbvm.h"
#include "hbstack.h"
#include "hbapiitm.h"
#include <wingdi.h>
#include "item.api"

static const char *sEVENTSFUNC = NULL;
static PHB_SYMB   pSymbolEvents = NULL;

HRGN              BitmapToRegion( HBITMAP hBmp, COLORREF cTransparentColor, COLORREF cTolerance );

BOOL              Array2Rect( PHB_ITEM aRect, RECT *rc );

static BOOL       ChangeNotifyIcon( HWND hWnd, HICON hIcon, LPSTR szText );
static BOOL       ShowNotifyIcon( HWND hWnd, BOOL bAdd, HICON hIcon, LPSTR szText );

HB_FUNC( SETEVENTSFUNC )
{
   pSymbolEvents = NULL;
   sEVENTSFUNC = ( const char * ) hb_parc( 1 );
}

LRESULT CALLBACK WndProc( HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam )
{
   long int r;

   if( !sEVENTSFUNC )
   {
      sEVENTSFUNC = "EVENTS";
   }

   if( !pSymbolEvents )
   {
      pSymbolEvents = hb_dynsymSymbol( hb_dynsymGet(sEVENTSFUNC) );
   }

   if( pSymbolEvents )
   {
      hb_vmPushSymbol( pSymbolEvents );
      hb_vmPushNil();
      hb_vmPushLong( (LONG) hWnd );
      hb_vmPushLong( message );
      hb_vmPushLong( wParam );
      hb_vmPushLong( lParam );
      hb_vmDo( 4 );
   }

   r = hb_parnl( -1 );

   if( r != 0 )
   {
      return r;
   }
   else
   {
      return( DefWindowProc(hWnd, message, wParam, lParam) );
   }
}

HB_FUNC( INITWINDOW )
{
   HWND  hwnd;
   int   Style = WS_POPUP, ExStyle;

   if( hb_parl(16) )
   {
      ExStyle = WS_EX_CONTEXTHELP;
   }
   else
   {
      ExStyle = 0;
      if( !hb_parl(6) )
      {
         Style = Style | WS_MINIMIZEBOX;
      }

      if( !hb_parl(7) )
      {
         Style = Style | WS_MAXIMIZEBOX;
      }
   }

   if( !hb_parl(8) )
   {
      Style = Style | WS_SIZEBOX;
   }

   if( !hb_parl(9) )
   {
      Style = Style | WS_SYSMENU;
   }

   if( !hb_parl(10) )
   {
      Style = Style | WS_CAPTION;
   }

   if( hb_parl(11) )
   {
      ExStyle = ExStyle | WS_EX_TOPMOST;
   }

   if( hb_parl(14) )
   {
      Style = Style | WS_VSCROLL;
   }

   if( hb_parl(15) )
   {
      Style = Style | WS_HSCROLL;
   }

   if( hb_parl(17) )
   {
      ExStyle = ExStyle | WS_EX_PALETTEWINDOW;
   }

   hwnd = CreateWindowEx
      (
         ExStyle,
         hb_parc(12),
         hb_parc(1),
         Style,
         hb_parni(2),
         hb_parni(3),
         hb_parni(4),
         hb_parni(5),
         (HWND) hb_parnl(13),
         (HMENU) NULL,
         GetModuleHandle(NULL),
         NULL
      );

   if( hwnd == NULL )
   {
      MessageBox( 0, "Window Creation Failed!", "Error!", MB_ICONEXCLAMATION | MB_OK | MB_SYSTEMMODAL );
      return;
   }

   hb_retnl( (LONG) hwnd );
}

HB_FUNC( INITMODALWINDOW )
{
   HWND  parent;
   HWND  hwnd;
   int   Style;
   int   ExStyle = 0;

   if( hb_parl(13) )
   {
      ExStyle = WS_EX_CONTEXTHELP;
   }

   parent = ( HWND ) hb_parnl( 6 );

   Style = WS_POPUP;

   if( !hb_parl(7) )
   {
      Style = Style | WS_SIZEBOX;
   }

   if( !hb_parl(8) )
   {
      Style = Style | WS_SYSMENU;
   }

   if( !hb_parl(9) )
   {
      Style = Style | WS_CAPTION;
   }

   if( hb_parl(11) )
   {
      Style = Style | WS_VSCROLL;
   }

   if( hb_parl(12) )
   {
      Style = Style | WS_HSCROLL;
   }

   hwnd = CreateWindowEx
      (
         ExStyle,
         hb_parc(10),
         hb_parc(1),
         Style,
         hb_parni(2),
         hb_parni(3),
         hb_parni(4),
         hb_parni(5),
         parent,
         (HMENU) NULL,
         GetModuleHandle(NULL),
         NULL
      );

   if( hwnd == NULL )
   {
      MessageBox( 0, "Window Creation Failed!", "Error!", MB_ICONEXCLAMATION | MB_OK | MB_SYSTEMMODAL );
      return;
   }

   hb_retnl( (LONG) hwnd );
}

HB_FUNC( _DOMESSAGELOOP )
{
   MSG   Msg;

   while( GetMessage(&Msg, NULL, 0, 0) )
   {
      if( !IsWindow(GetActiveWindow()) || !IsDialogMessage(GetActiveWindow(), &Msg) )
      {
         TranslateMessage( &Msg );
         DispatchMessage( &Msg );
      }
   }

   return;
}

HB_FUNC( SHOWWINDOW )
{
   ShowWindow( (HWND) hb_parnl(1), SW_SHOW );
}

HB_FUNC( EXITPROCESS )
{
   ExitProcess( 0 );
}

HB_FUNC( MAXIMIZE )
{
   ShowWindow( (HWND) hb_parnl(1), SW_MAXIMIZE );
}

HB_FUNC( MINIMIZE )
{
   ShowWindow( (HWND) hb_parnl(1), SW_MINIMIZE );
}

HB_FUNC( RESTORE )
{
   ShowWindow( (HWND) hb_parnl(1), SW_RESTORE );
}

HB_FUNC( GETACTIVEWINDOW )
{
   HWND  hwnd;

   hwnd = GetActiveWindow();

   hb_retnl( (LONG) hwnd );
}

HB_FUNC( SETACTIVEWINDOW )
{
   SetActiveWindow( (HWND) hb_parnl(1) );
}

HB_FUNC( POSTQUITMESSAGE )
{
   PostQuitMessage( hb_parnl(1) );
}

HB_FUNC( DESTROYWINDOW )
{
   DestroyWindow( (HWND) hb_parnl(1) );
}

HB_FUNC( ISWINDOWVISIBLE )
{
   hb_retl( IsWindowVisible((HWND) hb_parnl(1)) );
}

HB_FUNC( ISWINDOWSIZED )
{
   hb_retl( GetWindowLong((HWND) hb_parnl(1), GWL_STYLE) & WS_SIZEBOX );
}

HB_FUNC( GETWINDOWLONG )
{
   hb_retnl( GetWindowLong((HWND) hb_parnl(1), hb_parni(2)) );
}

HB_FUNC( ISWINDOWENABLED )
{
   hb_retl( IsWindowEnabled((HWND) hb_parnl(1)) );
}

HB_FUNC( ENABLEWINDOW )
{
   EnableWindow( (HWND) hb_parnl(1), TRUE );
}

HB_FUNC( DISABLEWINDOW )
{
   EnableWindow( (HWND) hb_parnl(1), FALSE );
}

HB_FUNC( SETFOREGROUNDWINDOW )
{
   SetForegroundWindow( (HWND) hb_parnl(1) );
}

HB_FUNC( BRINGWINDOWTOTOP )
{
   BringWindowToTop( (HWND) hb_parnl(1) );
}

HB_FUNC( GETFOREGROUNDWINDOW )
{
   HWND  hwnd;

   hwnd = GetForegroundWindow();
   hb_retnl( (LONG) hwnd );
}

HB_FUNC( GETNEXTWINDOW )
{
   HWND  hwnd;

   hwnd = GetWindow( (HWND) hb_parnl(1), GW_HWNDNEXT );
   hb_retnl( (LONG) hwnd );
}

HB_FUNC( GETPREVWINDOW )
{
   HWND  hwnd;

   hwnd = GetWindow( (HWND) hb_parnl(1), GW_HWNDPREV );
   hb_retnl( (LONG) hwnd );
}

HB_FUNC( SETWINDOWTEXT )
{
   SetWindowText( (HWND) hb_parnl(1), (LPCTSTR) hb_parc(2) );
}

HB_FUNC( SETWINDOWBACKGROUND )
{
   SetClassLong( (HWND) hb_parnl(1), // window handle
   GCL_HBRBACKGROUND, // change back color
   (LONG) hb_parnl(2) );  // new brush handle
}

HB_FUNC( SETWINDOWPOS )
{
   hb_retl( SetWindowPos((HWND) hb_parnl(1), (HWND) hb_parni(2), hb_parni(3), hb_parni(4), hb_parni(5), hb_parni(6), hb_parni(7)) );
}

HB_FUNC( C_CENTER )
{
   HWND  hwnd;
   RECT  rect;
   int   w, h, x, y;

   hwnd = ( HWND ) hb_parnl( 1 );

   GetWindowRect( hwnd, &rect );
   w = rect.right - rect.left;
   h = rect.bottom - rect.top;
   x = GetSystemMetrics( SM_CXSCREEN );
   y = GetSystemMetrics( SM_CYSCREEN );
   SetWindowPos( hwnd, HWND_TOP, (x - w) / 2, (y - h) / 2, 0, 0, SWP_NOSIZE | SWP_NOACTIVATE );
}

HB_FUNC( GETWINDOWTEXT )
{
   HWND  hWnd = ( HWND ) hb_parnl( 1 );
   int   iLen = GetWindowTextLength( hWnd );
   char  *cText = ( char * ) hb_xgrab( iLen + 1 );
   int   iRet = GetWindowText( hWnd, (LPSTR) cText, iLen + 1 );

   hb_retclen( cText, iRet );
   hb_xfree( cText );
}

HB_FUNC( SENDMESSAGE )
{
   hb_retnl( (LONG) SendMessage((HWND) hb_parnl(1), (UINT) hb_parni(2), (WPARAM) hb_parnl(3), (LPARAM) hb_parnl(4)) );
}

HB_FUNC( SENDMESSAGESTRING )
{
   hb_retnl( (LONG) SendMessage((HWND) hb_parnl(1), (UINT) hb_parni(2), (WPARAM) hb_parnl(3), (LPARAM) (LPSTR) hb_parc(4)) );
}

HB_FUNC( UPDATEWINDOW )
{
   hb_retnl( (LONG) UpdateWindow((HWND) hb_parnl(1)) );
}

HB_FUNC( GETNOTIFYCODE )
{
   hb_retnl( (LONG) (((NMHDR FAR *) hb_parnl(1))->code) );
}

//JP 107a

HB_FUNC( GETNOTIFYID )
{
   hb_retnl( (LONG) (((NMHDR FAR *) hb_parnl(1))->idFrom) );
}

HB_FUNC( GETHWNDFROM )
{
   hb_retnl( (LONG) (((NMHDR FAR *) hb_parnl(1))->hwndFrom) );
}

HB_FUNC( GETDRAWITEMHANDLE )
{
   hb_retnl( (LONG) (((DRAWITEMSTRUCT FAR *) hb_parnl(1))->hwndItem) );
}

HB_FUNC( GETFOCUS )
{
   hb_retnl( (LONG) GetFocus() );
}

HB_FUNC( GETGRIDCOLUMN )
{
   hb_retnl( (LPARAM) ((( NM_LISTVIEW * ) hb_parnl(1))->iSubItem) );
}

HB_FUNC( GETGRIDVKEY )
{
   hb_retnl( (LPARAM) ((( LV_KEYDOWN * ) hb_parnl(1))->wVKey) );
}

HB_FUNC( MOVEWINDOW )
{
   hb_retl( MoveWindow((HWND) hb_parnl(1), hb_parni(2), hb_parni(3), hb_parni(4), hb_parni(5), (ISNIL(6) ? TRUE : hb_parl(6))) );
}

HB_FUNC( GETWINDOWRECT )
{
   RECT  rect;
   hb_retl( GetWindowRect((HWND) hb_parnl(1), &rect) );
   hb_stornl( rect.left, 2, 1 );
   hb_stornl( rect.top, 2, 2 );
   hb_stornl( rect.right, 2, 3 );
   hb_stornl( rect.bottom, 2, 4 );
}

HB_FUNC( REGISTERWINDOW )
{
   WNDCLASS WndClass;

   HBRUSH   hbrush = 0;
   LPCTSTR  lpIcon = ISCHAR( 1 ) ? hb_parc( 1 ) : ( ISNIL(1) ? NULL : MAKEINTRESOURCE(hb_parni(1)) );

   WndClass.style = CS_OWNDC;
   WndClass.lpfnWndProc = WndProc;
   WndClass.cbClsExtra = 0;
   WndClass.cbWndExtra = 0;
   WndClass.hInstance = GetModuleHandle( NULL );
   WndClass.hIcon = LoadIcon( GetModuleHandle(NULL), lpIcon );
   if( WndClass.hIcon == NULL )
   {
      WndClass.hIcon = ( HICON ) LoadImage( GetModuleHandle(NULL), lpIcon, IMAGE_ICON, 0, 0, LR_LOADFROMFILE + LR_DEFAULTSIZE );
   }

   if( WndClass.hIcon == NULL )
   {
      WndClass.hIcon = LoadIcon( NULL, IDI_APPLICATION );
   }

   WndClass.hCursor = LoadCursor( NULL, IDC_ARROW );

   if( hb_parni(3, 1) == -1 )
   {
      WndClass.hbrBackground = ( HBRUSH ) ( COLOR_BTNFACE + 1 );
   }
   else
   {
      hbrush = CreateSolidBrush( RGB(hb_parni(3, 1), hb_parni(3, 2), hb_parni(3, 3)) );
      WndClass.hbrBackground = hbrush;
   }

   WndClass.lpszMenuName = NULL;
   WndClass.lpszClassName = hb_parc( 2 );

   if( !RegisterClass(&WndClass) )
   {
      MessageBox( 0, "Window Registration Failed!", "Error!", MB_ICONEXCLAMATION | MB_OK | MB_SYSTEMMODAL );
      ExitProcess( 0 );
   }

   hb_retnl( (LONG) hbrush );
}

HB_FUNC( UNREGISTERWINDOW )
{
   UnregisterClass( hb_parc(1), GetModuleHandle(NULL) );
}

HB_FUNC( GETDESKTOPWIDTH )
{
   hb_retni( GetSystemMetrics(SM_CXSCREEN) );
}

HB_FUNC( GETVSCROLLBARWIDTH )
{
   hb_retni( GetSystemMetrics(SM_CXVSCROLL) );
}

HB_FUNC( GETHSCROLLBARHEIGHT )
{
   hb_retni( GetSystemMetrics(SM_CYHSCROLL) );
}

HB_FUNC( GETDESKTOPHEIGHT )
{
   hb_retni( GetSystemMetrics(SM_CYSCREEN) );
}

HB_FUNC( GETWINDOWROW )
{
   RECT  rect;

   GetWindowRect( (HWND) hb_parnl(1), &rect );
   hb_retni( (INT) rect.top );
}

HB_FUNC( GETWINDOWCOL )
{
   RECT  rect;

   GetWindowRect( (HWND) hb_parnl(1), &rect );
   hb_retni( (INT) rect.left );
}

HB_FUNC( GETWINDOWWIDTH )
{
   RECT  rect;

   GetWindowRect( (HWND) hb_parnl(1), &rect );
   hb_retni( (INT) rect.right - rect.left );
}

HB_FUNC( GETWINDOWHEIGHT )
{
   RECT  rect;

   GetWindowRect( (HWND) hb_parnl(1), &rect );
   hb_retni( (INT) rect.bottom - rect.top );
}

HB_FUNC( GETTITLEHEIGHT )
{
   hb_retni( GetSystemMetrics(SM_CYCAPTION) );
}

HB_FUNC( GETBORDERHEIGHT )
{
   hb_retni( GetSystemMetrics(SM_CYSIZEFRAME) );
}

HB_FUNC( GETBORDERWIDTH )
{
   hb_retni( GetSystemMetrics(SM_CXSIZEFRAME) );
}

// JD 07/22/2007

HB_FUNC( GET3DEDGEHEIGHT )
{
   hb_retni( GetSystemMetrics(SM_CYEDGE) );
}

// JD 07/22/2007

HB_FUNC( GET3DEDGEWIDTH )
{
   hb_retni( GetSystemMetrics(SM_CXEDGE) );
}

HB_FUNC( GETMENUBARHEIGHT )
{
   hb_retni( GetSystemMetrics(SM_CYMENU) );
}

HB_FUNC( SHOWNOTIFYICON )
{
   hb_retl( (BOOL) ShowNotifyIcon((HWND) hb_parnl(1), (BOOL) hb_parl(2), (HICON) hb_parnl(3), (LPSTR) hb_parc(4)) );
}

static BOOL ShowNotifyIcon( HWND hWnd, BOOL bAdd, HICON hIcon, LPSTR szText )
{
   BOOL           bRet;
   NOTIFYICONDATA nid;

   ZeroMemory( &nid, sizeof(nid) );

   nid.cbSize = sizeof( NOTIFYICONDATA );
   nid.hIcon = hIcon;
   nid.hWnd = hWnd;
   nid.uID = 0;
   nid.uFlags = NIF_ICON | NIF_MESSAGE | NIF_TIP;
   nid.uCallbackMessage = WM_TASKBAR;
   lstrcpy( nid.szTip, TEXT(szText) );

   if( bAdd )
   {
      bRet = Shell_NotifyIcon( NIM_ADD, &nid );
   }
   else
   {
      bRet = Shell_NotifyIcon( NIM_DELETE, &nid );
   }

   if( hIcon )
   {
      DestroyIcon( hIcon );
   }

   return bRet;
}

HB_FUNC( GETINSTANCE )
{
   hb_retnl( (LONG) GetModuleHandle(NULL) );
}

HB_FUNC( GETCURSORPOS )
{
   POINT pt;

   GetCursorPos( &pt );

   hb_reta( 2 );

   hb_storni( pt.y, -1, 1 );
   hb_storni( pt.x, -1, 2 );
}

HB_FUNC( LOADTRAYICON )
{
   HICON       himage;
   HINSTANCE   hInstance = ( HINSTANCE ) hb_parnl( 1 );  // handle to application instance
   LPCTSTR     lpIconName = ISCHAR( 2 ) ? hb_parc( 2 ) : MAKEINTRESOURCE( hb_parni(2) );  // name string or resource identifier
   himage = LoadIcon( hInstance, lpIconName );

   if( himage == NULL )
   {
      himage = ( HICON ) LoadImage( hInstance, lpIconName, IMAGE_ICON, 0, 0, LR_LOADFROMFILE + LR_DEFAULTSIZE );
   }

   hb_retnl( (LONG) himage );
}

HB_FUNC( CHANGENOTIFYICON )
{
   hb_retl( (BOOL) ChangeNotifyIcon((HWND) hb_parnl(1), (HICON) hb_parnl(2), (LPSTR) hb_parc(3)) );
}

static BOOL ChangeNotifyIcon( HWND hWnd, HICON hIcon, LPSTR szText )
{
   BOOL           bRet;
   NOTIFYICONDATA nid;

   ZeroMemory( &nid, sizeof(nid) );

   nid.cbSize = sizeof( NOTIFYICONDATA );
   nid.hIcon = hIcon;
   nid.hWnd = hWnd;
   nid.uID = 0;
   nid.uFlags = NIF_ICON | NIF_MESSAGE | NIF_TIP;
   nid.uCallbackMessage = WM_TASKBAR;
   lstrcpy( nid.szTip, TEXT(szText) );

   bRet = Shell_NotifyIcon( NIM_MODIFY, &nid );

   if( hIcon )
   {
      DestroyIcon( hIcon );
   }

   return bRet;
}

HB_FUNC( INITSPLITBOX )
{
   HWND                 hwndOwner = ( HWND ) hb_parnl( 1 );
   REBARINFO            rbi;
   HWND                 hwndRB;
   INITCOMMONCONTROLSEX icex;

   int                  Style = WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN | RBS_BANDBORDERS | RBS_VARHEIGHT | RBS_FIXEDORDER;

   if( hb_parl(2) )
   {
      Style = Style | CCS_BOTTOM;
   }

   if( hb_parl(3) )
   {
      Style = Style | CCS_VERT;
   }

   icex.dwSize = sizeof( INITCOMMONCONTROLSEX );
   icex.dwICC = ICC_COOL_CLASSES | ICC_BAR_CLASSES;
   InitCommonControlsEx( &icex );

   hwndRB = CreateWindowEx
      (
         WS_EX_TOOLWINDOW | WS_EX_DLGMODALFRAME,
         REBARCLASSNAME,
         NULL,
         Style,
         0,
         0,
         0,
         0,
         hwndOwner,
         NULL,
         GetModuleHandle(NULL),
         NULL
      );

   // Initialize and send the REBARINFO structure.

   rbi.cbSize = sizeof( REBARINFO );   // Required when using this struct.
   rbi.fMask = 0;
   rbi.himl = ( HIMAGELIST ) NULL;
   SendMessage( hwndRB, RB_SETBARINFO, 0, (LPARAM) & rbi );

   hb_retnl( (LONG) hwndRB );
}

HB_FUNC( INITSPLITCHILDWINDOW )
{
   HWND  hwnd;
   int   Style;

   Style = WS_POPUP;

   if( !hb_parl(4) )
   {
      Style = Style | WS_CAPTION;
   }

   if( hb_parl(7) )
   {
      Style = Style | WS_VSCROLL;
   }

   if( hb_parl(8) )
   {
      Style = Style | WS_HSCROLL;
   }

   hwnd = CreateWindowEx
      (
         WS_EX_STATICEDGE | WS_EX_TOOLWINDOW,
         hb_parc(3),
         hb_parc(5),
         Style,
         0,
         0,
         hb_parni(1),
         hb_parni(2),
         0,
         (HMENU) NULL,
         GetModuleHandle(NULL),
         NULL
      );

   if( hwnd == NULL )
   {
      MessageBox( 0, "Window Creation Failed!", "Error!", MB_ICONEXCLAMATION | MB_OK | MB_SYSTEMMODAL );
      return;
   }

   hb_retnl( (LONG) hwnd );
}

HB_FUNC( SIZEREBAR )
{
   SendMessage( (HWND) hb_parnl(1), RB_SHOWBAND, (WPARAM) (INT) 0, (LPARAM) (BOOL) 0 );
   SendMessage( (HWND) hb_parnl(1), RB_SHOWBAND, (WPARAM) (INT) 0, (LPARAM) (BOOL) 1 );
}

HB_FUNC( REBARHEIGHT )
{
   LRESULT  lResult;
   lResult = SendMessage( (HWND) hb_parnl(1), (UINT) RB_GETBARHEIGHT, 0, 0 );
   hb_retnl( (UINT) lResult );
}

HB_FUNC( GETITEMPOS )
{
   hb_retnl( (LONG) (((NMMOUSE FAR *) hb_parnl(1))->dwItemSpec) );
}

HB_FUNC( SETSCROLLRANGE )
{
   hb_retl( SetScrollRange((HWND) hb_parnl(1), hb_parni(2), hb_parni(3), hb_parni(4), hb_parl(5)) );
}

HB_FUNC( GETSCROLLPOS )
{
   hb_retni( GetScrollPos((HWND) hb_parnl(1), hb_parni(2)) );
}

HB_FUNC( GETWINDOWSTATE )
{
   WINDOWPLACEMENT   wp;

   wp.length = sizeof( WINDOWPLACEMENT );

   GetWindowPlacement( (HWND) hb_parnl(1), &wp );

   hb_retni( wp.showCmd );
}

HB_FUNC( REDRAWWINDOW )
{
   RedrawWindow( (HWND) hb_parnl(1), NULL, NULL, RDW_ERASE | RDW_INVALIDATE | RDW_ALLCHILDREN | RDW_ERASENOW | RDW_UPDATENOW );
}

HB_FUNC( REDRAWWINDOWCONTROLRECT )
{
   RECT  r;

   r.top = hb_parni( 2 );
   r.left = hb_parni( 3 );
   r.bottom = hb_parni( 4 );
   r.right = hb_parni( 5 );

   RedrawWindow( (HWND) hb_parnl(1), &r, NULL, RDW_ERASE | RDW_INVALIDATE | RDW_ALLCHILDREN | RDW_ERASENOW | RDW_UPDATENOW );
}

HB_FUNC( ADDSPLITBOXITEM )
{
   REBARBANDINFO  rbBand;
   RECT           rc;
   int            Style = RBBS_CHILDEDGE | RBBS_GRIPPERALWAYS | RBBS_USECHEVRON;

   if( hb_parl(4) )
   {
      Style = Style | RBBS_BREAK;
   }

   GetWindowRect( (HWND) hb_parnl(1), &rc );

   rbBand.cbSize = sizeof( REBARBANDINFO );
   rbBand.fMask = RBBIM_TEXT | RBBIM_STYLE | RBBIM_CHILD | RBBIM_CHILDSIZE | RBBIM_SIZE;
   rbBand.fStyle = Style;
   rbBand.hbmBack = 0;

   rbBand.lpText = hb_parc( 5 );
   rbBand.hwndChild = ( HWND ) hb_parnl( 1 );

   if( hb_parni(9) )
   {
      rbBand.fMask = rbBand.fMask | RBBIM_IDEALSIZE;
   }

   if( !hb_parl(8) )
   {
      // Not Horizontal

      rbBand.cxMinChild = hb_parni( 6 ) ? hb_parni( 6 ) : 0;
      rbBand.cyMinChild = hb_parni( 7 ) ? hb_parni( 7 ) : rc.bottom - rc.top;
      rbBand.cx = hb_parni( 3 );
      if( hb_parni(9) )
      {
         rbBand.cxIdeal = hb_parni( 6 ) ? hb_parni( 6 ) : 0;
         rbBand.cxMinChild = hb_parni( 9 );
      }
      else
      {
         rbBand.cxMinChild = hb_parni( 6 ) ? hb_parni( 6 ) : 0;
      }
   }
   else
   {
      // Horizontal

      if( hb_parni(6) == 0 && hb_parni(7) == 0 )
      {
         // Not ToolBar

         rbBand.cxMinChild = 0;
         rbBand.cyMinChild = rc.right - rc.left;
         rbBand.cx = rc.bottom - rc.top;
      }
      else
      {
         // ToolBar

         rbBand.cyMinChild = hb_parni( 6 ) ? hb_parni( 6 ) : 0;
         rbBand.cx = hb_parni( 7 ) ? hb_parni( 7 ) : rc.bottom - rc.top;
         if( hb_parni(9) )
         {
            rbBand.cxIdeal = hb_parni( 7 ) ? hb_parni( 7 ) : rc.bottom - rc.top;
            rbBand.cxMinChild = hb_parni( 9 );
         }
         else
         {
            rbBand.cxMinChild = hb_parni( 7 ) ? hb_parni( 7 ) : rc.bottom - rc.top;
         }
      }
   }

   SendMessage( (HWND) hb_parnl(2), RB_INSERTBAND, (WPARAM) - 1, (LPARAM) & rbBand );
}

HB_FUNC( C_SETWINDOWRGN )
{
   HRGN     hrgn;
   HBITMAP  hbmp;

   if( hb_parni(6) == 0 )
   {
      SetWindowRgn( GetActiveWindow(), NULL, TRUE );
   }
   else
   {
      switch( hb_parni(6) )
      {
         case 1:
            hrgn = CreateRectRgn( hb_parni(2), hb_parni(3), hb_parni(4), hb_parni(5) );
            break;

         case 2:
            hrgn = CreateEllipticRgn( hb_parni(2), hb_parni(3), hb_parni(4), hb_parni(5) );
            break;

         case 3:
            hrgn = CreateRoundRectRgn( 0, 0, hb_parni(4), hb_parni(5), hb_parni(2), hb_parni(3) );
            break;

         case 4:
            hbmp = ( HBITMAP ) LoadImage( GetModuleHandle(NULL), hb_parc(2), IMAGE_BITMAP, 0, 0, LR_CREATEDIBSECTION );
            if( hbmp == NULL )
            {
               hbmp = ( HBITMAP ) LoadImage( NULL, hb_parc(2), IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE | LR_CREATEDIBSECTION );
            }

            hrgn = BitmapToRegion( hbmp, (COLORREF) RGB(( int ) hb_parni(3, 1), ( int ) hb_parni(3, 2), ( int ) hb_parni(3, 3)), 0x101010 );
            DeleteObject( hbmp );
            break;

         default:
            break;
      }

      SetWindowRgn( (HWND) hb_parnl(1), hrgn, TRUE );
      hb_retnl( (LONG) hrgn );
   }
}

HB_FUNC( C_SETPOLYWINDOWRGN )
{
   HRGN  hrgn;
   POINT lppt[512];
   int   i, fnPolyFillMode;
   int   cPoints = hb_parinfa( 2, 0 );

   if( hb_parni(4) == 1 )
   {
      fnPolyFillMode = WINDING;
   }
   else
   {
      fnPolyFillMode = ALTERNATE;
   }

   for( i = 0; i <= cPoints - 1; i++ )
   {
      lppt[i].x = hb_parni( 2, i + 1 );
      lppt[i].y = hb_parni( 3, i + 1 );
   }

   hrgn = CreatePolygonRgn( lppt, cPoints, fnPolyFillMode );
   SetWindowRgn( GetActiveWindow(), hrgn, TRUE );
   hb_retnl( (LONG) hrgn );
}

HB_FUNC( GETHELPDATA )
{
   hb_retnl( (LONG) (((HELPINFO FAR *) hb_parnl(1))->hItemHandle) );
}

HB_FUNC( GETMSKTEXTMESSAGE )
{
   hb_retnl( (LONG) (((MSGFILTER FAR *) hb_parnl(1))->msg) );
}

HB_FUNC( GETMSKTEXTWPARAM )
{
   hb_retnl( (LONG) (((MSGFILTER FAR *) hb_parnl(1))->wParam) );
}

HB_FUNC( GETMSKTEXTLPARAM )
{
   hb_retnl( (LONG) (((MSGFILTER FAR *) hb_parnl(1))->lParam) );
}

HB_FUNC( GETWINDOW )
{
   hb_retnl( (LONG) GetWindow((HWND) hb_parnl(1), hb_parni(2)) );
}

HB_FUNC( GETGRIDOLDSTATE )
{
   #define pnm ( ( NM_LISTVIEW * ) hb_parnl(1) )

   hb_retnl( (LPARAM) (pnm->uOldState) );

   #undef pnm
}

HB_FUNC( GETGRIDNEWSTATE )
{
   #define pnm ( ( NM_LISTVIEW * ) hb_parnl(1) )

   hb_retnl( (LPARAM) (pnm->uNewState) );

   #undef pnm
}

HB_FUNC( GETGRIDDISPINFOINDEX )
{
   LV_DISPINFO *pDispInfo = ( LV_DISPINFO * ) hb_parnl( 1 );

   int         iItem = pDispInfo->item.iItem;

   int         iSubItem = pDispInfo->item.iSubItem;

   hb_reta( 2 );
   hb_storni( iItem + 1, -1, 1 );
   hb_storni( iSubItem + 1, -1, 2 );
}

HB_FUNC( SETGRIDQUERYDATA )
{
   PHB_ITEM    pValue = hb_itemNew( NULL );
   LV_DISPINFO *pDispInfo = ( LV_DISPINFO * ) hb_parnl( 1 );
   hb_itemCopy( pValue, hb_param(2, HB_IT_STRING) );
   pDispInfo->item.pszText = hb_itemGetCPtr( pValue );
}

HB_FUNC( SETGRIDQUERYIMAGE )
{
   LV_DISPINFO *pDispInfo = ( LV_DISPINFO * ) hb_parnl( 1 );
   pDispInfo->item.iImage = hb_parni( 2 );
}

HB_FUNC( GETESCAPESTATE )
{
   hb_retni( GetKeyState(VK_ESCAPE) );
}

HB_FUNC( GETALTSTATE )
{
   hb_retni( GetKeyState(VK_MENU) );
}

HB_FUNC( GETCURSORROW )
{
   POINT pt;

   GetCursorPos( &pt );
   hb_retni( pt.y );
}

HB_FUNC( GETCURSORCOL )
{
   POINT pt;

   GetCursorPos( &pt );
   hb_retni( pt.x );
}

HB_FUNC( ISINSERTACTIVE )
{
   hb_retl( GetKeyState(VK_INSERT) );
}

HB_FUNC( ISCAPSLOCKACTIVE )
{
   hb_retl( GetKeyState(VK_CAPITAL) );
}

HB_FUNC( ISNUMLOCKACTIVE )
{
   hb_retl( GetKeyState(VK_NUMLOCK) );
}

HB_FUNC( ISSCROLLLOCKACTIVE )
{
   hb_retl( GetKeyState(VK_SCROLL) );
}

HB_FUNC( FINDWINDOWEX )
{
   hb_retnl( (LONG) FindWindowEx((HWND) hb_parnl(1), (HWND) hb_parnl(2), (LPCSTR) hb_parc(3), (LPCSTR) hb_parc(4)) );
}

HB_FUNC( INITDUMMY )
{
   CreateWindowEx( 0, "static", "", WS_CHILD, 0, 0, 0, 0, (HWND) hb_parnl(1), (HMENU) 0, GetModuleHandle(NULL), NULL );
}

HB_FUNC( REGISTERSPLITCHILDWINDOW )
{
   WNDCLASS WndClass;

   HBRUSH   hbrush = 0;
   LPCTSTR  lpIcon = ISCHAR( 1 ) ? hb_parc( 1 ) : ( ISNIL(1) ? NULL : MAKEINTRESOURCE(hb_parni(1)) );

   WndClass.style = CS_OWNDC;
   WndClass.lpfnWndProc = WndProc;
   WndClass.cbClsExtra = 0;
   WndClass.cbWndExtra = 0;
   WndClass.hInstance = GetModuleHandle( NULL );
   WndClass.hIcon = LoadIcon( GetModuleHandle(NULL), lpIcon );
   if( WndClass.hIcon == NULL )
   {
      WndClass.hIcon = ( HICON ) LoadImage( GetModuleHandle(NULL), lpIcon, IMAGE_ICON, 0, 0, LR_LOADFROMFILE + LR_DEFAULTSIZE );
   }

   if( WndClass.hIcon == NULL )
   {
      WndClass.hIcon = LoadIcon( NULL, IDI_APPLICATION );
   }

   WndClass.hCursor = LoadCursor( NULL, IDC_ARROW );

   if( hb_parni(3, 1) == -1 )
   {
      WndClass.hbrBackground = ( HBRUSH ) ( COLOR_BTNFACE + 1 );
   }
   else
   {
      hbrush = CreateSolidBrush( RGB(hb_parni(3, 1), hb_parni(3, 2), hb_parni(3, 3)) );
      WndClass.hbrBackground = hbrush;
   }

   WndClass.lpszMenuName = NULL;
   WndClass.lpszClassName = hb_parc( 2 );

   if( !RegisterClass(&WndClass) )
   {
      MessageBox( 0, "Window Registration Failed!", "Error!", MB_ICONEXCLAMATION | MB_OK | MB_SYSTEMMODAL );
      ExitProcess( 0 );
   }

   hb_retnl( (LONG) hbrush );
}

HB_FUNC( GETDS )
{
   LPARAM            lParam = hb_parnl( 1 );

   LPNMLVCUSTOMDRAW  lplvcd = ( LPNMLVCUSTOMDRAW ) lParam;

   if( lplvcd->nmcd.dwDrawStage == CDDS_PREPAINT )
   {
      hb_retni( CDRF_NOTIFYITEMDRAW );
   }
   else if( lplvcd->nmcd.dwDrawStage == CDDS_ITEMPREPAINT )
   {
      hb_retni( CDRF_NOTIFYSUBITEMDRAW );
   }
   else if( lplvcd->nmcd.dwDrawStage == (CDDS_SUBITEM | CDDS_ITEMPREPAINT) )
   {
      hb_retni( -1 );
   }
   else
   {
      hb_retni( CDRF_DODEFAULT );
   }
}

HB_FUNC( GETRC )
{
   LPARAM            lParam = hb_parnl( 1 );

   LPNMLVCUSTOMDRAW  lplvcd = ( LPNMLVCUSTOMDRAW ) lParam;

   hb_reta( 2 );
   hb_stornl( (LONG) lplvcd->nmcd.dwItemSpec + 1, -1, 1 );
   hb_storni( (INT) lplvcd->iSubItem + 1, -1, 2 );
}

HB_FUNC( SETBCBC )
{
   LPARAM            lParam = hb_parnl( 1 );

   LPNMLVCUSTOMDRAW  lplvcd = ( LPNMLVCUSTOMDRAW ) lParam;

   lplvcd->clrText = hb_parni( 3 );
   lplvcd->clrTextBk = hb_parni( 2 );
   hb_retni( CDRF_NEWFONT );
}

HB_FUNC( SETBRCCD )
{
   LPARAM            lParam = hb_parnl( 1 );

   LPNMLVCUSTOMDRAW  lplvcd = ( LPNMLVCUSTOMDRAW ) lParam;

   lplvcd->clrText = RGB( 0, 0, 0 );
   lplvcd->clrTextBk = RGB( 255, 255, 255 );
   hb_retni( CDRF_NEWFONT );
}

HB_FUNC( GETTABBEDCONTROLBRUSH )
{
   RECT     rc;
   HBRUSH   hBrush;

   SetBkMode( (HDC) hb_parnl(1), TRANSPARENT );
   GetWindowRect( (HWND) hb_parnl(2), &rc );
   MapWindowPoints( NULL, (HWND) hb_parnl(3), (LPPOINT) (&rc), 2 );
   SetBrushOrgEx( (HDC) hb_parnl(1), -rc.left, -rc.top, NULL );
   hBrush = ( HBRUSH ) hb_parnl( 4 );
   hb_retnl( (LONG) hBrush );
}

HB_FUNC( GETTABBRUSH )
{
   HBRUSH   hBrush;
   RECT     rc;
   HDC      hDC;
   HDC      hDCMem;
   HBITMAP  hBmp;
   HBITMAP  hOldBmp;
   HWND     hWnd;

   hWnd = ( HWND ) hb_parnl( 1 );

   GetWindowRect( hWnd, &rc );
   hDC = GetDC( hWnd );
   hDCMem = CreateCompatibleDC( hDC );

   hBmp = CreateCompatibleBitmap( hDC, rc.right - rc.left, rc.bottom - rc.top );

   hOldBmp = ( HBITMAP ) SelectObject( hDCMem, hBmp );

   SendMessage( hWnd, WM_PRINTCLIENT, (WPARAM) hDCMem, (LPARAM) PRF_ERASEBKGND | PRF_CLIENT | PRF_NONCLIENT );

   hBrush = CreatePatternBrush( hBmp );

   hb_retnl( (LONG) hBrush );

   SelectObject( hDCMem, hOldBmp );

   DeleteObject( hBmp );
   DeleteDC( hDCMem );
   ReleaseDC( hWnd, hDC );
}

HB_FUNC( INITMINMAXINFO )  // ( hWnd ) --> aMinMaxInfo
{
   long  x, y, mx, my;

   if( GetWindowLong((HWND) hb_parnl(1), GWL_STYLE) & WS_SIZEBOX )
   {
      x = -GetSystemMetrics( SM_CXFRAME );
      y = -GetSystemMetrics( SM_CYFRAME );
   }
   else
   {
      x = -GetSystemMetrics( SM_CXBORDER );
      y = -GetSystemMetrics( SM_CYBORDER );
   }

   mx = GetSystemMetrics( SM_CXSCREEN ) - 2 * x;
   my = GetSystemMetrics( SM_CYSCREEN ) - 2 * y;

   hb_reta( 8 );

   hb_stornl( (LONG) mx, -1, 1 );
   hb_stornl( (LONG) my, -1, 2 );
   hb_stornl( (LONG) x, -1, 3 );
   hb_stornl( (LONG) y, -1, 4 );
   hb_stornl( (LONG) 0, -1, 5 );
   hb_stornl( (LONG) 0, -1, 6 );
   hb_stornl( (LONG) mx, -1, 7 );
   hb_stornl( (LONG) my, -1, 8 );
}

HB_FUNC( SETMINMAXINFO )   // ( pMinMaxInfo, aMinMaxInfo ) --> 0
{
   MINMAXINFO  *pMinMaxInfo = ( MINMAXINFO * ) hb_parnl( 1 );

   pMinMaxInfo->ptMaxSize.x = hb_parnl( 2, 1 );
   pMinMaxInfo->ptMaxSize.y = hb_parnl( 2, 2 );
   pMinMaxInfo->ptMaxPosition.x = hb_parnl( 2, 3 );
   pMinMaxInfo->ptMaxPosition.y = hb_parnl( 2, 4 );
   pMinMaxInfo->ptMinTrackSize.x = hb_parnl( 2, 5 );
   pMinMaxInfo->ptMinTrackSize.y = hb_parnl( 2, 6 );
   pMinMaxInfo->ptMaxTrackSize.x = hb_parnl( 2, 7 );
   pMinMaxInfo->ptMaxTrackSize.y = hb_parnl( 2, 8 );

   hb_retni( 0 );
}

//
//  BitmapToRegion: Create a region from the "non-transparent" pixels of a bitmap
//  Author        : Jean-Edouard Lachand-Robert
// (http://www.geocities.com/Paris/LeftBank/1160/resume.htm), June 1998.
//
//  hBmp :              Source bitmap
//  cTransparentColor : Color base for the "transparent" pixels
//                      (default is black)
//  cTolerance :        Color tolerance for the "transparent" pixels.
//
//  A pixel is assumed to be transparent if the value of each of its 3
//  components (blue, green and red) is
//  greater or equal to the corresponding value in cTransparentColor and is
//  lower or equal to the corresponding value in cTransparentColor + cTolerance.
//

#define ALLOC_UNIT   100

HRGN BitmapToRegion( HBITMAP hBmp, COLORREF cTransparentColor, COLORREF cTolerance )
{
   HRGN  hRgn = NULL;
   VOID  *pbits32;
   DWORD maxRects = ALLOC_UNIT;

   if( hBmp )
   {
      // Create a memory DC inside which we will scan the bitmap content

      HDC   hMemDC = CreateCompatibleDC( NULL );
      if( hMemDC )
      {
         BITMAP            bm;
         BITMAPINFOHEADER  RGB32BITSBITMAPINFO;
         HBITMAP           hbm32;

         // Get bitmap size

         GetObject( hBmp, sizeof(bm), &bm );

         // Create a 32 bits depth bitmap and select it into the memory DC

         RGB32BITSBITMAPINFO.biSize = sizeof( BITMAPINFOHEADER );
         RGB32BITSBITMAPINFO.biWidth = bm.bmWidth;
         RGB32BITSBITMAPINFO.biHeight = bm.bmHeight;
         RGB32BITSBITMAPINFO.biPlanes = 1;
         RGB32BITSBITMAPINFO.biBitCount = 32;
         RGB32BITSBITMAPINFO.biCompression = BI_RGB;
         RGB32BITSBITMAPINFO.biSizeImage = 0;
         RGB32BITSBITMAPINFO.biXPelsPerMeter = 0;
         RGB32BITSBITMAPINFO.biYPelsPerMeter = 0;
         RGB32BITSBITMAPINFO.biClrUsed = 0;
         RGB32BITSBITMAPINFO.biClrImportant = 0;

         hbm32 = CreateDIBSection( hMemDC, ( BITMAPINFO * ) &RGB32BITSBITMAPINFO, DIB_RGB_COLORS, &pbits32, NULL, 0 );
         if( hbm32 )
         {
            HBITMAP  holdBmp = ( HBITMAP ) SelectObject( hMemDC, hbm32 );

            // Create a DC just to copy the bitmap into the memory DC

            HDC      hDC = CreateCompatibleDC( hMemDC );
            if( hDC )
            {
               // Get how many bytes per row we have for the bitmap bits (rounded up to 32 bits)

               BITMAP   bm32;
               HBITMAP  holdBmp;
               HANDLE   hData;
               RGNDATA  *pData;
               BYTE     *p32;
               BYTE     lr, lg, lb, hr, hg, hb;
               INT      y, x;
               HRGN     h;

               GetObject( hbm32, sizeof(bm32), &bm32 );
               while( bm32.bmWidthBytes % 4 )
               {
                  bm32.bmWidthBytes++;
               }

               // Copy the bitmap into the memory DC

               holdBmp = ( HBITMAP ) SelectObject( hDC, hBmp );
               BitBlt( hMemDC, 0, 0, bm.bmWidth, bm.bmHeight, hDC, 0, 0, SRCCOPY );

               // For better performances, we will use the  ExtCreateRegion() function to create the  region.
               // This function take a RGNDATA structure on  entry.
               // We will add rectangles by amount of ALLOC_UNIT number in this structure.

               hData = GlobalAlloc( GMEM_MOVEABLE, sizeof(RGNDATAHEADER) + (sizeof(RECT) * maxRects) );

               pData = ( RGNDATA * ) GlobalLock( hData );
               pData->rdh.dwSize = sizeof( RGNDATAHEADER );
               pData->rdh.iType = RDH_RECTANGLES;
               pData->rdh.nCount = pData->rdh.nRgnSize = 0;
               SetRect( &pData->rdh.rcBound, MAXLONG, MAXLONG, 0, 0 );

               // Keep on hand highest and lowest values for the  "transparent" pixels

               lr = GetRValue( cTransparentColor );
               lg = GetGValue( cTransparentColor );
               lb = GetBValue( cTransparentColor );
               hr = ( BYTE ) min( 0xff, lr + GetRValue(cTolerance) );
               hg = ( BYTE ) min( 0xff, lg + GetGValue(cTolerance) );
               hb = ( BYTE ) min( 0xff, lb + GetBValue(cTolerance) );

               // Scan each bitmap row from bottom to top (the bitmap is  inverted vertically)

               p32 = ( BYTE * ) bm32.bmBits + ( bm32.bmHeight - 1 ) * bm32.bmWidthBytes;
               for( y = 0; y < bm.bmHeight; y++ )
               {     // Scan each bitmap pixel from left to right
                  for( x = 0; x < bm.bmWidth; x++ )
                  {  // Search for a continuous range of "non transparent pixels"
                     int   x0 = x;
                     LONG  *p = ( LONG * ) p32 + x;
                     while( x < bm.bmWidth )
                     {
                        BYTE  b = GetRValue( *p );
                        if( b >= lr && b <= hr )
                        {
                           b = GetGValue( *p );
                           if( b >= lg && b <= hg )
                           {
                              b = GetBValue( *p );
                              if( b >= lb && b <= hb )
                              {
                                 break;   // This pixel is "transparent"
                              }
                           }
                        }

                        p++;
                        x++;
                     }

                     if( x > x0 )
                     {                    // Add the pixels (x0, y) to (x, y+1) as a new rectangle in the region
                        RECT  *pr;
                        if( pData->rdh.nCount >= maxRects )
                        {
                           GlobalUnlock( hData );
                           maxRects += ALLOC_UNIT;
                           hData = GlobalReAlloc( hData, sizeof(RGNDATAHEADER) + (sizeof(RECT) * maxRects), GMEM_MOVEABLE );
                           pData = ( RGNDATA * ) GlobalLock( hData );
                        }

                        pr = ( RECT * ) &pData->Buffer;
                        SetRect( &pr[pData->rdh.nCount], x0, y, x, y + 1 );
                        if( x0 < pData->rdh.rcBound.left )
                        {
                           pData->rdh.rcBound.left = x0;
                        }

                        if( y < pData->rdh.rcBound.top )
                        {
                           pData->rdh.rcBound.top = y;
                        }

                        if( x > pData->rdh.rcBound.right )
                        {
                           pData->rdh.rcBound.right = x;
                        }

                        if( y + 1 > pData->rdh.rcBound.bottom )
                        {
                           pData->rdh.rcBound.bottom = y + 1;
                        }

                        pData->rdh.nCount++;

                        // On Windows98, ExtCreateRegion() may fail if  the number of rectangles is too
                        // large (ie: > 4000).
                        // Therefore, we have to create the region by multiple steps.

                        if( pData->rdh.nCount == 2000 )
                        {
                           HRGN  h = ExtCreateRegion( NULL, sizeof(RGNDATAHEADER) + (sizeof(RECT) * maxRects), pData );
                           if( hRgn )
                           {
                              CombineRgn( hRgn, hRgn, h, RGN_OR );
                              DeleteObject( h );
                           }
                           else
                           {
                              hRgn = h;
                           }

                           pData->rdh.nCount = 0;
                           SetRect( &pData->rdh.rcBound, MAXLONG, MAXLONG, 0, 0 );
                        }
                     }
                  }

                  // Go to next row (remember, the bitmap is inverted vertically)

                  p32 -= bm32.bmWidthBytes;
               }

               // Create or extend the region with the remaining  rectangles

               h = ExtCreateRegion( NULL, sizeof(RGNDATAHEADER) + (sizeof(RECT) * maxRects), pData );
               if( hRgn )
               {
                  CombineRgn( hRgn, hRgn, h, RGN_OR );
                  DeleteObject( h );
               }
               else
               {
                  hRgn = h;
               }

               // Clean up

               GlobalFree( hData );
               SelectObject( hDC, holdBmp );
               DeleteDC( hDC );
            }

            DeleteObject( SelectObject(hMemDC, holdBmp) );
         }

         DeleteDC( hMemDC );
      }
   }

   return hRgn;
}
