/*----------------------------------------------------------------------------
 MINIGUI - Harbour Win32 GUI library source code

 Copyright 2002-2008 Roberto Lopez <harbourminigui@gmail.com>
 http://harbourminigui.googlepages.com/

 This program is free software; you can redistribute it and/or modify it under 
 the terms of the GNU General Public License as published by the Free Software 
 Foundation; either version 2 of the License, or (at your option) any later 
 version. 

 This program is distributed in the hope that it will be useful, but WITHOUT 
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS 
 FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

 You should have received a copy of the GNU General Public License along with 
 this software; see the file COPYING. If not, write to the Free Software 
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA (or 
 visit the web site http://www.gnu.org/).

 As a special exception, you have permission for additional uses of the text 
 contained in this release of Harbour Minigui.

 The exception is that, if you link the Harbour Minigui library with other 
 files to produce an executable, this does not by itself cause the resulting 
 executable to be covered by the GNU General Public License.
 Your use of that executable is in no way restricted on account of linking the 
 Harbour-Minigui library code into it.

 Parts of this project are based upon:

	"Harbour GUI framework for Win32"
 	Copyright 2001 Alexander S.Kresin <alex@belacy.belgorod.su>
 	Copyright 2001 Antonio Linares <alinares@fivetech.com>
	www - http://www.harbour-project.org

	"Harbour Project"
	Copyright 1999-2008, http://www.harbour-project.org/

	"WHAT32"
	Copyright 2002 AJ Wos <andrwos@aust1.net> 

	"HWGUI"
  	Copyright 2001-2007 Alexander S.Kresin <alex@belacy.belgorod.su>

   Parts of this code are contributed for MiniGUI Project and
   used here under permission of author :

   Copyright 2005 (C) Andy Wos <andywos@unwired.com.au>
        + SetProp()
        + GetProp()
        + RemoveProp()
---------------------------------------------------------------------------*/

#define HB_OS_WIN_32_USED

#include <windows.h>
#include <winuser.h>
#include "item.api"
#include "hbapi.h"

//------------------------------------------------------------------------------
//                   General, universal GetProp/SetProp functions
//------------------------------------------------------------------------------
// usage: SetProp( hWnd, cPropName, xValue, [lHandle] ) -> lSuccess
// [lHandle] is optional and indicates that no memory management is required
//           if lHandle = .T., xValue must be numerical (integer)

HB_FUNC( SETPROP )
{
   HGLOBAL  hMem;
   char     *lpMem;
   int      nLen;
   char     chType;
   BOOL     bValue;
   double   dValue;
   INT      iValue;

   // check params

   if( !IsWindow((HWND) hb_parnl(1)) )
   {
      hb_retl( FALSE );
      return;
   }

   if( !ISCHAR(2) )
   {
      hb_retl( FALSE );
      return;
   }

   // check data

   if( ISCHAR(3) )
   {
      chType = 'C';
      nLen = hb_parclen( 3 );
   }
   else if( ISLOG(3) )
   {
      chType = 'L';
      nLen = sizeof( BOOL );
   }
   else if( ISDATE(3) )
   {
      chType = 'D';
      nLen = 9;         // len of "yyyymmdd"
   }
   else if( HB_IS_NUMINT(hb_param(3, HB_IT_ANY)) )
   {
      if( hb_pcount() > 3 && ISLOG(4) && hb_parl(4) )
      {
         chType = 'X';  // memory handle passsed
      }
      else
      {
         chType = 'I';
      }

      nLen = sizeof( INT );
   }
   else if( ISNUM(3) )
   {
      chType = 'F';     //float
      nLen = sizeof( double );
   }
   else                 // unsupported type
   {
      hb_retl( FALSE );
      return;
   }

   // direct assignment of a long value

   if( chType == 'X' )
   {
      hb_retl( SetProp((HWND) hb_parnl(1), hb_parc(2), (HGLOBAL) hb_parnl(3)) );
      return;
   }

   // type conversion

   if( (hMem = GlobalAlloc(GPTR, nLen + sizeof(int) + 1)) == NULL )
   {
      hb_retl( FALSE );
      return;
   }
   else
   {
      lpMem = GlobalLock( hMem );
      if( lpMem == NULL )
      {
         GlobalFree( hMem );
         hb_retl( FALSE );
         return;
      }
   }

   lpMem[0] = chType;
   memcpy( lpMem + 1, ( char * ) &nLen, sizeof(int) );

   switch( chType )
   {
      case 'C':   memcpy( lpMem + sizeof(int) + 1, hb_parc(3), nLen ); break;
      case 'L':   bValue = hb_parl( 3 ); memcpy( lpMem + sizeof(int) + 1, ( char * ) &bValue, sizeof(BOOL) ); break;
      case 'D':   memcpy( lpMem + sizeof(int) + 1, hb_pards(3), nLen ); break;
      case 'I':   iValue = ( INT ) hb_parnl( 3 ); memcpy( lpMem + sizeof(int) + 1, ( char * ) &iValue, sizeof(INT) ); break;
      case 'F':   dValue = hb_parnd( 3 ); memcpy( lpMem + sizeof(int) + 1, ( char * ) &dValue, sizeof(double) ); break;
   }

   GlobalUnlock( hMem );
   hb_retl( SetProp((HWND) hb_parnl(1), hb_parc(2), hMem) );
}

// usage: GetProp( hWnd, cPropName, [lHandle] ) -> Value
// [lHandle] : .T. =  return the value directly

HB_FUNC( GETPROP )
{
   HGLOBAL  hMem;
   char     *lpMem;
   int      nLen;

   // check params

   if( !IsWindow((HWND) hb_parnl(1)) )
   {
      hb_ret();
      return;
   }

   if( !ISCHAR(2) )
   {
      hb_ret();
      return;
   }

   // read

   hMem = ( HGLOBAL ) GetProp( (HWND) hb_parnl(1), hb_parc(2) );

   if( hb_pcount() > 2 && ISLOG(3) && hb_parl(3) )
   {
      hb_retnl( (INT) hMem );
      return;
   }

   if( hMem != NULL )
   {
      lpMem = GlobalLock( hMem );
      if( lpMem == NULL )
      {
         hb_ret();      // return NIL
         return;
      }
   }
   else
   {
      hb_ret();
      return;
   }

   nLen = ( int ) * ( int * ) ( lpMem + 1 );

   switch( lpMem[0] )
   {
      case 'C':   hb_retclen( lpMem + sizeof(int) + 1, nLen ); break;
      case 'L':   hb_retl( (BOOL) * (BOOL *) (lpMem + sizeof(int) + 1) ); break;
      case 'D':   hb_retds( lpMem + sizeof(int) + 1 ); break;
      case 'I':   hb_retni( (INT) * (INT *) (lpMem + sizeof(int) + 1) ); break;
      case 'F':   hb_retnd( ( double ) * ( double * ) (lpMem + sizeof(int) + 1) ); break;
      default:    hb_ret();
   }

   GlobalUnlock( hMem );
}

// usage: RemoveProp( hWnd, cPropName, [lNoFree] ) -> hMem

HB_FUNC( REMOVEPROP )
{
   HGLOBAL  hMem;

   // check params

   if( !IsWindow((HWND) hb_parnl(1)) )
   {
      hb_ret();
      return;
   }

   if( !ISCHAR(2) )
   {
      hb_ret();
      return;
   }

   hMem = RemoveProp( (HWND) hb_parnl(1), hb_parc(2) );
   if( hMem != NULL && (hb_pcount() < 3 || !ISLOG(3) || !hb_parl(3)) )
   {
      GlobalFree( hMem );
   }

   hb_retnl( (ULONG) hMem );
}
