/*----------------------------------------------------------------------------
MINIGUI - Harbour Win32 GUI library source code

Copyright 2002-2008 Roberto Lopez <harbourminigui@gmail.com>
http://harbourminigui.googlepages.com/

BTNTEXTBOX control source code
(C)2006 Janusz Pora <januszpora@onet.eu>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along with
   this software; see the file COPYING. If not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA (or
   visit the web site http://www.gnu.org/).

   As a special exception, you have permission for additional uses of the text
   contained in this release of Harbour Minigui.

   The exception is that, if you link the Harbour Minigui library with other
   files to produce an executable, this does not by itself cause the resulting
   executable to be covered by the GNU General Public License.
   Your use of that executable is in no way restricted on account of linking the
   Harbour-Minigui library code into it.

   Parts of this project are based upon:

   "Harbour GUI framework for Win32"
   Copyright 2001 Alexander S.Kresin <alex@belacy.belgorod.su>
   Copyright 2001 Antonio Linares <alinares@fivetech.com>
   www - http://www.harbour-project.org

   "Harbour Project"
   Copyright 1999-2008, http://www.harbour-project.org/

   "WHAT32"
   Copyright 2002 AJ Wos <andrwos@aust1.net>

   "HWGUI"
   Copyright 2001-2007 Alexander S.Kresin <alex@belacy.belgorod.su>

---------------------------------------------------------------------------*/

#include "minigui.ch"

#define EM_LIMITTEXT        197
#define ES_WANTRETURN       4096
#define ES_MULTILINE        0x0004
#define ES_AUTOVSCROLL      0x0040
#define ES_AUTOHSCROLL      128
#define WS_CHILD      0x40000000
#define WS_BORDER     0x800000
#define WS_TABSTOP    0x10000
#define WS_VISIBLE    0x10000000
#define WS_HSCROLL    0x100000
#define WS_VSCROLL    0x200000
#define WS_THICKFRAME 0x40000
#define BS_FLAT             0x00008000
#define ES_NUMBER           8192
#define ES_RIGHT            2
#define ES_UPPERCASE        8
#define ES_LOWERCASE        16
#define ES_PASSWORD         32
#define ES_READONLY         2048
#define EM_GETSEL           176
#define EM_SETSEL           177
#define WM_SETTEXT          12
#define WM_CLEAR            771
#define WM_CHAR             258
#define WM_COMMAND          273  // 0x0111

*-----------------------------------------------------------------------------*
Function _DefineBtnTextBox ( ControlName, ParentFormName, x, y, w, h,;
   cValue, ProcedureName, ProcedureName2, abitmap, BtnWidth, FontName, FontSize,;
   ToolTip, nMaxLenght, lUpper, lLower, lNumeric, lPassword,;
   uLostFocus, uGotFocus, uChange, uEnter, right, HelpId,;
   bold, italic, underline, strikeout, field, backcolor, fontcolor ,;
   invisible, notabstop, nId )
*-----------------------------------------------------------------------------*
   Local ParentFormHandle, ControlHandle := 0
   Local mVar, k, Style
   Local FontHandle, aFont, cBmp, lBtn2 :=.f.
   Local WorkArea, blInit
   Local lDialogInMemory

   // Asign STANDARD values to optional params.
   DEFAULT w          TO 120
   DEFAULT h          TO 24
   DEFAULT cValue     TO ""
   DEFAULT uChange    TO ""
   DEFAULT uGotFocus  TO ""
   DEFAULT uLostFocus TO ""
   DEFAULT nMaxLenght TO 255
   DEFAULT lUpper     TO .f.
   DEFAULT lLower     TO .f.
   DEFAULT lNumeric   TO .f.
   DEFAULT lPassword  TO .f.
   DEFAULT uEnter     TO ""

   If ValType ( aBitmap ) != 'A'
      cBmp := aBitmap
      aBitmap := Array(2)
      aBitmap[1] := cBmp
   EndIf
   If ProcedureName2 != Nil
      lBtn2 := .T.
   EndIf
   If ValType ( Field ) != 'U'
      If  at ( '>', Field ) == 0
         MsgMiniGuiError ("Control " + ControlName + " Of " + ParentFormName + " : You must specify a fully qualified field name. Program Terminated")
      Else
         WorkArea := Left ( Field , at ( '>', Field ) - 2 )
         If Select (WorkArea) != 0
            cValue := &(Field)
         EndIf
      EndIf
   EndIf

   if ( FontHandle := GetFontHandle( FontName ) ) != 0
      aFont := GetFontParam(FontHandle)
      FontName      := aFont[1]
      FontSize      := aFont[2]
      bold          := aFont[3]
      italic        := aFont[4]
      underline     := aFont[5]
      strikeout     := aFont[6]
   endif

   if _HMG_BeginWindowActive = TRUE .or. _HMG_BeginDialogActive = TRUE
      ParentFormName := if(_HMG_BeginDialogActive,_HMG_ActiveDialogName,_HMG_ActiveFormName)
      If .Not. Empty (_HMG_ActiveFontName) .And. ValType(FontName) == "U"
         FontName := _HMG_ActiveFontName
      EndIf
      If .Not. Empty (_HMG_ActiveFontSize) .And. ValType(FontSize) == "U"
         FontSize := _HMG_ActiveFontSize
      EndIf
   endif

   if _HMG_FrameLevel > 0
      x    := x + _HMG_ActiveFrameCol [_HMG_FrameLevel]
      y    := y + _HMG_ActiveFrameRow [_HMG_FrameLevel]
      ParentFormName := _HMG_ActiveFrameParentFormName [_HMG_FrameLevel]
   EndIf
   lDialogInMemory := _HMG_DialogInMemory // ( GetWindowType ( ParentFormName ) == "L" )

   // Check if the window/form is defined.
   if .not. _IsWindowDefined( ParentFormName ) .and. .not. lDialogInMemory
      MsgMiniGuiError( "Window " + ParentFormName + " is not defined. Program terminated." )
   endif

   // Check if the control is already defined.
   if _IsControlDefined( ControlName, ParentFormName ) .and. .not. lDialogInMemory
      MsgMiniGuiError( "Control " + ControlName + " of " + ParentFormName + " already defined. Program Terminated." )
   endif

   mVar := '_' + ParentFormName + '_' + ControlName
   k := _GetControlFree()

   if _HMG_BeginDialogActive = TRUE
      ParentFormHandle = _HMG_ActiveDialogHandle

      Style = WS_CHILD + ES_AUTOHSCROLL + BS_FLAT + WS_BORDER

      if lNumeric
         Style += ES_NUMBER
      else
         if lUpper
            Style += ES_UPPERCASE
         endif
         if lLower
            Style += ES_LOWERCASE
         endif
      endif

      if lPassword
         Style += ES_PASSWORD
      endif

      if right
         Style +=  ES_RIGHT
      endif
/*
      if readonly
         Style += ES_READONLY
      endif
*/
      if !invisible
         Style += WS_VISIBLE
      endif

      if  !notabstop
         Style +=  WS_TABSTOP
      endif

      if len(_HMG_aDialogTemplate) > 0        //Dialog Template

//          {{'ID',k/hwnd,class,Style,ExStyle,x,y,w,h,caption,HelpId,tooltip,font,size, bold, italic, underline, strikeout}}  --->_HMG_aDialogItems
         blInit:= {|x,y,z| InitDialogTextBox(x,y,z) }
         aadd(_HMG_aDialogItems,{nId,k,"EDIT",style,0,x,y,w,h,cValue,HelpId,Tooltip,FontName,FontSize,bold,italic,underline,strikeout,blInit,_HMG_BeginTabActive,.f.,_HMG_ActiveTabPage })
/* JP 62
         if _HMG_aDialogTemplate[3]   // Modal
            _HMG_aControlDeleted [k] := .T.
            Return Nil
         endif
*/
      else

         ControlHandle := GetDialogItemHandle( ParentFormHandle, nId )

         x := GetWindowCol ( Controlhandle )
         y := GetWindowRow ( Controlhandle )
         w := GetWindowWidth  ( Controlhandle )
         h := GetWindowHeight ( Controlhandle )

         SetWindowStyle ( ControlHandle, Style )
         ReDefBtnTextBox( ControlHandle, abitmap[1], BtnWidth, abitmap[2], lBtn2 )

      endif

   else

      ParentFormHandle  := GetFormHandle( ParentFormName )
      // Creates the control window
      ControlHandle := InitBtnTextBox( ParentFormHandle, 0, x, y, w, h, '', 0, nMaxLenght,;
         lUpper, lLower, .f., lPassword, right, invisible, notabstop, abitmap[1], BtnWidth, abitmap[2], lBtn2 )

   endif

   if .Not. lDialogInMemory

      If FontHandle != 0
         _SetFontHandle(ControlHandle,FontHandle)
      Else

         if valtype(fontname) == "U"
            Fontname := _HMG_DefaultFontName
         endif
         if valtype(fontsize) == "U"
            FontSize := _HMG_DefaultFontSize
         endif
         FontHandle := _SetFont (ControlHandle,FontName,FontSize,bold,italic,underline,strikeout)
      EndIf

      If _HMG_BeginTabActive = .T.
         aAdd ( _HMG_ActiveTabCurrentPageMap , ControlHandle )
      EndIf

       // Add a tooltip if param has value
      if ValType( ToolTip ) != "U"
         SetToolTip( ControlHandle, ToolTip, GetFormToolTipHandle( ParentFormName ) )
      endif
   endif

   Public &mVar. := k

   _HMG_aControlType [k] := if( lNumeric, "BTNNUMTEXT", "BTNTEXT" )
   _HMG_aControlNames  [k] :=  ControlName
   _HMG_aControlHandles  [k] :=  ControlHandle
   _HMG_aControlParenthandles  [k] :=  ParentFormHandle
   _HMG_aControlIds  [k] :=  nId
   _HMG_aControlProcedures  [k] :=  ProcedureName
   _HMG_aControlPageMap  [k] :=  Field
   _HMG_aControlValue  [k] :=  cValue
   _HMG_aControlInputMask  [k] :=  ""
   _HMG_aControlLostFocusProcedure [k] :=   uLostFocus
   _HMG_aControlGotFocusProcedure  [k] := uGotFocus
   _HMG_aControlChangeProcedure  [k] :=  uChange
   _HMG_aControlDeleted  [k] :=  .F.
   _HMG_aControlBkColor  [k] :=  backcolor
   _HMG_aControlFontColor [k] :=   fontcolor
   _HMG_aControlDblClick  [k] :=  uEnter
   _HMG_aControlHeadClick  [k] :=  ProcedureName2
   _HMG_aControlRow  [k] :=  y
   _HMG_aControlCol  [k] :=  x
   _HMG_aControlWidth   [k] := w
   _HMG_aControlHeight   [k] := h
   _HMG_aControlSpacing  [k] :=  0
   _HMG_aControlContainerRow  [k] :=  iif ( _HMG_FrameLevel > 0 ,_HMG_ActiveFrameRow [_HMG_FrameLevel] , -1 )
   _HMG_aControlContainerCol  [k] :=  iif ( _HMG_FrameLevel > 0 ,_HMG_ActiveFrameCol [_HMG_FrameLevel] , -1 )
   _HMG_aControlPicture  [k] :=  abitmap
   _HMG_aControlContainerHandle  [k] :=  0
   _HMG_aControlFontName  [k] :=  FontName
   _HMG_aControlFontSize  [k] :=  FontSize
   _HMG_aControlFontAttributes  [k] :=  {bold,italic,underline,strikeout}
   _HMG_aControlToolTip  [k] :=   ToolTip
   _HMG_aControlRangeMin  [k] :=   BtnWidth
   _HMG_aControlRangeMax  [k] :=   nMaxLenght
   _HMG_aControlCaption  [k] :=   ''
   _HMG_aControlVisible  [k] :=  .Not.  invisible
   _HMG_aControlHelpId  [k] :=   HelpId
   _HMG_aControlFontHandle  [k] :=   FontHandle
   _HMG_aControlBrushHandle  [k] :=   0
   _HMG_aControlEnabled  [k] :=   .T.
   _HMG_aControlMiscData1 [k] := {0,lBtn2}

   if .Not. lDialogInMemory
      // With NUMERIC clause, transform numeric value into a string.
      if lNumeric == .T.
         If Valtype( cValue ) != 'C'
            cValue := AllTrim( Str( cValue ) )
         EndIf
      EndIf

      // Fill the TEXTBOX with the text given.
      if Len( cValue ) > 0
         SetWindowText( ControlHandle , cValue )
      endif

      If Valtype( Field ) != 'U'
         aAdd( _HMG_aFormBrowseList [ GetFormIndex ( ParentFormName ) ] , k )
      EndIf
   endif

return nil

*-----------------------------------------------------------------------------*
Function InitDialogBtnTextBox( ParentName, ControlHandle, k )
*-----------------------------------------------------------------------------*
   LOCAL nMaxLenght, Field, cValue, lNumeric, abitmap, BtnWidth, lBtn2
   Field       := _HMG_aControlPageMap  [k]
   nMaxLenght  := _HMG_aControlRangeMax  [k]
   cValue      := _HMG_aControlValue  [k]
   lNumeric    := ( _HMG_aControlType [k] == "BTNNUMTEXT" )
   abitmap     := _HMG_aControlPicture  [k]
   BtnWidth    := _HMG_aControlRangeMin  [k]
   lBtn2       := _HMG_aControlMiscData1  [k,2]

   if valtype ( nMaxLenght ) != 'U'
      SendMessage( ControlHandle , EM_LIMITTEXT , nMaxLenght , 0 )
   endif

    // With NUMERIC clause, transform numeric value into a string.
   if lNumeric == .T.
      If Valtype( cValue ) != 'C'
         cValue := AllTrim( Str( cValue ) )
      EndIf
   EndIf

    // Fill the TEXTBOX with the text given.
   if Len( cValue ) > 0
      SetWindowText ( ControlHandle , cValue )
   endif

   if valtype( Field ) != 'U'
      aAdd( _HMG_aFormBrowseList [ GetFormIndex ( ParentName ) ] , k )
   EndIf

   ReDefBtnTextBox( ControlHandle, abitmap[1] , BtnWidth, abitmap[2] , lBtn2 )
// JP 62
   if len(_HMG_aDialogTemplate) != 0 .and. _HMG_aDialogTemplate[3]   // Modal
      _HMG_aControlDeleted [k] := .T.
   endif

Return Nil