/*----------------------------------------------------------------------------
MINIGUI - Harbour Win32 GUI library source code

Copyright 2002-2008 Roberto Lopez <harbourminigui@gmail.com>
http://harbourminigui.googlepages.com/

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along with
   this software; see the file COPYING. If not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA (or
   visit the web site http://www.gnu.org/).

   As a special exception, you have permission for additional uses of the text
   contained in this release of Harbour Minigui.

   The exception is that, if you link the Harbour Minigui library with other
   files to produce an executable, this does not by itself cause the resulting
   executable to be covered by the GNU General Public License.
   Your use of that executable is in no way restricted on account of linking the
   Harbour-Minigui library code into it.

   Parts of this project are based upon:

   "Harbour GUI framework for Win32"
   Copyright 2001 Alexander S.Kresin <alex@belacy.belgorod.su>
   Copyright 2001 Antonio Linares <alinares@fivetech.com>
   www - http://www.harbour-project.org

   "Harbour Project"
   Copyright 1999-2008, http://www.harbour-project.org/

   "WHAT32"
   Copyright 2002 AJ Wos <andrwos@aust1.net>

   "HWGUI"
   Copyright 2001-2007 Alexander S.Kresin <alex@belacy.belgorod.su>

---------------------------------------------------------------------------*/

#include "minigui.ch"

#define BP_PUSHBUTTON 1

#define PBS_NORMAL 1
#define PBS_HOT 2
#define PBS_PRESSED 3
#define PBS_DISABLED 4
#define PBS_DEFAULTED 5

#define ODT_BUTTON 4
#define ODS_SELECTED 1
#define ODS_GRAYED 2
#define ODS_DISABLED 4
#define ODS_CHECKED 8
#define ODS_FOCUS 16
#define ODS_DEFAULT 32
#define ODS_COMBOBOXEDIT 4096
#define ODS_HOTLIGHT 64
#define ODS_INACTIVE 128
#define DFCS_BUTTONPUSH 16
#define DFCS_INACTIVE 256

#define COLOR_HIGHLIGHTTEXT 14
#define COLOR_BTNFACE 15
#define COLOR_BTNSHADOW 16
#define COLOR_GRAYTEXT 17
#define COLOR_BTNTEXT 18
#define COLOR_INACTIVECAPTIONTEXT 19
#define COLOR_BTNHIGHLIGHT 20
#define COLOR_3DDKSHADOW 21
#define COLOR_3DLIGHT 22
#define COLOR_INFOTEXT 23
#define COLOR_INFOBK 24
#define COLOR_HOTLIGHT 26
#define COLOR_GRADIENTACTIVECAPTION 27
#define COLOR_GRADIENTINACTIVECAPTION 28
#define COLOR_DESKTOP COLOR_BACKGROUND
#define COLOR_3DFACE COLOR_BTNFACE
#define COLOR_3DSHADOW COLOR_BTNSHADOW
#define COLOR_3DHIGHLIGHT COLOR_BTNHIGHLIGHT
#define COLOR_3DHILIGHT COLOR_BTNHIGHLIGHT
#define COLOR_BTNHILIGHT COLOR_BTNHIGHLIGHT

#define DT_TOP 0
#define DT_LEFT 0
#define DT_CENTER 1
#define DT_RIGHT 2
#define DT_VCENTER 4
#define DT_BOTTOM 8
#define DT_SINGLELINE 32

#define DFCS_PUSHED 512
#define DFCS_CHECKED 1024
#define DFCS_TRANSPARENT 2048
#define DFCS_HOT 4096
#define DFCS_ADJUSTRECT 8192
#define DFCS_FLAT 16384
#define DFCS_MONO 32768
#define TRANSPARENT   1

#define DST_COMPLEX          0
#define DST_TEXT             1
#define DST_PREFIXTEXT       2
#define DST_ICON             3
#define DST_BITMAP           4
// State type
#define DSS_NORMAL           0
#define DSS_UNION           16  // Gray string appearance
#define DSS_DISABLED        32
#define DSS_MONO           128
#define DSS_HIDEPREFIX     512
#define DSS_PREFIXONLY    1024
#define DSS_RIGHT        32768
/*
 * Owner draw actions
 */
#define ODA_DRAWENTIRE    1
#define ODA_SELECT        2
#define ODA_FOCUS         4
#define WM_COMMAND      0x0111
#define WM_SETFOCUS       7
#define WM_DRAWITEM      43
#define WM_LBUTTONDOWN  513
#define WM_MOUSELEAVE   675
#define WM_MOUSEMOVE    512


/* Ascpects for owner butons */

#define OBT_HORIZONTAL    0
#define OBT_VERTICAL      1
#define OBT_LEFTTEXT      2
#define OBT_UPTEXT        4
#define OBT_HOTLIGHT      8
#define OBT_FLAT          16
#define OBT_NOTRANSPARENT 32
#define OBT_NOXPSTYLE     64
#define OBT_ADJUST       128

#define BS_NOTIFY           0x00004000
#define BS_PUSHBUTTON       0x00000000
#define BS_FLAT             0x00008000
#define BS_BITMAP           0x00000080
#define WS_TABSTOP          0x00010000
#define WS_VISIBLE          0x10000000
#define WS_CHILD            0x40000000

STATIC lXPThemeActive   := .f.
STATIC lDialogInMemory  := .f.
*-----------------------------------------------------------------------------*
Function _DefineButton ( ControlName, ParentFormName, x, y, Caption,;
   ProcedureName, w, h, fontname, fontsize, tooltip,;
   gotfocus, lostfocus, flat, NoTabStop, HelpId,;
   invisible , bold, italic, underline, strikeout ,;
   multiline, default, nId )
*-----------------------------------------------------------------------------*
   Local ParentFormHandle , mVar , ControlHandle , blInit , FontHandle , aFont, k , Style

   DEFAULT w         TO 100
   DEFAULT h         TO 28
   DEFAULT lostfocus TO ""
   DEFAULT gotfocus  TO ""
   DEFAULT invisible TO FALSE
   DEFAULT flat      TO FALSE
   DEFAULT NoTabStop TO FALSE
   DEFAULT HelpId    TO 0

   if ( FontHandle := GetFontHandle( FontName ) ) != 0
      aFont := GetFontParam(FontHandle)
      FontName      := aFont[1]
      FontSize      := aFont[2]
      bold          := aFont[3]
      italic        := aFont[4]
      underline     := aFont[5]
      strikeout     := aFont[6]
   endif

   if _HMG_BeginWindowActive = TRUE .or. _HMG_BeginDialogActive = TRUE
      ParentFormName := if(_HMG_BeginDialogActive,_HMG_ActiveDialogName,_HMG_ActiveFormName)
      if .Not. Empty (_HMG_ActiveFontName) .And. ValType(FontName) == "U"
         FontName := _HMG_ActiveFontName
      EndIf
      if .Not. Empty (_HMG_ActiveFontSize) .And. ValType(FontSize) == "U"
         FontSize := _HMG_ActiveFontSize
      EndIf
   endif

   if _HMG_FrameLevel > 0
      x    := x + _HMG_ActiveFrameCol [_HMG_FrameLevel]
      y    := y + _HMG_ActiveFrameRow [_HMG_FrameLevel]
      ParentFormName := _HMG_ActiveFrameParentFormName [_HMG_FrameLevel]
   EndIf
   lDialogInMemory := _HMG_DialogInMemory // ( GetWindowType ( ParentFormName ) == "L" )

   If .Not. _IsWindowDefined (ParentFormName) .and. .Not. lDialogInMemory
      MsgMiniGuiError("Window: "+ ParentFormName + " is not defined. Program terminated" )
   Endif

   If _IsControlDefined (ControlName,ParentFormName) .and. .Not. lDialogInMemory
      MsgMiniGuiError ("Control: " + ControlName + " Of " + ParentFormName + " Already defined. Program Terminated" )
   Endif

   mVar := '_' + ParentFormName + '_' + ControlName
   k := _GetControlFree()

   if _HMG_BeginDialogActive = TRUE
      ParentFormHandle = _HMG_ActiveDialogHandle

      Style =  BS_NOTIFY + WS_CHILD + BS_PUSHBUTTON
      if flat
         style += BS_FLAT
      endif
      if !NoTabStop
         style += WS_TABSTOP
      endif
      if !invisible
         style += WS_VISIBLE
      endif

      if lDialogInMemory         //Dialog Template

//           {{'ID',k/hwnd,class,Style,ExStyle,x,y,w,h,caption,HelpId,tooltip,font,size, bold, italic, underline, strikeout}}  --->_HMG_aDialogItems
         blInit:= {|x,y,z| InitDialogButtonImage(x,y,z) }
         aadd(_HMG_aDialogItems,{nId,k,"button",style,0,x,y,w,h,caption,HelpId,tooltip,FontName,FontSize,bold,italic,underline,strikeout,blInit,_HMG_BeginTabActive,.f.,_HMG_ActiveTabPage })

      else
         ControlHandle := GetDialogItemHandle( ParentFormHandle, nId )

         x := GetWindowCol ( Controlhandle )
         y := GetWindowRow ( Controlhandle )
         w := GetWindowWidth  ( Controlhandle )
         h := GetWindowHeight ( Controlhandle )

         If FontHandle != 0
            _SetFontHandle(ControlHandle,FontHandle)
         Else
            if valtype(fontname) == "U"
               Fontname := _HMG_DefaultFontName
            endif
            if valtype(fontsize) == "U"
               FontSize := _HMG_DefaultFontSize
            endif
            FontHandle := _SetFont (ControlHandle,FontName,FontSize,bold,italic,underline,strikeout)
         Endif
         if valtype(caption) != "U"
            SetWindowText ( ControlHandle , caption )
         endif
         SetWindowStyle ( ControlHandle, style )

      endif
   else

      ParentFormHandle = GetFormHandle (ParentFormName)

      ControlHandle := InitButton ( ParentFormHandle, Caption, 0, x, y ,w ,h,'',0 , flat , NoTabStop, invisible , multiline , default )

   endif

   if .Not. lDialogInMemory
      If FontHandle != 0
         _SetFontHandle(ControlHandle,FontHandle)
      Else
         if valtype(fontname) == "U"
            Fontname := _HMG_DefaultFontName
         endif
         if valtype(fontsize) == "U"
            FontSize := _HMG_DefaultFontSize
         endif
         FontHandle := _SetFont (ControlHandle,FontName,FontSize,bold,italic,underline,strikeout)
      Endif

      If _HMG_BeginTabActive = TRUE
         aAdd ( _HMG_ActiveTabCurrentPageMap , Controlhandle )
      EndIf

      if valtype(tooltip) != "U"
         SetToolTip ( ControlHandle , tooltip , GetFormToolTipHandle (ParentFormName) )
      endif

   endif

   Public &mVar. := k

   _HMG_aControlType [k] := "BUTTON"
   _HMG_aControlNames [k] := ControlName
   _HMG_aControlHandles [k] := ControlHandle
   _HMG_aControlParenthandles [k] := ParentFormHandle
   _HMG_aControlIds [k] := nId
   _HMG_aControlProcedures [k] := ProcedureName
   _HMG_aControlPageMap [k] := {}
   _HMG_aControlValue [k] := Nil
   _HMG_aControlInputMask [k] := ""
   _HMG_aControllostFocusProcedure [k] := lostfocus
   _HMG_aControlGotFocusProcedure [k] := gotfocus
   _HMG_aControlChangeProcedure [k] := ""
   _HMG_aControlDeleted [k] := FALSE
   _HMG_aControlBkColor [k] := Nil
   _HMG_aControlFontColor [k] := Nil
   _HMG_aControlDblClick [k] := ""
   _HMG_aControlHeadClick [k] := {}
   _HMG_aControlRow [k] := y
   _HMG_aControlCol [k] := x
   _HMG_aControlWidth [k] := w
   _HMG_aControlHeight [k] := h
   _HMG_aControlSpacing [k] := 0
   _HMG_aControlContainerRow  [k] :=  iif ( _HMG_FrameLevel > 0 ,_HMG_ActiveFrameRow [_HMG_FrameLevel] , -1 )
   _HMG_aControlContainerCol  [k] :=  iif ( _HMG_FrameLevel > 0 ,_HMG_ActiveFrameCol [_HMG_FrameLevel] , -1 )
   _HMG_aControlPicture [k] := ""
   _HMG_aControlContainerHandle [k] := 0
   _HMG_aControlFontName [k] := fontname
   _HMG_aControlFontSize [k] := fontsize
   _HMG_aControlFontAttributes [k] := {bold,italic,underline,strikeout}
   _HMG_aControlToolTip [k] := tooltip
   _HMG_aControlRangeMin [k] := 0
   _HMG_aControlRangeMax [k] := 0
   _HMG_aControlCaption [k] := Caption
   _HMG_aControlVisible [k] := if(invisible,FALSE,TRUE)
   _HMG_aControlHelpId [k] := HelpId
   _HMG_aControlFontHandle [k] := FontHandle
   _HMG_aControlBrushHandle [k] := 0
   _HMG_aControlEnabled [k] := .T.
   _HMG_aControlMiscData1 [k] := 0

Return Nil

*-----------------------------------------------------------------------------*
Function _DefineImageButton ( ControlName, ParentFormName, x, y, Caption,;
   ProcedureName, w, h, image, tooltip, gotfocus,;
   lostfocus, flat, notrans, HelpId, invisible,;
   notabstop , default,icon , nId)
*-----------------------------------------------------------------------------*
   Local ParentFormHandle , mVar , ControlHandle, blInit , k , Style
   Local nhImage , nIcon , cPicture
   Local aRet /*[2]*/

   DEFAULT invisible TO FALSE
   DEFAULT notabstop TO FALSE
   DEFAULT flat      TO FALSE
   DEFAULT HelpId    TO 0

   If _HMG_ToolBarActive == TRUE
      Return Nil
   EndIf

   if _HMG_BeginWindowActive = TRUE .or. _HMG_BeginDialogActive = TRUE
      ParentFormName := if(_HMG_BeginDialogActive,_HMG_ActiveDialogName,_HMG_ActiveFormName)
   endif

   if _HMG_FrameLevel > 0
      x    := x + _HMG_ActiveFrameCol [_HMG_FrameLevel]
      y    := y + _HMG_ActiveFrameRow [_HMG_FrameLevel]
      ParentFormName := _HMG_ActiveFrameParentFormName [_HMG_FrameLevel]
   EndIf
   lDialogInMemory := _HMG_DialogInMemory

   If .Not. _IsWindowDefined (ParentFormName) .and. .Not. lDialogInMemory
      MsgMiniGuiError("Window: "+ ParentFormName + " is not defined. Program terminated" )
   Endif

   If _IsControlDefined (ControlName,ParentFormName) .and. .Not. lDialogInMemory
      MsgMiniGuiError ("Control: " + ControlName + " Of " + ParentFormName + " Already defined. Program terminated" )
   endif

   if !EMPTY(image) .and. !EMPTY(icon)
      MsgMiniGuiError ("Control: " + ControlName + " Of " + ParentFormName + ". Either bitmap or icon must be specified. Program terminated" )
   endif

   mVar := '_' + ParentFormName + '_' + ControlName
   k := _GetControlFree()

   nIcon:=IIF(!EMPTY(icon),1,0)
   cPicture:= IIF(!EMPTY(icon),icon,image)
   if VALTYPE(cPicture)=="A"
      image:= cPicture[1]
   endif

   if _HMG_BeginDialogActive = TRUE

      ParentFormHandle = _HMG_ActiveDialogHandle

      Style := BS_NOTIFY + BS_BITMAP + WS_CHILD + BS_PUSHBUTTON
      if flat
         style += BS_FLAT
      endif
      if !NoTabStop
         style += WS_TABSTOP
      endif
      if !invisible
         style += WS_VISIBLE
      endif


      if lDialogInMemory         //Dialog Template

//          {{'ID',k/hwnd,class,Style,ExStyle,x,y,w,h,caption,HelpId,ToolTip,FontName,FontSize,bold,italic,,}}  --->_HMG_aDialogItems
         blInit:= {|x,y,z| InitDialogButtonImage(x,y,z) }
         aadd(_HMG_aDialogItems,{nId,k,"button",style,0,x,y,w,h,caption,HelpId,tooltip,,,,,,,blInit,_HMG_BeginTabActive,.f.,_HMG_ActiveTabPage})

      else

         ControlHandle := GetDialogItemHandle( ParentFormHandle, nId )

         x := GetWindowCol ( Controlhandle )
         y := GetWindowRow ( Controlhandle )
         w := GetWindowWidth  ( Controlhandle )
         h := GetWindowHeight ( Controlhandle )

         SetWindowStyle ( ControlHandle, style )

         _SetBtnPicture(ControlHandle,image)
      endif

   else

      ParentFormHandle = GetFormHandle (ParentFormName)

      aRet := InitImageButton ( ParentFormHandle, Caption, 0, x, y, w, h, image , flat , notrans, invisible, notabstop , default , icon)

      ControlHandle := aRet [1]
      nhImage := aRet [2]

   endif

   if .Not.  lDialogInMemory
      If _HMG_BeginTabActive = TRUE
         aAdd ( _HMG_ActiveTabCurrentPageMap , Controlhandle )
      EndIf

      if valtype(tooltip) != "U"
         SetToolTip ( ControlHandle , tooltip , GetFormToolTipHandle (ParentFormName) )
      endif

   endif

   Public &mVar. := k

   _HMG_aControlType [k] := "BUTTON"
   _HMG_aControlNames [k] :=   ControlName
   _HMG_aControlHandles [k] :=   ControlHandle
   _HMG_aControlParenthandles [k] :=   ParentFormHandle
   _HMG_aControlIds [k] :=   nId
   _HMG_aControlProcedures [k] :=   ProcedureName
   _HMG_aControlPageMap [k] :=   {}
   _HMG_aControlValue [k] :=   Nil
   _HMG_aControlInputMask [k] :=   ""
   _HMG_aControllostFocusProcedure [k] :=   lostfocus
   _HMG_aControlGotFocusProcedure [k] :=   gotfocus
   _HMG_aControlChangeProcedure [k] :=   ""
   _HMG_aControlDeleted [k] :=   FALSE
   _HMG_aControlBkColor [k] :=   Nil
   _HMG_aControlFontColor [k] :=   Nil
   _HMG_aControlDblClick  [k] :=  ""
   _HMG_aControlHeadClick [k] :=   {}
   _HMG_aControlRow [k] :=   y
   _HMG_aControlCol [k] :=   x
   _HMG_aControlWidth [k] :=   w
   _HMG_aControlHeight [k] :=   h
   _HMG_aControlSpacing [k] :=   0
   _HMG_aControlContainerRow  [k] :=  iif ( _HMG_FrameLevel > 0 ,_HMG_ActiveFrameRow [_HMG_FrameLevel] , -1 )
   _HMG_aControlContainerCol  [k] :=  iif ( _HMG_FrameLevel > 0 ,_HMG_ActiveFrameCol [_HMG_FrameLevel] , -1 )
   _HMG_aControlPicture [k] :=   cPicture
   _HMG_aControlContainerHandle [k] :=   0
   _HMG_aControlFontName [k] :=   ''
   _HMG_aControlFontSize [k] :=   0
   _HMG_aControlFontAttributes [k] :=   {.f.,.f.,.f.,.f.}
   _HMG_aControlToolTip [k] :=    tooltip
   _HMG_aControlRangeMin [k] :=    0
   _HMG_aControlRangeMax [k] :=   0
   _HMG_aControlCaption [k] :=    Caption
   _HMG_aControlVisible [k] :=    if(invisible,FALSE,TRUE)
   _HMG_aControlHelpId [k] :=    HelpId
   _HMG_aControlFontHandle [k] :=    0
   _HMG_aControlBrushHandle [k] := nhImage
   _HMG_aControlEnabled [k] :=    .T.
   _HMG_aControlMiscData1 [k] := nIcon           // 0 - bitmap  1- icon

Return Nil


*-----------------------------------------------------------------------------*
Function InitDialogButtonImage(ParentFormName, ControlHandle, k)
*-----------------------------------------------------------------------------*
   Local image

   image := _HMG_aControlPicture  [k]
   If !empty(image) .and. valtype(ParentFormName) <> 'U'
      _SetBtnPicture(ControlHandle,image)
   endif
// JP 62
   if len(_HMG_aDialogTemplate) != 0 .and. _HMG_aDialogTemplate[3]   // Modal
      _HMG_aControlDeleted [k] := .T.
   endif

Return Nil

*-----------------------------------------------------------------------------*
Function _DefineOwnerButton ( ControlName, ParentForm, x, y, Caption,;
   ProcedureName, w, h, image, tooltip, gotfocus,;
   lostfocus, flat, notrans, HelpId, invisible,;
   notabstop , default,icon, fontname, fontsize,bold, italic, underline, strikeout,;
   lvertical,lefttext,uptext, aRGB_bk, aRGB_font, lnohotlight, lnoxpstyle ,ladjust )
*-----------------------------------------------------------------------------*
   Local cParentForm , mVar , ControlHandle , k
   Local nhImage , nIcon , cPicture , FontHandle , aFont
   Local aRet /*[2]*/

   default w         TO 100
   default h         TO 28
   default invisible TO FALSE
   default notabstop TO FALSE
   default lvertical TO .f.
   default lefttext to .f.
   default uptext to .f.
   default Caption to ""
   default lnohotlight to .f.
   default flat to .f.
   default notrans to .f.
   default lnoxpstyle to .f.
   default ladjust to .f.

   // ignore CAPTION clause when ADJUST is defined
   If ladjust==.t.
      Caption := ""
   endif

   If _HMG_ToolBarActive == TRUE
      Return Nil
   EndIf

   if ( FontHandle := GetFontHandle( FontName ) ) != 0
      aFont := GetFontParam(FontHandle)
      FontName      := aFont[1]
      FontSize      := aFont[2]
      bold          := aFont[3]
      italic        := aFont[4]
      underline     := aFont[5]
      strikeout     := aFont[6]
   endif

   if _HMG_BeginWindowActive = TRUE .or. _HMG_BeginDialogActive = TRUE
      ParentForm := if(_HMG_BeginDialogActive,_HMG_ActiveDialogName,_HMG_ActiveFormName)
      if .Not. Empty (_HMG_ActiveFontName) .And. ValType(FontName) == "U"
         FontName := _HMG_ActiveFontName
      EndIf
      if .Not. Empty (_HMG_ActiveFontSize) .And. ValType(FontSize) == "U"
         FontSize := _HMG_ActiveFontSize
      EndIf
   endif

   if _HMG_FrameLevel > 0
      x    := x + _HMG_ActiveFrameCol [_HMG_FrameLevel]
      y    := y + _HMG_ActiveFrameRow [_HMG_FrameLevel]
      ParentForm := _HMG_ActiveFrameParentFormName [_HMG_FrameLevel]
   EndIf
   lDialogInMemory := _HMG_DialogInMemory // ( GetWindowType ( ParentForm ) == "L" )

   If .Not. _IsWindowDefined (ParentForm)
      MsgMiniGuiError("Window: "+ ParentForm + " is not defined. Program terminated" )
   Endif

   If _IsControlDefined (ControlName,ParentForm)
      MsgMiniGuiError ("Control: " + ControlName + " Of " + ParentForm + " Already defined. Program terminated" )
   endif

   if !EMPTY(image) .and. !EMPTY(icon)
      MsgMiniGuiError ("Control: " + ControlName + " Of " + ParentForm + ". Either bitmap or icon must be specified. Program terminated" )
   endif

   nIcon := IIF(!EMPTY(icon), 1, 0)

   cPicture := IIF(!EMPTY(icon),icon,image)

   mVar := '_' + ParentForm + '_' + ControlName

   cParentForm := ParentForm

   ParentForm = GetFormHandle (ParentForm)

   if VALTYPE(cPicture)=="A"
      image:= cPicture[1]
   endif

   aRet := InitOwnerButton ( ParentForm, Caption, 0, x, y, w, h, image , flat , notrans, invisible, notabstop , default , icon)

   if valtype(fontname) == "U"
      Fontname := _HMG_DefaultFontName
   endif
   if valtype(fontsize) == "U"
      FontSize := _HMG_DefaultFontSize
   endif
   FontHandle := _SetFont (ControlHandle,FontName,FontSize,bold,italic,underline,strikeout)

   ControlHandle := aRet [1]
   nhImage := aRet [2]

   If _HMG_BeginTabActive = TRUE
      aAdd ( _HMG_ActiveTabCurrentPageMap , Controlhandle )
   EndIf

   if valtype(tooltip) != "U"
      SetToolTip ( ControlHandle , tooltip , GetFormToolTipHandle (cParentForm) )
   endif

   k := _GetControlFree()

   Public &mVar. := k

   _HMG_aControlType [k] := "OBUTTON"
   _HMG_aControlNames [k] :=   ControlName
   _HMG_aControlHandles [k] :=   ControlHandle
   _HMG_aControlParenthandles [k] :=   ParentForm
   _HMG_aControlIds [k] :=   0
   _HMG_aControlProcedures [k] :=   ProcedureName
   _HMG_aControlPageMap [k] :=   {}
   _HMG_aControlValue [k] :=   Nil
   _HMG_aControlInputMask [k] :=   ""
   _HMG_aControllostFocusProcedure [k] :=   lostfocus
   _HMG_aControlGotFocusProcedure [k] :=   gotfocus
   _HMG_aControlChangeProcedure [k] :=   ""
   _HMG_aControlDeleted [k]   :=   FALSE
   _HMG_aControlBkColor  [k] :=  aRGB_bk
   _HMG_aControlFontColor  [k] :=  aRGB_font
   _HMG_aControlDblClick  [k] :=  ""
   _HMG_aControlHeadClick [k] :=   {}
   _HMG_aControlRow [k]       :=   y
   _HMG_aControlCol [k]       :=   x
   _HMG_aControlWidth [k]     :=   w
   _HMG_aControlHeight [k]    :=   h
   _HMG_aControlSpacing [k]   :=   IIF(lvertical,1,0)+IIF(lefttext,OBT_LEFTTEXT,0)+IIF(uptext,OBT_UPTEXT,0)+iif(!lnohotlight,OBT_HOTLIGHT,0)+ iif(flat,OBT_FLAT,0)+iif(notrans,OBT_NOTRANSPARENT,0)+iif(lnoxpstyle,OBT_NOXPSTYLE,0)+iif(ladjust,OBT_ADJUST,0)
   _HMG_aControlContainerRow  [k] :=  iif ( _HMG_FrameLevel > 0 ,_HMG_ActiveFrameRow [_HMG_FrameLevel] , -1 )
   _HMG_aControlContainerCol  [k] :=  iif ( _HMG_FrameLevel > 0 ,_HMG_ActiveFrameCol [_HMG_FrameLevel] , -1 )
   _HMG_aControlPicture [k]      :=   cPicture
   _HMG_aControlContainerHandle [k] :=   0
   _HMG_aControlFontName [k]    := fontname
   _HMG_aControlFontSize [k]    := fontsize
   _HMG_aControlFontAttributes[k] := {bold,italic,underline,strikeout}
   _HMG_aControlToolTip [k]     :=  tooltip
   _HMG_aControlRangeMin [k]    :=  0
   _HMG_aControlRangeMax [k]    :=  IIF(!lnohotlight,0,2)       // used for mouse hot tracking !!
   _HMG_aControlCaption [k]     :=  Caption
   _HMG_aControlVisible [k]     :=  if(invisible,FALSE,TRUE)
   _HMG_aControlHelpId [k]      :=  HelpId
   _HMG_aControlFontHandle [k]  :=  FontHandle
   _HMG_aControlBrushHandle [k] := nhImage   // handle to an Image (Icon or Bitmap)
   _HMG_aControlEnabled [k]   :=    .T.
   _HMG_aControlMiscData1 [k] := nIcon       // 0 - bitmap  1- icon

Return Nil

* HMG 1.0 Experimental Build 9a (JK)
*-----------------------------------------------------------------------------*
Function OBTNEVENTS(hWnd, nMsg, wParam, lParam)
*-----------------------------------------------------------------------------*
   Local i := ascan ( _HMG_aControlHandles , hWnd )

   if EMPTY(wParam) .and. EMPTY(lParam)
      //return 0
   endif

   if (i > 0 .and. _HMG_aControlType[ i ] <> "OBUTTON") .or. i <= 0
      return 0
   endif

   DO CASE

   CASE nMsg == WM_MOUSEMOVE

      if AND(_HMG_aControlSpacing [ i ], OBT_HOTLIGHT) == OBT_HOTLIGHT .or. IsXPThemeActive()
         if  _HMG_aControlRangeMax[i]==0
            InvalidateRect(hWnd,0)
            _HMG_aControlRangeMax[i]:=1
         endif
         Return 0
      endif

   CASE nMsg == WM_MOUSELEAVE

      if AND(_HMG_aControlSpacing [ i ], OBT_HOTLIGHT) == OBT_HOTLIGHT .or. lXPThemeActive
         if  _HMG_aControlRangeMax[i] == 1
            _HMG_aControlRangeMax [i]:=0
            InvalidateRect(hWnd,0)
         endif
      endif

   ENDCASE

Return 0

* HMG 1.0 Experimental Build 9a (JK)
* (C) 2005 Jacek Kubica <kubica@wssk.wroc.pl>
*-----------------------------------------------------------------------------*
FUNCTION OwnButtonPaint( pdis )
*-----------------------------------------------------------------------------*
   Local hDC, itemState, itemAction, i, rgbTrans, hWnd, lFlat, lNotrans
   Local oldBkMode, oldTextColor, hOldFont, hBrush, nFreeSpace
   Local x1, y1, x2, y2, xp1:=0, yp1, xp2:=0, yp2
   Local aBmp:={}, aMetr, aBtnRc, aDarkColor
   LOCAL lDisabled, lSelected, lFocus, lDrawEntire, loFocus, loSelect
   LOCAL hTheme, nStyle, lnoxpstyle
   LOCAL pozYpic, pozYtext:=0, xPoz
   Local nCRLF, lXPThemeActive := .f.

   hDC := GETOWNBTNDC( pdis )

   if empty( hDC ) .or. hDC == 0
      return ( 1 )
   endif

   IF GETOWNBTNCTLTYPE( pdis ) <> ODT_BUTTON
      return ( 1 )
   endif

   itemAction := GETOWNBTNITEMACTION ( pdis )
   lDrawEntire := AND( itemAction, ODA_DRAWENTIRE ) == ODA_DRAWENTIRE
   loFocus := AND( itemAction, ODA_FOCUS ) == ODA_FOCUS
   loSelect := AND( itemAction, ODA_SELECT ) == ODA_SELECT

   if ! lDrawEntire  .and. ! loFocus .and. ! loSelect
      return ( 1 )
   endif

   hWnd := GETOWNBTNHANDLE( pdis )
   aBtnRc := GETOWNBTNRECT( pdis )
   itemState := GETOWNBTNSTATE( pdis )

   i := ascan ( _HMG_aControlHandles , hWnd )

   if ( i <= 0 .or. _HMG_aControlType[ i ] <> "OBUTTON" )
      return ( 1 )
   endif

   nCRLF := CountIt( _HMG_aControlCaption[ i ] ) + 1
   lDisabled := AND( itemState, ODS_DISABLED ) == ODS_DISABLED
   lSelected := AND( itemState, ODS_SELECTED ) == ODS_SELECTED
   lFocus := AND( itemState, ODS_FOCUS ) == ODS_FOCUS
   lFlat := AND( _HMG_aControlSpacing [ i ], OBT_FLAT ) == OBT_FLAT
   lNotrans := AND( _HMG_aControlSpacing [ i ], OBT_NOTRANSPARENT ) == OBT_NOTRANSPARENT
   lnoxpstyle := AND( _HMG_aControlSpacing [ i ], OBT_NOXPSTYLE ) == OBT_NOXPSTYLE

   if ! lNotrans
      rgbTrans := NIL
   else

      if ! EMPTY( _HMG_aControlBkColor [ i ] ) .and. ! lXPThemeActive
         rgbTrans := RGB( _HMG_aControlBkColor [ i, 1 ], _HMG_aControlBkColor [ i, 2 ], _HMG_aControlBkColor [ i, 3 ] )
      else
         rgbTrans := GetSysColor ( COLOR_BTNFACE )
      endif

   endif

   IF ! lnoxpstyle .and. IsXPThemeActive()

      lXPThemeActive := .t.

      nStyle := PBS_NORMAL

      IF AND( itemState, ODS_HOTLIGHT ) == ODS_HOTLIGHT  .or. _HMG_aControlRangeMax [ i ] == 1
         nStyle := PBS_HOT
      elseif lFocus
         nStyle := PBS_DEFAULTED
      ENDIF

      IF lDisabled
         nStyle := PBS_DISABLED
      ENDIF

      IF lSelected
         nStyle := PBS_PRESSED
      ENDIF

      hTheme := OpenThemeData( hWnd, ToUnicode( "BUTTON" ) )
      DrawThemeBackground( hTheme, hDC, BP_PUSHBUTTON , nStyle, aBtnRc, aBtnRc )
      CloseThemeData( hTheme )

   else

      DrawButton( hDC, IF( lFocus .or. _HMG_aControlRangeMax [ i ] == 1 , 1, 0 ),;
         DFCS_BUTTONPUSH + IF( lSelected, DFCS_PUSHED, 0 ) + IF( lDisabled, DFCS_INACTIVE, 0 ) + IF( lflat, DFCS_FLAT, 0 ),;
         pdis, IIF( AND( _HMG_aControlSpacing [ i ], OBT_HOTLIGHT ) == OBT_HOTLIGHT, _HMG_aControlRangeMax [ i ], 2 ), iif( lflat, 1, 0 ) )

   endif

   hOldFont := SelectObject( hDC, _HMG_aControlFontHandle [ i ] )
   aMetr := GetTextMetric( hDC )
   oldBkMode := SetBkMode( hDC, TRANSPARENT )
   oldTextColor := SetTextColor( hDC, GetRed ( GetSysColor ( COLOR_BTNTEXT ) ) , GetGreen ( GetSysColor ( COLOR_BTNTEXT ) ) , GetBlue ( GetSysColor ( COLOR_BTNTEXT ) ) )

   if ! lDisabled

      if EMPTY( _HMG_aControlFontColor [ i ] )
         SetTextColor( hDC, GetRed ( GetSysColor ( COLOR_BTNTEXT ) ) , GetGreen ( GetSysColor ( COLOR_BTNTEXT ) ) , GetBlue ( GetSysColor ( COLOR_BTNTEXT ) ) )
      else
         SetTextColor( hDC, _HMG_aControlFontColor [ i, 1 ], _HMG_aControlFontColor [ i, 2 ], _HMG_aControlFontColor [ i, 3 ] )
      endif

      if ! EMPTY( _HMG_aControlBkColor [ i ] ) .and. ! lXPThemeActive

         if ( lSelected )
            aDarkColor := Darker( { _HMG_aControlBkColor [ i, 1 ]  , _HMG_aControlBkColor [ i, 2 ]  , _HMG_aControlBkColor [ i, 3 ] }, 97 )
         else
            aDarkColor := { _HMG_aControlBkColor [ i, 1 ]  , _HMG_aControlBkColor [ i, 2 ]  , _HMG_aControlBkColor [ i, 3 ] }
         endif

         hBrush := CreateSolidBrush( aDarkColor [ 1 ]  , aDarkColor [ 2 ]  , aDarkColor [ 3 ]  )

         // paint button background

         if ! lflat
            FILLRECT( hDC, aBtnRc[ 1 ] + 2, aBtnRc[ 2 ] + 2, aBtnRc[ 3 ] - 3, aBtnRc[ 4 ] - 3, hBrush )
         else
            if ! lfocus .and. ! lSelected .and. ! ( _HMG_aControlRangeMax [ i ] == 1 )
               FILLRECT( hDC, aBtnRc[ 1 ] + 1, aBtnRc[ 2 ] + 1, aBtnRc[ 3 ] - 1, aBtnRc[ 4 ] - 1, hBrush )
            else
               FILLRECT( hDC, aBtnRc[ 1 ] + 2, aBtnRc[ 2 ] + 2, aBtnRc[ 3 ] - 2, aBtnRc[ 4 ] - 2, hBrush )
            endif

         endif
         DeleteObject( hBrush )
      endif

   endif

   if _HMG_aControlMiscData1 [ i ] == 0  .and. ! EMPTY( _HMG_aControlBrushHandle [ i ] )
      aBmp := GetBitmapSize( _HMG_aControlBrushHandle [ i ] )
   elseif  _HMG_aControlMiscData1 [ i ] == 1  .and. ! EMPTY( _HMG_aControlBrushHandle [ i ] )
      aBmp := GetIconSize( _HMG_aControlBrushHandle [ i ] )
   endif

   if AND( _HMG_aControlSpacing [ i ], OBT_VERTICAL ) == OBT_VERTICAL

      // vertical text/picture aspect

//      x1 := aBtnRc[ 1 ] + 2
      y2 := aMetr[ 1 ]*nCRLF
//      y1 := Round( ( aBtnRc[ 4 ] - aBtnRc[ 2 ] - aMetr[ 1 ] ) / 2, 0 )
      x2 := aBtnRc[ 3 ] - 2

//      yp1 := Round( y1 / 2, 0 )
      xp2 := IIF( ! EMPTY( aBmp ), aBmp[ 1 ], 0 ) // picture width
      yp2 := IIF( ! EMPTY( aBmp ), aBmp[ 2 ], 0 ) // picture height
      xp1 := ROUND( ( aBtnRc[ 3 ] / 2 ) - ( xp2 / 2 ), 0 )

      If AT( CRLF, _HMG_aControlCaption[ i ] ) <= 0
         nFreeSpace := ROUND( ( aBtnRc[ 4 ] - 4 - ( aMetr[ 4 ] + yp2 ) ) / 3, 0 )
         nCRLF:=1
      else
//         y1 := max( ( ( aBtnRc[ 4 ] ) / 2 ) - ( nCRLF * aMetr[ 1 ] ) / 2, 1 )
         nFreeSpace := ROUND( ( aBtnRc[ 4 ] - 4 - ( y2 + yp2 ) ) / 3, 0 )
      endif

      if !EMPTY( _HMG_aControlCaption[ i ] )  // button has caption

         If !EMPTY( _HMG_aControlBrushHandle [ i ] )
            if !( AND( _HMG_aControlSpacing [ i ], OBT_UPTEXT ) == OBT_UPTEXT ) // upper text aspect not set
               pozYpic := max( aBtnRc[ 2 ] + nFreeSpace, 5 )
               pozYtext := aBtnRc[ 2 ] + IIF( ! EMPTY( aBmp ), nFreeSpace, 0 ) + yp2 + IIF( ! EMPTY( aBmp ), nFreeSpace, 0 ) //+ nFreeSpace+2 // +ROUND(aMetr[1]/2,0)
            else
               pozYtext := max( aBtnRc[ 2 ] + nFreeSpace, 5 )
               aBtnRc[ 4 ] := nFreeSpace + ( ( aMetr[ 1 ] ) * nCRLF )   + nFreeSpace
               pozYpic := aBtnRc[ 4 ]
            endif
         else
            pozYpic := 0
            pozYtext := ROUND((aBtnRc[ 4 ] - y2) / 2,0)
         endif

      else  // button without caption

         if ! ( AND( _HMG_aControlSpacing [ i ], OBT_ADJUST ) == OBT_ADJUST )
            pozYpic := Round( ( ( aBtnRc[ 4 ] / 2 ) - ( yp2 / 2 ) ), 0 )
            pozYtext := 0
         else  // strech image
            pozYpic := 1
         endif

      endif

      if ! lDisabled

         if lSelected // vertical selected

            If ! lXPThemeActive
               xp1 ++
               xPoz := 2
               pozYtext ++
               pozYpic ++
            else
               xPoz := 0
            endif

            if ! ( AND( _HMG_aControlSpacing [ i ], OBT_ADJUST ) == OBT_ADJUST )
               DrawGlyph( hDC, xp1, pozYpic , xp2, yp2 , _HMG_aControlBrushHandle [ i ] , rgbTrans , .f., .f. )
               DrawText( hDC, _HMG_aControlCaption[ i ], xPoz , pozYtext-1 , x2,  aBtnRc[ 4 ] , DT_CENTER )
            else
               DrawGlyph( hDC, aBtnRc[ 1 ] + 4, aBtnRc[ 2 ] + 4 , aBtnRc[ 3 ] - 6, aBtnRc[ 4 ] - 6 , _HMG_aControlBrushHandle [ i ] , rgbTrans , .f., .t. )
            endif

         else  // vertical non selected

            if ! ( AND( _HMG_aControlSpacing [ i ], OBT_ADJUST ) == OBT_ADJUST )
               DrawGlyph( hDC, xp1, pozYpic, xp2, yp2 , _HMG_aControlBrushHandle [ i ] , rgbTrans , .f., .f. )
               DrawText( hDC, _HMG_aControlCaption[ i ],  0 , pozYtext-1 , x2,  aBtnRc[ 4 ], DT_CENTER )
            else
               DrawGlyph( hDC, aBtnRc[ 1 ] + 3, aBtnRc[ 2 ] + 3 , aBtnRc[ 3 ] - 6, aBtnRc[ 4 ] - 6 , _HMG_aControlBrushHandle [ i ] , rgbTrans , .f., .t. )
            endif

         endif

      else // vertical disabled

         if ! ( AND( _HMG_aControlSpacing [ i ], OBT_ADJUST ) == OBT_ADJUST )
            DrawGlyph( hDC, xp1, pozYpic, xp2, yp2 , _HMG_aControlBrushHandle [ i ] , , .t., .f. )
            // disabled  vertical
            SetTextColor( hDC, GetRed ( GetSysColor ( COLOR_3DHILIGHT ) ) , GetGreen ( GetSysColor ( COLOR_3DHILIGHT ) ) , GetBlue ( GetSysColor ( COLOR_3DHILIGHT ) ) )
            DrawText( hDC, _HMG_aControlCaption[ i ], 2, pozYtext + 1 , x2, aBtnRc[ 4 ] + 1,  DT_CENTER )
            SetTextColor( hDC, GetRed ( GetSysColor ( COLOR_3DSHADOW ) ) , GetGreen ( GetSysColor ( COLOR_3DSHADOW ) ) , GetBlue ( GetSysColor ( COLOR_3DSHADOW ) ) )
            DrawText( hDC, _HMG_aControlCaption[ i ], 0, pozYtext, x2, aBtnRc[ 4 ],    DT_CENTER )
         else
            DrawGlyph( hDC, aBtnRc[ 1 ] + 4, aBtnRc[ 2 ] + 4 , aBtnRc[ 3 ] - 6, aBtnRc[ 4 ] - 6 , _HMG_aControlBrushHandle [ i ] , , .t., .t. )
         endif

      endif

   else

      if ! EMPTY( _HMG_aControlBrushHandle [ i ] ) // horizontal

         if ! EMPTY( _HMG_aControlCaption[ i ] )

            if ! ( AND( _HMG_aControlSpacing [ i ], OBT_LEFTTEXT ) == OBT_LEFTTEXT )

               xp1 := 5
               xp2 := IIF( ! EMPTY( aBmp ), aBmp[ 1 ], 0 )
               yp2 := IIF( ! EMPTY( aBmp ), aBmp[ 2 ], 0 )
               yp1 := Round( ( ( aBtnRc[ 4 ] / 2 ) - ( yp2 / 2 ) ), 0 )

               x1 := aBtnRc[ 1 ] + xp1 + xp2
               y1 := Round( aBtnRc[ 4 ] / 2, 0 ) - ( aMetr[ 1 ] - 10 )
               x2 := aBtnRc[ 3 ] - 2
               y2 := y1 + aMetr[ 1 ]

            else

               xp1 := aBtnRc[ 3 ] - IIF( ! EMPTY( aBmp ), aBmp[ 1 ], 0 ) - 5
               xp2 := IIF( ! EMPTY( aBmp ), aBmp[ 1 ], 0 )
               yp2 := IIF( ! EMPTY( aBmp ), aBmp[ 2 ], 0 )
               yp1 := Round( ( ( aBtnRc[ 4 ] / 2 ) - ( yp2 / 2 ) ), 0 )


               x1 := 3
               y1 := Round( aBtnRc[ 4 ] / 2, 0 ) - ( aMetr[ 1 ] - 10 )
               x2 := aBtnRc[ 3 ] - xp2
               y2 := y1 + aMetr[ 1 ]

            endif

         else
            x1 := aBtnRc[ 1 ] + xp1 + xp2
            y1 := Round( aBtnRc[ 4 ] / 2, 0 ) - ( aMetr[ 1 ] - 10 )
            x2 := aBtnRc[ 3 ] - 2
            y2 := y1 + aMetr[ 1 ]

            xp2 := IIF( ! EMPTY( aBmp ), aBmp[ 1 ], 0 ) // picture width
            yp2 := IIF( ! EMPTY( aBmp ), aBmp[ 2 ], 0 ) // picture height
            xp1 := ROUND( ( aBtnRc[ 3 ] / 2 ) - ( xp2 / 2 ), 0 )
            yp1 := Round( ( ( aBtnRc[ 4 ] / 2 ) - ( yp2 / 2 ) ), 0 )
         endif

      else

         xp1 := 2
         xp2 := 0
         yp1 := 0
         yp2 := 0

         x1 := aBtnRc[ 1 ] + xp1 + xp2
         y1 := Round( aBtnRc[ 4 ] / 2, 0 ) - ( aMetr[ 1 ] - 10 )
         x2 := aBtnRc[ 3 ] - 2
         y2 := y1 + aMetr[ 1 ]

      endif

      if ! ( AND( _HMG_aControlSpacing [ i ], OBT_ADJUST ) == OBT_ADJUST )
         y1 := max(( ( ( aBtnRc[ 4 ] ) / 2 ) - ( nCRLF * aMetr[ 1 ] ) / 2)-1, 1 )
         y2 := ( aMetr[ 1 ] + aMetr[ 5 ] ) * nCRLF
//      else
//         pozYpic := 1
      endif

      if ! lDisabled

         if lSelected

            If ! lXPThemeActive
               x1 := x1 + 2
            //   y1 := 3
               xp1 ++
               yp1 ++
            //else
            //   y1 := 1
            //   xPoz := 0
            endif

            if ! ( AND( _HMG_aControlSpacing [ i ], OBT_ADJUST ) == OBT_ADJUST )
               DrawGlyph( hDC, xp1, yp1, xp2, yp2 , _HMG_aControlBrushHandle [ i ] , rgbTrans, .f., .f. )
               DrawText( hDC, _HMG_aControlCaption[ i ], x1, y1 + 1, x2, y1 + y2 , DT_CENTER )
            else
               DrawGlyph( hDC, aBtnRc[ 1 ] + 4, aBtnRc[ 2 ] + 4 , aBtnRc[ 3 ] - 6, aBtnRc[ 4 ] - 6 , _HMG_aControlBrushHandle [ i ] , rgbTrans , .f., .t. )
            endif

         else
            if ! ( AND( _HMG_aControlSpacing [ i ], OBT_ADJUST ) == OBT_ADJUST )
               DrawGlyph( hDC, xp1, yp1, xp2, yp2 , _HMG_aControlBrushHandle [ i ] , rgbTrans , .f., .f. )
               DrawText( hDC, _HMG_aControlCaption[ i ], x1, y1, x2, y1 + y2, DT_CENTER )
            else
               DrawGlyph( hDC, aBtnRc[ 1 ] + 3, aBtnRc[ 2 ] + 3 , aBtnRc[ 3 ] - 6, aBtnRc[ 4 ] - 6 , _HMG_aControlBrushHandle [ i ] , rgbTrans , .f., .t. )
            endif
         endif

      else
         // disabled horizontal
         if ! ( AND( _HMG_aControlSpacing [ i ], OBT_ADJUST ) == OBT_ADJUST )
            DrawGlyph( hDC, xp1, yp1, xp2, yp2 , _HMG_aControlBrushHandle [ i ] , , .t., .f. )
            SetTextColor( hDC, GetRed ( GetSysColor ( COLOR_3DHILIGHT ) ) , GetGreen ( GetSysColor ( COLOR_3DHILIGHT ) ) , GetBlue ( GetSysColor ( COLOR_3DHILIGHT ) ) )
            DrawText( hDC, _HMG_aControlCaption[ i ], x1 + 1, y1 + 1, x2 + 1, y1 + y2 + 1,  DT_CENTER )
            SetTextColor( hDC, GetRed ( GetSysColor ( COLOR_3DSHADOW ) ) , GetGreen ( GetSysColor ( COLOR_3DSHADOW ) ) , GetBlue ( GetSysColor ( COLOR_3DSHADOW ) ) )
            DrawText( hDC, _HMG_aControlCaption[ i ], x1, y1, x2, y1 + y2 ,  DT_CENTER )
         else
            DrawGlyph( hDC, aBtnRc[ 1 ] + 3, aBtnRc[ 2 ] + 3 , aBtnRc[ 3 ] - 6, aBtnRc[ 4 ] - 6 , _HMG_aControlBrushHandle [ i ] , , .t., .t. )
         endif
      endif
   endif

   if ( lSelected .or. lFocus ) .and. ! lDisabled .and. ! lXPThemeActive
      SetTextColor( hDC, GetRed ( GetSysColor ( COLOR_BTNTEXT ) ) , GetGreen ( GetSysColor ( COLOR_BTNTEXT ) ) , GetBlue ( GetSysColor ( COLOR_BTNTEXT ) ) )
      DrawFocusRect( pdis )
   endif

   SelectObject( hDC, hOldFont )
   SetBkMode( hDC, oldBkMode )
   SetTextColor( hDC, oldTextColor )

return ( 1 )

*-----------------------------------------------------------------------------*
function ToUnicode( cString )
*-----------------------------------------------------------------------------*
   Local i, cTemp := ""

   for i = 1 to len( cString )
      cTemp += substr( cString, i, 1 ) + chr( 0 )
   next

return cTemp + chr( 0 )

*-----------------------------------------------------------------------------*
Function _SetBtnPictureMask(Hwnd, ControlIndex)
*-----------------------------------------------------------------------------*
   Local hDC := GetDC( Hwnd ), aBMP
   Local aBtnRc := Array(4)
   Local i := ControlIndex
   Local x, y

   If Empty(hDC) .or. Empty(_HMG_aControlBrushHandle [ i ])
      Return NIL
   endif

   aBtnRc[1] := _HMG_aControlRow    [i]
   aBtnRc[2] := _HMG_aControlCol    [i]
   aBtnRc[3] := _HMG_aControlWidth  [i]
   aBtnRc[4] := _HMG_aControlHeight [i]

   aBmp := GetBitmapSize( _HMG_aControlBrushHandle [ i ] )
   x := aBtnRc[3] / 2 - aBmp[1] / 2
   y := aBtnRc[4] / 2 - aBmp[2] / 2

   DrawGlyphMask( hDC, x, y, aBmp[1], aBmp[2], _HMG_aControlBrushHandle [ i ], , .t., .f., Hwnd )

   ReleaseDC( hWnd, hDC )

Return NIL

*-----------------------------------------------------------------------------*
Function _DestroyBtnPictureMask(Hwnd, ControlIndex)
*-----------------------------------------------------------------------------*
   Local MaskHwnd := _GetBtnPictureHandle(Hwnd)

   if !Empty(MaskHwnd) .and. MaskHwnd <> _HMG_aControlBrushHandle [ControlIndex]
      DeleteObject(MaskHwnd)
   endif

Return NIL

*-----------------------------------------------------------------------------*
Function _DestroyBtnPicture(Hwnd, ControlIndex)
*-----------------------------------------------------------------------------*
   Local BtnPicHwnd := _GetBtnPictureHandle(Hwnd)

   if !Empty(BtnPicHwnd) .and. BtnPicHwnd == _HMG_aControlBrushHandle [ControlIndex]
      DeleteObject(BtnPicHwnd)
   endif

Return NIL

*-----------------------------------------------------------------------------*
Function Darker(aColor, Percent)
*-----------------------------------------------------------------------------*
   Local R, G, B, aDark:=Array(3)

   R := aColor[1]
   G := aColor[2]
   B := aColor[3]

   R := Round(R*Percent/100,0)
   G := Round(G*Percent/100,0)
   B := Round(B*Percent/100,0)

   aDark[1] := R
   aDark[2] := G
   aDark[3] := B

RETURN aDark

*-----------------------------------------------------------------------------*
Function Lighter(aColor, Percent)
*-----------------------------------------------------------------------------*
   Local R, G, B, aLight:=Array(3)

   R := aColor[1]
   G := aColor[2]
   B := aColor[3]

   R := Round(R*Percent/100,0) + Round(255 - Percent/100*255,0)
   G := Round(G*Percent/100,0) + Round(255 - Percent/100*255,0)
   B := Round(B*Percent/100,0) + Round(255 - Percent/100*255,0)

   aLight[1] := R
   aLight[2] := G
   aLight[3] := B

RETURN aLight

*-----------------------------------------------------------------------------*
Static Function CountIt( cText )
*-----------------------------------------------------------------------------*
   Local nPoz, nCount := 0

   If At(CRLF, cText) > 0
      Do While ( nPoz := At(CRLF, cText) ) > 0
         nCount++
         cText := Substr(cText, nPoz+2)
      EndDo
   endif

Return nCount
