/*----------------------------------------------------------------------------
MINIGUI - Harbour Win32 GUI library source code

Copyright 2002-2008 Roberto Lopez <harbourminigui@gmail.com>
http://harbourminigui.googlepages.com/

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along with
   this software; see the file COPYING. If not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA (or
   visit the web site http://www.gnu.org/).

   As a special exception, you have permission for additional uses of the text
   contained in this release of Harbour Minigui.

   The exception is that, if you link the Harbour Minigui library with other
   files to produce an executable, this does not by itself cause the resulting
   executable to be covered by the GNU General Public License.
   Your use of that executable is in no way restricted on account of linking the
   Harbour-Minigui library code into it.

   Parts of this project are based upon:

   "Harbour GUI framework for Win32"
   Copyright 2001 Alexander S.Kresin <alex@belacy.belgorod.su>
   Copyright 2001 Antonio Linares <alinares@fivetech.com>
   www - http://www.harbour-project.org

   "Harbour Project"
   Copyright 1999-2008, http://www.harbour-project.org/

   "WHAT32"
   Copyright 2002 AJ Wos <andrwos@aust1.net>

   "HWGUI"
   Copyright 2001-2007 Alexander S.Kresin <alex@belacy.belgorod.su>

---------------------------------------------------------------------------*/

#include "minigui.ch"

#define BS_NOTIFY           0x00004000
#define BS_PUSHBUTTON       0x00000000
#define BS_FLAT             0x00008000
#define BS_BITMAP           0x00000080
#define BS_PUSHLIKE         0x00001000
#define BS_AUTOCHECKBOX     0x00000003
#define BS_AUTO3STATE       0x00000006
#define WS_TABSTOP          0x00010000
#define WS_VISIBLE          0x10000000
#define WS_CHILD            0x40000000

#define BM_GETCHECK         240
#define BM_SETCHECK         241
#define BST_UNCHECKED       0
#define BST_CHECKED         1
#define BST_INDETERMINATE   2
#define COLOR_BTNFACE       15

*-----------------------------------------------------------------------------*
Function _DefineCheckBox ( ControlName, ParentFormName, x, y, Caption, Value,;
   fontname, fontsize, tooltip, changeprocedure, w, h, lostfocus, gotfocus, HelpId, invisible, notabstop,;
   bold, italic, underline, strikeout, field, backcolor, fontcolor, transparent, leftjustify, threestate, nId )
*-----------------------------------------------------------------------------*
   Local ParentFormHandle , blInit , mVar , k , style
   Local ControlHandle , FontHandle , aFont , WorkArea
   Local lDialogInMemory

   DEFAULT w               TO 100
   DEFAULT h               TO 28
   DEFAULT lostfocus       TO ""
   DEFAULT gotfocus        TO ""
   DEFAULT changeprocedure TO ""
   DEFAULT invisible       TO FALSE
   DEFAULT notabstop       TO FALSE
   DEFAULT leftjustify     TO FALSE
   DEFAULT threestate      TO FALSE

   if .Not. threestate
      DEFAULT value      TO FALSE
   endif

   if ( FontHandle := GetFontHandle( FontName ) ) != 0
      aFont := GetFontParam(FontHandle)
      FontName      := aFont[1]
      FontSize      := aFont[2]
      bold          := aFont[3]
      italic        := aFont[4]
      underline     := aFont[5]
      strikeout     := aFont[6]
   endif

   If ValType ( Field ) != 'U'
      if at ( '>', Field ) == 0
         MsgMiniGuiError ("Control: " + ControlName + " Of " + ParentFormName + " : You must specify a fully qualified field name. Program Terminated" )
      Else
         WorkArea := Left ( Field , at ( '>', Field ) - 2 )
         If Select (WorkArea) != 0
            Value := &(Field)
         EndIf
      EndIf
   EndIf

   if _HMG_BeginWindowActive = TRUE .or. _HMG_BeginDialogActive = TRUE
      ParentFormName := if(_HMG_BeginDialogActive,_HMG_ActiveDialogName,_HMG_ActiveFormName)
      if .Not. Empty (_HMG_ActiveFontName) .And. ValType(FontName) == "U"
         FontName := _HMG_ActiveFontName
      EndIf
      if .Not. Empty (_HMG_ActiveFontSize) .And. ValType(FontSize) == "U"
         FontSize := _HMG_ActiveFontSize
      EndIf
   endif
   if _HMG_FrameLevel > 0
      x    := x + _HMG_ActiveFrameCol [_HMG_FrameLevel]
      y    := y + _HMG_ActiveFrameRow [_HMG_FrameLevel]
      ParentFormName := _HMG_ActiveFrameParentFormName [_HMG_FrameLevel]
   EndIf
   lDialogInMemory := _HMG_DialogInMemory // ( GetWindowType ( ParentFormName ) == "L" )

   If .Not. _IsWindowDefined (ParentFormName) .and. .Not. lDialogInMemory
      MsgMiniGuiError("Window: "+ ParentFormName + " is not defined. Program terminated" )
   Endif

   If _IsControlDefined (ControlName,ParentFormName) .and. .Not. lDialogInMemory
      MsgMiniGuiError ("Control: " + ControlName + " Of " + ParentFormName + " Already defined. Program Terminated" )
   endif

   If transparent == .T. .And. IsXPThemeActive() // Fixed for transparent problem at themed WinXP and WinVista
      transparent := FALSE
      mVar := _HMG_aFormBkColor[ GetFormIndex ( ParentFormName ) ]
      If backcolor == NIL .and. mVar[1] < 0 .and. mVar[2] < 0 .and. mVar[3] < 0
         mVar := GetSysColor( COLOR_BTNFACE )
         backcolor := { GetRed(mVar), GetGreen(mVar), GetBlue(mVar) }
      else
         backcolor := mVar
      endif
   EndIf

   mVar := '_' + ParentFormName + '_' + ControlName
   k := _GetControlFree()

   if _HMG_BeginDialogActive = TRUE
      ParentFormHandle = _HMG_ActiveDialogHandle
      Style = BS_NOTIFY + WS_CHILD  // + BS_LEFT

      if !NoTabStop
         Style += WS_TABSTOP
      endif
      if !invisible
         Style += WS_VISIBLE
      endif

      if threestate
         Style += BS_AUTO3STATE
      else
         Style += BS_AUTOCHECKBOX
      endif

      if lDialogInMemory         //Dialog Template

//           {{'ID',k/hwnd,class,Style,ExStyle,x,y,w,h,caption,HelpId,tooltip,font,size, bold, italic, underline, strikeout}}  --->_HMG_aDialogItems
         blInit:= {|x,y,z| InitDialogCheckButton(x,y,z) }
         aadd(_HMG_aDialogItems,{nId,k,"button",style,0,x,y,w,h,caption,HelpId,tooltip,FontName,FontSize,bold,italic,underline,strikeout,blInit,_HMG_BeginTabActive,.f.,_HMG_ActiveTabPage })
/* JP 62
         if _HMG_aDialogTemplate[3]   // Modal
            _HMG_aControlDeleted [k] := .T.
            Return Nil
         endif
*/
      else

         ControlHandle := GetDialogItemHandle( ParentFormHandle, nId )

         x := GetWindowCol ( Controlhandle )
         y := GetWindowRow ( Controlhandle )
         w := GetWindowWidth  ( Controlhandle )
         h := GetWindowHeight ( Controlhandle )

         if valtype(caption) != "U"
            SetWindowText ( ControlHandle , caption )
         endif

         SetWindowStyle ( ControlHandle, Style )
      endif

   else

      ParentFormHandle = GetFormHandle (ParentFormName)
      Controlhandle := InitCheckBox ( ParentFormHandle, Caption, 0, x, y, '', 0 , w , h, invisible, notabstop, leftjustify, threestate, transparent )

   endif

   if .Not. lDialogInMemory

      If FontHandle != 0
         _SetFontHandle(ControlHandle,FontHandle)
      Else
         if valtype(fontname) == "U"
            Fontname := _HMG_DefaultFontName
         endif
         if valtype(fontsize) == "U"
            FontSize := _HMG_DefaultFontSize
         endif
         FontHandle := _SetFont (ControlHandle,FontName,FontSize,bold,italic,underline,strikeout)
      EndIf

      If _HMG_BeginTabActive = .T.
         aAdd ( _HMG_ActiveTabCurrentPageMap , Controlhandle )
      EndIf

      if valtype(tooltip) != "U"
         SetToolTip ( ControlHandle , tooltip , GetFormToolTipHandle (ParentFormName) )
      endif

   endif

   Public &mVar. := k

   _HMG_aControlType [k] := "CHECKBOX"
   _HMG_aControlNames [k] :=  ControlName
   _HMG_aControlHandles [k] :=  ControlHandle
   _HMG_aControlParenthandles [k] :=  ParentFormHandle
   _HMG_aControlIds [k] :=  nId
   _HMG_aControlProcedures [k] :=  ""
   _HMG_aControlPageMap [k] :=  Field
   _HMG_aControlValue [k] :=  Value
   _HMG_aControlInputMask [k] :=  transparent
   _HMG_aControllostFocusProcedure [k] :=  lostfocus
   _HMG_aControlGotFocusProcedure [k] :=  gotfocus
   _HMG_aControlChangeProcedure [k] :=  changeprocedure
   _HMG_aControlDeleted [k] :=  .F.
   _HMG_aControlBkColor [k] :=  backcolor
   _HMG_aControlFontColor [k] :=  fontcolor
   _HMG_aControlDblClick  [k] :=  _HMG_ActiveTabButtons
   _HMG_aControlHeadClick [k] :=  {}
   _HMG_aControlRow  [k] :=  y
   _HMG_aControlCol  [k] :=  x
   _HMG_aControlWidth  [k] :=  w
   _HMG_aControlHeight [k] :=  h
   _HMG_aControlSpacing  [k] :=  threestate
   _HMG_aControlContainerRow  [k] :=  iif ( _HMG_FrameLevel > 0 ,_HMG_ActiveFrameRow [_HMG_FrameLevel] , -1 )
   _HMG_aControlContainerCol  [k] :=  iif ( _HMG_FrameLevel > 0 ,_HMG_ActiveFrameCol [_HMG_FrameLevel] , -1 )
   _HMG_aControlPicture  [k] :=  ""
   _HMG_aControlContainerHandle  [k] :=  0
   _HMG_aControlFontName  [k] :=  fontname
   _HMG_aControlFontSize  [k] :=  fontsize
   _HMG_aControlFontAttributes  [k] :=  {bold,italic,underline,strikeout}
   _HMG_aControlToolTip  [k] :=  tooltip
   _HMG_aControlRangeMin [k] :=  iif ( _HMG_FrameLevel > 0 ,_HMG_ActiveTabName , '' )
   _HMG_aControlRangeMax [k] :=  iif ( _HMG_FrameLevel > 0 ,_HMG_ActiveFrameParentFormName [_HMG_FrameLevel] , '' )
   _HMG_aControlCaption  [k] :=  Caption
   _HMG_aControlVisible  [k] :=  if(invisible, FALSE, TRUE)
   _HMG_aControlHelpId   [k] :=  HelpId
   _HMG_aControlFontHandle   [k] :=  FontHandle
   _HMG_aControlBrushHandle  [k] :=  0
   _HMG_aControlEnabled   [k] :=  .T.
   _HMG_aControlMiscData1 [k] :=  0

   if .Not. lDialogInMemory
      if value = .T.
         SendMessage( Controlhandle , BM_SETCHECK , BST_CHECKED , 0 )
      elseif threestate .and. value == NIL
         SendMessage( Controlhandle , BM_SETCHECK , BST_INDETERMINATE , 0 )
      endif
   endif

   if valtype ( Field ) != 'U'
      aAdd ( _HMG_aFormBrowseList [ GetFormIndex ( ParentFormName ) ] , k )
   EndIf

Return Nil

*-----------------------------------------------------------------------------*
Function _DefineCheckButton ( ControlName, ParentFormName, x, y, Caption, Value,;
   fontname, fontsize, tooltip, changeprocedure,;
   w, h, lostfocus, gotfocus, HelpId, invisible,;
   notabstop , bold, italic, underline, strikeout, nId )
*-----------------------------------------------------------------------------*
   Local ParentFormHandle , blInit, mVar , k , Style
   Local ControlHandle , FontHandle , aFont
   Local lDialogInMemory

   DEFAULT value           TO FALSE
   DEFAULT w               TO 100
   DEFAULT h               TO 28
   DEFAULT lostfocus       TO ""
   DEFAULT gotfocus        TO ""
   DEFAULT changeprocedure TO ""
   DEFAULT invisible       TO FALSE
   DEFAULT notabstop       TO FALSE

   if ( FontHandle := GetFontHandle( FontName ) ) != 0
      aFont := GetFontParam(FontHandle)
      FontName      := aFont[1]
      FontSize      := aFont[2]
      bold          := aFont[3]
      italic        := aFont[4]
      underline     := aFont[5]
      strikeout     := aFont[6]
   endif

   if _HMG_BeginWindowActive = TRUE .or. _HMG_BeginDialogActive = TRUE
      ParentFormName := if(_HMG_BeginDialogActive,_HMG_ActiveDialogName,_HMG_ActiveFormName)
      if .Not. Empty (_HMG_ActiveFontName) .And. ValType(FontName) == "U"
         FontName := _HMG_ActiveFontName
      EndIf
      if .Not. Empty (_HMG_ActiveFontSize) .And. ValType(FontSize) == "U"
         FontSize := _HMG_ActiveFontSize
      EndIf
   endif
   if _HMG_FrameLevel > 0
      x    := x + _HMG_ActiveFrameCol [_HMG_FrameLevel]
      y    := y + _HMG_ActiveFrameRow [_HMG_FrameLevel]
      ParentFormName := _HMG_ActiveFrameParentFormName [_HMG_FrameLevel]
   EndIf
   lDialogInMemory := _HMG_DialogInMemory // ( GetWindowType ( ParentFormName ) == "L" )

   If .Not. _IsWindowDefined (ParentFormName) .and. .Not. lDialogInMemory
      MsgMiniGuiError("Window: "+ ParentFormName + " is not defined. Program terminated" )
   Endif

   If _IsControlDefined (ControlName,ParentFormName) .and. .Not. lDialogInMemory
      MsgMiniGuiError ("Control: " + ControlName + " Of " + ParentFormName + " Already defined. Program Terminated" )
   endif

   mVar := '_' + ParentFormName + '_' + ControlName
   k := _GetControlFree()

   if _HMG_BeginDialogActive = TRUE
      ParentFormHandle = _HMG_ActiveDialogHandle

      Style := BS_NOTIFY + WS_CHILD + BS_AUTOCHECKBOX + BS_PUSHLIKE

      if !NoTabStop
         Style += WS_TABSTOP
      endif

      if !invisible
         Style += WS_VISIBLE
      endif

      if lDialogInMemory         //Dialog Template

//          {{ID,k/hwnd,class,Style,ExStyle,x,y,w,h,caption,HelpId,tooltip,font,size, bold, italic, underline, strikeout}}  --->_HMG_aDialogItems
         blInit:= {|x,y,z| InitDialogCheckButton(x,y,z) }
         aadd(_HMG_aDialogItems,{nId,k,"button",style,0,x,y,w,h,"",HelpId,tooltip,FontName,FontSize,bold,italic,underline,strikeout,blInit,_HMG_BeginTabActive,.f.,_HMG_ActiveTabPage })
/* JP 62
         if _HMG_aDialogTemplate[3]   // Modal
            Return Nil
         endif
*/
      else

         ControlHandle := GetDialogItemHandle( ParentFormHandle, nId )

         x := GetWindowCol ( Controlhandle )
         y := GetWindowRow ( Controlhandle )
         w := GetWindowWidth  ( Controlhandle )
         h := GetWindowHeight ( Controlhandle )

         SetWindowStyle ( ControlHandle, Style )

      endif

   else

      ParentFormHandle = GetFormHandle (ParentFormName)

      Controlhandle := InitCheckButton ( ParentFormHandle, Caption, 0, x, y, '', 0 , w , h, invisible, notabstop )

   Endif

   if .Not. lDialogInMemory

      If FontHandle != 0
         _SetFontHandle(ControlHandle,FontHandle)
      Else
         if valtype(fontname) == "U"
            Fontname := _HMG_DefaultFontName
         endif
         if valtype(fontsize) == "U"
            FontSize := _HMG_DefaultFontSize
         endif
         FontHandle := _SetFont (ControlHandle,FontName,FontSize,bold,italic,underline,strikeout)
      EndIf

      If _HMG_BeginTabActive = .T.
         aAdd ( _HMG_ActiveTabCurrentPageMap , ControlHandle )
      EndIf

      if valtype(tooltip) != "U"
         SetToolTip ( ControlHandle , tooltip , GetFormToolTipHandle (ParentFormName) )
      endif

   Endif

   Public &mVar. := k

   _HMG_aControlType [k] := "CHECKBOX"
   _HMG_aControlNames [k] :=   ControlName
   _HMG_aControlHandles [k] :=   ControlHandle
   _HMG_aControlParenthandles [k] :=   ParentFormHandle
   _HMG_aControlIds [k] :=   nId
   _HMG_aControlProcedures [k] :=   ""
   _HMG_aControlPageMap  [k] :=  {}
   _HMG_aControlValue  [k] :=  Value
   _HMG_aControlInputMask   [k] := ""
   _HMG_aControllostFocusProcedure  [k] :=  lostfocus
   _HMG_aControlGotFocusProcedure  [k] :=  gotfocus
   _HMG_aControlChangeProcedure  [k] :=  changeprocedure
   _HMG_aControlDeleted  [k] :=  .F.
   _HMG_aControlBkColor   [k] := Nil
   _HMG_aControlFontColor   [k] := Nil
   _HMG_aControlDblClick   [k] :=  ""
   _HMG_aControlHeadClick  [k] :=  {}
   _HMG_aControlRow  [k] :=  y
   _HMG_aControlCol   [k] := x
   _HMG_aControlWidth   [k] := w
   _HMG_aControlHeight   [k] := h
   _HMG_aControlSpacing   [k] := .f.
   _HMG_aControlContainerRow  [k] :=  iif ( _HMG_FrameLevel > 0 ,_HMG_ActiveFrameRow [_HMG_FrameLevel] , -1 )
   _HMG_aControlContainerCol  [k] :=  iif ( _HMG_FrameLevel > 0 ,_HMG_ActiveFrameCol [_HMG_FrameLevel] , -1 )
   _HMG_aControlPicture  [k] :=  ""
   _HMG_aControlContainerHandle  [k] :=  0
   _HMG_aControlFontName   [k] := fontname
   _HMG_aControlFontSize   [k] := fontsize
   _HMG_aControlFontAttributes   [k] := {bold,italic,underline,strikeout}
   _HMG_aControlToolTip  [k] :=   tooltip
   _HMG_aControlRangeMin   [k] :=  0
   _HMG_aControlRangeMax  [k] :=   0
   _HMG_aControlCaption   [k] :=  Caption
   _HMG_aControlVisible  [k] :=   if(invisible,FALSE,TRUE)
   _HMG_aControlHelpId  [k] :=   HelpId
   _HMG_aControlFontHandle   [k] :=  FontHandle
   _HMG_aControlBrushHandle   [k] :=  0
   _HMG_aControlEnabled  [k] :=  .T.
   _HMG_aControlMiscData1 [k] := 2

   if value = .t. .and. .Not. lDialogInMemory
      SendMessage( Controlhandle , BM_SETCHECK , BST_CHECKED , 0 )
   endif

Return Nil

*-----------------------------------------------------------------------------*
Function InitDialogCheckButton ( ParentName, ControlHandle, k )
*-----------------------------------------------------------------------------*
   Local Value , BitMap , threestate

   BitMap:= _HMG_aControlPicture [k]
   Value := _HMG_aControlValue [k]
   threestate := _HMG_aControlSpacing [k]
   If !empty(BitMap) .and. valtype(ParentName) <> 'U'
      _SetBtnPicture(ControlHandle, BitMap)
   endif
   if value = .T.
      SendMessage( Controlhandle , BM_SETCHECK , BST_CHECKED , 0 )
   elseif threestate .and. value == NIL
      SendMessage( Controlhandle , BM_SETCHECK , BST_INDETERMINATE , 0 )
   endif
// JP 62
   if len(_HMG_aDialogTemplate) != 0 .and. _HMG_aDialogTemplate[3]   // Modal
      _HMG_aControlDeleted [k] := .T.
   endif

Return Nil

*-----------------------------------------------------------------------------*
Function _DefineImageCheckButton ( ControlName, ParentFormName, x, y, BitMap,;
   Value, fontname, fontsize, tooltip,;
   changeprocedure, w, h, lostfocus, gotfocus,;
   HelpId, invisible, notabstop, nId )
*-----------------------------------------------------------------------------*
   Local ParentFormHandle , blInit , mVar , k , Style
   Local ControlHandle , nhImage , aRet //[2]
   Local lDialogInMemory

   DEFAULT value           TO FALSE
   DEFAULT w               TO 100
   DEFAULT h               TO 28
   DEFAULT lostfocus       TO ""
   DEFAULT gotfocus        TO ""
   DEFAULT changeprocedure TO ""
   DEFAULT invisible       TO FALSE
   DEFAULT notabstop       TO FALSE

   if _HMG_BeginWindowActive = TRUE .or. _HMG_BeginDialogActive = TRUE
      ParentFormName := if(_HMG_BeginDialogActive,_HMG_ActiveDialogName,_HMG_ActiveFormName)
   endif
   if _HMG_FrameLevel > 0
      x    := x + _HMG_ActiveFrameCol [_HMG_FrameLevel]
      y    := y + _HMG_ActiveFrameRow [_HMG_FrameLevel]
      ParentFormName := _HMG_ActiveFrameParentFormName [_HMG_FrameLevel]
   EndIf
   lDialogInMemory := _HMG_DialogInMemory // ( GetWindowType ( ParentFormName ) == "L" )

   If .Not. _IsWindowDefined (ParentFormName) .and. .Not. lDialogInMemory
      MsgMiniGuiError("Window: "+ ParentFormName + " is not defined. Program terminated" )
   Endif

   If _IsControlDefined (ControlName,ParentFormName) .and. .Not. lDialogInMemory
      MsgMiniGuiError ("Control: " + ControlName + " Of " + ParentFormName + " Already defined. Program Terminated" )
   endif

   mVar := '_' + ParentFormName + '_' + ControlName
   k := _GetControlFree()

   if _HMG_BeginDialogActive = TRUE

      ParentFormHandle = _HMG_ActiveDialogHandle

      Style := BS_NOTIFY + BS_BITMAP + WS_CHILD + BS_AUTOCHECKBOX + BS_PUSHLIKE
      if !NoTabStop
         Style += WS_TABSTOP
      endif
      if !invisible
         Style += WS_VISIBLE
      endif

      if lDialogInMemory         //Dialog Template

//          {{ID,k/hwnd,class,Style,ExStyle,x,y,w,h,caption,HelpId,tooltip,font,size, bold, italic, underline, strikeout}}  --->_HMG_aDialogItems
         blInit:= {|x,y,z| InitDialogCheckButton(x,y,z) }
         aadd(_HMG_aDialogItems,{nId,k,"button",style,0,x,y,w,h,"",HelpId,tooltip,FontName,FontSize,,,,,blInit,_HMG_BeginTabActive,.f.,_HMG_ActiveTabPage })
/* JP 62
         if _HMG_aDialogTemplate[3]   // Modal
            Return Nil
         endif
*/
      else

         ControlHandle := GetDialogItemHandle( ParentFormHandle, nId )

         x := GetWindowCol ( Controlhandle )
         y := GetWindowRow ( Controlhandle )
         w := GetWindowWidth  ( Controlhandle )
         h := GetWindowHeight ( Controlhandle )

         SetWindowStyle ( ControlHandle, Style )

         _SetBtnPicture(ControlHandle,bitmap)
      endif

   else

      ParentFormHandle = GetFormHandle (ParentFormName)

      aRet := InitImageCheckButton ( ParentFormHandle, "", 0, x, y, '', 0 , bitmap , w , h, invisible, notabstop )

      ControlHandle := aRet [1]
      nhImage := aRet [2]

   endif

   if .Not.  lDialogInMemory
      If _HMG_BeginTabActive = .T.
         aAdd ( _HMG_ActiveTabCurrentPageMap , ControlHandle )
      EndIf

      if valtype(tooltip) != "U"
         SetToolTip ( ControlHandle , tooltip , GetFormToolTipHandle (ParentFormName) )
      endif

   Endif

   Public &mVar. := k

   _HMG_aControlType [k] := "CHECKBOX"
   _HMG_aControlNames [k] :=   ControlName
   _HMG_aControlHandles [k] :=   ControlHandle
   _HMG_aControlParenthandles [k] :=   ParentFormHandle
   _HMG_aControlIds [k] :=   nId
   _HMG_aControlProcedures [k] :=   ""
   _HMG_aControlPageMap [k] :=   {}
   _HMG_aControlValue [k] :=   Value
   _HMG_aControlInputMask  [k] :=  ""
   _HMG_aControllostFocusProcedure  [k] :=  lostfocus
   _HMG_aControlGotFocusProcedure  [k] :=  gotfocus
   _HMG_aControlChangeProcedure  [k] :=  changeprocedure
   _HMG_aControlDeleted  [k] :=  .F.
   _HMG_aControlBkColor  [k] :=  Nil
   _HMG_aControlFontColor  [k] :=  Nil
   _HMG_aControlDblClick  [k] :=  ""
   _HMG_aControlHeadClick  [k] :=  {}
   _HMG_aControlRow   [k] :=  y
   _HMG_aControlCol   [k] :=  x
   _HMG_aControlWidth   [k] :=  w
   _HMG_aControlHeight  [k] :=  h
   _HMG_aControlSpacing [k] :=  .f.
   _HMG_aControlContainerRow  [k] :=  iif ( _HMG_FrameLevel > 0 ,_HMG_ActiveFrameRow [_HMG_FrameLevel] , -1 )
   _HMG_aControlContainerCol  [k] :=  iif ( _HMG_FrameLevel > 0 ,_HMG_ActiveFrameCol [_HMG_FrameLevel] , -1 )
   _HMG_aControlPicture  [k] :=  BitMap
   _HMG_aControlContainerHandle  [k] :=  0
   _HMG_aControlFontName  [k] :=  fontname
   _HMG_aControlFontSize  [k] :=  fontsize
   _HMG_aControlFontAttributes  [k] :=  {.f.,.f.,.f.,.f.}
   _HMG_aControlToolTip  [k] :=   tooltip
   _HMG_aControlRangeMin   [k] :=  0
   _HMG_aControlRangeMax   [k] :=  0
   _HMG_aControlCaption  [k] :=   ''
   _HMG_aControlVisible  [k] :=   if(invisible,FALSE,TRUE)
   _HMG_aControlHelpId  [k] :=   HelpId
   _HMG_aControlFontHandle  [k] :=   0
   _HMG_aControlBrushHandle  [k] :=  nhImage
   _HMG_aControlEnabled  [k] :=  .T.
   _HMG_aControlMiscData1 [k] := 1

   if value = .t. .and. .Not. lDialogInMemory
      SendMessage( Controlhandle , BM_SETCHECK , BST_CHECKED , 0 )
   endif

Return Nil

Procedure _DataCheckBoxRefresh (i)
   Local Field

   Field := _HMG_aControlPageMap [i]
   _SetValue ( '' , '' , &Field , i )

Return

Procedure _DataCheckBoxSave ( ControlName , ParentFormName )
   Local Field , i

   i := GetControlIndex ( ControlName , ParentFormName )

   Field := _HMG_aControlPageMap [i]

   REPLACE &Field WITH _GetValue ( Controlname , ParentFormName )

Return
