/*----------------------------------------------------------------------------
MINIGUI - Harbour Win32 GUI library source code

Copyright 2002-2008 Roberto Lopez <harbourminigui@gmail.com>
http://harbourminigui.googlepages.com/

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along with
   this software; see the file COPYING. If not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA (or
   visit the web site http://www.gnu.org/).

   As a special exception, you have permission for additional uses of the text
   contained in this release of Harbour Minigui.

   The exception is that, if you link the Harbour Minigui library with other
   files to produce an executable, this does not by itself cause the resulting
   executable to be covered by the GNU General Public License.
   Your use of that executable is in no way restricted on account of linking the
   Harbour-Minigui library code into it.

   Parts of this project are based upon:

   "Harbour GUI framework for Win32"
   Copyright 2001 Alexander S.Kresin <alex@belacy.belgorod.su>
   Copyright 2001 Antonio Linares <alinares@fivetech.com>
   www - http://www.harbour-project.org

   "Harbour Project"
   Copyright 1999-2008, http://www.harbour-project.org/

   "WHAT32"
   Copyright 2002 AJ Wos <andrwos@aust1.net>

   "HWGUI"
   Copyright 2001-2007 Alexander S.Kresin <alex@belacy.belgorod.su>

---------------------------------------------------------------------------*/

#include "minigui.ch"

#define WS_TABSTOP          0x00010000
#define WS_VISIBLE          0x10000000
#define WS_CHILD            0x40000000
#define WS_VSCROLL          0x200000
#define CBS_DROPDOWN          0x0002
#define CBS_DROPDOWNLIST      0x0003
#define CBS_NOINTEGRALHEIGHT  0x0400
#define CBS_SORT              0x0100

#define CBEM_GETCOMBOCONTROL  1030
#define CBEM_GETEDITCONTROL   1031

*-----------------------------------------------------------------------------*
Function _DefineCombo ( ControlName, ParentFormName, x, y, w, rows, value,;
   fontname, fontsize, tooltip, changeprocedure, h, gotfocus,;
   lostfocus, uEnter, HelpId, invisible, notabstop, sort , bold,;
   italic, underline, strikeout , itemsource , valuesource , displaychange ,;
   ondisplaychangeprocedure , break , GripperText , ListWidth , nId,;
   OnListDisplayProcedure, OnListCloseProcedure, backcolor , fontcolor )
*-----------------------------------------------------------------------------*
   Local i , ParentFormHandle , mVar , ControlHandle , FontHandle , WorkArea , cField , ContainerHandle := 0 , k
   Local Style, blInit, aFont
   Local lDialogInMemory

   DEFAULT w               TO 120
   DEFAULT h               TO 150
   DEFAULT changeprocedure TO ""
   DEFAULT gotfocus     TO ""
   DEFAULT lostfocus    TO ""
   DEFAULT rows         TO {}
   DEFAULT invisible    TO FALSE
   DEFAULT notabstop    TO FALSE
   DEFAULT sort      TO FALSE
   DEFAULT GripperText   TO ""
   DEFAULT  ListWidth   TO w

   if ( FontHandle := GetFontHandle( FontName ) ) != 0
      aFont := GetFontParam(FontHandle)
      FontName      := aFont[1]
      FontSize      := aFont[2]
      bold          := aFont[3]
      italic        := aFont[4]
      underline     := aFont[5]
      strikeout     := aFont[6]
   endif

   if _HMG_BeginWindowActive = TRUE .or. _HMG_BeginDialogActive = TRUE
      ParentFormName := if(_HMG_BeginDialogActive,_HMG_ActiveDialogName,_HMG_ActiveFormName)
      if .Not. Empty (_HMG_ActiveFontName) .And. ValType(FontName) == "U"
         FontName := _HMG_ActiveFontName
      EndIf
      if .Not. Empty (_HMG_ActiveFontSize) .And. ValType(FontSize) == "U"
         FontSize := _HMG_ActiveFontSize
      EndIf
   endif

   if _HMG_FrameLevel > 0
      x    := x + _HMG_ActiveFrameCol [_HMG_FrameLevel]
      y    := y + _HMG_ActiveFrameRow [_HMG_FrameLevel]
      ParentFormName := _HMG_ActiveFrameParentFormName [_HMG_FrameLevel]
   EndIf
   lDialogInMemory := _HMG_DialogInMemory // ( GetWindowType ( ParentFormName ) == "L" )

   If .Not. _IsWindowDefined (ParentFormName) .and. .Not. lDialogInMemory
      MsgMiniGuiError("Window: "+ ParentFormName + " is not defined. Program terminated" )
   Endif

   If _IsControlDefined (ControlName,ParentFormName) .and. .Not. lDialogInMemory
      MsgMiniGuiError ("Control: " + ControlName + " Of " + ParentFormName + " Already defined. Program Terminated" )
   endif

   if ValType ( ItemSource ) != 'U' .And. Sort == .T.
      MsgMiniGuiError ("Sort and ItemSource clauses can't be used simultaneously. Program Terminated" )
   EndIf

   if ValType ( ValueSource ) != 'U' .And. Sort == .T.
      MsgMiniGuiError ("Sort and ValueSource clauses can't be used simultaneously. Program Terminated" )
   EndIf

   if valtype ( itemsource ) != 'U'
      if  at ( '>',ItemSource ) == 0
         MsgMiniGuiError ("Control: " + ControlName + " Of " + ParentFormName + " (ItemSource): You must specify a fully qualified field name. Program Terminated" )
      Else
         WorkArea := Left ( ItemSource , at ( '>', ItemSource ) - 2 )
         cField := Right ( ItemSource , Len (ItemSource) - at ( '>', ItemSource ) )
      EndIf
   EndIf

   if valtype(value) == "U"
      value := 0
   endif

   if valtype(uEnter) == "U"
      uEnter := ""
   endif

   mVar := '_' + ParentFormName + '_' + ControlName
   k := _GetControlFree()

   if _HMG_BeginDialogActive = TRUE
      ParentFormHandle = _HMG_ActiveDialogHandle

      Style := WS_CHILD + WS_VSCROLL

      if !NoTabStop
         Style += WS_TABSTOP
      endif

      if !invisible
         Style += WS_VISIBLE
      endif

      if sort
         Style += CBS_SORT
      endif

      if !displaychange
         Style += CBS_DROPDOWNLIST
      else
         Style += CBS_DROPDOWN
      endif

      if valtype(_HMG_IsXP) == "L"
         if _HMG_IsXP
            Style += CBS_NOINTEGRALHEIGHT
         endif
      endif

      if lDialogInMemory         //Dialog Template

//          {{'ID',k/hwnd,class,Style,ExStyle,x,y,w,h,caption,HelpId,tooltip,font,size, bold, italic, underline, strikeout}}  --->_HMG_aDialogItems
         blInit:= {|x,y,z| InitDialogComboBox(x,y,z) }
         aadd(_HMG_aDialogItems,{nId,k,"COMBOBOX",style,0,x,y,w,h,"",HelpId,tooltip,FontName,FontSize,bold,italic,underline,strikeout,blInit,_HMG_BeginTabActive,.f.,_HMG_ActiveTabPage })
/* JP 62
         if _HMG_aDialogTemplate[3]   // Modal
            _HMG_aControlDeleted [k] := .T.
            Return Nil
         endif
*/
      else

         ControlHandle := GetDialogItemHandle( ParentFormHandle, nId )

         x := GetWindowCol ( Controlhandle )
         y := GetWindowRow ( Controlhandle )
         w := GetWindowWidth  ( Controlhandle )
         h := GetWindowHeight ( Controlhandle )

         If FontHandle != 0
            _SetFontHandle(ControlHandle,FontHandle)
         Else
            if valtype(fontname) == "U"
               Fontname := _HMG_DefaultFontName
            endif
            if valtype(fontsize) == "U"
               FontSize := _HMG_DefaultFontSize
            endif
            FontHandle := _SetFont (ControlHandle,FontName,FontSize,bold,italic,underline,strikeout)
         endif

         SetWindowStyle ( ControlHandle, Style )

      endif

   else

      ParentFormHandle = GetFormHandle (ParentFormName)

      if valtype(x) == "U" .or. valtype(y) == "U"

         _HMG_SplitLastControl   := 'COMBOBOX'

         i := GetFormIndex ( ParentFormName )

         If i > 0

            ControlHandle := InitComboBox ( _HMG_aFormReBarHandle [i], 0, x, y, w, '', 0 , h, invisible, notabstop, sort, displaychange , _HMG_IsXPorLater )

            If FontHandle != 0
               _SetFontHandle(ControlHandle,FontHandle)
            Else
               if valtype(fontname) == "U"
                  Fontname := _HMG_DefaultFontName
               endif
               if valtype(fontsize) == "U"
                  FontSize := _HMG_DefaultFontSize
               endif
               FontHandle := _SetFont (ControlHandle,FontName,FontSize,bold,italic,underline,strikeout)
            endif

            AddSplitBoxItem ( Controlhandle , _HMG_aFormReBarHandle [i] , w , break , GripperText , w , , _HMG_ActiveSplitBoxInverted )

            Containerhandle := _HMG_aFormReBarHandle [i]

         EndIf

      else

         ControlHandle := InitComboBox ( ParentFormHandle, 0, x, y, w, '', 0 , h, invisible, notabstop, sort , displaychange , _HMG_IsXPorLater )

      endif

   endif

   if .Not. lDialogInMemory

      If FontHandle != 0
         _SetFontHandle(ControlHandle,FontHandle)
      Else
         if valtype(fontname) == "U"
            Fontname := _HMG_DefaultFontName
         endif
         if valtype(fontsize) == "U"
            FontSize := _HMG_DefaultFontSize
         endif
         FontHandle := _SetFont (ControlHandle,FontName,FontSize,bold,italic,underline,strikeout)
      endif

      If _HMG_BeginTabActive = TRUE
         aAdd ( _HMG_ActiveTabCurrentPageMap , Controlhandle )
      EndIf

      if valtype(tooltip) != "U"
         SetToolTip ( ControlHandle , tooltip , GetFormToolTipHandle (ParentFormName) )
      endif

   endif

   Public &mVar. := k

   _HMG_aControlType [k] := "COMBO"
   _HMG_aControlNames  [k] :=  ControlName
   _HMG_aControlHandles  [k] :=  ControlHandle
   _HMG_aControlParenthandles  [k] :=  ParentFormHandle
   _HMG_aControlIds  [k] :=  nId
   _HMG_aControlProcedures  [k] :=  ondisplaychangeprocedure
   _HMG_aControlPageMap  [k] :=  cField
   _HMG_aControlValue  [k] :=  Value
   _HMG_aControlInputMask   [k] := OnListDisplayProcedure
   _HMG_aControllostFocusProcedure  [k] :=  lostfocus
   _HMG_aControlGotFocusProcedure  [k] :=  gotfocus
   _HMG_aControlChangeProcedure  [k] :=  changeprocedure
   _HMG_aControlDeleted   [k] := FALSE
   _HMG_aControlBkColor  [k] :=  backcolor
   _HMG_aControlFontColor  [k] :=  fontcolor
   _HMG_aControlDblClick  [k] :=  uEnter
   _HMG_aControlHeadClick  [k] := {}
   _HMG_aControlRow   [k] := y
   _HMG_aControlCol   [k] := x
   _HMG_aControlWidth   [k] := w
   _HMG_aControlHeight   [k] := h
   _HMG_aControlSpacing   [k] := WorkArea
   _HMG_aControlContainerRow  [k] :=  iif ( _HMG_FrameLevel > 0 ,_HMG_ActiveFrameRow [_HMG_FrameLevel] , -1 )
   _HMG_aControlContainerCol  [k] :=  iif ( _HMG_FrameLevel > 0 ,_HMG_ActiveFrameCol [_HMG_FrameLevel] , -1 )
   _HMG_aControlPicture  [k] :=  OnListCloseProcedure
   _HMG_aControlContainerHandle  [k] :=  ContainerHandle
   _HMG_aControlFontName  [k] :=  fontname
   _HMG_aControlFontSize  [k] :=  fontsize
   _HMG_aControlFontAttributes  [k] :=  {bold,italic,underline,strikeout}
   _HMG_aControlToolTip   [k] :=  tooltip
   _HMG_aControlRangeMin  [k] :=   0
   _HMG_aControlRangeMax  [k] :=   0
   _HMG_aControlCaption   [k] :=  valuesource
   _HMG_aControlVisible  [k] :=   if(invisible,FALSE,TRUE)
   _HMG_aControlHelpId  [k] :=  HelpId
   _HMG_aControlFontHandle  [k] :=   FontHandle
   _HMG_aControlBrushHandle  [k] :=   0
   _HMG_aControlEnabled  [k] :=   .T.
   _HMG_aControlMiscData1 [k] := {0, DisplayChange, ItemSource, rows, ListWidth}

   if len(_HMG_aDialogTemplate) == 0
      InitDialogComboBox(ParentFormName, ControlHandle, k)
   endif

Return Nil

*-----------------------------------------------------------------------------*
Function InitDialogComboBox(ParentName, ControlHandle, k)
*-----------------------------------------------------------------------------*
   Local WorkArea , BackRec , rcount := 0 , cset := 0 , Value , rows , DisplayChange , ItemSource, cField,i, ListWidth

   WorkArea        := _HMG_aControlSpacing   [k]
   cField          := _HMG_aControlPageMap  [k]
   Value           := _HMG_aControlValue  [k]
   rows            := _HMG_aControlMiscData1 [k,4]
   DisplayChange   := _HMG_aControlMiscData1 [k,2]
   ItemSource      := _HMG_aControlMiscData1 [k,3]
   ListWidth       := _HMG_aControlMiscData1 [k,5]

   If DisplayChange == .T.
      _hmg_acontrolrangemin [k] := FindWindowEx( Controlhandle , 0, "Edit", Nil )
        // add tooltip for editable combo window if defined //(JK) HMG Exp. Build 8
      if valtype(_HMG_aControlToolTip   [k]) != "U"
         SetToolTip ( _hmg_acontrolrangemin [k] , _HMG_aControlToolTip   [k] , GetFormToolTipHandle (ParentName) )
      endif

   EndIf

   SETDROPDOWNWIDTH( ControlHandle , ListWidth )

   If  ValType( WorkArea ) == "C"

      If Select ( WorkArea ) != 0

         BackRec := (WorkArea)->(RecNo())

         (WorkArea)->(DBGoTop())

         Do While ! (WorkArea)->(Eof())
            rcount++
            if value == (WorkArea)->(RecNo())
               cset := rcount
            EndIf
            ComboAddString (ControlHandle, (WorkArea)->&(cField))
            (WorkArea)->(DBSkip())
         EndDo

         (WorkArea)->(DBGoTo(BackRec))

         ComboSetCurSel (ControlHandle, cset)

      EndIf

   Else

      for i = 1 to len (rows)
         ComboAddString (ControlHandle, rows[i])
      next

      if value <> 0
         ComboSetCurSel (ControlHandle, Value)
      endif

   EndIf

   if valtype ( ItemSource ) != 'U'
      aAdd ( _HMG_aFormBrowseList [ GetFormIndex ( ParentName ) ] , k )
   EndIf
// JP 62
   if len(_HMG_aDialogTemplate) != 0 .and. _HMG_aDialogTemplate[3]   // Modal
      _HMG_aControlDeleted [k] := .T.
   endif

Return Nil

// (JK) Extend combobox control -  COMBOBOXEX
// HMG 1.0 Experimental Build 8
*-----------------------------------------------------------------------------*
Function _DefineComboEx (;
   ControlName, ; // 1
   ParentForm, ;  // 2
   x, ;           // 3
   y, ;           // 4
   w, ;           // 5
   rows, ;         // 6
   value, ;        // 7
   fontname, ;     // 8
   fontsize, ;     // 9
   tooltip, ;         // 10
   changeprocedure, ; //11
   h, ;               // 12
   gotfocus, ;        // 13
   lostfocus, ;       // 14
   uEnter, ;          // 15
   HelpId, ;          // 16
   invisible, ;       // 17
   notabstop, ;       // 18
   sort , ;           // 19   // not used with extend COMBO
   bold, ;            // 20
   italic, ;          // 21
   underline, ;         // 22
   strikeout , ;        // 23
   itemsource , ;       // 24
   valuesource , ;      // 25
   displaychange , ;    // 26
   ondisplaychangeprocedure , ; // 27
   break , ;            // 28
   GripperText, ;       // 29
   aImage , ;           // 30
   ListWidth,;
      OnListDisplayProcedure,;
      OnListCloseProcedure, backcolor , fontcolor, ImageList )
*-----------------------------------------------------------------------------*
   Local i , cParentForm , mVar , ControlHandle , FontHandle , rcount := 0 , BackRec , cset := 0 , WorkArea , cField , ContainerHandle := 0 , k
   Local im , aImages := {} , aFont

   DEFAULT w               TO 120
   DEFAULT h               TO 150
   DEFAULT changeprocedure TO ""
   DEFAULT gotfocus     TO ""
   DEFAULT lostfocus    TO ""
   DEFAULT rows         TO {}
   DEFAULT invisible    TO FALSE
   DEFAULT notabstop    TO FALSE
   DEFAULT sort      TO FALSE
   DEFAULT GripperText   TO ""
   DEFAULT aImage TO {}
   DEFAULT ListWidth TO w
   DEFAULT ImageList    TO 0

   if ( FontHandle := GetFontHandle( FontName ) ) != 0
      aFont := GetFontParam(FontHandle)
      FontName      := aFont[1]
      FontSize      := aFont[2]
      bold          := aFont[3]
      italic        := aFont[4]
      underline     := aFont[5]
      strikeout     := aFont[6]
   endif

   if _HMG_BeginWindowActive = TRUE
      ParentForm := _HMG_ActiveFormName
      if .Not. Empty (_HMG_ActiveFontName) .And. ValType(FontName) == "U"
         FontName := _HMG_ActiveFontName
      EndIf
      if .Not. Empty (_HMG_ActiveFontSize) .And. ValType(FontSize) == "U"
         FontSize := _HMG_ActiveFontSize
      EndIf
   endif

   if _HMG_FrameLevel > 0
      x    := x + _HMG_ActiveFrameCol [_HMG_FrameLevel]
      y    := y + _HMG_ActiveFrameRow [_HMG_FrameLevel]
      ParentForm := _HMG_ActiveFrameParentFormName [_HMG_FrameLevel]
   EndIf

   If .Not. _IsWindowDefined (ParentForm)
      MsgMiniGuiError("Window: "+ ParentForm + " is not defined. Program terminated" )
   Endif

   If _IsControlDefined (ControlName,ParentForm)
      MsgMiniGuiError ("Control: " + ControlName + " Of " + ParentForm + " Already defined. Program Terminated" )
   endif

   if valtype ( itemsource ) != 'U'
      if  at ( '>',ItemSource ) == 0
         MsgMiniGuiError ("Control: " + ControlName + " Of " + ParentForm + " (ItemSource): You must specify a fully qualified field name. Program Terminated" )
      Else
         WorkArea := Left ( ItemSource , at ( '>', ItemSource ) - 2 )
         cField   := Right ( ItemSource , Len (ItemSource) - at ( '>', ItemSource ) )
      EndIf
   EndIf

   if valtype(value) == "U"
      value := 0
   endif

   if (ValType(ImageList) == "C" .or. ImageList > 0) .And. Len(aImage) > 0
      MsgMiniGuiError ("Control: " + ControlName + " Of " + ParentForm + " : Image and ImageList clauses can't be used simultaneously. Program Terminated" )
   EndIf

   IF ValType(ImageList) == "C"
      IF Len(ImageList) > 0
         ImageList := GetControlHandle ( ImageList, ParentForm )
      ENDIF
   endif


   mVar := '_' + ParentForm + '_' + ControlName

   cParentForm := ParentForm

   ParentForm = GetFormHandle (ParentForm)

   if !EMPTY(aImage)
      aImages:={}
      For im = 1 to len(aImage)
         If VALTYPE(aImage[im])=="C"
            AADD(aImages,aImage[im])
            AADD(aImages,aImage[im])
            AADD(aImages,aImage[im])

         elseif VALTYPE(aImage[im])=="A"
            If len(aImage[im])<>2
               MsgMiniGuiError ("Control: " + ControlName + " Of " + cParentForm + " : You must specify 2 elements characters array as image param. Program Terminated" )
            else
               AADD(aImages,aImage[im,1])
               AADD(aImages,aImage[im,2])
               AADD(aImages,aImage[im,1])
            endif
         endif
      next im
   endif

   if valtype(x) == "U" .or. valtype(y) == "U"

      _HMG_SplitLastControl   := 'COMBOBOX'

      i := GetFormIndex ( cParentForm )

      If i > 0

         ControlHandle := InitComboBoxEx ( _HMG_aFormReBarHandle [i], 0, x, y, w, '', 0 , h, invisible, notabstop, .f., displaychange, _HMG_IsXPorLater, aImages, ImageList )

         If FontHandle != 0
            _SetFontHandle(ControlHandle,FontHandle)
         Else
            if valtype(fontname) == "U"
               Fontname := _HMG_DefaultFontName
            endif
            if valtype(fontsize) == "U"
               FontSize := _HMG_DefaultFontSize
            endif
            FontHandle := _SetFont (ControlHandle,FontName,FontSize,bold,italic,underline,strikeout)
         endif

         AddSplitBoxItem ( Controlhandle , _HMG_aFormReBarHandle [i] , w , break , GripperText , w , , _HMG_ActiveSplitBoxInverted )

         Containerhandle := _HMG_aFormReBarHandle [i]

      EndIf

   else

      ControlHandle := InitComboBoxEx ( ParentForm, 0, x, y, w, '', 0 , h, invisible, notabstop, .f. , displaychange, _HMG_IsXPorLater, aImages, ImageList )

      If FontHandle != 0
         _SetFontHandle(ControlHandle,FontHandle)
      Else
         if valtype(fontname) == "U"
            Fontname := _HMG_DefaultFontName
         endif
         if valtype(fontsize) == "U"
            FontSize := _HMG_DefaultFontSize
         endif
         FontHandle := _SetFont (ControlHandle,FontName,FontSize,bold,italic,underline,strikeout)
      endif

   endif

   If _HMG_BeginTabActive = TRUE
      aAdd ( _HMG_ActiveTabCurrentPageMap , Controlhandle )
   EndIf

   if valtype(uEnter) == "U"
      uEnter := ""
   endif

   if valtype(tooltip) != "U"
      SetToolTip ( ControlHandle , tooltip , GetFormToolTipHandle (cParentForm) )
   endif

   if len( aImages ) > 0
      AddListViewBitmap( ControlHandle, aImages )
   endif

   k := _GetControlFree()

   Public &mVar. := k

   _HMG_aControlType [k] := "COMBO"
   _HMG_aControlNames  [k] :=  ControlName
   _HMG_aControlHandles  [k] :=  ControlHandle
   _HMG_aControlParenthandles  [k] :=  ParentForm
   _HMG_aControlIds  [k] :=  0
   _HMG_aControlProcedures  [k] :=  ondisplaychangeprocedure
   _HMG_aControlPageMap  [k] :=  cField
   _HMG_aControlValue  [k] :=  Value
   _HMG_aControlInputMask   [k] := OnListDisplayProcedure
   _HMG_aControllostFocusProcedure  [k] :=  lostfocus
   _HMG_aControlGotFocusProcedure  [k] :=  gotfocus
   _HMG_aControlChangeProcedure  [k] :=  changeprocedure
   _HMG_aControlDeleted   [k] := FALSE
   _HMG_aControlBkColor  [k] :=  backcolor
   _HMG_aControlFontColor  [k] :=  fontcolor
   _HMG_aControlDblClick  [k] :=  uEnter
   _HMG_aControlHeadClick  [k] := {}
   _HMG_aControlRow   [k] := y
   _HMG_aControlCol   [k] := x
   _HMG_aControlWidth   [k] := w
   _HMG_aControlHeight   [k] := h
   _HMG_aControlSpacing   [k] := WorkArea
   _HMG_aControlContainerRow  [k] :=  iif ( _HMG_FrameLevel > 0 ,_HMG_ActiveFrameRow [_HMG_FrameLevel] , -1 )
   _HMG_aControlContainerCol  [k] :=  iif ( _HMG_FrameLevel > 0 ,_HMG_ActiveFrameCol [_HMG_FrameLevel] , -1 )
   _HMG_aControlPicture  [k] :=  OnListCloseProcedure
   _HMG_aControlContainerHandle  [k] :=  ContainerHandle
   _HMG_aControlFontName  [k] :=  fontname
   _HMG_aControlFontSize  [k] :=  fontsize
   _HMG_aControlFontAttributes  [k] :=  {bold,italic,underline,strikeout}
   _HMG_aControlToolTip   [k] :=  tooltip
   _HMG_aControlRangeMin  [k] :=   0
   _HMG_aControlRangeMax  [k] :=   0
   _HMG_aControlCaption   [k] :=  valuesource
   _HMG_aControlVisible  [k] :=   if(invisible,FALSE,TRUE)
   _HMG_aControlHelpId  [k] :=  HelpId
   _HMG_aControlFontHandle  [k] :=   FontHandle
   _HMG_aControlBrushHandle  [k] :=   0
   _HMG_aControlEnabled  [k] :=   .T.
   _HMG_aControlMiscData1 [k] := {1}     // value used for recognintion between extend and std. COMBO

   If DisplayChange == .T.

      // handle for ComboBoxEx edit window
      _hmg_acontrolrangemin [k] := SendMessage ( Controlhandle, CBEM_GETEDITCONTROL, 0, 0 )
      if valtype(tooltip) != "U"
         SetToolTip ( _hmg_acontrolrangemin [k] , tooltip , GetFormToolTipHandle (cParentForm) )
      endif
   endif

   // handle for ComboBoxEx child window
   _hmg_acontrolrangemax [k] := SendMessage ( Controlhandle, CBEM_GETCOMBOCONTROL, 0, 0 )
   if valtype(tooltip) != "U"
      SetToolTip ( _hmg_acontrolrangemax [k] , tooltip , GetFormToolTipHandle (cParentForm) )
   endif

   SETDROPDOWNWIDTH( _hmg_acontrolrangemax [k] , ListWidth )

   If  ValType( WorkArea ) == "C"

      If Select( WorkArea ) != 0

         BackRec := (WorkArea)->(RecNo())

         (WorkArea)->(DBGoTop())

         Do While ! (WorkArea)->(Eof())
            rcount++
            if value == (WorkArea)->(RecNo())
               cset := rcount
            EndIf
            ComboAddStringEx ( ControlHandle, (WorkArea)->&(cField), 1 )
            (WorkArea)->(DBSkip())
         EndDo

         (WorkArea)->(DBGoTo(BackRec))

         ComboSetCurSel (ControlHandle,cset)

      EndIf

   Else

      for i = 1 to len (rows)
         ComboAddStringEx (ControlHandle, rows[i], i)
      next

      if value <> 0
         ComboSetCurSel (ControlHandle,Value)
      endif

   EndIf

   if valtype ( ItemSource ) != 'U'
      aAdd ( _HMG_aFormBrowseList [ GetFormIndex ( cParentForm ) ] , k )
   EndIf

Return Nil

//------------------------------------------------
// (JK) Modified for extend COMBO HMG 1.0 Build 8
Procedure _DataComboRefresh (i)
//------------------------------------------------
   Local BackValue , BackRec , WorkArea , cField , ControlHandle

   If Empty ( _HMG_aControlCaption [i] )
      BackValue := _GetValue ( '' , '' , i )
   else
      cField := _HMG_aControlCaption [i]
      _HMG_aControlCaption [i] := ""
      BackValue := _GetValue ( '' , '' , i )
      _HMG_aControlCaption [i] := cField
   endif

   cField := _HMG_aControlPageMap [i]

   ControlHandle := _HMG_aControlHandles [i]

   WorkArea := _HMG_aControlSpacing [i]

   BackRec := (WorkArea)->(RecNo())

   (WorkArea)->(DBGoTop())

   ComboboxReset ( ControlHandle )

   Do While ! (WorkArea)->(Eof())

      // (JK) HMG 1.0 Experimental Build 8

      If  _HMG_aControlMiscData1 [i][1] <> 1  // standard Combo
         ComboAddString ( ControlHandle , (WorkArea)->&(cField) )
      else   // extend Combo
         ComboAddDataStringEx ( ControlHandle , (WorkArea)->&(cField) )
      endif

      (WorkArea)->(DBSkip())
   EndDo

   (WorkArea)->(DBGoTo(BackRec))

   If BackValue > 0 .And. BackValue <= (WorkArea)->(LastRec())
      _SetValue ( '' , '' , BackValue , i )
   EndIf

Return
//------------------------------------------------
// (JK) HMG 1.0 Experimental Build 8e
Function _GetDropDownWidth ( ControlName, ParentForm )
//------------------------------------------------
   Local nWidth, i

   If _IsComboExtend( ControlName, ParentForm )
      i := _hmg_acontrolrangemax [ GetControlIndex ( ControlName, ParentForm ) ]
   else
      i := GetControlHandle ( ControlName, ParentForm )
   endif

   nWidth := GETDROPDOWNWIDTH(i)

Return nWidth
//------------------------------------------------
// (JK) HMG 1.0 Experimental Build 8e
Function _SetDropDownWidth ( ControlName, ParentForm, nWidth )
//------------------------------------------------
   Local i

   If _IsComboExtend( ControlName, ParentForm )
      i := _hmg_acontrolrangemax [ GetControlIndex ( ControlName, ParentForm ) ]
   else
      i := GetControlHandle ( ControlName, ParentForm )
   endif

   SETDROPDOWNWIDTH(i, nWidth)

Return NIL