/*----------------------------------------------------------------------------
MINIGUI - Harbour Win32 GUI library source code

Copyright 2002-2008 Roberto Lopez <harbourminigui@gmail.com>
http://harbourminigui.googlepages.com/

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along with
   this software; see the file COPYING. If not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA (or
   visit the web site http://www.gnu.org/).

   As a special exception, you have permission for additional uses of the text
   contained in this release of Harbour Minigui.

   The exception is that, if you link the Harbour Minigui library with other
   files to produce an executable, this does not by itself cause the resulting
   executable to be covered by the GNU General Public License.
   Your use of that executable is in no way restricted on account of linking the
   Harbour-Minigui library code into it.

   Parts of this project are based upon:

   "Harbour GUI framework for Win32"
   Copyright 2001 Alexander S.Kresin <alex@belacy.belgorod.su>
   Copyright 2001 Antonio Linares <alinares@fivetech.com>
   www - http://www.harbour-project.org

   "Harbour Project"
   Copyright 1999-2008, http://www.harbour-project.org/

   "WHAT32"
   Copyright 2002 AJ Wos <andrwos@aust1.net>

   "HWGUI"
   Copyright 2001-2007 Alexander S.Kresin <alex@belacy.belgorod.su>

---------------------------------------------------------------------------*/

#include "minigui.ch"

#define WS_TABSTOP          0x00010000
#define WS_VISIBLE          0x10000000
#define WS_CHILD            0x40000000

#define DTS_UPDOWN          0x0001 // use UPDOWN instead of MONTHCAL
#define DTS_SHOWNONE        0x0002 // allow a NONE selection
#define DTS_RIGHTALIGN      0x0020 // right-align popup instead of left-align it

*-----------------------------------------------------------------------------*
Function _DefineDatePick ( ControlName, ParentFormName, x, y, w, h, value,;
   fontname, fontsize, tooltip, change, lostfocus,;
   gotfocus, shownone, updown, rightalign, HelpId,;
   invisible, notabstop , bold, italic, underline, strikeout ,;
   Field, Enter, backcolor, fontcolor, titlebkclr, titlefrclr, trlfontclr, cDateFormat, nId )
*-----------------------------------------------------------------------------*
   Local ParentFormHandle , mVar , k , Style , blInit
   Local ControlHandle , FontHandle , WorkArea , aFont
   Local lDialogInMemory

   DEFAULT value     TO ctod ('  /  /  ')
   DEFAULT w         TO 120
   DEFAULT h         TO 24
   DEFAULT change    TO ""
   DEFAULT lostfocus TO ""
   DEFAULT gotfocus  TO ""
   DEFAULT invisible TO FALSE
   DEFAULT notabstop TO FALSE

   if ( FontHandle := GetFontHandle( FontName ) ) != 0
      aFont := GetFontParam(FontHandle)
      FontName      := aFont[1]
      FontSize      := aFont[2]
      bold          := aFont[3]
      italic        := aFont[4]
      underline     := aFont[5]
      strikeout     := aFont[6]
   endif

   If ValType ( Field ) != 'U'
      if  at ( '>', Field ) == 0
         MsgMiniGuiError ("Control: " + ControlName + " Of " + ParentFormName + " : You must specify a fully qualified field name. Program Terminated" )
      Else
         WorkArea := Left ( Field , at ( '>', Field ) - 2 )
         If Select (WorkArea) != 0
            Value := &(Field)
         EndIf
      EndIf
   EndIf

   if _HMG_BeginWindowActive = TRUE .or. _HMG_BeginDialogActive = TRUE
      ParentFormName := if(_HMG_BeginDialogActive,_HMG_ActiveDialogName,_HMG_ActiveFormName)
      if .Not. Empty (_HMG_ActiveFontName) .And. ValType(FontName) == "U"
         FontName := _HMG_ActiveFontName
      EndIf
      if .Not. Empty (_HMG_ActiveFontSize) .And. ValType(FontSize) == "U"
         FontSize := _HMG_ActiveFontSize
      EndIf
   endif
   if _HMG_FrameLevel > 0
      x    := x + _HMG_ActiveFrameCol [_HMG_FrameLevel]
      y    := y + _HMG_ActiveFrameRow [_HMG_FrameLevel]
      ParentFormName := _HMG_ActiveFrameParentFormName [_HMG_FrameLevel]
   EndIf
   lDialogInMemory := _HMG_DialogInMemory // ( GetWindowType ( ParentFormName ) == "L" )

   If .Not. _IsWindowDefined (ParentFormName) .and. .Not. lDialogInMemory
      MsgMiniGuiError("Window: "+ ParentFormName + " is not defined. Program terminated" )
   Endif

   If _IsControlDefined (ControlName,ParentFormName) .and. .Not. lDialogInMemory
      MsgMiniGuiError ("Control: " + ControlName + " Of " + ParentFormName + " Already defined. Program terminated" )
   endif

   mVar := '_' + ParentFormName + '_' + ControlName
   k := _GetControlFree()

   if _HMG_BeginDialogActive = TRUE
      ParentFormHandle = _HMG_ActiveDialogHandle

      Style := WS_CHILD

      if shownone
         Style += DTS_SHOWNONE
      Endif

      if updown
         Style += DTS_UPDOWN
      Endif

      if rightalign
         Style += DTS_RIGHTALIGN
      Endif

      if !invisible
         Style += WS_VISIBLE
      Endif

      if !NoTabStop
         Style += WS_TABSTOP
      Endif

      if lDialogInMemory         //Dialog Template
         InitExCommonControls(1)

//           {{'ID',k/hwnd,class,Style,ExStyle,x,y,w,h,caption,HelpId,tooltip,font,size, bold, italic, underline, strikeout}}  --->_HMG_aDialogItems
         blInit:= {|x,y,z| InitDialogDatePicker(x,y,z) }
         aadd(_HMG_aDialogItems,{nId,k,"SysDateTimePick32",style,0,x,y,w,h,"",HelpId,tooltip,FontName,FontSize,bold,italic,underline,strikeout,blInit,_HMG_BeginTabActive,.f.,_HMG_ActiveTabPage })
/* JP 62
         if _HMG_aDialogTemplate[3]   // Modal
            _HMG_aControlDeleted [k] := .T.
            Return Nil
         endif
*/
      else
         ControlHandle := GetDialogItemHandle( ParentFormHandle, nId )

         x := GetWindowCol ( Controlhandle )
         y := GetWindowRow ( Controlhandle )
         w := GetWindowWidth  ( Controlhandle )
         h := GetWindowHeight ( Controlhandle )

         SetWindowStyle ( ControlHandle, style )

      endif

   else

      ParentFormHandle = GetFormHandle (ParentFormName)

      ControlHandle := InitDatePick ( ParentFormHandle, 0, x, y, w, h , '' , 0 , shownone , updown , rightalign, invisible, notabstop )

   Endif

   if .Not. lDialogInMemory

      If FontHandle != 0
         _SetFontHandle(ControlHandle,FontHandle)
      Else
         if valtype(fontname) == "U"
            Fontname := _HMG_DefaultFontName
         endif
         if valtype(fontsize) == "U"
            FontSize := _HMG_DefaultFontSize
         endif
         FontHandle := _SetFont (ControlHandle,FontName,FontSize,bold,italic,underline,strikeout)
      endif

      If _HMG_BeginTabActive = .T.
         aAdd ( _HMG_ActiveTabCurrentPageMap , Controlhandle )
      EndIf

      If Empty (Value)
         SetDatePickNull (ControlHandle)
      Else
         SetDatePick( ControlHandle, year(value), month(value), day(value) )
      EndIf

      if valtype (tooltip) != "U"
         SetToolTip ( ControlHandle , tooltip , GetFormToolTipHandle (ParentFormName) )
      Endif

      if valtype ( Field ) != 'U'
         aAdd ( _HMG_aFormBrowseList [ GetFormIndex ( ParentFormName ) ] , k )
      EndIf

   Endif

   Public &mVar. := k

   _HMG_aControlType [k] := "DATEPICK"
   _HMG_aControlNames  [k] :=  ControlName
   _HMG_aControlHandles  [k] :=  ControlHandle
   _HMG_aControlParentHandles  [k] :=  ParentFormHandle
   _HMG_aControlIds  [k] :=  nId
   _HMG_aControlProcedures  [k] :=  Enter
   _HMG_aControlPageMap  [k] :=  Field
   _HMG_aControlValue  [k] :=  Value
   _HMG_aControlInputMask  [k] :=  ""
   _HMG_aControllostFocusProcedure  [k] :=  lostfocus
   _HMG_aControlGotFocusProcedure  [k] :=  gotfocus
   _HMG_aControlChangeProcedure  [k] :=  change
   _HMG_aControlDeleted  [k] :=  .F.
   _HMG_aControlBkColor  [k] :=  BackColor
   _HMG_aControlFontColor  [k] :=  FontColor
   _HMG_aControlDblClick  [k] :=  ""
   _HMG_aControlHeadClick [k] :=  {}
   _HMG_aControlRow   [k] :=  y
   _HMG_aControlCol   [k] :=  x
   _HMG_aControlWidth   [k] :=  w
   _HMG_aControlHeight   [k] :=  h
   _HMG_aControlSpacing   [k] :=  0
   _HMG_aControlContainerRow  [k] :=  iif ( _HMG_FrameLevel > 0 ,_HMG_ActiveFrameRow [_HMG_FrameLevel] , -1 )
   _HMG_aControlContainerCol  [k] :=  iif ( _HMG_FrameLevel > 0 ,_HMG_ActiveFrameCol [_HMG_FrameLevel] , -1 )
   _HMG_aControlPicture  [k] :=  ""
   _HMG_aControlContainerHandle  [k] :=  0
   _HMG_aControlFontName  [k] :=  fontname
   _HMG_aControlFontSize  [k] :=  fontsize
   _HMG_aControlFontAttributes  [k] :=  {bold,italic,underline,strikeout}
   _HMG_aControlToolTip  [k] :=  tooltip
   _HMG_aControlRangeMin [k] :=  0
   _HMG_aControlRangeMax [k] :=  0
   _HMG_aControlCaption  [k] :=  ''
   _HMG_aControlVisible  [k] :=  iif(invisible,FALSE,TRUE)
   _HMG_aControlHelpId   [k] :=  HelpId
   _HMG_aControlFontHandle  [k] :=  FontHandle
   _HMG_aControlBrushHandle [k] :=  0
   _HMG_aControlEnabled  [k] :=  .T.
   _HMG_aControlMiscData1 [k] := 0

   if BackColor <> Nil
      if BackColor[1] <> Nil .and. BackColor[2] <> Nil .and. BackColor[3] <> Nil
         SetDatePickBkColor(ControlHandle,BackColor[1],BackColor[2],BackColor[3])
      endif
   endif

   if FontColor <> Nil
      if FontColor[1] <> Nil .and. FontColor[2] <> Nil .and. FontColor[3] <> Nil
         SetDatePickFontColor(ControlHandle,FontColor[1],FontColor[2],FontColor[3])
      endif
   endif

   if TitleBkClr <> Nil
      if TitleBkClr[1] <> Nil .and. TitleBkClr[2] <> Nil .and. TitleBkClr[3] <> Nil
         SetDatePickTitleBkColor(ControlHandle,TitleBkClr[1],TitleBkClr[2],TitleBkClr[3])
      endif
   endif

   if TitleFrClr <> Nil
      if TitleFrClr[1] <> Nil .and. TitleFrClr[2] <> Nil .and. TitleFrClr[3] <> Nil
         SetDatePickTitleFontColor(ControlHandle,TitleFrClr[1],TitleFrClr[2],TitleFrClr[3])
      endif
   endif

   if TrlFontClr <> Nil
      if TrlFontClr[1] <> Nil .and. TrlFontClr[2] <> Nil .and. TrlFontClr[3] <> Nil
         SetDatePickTrlFontColor(ControlHandle,TrlFontClr[1],TrlFontClr[2],TrlFontClr[3])
      endif
   endif

   if cDateFormat <> NIL .and. ISCHARACTER(cDateFormat)

      if SetDatePickerDateFormat( ControlHandle , cDateFormat )
         _HMG_aControlSpacing [k] := cDateFormat
      else
         MsgMiniGuiError ("Control: " + ControlName + " Of " + ParentFormName + " Wrong format string. Program terminated" )
      endif

   else
      _HMG_aControlSpacing [k] := ""
   endif

Return Nil

*-----------------------------------------------------------------------------*
Function _DefineTimePick ( ControlName, ParentFormName, x, y, w, h, value,;
   fontname, fontsize, tooltip, change, lostfocus,;
   gotfocus, shownone, HelpId,;
   invisible, notabstop , bold, italic, underline, strikeout ,;
   Field, Enter, cTimeFormat, nId )
*-----------------------------------------------------------------------------*
   Local ParentFormHandle , mVar , k , Style , blInit
   Local ControlHandle , FontHandle , WorkArea
   Local lDialogInMemory

   DEFAULT value     TO IIF(shownone, "", time())
   DEFAULT w         TO 120
   DEFAULT h         TO 24
   DEFAULT change    TO ""
   DEFAULT lostfocus TO ""
   DEFAULT gotfocus  TO ""
   DEFAULT invisible TO FALSE
   DEFAULT notabstop TO FALSE
   DEFAULT cTimeFormat  TO "HH:mm:ss"

   If ValType ( Field ) != 'U'
      if  at ( '>', Field ) == 0
         MsgMiniGuiError ("Control: " + ControlName + " Of " + ParentFormName + " : You must specify a fully qualified field name. Program Terminated" )
      Else
         WorkArea := Left ( Field , at ( '>', Field ) - 2 )
         If Select (WorkArea) != 0
            Value := &(Field)
         EndIf
      EndIf
   EndIf

   if _HMG_BeginWindowActive = TRUE .or. _HMG_BeginDialogActive = TRUE
      ParentFormName := if(_HMG_BeginDialogActive,_HMG_ActiveDialogName,_HMG_ActiveFormName)
      if .Not. Empty (_HMG_ActiveFontName) .And. ValType(FontName) == "U"
         FontName := _HMG_ActiveFontName
      EndIf
      if .Not. Empty (_HMG_ActiveFontSize) .And. ValType(FontSize) == "U"
         FontSize := _HMG_ActiveFontSize
      EndIf
   endif
   if _HMG_FrameLevel > 0
      x    := x + _HMG_ActiveFrameCol [_HMG_FrameLevel]
      y    := y + _HMG_ActiveFrameRow [_HMG_FrameLevel]
      ParentFormName := _HMG_ActiveFrameParentFormName [_HMG_FrameLevel]
   EndIf
   lDialogInMemory := _HMG_DialogInMemory // ( GetWindowType ( ParentFormName ) == "L" )

   If .Not. _IsWindowDefined (ParentFormName) .and. .Not. lDialogInMemory
      MsgMiniGuiError("Window: "+ ParentFormName + " is not defined. Program terminated" )
   Endif

   If _IsControlDefined (ControlName,ParentFormName) .and. .Not. lDialogInMemory
      MsgMiniGuiError ("Control: " + ControlName + " Of " + ParentFormName + " Already defined. Program terminated" )
   endif

   mVar := '_' + ParentFormName + '_' + ControlName
   k := _GetControlFree()

   if _HMG_BeginDialogActive = TRUE
      ParentFormHandle = _HMG_ActiveDialogHandle

      Style := WS_CHILD

      if shownone
         Style += DTS_SHOWNONE
      Endif

      Style += DTS_UPDOWN

      if !invisible
         Style += WS_VISIBLE
      Endif

      if !NoTabStop
         Style += WS_TABSTOP
      Endif

      if lDialogInMemory         //Dialog Template
         InitExCommonControls(1)

//           {{'ID',k/hwnd,class,Style,ExStyle,x,y,w,h,caption,HelpId,tooltip,font,size, bold, italic, underline, strikeout}}  --->_HMG_aDialogItems
         blInit:= {|x,y,z| InitDialogDatePicker(x,y,z) }
         aadd(_HMG_aDialogItems,{nId,k,"SysDateTimePick32",style,0,x,y,w,h,"",HelpId,tooltip,FontName,FontSize,bold,italic,underline,strikeout,blInit,_HMG_BeginTabActive,.f.,_HMG_ActiveTabPage })
/* JP 62
         if _HMG_aDialogTemplate[3]   // Modal
            Return Nil
         endif
*/
      else
         ControlHandle := GetDialogItemHandle( ParentFormHandle, nId )

         x := GetWindowCol ( Controlhandle )
         y := GetWindowRow ( Controlhandle )
         w := GetWindowWidth  ( Controlhandle )
         h := GetWindowHeight ( Controlhandle )

         SetWindowStyle ( ControlHandle, style )

      endif

   else

      ParentFormHandle = GetFormHandle (ParentFormName)

      ControlHandle := InitTimePick ( ParentFormHandle, 0, x, y, w, h , '' , 0 , shownone , invisible, notabstop )

   Endif

   if .Not. lDialogInMemory

      If FontHandle != 0
         _SetFontHandle (ControlHandle, FontHandle)
      Else
         if valtype(fontname) == "U"
            Fontname := _HMG_DefaultFontName
         endif
         if valtype(fontsize) == "U"
            FontSize := _HMG_DefaultFontSize
         endif
         FontHandle := _SetFont (ControlHandle,FontName,FontSize,bold,italic,underline,strikeout)
      endif

      If _HMG_BeginTabActive = .T.
         aAdd ( _HMG_ActiveTabCurrentPageMap , Controlhandle )
      EndIf

      If Empty (Value)
         if shownone
            SetDatePickNull ( ControlHandle )
         else
            SetTimePick ( ControlHandle, VAL(LEFT(TIME(),2)), VAL(SUBSTR(TIME(),4,2)), VAL(SUBSTR(TIME(),7,2)) )
         endIf
      Else
         SetTimePick ( ControlHandle, VAL(LEFT(Value,2)), VAL(SUBSTR(Value,4,2)), VAL(SUBSTR(Value,7,2)) )
      EndIf

      if valtype (tooltip) != "U"
         SetToolTip ( ControlHandle , tooltip , GetFormToolTipHandle (ParentFormName) )
      Endif

      if valtype ( Field ) != 'U'
         aAdd ( _HMG_aFormBrowseList [ GetFormIndex ( ParentFormName ) ] , k )
      EndIf

   Endif

   Public &mVar. := k

   _HMG_aControlType [k] := "TIMEPICK"
   _HMG_aControlNames  [k] :=  ControlName
   _HMG_aControlHandles  [k] :=  ControlHandle
   _HMG_aControlParentHandles  [k] :=  ParentFormHandle
   _HMG_aControlIds  [k] :=  nId
   _HMG_aControlProcedures  [k] :=  Enter
   _HMG_aControlPageMap  [k] :=  Field
   _HMG_aControlValue  [k] :=  Value
   _HMG_aControlInputMask  [k] :=  ""
   _HMG_aControllostFocusProcedure  [k] :=  lostfocus
   _HMG_aControlGotFocusProcedure  [k] :=  gotfocus
   _HMG_aControlChangeProcedure  [k] :=  change
   _HMG_aControlDeleted  [k] :=  .F.
   _HMG_aControlBkColor  [k] :=  ""
   _HMG_aControlFontColor [k] :=  ""
   _HMG_aControlDblClick  [k] :=  ""
   _HMG_aControlHeadClick [k] :=  {}
   _HMG_aControlRow   [k] :=  y
   _HMG_aControlCol   [k] :=  x
   _HMG_aControlWidth   [k] :=  w
   _HMG_aControlHeight   [k] :=  h
   _HMG_aControlSpacing   [k] :=  0
   _HMG_aControlContainerRow  [k] :=  iif ( _HMG_FrameLevel > 0 ,_HMG_ActiveFrameRow [_HMG_FrameLevel] , -1 )
   _HMG_aControlContainerCol  [k] :=  iif ( _HMG_FrameLevel > 0 ,_HMG_ActiveFrameCol [_HMG_FrameLevel] , -1 )
   _HMG_aControlPicture  [k] :=  ""
   _HMG_aControlContainerHandle  [k] :=  0
   _HMG_aControlFontName  [k] :=  fontname
   _HMG_aControlFontSize  [k] :=  fontsize
   _HMG_aControlFontAttributes  [k] :=  {bold,italic,underline,strikeout}
   _HMG_aControlToolTip   [k] :=  tooltip
   _HMG_aControlRangeMin  [k] :=  0
   _HMG_aControlRangeMax  [k] :=  0
   _HMG_aControlCaption   [k] :=  ''
   _HMG_aControlVisible  [k] :=   iif(invisible,FALSE,TRUE)
   _HMG_aControlHelpId   [k] :=  HelpId
   _HMG_aControlFontHandle  [k] :=  FontHandle
   _HMG_aControlBrushHandle [k] :=  0
   _HMG_aControlEnabled  [k] :=  .T.
   _HMG_aControlMiscData1 [k] := 0

   if SetDatePickerDateFormat( ControlHandle , cTimeFormat )
      _HMG_aControlSpacing [k] := cTimeFormat
      if ascan(_HMG_aControlFontAttributes [k], .T.) > 0
         _SetFontName ( ControlName, ParentFormName , fontname )
      endif
   else
      MsgMiniGuiError ("Control: " + ControlName + " Of " + ParentFormName + "Wrong format string. Program terminated" )
   endif

Return Nil

*-----------------------------------------------------------------------------*
Function InitDialogDatePicker(ParentFormName, ControlHandle, k)
*-----------------------------------------------------------------------------*
   ParentFormName := Nil
   ControlHandle  := Nil

   _SetValue ( '' , '' , _HMG_aControlValue [k] , k )
// JP 62
   if len(_HMG_aDialogTemplate) != 0 .and. _HMG_aDialogTemplate[3]   // Modal
      _HMG_aControlDeleted [k] := .T.
   endif

Return Nil

*-----------------------------------------------------------------------------*
Procedure _DataDatePickerRefresh (i)
*-----------------------------------------------------------------------------*
   Local Field

   Field := _HMG_aControlPageMap [i]
   _SetValue ( '' , '' , &Field , i )

Return

*-----------------------------------------------------------------------------*
Procedure _DataDatePickerSave ( ControlName, ParentForm )
*-----------------------------------------------------------------------------*
   Local Field , i := GetControlIndex ( ControlName , ParentForm )

   Field := _HMG_aControlPageMap [i]
   REPLACE &Field WITH _GetValue ( Controlname , ParentForm )

Return

*-----------------------------------------------------------------------------*
Function _SetDatePickerDateFormat( ControlName, ParentForm, cFormat )
*-----------------------------------------------------------------------------*
   Local lOK := .f., ix, T
   ix := GetControlIndex ( ControlName, ParentForm )
   if ix > 0
      T := _HMG_aControlType[ix]
      if T=="DATEPICK" .or. T=="TIMEPICK"
         if ( lOK := SetDatePickerDateFormat(_HMG_aControlHandles[ix], cFormat) )
            _HMG_aControlSpacing[ix] := cFormat
         endif
      endif
   endif
Return lOK

*-----------------------------------------------------------------------------*
Function _GetDatePickerDateFormat( ControlName, ParentForm )
*-----------------------------------------------------------------------------*
   Local cFormat:="", ix, T
   ix := GetControlIndex ( ControlName, ParentForm )
   if ix > 0
      T := _HMG_aControlType[ix]
      if T=="DATEPICK" .or. T=="TIMEPICK"
         cFormat := _HMG_aControlSpacing[ix]
      endif
   endif
Return cFormat