/*----------------------------------------------------------------------------
MINIGUI - Harbour Win32 GUI library source code

Copyright 2002-2008 Roberto Lopez <harbourminigui@gmail.com>
http://harbourminigui.googlepages.com/

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along with
   this software; see the file COPYING. If not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA (or
   visit the web site http://www.gnu.org/).

   As a special exception, you have permission for additional uses of the text
   contained in this release of Harbour Minigui.

   The exception is that, if you link the Harbour Minigui library with other
   files to produce an executable, this does not by itself cause the resulting
   executable to be covered by the GNU General Public License.
   Your use of that executable is in no way restricted on account of linking the
   Harbour-Minigui library code into it.

   Parts of this project are based upon:

   "Harbour GUI framework for Win32"
   Copyright 2001 Alexander S.Kresin <alex@belacy.belgorod.su>
   Copyright 2001 Antonio Linares <alinares@fivetech.com>
   www - http://www.harbour-project.org

   "Harbour Project"
   Copyright 1999-2008, http://www.harbour-project.org/

   "WHAT32"
   Copyright 2002 AJ Wos <andrwos@aust1.net>

   "HWGUI"
   Copyright 2001-2007 Alexander S.Kresin <alex@belacy.belgorod.su>

---------------------------------------------------------------------------*/

#include "minigui.ch"

#define EM_LIMITTEXT        197
#define ES_WANTRETURN       4096
#define ES_MULTILINE            0x0004
#define ES_AUTOVSCROLL        0x0040
#define ES_AUTOHSCROLL           128
#define EM_GETSEL   176
#define EM_SETSEL   177
#define WS_CHILD    0x40000000
#define WS_BORDER    0x800000
#define WS_TABSTOP    0x10000
#define WS_VISIBLE    0x10000000
#define WS_HSCROLL    0x100000
#define WS_VSCROLL    0x200000

*-----------------------------------------------------------------------------*
Function _DefineEditbox ( ControlName, ParentFormName, x, y, w, h, value,;
   fontname, fontsize, tooltip, maxlength, gotfocus, change, lostfocus, readonly, break, HelpId,;
   invisible, notabstop, bold, italic, underline, strikeout, field, backcolor, fontcolor, novscroll, nohscroll, nId )
*-----------------------------------------------------------------------------*
   Local i , ParentFormHandle , mVar , ContainerHandle := 0 , k , Style
   Local ControlHandle , FontHandle , WorkArea , blInit, aFont
   Local lDialogInMemory

   DEFAULT w         TO 120
   DEFAULT h         TO 240
   DEFAULT value     TO ""
   DEFAULT change    TO ""
   DEFAULT lostfocus TO ""
   DEFAULT gotfocus  TO ""
   DEFAULT maxlength TO 64738
   DEFAULT invisible TO FALSE
   DEFAULT notabstop TO FALSE

   if ( FontHandle := GetFontHandle( FontName ) ) != 0
      aFont := GetFontParam(FontHandle)
      FontName      := aFont[1]
      FontSize      := aFont[2]
      bold          := aFont[3]
      italic        := aFont[4]
      underline     := aFont[5]
      strikeout     := aFont[6]
   endif

   If ValType ( Field ) != 'U'
      if  at ( '>', Field ) == 0
         MsgMiniGuiError ("Control: " + ControlName + " Of " + ParentFormName + " : You must specify a fully qualified field name. Program Terminated" )
      Else
         WorkArea := Left ( Field , at ( '>', Field ) - 2 )
         If Select (WorkArea) != 0
            Value := &(Field)
         EndIf
      EndIf
   EndIf

   if _HMG_BeginWindowActive = TRUE .or. _HMG_BeginDialogActive = TRUE
      ParentFormName := if(_HMG_BeginDialogActive,_HMG_ActiveDialogName,_HMG_ActiveFormName)
      if .Not. Empty (_HMG_ActiveFontName) .And. ValType(FontName) == "U"
         FontName := _HMG_ActiveFontName
      EndIf
      if .Not. Empty (_HMG_ActiveFontSize) .And. ValType(FontSize) == "U"
         FontSize := _HMG_ActiveFontSize
      EndIf
   endif
   if _HMG_FrameLevel > 0
      x    := x + _HMG_ActiveFrameCol [_HMG_FrameLevel]
      y    := y + _HMG_ActiveFrameRow [_HMG_FrameLevel]
      ParentFormName := _HMG_ActiveFrameParentFormName [_HMG_FrameLevel]
   EndIf
   lDialogInMemory := _HMG_DialogInMemory // ( GetWindowType ( ParentFormName ) == "L" )

   If .Not. _IsWindowDefined (ParentFormName) .and. .Not. lDialogInMemory
      MsgMiniGuiError("Window: "+ ParentFormName + " is not defined. Program terminated" )
   Endif

   If _IsControlDefined (ControlName,ParentFormName) .and. .Not. lDialogInMemory
      MsgMiniGuiError ("Control: " + ControlName + " Of " + ParentFormName + " Already defined. Program Terminated" )
   endif

   mVar := '_' + ParentFormName + '_' + ControlName
   k := _GetControlFree()

   if _HMG_BeginDialogActive = TRUE

      ParentFormHandle = _HMG_ActiveDialogHandle

      Style := ES_MULTILINE + ES_WANTRETURN + WS_CHILD + WS_BORDER

      if !NoTabStop
         Style += WS_TABSTOP
      endif

      if !invisible
         Style += WS_VISIBLE
      endif

      if !novscroll
         Style += WS_VSCROLL
      ELSE
         Style += ES_AUTOVSCROLL
      endif

      if !nohscroll
         Style += WS_HSCROLL
      endif

      if lDialogInMemory         //Dialog Template

         blInit:= {|x,y,z| InitDialogEdit(x,y,z) }

//          {{'ID',k/hwnd,class,Style,ExStyle,x,y,w,h,caption,HelpId,tooltip,font,size, bold, italic, underline, strikeout}}  --->_HMG_aDialogItems
         aadd(_HMG_aDialogItems,{nId,k,"edit",style,0,x,y,w,h,value,HelpId,tooltip,FontName,FontSize,bold,italic,underline,strikeout,blInit,_HMG_BeginTabActive,.f.,_HMG_ActiveTabPage})
/* JP 62
         if _HMG_aDialogTemplate[3]   // Modal
            _HMG_aControlDeleted [k] := .T.
            Return Nil
         endif
*/
      else

         ControlHandle := GetDialogItemHandle( ParentFormHandle, nId )


         x := GetWindowCol ( Controlhandle )
         y := GetWindowRow ( Controlhandle )
         w := GetWindowWidth  ( Controlhandle )
         h := GetWindowHeight ( Controlhandle )

         If Valtype (Value) == 'C' .or. Valtype (Value) == 'M'
            If .Not. Empty (Value)
               SetWindowText ( ControlHandle , value )
            EndIf
         EndIf
         SetWindowStyle ( ControlHandle, Style )

      endif
   else

      ParentFormHandle = GetFormHandle (ParentFormName)

      if valtype(x) == "U" .or. valtype(y) == "U"

         If _HMG_SplitLastControl == 'TOOLBAR'
            Break := .T.
         EndIf

         _HMG_SplitLastControl   := 'EDIT'

         i := GetFormIndex ( ParentFormName )

         if i > 0

            ControlHandle := InitEditBox ( ParentFormHandle , 0, x, y, w, h, '', 0 , maxlength , readonly, invisible, notabstop , novscroll , nohscroll )

            If FontHandle != 0
               _SetFontHandle(ControlHandle,FontHandle)
            Else
               if valtype(fontname) == "U"
                  Fontname := _HMG_DefaultFontName
               endif
               if valtype(fontsize) == "U"
                  FontSize := _HMG_DefaultFontSize
               endif
               FontHandle := _SetFont (ControlHandle,FontName,FontSize,bold,italic,underline,strikeout)
            endif
            AddSplitBoxItem ( Controlhandle , _HMG_aFormReBarHandle [i] , w , break , , , , _HMG_ActiveSplitBoxInverted )
            Containerhandle := _HMG_aFormReBarHandle [i]

            If Valtype (Value) == 'C';
               .or.;
               Valtype (Value) == 'M'

               If .Not. Empty (Value)
                  SetWindowText ( ControlHandle , value )
               EndIf

            EndIf

         EndIf

      Else

         ControlHandle := InitEditBox ( ParentFormHandle, 0, x, y, w, h, '', 0 , maxlength , readonly, invisible, notabstop , novscroll , nohscroll )

         If Valtype (Value) == 'C';
            .or.;
            Valtype (Value) == 'M'

            If .Not. Empty (Value)
               SetWindowText ( ControlHandle , value )
            EndIf

         EndIf

      endif
   endif

   if .Not. lDialogInMemory

      If _HMG_BeginTabActive = .T.
         aAdd ( _HMG_ActiveTabCurrentPageMap , Controlhandle )
      EndIf

      If FontHandle != 0
         _SetFontHandle(ControlHandle,FontHandle)
      Else
         if valtype(fontname) == "U"
            Fontname := _HMG_DefaultFontName
         endif
         if valtype(fontsize) == "U"
            FontSize := _HMG_DefaultFontSize
         endif
         FontHandle := _SetFont (ControlHandle,FontName,FontSize,bold,italic,underline,strikeout)
      EndIf

      if valtype(tooltip) != "U"
         SetToolTip ( ControlHandle , tooltip , GetFormToolTipHandle (ParentFormName) )
      endif
   endif

   Public &mVar. := k

   _HMG_aControlType [k] := "EDIT"
   _HMG_aControlNames  [k] :=  ControlName
   _HMG_aControlHandles  [k] :=  ControlHandle
   _HMG_aControlParenthandles  [k] :=  ParentFormHandle
   _HMG_aControlIds  [k] :=  nId
   _HMG_aControlProcedures  [k] :=  ""
   _HMG_aControlPageMap  [k] :=  Field
   _HMG_aControlValue  [k] :=  Nil
   _HMG_aControlInputMask  [k] :=  ""
   _HMG_aControllostFocusProcedure  [k] :=  lostfocus
   _HMG_aControlGotFocusProcedure [k] :=   gotfocus
   _HMG_aControlChangeProcedure  [k] :=  change
   _HMG_aControlDeleted  [k] :=  .F.
   _HMG_aControlBkColor  [k] :=  backcolor
   _HMG_aControlFontColor  [k] :=  fontcolor
   _HMG_aControlDblClick  [k] :=  ""
   _HMG_aControlHeadClick  [k] :=  {}
   _HMG_aControlRow  [k] :=  y
   _HMG_aControlCol   [k] := x
   _HMG_aControlWidth   [k] := w
   _HMG_aControlHeight   [k] := h
   _HMG_aControlSpacing   [k] := 0
   _HMG_aControlContainerRow  [k] :=  iif ( _HMG_FrameLevel > 0 ,_HMG_ActiveFrameRow [_HMG_FrameLevel] , -1 )
   _HMG_aControlContainerCol  [k] :=  iif ( _HMG_FrameLevel > 0 ,_HMG_ActiveFrameCol [_HMG_FrameLevel] , -1 )
   _HMG_aControlPicture  [k] :=  ""
   _HMG_aControlContainerHandle  [k] :=  ContainerHandle
   _HMG_aControlFontName  [k] :=  fontname
   _HMG_aControlFontSize  [k] :=  fontsize
   _HMG_aControlFontAttributes  [k] :=  {bold,italic,underline,strikeout}
   _HMG_aControlToolTip   [k] :=  tooltip
   _HMG_aControlRangeMin  [k] :=   0
   _HMG_aControlRangeMax  [k] :=   0
   _HMG_aControlCaption  [k] :=   ''
   _HMG_aControlVisible  [k] :=   if(invisible,FALSE,TRUE)
   _HMG_aControlHelpId   [k] :=  HelpId
   _HMG_aControlFontHandle  [k] :=   FontHandle
   _HMG_aControlBrushHandle  [k] :=   0
   _HMG_aControlEnabled  [k] :=   .T.
   _HMG_aControlMiscData1 [k] := {0 , maxlength , readonly }

   if len(_HMG_aDialogTemplate) == 0        //Dialog Template
      InitDialogEdit(ParentFormName, ControlHandle, k)
   endif

   if valtype ( Field ) != 'U'
      aAdd ( _HMG_aFormBrowseList [ GetFormIndex ( ParentFormName ) ] , k )
   EndIf

Return Nil

*-----------------------------------------------------------------------------*
Procedure _DataEditBoxRefresh (i)
*-----------------------------------------------------------------------------*
   Local Field := _HMG_aControlPageMap [i], icp

   if ValType ( Field ) != 'U'
      _SetValue ( '' , '' , &Field , i )
   else
      // Store Initial CaretPos
      icp := HiWord ( SendMessage( _HMG_aControlhandles [i] , EM_GETSEL , 0 , 0 ) )

      _SetValue ( '' , '' , _GetValue ( '' , '' , i ) , i )

      // Restore Initial CaretPos
      SendMessage( _HMG_aControlhandles [i] , EM_SETSEL , icp , icp )
   endif

Return

*-----------------------------------------------------------------------------*
Procedure _DataEditBoxSave ( ControlName , ParentFormName)
*-----------------------------------------------------------------------------*
   Local Field , i

   i := GetControlIndex ( ControlName , ParentFormName)

   Field := _HMG_aControlPageMap [i]

   REPLACE &Field WITH _GetValue ( Controlname , ParentFormName )

Return

*-----------------------------------------------------------------------------*
Function InitDialogEdit(ParentName, ControlHandle, k)
*-----------------------------------------------------------------------------*
   Local maxlength , readonly
   ParentName := Nil

   maxlength := _HMG_aControlMiscData1 [k,2]
   readonly  := _HMG_aControlMiscData1 [k,3]

   if valtype ( readonly ) == 'L'
      SetTextEditReadOnly ( ControlHandle , readonly )
   endif
   if valtype ( maxlength ) != 'U'
      SendMessage( ControlHandle , EM_LIMITTEXT , maxlength , 0 )
   endif
// JP 62
   if len(_HMG_aDialogTemplate) != 0 .and. _HMG_aDialogTemplate[3]   // Modal
      _HMG_aControlDeleted [k] := .T.
   endif

Return Nil