/*----------------------------------------------------------------------------
MINIGUI - Harbour Win32 GUI library source code

Copyright 2002-2008 Roberto Lopez <harbourminigui@gmail.com>
http://harbourminigui.googlepages.com/

GETBOX Control Source Code
Copyright 2006 Jacek Kubica <kubica@wssk.wroc.pl>
http://www.wssk.wroc.pl/~kubica
Revision patch 2008 By Pierpaolo Martinello <pier.martinello[at]alice.it>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along with
   this software; see the file COPYING. If not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA (or
   visit the web site http://www.gnu.org/).

   As a special exception, you have permission for additional uses of the text
   contained in this release of Harbour Minigui.

   The exception is that, if you link the Harbour Minigui library with other
   files to produce an executable, this does not by itself cause the resulting
   executable to be covered by the GNU General Public License.
   Your use of that executable is in no way restricted on account of linking the
   Harbour-Minigui library code into it.

   Parts of this project are based upon:

   "Harbour GUI framework for Win32"
   Copyright 2001 Alexander S.Kresin <alex@belacy.belgorod.su>
   Copyright 2001 Antonio Linares <alinares@fivetech.com>
   www - http://www.harbour-project.org

   "Harbour Project"
   Copyright 1999-2008, http://www.harbour-project.org/

   "WHAT32"
   Copyright 2002 AJ Wos <andrwos@aust1.net>

   "HWGUI"
   Copyright 2001-2007 Alexander S.Kresin <alex@belacy.belgorod.su>

---------------------------------------------------------------------------*/

#include "minigui.ch"
#include "hblang.ch"

#define EM_LIMITTEXT   197
#define ES_WANTRETURN  4096
#define ES_MULTILINE   0x0004
#define ES_AUTOVSCROLL 0x0040
#define ES_AUTOHSCROLL 128
#define WS_CHILD       0x40000000
#define WS_BORDER      0x800000
#define WS_TABSTOP     0x10000
#define WS_VISIBLE     0x10000000
#define WS_HSCROLL     0x100000
#define WS_VSCROLL     0x200000
#define BS_FLAT        0x00008000
#define ES_NUMBER     8192
#define ES_RIGHT         2
#define ES_UPPERCASE     8
#define ES_LOWERCASE    16
#define ES_PASSWORD     32
#define ES_READONLY   2048
#define WS_THICKFRAME 0x40000
#define EM_GETSEL     176
#define EM_SETSEL     177
#define WM_COMMAND    0x0111
#define WM_PASTE      770
#define WM_CUT        768
#define WM_COPY       769
#define WM_CLEAR      771
#define WM_SETTEXT     12
#define WM_CHAR       258
#define WM_KEYDOWN    256
#define WM_KEYUP      257
#define WM_SETFOCUS     7
#define WM_KILLFOCUS    8
#define WM_INVALID WM_USER + 50
   #Define WM_CARET   WM_USER + 51
#define GWL_STYLE           (-16)

Static lInValid := .F.
Static lInsert :=.f.
Static readonly:=.f.
//Static lrange:=.f.
*-----------------------------------------------------------------------------*
Function _DefineGetBox ( ControlName, ParentFormName, x, y, w, h,;
   Value, FontName, FontSize, ToolTip, lPassword,;
   uLostFocus, uGotFocus, uChange , right ,;
   HelpId , readonly , bold, italic, underline,;
   strikeout , field , backcolor , fontcolor ,;
   invisible, notabstop, nId, valid, cpicture,;
   cmessage, cvalidmessage, when,;
   ProcedureName, ProcedureName2, abitmap, BtnWidth )
*-----------------------------------------------------------------------------*
   Local ParentFormHandle
   Local ControlHandle := 0
   Local mVar
   Local FontHandle, aFont, nMaxLength
   Local WorkArea, blInit, cBmp, lBtns :=.f., lBtn2 :=.f.
   Local k, Style, aPicData
   Local oGet
   Local lDialogInMemory

   DEFAULT w          TO 120
   DEFAULT h          TO 24
   DEFAULT uGotFocus  TO ""
   DEFAULT uLostFocus TO ""
   DEFAULT lPassword  TO .f.
   DEFAULT cpicture   TO ""

   IF EMPTY(field) .and. VALTYPE(Value)=="U"
      MsgMiniGUIError("GETBOX: Initial Value or Field must be specified. Program terminated")
   endif

   If ValType ( Field ) != 'U'
      if  at ( '>', Field ) == 0
         MsgMiniGuiError ("Control: " + ControlName + " Of " + ParentFormName + " : You must specify a fully qualified field name. Program Terminated")
      Else
         WorkArea := Left ( Field , at ( '>', Field ) - 2 )
         If Select (WorkArea) != 0
            Value := &(Field)
         EndIf
      EndIf
   EndIf

   cPicture := UPPER(cPicture)

   do case
   case VALTYPE( Value )=="N" .and. !( "B" $ cPicture )
      right := .t.
   case VALTYPE( Value )=="D"
      nMaxLength := Len(DTOC(Date()))
   case VALTYPE( Value )=="L"
      nMaxLength := 1
   case VALTYPE( Value )=="C"
      nMaxLength := Len(Value)
   endcase

   If ValType( aBitmap ) != 'A'
      cBmp := aBitmap
      aBitmap := Array(2)
      aBitmap[1] := cBmp
   EndIf
   If ProcedureName != Nil
      lBtns := .T.
   EndIf
   If ProcedureName2 != Nil
      lBtn2 := .T.
   EndIf

   if ( FontHandle := GetFontHandle( FontName ) ) != 0
      aFont := GetFontParam(FontHandle)
      FontName  := aFont[1]
      FontSize  := aFont[2]
      bold      := aFont[3]
      italic    := aFont[4]
      underline := aFont[5]
      strikeout := aFont[6]
   endif

   if _HMG_BeginWindowActive = TRUE .or. _HMG_BeginDialogActive = TRUE
      if _HMG_BeginWindowMDIActive
         ParentFormHandle := GetActiveMdiHandle()
         ParentFormName := _GetWindowProperty ( ParentFormHandle, "PROP_FORMNAME" )
      else
         ParentFormName := if(_HMG_BeginDialogActive,_HMG_ActiveDialogName,_HMG_ActiveFormName)
      endif
      if .Not. Empty(_HMG_ActiveFontName) .And. ValType(FontName) == "U"
         FontName := _HMG_ActiveFontName
      EndIf
      if .Not. Empty(_HMG_ActiveFontSize) .And. ValType(FontSize) == "U"
         FontSize := _HMG_ActiveFontSize
      EndIf
   endif

   if _HMG_FrameLevel > 0
      x    := x + _HMG_ActiveFrameCol [_HMG_FrameLevel]
      y    := y + _HMG_ActiveFrameRow [_HMG_FrameLevel]
      ParentFormName := _HMG_ActiveFrameParentFormName [_HMG_FrameLevel]
   EndIf
   lDialogInMemory := _HMG_DialogInMemory // ( GetWindowType ( ParentFormName ) == "L" )

   if ! _IsWindowDefined( ParentFormName ) .and. ! lDialogInMemory
      MsgMiniGuiError( "Window: " + ParentFormName + " is not defined. Program terminated." )
   endif

   if _IsControlDefined( ControlName, ParentFormName ) .and. ! lDialogInMemory
      MsgMiniGuiError( "Control: " + ControlName + " of " + ParentFormName + " already defined. Program Terminated." )
   endif

   mVar := '_' + ParentFormName + '_' + ControlName
   k := _GetControlFree()

   if _HMG_BeginDialogActive = TRUE
      ParentFormHandle = _HMG_ActiveDialogHandle

      Style := WS_CHILD + ES_AUTOHSCROLL + BS_FLAT + WS_BORDER

      if lPassword
         Style += ES_PASSWORD
      endif

      if right
         Style += ES_RIGHT
      endif

      if readonly
         Style += ES_READONLY
      endif

      if !invisible
         Style += WS_VISIBLE
      endif

      if !notabstop
         Style += WS_TABSTOP
      endif

      if len(_HMG_aDialogTemplate) > 0        //Dialog Template

         blInit:= {|x,y,z| InitDialogTextBox(x,y,z) }
         aadd(_HMG_aDialogItems,{nId,k,"EDIT",style,0,x,y,w,h,Value,HelpId,Tooltip,FontName,FontSize,bold,italic,underline,strikeout,blInit,_HMG_BeginTabActive,.f.,_HMG_ActiveTabPage })
/* JP 62
         if _HMG_aDialogTemplate[3]   // Modal
            _HMG_aControlDeleted [k] := .T.
            Return Nil
         endif
*/
      else

         ControlHandle := GetDialogItemHandle( ParentFormHandle, nId )

         x := GetWindowCol    ( Controlhandle )
         y := GetWindowRow    ( Controlhandle )
         w := GetWindowWidth  ( Controlhandle )
         h := GetWindowHeight ( Controlhandle )

         SetWindowStyle ( ControlHandle, Style )
      endif

   else

      ParentFormHandle  := GetFormHandle( ParentFormName )

      ControlHandle := InitGetBox( ParentFormHandle, 0, x, y, w, h, '', 0, nMaxLength,;
         .f., .f. , .f., lPassword , right , readonly , invisible , notabstop, abitmap[1], BtnWidth, lBtns, abitmap[2], lBtn2 )

   endif

   if .Not. lDialogInMemory

      If FontHandle != 0

         _SetFontHandle( ControlHandle, FontHandle )

      Else

         if valtype(fontname) == "U"
            Fontname := _HMG_DefaultFontName
         endif
         if valtype(fontsize) == "U"
            FontSize := _HMG_DefaultFontSize
         endif
         FontHandle := _SetFont (ControlHandle,FontName,FontSize,bold,italic,underline,strikeout)

      EndIf

      If _HMG_BeginTabActive == .T.

         aAdd ( _HMG_ActiveTabCurrentPageMap , ControlHandle )

      EndIf

      if ValType( ToolTip ) != "U"

         SetToolTip( ControlHandle, ToolTip, GetFormToolTipHandle( ParentFormName ) )

      endif

   endif

   oget:= GET()
   oget:New( - 1, - 1, { | x | If( x == NIL, oget:cargo, oget:cargo := x ) } , '', cPicture )
   oget:cargo     := Value
   oget:preblock  := when
   oget:postblock := valid
   oget:message   := cmessage
   oget:name      := mVar
   oget:SetFocus()
   oget:original  := oGet:Buffer

   aPicData := _GetPictureData( oGet, cPicture )

   If cPicture == NIL .OR. ! ( '@K' $ cPicture )
      oget:Clear := .F.
   EndIf

   if !empty(aPicData[2]) .and. oget:type=="C"
      Value := PadR( Value, Len(aPicData[2]) )
      oget:cargo := Value
   endif

   oGet:Updatebuffer()

   Public &mVar. := k

   _HMG_aControlType [k] := "GETBOX"
   _HMG_aControlNames  [k] :=  ControlName
   _HMG_aControlHandles  [k] :=  ControlHandle
   _HMG_aControlParenthandles  [k] :=  ParentFormHandle
   _HMG_aControlIds  [k] :=  nId
   _HMG_aControlProcedures  [k] :=  if( readonly, NIL, ProcedureName )
   _HMG_aControlPageMap  [k] :=  Field
   _HMG_aControlValue  [k] :=  Value
   _HMG_aControlInputMask  [k] :=  aPicData
   _HMG_aControlLostFocusProcedure [k] :=  uLostFocus
   _HMG_aControlGotFocusProcedure  [k] :=  uGotFocus
   _HMG_aControlChangeProcedure  [k] :=  uChange
   _HMG_aControlDeleted  [k] :=  .F.
   _HMG_aControlBkColor  [k] :=  backcolor
   _HMG_aControlFontColor [k] :=  fontcolor
   _HMG_aControlDblClick  [k] :=  if( readonly, NIL, ProcedureName2 )
   _HMG_aControlHeadClick  [k] :=  oget
   _HMG_aControlRow  [k] :=  y
   _HMG_aControlCol  [k] :=  x
   _HMG_aControlWidth   [k] :=  w
   _HMG_aControlHeight  [k] :=  h
   _HMG_aControlSpacing  [k] :=  cValidMessage
   _HMG_aControlContainerRow  [k] :=  iif ( _HMG_FrameLevel > 0 ,_HMG_ActiveFrameRow [_HMG_FrameLevel] , -1 )
   _HMG_aControlContainerCol  [k] :=  iif ( _HMG_FrameLevel > 0 ,_HMG_ActiveFrameCol [_HMG_FrameLevel] , -1 )
   _HMG_aControlPicture  [k] :=  ""
   _HMG_aControlContainerHandle  [k] :=  0
   _HMG_aControlFontName  [k] :=  FontName
   _HMG_aControlFontSize  [k] :=  FontSize
   _HMG_aControlFontAttributes  [k] :=  {bold,italic,underline,strikeout}
   _HMG_aControlToolTip  [k] :=  ToolTip
   _HMG_aControlRangeMin [k] :=  0
   _HMG_aControlRangeMax [k] :=  nMaxLength
   _HMG_aControlCaption  [k] :=  ''
   _HMG_aControlVisible  [k] :=  .Not. invisible
   _HMG_aControlHelpId   [k] :=  HelpId
   _HMG_aControlFontHandle  [k] :=  FontHandle
   _HMG_aControlBrushHandle [k] :=  0
   _HMG_aControlEnabled  [k] :=  .T.
   _HMG_aControlMiscData1 [k] := {0, readonly, 0, ProcedureName, ProcedureName2 }

   if .Not. lDialogInMemory
      if !Empty( Value )
         if oGet:type == "N" .and. AT("B", aPicData[1]) > 0
            oGet:buffer := LTRIM( oGet:buffer )
         endif
         _DispGetBoxText ( ControlHandle , oget:buffer )
      endif

      If Valtype ( Field ) != 'U'
         aAdd ( _HMG_aFormBrowseList [ GetFormIndex ( ParentFormName ) ] , k )
      EndIf
   endif

Return nil

*-----------------------------------------------------------------------------*
Procedure _GetBoxSetNextFocus()
*-----------------------------------------------------------------------------*
   Local NextControlHandle

NextControlHandle := GetNextDlgTabITem ( GetActiveWindow() , GetFocus() , 0 )
setfocus( NextControlHandle )

Return

*-----------------------------------------------------------------------------*
Procedure _GetBoxSetPreviousFocus()
*-----------------------------------------------------------------------------*
   Local PreviousControlHandle

   PreviousControlHandle := GetNextDlgTabITem ( GetActiveWindow() , GetFocus() , 1 )
   setfocus( PreviousControlHandle )

Return

*-----------------------------------------------------------------------------*
Procedure _DataGetBoxRefresh (i)
*-----------------------------------------------------------------------------*
   Local Field := _HMG_aControlPageMap [i], hWnd := _HMG_aControlHandles [i]

   if ValType ( Field ) != 'U'
      _SetGetBoxValue(i, hWnd, &(Field))
   else
      _SetGetBoxValue(i, hWnd, _HMG_aControlValue [i])
   endif

Return

*-----------------------------------------------------------------------------*
Procedure _DataGetBoxSave ( ControlName, ParentForm )
*-----------------------------------------------------------------------------*
   Local Field , i
   Local oGet

   i := GetControlIndex ( ControlName , ParentForm )
   oGet := _HMG_aControlHeadClick [i]

   oGet:SetFocus()
   Field := _HMG_aControlPageMap [i]
   &(Field) := _GetValue ( ControlName , ParentForm )

   oGet:VarPut(&(Field))
   oGet:cargo := &(Field)

   if  oGet:type == "D"
      oGet:Buffer := DTOC(oGet:cargo)
   endif

Return

*-----------------------------------------------------------------------------*
Function OGETEVENTS( hWnd, nMsg, wParam, lParam )
*-----------------------------------------------------------------------------*
   Local i := ascan ( _HMG_aControlHandles , hWnd ), ParentHandle, ctempbuff, lshift, lCtrl
   Local nStart, nEnd, coldbuff, h, oGet, ipp, nlen, lAllowEdit := .t.
   Local cText, cPicMask, cPicFunc, lCleanZero, MinDec

   If i <= 0
      Return(0)
   endif

   cPicFunc   := _HMG_aControlInputMask[i,1]
   cPicMask   := _HMG_aControlInputMask[i,2]
   lCleanZero := _HMG_aControlInputMask[i,3]

   oGet       := _HMG_aControlHeadClick [i]
   readonly   := _HMG_aControlMiscData1 [i,2]

   ParentHandle := ASCAN(_HMG_aFormHandles,_HMG_aControlParenthandles[i])

   _HMG_ThisFormName :=  _HMG_aFormNames [ ParentHandle ]
   _HMG_ThisControlName :=  _HMG_aControlNames [i]

   If ValType( oget:preblock ) == 'B'
      if !eval( oGet:preblock, oGet, .F. )
         if (oGet:VarGet()==oGet:UnTransform(oGet:Original))
            lAllowEdit:=.f.
         endif
      endif
   EndIf

   DO CASE

   Case nMsg == WM_SETFOCUS

      nStart := LoWord( SendMessage( hWnd, EM_GETSEL, 0, 0 ) )
      nEnd := HiWord( SendMessage( hWnd, EM_GETSEL, 0, 0 ) )

      nStart := min( nStart, Len( Trim( oGet:buffer ) ) )
      nEnd := min( nEnd, Len( Trim( oGet:buffer ) ) )

      coldbuff := oGet:buffer

      If ValType( oGet:preblock ) == 'B'
         if !eval( oGet:preblock, oGet, .F. )
            if (oGet:VarGet()==oGet:UnTransform(oGet:Original))
               lAllowEdit := .f.
            endif
         endif
      EndIf

      If ! ( coldbuff == oGet:buffer )
         If oGet:BadDate
            _DispGetBoxText( hWnd, CTOD( "" ) )
         Else
            _DispGetBoxText( hWnd, oGet:buffer )
         EndIf
      EndIf

      InvalidateRect( hWnd )
      PostMessage( hWnd, WM_CARET, 0, 0 )
      _HMG_aControlMiscData1 [i][1] := 1

      oGet:pos := nEnd + 1
      oget:changed := .f.
      oGet:Updatebuffer()

      if !readonly .and. lAllowEdit
         _DispGetBoxText( hWnd, oGet:buffer )
      else
         if oGet:type == "N" .and. AT("B",cPicFunc) > 0
            oGet:buffer := LTRIM( oGet:buffer )
            _DispGetBoxText( hWnd, oGet:buffer )
         endif
      endif

      SendMessage( hWnd, EM_SETSEL, nStart, nStart )

      _SetGetBoxCaret( hWnd, IsInsertActive() )

      SendMessage( hWnd, EM_SETSEL, nStart, nStart )

      // show message if any

      if ( ParentHandle ) > 0
         if _IsControlDefined ( "StatusBar" , _HMG_aFormNames [ParentHandle] )
            If !EMPTY(oget:message)
               SetProperty( _HMG_aFormNames [ParentHandle], "StatusBar", "Item", 1, oget:message )
            endif
         endif
      endif

      Return( 0 )

   Case nMsg == WM_INVALID

      If _IsChildOfActiveWindow(hWnd) .and. !readonly .and. lAllowEdit
         If ! lInValid

            lInValid := .T.

            if ValType( oGet:postblock ) == 'B'
               coldbuff := oGet:buffer

               h := GetFocus()
               HideCaret(hWnd)
               HideCaret(h)

               If !Eval( oGet:postblock, oGet )

                  SetFocus( hWnd )
                  If Empty(_HMG_aControlSpacing [i])
                     oget:changed := .t.
                  else
                     oGet:changed := MsgRetryCancel( _HMG_aControlSpacing [i], _HMG_BRWLangError[11], , .f. )
                     If !oget:changed
                        PostMessage(hWnd,WM_KEYDOWN,VK_ESCAPE,0)
                     EndIf
//                   lRange:=.f.
                     SetFocus( hWnd )
                  EndIf
               Else
                  oget:changed := .t.
                  oget:original := oget:buffer
                  ShowCaret(h)
               EndIf
               If ! ( coldbuff == oGet:buffer )
                  _DispGetBoxText( hWnd, oGet:buffer )
               EndIf
            else
               oget:changed := .t.
               oget:original := oget:buffer
            EndIf

            lInValid := .F.

         EndIf
      EndIf

   Case nMsg == WM_KILLFOCUS

      if oGet:Changed
         oGet:assign( )
         // Patch By Pier July -2008
         // Add By Pier patch for the smaller negative numbers of zero START
         if oGet:minus
            if val(substr(oget:buffer,at(",",oget:buffer)+1)) #  0
               MinDec := strtran(oget:buffer,",",".")
               oget:VarPut(val(MinDec)*if(val(MinDec) > 0,-1,1))
            endif
            oget:buffer:=oget:varget()
         endif
         // Add By Pier patch for the smaller negative numbers of zero STOP
         oGet:updatebuffer( )
      endif

      If  Valtype( oGet:cargo ) == "D" .and. oGet:BadDate
         PostMessage( hWnd, WM_INVALID, wParam, 0 )
         Return( 0 )
      EndIf

      if lCleanZero .and. oGet:type == "N" .and. oGet:VarGet() == 0
         oGet:buffer := space( len( oGet:buffer ) )
      endif

      if oGet:type == "N" .and. AT("B",cPicFunc)>0
         oGet:buffer := LTRIM( oGet:buffer )
      endif

      _DispGetBoxText( hWnd, oGet:buffer )

      // message

      if ( ParentHandle ) > 0
         if _IsControlDefined ( "StatusBar" , _HMG_aFormNames [ParentHandle] )

            If VALTYPE( _GetDefinedStatusBarItemValue( _HMG_aControlParenthandles[i] ,1 ) )=="C"
               SetProperty( _HMG_aFormNames [ParentHandle], "StatusBar", "Item", 1, _GetDefinedStatusBarItemValue( _HMG_aControlParenthandles[i] ,1 ) )
            else
               SetProperty( _HMG_aFormNames [ParentHandle], "StatusBar", "Item", 1, "" )
            endif

         endif
      endif

      // check postvalidation

      If ! lInValid
         PostMessage( hWnd, WM_INVALID, wParam, 0 )
      EndIf

      Return( 0 )

   CASE nMsg == WM_CHAR

      nStart := LoWord ( SendMessage( _HMG_aControlhandles [ i ] , EM_GETSEL , 0 , 0 ) )+1
      nEnd   := HiWord ( SendMessage( _HMG_aControlhandles [ i ] , EM_GETSEL , 0 , 0 ) )+1
      oGet:pos := nEnd
      _HMG_aControlMiscData1 [i,3] := wParam   //JP

      Do Case

      Case wParam == 1  // CTRL-'A' SelectAll
         SendMessage( _HMG_aControlhandles [ i ] , EM_SETSEL , 0 , -1 )
         Return( 0 )

      Case wParam == 22 .and. !readonly// CTRL+V Paste
         SendMessage( _HMG_aControlhandles [ i ] , WM_PASTE , 0 , 0 )
         Return( 0 )

      Case wParam == 3  // CTRL+C Copy
         CopyToClipboard(oGet:Buffer)
         SendMessage( _HMG_aControlhandles [ i ] , WM_COPY , 0 , 0 )
         Return( 0 )

      Case wParam == 24 .and. !readonly// CTRL+X Cut
         If readonly .or. !lAllowEdit .or. oGet:type == "L"
            Return( 0 )
         endif

         nStart := LoWord( SendMessage( hWnd, EM_GETSEL, 0, 0 ) ) + 1
         nEnd   := HiWord( SendMessage( hWnd, EM_GETSEL, 0, 0 ) ) + 1

         oGet:pos := nEnd

         If nStart # nEnd

            If nEnd > Len( oGet:buffer )
               oGet:delete( )
            EndIf

            For ipp := nStart To nEnd

               If oGet:pos > nStart
                  oGet:backSpace( )
               Else
                  Exit
               EndIf

            Next
         Else
            If _IsEditable( oGet:pos , i )
               oGet:delete( )
            endif
         EndIf

         oGet:Assign()
         _DispGetBoxText( hWnd, oGet:buffer )
         SendMessage( hWnd, EM_SETSEL, ( oGet:pos - 1 ) , ( oGet:pos - 1 )  )

         Return( 0 )

      Case wParam ==0x0D .or. wParam ==0x09    // Return key or TAB key pressed

         lShift := CheckBit( GetKeyState( VK_SHIFT ) , 32768 ) // Shift key pressed (or not)

         If  Valtype(oGet:cargo)=="D" .and. oGet:BadDate .and. !readonly .and. lAllowEdit
            SendMessage( hWnd, EM_SETSEL, 0 , 0 )
            oGet:Pos:=1
            Return( 0 )
         else
            if !lshift
               _GetBoxSetNextFocus()
            else
               _GetBoxSetPreviousFocus()
            endif

            Return( 0 )
         EndIf

      Case ( wParam == VK_BACK .and. lParam # 0 .and. !readonly .and. lAllowEdit .and. oGet:type != "L" )

         If nEnd > nStart

            If nEnd > Len( oGet:Buffer )+1
               oGet:DELETE( )
            EndIf

            For ipp = nStart To nEnd
               If oGet:pos > nStart
                  oGet:backSpace( )
               Else
                  Exit
               EndIf
            Next ipp
         Else
            If nEnd > Len( oGet:buffer )+1
               oGet:delete( )
            Else
               oGet:backspace( )
            EndIf
         EndIf

         oGet:Assign()
         _HMG_aControlValue[i]:=oGet:VarGet()
         _DispGetBoxText( hWnd, oGet:buffer )
         SendMessage( hWnd, EM_SETSEL, ( oGet:pos - 1 ) , ( oGet:pos - 1 ) )

         Return( 0 )

      Case wParam >= 32 .AND. wParam < 256 // regular input
         _HMG_aControlMiscData1 [i,3] := 0

         If readonly .or. !lAllowEdit
            Return( 0 )
         endif

         if oGet:type=="L"
            nStart:=0
            nEnd:=0
            SendMessage( hWnd, EM_SETSEL, nStart, nEnd )
            oGet:Pos:=1
         endif

         if wParam <> 46 .and. wParam <> 44 .and. ( ASC( _Input(chr(wParam),i) ) <= 0 ) // dot and coma
            Return(0)
         endif

         if ("A" $ cPicFunc) .and. !IsAlpha( chr( wParam ) )
            Return ( 0 )
         endif

         oGet:changed:=.t.

         If nStart # nEnd
            If nEnd > Len( oGet:Buffer )+1
               oGet:DELETE( )
            EndIf
            For ipp := nStart To nEnd // clear selection by backspacing
               If oGet:pos > nStart
                  oGet:backSpace( )
               Else
                  Exit
               EndIf
            Next ipp
         EndIf

         IF oGet:pos==1
            oGet:home()
         Endif

         if oGet:pos == _FirstEditable(i) .and. _HMG_aControlMiscData1 [i][1] == 1 .and. (oGet:type == "N" .or. AT("K", cPicFunc) > 0)
            _HMG_aControlMiscData1 [i][1] := 0
            oGet:minus   := .f.
            oGet:clear   := .t.
            oget:changed := .t.
            oGet:Assign()
            _DispGetBoxText( hWnd, oGet:buffer )
         else
            _HMG_aControlMiscData1 [i][1] := 1
         endif

         If oGet:type == "N" .and. ( wParam == 46 .or. wParam == 44 )

            If oGet:type == "N"
               nlen := Len( oGet:buffer )
               If ( ipp := AT( '.', oGet:buffer ) ) > 0
                  oGet:buffer := PadL( StrTran( Left( oGet:Buffer, ipp - 1 ) , " ", "" ) , ipp - 1 ) +;
                     '.' + PadR( StrTran( SubStr( oGet:Buffer, ipp + 1 ) , " ", "" ) , nlen - ipp, "0" )

               Else
                  oGet:buffer := PadL( StrTran( oGet:Buffer, " ", "" ) , nlen )
               EndIf
            EndIf

         else

            If  IsInsertActive()
               oGet:insert( chr( wParam ) )
            Else
               oGet:overstrike( chr( wParam ) )
            EndIf

            If oGet:Rejected
               TONE(400)
            else
               oGet:Assign()
               _HMG_aControlValue[i] := oGet:varGet()
            endIf

         EndIf

         if oGet:type == "L"
            oGet:buffer := UPPER(oGet:buffer )
            oGet:pos := 1
         endif

         If oGet:type == "N" .and. ( wParam == 46 .or. wParam == 44 )

            oGet:UpdateBuffer()

            if oGet:DecPos != 0
               IF( oGet:DecPos == LEN(cPicMask) )
               oGet:pos := oGet:DecPos - 1   //9999.
            ELSE
               oGet:pos := oGet:DecPos + 1   //9999.9
            ENDIF
         else
            oGet:pos := oGet:nDispLen
         endif

      endif

      _DispGetBoxText( hWnd, oGet:buffer )
      SendMessage( hWnd, EM_SETSEL, ( oGet:pos - 1 ) , ( oGet:pos - 1 ) )

      Return( 0 )

   EndCase

CASE nMsg == WM_KEYDOWN

   If wParam == 110 .OR. wParam == 190
      Return( 0 )
   endif

   nStart := LoWord ( SendMessage( _HMG_aControlhandles [ i ] , EM_GETSEL , 0 , 0 ) )
   nEnd   := HiWord ( SendMessage( _HMG_aControlhandles [ i ] , EM_GETSEL , 0 , 0 ) )
   oGet:pos := nEnd
   _HMG_aControlMiscData1 [i,3] := wParam  //JP

   if wParam == VK_ESCAPE .and. !readonly

      IF oGet:Type == "N" .and. oGet:minus == .t. .and. AT( "-", oGet:original ) <= 0
         oGet:Buffer := oGet:Original
         oGet:VarPut( ( oGet:unTransform() ) * ( - 1 ), .t. )
         oGet:minus := .f.
         if oGet:Changed
            oGet:Assign()
            oGet:UpdateBuffer()
         endif
         _HMG_aControlValue[ i ] := oGet:VarGet()
      else
         if oGet:Changed
            oGet:Buffer := oGet:Original
            oGet:Assign()
            oGet:UpdateBuffer()
         endif
         _HMG_aControlValue[ i ] := oGet:VarGet()
      endif

      _DispGetBoxText( hWnd, oGet:buffer )
      SendMessage( hWnd , EM_SETSEL , 0 , 0 )

      oGet:BadDate:=.f.
      lInValid := .F.
      oGet:Pos:=1

      if !oGet:changed //.and. !readonly
         oGet:Buffer := oGet:Original
         IF oGet:Type == "N" .and. oGet:minus == .t. .and. AT( "-", oGet:original ) <= 0
            oGet:VarPut( ( oGet:unTransform() ) * ( - 1 ), .t. )
            oGet:minus := .f.
         endif
         oGet:Assign()
         oGet:UpdateBuffer()
         _HMG_aControlValue[ i ] := oGet:VarGet()
         _GetBoxSetNextFocus()
      endif

      oGet:Changed:=.f.

      Return( 0 )

   endif

   if wParam == VK_DOWN

      lShift := CheckBit( GetKeyState( VK_SHIFT ) , 32768 )
      lCtrl  := CheckBit( GetKeyState( VK_CONTROL ) , 32768 )

      if !lCtrl .and. !lShift
         SendMessage( _HMG_aControlhandles [ i ] , EM_SETSEL , nEnd-1 , nEnd-1 )
         If  Valtype(oGet:cargo)=="D" .and. oGet:BadDate
            Return( 0 )
         else
            _GetBoxSetNextFocus()
            Return( 0 )
         EndIf
      else
         if lCtrl .and.  lAllowEdit
            if oGet:type=="D" .or. oGet:type=="N"
               oGet:VarPut(oGet:VarGet()-1)
               oGet:UpdateBuffer()
               _DispGetBoxText(hWnd,oGet:Buffer)
               oGet:changed:=.t.
            endif
            if oGet:type=="L"
               oGet:VarPut(!oGet:VarGet())
               oGet:UpdateBuffer()
               _DispGetBoxText(hWnd,oGet:Buffer)
               oGet:changed:=.t.
            endif
         endif
      endif

      Return(0)
   endif

   if wParam == VK_UP

      lShift := CheckBit( GetKeyState( VK_SHIFT ) , 32768 )
      lCtrl  := CheckBit( GetKeyState( VK_CONTROL ) , 32768 )

      if !lCtrl .and. !lShift
         SendMessage( _HMG_aControlhandles [ i ] , EM_SETSEL , nEnd-1 , nEnd-1 )
         If Valtype(oGet:cargo)=="D" .and. oGet:BadDate
            Return( 0 )
         Else
            _GetBoxSetPreviousFocus()
            Return( 0 )
         EndIf
      else
         if lCtrl .and. lAllowEdit
            if oGet:type=="D" .or. oGet:type=="N"
               oGet:VarPut(oGet:VarGet()+1)
               oGet:UpdateBuffer()
               _DispGetBoxText(hWnd,oGet:Buffer)
               oGet:changed:=.t.
            endif
            if oGet:type=="L"
               oGet:VarPut(!oGet:VarGet())
               oGet:UpdateBuffer()
               _DispGetBoxText(hWnd,oGet:Buffer)
               oGet:changed:=.t.
            endif
         endif
      endif

      Return(0)
   endif

   if wParam == VK_LEFT
      SendMessage( _HMG_aControlhandles [ i ] , EM_SETSEL , nEnd-1 , nEnd-1 )
      _HMG_aControlMiscData1 [i][1] := 0
   endif

   if wParam == VK_RIGHT
      SendMessage( _HMG_aControlhandles [ i ] , EM_SETSEL , nStart+1 , nStart+1 )
      _HMG_aControlMiscData1 [i][1] := 0
   endif

   if wParam == VK_HOME
      SendMessage( _HMG_aControlhandles [ i ] , EM_SETSEL , 0 , 0 )
   endif

   if wParam == VK_END
         // Patch By Pier July -2008
         // Add By Pier patch for the incorrect end position START
      if HiWord( SendMessage( _HMG_aControlhandles [ i ] , EM_GETSEL ,0 ,0) ) < len(trim(oGet:Buffer))
         SendMessage( _HMG_aControlhandles [ i ] , EM_SETSEL , len(trim(oGet:Buffer)) , len(trim(oGet:Buffer)) )
      Else
         SendMessage( _HMG_aControlhandles [ i ] , EM_SETSEL , len(oGet:Buffer) , len(oGet:Buffer) )
      Endif
         // Add By Pier patch for the incorrect end position STOP
   endif

   If ( wParam == VK_INSERT )
      _SetGetBoxCaret( hWnd, !IsInsertActive() )
      lInsert := !lInsert
   endif

   If ( wParam == VK_DELETE )

      If readonly .or. !lAllowEdit .or. oGet:type == "L"
         Return( 0 )
      endif

      nStart := LoWord( SendMessage( hWnd, EM_GETSEL, 0, 0 ) ) + 1
      nEnd   := HiWord( SendMessage( hWnd, EM_GETSEL, 0, 0 ) ) + 1

      oGet:pos := nEnd

      If nStart # nEnd

         If nEnd > Len( oGet:buffer )
            oGet:delete( )
         EndIf

         For ipp := nStart To nEnd

            If oGet:pos > nStart
               oGet:backSpace( )
            Else
               Exit
            EndIf

         Next
      Else
         If _IsEditable( oGet:pos , i )
            oGet:delete( )
         endif
      EndIf
      oGet:Assign()
      _DispGetBoxText( hWnd, oGet:buffer )
      SendMessage( hWnd, EM_SETSEL, ( oGet:pos - 1 ) , ( oGet:pos - 1 )  )
      Return( 0 )
   ENDIF

Case nMsg == WM_PASTE
   If readonly .or. !lAllowEdit
      Return( 0 )
   endif
   nStart := LoWord( SendMessage( hWnd, EM_GETSEL, 0, 0 ) ) + 1
   nEnd   := HiWord( SendMessage( hWnd, EM_GETSEL, 0, 0 ) ) + 1

   If ( cText := RetriveTextFromClipboard()  ) <> NIL
      If nStart # nEnd
         For i := nStart To nEnd // clear selection by backspacing
            If oGet:pos > nStart
               oGet:backSpace( )
            Else
               Exit
            EndIf
         Next
      EndIf

      For i := 1 To Len( cText )

         wParam := asc( SubStr( cText, i, 1 ) )

         If oGet:type == "N" .and. wParam == 46
            oGet:toDecPos( )
         Else
            If IsInsertActive()
               oGet:insert( chr( wParam ) )
            Else
               oGet:overstrike( chr( wParam ) )
            EndIf

         EndIf
      Next
      oGet:Assign( )
      oGet:VarPut(oget:untransform())
      _DispGetBoxText( hWnd, oGet:buffer )
      SendMessage( hWnd, EM_SETSEL, ( oGet:pos - 1 ) , ( oGet:pos - 1 ) )
   EndIf

   Return ( 0 )

Case nMsg == WM_CUT .OR. nMsg == WM_CLEAR

   If IsWindowEnabled( hWnd ) .and. !readonly .and. lAllowEdit
      nStart := LoWord( SendMessage( hWnd, EM_GETSEL, 0, 0 ) ) + 1
      nEnd := HiWord( SendMessage( hWnd, EM_GETSEL, 0, 0 ) ) + 1
      oGet:pos := nEnd

      If nStart # nEnd
         If nEnd > Len( oGet:buffer )
            oGet:delete( )
         EndIf

         For i := nStart To nEnd
            If oGet:pos > nStart
               oGet:backSpace( )
            Else
               Exit
            EndIf
         Next
      Else
         oGet:delete( )
      EndIf
      ctempbuff := oGet:buffer

      _DispGetBoxText( hWnd, ctempbuff )
      SendMessage( hWnd, EM_SETSEL, ( oGet:pos - 1 ) , ( oGet:pos - 1 ) )
   EndIf

   Return( 0 )

Case nMsg == WM_CARET

   _SetGetBoxCaret( hWnd, IsInsertActive())

ENDCASE

Return 0

*-----------------------------------------------------------------------------*
Procedure _SetGetBoxValue( nId, hWnd, Value )
*-----------------------------------------------------------------------------*
   Local aPicData
   Local oGet       := _HMG_aControlHeadClick [nId]
   Local cPicFunc   := _HMG_aControlInputMask [nId, 1]
   Local lCleanZero := _HMG_aControlInputMask [nId, 3]

   If Valtype(Value) == Valtype(oGet:varGet())

      _HMG_ThisFormIndex   := Ascan ( _HMG_aFormHandles , _HMG_aControlParentHandles[nId] )
      _HMG_ThisFormName    := _HMG_aFormNames [ _HMG_ThisFormIndex ]
      _HMG_ThisControlName := _HMG_aControlNames [nId]

      oGet:varPut(Value)
      oGet:UpdateBuffer()
      oget:original := oget:buffer

      _HMG_aControlValue [nId] := Value

      if oGet:type=="N" .and. Value >= 0
         oGet:minus:=.f.
      endif

      if oGet:type == "N" .and. ( at("B", cPicFunc) > 0 .or. lCleanZero )
         oGet:buffer := ltrim( oGet:buffer )
      endif

      aPicData := _GetPictureData( oGet, oGet:Picture )

      If oGet:Picture == NIL .or. ! ( '@K' $ oGet:Picture )
         oGet:Clear := .F.
      EndIf

      if !EMPTY(aPicData[2]) .and. oGet:type == "C"
         Value := PadR( Value, Len(aPicData[2]) )
         oGet:cargo := Value
      endif
      if aPicData[3] .and. oGet:type == "N" .and. oGet:VarGet() == 0
         oGet:buffer := space( max( len( oGet:buffer ), len(aPicData[2]) ) )
      endif

      _DispGetBoxText( hWnd, oget:buffer )

   else
      MsgMiniGuiError('GETBOX: Wrong new value type. Program terminated')
   endif

Return

*-----------------------------------------------------------------------------*
Procedure _SetGetBoxValidMessage( ControlName, ParentForm , cNewMessage )
*-----------------------------------------------------------------------------*
   Local ix := GetControlIndex ( ControlName, ParentForm )

   If ix > 0 .and. _HMG_aControlType [ix] == "GETBOX"
      _HMG_aControlSpacing [ix] := cNewMessage
   endif

Return

*-----------------------------------------------------------------------------*
Function _GetGetBoxValidMessage( ControlName, ParentForm )
*-----------------------------------------------------------------------------*
   Local ix := GetControlIndex ( ControlName, ParentForm )
   Local cValidMsg := ""

   if ix > 0 .and. _HMG_aControlType [ix] == "GETBOX"
      cValidMsg := _HMG_aControlSpacing [ix]
   endif

Return cValidMsg

*-----------------------------------------------------------------------------*
Function _RangeCheck( oGet, lo, hi )
*-----------------------------------------------------------------------------*
   Local value := oGet:VarGet()

//   lRange:=.t.
   if !oGet:changed
      Return .T.
   endif

   Return ( value>=lo .and. value<=hi )

*-----------------------------------------------------------------------------*
   Static Procedure _DispGetBoxText( hWnd, cText )
*-----------------------------------------------------------------------------*
   Local i := ascan ( _HMG_aControlHandles , hWnd )

   if And( GetWindowLong( _HMG_aControlHandles [i], GWL_STYLE ), ES_PASSWORD )== ES_PASSWORD
      SetWindowText( _HMG_aControlHandles [i], Replicate( "*", Len( Trim( cText ) ) ) )
   else
      SetWindowText( _HMG_aControlHandles [i], cText )
   endif

   return

*-----------------------------------------------------------------------------*
   Static Procedure _SetGetBoxCaret( hWnd, lInsert )
*-----------------------------------------------------------------------------*
   Local hDC := GetDC( hWnd )
   Local aTM := GetTextMetric( hDC )

   ReleaseDC( hWnd, hDC )
   HideCaret( hWnd )
   DestroyCaret()
   CreateCaret( hWnd, 0, If( lInsert, 1, 5 ) , aTM[ 1 ] - 1 )
   ShowCaret( hWnd )

Return

*-----------------------------------------------------------------------------*
Function _GetPictureData( oGet, cPicture )
*-----------------------------------------------------------------------------*
   local nAt, nFor
   local cNum, lDecRev
   local cPicMask, cPicFunc, lCleanZero

   if Left( cPicture, 1 ) == "@"

      nAt := At( " ", cPicture )

      if nAt == 0
         cPicFunc := Upper( cPicture )
         cPicMask := ""
      else
         cPicFunc := Upper( SubStr( cPicture, 1, nAt - 1 ) )
         cPicMask := Ltrim(SubStr( cPicture, nAt + 1 ))
      endif

      if "D" $ cPicFunc

         cPicMask := Set( _SET_DATEFORMAT )
         cPicMask := StrTran( cPicmask, "y", "9" )
         cPicMask := StrTran( cPicmask, "Y", "9" )
         cPicMask := StrTran( cPicmask, "m", "9" )
         cPicMask := StrTran( cPicmask, "M", "9" )
         cPicMask := StrTran( cPicmask, "d", "9" )
         cPicMask := StrTran( cPicmask, "D", "9" )

      endif

      if ( nAt := At( "S", cPicFunc ) ) > 0
         for nFor := nAt + 1 to Len( cPicFunc )
            if ! IsDigit( SubStr( cPicFunc, nFor, 1 ) )
               exit
            endif
         next
         cPicFunc := SubStr( cPicFunc, 1, nAt - 1 ) + SubStr( cPicFunc, nFor )
      endif

      if "Z" $ cPicFunc
         lCleanZero := .t.
      else
         lCleanZero := .f.
      endif
      cPicFunc := StrTran( cPicFunc, "Z", "" )

      if cPicFunc == "@"
         cPicFunc := ""
      endif
   else
      cPicFunc   := ""
      cPicMask   := cPicture
      lCleanZero := .f.
   endif

   if oGet:type == "D"
      cPicMask := LTrim( cPicMask )
   endif

   if Empty( cPicMask )

      do case
      case oGet:type == "D"

         cPicMask := Set( _SET_DATEFORMAT )
         cPicMask := StrTran( cPicmask, "y", "9" )
         cPicMask := StrTran( cPicmask, "Y", "9" )
         cPicMask := StrTran( cPicmask, "m", "9" )
         cPicMask := StrTran( cPicmask, "M", "9" )
         cPicMask := StrTran( cPicmask, "d", "9" )
         cPicMask := StrTran( cPicmask, "D", "9" )

      case oGet:type == "N"

         lDecRev := "," $ Transform( 1.1, "9.9" )
         cNum := Str( oGet:VarGet() )
         if ( nAt := At( iif( lDecRev, ",", "." ), cNum ) ) > 0
            cPicMask := Replicate( "9", nAt - 1 ) + iif( lDecRev, ",", "." )
            cPicMask += Replicate( "9", Len( cNum ) - Len( cPicMask ) )
         else
            cPicMask := Replicate( "9", Len( cNum ) )
         endif

      case oGet:type == "C" .and. cPicFunc == "@9"

         cPicMask := Replicate( "9", Len( oGet:VarGet() ) )
         cPicFunc := ""
      endcase

   endif

Return { cPicFunc, cPicMask, lCleanZero }

*-----------------------------------------------------------------------------*
Static Function _FirstEditable( nId )
*-----------------------------------------------------------------------------*
   Local nFor
   Local oGet    := _HMG_aControlHeadClick [nId]
   Local nMaxLen := Len( oGet:Buffer )

   If nMaxLen != NIL
      If _IsEditable( 1 , nId )
         Return 1
      Endif
      For nFor := 2 to nMaxLen
         If _IsEditable( nFor , nId )
            Return nFor
         Endif
      Next
   Endif

   oGet:TypeOut := .t.

Return 0

*-----------------------------------------------------------------------------*
Static Function _IsEditable( nPos , nId )
*-----------------------------------------------------------------------------*
   Local cChar, lRet := .f.
   Local cPicMask := _HMG_aControlInputMask [nId,2]
   Local oGet     := _HMG_aControlHeadClick [nId]
   Local nMaxLen  := Len( oGet:Buffer )

   if Empty( cPicMask ) .or. nPos > nMaxLen
      return .t.
   endif

   cChar := UPPER( SubStr( cPicMask, nPos, 1 ) )

   DO CASE
   case oGet:type == "C"
      lRet := (cChar $ "!ANXLY9#")
   case oGet:type == "N"
      lRet := (cChar $ "9#$*" )
   case oGet:type == "D"
      lRet := (cChar $ "9")
   case oGet:type == "L"
      lRet := (cChar $ "LY")
   endcase

Return lRet

*-----------------------------------------------------------------------------*
Static Function _Input( cChar , nID )
*-----------------------------------------------------------------------------*
   Local oGet     := _HMG_aControlHeadClick [nId]
   Local cPicFunc := _HMG_aControlInputMask [nId,1]
   Local cPicMask := _HMG_aControlInputMask [nId,2]
   Local cLangItem_1 := hb_langmessage( HB_LANG_ITEM_BASE_TEXT + 1 )
   Local cLangItem_2 := hb_langmessage( HB_LANG_ITEM_BASE_TEXT + 2 )
   Local cPic

   do case

   case oGet:type == "N"

      do case
      case cChar == "-"
         oGet:minus := .t.  /* The minus symbol can be write in any place */
      case cChar == "."

      case cChar == ","
         oGet:toDecPos()
         return ""
      case !( cChar $ "0123456789+" )
         return ""
      endcase

   case oGet:type == "D"

      if !( cChar $ "0123456789" )
         return ""
      endif

   case oGet:type == "L"

      if !( Upper( cChar ) $ "YNTF"+cLangItem_1+cLangItem_2 )
         return ""
      endif
      Do Case
      case UPPER( cChar ) == cLangItem_1
         cChar := "Y"
      case UPPER( cChar ) == cLangItem_2
         cChar := "N"
      endcase

   endcase

   if ! Empty( cPicFunc )
      if "R" $ cPicFunc .AND. "E" $ cPicFunc
         cChar := SubStr( Transform( cChar, cPicFunc ), 4, 1 ) // Needed for @RE
      else
         cChar := Left( Transform( cChar, cPicFunc ), 1 ) // Left needed for @D
      endif
   endif

   if ! Empty( cPicMask )

      cPic  := Substr( cPicMask, oGet:pos, 1 )

      If cPic == "A"
         if ! IsAlpha( cChar )
            cChar := ""
         endif
      elseif cPic == "N"
         if ! IsAlpha( cChar ) .and. ! IsDigit( cChar )
            cChar := ""
         endif
      elseif cPic == "9"
         if ! IsDigit( cChar ) .and. cChar != "-"
            cChar := ""
         endif
      elseif cPic == "#"
         if ! IsDigit( cChar ) .and. !( cChar == " " ) .and. !( cChar $ ".+-" )
            cChar := ""
         endif
      elseif cPic == "L"
         if !( Upper( cChar ) $ "YNTF"+cLangItem_1+cLangItem_2 )
            cChar := ""
         endif
      elseif cPic == "Y"
         if !( Upper( cChar ) $ "YN" )
            cChar := ""
         endif

      elseif cPic == "$"

      elseif cPic == "*"
         if oGet:type == "N"
            if ! IsDigit( cChar ) .and. cChar != "-"
               cChar := ""
            endif
         else
            cChar := Transform( cChar, cPic )
         endif
      else
         cChar := Transform( cChar, cPic )
      endif

   endif

return cChar