/*----------------------------------------------------------------------------
MINIGUI - Harbour Win32 GUI library source code

Copyright 2002-2008 Roberto Lopez <harbourminigui@gmail.com>
http://harbourminigui.googlepages.com/

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along with
   this software; see the file COPYING. If not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA (or
   visit the web site http://www.gnu.org/).

   As a special exception, you have permission for additional uses of the text
   contained in this release of Harbour Minigui.

   The exception is that, if you link the Harbour Minigui library with other
   files to produce an executable, this does not by itself cause the resulting
   executable to be covered by the GNU General Public License.
   Your use of that executable is in no way restricted on account of linking the
   Harbour-Minigui library code into it.

   Parts of this project are based upon:

   "Harbour GUI framework for Win32"
   Copyright 2001 Alexander S.Kresin <alex@belacy.belgorod.su>
   Copyright 2001 Antonio Linares <alinares@fivetech.com>
   www - http://www.harbour-project.org

   "Harbour Project"
   Copyright 1999-2008, http://www.harbour-project.org/

   "WHAT32"
   Copyright 2002 AJ Wos <andrwos@aust1.net>

   "HWGUI"
   Copyright 2001-2007 Alexander S.Kresin <alex@belacy.belgorod.su>

---------------------------------------------------------------------------*/

#include 'minigui.ch'

#define WS_BORDER       0x800000
#define WS_CHILD        0x40000000
#define WS_TABSTOP      0x10000
#define WS_VISIBLE      0x10000000
#define LVS_SHOWSELALWAYS       0x0008
#define LVS_REPORT              0x0001
#define LVS_SINGLESEL           0x0004
#define LVS_OWNERDATA         0x1000
#define LVM_FIRST               0x1000      // ListView messages
#define LVS_EX_HEADERDRAGDROP   0x00000010
#define LVS_EX_FULLROWSELECT    0x00000020 // applies to report mode only

#define LVM_SETEXTENDEDLISTVIEWSTYLE (LVM_FIRST + 54)   // optional wParam == mask
#define LVS_NOCOLUMNHEADER      0x4000

*-----------------------------------------------------------------------------*
Function _DefineGrid ( ControlName, ParentFormName, x, y, w, h, aHeaders, aWidths, aRows, value,;
   fontname, fontsize , tooltip , change , dblclick , aHeadClick , gotfocus , lostfocus , nogrid,;
   aImage, aJust, break  , HelpId , bold, italic, underline, strikeout , ownerdata , ondispinfo ,;
   itemcount , editable , dynamicforecolor , dynamicbackcolor , multiselect , editcontrols ,;
   backcolor , fontcolor , nId , columnvalid , columnwhen , showheaders, aImageHeader, NoTabStop )
*-----------------------------------------------------------------------------*
   Local i , ParentFormHandle , blInit , mVar, wBitmap , k , Style
   Local ControlHandle , FontHandle , aFont
   Local nHeaderImageListHandle := 0
   Local lDialogInMemory

   if ( FontHandle := GetFontHandle( FontName ) ) != 0
      aFont := GetFontParam(FontHandle)
      FontName      := aFont[1]
      FontSize      := aFont[2]
      bold          := aFont[3]
      italic        := aFont[4]
      underline     := aFont[5]
      strikeout     := aFont[6]
   endif

   if _HMG_BeginWindowActive = TRUE .or. _HMG_BeginDialogActive = TRUE
      ParentFormName := if(_HMG_BeginDialogActive,_HMG_ActiveDialogName,_HMG_ActiveFormName)
      if .Not. Empty (_HMG_ActiveFontName) .And. ValType(FontName) == "U"
         FontName := _HMG_ActiveFontName
      EndIf
      if .Not. Empty (_HMG_ActiveFontSize) .And. ValType(FontSize) == "U"
         FontSize := _HMG_ActiveFontSize
      EndIf
   endif

   if _HMG_FrameLevel > 0
      x    := x + _HMG_ActiveFrameCol [_HMG_FrameLevel]
      y    := y + _HMG_ActiveFrameRow [_HMG_FrameLevel]
      ParentFormName := _HMG_ActiveFrameParentFormName [_HMG_FrameLevel]
   EndIf
   lDialogInMemory := _HMG_DialogInMemory // ( GetWindowType ( ParentFormName ) == "L" )

   If .Not. _IsWindowDefined (ParentFormName)  .and. .Not. lDialogInMemory
      MsgMiniGuiError(_HMG_BRWLangError[1]+ ParentFormName + _HMG_BRWLangError[2])
   Endif

   If _IsControlDefined (ControlName,ParentFormName)  .and. .Not. lDialogInMemory
      MsgMiniGuiError (_HMG_BRWLangError[4] + ControlName + _HMG_BRWLangError[5] + ParentFormName + _HMG_BRWLangError[6])
   endif

   if ValType ( aWidths ) == 'U'
      MsgMiniGuiError ("Grid: WIDTHS not defined. Program Terminated")
   EndIf

   DEFAULT notabstop TO FALSE
   DEFAULT showheaders TO TRUE

   if showheaders == .F.
      aHeaders := Array ( Len ( aWidths ) )
      aFill ( aHeaders , '' )
   endif

   if ValType ( aHeaders ) == 'U'
      MsgMiniGuiError ("Grid: HEADERS not defined. Program Terminated")
   EndIf

   if Len ( aHeaders ) != Len ( aWidths )
      MsgMiniGuiError ("Browse/Grid: FIELDS/HEADERS/WIDTHS array size mismatch. Program Terminated")
   EndIf

   if ValType (aRows) != 'U'
      if Len (aRows) > 0
         if Len (aRows[1]) != Len ( aHeaders )
            MsgMiniGuiError ("Grid: ITEMS length mismatch. Program Terminated")
         EndIf
      EndIf
   EndIf

   if valtype(w) == "U"
      w := 240
   endif
   if valtype(h) == "U"
      h := 120
   endif
   if valtype(value) == "U" .and. !MultiSelect
      value := 0
   endif
   if valtype(aRows) == "U"
      aRows := {}
   endif
   /* code borrowed from ooHG project */
   If !HB_IsArray( aJust )
      aJust := aFill( Array( len( aHeaders ) ), 0 )
   else
      aSize( aJust, len( aHeaders ) )
      aEval( aJust, { |x,i| aJust[ i ] := iif( HB_IsNumeric( x ), x, 0 ) } )
   endif
   /* end code borrowed */
   if valtype(aImage) == "U"        // Grid+
      aImage := {}
   endif
   if valtype(aImageHeader) == "U"  // Grid+
      aImageHeader := {}
   endif

   if valtype(aHeadClick) == "U"
      aHeadClick := {}
   endif

   if valtype(change) == "U"
      change := ""
   endif

   if valtype(dblclick) == "U"
      dblclick := ""
   endif

   mVar := '_' + ParentFormName + '_' + ControlName
   k := _GetControlFree()

   if _HMG_BeginDialogActive = TRUE

      ParentFormHandle = _HMG_ActiveDialogHandle
      Style := WS_BORDER+ LVS_SHOWSELALWAYS + WS_CHILD + WS_VISIBLE + LVS_REPORT

      if !NoTabStop
         Style += WS_TABSTOP
      endif

      if  !multiselect
         Style += LVS_SINGLESEL
      endif

      if  ownerdata
         Style += LVS_OWNERDATA
      endif

      if ! showheaders
         Style += LVS_NOCOLUMNHEADER
      endif

      if len(_HMG_aDialogTemplate) > 0        //Dialog Template

//          {{'ID',k/hwnd,class,Style,ExStyle,x,y,w,h,caption,HelpId,tooltip,font,size, bold, italic, underline, strikeout}}  --->_HMG_aDialogItems
         blInit:= {|x,y,z| InitDialogGrid(x,y,z) }
         aadd(_HMG_aDialogItems,{nId,k,"SysListView32",style,0,x,y,w,h,"",HelpId,tooltip,FontName,FontSize,bold,italic,underline,strikeout,blInit,,_HMG_BeginTabActive,.f.,_HMG_ActiveTabPage})
/* JP 62
         if _HMG_aDialogTemplate[3]   // Modal
            _HMG_aControlDeleted [k] := .T.
            Return Nil
         endif
*/
      else

         ControlHandle := GetDialogItemHandle( ParentFormHandle, nId )

         SetWindowStyle ( ControlHandle, Style )

         x := GetWindowCol ( Controlhandle )
         y := GetWindowRow ( Controlhandle )
         w := GetWindowWidth  ( Controlhandle )
         h := GetWindowHeight ( Controlhandle )

         if  ownerdata
            LISTVIEW_SETITEMCOUNT (  ControlHandle , itemcount )
         endif

         SendMessage(ControlHandle,LVM_SETEXTENDEDLISTVIEWSTYLE, 0, iif( nogrid, 0, 1 )+ LVS_EX_FULLROWSELECT + LVS_EX_HEADERDRAGDROP )

      endif

   else

      ParentFormHandle := GetFormHandle (ParentFormName)

      If valtype(x) == "U" .or. valtype(y) == "U"

         If _HMG_SplitLastControl == 'TOOLBAR'
            Break := .T.
         EndIf

         _HMG_SplitLastControl := 'GRID'

         i := GetFormIndex ( ParentFormName )

         if i > 0

            ControlHandle := InitListView ( _HMG_aFormReBarHandle [i], 0, 0, 0, w, h , '', 0, iif( nogrid, 0, 1 ), ownerdata, itemcount, multiselect, showheaders, NoTabStop )

            AddSplitBoxItem ( Controlhandle, _HMG_aFormReBarHandle [i] , w , break , , , , _HMG_ActiveSplitBoxInverted )
         EndIf

      Else

         ControlHandle := InitListView ( ParentFormHandle, 0, x, y, w, h , '', 0, iif( nogrid, 0, 1 ), ownerdata, itemcount, multiselect, showheaders, NoTabStop )

      EndIf

   endif

   if .Not. lDialogInMemory
      If ValType (backcolor) != 'U'
         ListView_SetBkColor ( ControlHandle , backcolor[1] , backcolor[2] , backcolor[3] )
         ListView_SetTextBkColor ( ControlHandle , backcolor[1] , backcolor[2] , backcolor[3]  )
      EndIf

      If ValType (fontcolor) != 'U'
         ListView_SetTextColor ( ControlHandle , fontcolor[1] , fontcolor[2] , fontcolor[3]  )
      EndIf

      wBitmap := iif( len( aImage ) > 0, AddListViewBitmap( ControlHandle, aImage ), 0 )
      aWidths[1] := max ( aWidths[1], wBitmap + 2 ) // Set Column 1 width to Bitmap width

      if len( aImageHeader ) > 0
         nHeaderImageListHandle := AddListViewBitmapHeader( ControlHandle, aImageHeader )
      endif

      If _HMG_BeginTabActive = .T.
         aAdd ( _HMG_ActiveTabCurrentPageMap , ControlHandle )
      EndIf

      If FontHandle != 0
         _SetFontHandle(ControlHandle,FontHandle)
      Else
         if valtype(fontname) == "U"
            Fontname := _HMG_DefaultFontName
         endif
         if valtype(fontsize) == "U"
            FontSize := _HMG_DefaultFontSize
         endif
         FontHandle := _SetFont (ControlHandle,FontName,FontSize,bold,italic,underline,strikeout)
      endif

      if valtype(tooltip) != "U"
         SetToolTip ( ControlHandle , tooltip , GetFormToolTipHandle (ParentFormName) )
      endif

   endif

   Public &mVar. := k

   _HMG_aControlType [k] := if ( multiselect , "MULTIGRID" , "GRID" )
   _HMG_aControlNames [k] :=   ControlName
   _HMG_aControlHandles [k] :=   ControlHandle
   _HMG_aControlParenthandles [k] :=   ParentFormHandle
   _HMG_aControlIds [k] :=   nId
   _HMG_aControlProcedures [k] :=   ondispinfo
   _HMG_aControlPageMap  [k] :=  aHeaders
   _HMG_aControlValue  [k] :=  Value
   _HMG_aControlInputMask  [k] :=  Nil
   _HMG_aControllostFocusProcedure  [k] :=  lostfocus
   _HMG_aControlGotFocusProcedure  [k] :=  gotfocus
   _HMG_aControlChangeProcedure  [k] :=  change
   _HMG_aControlDeleted  [k] :=  .F.
   _HMG_aControlBkColor  [k] :=  aImage
   _HMG_aControlFontColor   [k] := Nil
   _HMG_aControlDblClick  [k] :=  dblclick
   _HMG_aControlHeadClick  [k] :=  aHeadClick
   _HMG_aControlRow   [k] := y
   _HMG_aControlCol   [k] := x
   _HMG_aControlWidth  [k] :=  w
   _HMG_aControlHeight  [k] :=  h
   _HMG_aControlSpacing  [k] :=  Editable
   _HMG_aControlContainerRow  [k] :=  iif ( _HMG_FrameLevel > 0 ,_HMG_ActiveFrameRow [_HMG_FrameLevel] , -1 )
   _HMG_aControlContainerCol  [k] :=  iif ( _HMG_FrameLevel > 0 ,_HMG_ActiveFrameCol [_HMG_FrameLevel] , -1 )
   _HMG_aControlPicture  [k] :=  aImageHeader
   _HMG_aControlContainerHandle  [k] :=  nHeaderImageListHandle
   _HMG_aControlFontName  [k] :=  fontname
   _HMG_aControlFontSize  [k] :=  fontsize
   _HMG_aControlFontAttributes  [k] :=  {bold,italic,underline,strikeout}
   _HMG_aControlToolTip   [k] :=  tooltip
   _HMG_aControlRangeMin   [k] :=  Nil
   _HMG_aControlRangeMax  [k] :=   {}
   _HMG_aControlCaption  [k] :=   aHeaders
   _HMG_aControlVisible   [k] :=  .t.
   _HMG_aControlHelpId  [k] :=   HelpId
   _HMG_aControlFontHandle   [k] :=  FontHandle
   _HMG_aControlBrushHandle   [k] :=  0
   _HMG_aControlEnabled  [k] :=   .T.
   _HMG_aControlMiscData1 [k] := {0, aWidths, aJust, aRows, ownerdata, itemcount, nogrid, backcolor, fontcolor, multiselect, dynamicforecolor, dynamicbackcolor, editcontrols, columnvalid, columnwhen }

   if len(_HMG_aDialogTemplate) == 0        //Dialog Template
      InitDialogGrid ( ParentFormName, ControlHandle, k )
   endif

Return Nil

*-----------------------------------------------------------------------------*
Function InitDialogGrid(ParentName, ControlHandle, k)
*-----------------------------------------------------------------------------*
   Local i, Value , wBitmap
   Local ownerdata, itemcount, fontcolor, backcolor, aRows, multiselect
   Local aWidths, aJust, nogrid, dynamicforecolor, dynamicbackcolor, editcontrols, columnvalid, columnwhen

   Value        := _HMG_aControlValue  [k]

   aWidths      := _HMG_aControlMiscData1 [k,2]
   aJust        := _HMG_aControlMiscData1 [k,3]
   aRows        := _HMG_aControlMiscData1 [k,4]
   ownerdata    := _HMG_aControlMiscData1 [k,5]
   itemcount    := _HMG_aControlMiscData1 [k,6]
   nogrid       := _HMG_aControlMiscData1 [k,7]
   backcolor    := _HMG_aControlMiscData1 [k,8]
   fontcolor    := _HMG_aControlMiscData1 [k,9]
   multiselect  := _HMG_aControlMiscData1 [k,10]
   dynamicforecolor := _HMG_aControlMiscData1 [k,11]
   dynamicbackcolor := _HMG_aControlMiscData1 [k,12]
   editcontrols := _HMG_aControlMiscData1 [k,13]
   columnvalid  := _HMG_aControlMiscData1 [k,14]
   columnwhen   := _HMG_aControlMiscData1 [k,15]

   if ownerdata
      LISTVIEW_SETITEMCOUNT ( ControlHandle , itemcount )
   endif

   SendMessage( ControlHandle, LVM_SETEXTENDEDLISTVIEWSTYLE, 0, iif( nogrid, 0, 1 )+ LVS_EX_FULLROWSELECT + LVS_EX_HEADERDRAGDROP )

   If ValType (backcolor) != 'U'
      ListView_SetBkColor ( ControlHandle , backcolor[1] , backcolor[2] , backcolor[3] )
      ListView_SetTextBkColor ( ControlHandle , backcolor[1] , backcolor[2] , backcolor[3] )
   EndIf

   If ValType (fontcolor) != 'U'
      ListView_SetTextColor ( ControlHandle , fontcolor[1] , fontcolor[2] , fontcolor[3] )
   EndIf

   wBitmap := iif( len( _HMG_aControlBkColor  [k] ) > 0, AddListViewBitmap( ControlHandle, _HMG_aControlBkColor  [k] ), 0 ) //Add Bitmap Column
   aWidths[1] := max ( aWidths[1], wBitmap + 2 ) // Set Column 1 width to Bitmap width

   __MVPUBLIC ( '_HMG_' + ALLTRIM(STR(_HMG_aControlhandles[k])) + '_INPLACE' )
   __MVPUT ( '_HMG_' + ALLTRIM(STR(_HMG_aControlhandles[k])) + '_INPLACE' , VALTYPE(editcontrols) == 'A' )

   __MVPUBLIC ( '_HMG_' + ALLTRIM(STR(_HMG_aControlhandles[k])) + '_EDITCONTROLS' )
   __MVPUT ( '_HMG_' + ALLTRIM(STR(_HMG_aControlhandles[k])) + '_EDITCONTROLS' , editcontrols )

   __MVPUBLIC ( '_HMG_' + ALLTRIM(STR(_HMG_aControlhandles[k])) + '_DYNAMICFORECOLOR' )
   __MVPUT ( '_HMG_' + ALLTRIM(STR(_HMG_aControlhandles[k])) + '_DYNAMICFORECOLOR' , dynamicforecolor )

   __MVPUBLIC ( '_HMG_' + ALLTRIM(STR(_HMG_aControlhandles[k])) + '_DYNAMICBACKCOLOR' )
   __MVPUT ( '_HMG_' + ALLTRIM(STR(_HMG_aControlhandles[k])) + '_DYNAMICBACKCOLOR' , dynamicbackcolor )

   __MVPUBLIC ( '_HMG_' + ALLTRIM(STR(_HMG_aControlhandles[k])) + '_COLUMNVALID' )
   __MVPUT ( '_HMG_' + ALLTRIM(STR(_HMG_aControlhandles[k])) + '_COLUMNVALID' , columnvalid )

   __MVPUBLIC ( '_HMG_' + ALLTRIM(STR(_HMG_aControlhandles[k])) + '_COLUMNWHEN' )
   __MVPUT ( '_HMG_' + ALLTRIM(STR(_HMG_aControlhandles[k])) + '_COLUMNWHEN' , columnwhen )

   __MVPUBLIC ( '_HMG_' + ALLTRIM(STR(_HMG_aControlhandles[k])) + '_DBC' )
  __MVPUBLIC ( '_HMG_' + ALLTRIM(STR(_HMG_aControlhandles[k])) + '_DFC' )

   InitListViewColumns ( ControlHandle , _HMG_aControlCaption[k] , aWidths, aJust )

   for i := 1 to len (aRows)

      _AddGridRow (  _HMG_aControlNames [k], ParentName, aRows [i] )

   next i

   If multiselect == .T.

      if valtype ( value ) == 'A'
         ListViewSetMultiSel ( ControlHandle, value )
      endif

   Else

      if value <> 0
         ListView_SetCursel ( ControlHandle, value )
      endif

   EndIf
// JP 62
   if len(_HMG_aDialogTemplate) != 0 .and. _HMG_aDialogTemplate[3]   // Modal
      _HMG_aControlDeleted [k] := .T.
   endif

Return Nil

*-----------------------------------------------------------------------------*
Function _AddGridRow ( ControlName, ParentForm, aRow )
*-----------------------------------------------------------------------------*
   Local i, h, iIm := 0, aGridRow := aClone ( aRow )

   i := GetControlIndex ( ControlName , ParentForm )

   if Len ( _HMG_aControlPageMap [i] )  != Len ( aRow )
      MsgMiniGuiError ("Grid.AddItem: Item size mismatch. Program Terminated")
   EndIf

   if Len( _HMG_aControlBkColor[i] ) > 0
      iIm := aGridRow[1]
      aGridRow[1] := NIL
   endif

   h := GetControlHandle ( ControlName , ParentForm )

   AddListViewItems ( h , aGridRow , iIm )

   if ValType ( __MVGET ( '_HMG_' + ALLTRIM(STR(_HMG_aControlhandles[i])) + '_EDITCONTROLS' ) ) == 'A'

      _SetItem ( ControlName , ParentForm , ListViewGetItemCount ( h ) , aGridRow )
   else
      _UpdateGridColors ( i )
   endif

Return Nil

*-----------------------------------------------------------------------------*
Function _EditItem ( GridHandle )
*-----------------------------------------------------------------------------*
   Local a, l, g, actpos := {0,0,0,0}, h := GridHandle, /*GRow,*/ GCol, GWidth,;
      Col, IRow, LN , TN , i , ControlName , j , FormName , item

   _HMG_ActiveFormNameBak := _HMG_ActiveFormName

   i := ascan (_HMG_aControlHandles , GridHandle )

   a := _HMG_aControlPageMap [i]
   ControlName := _HMG_aControlNames [i]

   j := ascan ( _HMG_aFormHandles , _HMG_aControlParentHandles [ i ] )

   FormName := _HMG_aFormNames [ j ]

   item := _GetValue ( ControlName , FormName )

   l := Len(a)

   g := _GetItem ( ControlName , FormName , Item )

   IRow := ListViewGetItemRow ( h, Item )

   GetWindowRect ( h, actpos )

//   GRow      := actpos [2]
   GCol      := actpos [1]
   GWidth   := actpos [3] - actpos [1]

   Col := GCol + ( ( GWidth - 260 ) / 2 )

   DEFINE WINDOW _EditItem;
      AT IRow,Col;
      WIDTH 260;
      HEIGHT (l*30) + 70 + GetTitleHeight();
      TITLE _HMG_MESSAGE [5];
      MODAL;
      NOSIZE

      For i := 1 to l
         LN := 'Label_' + Alltrim(Str(i,2,0))
         TN := 'Text_' + Alltrim(Str(i,2,0))
         @ (i*30) - 17 , 10 LABEL &LN OF _EditItem VALUE Alltrim(a[i]) +":"
         @ (i*30) - 20 , 120 TEXTBOX &TN OF _EditItem VALUE g[i]
      Next i

      @ (l*30) + 20 , 20 BUTTON BUTTON_1;
         OF _EDITITEM;
         CAPTION _HMG_MESSAGE [6];
         ACTION { || _EditItemOk ( ControlName , FormName , Item , l ) }

      @ (l*30) + 20 , 130 BUTTON BUTTON_2;
         OF _EDITITEM;
         CAPTION _HMG_MESSAGE [7];
         ACTION _EditItem.Release

   END WINDOW

   _SetFocus ('Text_1','_EditItem')

   ACTIVATE WINDOW _EditItem

   _HMG_ActiveFormName := _HMG_ActiveFormNameBak

   _SetFocus( ControlName , FormName )

Return Nil

*-----------------------------------------------------------------------------*
Function _EditItemOk ( ControlName , FormName , Item , l )
*-----------------------------------------------------------------------------*
   Local i , a [l] , cn

   for i := 1 to l
      cn := 'Text_' + Alltrim(Str(i, 0))

      a [i] := GetProperty ( '_EditItem' , cn , 'Value' )

   next i

   _SetItem( ControlName , FormName , Item , a )

   _EditItem.Release

Return Nil

*-----------------------------------------------------------------------------*
Procedure _AddGridColumn ( cControlName , cParentForm , nColIndex , cCaption , nWidth , nJustify )
*-----------------------------------------------------------------------------*
   Local i , x , atemp := {}

   // Get Control Index

   i := GetControlIndex ( cControlName , cParentForm )

   // Set Default Values

   If ValType ( nColIndex ) == 'U'
      nColIndex := Len ( _HMG_aControlPageMap [i] ) + 1
   EndIf

   If ValType ( cCaption ) == 'U'
      cCaption := ''
   EndIf

   If ValType ( nWidth ) == 'U'
      nWidth := 120
   EndIf

   If ValType ( nJustify ) == 'U'
      nJustify := 0
   EndIf

   // Update Grid Object

   For x := 1 To nColIndex - 1
      aadd ( atemp , _HMG_aControlPageMap [i] [x] )
   Next x

   aadd ( atemp , cCaption )

   For x := nColIndex + 1 To Len ( _HMG_aControlPageMap [i] ) + 1
      aadd ( atemp , _HMG_aControlPageMap [i] [x-1] )
   Next x

   _HMG_aControlPageMap [i] := atemp
   _HMG_aControlCaption [i] := atemp

   // Call C-Level Routine

   ListView_AddColumn( _HMG_aControlHandles[i] , nColIndex , nWidth , cCaption , nJustify )

Return

*-----------------------------------------------------------------------------*
Procedure _DeleteGridColumn ( cControlName , cParentForm , nColIndex )
*-----------------------------------------------------------------------------*
   Local i , x , atemp := {}

   // Get Control Index

   i := GetControlIndex ( cControlName , cParentForm )

   // Update Grid Object

   For x := 1 To nColIndex - 1
      aadd ( atemp , _HMG_aControlPageMap [i] [x] )
   Next x

   For x := nColIndex + 1 To Len ( _HMG_aControlPageMap [i] )
      aadd ( atemp , _HMG_aControlPageMap [i] [x] )
   Next x

   _HMG_aControlPageMap [i] := atemp
   _HMG_aControlCaption [i] := atemp

   // Call C-Level Routine

   ListView_DeleteColumn ( _HMG_aControlHandles[i] , nColIndex )

Return

*-----------------------------------------------------------------------------*
Procedure _UpdateGridColors ( i )
*-----------------------------------------------------------------------------*
   Local dbc , dfc
   Local processdbc , processdfc
   Local Cols , Rows , r , c
   Local aValues , aTemp

   dBc := __MVGET( '_HMG_' + ALLTRIM(STR(_HMG_aControlhandles[i])) + '_DYNAMICBACKCOLOR' )
   dFc := __MVGET( '_HMG_' + ALLTRIM(STR(_HMG_aControlhandles[i])) + '_DYNAMICFORECOLOR' )

   processdbc := ( valtype (dbc) == 'A' )
   processdFc := ( valtype (dfc) == 'A' )

   if processdbc == .F. .and. processdfc == .F.
      Return
   endif

   Rows := ListViewGetItemCount ( _HMG_aControlHandles [i] )

   Cols := Len ( _HMG_aControlCaption [i] )

   if processdbc == .t.

      aTemp := Array ( Rows , Cols )

      for r := 1 to Rows

         aValues := _GetItem ( , , r, i )

         For c := 1 To Cols

            _HMG_ThisItemRowIndex := R
            _HMG_ThisItemColIndex := C

            _HMG_ThisItemCellValue := aValues [c]

            aTemp [r] [c] := if( valtype ( dbc [c] ) == 'B', eval ( dbc [c], aValues, r ) , -1 )

         next c

      next r

      __MVPUT( '_HMG_' + ALLTRIM(STR(_HMG_aControlhandles[i])) + '_DBC' , aTemp )
   else
      __MVPUT( '_HMG_' + ALLTRIM(STR(_HMG_aControlhandles[i])) + '_DBC' , Nil )
   endif

   if processdfc == .t.

      aTemp := Array ( Rows , Cols )

      for r := 1 to Rows

         aValues := _GetItem ( , , r, i )

         For c := 1 To Cols

            _HMG_ThisItemRowIndex := R
            _HMG_ThisItemColIndex := C

            _HMG_ThisItemCellValue := aValues [c]

            aTemp [r] [c] := if( valtype ( dfc [c] ) == 'B', eval ( dfc [c], aValues, r ) , -1 )

         next c

      next r

      __MVPUT( '_HMG_' + ALLTRIM(STR(_HMG_aControlhandles[i])) + '_DFC' , aTemp )
   else
      __MVPUT( '_HMG_' + ALLTRIM(STR(_HMG_aControlhandles[i])) + '_DFC' , Nil )
   endif

   ReDrawWindow ( _HMG_aControlHandles [i] )

Return

*-----------------------------------------------------------------------------*
PROCEDURE _GridInplaceEdit(idx)
*-----------------------------------------------------------------------------*
   LOCAL r , c , v, h , aTemp , ri , ci , DH := 0 , DR := 0
   LOCAL AEDITCONTROLS
   LOCAL AEC := 'TEXTBOX'
   LOCAL AITEMS
   LOCAL ARANGE
   LOCAL DTYPE
   LOCAL ALABELS := { '.T.' ,'.F.' }
   LOCAL CTYPE
   LOCAL CINPUTMASK
   LOCAL CFORMAT
   LOCAL XRES
   LOCAL CWH, WHEN, abKeyBlocks := {}

   If This.CellRowIndex != LISTVIEW_GETFIRSTITEM ( _hmg_acontrolhandles [ idx ] )
      Return
   EndIf

   ri := This.CellRowIndex
   ci := This.CellColIndex

   if ri == 0 .or. ci == 0
      return
   endif

   aTemp := This.Item(ri)

   v := aTemp [ci]

   CWH :=    __MVGET   ( '_HMG_' + ALLTRIM(STR(_HMG_aControlHandles[idx])) + '_COLUMNWHEN' )

   IF VALTYPE ( CWH ) == 'A' .AND. LEN ( CWH ) >= ci

      IF VALTYPE ( CWH [CI] ) == 'B'

         _HMG_THISItemCellValue := v

         WHEN := EVAL ( CWH [ci] )

         IF WHEN = .F.
            _HMG_IPE_CANCELLED := .F.
            RETURN
         ENDIF

      ENDIF

   ENDIF

   h := _HMG_aControlHandles [idx]

   r := This.CellRow + GetWindowRow ( h ) - This.Row - 1

   if _HMG_aControlContainerRow  [idx] <> -1
      r := r - _HMG_aControlContainerRow [idx]
   endif

   c := This.CellCol + GetWindowCol ( h ) - This.Col + 2

   if _HMG_aControlContainerCol [idx] <> -1
      c := c - _HMG_aControlContainerCol [idx]
   endif

   aEditControls := __MVGET ( '_HMG_' + ALLTRIM(STR(_HMG_aControlHandles[ idx ] )) + '_EDITCONTROLS' )

   XRES := _ParseGridControls ( aEditControls , ci )

   AEC      := XRES [1]
   CTYPE      := XRES [2]
   CINPUTMASK   := XRES [3]
   CFORMAT      := XRES [4]
   AITEMS      := XRES [5]
   ARANGE      := XRES [6]
   DTYPE      := XRES [7]
   ALABELS      := XRES [8]

   IF AEC = 'COMBOBOX'
      DH := 1
   ELSEIF AEC = 'CHECKBOX'
      DR := 3
      DH := -7
   ENDIF

   _HMG_InplaceParentHandle := GetActiveWindow()

   DEFINE WINDOW _hmg_grid_inplaceedit AT r + DR , c;
      WIDTH This.CellWidth;
      HEIGHT This.CellHeight + 6 + DH;
      TITLE '' MODAL;
      NOSIZE;
      NOCAPTION

      ON KEY ESCAPE OF W ACTION ( _HMG_IPE_CANCELLED := .T. , ThisWindow.Release )
      ON KEY RETURN OF W ACTION IF ( _IsWindowActive ( '_hmg_grid_inplaceedit' ) , ( _HMG_IPE_CANCELLED := .F. , _GridInplaceEditOK ( IDX , CI , RI , AEC ) ) , NIL )

      IF AEC == 'TEXTBOX'

         DEFINE TEXTBOX TG
         ROW      0
         COL      0
         WIDTH       This.CellWidth
         HEIGHT      This.CellHeight + 6
         FONTNAME   _hmg_aControlFontName [idx]
         FONTSIZE   _hmg_aControlFontSize [idx]

         IF CTYPE == 'NUMERIC'
            NUMERIC .T.
         ELSEIF CTYPE == 'DATE'
            DATE .T.
         ENDIF

         VALUE      v

         IF ! EMPTY ( CINPUTMASK )
            INPUTMASK CINPUTMASK
         ENDIF

         IF ! EMPTY ( CFORMAT )
            FORMAT CFORMAT
         ENDIF

         END TEXTBOX

         IF EMPTY ( CINPUTMASK ) .AND. EMPTY ( CFORMAT )
            _hmg_grid_inplaceedit.TG.setfocus
         ENDIF

      ELSEIF AEC == 'DATEPICKER'

         DEFINE DATEPICKER D
         ROW      0
         COL      0
         WIDTH       This.CellWidth
         HEIGHT      This.CellHeight + 6
         VALUE      v
         FONTNAME   _hmg_aControlFontName [idx]
         FONTSIZE   _hmg_aControlFontSize [idx]

         IF DTYPE = 'DROPDOWN'
            UPDOWN .F.
         ELSEIF DTYPE = 'UPDOWN'
            UPDOWN .T.
         ENDIF

         END DATEPICKER

      ELSEIF AEC == 'COMBOBOX'

         DEFINE COMBOBOX C
         ROW      0
         COL      0
         WIDTH       This.CellWidth
         ITEMS      AITEMS
         VALUE      v
         FONTNAME   _hmg_aControlFontName [idx]
         FONTSIZE   _hmg_aControlFontSize [idx]
         ON LISTDISPLAY ( AADD(abKeyBlocks, _GetHotKey ( '_hmg_grid_inplaceedit', 0, 27 )),;
            _ReleaseHotKey ('_hmg_grid_inplaceedit',0,27 ),;
            AADD(abKeyBlocks, _GetHotKey ('_hmg_grid_inplaceedit', 0, 13 )),;
            _ReleaseHotKey ('_hmg_grid_inplaceedit',0,13 ) )
         ON LISTCLOSE ( _DefineHotKey('_hmg_grid_inplaceedit', 0, 27, abKeyBlocks[1]),;
            _DefineHotKey('_hmg_grid_inplaceedit', 0, 13, abKeyBlocks[2]),;
            abKeyBlocks := {} )
         END COMBOBOX

      ELSEIF AEC == 'SPINNER'

         DEFINE SPINNER S
         ROW      0
         COL      0
         WIDTH       This.CellWidth
         HEIGHT      This.CellHeight + 6
         VALUE      v
         RANGEMIN   ARANGE [1]
         RANGEMAX   ARANGE [2]
         FONTNAME   _hmg_aControlFontName [idx]
         FONTSIZE   _hmg_aControlFontSize [idx]
         END SPINNER

      ELSEIF AEC == 'CHECKBOX'

         DEFINE CHECKBOX C
         ROW      0
         COL      0
         WIDTH       This.CellWidth
         HEIGHT      This.CellHeight + 6 + DH
         VALUE      v
         FONTNAME   _hmg_aControlFontName [idx]
         FONTSIZE   _hmg_aControlFontSize [idx]

         IF V == .T.
            CAPTION ALABELS [1]
         ELSEIF V == .F.
            CAPTION ALABELS [2]
         ENDIF

         BACKCOLOR WHITE
         ONCHANGE IF ( THIS.VALUE == .T. , THIS.CAPTION := ALABELS [1] , THIS.CAPTION := ALABELS [2] )
         END CHECKBOX

      ENDIF

   END WINDOW

   ACTIVATE WINDOW _hmg_grid_inplaceedit

   _HMG_InplaceParentHandle := 0

RETURN

*-----------------------------------------------------------------------------*
Function _ParseGridControls ( aEditControls , ci )
*-----------------------------------------------------------------------------*
   LOCAL AEC := 'TEXTBOX'
   LOCAL AITEMS := {}
   LOCAL ARANGE := {}
   LOCAL DTYPE := 'D'
   LOCAL ALABELS := { '.T.' ,'.F.' }
   LOCAL CTYPE := 'CHARACTER'
   LOCAL CINPUTMASK := ''
   LOCAL CFORMAT := ''

   LOCAL aRet

   IF VALTYPE ( aEditControls ) = 'A'

      IF LEN ( aEditControls ) >= ci

         IF VALTYPE ( aEditControls [CI] ) = 'A'

            IF LEN ( aEditControls [CI] ) >= 1

               AEC := aEditControls [CI] [1]

               IF LEN ( aEditControls [CI] ) >= 2 .AND. AEC == 'TEXTBOX'

                  IF VALTYPE ( AEDITCONTROLS [CI] [2] ) = 'C'
                     CTYPE := AEDITCONTROLS [CI] [2]
                  ENDIF

                  IF LEN ( AEDITCONTROLS [CI] ) >= 3
                     IF VALTYPE ( AEDITCONTROLS [CI] [3] ) = 'C'
                        CINPUTMASK := AEDITCONTROLS [CI] [3]
                     ENDIF
                  ENDIF

                  IF LEN ( AEDITCONTROLS [CI] ) >= 4
                     IF VALTYPE ( AEDITCONTROLS [CI] [4] ) = 'C'
                        CFORMAT := AEDITCONTROLS [CI] [4]
                     ENDIF
                  ENDIF

               ENDIF

               IF LEN ( aEditControls [CI] ) >= 2 .AND. AEC == 'COMBOBOX'
                  IF VALTYPE ( AEDITCONTROLS [CI] [2] ) = 'A'
                     AITEMS := AEDITCONTROLS [CI] [2]
                  ENDIF
               ENDIF

               IF LEN ( aEditControls [CI] ) == 3 .AND. AEC == 'SPINNER'
                  IF   VALTYPE ( AEDITCONTROLS [CI] [2] ) = 'N' .AND.;
                     VALTYPE ( AEDITCONTROLS [CI] [3] ) = 'N'
                     ARANGE := { AEDITCONTROLS [CI] [2] , AEDITCONTROLS [CI] [3] }
                  ENDIF
               ENDIF

               IF LEN ( aEditControls [CI] ) >= 2 .AND. AEC == 'DATEPICKER'
                  IF    VALTYPE ( AEDITCONTROLS [CI] [2] ) = 'C'
                     DTYPE := AEDITCONTROLS [CI] [2]
                  ENDIF
               ENDIF

               IF LEN ( aEditControls [CI] ) == 3 .AND. AEC == 'CHECKBOX'
                  IF    VALTYPE ( AEDITCONTROLS [CI] [2] ) = 'C' .AND.;
                     VALTYPE ( AEDITCONTROLS [CI] [3] ) = 'C'
                     ALABELS := { AEDITCONTROLS [CI] [2] , AEDITCONTROLS [CI] [3] }
                  ENDIF
               ENDIF

            ENDIF

         ENDIF

      ENDIF

   ENDIF

   aRet := { AEC , CTYPE , CINPUTMASK , CFORMAT , AITEMS , ARANGE , DTYPE , ALABELS }

RETURN ( aRet )

*-----------------------------------------------------------------------------*
PROCEDURE _GridInplaceEditOK ( IDX , CI , RI , AEC )
*-----------------------------------------------------------------------------*
   LOCAL CVA, VALID, aTemp

   CVA :=    __MVGET   ( '_HMG_' + ALLTRIM(STR(_HMG_aControlHandles[idx])) + '_COLUMNVALID' )

   IF VALTYPE ( CVA ) = 'A' .AND. LEN ( CVA ) >= ci

      IF VALTYPE ( CVA [CI] ) = 'B'

         IF   AEC == 'TEXTBOX'
            _HMG_THISItemCellValue := _hmg_grid_inplaceedit.tg.value
         ELSEIF   AEC == 'DATEPICKER'
            _HMG_THISItemCellValue := _hmg_grid_inplaceedit.d.value
         ELSEIF   AEC == 'COMBOBOX'
            _HMG_THISItemCellValue := _hmg_grid_inplaceedit.c.value
         ELSEIF   AEC == 'SPINNER'
            _HMG_THISItemCellValue := _hmg_grid_inplaceedit.s.value
         ELSEIF   AEC == 'CHECKBOX'
            _HMG_THISItemCellValue := _hmg_grid_inplaceedit.c.value
         ENDIF

         VALID := EVAL ( CVA [ci] )

         IF VALID = .F.

            MSGEXCLAMATION ( _HMG_BRWLangError[11] )
            RETURN

         ENDIF

      ENDIF

   ENDIF

   aTemp := _GetItem ( , , ri , idx )

   IF   AEC == 'TEXTBOX'
      aTemp [ci] := _hmg_grid_inplaceedit.tg.value
   ELSEIF   AEC == 'DATEPICKER'
      aTemp [ci] := _hmg_grid_inplaceedit.d.value
   ELSEIF   AEC == 'COMBOBOX'
      aTemp [ci] := _hmg_grid_inplaceedit.c.value
   ELSEIF   AEC == 'SPINNER'
      aTemp [ci] := _hmg_grid_inplaceedit.s.value
   ELSEIF   AEC == 'CHECKBOX'
      aTemp [ci] := _hmg_grid_inplaceedit.c.value
   ENDIF

   _SetItem ( , , ri , aTemp , idx )

   _hmg_grid_inplaceedit.Release

RETURN

*-----------------------------------------------------------------------------*
PROCEDURE _GridInplaceKbdEdit(i)
*-----------------------------------------------------------------------------*
   LOCAL IPE_MAXCOL := Len ( _HMG_aControlCaption [i] )
   LOCAL TmpRow
   LOCAL XS
   LOCAL XD
   LOCAL R

   Do While .T.

      TmpRow := LISTVIEW_GETFIRSTITEM ( _HMG_aControlHandles [i] )

      If TmpRow != _HMG_IPE_ROW

         _HMG_IPE_ROW := TmpRow

         if Len ( _HMG_aControlBkColor [i] ) > 0
            _HMG_IPE_COL := 2
         Else
            _HMG_IPE_COL := 1
         EndIf

      EndIf

      _HMG_ThisItemRowIndex := _HMG_IPE_ROW
      _HMG_ThisItemColIndex := _HMG_IPE_COL

      If _HMG_IPE_COL == 1
         r := LISTVIEW_GETITEMRECT ( _HMG_aControlHandles [i]  , _HMG_IPE_ROW - 1 )
      Else
         r := LISTVIEW_GETSUBITEMRECT ( _HMG_aControlHandles [i]  , _HMG_IPE_ROW - 1 , _HMG_IPE_COL - 1 )
      EndIf

      xs :=   ( ( _HMG_aControlCol [i] + r [2] ) +( r[3] ))  -  ( _HMG_aControlCol [i] + _HMG_aControlWidth [i] )

      xd := 20

      If xs > -xd
         ListView_Scroll( _HMG_aControlHandles [i] ,   xs + xd , 0 )
      Else

         If r [2] < 0
            ListView_Scroll( _HMG_aControlHandles [i] , r[2]   , 0 )
         EndIf

      EndIf

      If _HMG_IPE_COL == 1
         r := LISTVIEW_GETITEMRECT ( _HMG_aControlHandles [i]  , _HMG_IPE_ROW - 1 )
      Else
         r := LISTVIEW_GETSUBITEMRECT ( _HMG_aControlHandles [i]  , _HMG_IPE_ROW - 1 , _HMG_IPE_COL - 1 )
      EndIf

      _HMG_ThisItemCellRow := IFNUMERIC( _HMG_aControlRow [i] , _HMG_aControlRow [i] , 0 ) + r [1]
      _HMG_ThisItemCellCol := IFNUMERIC( _HMG_aControlCol [i] , _HMG_aControlCol [i] , 0 ) + r [2]
      _HMG_ThisItemCellWidth := r[3]
      _HMG_ThisItemCellHeight := r[4]

      _HMG_ThisFormIndex := ascan ( _HMG_aFormHandles , _HMG_aControlParentHandles[i] )
      _HMG_ThisType := 'C'
      _HMG_ThisIndex := i

      _HMG_ThisFormName :=  _HMG_aFormNames [ _HMG_ThisFormIndex ]
      _HMG_ThisControlName :=  _HMG_aControlNames [_HMG_THISIndex]

      _GridInplaceEdit(i)

      _HMG_ThisIndex := 0
      _HMG_ThisType := ''

      _HMG_ThisItemRowIndex := 0
      _HMG_ThisItemColIndex := 0
      _HMG_ThisItemCellRow := 0
      _HMG_ThisItemCellCol := 0
      _HMG_ThisItemCellWidth := 0
      _HMG_ThisItemCellHeight := 0

      _HMG_ThisFormIndex := 0
      _HMG_ThisEventType := ''
      _HMG_ThisFormName :=  ''
      _HMG_ThisControlName := ''

      If _HMG_IPE_CANCELLED == .T.

         If _HMG_IPE_COL == IPE_MAXCOL

            if Len ( _HMG_aControlBkColor [i] ) > 0
               _HMG_IPE_COL := 2
            Else
               _HMG_IPE_COL := 1
            EndIf

            ListView_Scroll( _HMG_aControlHandles [i] , -10000 , 0 )
         EndIf

         Exit

      Else

         _HMG_IPE_COL++

         If _HMG_IPE_COL > IPE_MAXCOL

            if Len ( _HMG_aControlBkColor [i] ) > 0
               _HMG_IPE_COL := 2
            Else
               _HMG_IPE_COL := 1
            EndIf

            ListView_Scroll( _HMG_aControlHandles [i] , -10000 , 0 )
            Exit
         EndIf

      EndIf

   EndDo

RETURN

*-----------------------------------------------------------------------------*
Function GetNumFromCellText ( Text )
*-----------------------------------------------------------------------------*
   Local x , c , s

   s := ''

   For x := 1 To Len ( Text )

      c := SubStr(Text,x,1)

      If c='0' .or. c='1' .or. c='2' .or. c='3' .or. c='4' .or. c='5' .or. c='6' .or. c='7' .or. c='8' .or. c='9' .or. c='.' .or. c='-'
         s := s + c
      EndIf

   Next x

   If Left ( AllTrim(Text) , 1 ) == '(' .OR.  Right ( AllTrim(Text) , 2 ) == 'DB'
      s := '-' + s
   EndIf

Return Val(s)

*-----------------------------------------------------------------------------*
Function GETNumFromCellTextSP(Text)
*-----------------------------------------------------------------------------*
   Local x , c , s

   s := ''

   For x := 1 To Len ( Text )

      c := SubStr(Text,x,1)

      If c='0' .or. c='1' .or. c='2' .or. c='3' .or. c='4' .or. c='5' .or. c='6' .or. c='7' .or. c='8' .or. c='9' .or. c=',' .or. c='-' .or. c = '.'

         if c == '.'
            c :=''
         endif

         IF C == ','
            C:= '.'
         ENDIF

         s := s + c

      EndIf

   Next x

   If Left ( AllTrim(Text) , 1 ) == '(' .OR.  Right ( AllTrim(Text) , 2 ) == 'DB'
      s := '-' + s
   EndIf

Return Val(s)

*-----------------------------------------------------------------------------*
FUNCTION _GetGridCellValue ( ControlName , ParentForm , Row , Col )
*-----------------------------------------------------------------------------*
   LOCAL A := _GetItem ( ControlName , ParentForm , Row )
   LOCAL V := A [ COL ]

RETURN V

*-----------------------------------------------------------------------------*
PROCEDURE _SetGridCellValue ( ControlName , ParentForm , Row , Col , CellValue )
*-----------------------------------------------------------------------------*
   LOCAL A := _GetItem ( ControlName , ParentForm , Row )

   A [ COL ] := CellValue

   _SetItem ( ControlName , ParentForm , Row , A )

RETURN

// (JK) HMG 1.0 Experimental Build 6
/*
   _GetColumnWidth( ControlName , ParentForm, nColumnNo )->nColumnWidth

   _SetColumnWidth( ControlName , ParentForm, nColumnNo ,nWidth )->lSuccess

   _SetColumnWidthAuto( ControlName , ParentForm, nColumnNo )->lSuccess
   - Autofit column width to largest grid item width in column nColumnNo

   _SetColumnWidthAutoH( ControlName , ParentForm, nColumnNo )->lSuccess
   - Autofit column width to largest grid header width in column nColumnNo

   _SetColumnsWidthAuto( ControlName , ParentForm )
   - Autofit all columns width to largest grid item width in column

   _SetColumnsWidthAutoH( ControlName , ParentForm )
   - Autofit all columns width to largest grid header width in column
*/
*-----------------------------------------------------------------------------*
Function _GetColumnWidth( ControlName , ParentForm, nColumnNo )
*-----------------------------------------------------------------------------*
   Local nWidth, i, z := nColumnNo
   i := GetControlIndex( ControlName , ParentForm )
   nWidth := ListView_GetColumnWidth( _HMG_aControlHandles [i] , z - 1 )
Return nWidth

*-----------------------------------------------------------------------------*
Function _SetColumnWidth( ControlName , ParentForm , nColumnNo , nWidth )
*-----------------------------------------------------------------------------*
   Local z := nColumnNo, i, lSuccess
   i := GetControlIndex( ControlName , ParentForm )
   lSuccess := ListView_SetColumnWidth( _HMG_aControlHandles [i] , z - 1 , nWidth )
Return lSuccess

*-----------------------------------------------------------------------------*
Function _SetColumnWidthAuto( ControlName , ParentForm , nColumnNo )
*-----------------------------------------------------------------------------*
   Local z := nColumnNo, i, lSuccess
   i := GetControlIndex( ControlName , ParentForm )
   lSuccess := ListView_SetColumnWidthAuto( _HMG_aControlHandles [i], z - 1 )
Return lSuccess

*-----------------------------------------------------------------------------*
Function _SetColumnWidthAutoH( ControlName , ParentForm , nColumnNo )
*-----------------------------------------------------------------------------*
   Local z := nColumnNo, i, lSuccess
   i := GetControlIndex( ControlName , ParentForm )
   lSuccess := ListView_SetColumnWidthAutoH( _HMG_aControlHandles [i], z - 1 )
Return lSuccess

*-----------------------------------------------------------------------------*
Function _SetColumnsWidthAuto( ControlName , ParentForm )
*-----------------------------------------------------------------------------*
   Local h, z, i, ColumnCount, lSuccess := .f.
   i := GetControlIndex( ControlName , ParentForm )
   h := _HMG_aControlHandles [i]
   ColumnCount:= len(_HMG_aControlCaption [i])
   If ColumnCount>0
      For z := 1 to ColumnCount
         lSuccess := ListView_SetColumnWidthAuto( h, z - 1 )
      next z
   endif
Return lSuccess

*-----------------------------------------------------------------------------*
Function _SetColumnsWidthAutoH( ControlName , ParentForm )
*-----------------------------------------------------------------------------*
   Local h, z, i, ColumnCount, lSuccess := .f.
   i := GetControlIndex ( ControlName , ParentForm )
   h := _HMG_aControlHandles [i]
   ColumnCount:= len(_HMG_aControlCaption [i])
   If ColumnCount>0
      For z := 1 to ColumnCount
         lSuccess := ListView_SetColumnWidthAutoH( h, z - 1 )
      next z
   endif
Return lSuccess

   //(JP) HMG 1.4 Extended Build 40
*-----------------------------------------------------------------------------*
Procedure _GridPgDn ( ControlName , ParentForm , z )
*-----------------------------------------------------------------------------*
   Local i , PageLength , _DeltaScroll /*[4]*/ , s

   If pcount() == 2
      i := GetControlIndex ( ControlName , ParentForm )
   Else
      i := z
   EndIf

   _DeltaScroll := ListView_GetSubItemRect ( _HMG_aControlhandles[i] , 0 , 0 )
   s            := LISTVIEW_GETFIRSTITEM ( _HMG_aControlHandles [i] )
   PageLength   := LISTVIEWGETCOUNTPERPAGE ( _HMG_aControlhandles[i] )
   IF s + PageLength <=  LISTVIEWGETITEMCOUNT( _HMG_aControlhandles[i] )
      ListView_SetCursel ( _HMG_aControlHandles [i] , s + PageLength )
      ListView_Scroll( _HMG_aControlHandles[i],0 , _DeltaScroll[4] * PageLength )
   ELSE
      ListView_SetCursel ( _HMG_aControlHandles [i] , LISTVIEWGETITEMCOUNT( _HMG_aControlhandles[i] ) )
      ListView_Scroll( _HMG_aControlHandles[i], 0, _DeltaScroll[4] * (LISTVIEWGETITEMCOUNT( _HMG_aControlhandles[i] ) - s) )
   ENDIF

Return

*-----------------------------------------------------------------------------*
Procedure _GridPgUp ( ControlName , ParentForm , z )
*-----------------------------------------------------------------------------*
   Local i , PageLength , s , _DeltaScroll /*[4]*/

   If pcount() == 2
      i := GetControlIndex ( ControlName , ParentForm )
   Else
      i := z
   EndIf

   _DeltaScroll := ListView_GetSubItemRect ( _HMG_aControlhandles[i] , 0 , 0 )
   s      := LISTVIEW_GETFIRSTITEM ( _HMG_aControlHandles [i] )
   PageLength   := LISTVIEWGETCOUNTPERPAGE ( _HMG_aControlhandles[i] )

   IF s - PageLength >= 1
      ListView_SetCursel ( _HMG_aControlHandles [i] , s-PageLength )
      ListView_Scroll( _HMG_aControlHandles[i],0 , _DeltaScroll[4] * PageLength * (-1) )
   ELSE
      ListView_SetCursel ( _HMG_aControlHandles [i] , 1 )
      ListView_Scroll( _HMG_aControlHandles[i], 0, _DeltaScroll[4] * s * (-1) )
   ENDIF

Return

*-----------------------------------------------------------------------------*
Procedure _GridHome ( ControlName , ParentForm , z )
*-----------------------------------------------------------------------------*
   Local i , s , _DeltaScroll /*[4]*/

   If pcount() == 2
      i := GetControlIndex ( ControlName , ParentForm )
   Else
      i := z
   EndIf

   _DeltaScroll := ListView_GetSubItemRect ( _HMG_aControlhandles[i] , 0 , 0 )
   s      := LISTVIEW_GETFIRSTITEM ( _HMG_aControlHandles [i] )

   ListView_Scroll( _HMG_aControlHandles[i], 0, _DeltaScroll[4] * (-1) * s )

   ListView_SetCursel ( _HMG_aControlHandles [i], 1 )

Return

*-----------------------------------------------------------------------------*
Procedure _GridEnd ( ControlName , ParentForm , z )
*-----------------------------------------------------------------------------*
   Local i , s , _DeltaScroll /*[4]*/ , _BottomPos

   If pcount() == 2
      i := GetControlIndex ( ControlName , ParentForm )
   Else
      i := z
   EndIf

   _DeltaScroll := ListView_GetSubItemRect ( _HMG_aControlhandles[i] , 0 , 0 )
   _BottomPos   := LISTVIEWGETITEMCOUNT( _HMG_aControlhandles[i] )
   s      := LISTVIEW_GETFIRSTITEM ( _HMG_aControlHandles [i] )

   ListView_Scroll( _HMG_aControlHandles[i] , 0 , _DeltaScroll[4] * (_BottomPos - s) )

   ListView_SetCursel ( _HMG_aControlHandles [i],  _BottomPos )

Return

*-----------------------------------------------------------------------------*
Procedure _GridPrior ( ControlName , ParentForm , z )
*-----------------------------------------------------------------------------*
   Local i /*, PageLength*/ , s , TopInx , _DeltaScroll /*[4]*/

   If pcount() == 2
      i := GetControlIndex ( ControlName , ParentForm )
   Else
      i := z
   EndIf

   _DeltaScroll := ListView_GetSubItemRect ( _HMG_aControlhandles[i] , 0 , 0 )
   TopInx   := LISTVIEW_GETTOPINDEX ( _HMG_aControlHandles [i] )
//   PageLength   := LISTVIEWGETCOUNTPERPAGE ( _HMG_aControlhandles[i] )

   s := LISTVIEW_GETFIRSTITEM ( _HMG_aControlHandles [i] )
   ListView_SetCursel ( _HMG_aControlHandles [i] , s - 1 )

   If s <= TopInx + 1
      ListView_Scroll( _HMG_aControlHandles[i], 0, _DeltaScroll[4] * (-1) )
   EndIf

Return

*-----------------------------------------------------------------------------*
Procedure _GridNext ( ControlName , ParentForm , z )
*-----------------------------------------------------------------------------*
   Local i , PageLength , s , TopInx , _DeltaScroll /*[4]*/

   If pcount() == 2
      i := GetControlIndex ( ControlName , ParentForm )
   Else
      i := z
   EndIf

   _DeltaScroll := ListView_GetSubItemRect ( _HMG_aControlhandles[i] , 0 , 0 )
   s      := LISTVIEW_GETFIRSTITEM ( _HMG_aControlHandles [i] )
   TopInx   := LISTVIEW_GETTOPINDEX ( _HMG_aControlHandles [i] )
   PageLength   := LISTVIEWGETCOUNTPERPAGE ( _HMG_aControlhandles[i] )
   ListView_SetCursel ( _HMG_aControlHandles [i] , s + 1 )

   If s - TopInx == PageLength
      ListView_Scroll( _HMG_aControlHandles[i], 0, _DeltaScroll[4] )
   endif

Return

*-----------------------------------------------------------------------------*
Procedure _GridScrollToPos ( ControlName , ParentForm , z )
*-----------------------------------------------------------------------------*
   Local i, s, TopInx, PageLength, _DeltaScroll /*[4]*/

   If pcount() == 2
      i := GetControlIndex ( ControlName , ParentForm )
   Else
      i := z
   EndIf

   _DeltaScroll := ListView_GetSubItemRect ( _HMG_aControlhandles[i] , 0 , 0 )
   s      := LISTVIEW_GETFIRSTITEM ( _HMG_aControlHandles [i] )
   TopInx   := LISTVIEW_GETTOPINDEX ( _HMG_aControlHandles [i] )
   PageLength   := LISTVIEWGETCOUNTPERPAGE ( _HMG_aControlhandles[i] )
   IF topInx < s
      IF s - TopInx > PageLength
         ListView_Scroll( _HMG_aControlHandles[i], 0, _DeltaScroll[4] * (s - TopInx - PageLength) )
      endif
   ELSE
      ListView_Scroll( _HMG_aControlHandles[i], 0, _DeltaScroll[4] * (-1) * (s - TopInx) )
   endif

Return