/*----------------------------------------------------------------------------
MINIGUI - Harbour Win32 GUI library source code

Copyright 2002-2008 Roberto Lopez <harbourminigui@gmail.com>
http://harbourminigui.googlepages.com/

IMAGELIST control source code
(C)2005 Janusz Pora <januszpora@onet.eu>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along with
   this software; see the file COPYING. If not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA (or
   visit the web site http://www.gnu.org/).

   As a special exception, you have permission for additional uses of the text
   contained in this release of Harbour Minigui.

   The exception is that, if you link the Harbour Minigui library with other
   files to produce an executable, this does not by itself cause the resulting
   executable to be covered by the GNU General Public License.
   Your use of that executable is in no way restricted on account of linking the
   Harbour-Minigui library code into it.

   Parts of this project are based upon:

   "Harbour GUI framework for Win32"
   Copyright 2001 Alexander S.Kresin <alex@belacy.belgorod.su>
   Copyright 2001 Antonio Linares <alinares@fivetech.com>
   www - http://www.harbour-project.org

   "Harbour Project"
   Copyright 1999-2008, http://www.harbour-project.org/

   "WHAT32"
   Copyright 2002 AJ Wos <andrwos@aust1.net>

   "HWGUI"
   Copyright 2001-2007 Alexander S.Kresin <alex@belacy.belgorod.su>

---------------------------------------------------------------------------*/

#include "minigui.ch"

#define MAXIMAGE 10
*-----------------------------------------------------------------------------*
Function _DefineImageList ( ControlName , ParentForm , w , h , aImage , aImageMask , aColor , ImageCount , mask )
*-----------------------------------------------------------------------------*
   Local i , mVar , maskimage , color , k
   Local ControlHandles , id , PosImage

   DEFAULT w               TO 24
   DEFAULT h               TO 24
   DEFAULT aImage          TO {}
   DEFAULT aImageMask      TO {}
   DEFAULT aColor          TO {0,0,0}
   DEFAULT ImageCount      TO 0
   DEFAULT mask            TO .f.

   if _HMG_BeginWindowActive = .T.
      ParentForm := _HMG_ActiveFormName
   endif

   If .Not. _IsWindowDefined (ParentForm)
      MsgMiniGuiError( "Window: "+ ParentForm + " is not defined. Program terminated" )
   Endif

   If _IsControlDefined (ControlName,ParentForm)
      MsgMiniGuiError( "Control: " + ControlName + " Of " + ParentForm + " Already defined. Program Terminated" )
   Endif

   mVar := '_' + ParentForm + '_' + ControlName

   if ImageCount == 0
      if len (aImage) == 0
         ImageCount := MAXIMAGE
      else
         ImageCount := len(aImage)
      EndIf
   endif

   Id := _GetId()
   ControlHandles := InitImageList ( w , h, mask, ImageCount )

   k := _GetControlFree()

   Public &mVar. := k

   _HMG_aControlType [k] := "IMAGELIST"
   _HMG_aControlNames [k] :=   ControlName
   _HMG_aControlHandles [k] :=   ControlHandles
   _HMG_aControlParenthandles [k] :=   GetFormhandle(Upper(ParentForm))
   _HMG_aControlIds [k] :=   id
   _HMG_aControlProcedures  [k] :=  ""
   _HMG_aControlPageMap  [k] :=  {}
   _HMG_aControlValue  [k] :=  ImageCount
   _HMG_aControlInputMask  [k] :=  ""
   _HMG_aControllostFocusProcedure  [k] :=  ""
   _HMG_aControlGotFocusProcedure  [k] :=  ""
   _HMG_aControlChangeProcedure  [k] :=  ""
   _HMG_aControlDeleted  [k] :=  .F.
   _HMG_aControlBkColor   [k] := Nil
   _HMG_aControlFontColor   [k] := Nil
   _HMG_aControlDblClick   [k] := ""
   _HMG_aControlHeadClick   [k] := {}
   _HMG_aControlRow  [k] :=  0
   _HMG_aControlCol  [k] :=  0
   _HMG_aControlWidth   [k] := w
   _HMG_aControlHeight  [k] :=  h
   _HMG_aControlSpacing  [k] :=  0
   _HMG_aControlContainerRow  [k] :=  iif ( _HMG_FrameLevel > 0 ,_HMG_ActiveFrameRow [_HMG_FrameLevel] , -1 )
   _HMG_aControlContainerCol  [k] :=  iif ( _HMG_FrameLevel > 0 ,_HMG_ActiveFrameCol [_HMG_FrameLevel] , -1 )
   _HMG_aControlPicture  [k] :=  ""
   _HMG_aControlContainerHandle  [k] :=  0
   _HMG_aControlFontName  [k] :=  ''
   _HMG_aControlFontSize  [k] :=  0
   _HMG_aControlFontAttributes  [k] :=  {.f.,.f.,.f.,.f.}
   _HMG_aControlToolTip   [k] :=  ''
   _HMG_aControlRangeMin  [k] :=   0
   _HMG_aControlRangeMax  [k] :=   0
   _HMG_aControlCaption  [k] :=   ''
   _HMG_aControlVisible  [k] :=   .t.
   _HMG_aControlHelpId  [k] :=   0
   _HMG_aControlFontHandle  [k] :=   0
   _HMG_aControlBrushHandle  [k] :=  0
   _HMG_aControlEnabled  [k] :=   .T.
   _HMG_aControlMiscData1 [k] := 0

   for i = 1 to len (aImage)
      If mask
         if len(aImageMask) > 0
            maskimage := if(i <= len(aImageMask), aImageMask[i], "")
            PosImage := IL_Add( ControlHandles , aImage[i] , maskimage , w , h , ImageCount )
         else
            if valtype(aColor) == 'A'
               color := RGB ( aColor[1] , aColor[2] , aColor[3] )
            else
               color := aColor
            endif
            PosImage := IL_AddMasked( ControlHandles , aImage[i] , color , w , h , ImageCount )
         endif
      Else
         PosImage := IL_Add( ControlHandles , aImage[i] , "" , w , h )
      Endif
      If PosImage == -1
         MsgMiniGuiError("Image: "+ aImage[i] + " is not added. Check image size" )
//            Exit
      Endif
   next

Return Nil

*-----------------------------------------------------------------------------*
Function _AddImageToImageList ( ControlName, ParentControl, Image , MaskImage )
*-----------------------------------------------------------------------------*
   LOCAl w , h , c
   w := _GetControlWidth (ControlName, ParentControl)
   h := _GetControlHeight (ControlName, ParentControl)
   c = GetControlHandle (ControlName , ParentControl )
   if valtype (maskimage) == 'U'
      maskimage := ""
   EndIf

Return IL_Add( c , image , maskimage , w , h )

*-----------------------------------------------------------------------------*
Function _AddImageMaskedToImageList ( ControlName, ParentControl, Image , aColor )
*-----------------------------------------------------------------------------*
   LOCAl w , h , c, color := 0
   w := _GetControlWidth (ControlName, ParentControl)
   h := _GetControlHeight (ControlName, ParentControl)
   c := GetControlHandle (ControlName , ParentControl )
   if valtype(aColor) == 'A'
      color := RGB ( aColor[1] , aColor[2] , aColor[3] )
   endif

Return IL_AddMasked( c , image , color , w , h )

*-----------------------------------------------------------------------------*
Function _RemoveImageFromImageList ( ControlName, ParentControl, ImageIndex )
*-----------------------------------------------------------------------------*
   LOCAl c := GetControlHandle ( ControlName , ParentControl )

Return IL_Remove( c , ImageIndex )

*-----------------------------------------------------------------------------*
Function _DrawImageFromImageList ( ControlName, ParentControl, ImageIndex, cx , cy )
*-----------------------------------------------------------------------------*
   LOCAl c := GetControlHandle (ControlName , ParentControl )

Return IL_Draw(GetFormHandle (ParentControl), c , ImageIndex , cx , cy )

*-----------------------------------------------------------------------------*
Function _DestroyImageList ( ControlName, ParentControl )
*-----------------------------------------------------------------------------*
   LOCAl c := GetControlHandle ( ControlName , ParentControl )

Return IL_Destroy( c )

*-----------------------------------------------------------------------------*
Function _ImageListSetBkColor ( ControlName, ParentControl, acolor )
*-----------------------------------------------------------------------------*
   LOCAl c, color := 0
   c := GetControlHandle ( ControlName , ParentControl )
   if valtype(aColor) == 'A'
      color := RGB ( aColor[1] , aColor[2] , aColor[3] )
   endif
Return IL_SetBkColor( c , color )

*-----------------------------------------------------------------------------*
Function _EraseImage ( ControlName, ParentControl, ix , iy)
*-----------------------------------------------------------------------------*
   LOCAl w , h
   w := _GetControlWidth (ControlName, ParentControl)
   h := _GetControlHeight (ControlName, ParentControl)
   IL_EraseImage(GetFormHandle (ParentControl) , ix, iy ,w, h)
Return Nil

*-----------------------------------------------------------------------------*
Function _BeginDragImage ( ControlName, ParentControl, imageindex, ix , iy)
*-----------------------------------------------------------------------------*
   Local c, h
   c := GetControlHandle ( ControlName , ParentControl )
   h := GetFormHandle (ParentControl)
   _HMG_ActiveDragImageHandle := h
   IL_BeginDrag( h, c , ImageIndex, ix, iy )
Return Nil

*-----------------------------------------------------------------------------*
Function _DragEnterImage ( ix , iy )
*-----------------------------------------------------------------------------*
   Local h := _HMG_ActiveDragImageHandle
   IL_DragEnter( h, ix, iy )
Return Nil

*-----------------------------------------------------------------------------*
Function _MoveImage ( ix, iy )
*-----------------------------------------------------------------------------*
   IL_DragMove( ix, iy )
Return Nil

*-----------------------------------------------------------------------------*
Function _EndDragImage ()
*-----------------------------------------------------------------------------*
   Local h := _HMG_ActiveDragImageHandle
   IL_EndDrag( h )
Return Nil

*-----------------------------------------------------------------------------*
Function _GetImageCount ( ControlName, ParentControl )
*-----------------------------------------------------------------------------*
   Local c := GetControlHandle ( ControlName , ParentControl )
Return IL_GetImageCount( c )