/*----------------------------------------------------------------------------
MINIGUI - Harbour Win32 GUI library source code

Copyright 2002-2008 Roberto Lopez <harbourminigui@gmail.com>
http://harbourminigui.googlepages.com/

INI Files support procedures
(c) 2003 Grigory Filatov
(c) 2003 Janusz Pora

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along with
   this software; see the file COPYING. If not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA (or
   visit the web site http://www.gnu.org/).

   As a special exception, you have permission for additional uses of the text
   contained in this release of Harbour Minigui.

   The exception is that, if you link the Harbour Minigui library with other
   files to produce an executable, this does not by itself cause the resulting
   executable to be covered by the GNU General Public License.
   Your use of that executable is in no way restricted on account of linking the
   Harbour-Minigui library code into it.

   Parts of this project are based upon:

   "Harbour GUI framework for Win32"
   Copyright 2001 Alexander S.Kresin <alex@belacy.belgorod.su>
   Copyright 2001 Antonio Linares <alinares@fivetech.com>
   www - http://www.harbour-project.org

   "Harbour Project"
   Copyright 1999-2008, http://www.harbour-project.org/

   "WHAT32"
   Copyright 2002 AJ Wos <andrwos@aust1.net>

   "HWGUI"
   Copyright 2001-2007 Alexander S.Kresin <alex@belacy.belgorod.su>

---------------------------------------------------------------------------*/

#include 'minigui.ch'
#include 'fileio.ch'

*-----------------------------------------------------------------------------*
Function _BeginIni( cIniFile )
*-----------------------------------------------------------------------------*
   Local hFile

   if AT( "\", cIniFile ) == 0
      cIniFile := ".\" + cIniFile
   endif

   If !File( cIniFile )
      hFile := FCreate( cIniFile )
   Else
      hFile := FOpen( cIniFile, FO_READ + FO_SHARED )
   EndIf

   If FError() != 0
      MsgInfo( "Error opening a file INI. DOS ERROR: " + Str( FError(), 2, 0 ) )
      Return -1
   Else
      _HMG_ActiveIniFile := cIniFile
   EndIf

   FClose( hFile )

Return 0

*-----------------------------------------------------------------------------*
Function _GetIni( cSection, cEntry, cDefault, uVar )
*-----------------------------------------------------------------------------*
   Local cFile, cVar :=''

   If !empty(_HMG_ActiveIniFile)
      if valtype(cDefault) == 'U'
         cDefault := cVar
      endif
      cFile := _HMG_ActiveIniFile
      cVar  := GetPrivateProfileString(cSection, cEntry, xChar( cDefault ), cFile )
   Else
      if cDefault != NIL
         cVar := xChar( cDefault )
      endif
   Endif
   uVar := xValue(cVar,ValType( uVar))

Return uVar

*-----------------------------------------------------------------------------*
Function _SetIni( cSection, cEntry, cValue )
*-----------------------------------------------------------------------------*
   Local ret:=.f., cFile

   If !empty(_HMG_ActiveIniFile)
      cFile := _HMG_ActiveIniFile
      ret   := WritePrivateProfileString( cSection, cEntry, xChar(cValue), cFile )
   Endif

Return ret

*-----------------------------------------------------------------------------*
Function _DelIniEntry( cSection, cEntry )
*-----------------------------------------------------------------------------*
   Local ret:=.f., cFile

   If !empty(_HMG_ActiveIniFile)
      cFile := _HMG_ActiveIniFile
      ret   := DelIniEntry( cSection, cEntry, cFile )
   Endif

Return ret

*-----------------------------------------------------------------------------*
Function _DelIniSection( cSection )
*-----------------------------------------------------------------------------*
   Local ret:=.f., cFile

   If !empty(_HMG_ActiveIniFile)
      cFile := _HMG_ActiveIniFile
      ret   := DelIniSection( cSection, cFile )
   Endif

Return ret

*-----------------------------------------------------------------------------*
Function _EndIni()
*-----------------------------------------------------------------------------*
   _HMG_ActiveIniFile :=''
Return Nil

*-----------------------------------------------------------------------------*
FUNCTION xChar( xValue )
*-----------------------------------------------------------------------------*
   LOCAL cType := ValType( xValue )
   LOCAL cValue := "", nDecimals := Set( _SET_DECIMALS )

   DO CASE
   CASE cType $  "CM";  cValue := xValue
   CASE cType == "N" ;  nDecimals := iif( xValue == int(xValue), 0, nDecimals) ; cValue := LTrim( Str( xValue, 20, nDecimals ) )
   CASE cType == "D" ;  cValue := DToS( xValue )
   CASE cType == "L" ;  cValue := IIf( xValue, "T", "F" )
   CASE cType == "A" ;  cValue := AToC( xValue )
   CASE cType $  "UE";  cValue := "NIL"
   CASE cType == "B" ;  cValue := "{|| ... }"
   CASE cType == "O" ;  cValue := "{" + xValue:className + "}"
   ENDCASE

RETURN cValue

*-----------------------------------------------------------------------------*
FUNCTION xValue( cValue, cType )
*-----------------------------------------------------------------------------*
   LOCAL xValue

   DO CASE
   CASE cType $  "CM";  xValue := cValue
   CASE cType == "D" ;  xValue := SToD( cValue )
   CASE cType == "N" ;  xValue := Val( cValue )
   CASE cType == "L" ;  xValue := ( cValue == 'T' )
   CASE cType == "A" ;  xValue := CToA( cValue )
   OTHERWISE;           xValue := NIL                     // nil, block, object
   ENDCASE

RETURN xValue

*-----------------------------------------------------------------------------*
FUNCTION AToC( aArray )
*-----------------------------------------------------------------------------*
   LOCAL i, nLen := Len( aArray )
   LOCAL cType, cElement, cArray := ""

   FOR i := 1 TO nLen
      cElement := xChar( aArray[ i ] )
      IF ( cType := ValType( aArray[ i ] ) ) == "A"
         cArray += cElement
      ELSE
         cArray += Left( cType, 1) + str( Len( cElement ),4 ) + cElement
      ENDIF
      ENDFOR

RETURN "A" + str( Len( cArray ),4 ) + cArray

*-----------------------------------------------------------------------------*
FUNCTION CToA( cArray )
*-----------------------------------------------------------------------------*
   LOCAL cType, nLen, aArray := {}

   cArray := SubStr( cArray, 6 )    // strip off array and length
   WHILE Len( cArray ) > 0
      nLen := Val( SubStr( cArray, 2, 4 ) )
      IF ( cType := Left( cArray, 1 ) ) == "A"
         AAdd( aArray, CToA( SubStr( cArray, 1, nLen + 5 ) ) )
      ELSE
         AAdd( aArray, xValue( SubStr( cArray, 6, nLen ), cType ) )
      ENDIF
      cArray := SubStr( cArray, 6 + nLen )
   END

RETURN aArray

// JK HMG 1.0 experimental build 6
*-----------------------------------------------------------------------------*
Function _GetSectionNames(cIniFile)
*-----------------------------------------------------------------------------*
// return 1-dimmensional array with section list in cIniFile
// or empty array if no sections are present
   Local aSectionList := {}, cLista, aLista

   If FILE(cIniFile)
      cLista := _GetPrivateProfileSectionNames( cIniFile )
      aLista := _SPLIT( cLista, CHR(0) )
      if !EMPTY(aLista)
         AEVAL( aLista, {|cVal| IF( EMPTY(cVal), , AADD( aSectionList, cVal ) )} ,, LEN( aLista ) - 1 )
      endif
   Else
      MsgStop( "Can`t open " + cIniFile, "Error" )
   EndIf

Return ( aSectionList )

*-----------------------------------------------------------------------------*
Function _GetSection( cSection, cIniFile )
*-----------------------------------------------------------------------------*
// return 2-dimmensional array with {key,value} pairs from section <cSection> in <cIniFile>
   Local aKeyValueList := {}, aLista, cLista, MyLista, i

   If FILE(cIniFile)
      cLista := _GetPrivateProfileSection( cSection, cIniFile )
      aLista := _SPLIT( cLista, CHR(0) )

      if !EMPTY(aLista)
         for i := 1 to LEN( aLista ) - 1
            myLista := _SPLIT( aLista[i], "=" )
            if LEN( myLista ) == 2
               AADD( aKeyValueList, { myLista[1], myLista[2] } )
            endif
         next i
      endif
   Else
      MsgStop( "Can`t open " + cIniFile, "Error" )
   Endif

Return ( aKeyValueList )

*-----------------------------------------------------------------------------*
Static Function _SPLIT( elstr, _separator )
*-----------------------------------------------------------------------------*
   Local aElems := {}, Elem, _sep := IIF( ! EMPTY( _separator ), _separator, " " )
   Local nSepPos

   If !EMPTY(elstr)
      elstr := alltrim( elstr )
      do while AT( _sep, elstr ) > 0
         nSepPos := AT( _sep, elstr )
         if nSepPos > 0
            Elem  := LEFT( elstr, nSepPos - 1 )
            elstr := SUBSTR( elstr, LEN( Elem ) + 2 )
            AADD( aElems, Elem )
         endif
      enddo
      AADD( aElems, elstr )
   endif

Return ( aElems )