/*----------------------------------------------------------------------------
MINIGUI - Harbour Win32 GUI library source code

Copyright 2002-2008 Roberto Lopez <harbourminigui@gmail.com>
http://harbourminigui.googlepages.com/

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along with
   this software; see the file COPYING. If not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA (or
   visit the web site http://www.gnu.org/).

   As a special exception, you have permission for additional uses of the text
   contained in this release of Harbour Minigui.

   The exception is that, if you link the Harbour Minigui library with other
   files to produce an executable, this does not by itself cause the resulting
   executable to be covered by the GNU General Public License.
   Your use of that executable is in no way restricted on account of linking the
   Harbour-Minigui library code into it.

   Parts of this project are based upon:

   "Harbour GUI framework for Win32"
   Copyright 2001 Alexander S.Kresin <alex@belacy.belgorod.su>
   Copyright 2001 Antonio Linares <alinares@fivetech.com>
   www - http://www.harbour-project.org

   "Harbour Project"
   Copyright 1999-2008, http://www.harbour-project.org/

   "WHAT32"
   Copyright 2002 AJ Wos <andrwos@aust1.net>

   "HWGUI"
   Copyright 2001-2007 Alexander S.Kresin <alex@belacy.belgorod.su>

---------------------------------------------------------------------------*/

#include "minigui.ch"

#define WS_TABSTOP            0x00010000
#define WS_VISIBLE            0x10000000
#define WS_CHILD              0x40000000
#define WS_BORDER             0x800000
#define WS_VSCROLL            0x200000
#define LBS_NOTIFY            0x0001
#define LBS_NOINTEGRALHEIGHT  0x0100
#define LBS_MULTIPLESEL       0x0008
#define LBS_SORT              0x0002
#define LBS_DISABLENOSCROLL   0x1000

*-----------------------------------------------------------------------------*
Function _DefineListbox ( ControlName, ParentFormName, x, y, w, h, rows, value,;
   fontname, fontsize, tooltip, changeprocedure, dblclick, gotfocus, lostfocus, break, HelpId,;
   invisible, notabstop, sort, bold, italic, underline, strikeout, backcolor, fontcolor, multiselect, nId )
*-----------------------------------------------------------------------------*
   Local i , ParentFormHandle , blInit , mVar , ControlHandle , Style
   Local FontHandle , aFont , k
   Local lDialogInMemory

   DEFAULT w               TO 120
   DEFAULT h               TO 120
   DEFAULT gotfocus        TO ""
   DEFAULT lostfocus       TO ""
   DEFAULT rows            TO {}
   DEFAULT value           TO 0
   DEFAULT changeprocedure TO ""
   DEFAULT dblclick        TO ""
   DEFAULT invisible       TO FALSE
   DEFAULT notabstop       TO FALSE
   DEFAULT sort            TO FALSE

   if ( FontHandle := GetFontHandle( FontName ) ) != 0
      aFont := GetFontParam(FontHandle)
      FontName      := aFont[1]
      FontSize      := aFont[2]
      bold          := aFont[3]
      italic        := aFont[4]
      underline     := aFont[5]
      strikeout     := aFont[6]
   endif

   if _HMG_BeginWindowActive = TRUE .or. _HMG_BeginDialogActive = TRUE
      ParentFormName := if(_HMG_BeginDialogActive,_HMG_ActiveDialogName,_HMG_ActiveFormName)
      if .Not. Empty (_HMG_ActiveFontName) .And. ValType(FontName) == "U"
         FontName := _HMG_ActiveFontName
      EndIf
      if .Not. Empty (_HMG_ActiveFontSize) .And. ValType(FontSize) == "U"
         FontSize := _HMG_ActiveFontSize
      EndIf
   endif

   if _HMG_FrameLevel > 0
      x    := x + _HMG_ActiveFrameCol [_HMG_FrameLevel]
      y    := y + _HMG_ActiveFrameRow [_HMG_FrameLevel]
      ParentFormName := _HMG_ActiveFrameParentFormName [_HMG_FrameLevel]
   EndIf
   lDialogInMemory := _HMG_DialogInMemory // ( GetWindowType ( ParentFormName ) == "L" )

   If .Not. _IsWindowDefined (ParentFormName) .and. .Not. lDialogInMemory
      MsgMiniGuiError("Window: "+ ParentFormName + " is not defined. Program terminated")
   Endif

   If _IsControlDefined (ControlName,ParentFormName) .and. .Not. lDialogInMemory
      MsgMiniGuiError ("Control: " + ControlName + " Of " + ParentFormName + " Already defined. Program Terminated")
   endif


   mVar := '_' + ParentFormName + '_' + ControlName
   k := _GetControlFree()

   if _HMG_BeginDialogActive = TRUE
      ParentFormHandle = _HMG_ActiveDialogHandle

      Style := WS_BORDER+ WS_CHILD + WS_VSCROLL + LBS_DISABLENOSCROLL + LBS_NOTIFY + LBS_NOINTEGRALHEIGHT

      if multiselect == .t.
         Style += LBS_MULTIPLESEL
      endif

      if !notabstop
         Style += WS_TABSTOP
      endif

      if !invisible
         Style += WS_VISIBLE
      endif

      if sort
         Style += LBS_SORT
      endif

      if lDialogInMemory         //Dialog Template

//          {{'ID',k/hwnd,class,Style,ExStyle,x,y,w,h,caption,HelpId,tooltip,font,size, bold, italic, underline, strikeout}}  --->_HMG_aDialogItems
         blInit:= {|x,y,z| InitDialogListBox(x,y,z) }
         aadd(_HMG_aDialogItems,{nId,k,"LISTBOX",style,0,x,y,w,h,"",HelpId,tooltip,FontName,FontSize,bold,italic,underline,strikeout,blInit,_HMG_BeginTabActive,.f.,_HMG_ActiveTabPage })
/* JP 62
         if _HMG_aDialogTemplate[3]   // Modal
            _HMG_aControlDeleted [k] := .T.
            Return Nil
         endif
*/
      else

         ControlHandle := GetDialogItemHandle( ParentFormHandle, nId )

         x := GetWindowCol ( Controlhandle )
         y := GetWindowRow ( Controlhandle )
         w := GetWindowWidth  ( Controlhandle )
         h := GetWindowHeight ( Controlhandle )

         SetWindowStyle ( ControlHandle, Style )

      endif

   else

      ParentFormHandle = GetFormHandle (ParentFormName)

      if valtype(x) == "U" .or. valtype(y) == "U"

         If _HMG_SplitLastControl == "TOOLBAR"
            Break := TRUE
         EndIf

         i := GetFormIndex ( ParentFormName )

         if i > 0

            if multiselect == .t.
               ControlHandle := InitMultiListBox ( _HMG_aFormReBarHandle [i], 0, x, y, w, h, fontname, fontsize, invisible, notabstop, sort )
            else
               ControlHandle := InitListBox ( _HMG_aFormReBarHandle [i] , 0 , 0 , 0 , w , h , '' , 0 , invisible , notabstop, sort )
            endif

            If FontHandle != 0
               _SetFontHandle(ControlHandle,FontHandle)
            Else
               if valtype(fontname) == "U"
                  Fontname := _HMG_DefaultFontName
               endif
               if valtype(fontsize) == "U"
                  FontSize := _HMG_DefaultFontSize
               endif
               FontHandle := _SetFont (ControlHandle,FontName,FontSize,bold,italic,underline,strikeout)
            EndIf

            AddSplitBoxItem ( Controlhandle , _HMG_aFormReBarHandle [i] , w , break , , , , _HMG_ActiveSplitBoxInverted )

            _HMG_SplitLastControl   := "LISTBOX"

         EndIf

      Else

         if multiselect == .t.
            ControlHandle := InitMultiListBox ( ParentFormHandle, 0, x, y, w, h, fontname, fontsize, invisible, notabstop, sort )
         else
            ControlHandle := InitListBox ( ParentFormHandle , 0 , x , y , w , h , '' , 0 , invisible , notabstop, sort )
         endif

      endif

   endif

   if .Not. lDialogInMemory

      If FontHandle != 0
         _SetFontHandle(ControlHandle,FontHandle)
      Else
         if valtype(fontname) == "U"
            Fontname := _HMG_DefaultFontName
         endif
         if valtype(fontsize) == "U"
            FontSize := _HMG_DefaultFontSize
         endif
         FontHandle := _SetFont (ControlHandle,FontName,FontSize,bold,italic,underline,strikeout)
      EndIf

      If _HMG_BeginTabActive = TRUE
         aAdd ( _HMG_ActiveTabCurrentPageMap , ControlHandle )
      EndIf

      if valtype(tooltip) != "U"
         SetToolTip ( ControlHandle , tooltip , GetFormToolTipHandle (ParentFormName) )
      endif

   endif

   Public &mVar. := k

   _HMG_aControlType [k] := if ( multiselect , "MULTILIST" , "LIST" )
   _HMG_aControlNames  [k] :=  ControlName
   _HMG_aControlHandles  [k] :=  ControlHandle
   _HMG_aControlParenthandles  [k] :=  ParentFormHandle
   _HMG_aControlIds  [k] :=  nId
   _HMG_aControlProcedures  [k] :=  ""
   _HMG_aControlPageMap  [k] :=  {}
   _HMG_aControlValue  [k] :=  Value
   _HMG_aControlInputMask  [k] :=  ""
   _HMG_aControllostFocusProcedure  [k] :=  lostfocus
   _HMG_aControlGotFocusProcedure  [k] :=  gotfocus
   _HMG_aControlChangeProcedure  [k] :=  ChangeProcedure
   _HMG_aControlDeleted  [k] :=  FALSE
   _HMG_aControlBkColor  [k] :=  backcolor
   _HMG_aControlFontColor  [k] :=  fontcolor
   _HMG_aControlDblClick  [k] :=  dblclick
   _HMG_aControlHeadClick  [k] :=  {}
   _HMG_aControlRow  [k] :=  y
   _HMG_aControlCol   [k] := x
   _HMG_aControlWidth   [k] := w
   _HMG_aControlHeight   [k] := h
   _HMG_aControlSpacing   [k] := 0
   _HMG_aControlContainerRow  [k] :=  iif ( _HMG_FrameLevel > 0 ,_HMG_ActiveFrameRow [_HMG_FrameLevel] , -1 )
   _HMG_aControlContainerCol  [k] :=  iif ( _HMG_FrameLevel > 0 ,_HMG_ActiveFrameCol [_HMG_FrameLevel] , -1 )
   _HMG_aControlPicture   [k] := ""
   _HMG_aControlContainerHandle  [k] :=  0
   _HMG_aControlFontName  [k] :=  fontname
   _HMG_aControlFontSize  [k] :=  fontsize
   _HMG_aControlFontAttributes  [k] :=  {bold,italic,underline,strikeout}
   _HMG_aControlToolTip   [k] :=  tooltip
   _HMG_aControlRangeMin   [k] :=  rows
   _HMG_aControlRangeMax  [k] :=   0
   _HMG_aControlCaption   [k] :=  ""
   _HMG_aControlVisible  [k] :=   if(invisible,FALSE,TRUE)
   _HMG_aControlHelpId   [k] :=  HelpId
   _HMG_aControlFontHandle  [k] :=   FontHandle
   _HMG_aControlBrushHandle  [k] :=   0
   _HMG_aControlEnabled  [k] :=   .T.
   _HMG_aControlMiscData1 [k] := 0

   if len(_HMG_aDialogTemplate) == 0        //Dialog Template

      for i = 1 to len (rows)
         ListboxAddString (ControlHandle,rows[i])
      next x

      if multiselect == .t.
         if value <> Nil
            LISTBOXSETMULTISEL (ControlHandle,Value)
         endif
      else
         if value <> 0
            ListboxSetCurSel (ControlHandle,Value)
         endif
      endif
   endif

Return Nil

*-----------------------------------------------------------------------------*
Function InitDialogListBox(ParentName, ControlHandle, k)
*-----------------------------------------------------------------------------*
   Local rows, i, Value

   rows            := _HMG_aControlRangeMin  [k]
   Value           := _HMG_aControlValue  [k]

   ParentName := Nil

   for i = 1 to len (rows)
      ListboxAddString (ControlHandle,rows[i])
   next x

   if _HMG_aControlType [k] == "MULTILIST"
      if value <> Nil
         LISTBOXSETMULTISEL (ControlHandle,Value)
      endif
   else
      if value <> 0
         ListboxSetCurSel (ControlHandle,Value)
      endif
   endif
// JP 62
   if len(_HMG_aDialogTemplate) != 0 .and. _HMG_aDialogTemplate[3]   // Modal
      _HMG_aControlDeleted [k] := .T.
   endif

Return Nil