/*----------------------------------------------------------------------------
MINIGUI - Harbour Win32 GUI library source code

Copyright 2002-2008 Roberto Lopez <harbourminigui@gmail.com>
http://harbourminigui.googlepages.com/

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along with
   this software; see the file COPYING. If not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA (or
   visit the web site http://www.gnu.org/).

   As a special exception, you have permission for additional uses of the text
   contained in this release of Harbour Minigui.

   The exception is that, if you link the Harbour Minigui library with other
   files to produce an executable, this does not by itself cause the resulting
   executable to be covered by the GNU General Public License.
   Your use of that executable is in no way restricted on account of linking the
   Harbour-Minigui library code into it.

   Parts of this project are based upon:

   "Harbour GUI framework for Win32"
   Copyright 2001 Alexander S.Kresin <alex@belacy.belgorod.su>
   Copyright 2001 Antonio Linares <alinares@fivetech.com>
   www - http://www.harbour-project.org

   "Harbour Project"
   Copyright 1999-2008, http://www.harbour-project.org/

   "WHAT32"
   Copyright 2002 AJ Wos <andrwos@aust1.net>

   "HWGUI"
   Copyright 2001-2007 Alexander S.Kresin <alex@belacy.belgorod.su>

---------------------------------------------------------------------------*/

#include "minigui.ch"

#define WS_TABSTOP          0x00010000
#define WS_VISIBLE          0x10000000
#define WS_CHILD            0x40000000
#define WS_BORDER           0x800000

#define MCS_WEEKNUMBERS     0x0004
#define MCS_NOTODAYCIRCLE   0x0008
#define MCS_NOTODAY         0x0010

*-----------------------------------------------------------------------------*
Function _DefineMonthCal ( ControlName, ParentFormName, x, y, w, h, value,;
   fontname, fontsize, tooltip, notoday, notodaycircle,;
   weeknumbers, change, HelpId, invisible, notabstop,;
   bold, italic, underline, strikeout,;
   backcolor, fontcolor, titlebkclr, titlefrclr,;
   background, trlfontclr, nId )
*-----------------------------------------------------------------------------*
   Local ParentFormHandle, mVar, k, Style, blInit
   Local aControlHandle := {0, 0}, aFont
   Local lDialogInMemory

   DEFAULT value     TO date()
   DEFAULT change    TO ""
   DEFAULT bold      TO FALSE
   DEFAULT italic    TO FALSE
   DEFAULT underline TO FALSE
   DEFAULT strikeout TO FALSE

   if ( aControlHandle[2] := GetFontHandle( FontName ) ) != 0
      aFont := GetFontParam(aControlHandle[2])
      FontName      := aFont[1]
      FontSize      := aFont[2]
      bold          := aFont[3]
      italic        := aFont[4]
      underline     := aFont[5]
      strikeout     := aFont[6]
   endif

   if _HMG_BeginWindowActive = TRUE .or. _HMG_BeginDialogActive = TRUE
      ParentFormName := if(_HMG_BeginDialogActive,_HMG_ActiveDialogName,_HMG_ActiveFormName)
      if .Not. Empty (_HMG_ActiveFontName) .And. ValType(FontName) == "U"
         FontName := _HMG_ActiveFontName
      EndIf
      if .Not. Empty (_HMG_ActiveFontSize) .And. ValType(FontSize) == "U"
         FontSize := _HMG_ActiveFontSize
      EndIf
   endif
   if _HMG_FrameLevel > 0
      x    := x + _HMG_ActiveFrameCol [_HMG_FrameLevel]
      y    := y + _HMG_ActiveFrameRow [_HMG_FrameLevel]
      ParentFormName := _HMG_ActiveFrameParentFormName [_HMG_FrameLevel]
   EndIf
   lDialogInMemory := _HMG_DialogInMemory // ( GetWindowType ( ParentFormName ) == "L" )

   If .Not. _IsWindowDefined (ParentFormName) .and. .Not. lDialogInMemory
      MsgMiniGuiError("Window: "+ ParentFormName + " is not defined. Program terminated")
   Endif

   If _IsControlDefined (ControlName,ParentFormName) .and. .Not. lDialogInMemory
      MsgMiniGuiError ("Control: " + ControlName + " Of " + ParentFormName + " Already defined. Program terminated")
   endif

   mVar := '_' + ParentFormName + '_' + ControlName
   k := _GetControlFree()

   if _HMG_BeginDialogActive = TRUE
      ParentFormHandle = _HMG_ActiveDialogHandle

      Style := WS_BORDER + WS_CHILD

      if !invisible
         Style += WS_VISIBLE
      Endif

      if !NoTabStop
         Style += WS_TABSTOP
      Endif

      if notoday
         Style += MCS_NOTODAY
      Endif

      if notodaycircle
         Style += MCS_NOTODAYCIRCLE
      Endif

      if weeknumbers
         Style += MCS_WEEKNUMBERS
      Endif

      if lDialogInMemory         //Dialog Template
         InitExCommonControls(1)

//           {{'ID',k/hwnd,class,Style,ExStyle,x,y,w,h,caption,HelpId,tooltip,font,size, bold, italic, underline, strikeout}}  --->_HMG_aDialogItems
         blInit:= {|x,y,z| InitDialogMonthCalendar(x,y,z) }

         aadd(_HMG_aDialogItems,{nId,k,"SysMonthCal32",style,0,x,y,w,h,"",HelpId,tooltip,FontName,FontSize,bold,italic,underline,strikeout,blInit,_HMG_BeginTabActive,.f.,_HMG_ActiveTabPage })
/* JP 62
         if _HMG_aDialogTemplate[3]   // Modal
            _HMG_aControlDeleted [k] := .T.
            Return Nil
         endif
*/
      else

         aControlHandle[1] := GetDialogItemHandle( ParentFormHandle, nId )

         SetWindowStyle ( aControlHandle[1], style )

         If aControlHandle[2] != 0
            _SetFontHandle(aControlHandle[1],aControlHandle[2])
         Else
            if valtype(fontname) == "U"
               FontName := _HMG_DefaultFontName
            endif
            if valtype(fontsize) == "U"
               FontSize := _HMG_DefaultFontSize
            endif
            aControlHandle[2] := _SetFont (aControlHandle[1],FontName,FontSize,bold,italic,underline,strikeout)
         Endif

         x := GetWindowCol ( aControlHandle[1] )
         y := GetWindowRow ( aControlHandle[1] )

      endif

   else

      ParentFormHandle = GetFormHandle (ParentFormName)

      if valtype(fontname) == "U"
         fontname := _HMG_DefaultFontName
      endif
      if valtype(fontsize) == "U"
         fontsize := _HMG_DefaultFontSize
      endif

      aControlHandle := InitMonthCal ( ParentFormHandle, 0, x, y, w, h , fontname , fontsize , notoday , notodaycircle , weeknumbers, invisible, notabstop, bold, italic, underline, strikeout )

   Endif

   if .Not. lDialogInMemory

      If _HMG_BeginTabActive = .T.
         aAdd ( _HMG_ActiveTabCurrentPageMap , aControlhandle[1] )
      EndIf

      SetMonthCal( aControlHandle[1] ,year(value), month(value), day(value) )

      if valtype(tooltip) != "U"
         SetToolTip ( aControlHandle[1] , tooltip , GetFormToolTipHandle (ParentFormName) )
      endif

      w := GetWindowWidth ( aControlHandle[1] )
      h := GetWindowHeight ( aControlHandle[1] )
   endif

   Public &mVar. := k

   _HMG_aControlType [k] := "MONTHCAL"
   _HMG_aControlNames  [k] :=  ControlName
   _HMG_aControlHandles  [k] :=  aControlHandle[1]
   _HMG_aControlParentHandles [k] :=   ParentFormHandle
   _HMG_aControlIds  [k] :=  nId
   _HMG_aControlProcedures  [k] :=  ""
   _HMG_aControlPageMap  [k] :=  {}
   _HMG_aControlValue  [k] :=  Value
   _HMG_aControlInputMask  [k] :=  ""
   _HMG_aControllostFocusProcedure  [k] :=  ""
   _HMG_aControlGotFocusProcedure  [k] :=  ""
   _HMG_aControlChangeProcedure  [k] :=  change
   _HMG_aControlDeleted  [k] :=  .F.
   _HMG_aControlBkColor  [k] :=  BackColor
   _HMG_aControlFontColor  [k] :=  FontColor
   _HMG_aControlDblClick  [k] :=  ""
   _HMG_aControlHeadClick  [k] :=  {}
   _HMG_aControlRow  [k] :=  y
   _HMG_aControlCol   [k] := x
   _HMG_aControlWidth   [k] := w
   _HMG_aControlHeight   [k] := h
   _HMG_aControlSpacing   [k] := 0
   _HMG_aControlContainerRow  [k] :=  iif ( _HMG_FrameLevel > 0 ,_HMG_ActiveFrameRow [_HMG_FrameLevel] , -1 )
   _HMG_aControlContainerCol  [k] :=  iif ( _HMG_FrameLevel > 0 ,_HMG_ActiveFrameCol [_HMG_FrameLevel] , -1 )
   _HMG_aControlPicture  [k] :=  ""
   _HMG_aControlContainerHandle  [k] :=  0
   _HMG_aControlFontName  [k] :=  fontname
   _HMG_aControlFontSize  [k] :=  fontsize
   _HMG_aControlFontAttributes  [k] :=  {bold,italic,underline,strikeout}
   _HMG_aControlToolTip   [k] :=  tooltip
   _HMG_aControlRangeMin  [k] :=   0
   _HMG_aControlRangeMax  [k] :=   0
   _HMG_aControlCaption   [k] :=  ''
   _HMG_aControlVisible  [k] :=   if(invisible,FALSE,TRUE)
   _HMG_aControlHelpId  [k] :=   HelpId
   _HMG_aControlFontHandle  [k] :=   aControlHandle[2]
   _HMG_aControlBrushHandle  [k] :=   0
   _HMG_aControlEnabled  [k] :=   .T.
   _HMG_aControlMiscData1 [k] :=   0

   if BackColor <> Nil
      if BackColor[1] <> Nil .and. BackColor[2] <> Nil .and. BackColor[3] <> Nil
         SetMonthCalMonthBkColor(aControlHandle[1],BackColor[1],BackColor[2],BackColor[3])
      endif
   endif

   if FontColor <> Nil
      if FontColor[1] <> Nil .and. FontColor[2] <> Nil .and. FontColor[3] <> Nil
         SetMonthCalFontColor(aControlHandle[1],FontColor[1],FontColor[2],FontColor[3])
      endif
   endif

   if TitleBkClr <> Nil
      if TitleBkClr[1] <> Nil .and. TitleBkClr[2] <> Nil .and. TitleBkClr[3] <> Nil
         SetMonthCalTitleBkColor(aControlHandle[1],TitleBkClr[1],TitleBkClr[2],TitleBkClr[3])
      endif
   endif

   if TitleFrClr <> Nil
      if TitleFrClr[1] <> Nil .and. TitleFrClr[2] <> Nil .and. TitleFrClr[3] <> Nil
         SetMonthCalTitleFontColor(aControlHandle[1],TitleFrClr[1],TitleFrClr[2],TitleFrClr[3])
      endif
   endif

   if BackGround <> Nil
      if BackGround[1] <> Nil .and. BackGround[2] <> Nil .and. BackGround[3] <> Nil
         SetMonthCalBkColor(aControlHandle[1],BackGround[1],BackGround[2],BackGround[3])
      endif
   endif

   if TrlFontClr <> Nil
      if TrlFontClr[1] <> Nil .and. TrlFontClr[2] <> Nil .and. TrlFontClr[3] <> Nil
         SetMonthCalTrlFontColor(aControlHandle[1],TrlFontClr[1],TrlFontClr[2],TrlFontClr[3])
      endif
   endif

Return Nil

*-----------------------------------------------------------------------------*
Function InitDialogMonthCalendar(ParentFormName, ControlHandle, k)
*-----------------------------------------------------------------------------*
   ParentFormName := Nil

   SETPOSMONTHCAL ( ControlHandle , _HMG_aControlCol [k] , _HMG_aControlRow [k] )
// JP 62
   if len(_HMG_aDialogTemplate) != 0 .and. _HMG_aDialogTemplate[3]   // Modal
      _HMG_aControlDeleted [k] := .T.
   endif


Return Nil