/*----------------------------------------------------------------------------
MINIGUI - Harbour Win32 GUI library source code

Copyright 2002-2008 Roberto Lopez <harbourminigui@gmail.com>
http://harbourminigui.googlepages.com/

MsgBox code rewrotten by Jacek Kubica <kubica@wssk.wroc.pl>
(c) 2006 HMG Experimental Build 16g

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along with
   this software; see the file COPYING. If not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA (or
   visit the web site http://www.gnu.org/).

   As a special exception, you have permission for additional uses of the text
   contained in this release of Harbour Minigui.

   The exception is that, if you link the Harbour Minigui library with other
   files to produce an executable, this does not by itself cause the resulting
   executable to be covered by the GNU General Public License.
   Your use of that executable is in no way restricted on account of linking the
   Harbour-Minigui library code into it.

   Parts of this project are based upon:

   "Harbour GUI framework for Win32"
   Copyright 2001 Alexander S.Kresin <alex@belacy.belgorod.su>
   Copyright 2001 Antonio Linares <alinares@fivetech.com>
   www - http://www.harbour-project.org

   "Harbour Project"
   Copyright 1999-2008, http://www.harbour-project.org/

   "WHAT32"
   Copyright 2002 AJ Wos <andrwos@aust1.net>

   "HWGUI"
   Copyright 2001-2007 Alexander S.Kresin <alex@belacy.belgorod.su>

---------------------------------------------------------------------------*/

#include 'common.ch'

#define IDOK      1
#define IDCANCEL  2
#define IDABORT   3
#define IDRETRY   4
#define IDIGNORE  5
#define IDYES     6
#define IDNO      7
#define IDCLOSE   8
#define IDHELP    9

#define MB_OK                                 0
#define MB_OKCANCEL                           1
#define MB_ABORTRETRYIGNORE                   2
#define MB_YESNOCANCEL                        3
#define MB_YESNO                              4
#define MB_RETRYCANCEL                        5
#define MB_CANCELTRYCONTINUE                  6
#define MB_ICONHAND                          16
#define MB_ICONQUESTION                      32
#define MB_ICONEXCLAMATION                   48
#define MB_ICONASTERISK                      64
#define MB_USERICON                         128
#define MB_ICONWARNING              MB_ICONEXCLAMATION
#define MB_ICONERROR                MB_ICONHAND
#define MB_ICONINFORMATION          MB_ICONASTERISK
#define MB_ICONSTOP                 MB_ICONHAND
#define MB_DEFBUTTON1                         0
#define MB_DEFBUTTON2                       256
#define MB_DEFBUTTON3                       512
#define MB_DEFBUTTON4                       768
#define MB_APPLMODAL                          0
#define MB_SYSTEMMODAL                     4096
#define MB_TASKMODAL                       8192

#define MB_HELP                           16384 // Help Button
#define MB_NOFOCUS                        32768
#define MB_SETFOREGROUND                  65536
#define MB_DEFAULT_DESKTOP_ONLY          131072
#define MB_TOPMOST                       262144
#define MB_RIGHT                         524288
#define MB_RTLREADING                   1048576

*-----------------------------------------------------------------------------*
Function MsgYesNo ( Message , Title , RevertDefault , nIcon ,;
   lSysModal , lTopMost )
*-----------------------------------------------------------------------------*
   Local Retval, t ,nStyle := MB_YESNO

   DEFAULT Message TO ''
   DEFAULT Title TO ''
   DEFAULT RevertDefault TO .f.
   DEFAULT nIcon TO 0
   DEFAULT lSysModal TO .t.
   DEFAULT lTopMost TO .t.

   If EMPTY(nIcon)
      nStyle += MB_ICONQUESTION
   ELSE
      nStyle += MB_USERICON
   ENDIF

   IF lSysModal==.t.
      nStyle += MB_SYSTEMMODAL
   ELSE
      nStyle += MB_APPLMODAL
   ENDIF

   IF lTopMost==.t.
      nStyle += MB_TOPMOST
   ENDIF

   IF RevertDefault==.t.
      nStyle += MB_DEFBUTTON2
   ENDIF

   t := MESSAGEBOXINDIRECT( , message, title, nStyle, nIcon )

   If t = IDYES
      RetVal := .t.
   Else
      RetVal := .f.
   Endif

Return (RetVal)

*-----------------------------------------------------------------------------*
Function MsgYesNoCancel ( Message , Title , nIcon ,;
   lSysModal , nDefaultButton , lTopMost )
*-----------------------------------------------------------------------------*
   Local Retval, t , nStyle:= MB_YESNOCANCEL

   DEFAULT Message TO ''
   DEFAULT Title TO ''
   DEFAULT nIcon TO 0
   DEFAULT lSysModal TO .t.
   DEFAULT nDefaultButton TO 1
   DEFAULT lTopMost TO .t.

   If EMPTY(nIcon)
      nStyle += MB_ICONQUESTION
   ELSE
      nStyle += MB_USERICON
   ENDIF

   IF lSysModal==.t.
      nStyle += MB_SYSTEMMODAL
   ELSE
      nStyle += MB_APPLMODAL
   ENDIF

   IF  lTopMost==.t.
      nStyle += MB_TOPMOST
   ENDIF

   do case
   case nDefaultButton == 2

      nStyle += MB_DEFBUTTON2

   case nDefaultButton == 3

      nStyle += MB_DEFBUTTON3

   endcase

   t := MESSAGEBOXINDIRECT( , message, title, nStyle, nIcon )

   If t = IDYES
      RetVal := 1
   ElseIf t = IDNO
      RetVal := 0
   Else
      RetVal := -1
   Endif

Return (RetVal)

*-----------------------------------------------------------------------------*
Function MsgRetryCancel ( Message , Title , nIcon ,;
   lSysModal , nDefaultButton , lTopMost )
*-----------------------------------------------------------------------------*
   Local Retval, t  , nStyle := MB_RETRYCANCEL

   DEFAULT Message TO ''
   DEFAULT Title TO ''
   DEFAULT nIcon TO 0
   DEFAULT lSysModal TO .t.
   DEFAULT nDefaultButton TO 1
   DEFAULT lTopMost TO .t.

   If EMPTY(nIcon)
      nStyle += MB_ICONQUESTION
   else
      nStyle += MB_USERICON
   ENDIF

   IF lSysModal==.t.
      nStyle += MB_SYSTEMMODAL
   ELSE
      nStyle += MB_APPLMODAL
   ENDIF

   IF lTopMost==.t.
      nStyle += MB_TOPMOST
   ENDIF

   if nDefaultButton==2
      nStyle += MB_DEFBUTTON2
   endif

   t := MESSAGEBOXINDIRECT( , message, title, nStyle, nIcon )

   if t = IDRETRY
      RetVal := .t.
   Else
      RetVal := .f.
   Endif

Return (RetVal)

*-----------------------------------------------------------------------------*
Function MsgOkCancel ( Message , Title , nIcon ,;
   lSysModal , nDefaultButton , lTopMost )
*-----------------------------------------------------------------------------*
   Local Retval, t, nStyle := MB_OKCANCEL

   DEFAULT Message TO ''
   DEFAULT Title TO ''
   DEFAULT nIcon TO 0
   DEFAULT lSysModal TO .t.
   DEFAULT nDefaultButton TO 1
   DEFAULT lTopMost TO .t.

   IF EMPTY(nIcon)
      nStyle += MB_ICONQUESTION
   ELSE
      nStyle += MB_USERICON
   ENDIF

   IF lSysModal==.t.
      nStyle += MB_SYSTEMMODAL
   ELSE
      nStyle += MB_APPLMODAL
   ENDIF

   IF lTopMost==.t.
      nStyle += MB_TOPMOST
   ENDIF

   IF nDefaultButton==2
      nStyle += MB_DEFBUTTON2
   ENDIF

   t := MESSAGEBOXINDIRECT( , message, title, nStyle, nIcon )

   IF t = IDOK
      RetVal := .t.
   ELSE
      RetVal := .f.
   ENDIF

Return (RetVal)

*-----------------------------------------------------------------------------*
Function MsgInfo ( Message , Title , nIcon , lSysModal , lTopMost )
*-----------------------------------------------------------------------------*
   Local nStyle := MB_OK

   DEFAULT Message TO ''
   DEFAULT Title TO 'Information'
   DEFAULT nIcon TO 0
   DEFAULT lSysModal TO .t.
   DEFAULT lTopMost TO .t.

   If EMPTY(nIcon)
      nStyle += MB_ICONINFORMATION
   else
      nStyle += MB_USERICON
   ENDIF

   IF lSysModal==.t.
      nStyle += MB_SYSTEMMODAL
   ELSE
      nStyle += MB_APPLMODAL
   ENDIF

   IF lTopMost==.t.
      nStyle += MB_TOPMOST
   ENDIF

   MESSAGEBOXINDIRECT( , message, title, nStyle, nIcon )

Return Nil

*-----------------------------------------------------------------------------*
Function MsgStop ( Message , Title , nIcon , lSysModal , lTopMost )
*-----------------------------------------------------------------------------*
   Local nStyle := MB_OK

   DEFAULT Message TO ''
   DEFAULT Title TO 'Stop'
   DEFAULT nIcon TO 0
   DEFAULT lSysModal TO .t.
   DEFAULT lTopMost TO .t.

   If EMPTY(nIcon)
      nStyle += MB_ICONSTOP
   else
      nStyle += MB_USERICON
   ENDIF

   IF lSysModal==.t.
      nStyle += MB_SYSTEMMODAL
   ELSE
      nStyle += MB_APPLMODAL
   ENDIF

   IF lTopMost==.t.
      nStyle += MB_TOPMOST
   ENDIF

   MESSAGEBOXINDIRECT( , message, title, nStyle, nIcon )

Return Nil

*-----------------------------------------------------------------------------*
Function MsgExclamation ( Message , Title , nIcon, lSysModal , lTopMost )
*-----------------------------------------------------------------------------*
   Local nStyle := MB_OK

   DEFAULT Message TO ''
   DEFAULT Title TO 'Alert'
   DEFAULT nIcon TO 0
   DEFAULT lSysModal TO .t.
   DEFAULT lTopMost TO .t.

   If EMPTY(nIcon)
      nStyle += MB_ICONEXCLAMATION
   else
      nStyle += MB_USERICON
   ENDIF

   IF lSysModal==.t.
      nStyle += MB_SYSTEMMODAL
   ELSE
      nStyle += MB_APPLMODAL
   ENDIF

   IF lTopMost==.t.
      nStyle += MB_TOPMOST
   ENDIF

   MESSAGEBOXINDIRECT( , message, title, nStyle, nIcon )

Return Nil

*-----------------------------------------------------------------------------*
Function MsgBox ( Message , Title , lSysModal , lTopMost )
*-----------------------------------------------------------------------------*
   Local nStyle := MB_OK

   DEFAULT Message TO ''
   DEFAULT Title TO ''
   DEFAULT lSysModal TO .t.
   DEFAULT lTopMost TO .t.

   IF lSysModal==.t.
      nStyle += MB_SYSTEMMODAL
   ELSE
      nStyle += MB_APPLMODAL
   ENDIF

   IF lTopMost==.t.
      nStyle += MB_TOPMOST
   ENDIF

   MESSAGEBOXINDIRECT( , message, title, nStyle )

Return Nil