/*----------------------------------------------------------------------------
MINIGUI - Harbour Win32 GUI library source code

Copyright 2002-2008 Roberto Lopez <harbourminigui@gmail.com>
http://harbourminigui.googlepages.com/

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along with
   this software; see the file COPYING. If not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA (or
   visit the web site http://www.gnu.org/).

   As a special exception, you have permission for additional uses of the text
   contained in this release of Harbour Minigui.

   The exception is that, if you link the Harbour Minigui library with other
   files to produce an executable, this does not by itself cause the resulting
   executable to be covered by the GNU General Public License.
   Your use of that executable is in no way restricted on account of linking the
   Harbour-Minigui library code into it.

   Parts of this project are based upon:

   "Harbour GUI framework for Win32"
   Copyright 2001 Alexander S.Kresin <alex@belacy.belgorod.su>
   Copyright 2001 Antonio Linares <alinares@fivetech.com>
   www - http://www.harbour-project.org

   "Harbour Project"
   Copyright 1999-2008, http://www.harbour-project.org/

   "WHAT32"
   Copyright 2002 AJ Wos <andrwos@aust1.net>

   "HWGUI"
   Copyright 2001-2007 Alexander S.Kresin <alex@belacy.belgorod.su>

---------------------------------------------------------------------------*/
/*
	Class TReg32 for (x)Harbour
*/

#include "hbclass.ch"
#include "common.ch"

#define KEY_ALL_ACCESS		63
#define REG_SZ			1
#define REG_DWORD		4

CLASS TReg32

   DATA cRegKey, nHandle, nError, lError

   METHOD New( nKey, cRegKey, lShowError )

   METHOD Create( nKey, cRegKey, lShowError )

   METHOD Get( cRegVar, uVar )

   METHOD Set( cRegVar, uVar )

   METHOD Delete( cRegVar )

   METHOD KeyDelete( cSubKey )

   METHOD Close() BLOCK { |Self| If( ::lError, , RegCloseKey( ::nHandle ) ) } EXPORTED

ENDCLASS


METHOD New( nKey, cRegKey, lShowError ) CLASS TReg32

   local nReturn, nHandle := 0

   DEFAULT cRegKey TO ""

   nReturn := RegOpenKeyExA( nKey, cRegKey,, KEY_ALL_ACCESS, @nHandle )

   ::cRegKey := cRegKey
   ::nHandle := nHandle

   if nReturn != 0
      ::lError := .t.
      IF lShowError == NIL .OR. lShowError
         MsgStop( "Error opening TReg32 object ("+LTrim(Str(nReturn))+")" )
      ENDIF
   else
      ::lError := .f.
   endif

return Self


METHOD Create( nKey, cRegKey, lShowError ) CLASS TReg32

   local nReturn, nHandle := 0

   DEFAULT cRegKey TO ""

   nReturn := RegCreateKey( nKey, cRegKey, @nHandle )

   if nReturn != 0
      ::lError := .t.
      IF lShowError == NIL .OR. lShowError
         MsgStop("Error creating TReg32 object ("+LTrim(Str(nReturn))+")" )
      ENDIF
   else
      ::lError := .f.
   endif

   ::nError := RegOpenKeyExA( nKey, cRegKey,, KEY_ALL_ACCESS, @nHandle )

   ::cRegKey := cRegKey
   ::nHandle := nHandle

return Self


METHOD Get( cRegVar, uVar ) CLASS TReg32

   local cValue := ""
   local nType := 0
   local nLen := 0
   local cType

   if !::lError

      default cRegVar to ''
      cType := ValType( uVar )

      if cType == 'N'
         nType := REG_DWORD
         nLen  := 4
         cValue := L2Bin( uVar )
      endif

      ::nError = RegQueryValueExA( ::nHandle, cRegVar, 0, @nType, @cValue, @nLen )
      uVar := cValue

      do case
         case cType == "N"
              uVar := Bin2L( uVar )

         case cType == "D"
              uVar := CToD( uVar )

         case cType == "L"
              uVar := ( Upper( uVar ) == ".T." )
      endcase

   endif

return uVar


METHOD Set( cRegVar, uVar ) CLASS TReg32

   local nType, nLen, cType

   if !::lError

      default cRegVar to ''
      cType := ValType( uVar )

      if cType == 'N'
         nType := REG_DWORD
         nLen  := 4
         uVar  := L2Bin( uVar )
      else
         nType := REG_SZ
         do case
            case cType == "D"
                 uVar := DToC( uVar )

            case cType == "L"
                 uVar := If( uVar, ".T.", ".F." )
         endcase
         nLen := Len( uVar )
      endif
      ::nError := RegSetValueExA( ::nHandle, cRegVar, 0, nType, @uVar, nLen )

   endif

return nil


METHOD Delete( cRegVar ) CLASS TReg32

   if !::lError

      ::nError := RegDeleteValueA( ::nHandle, cRegVar )

   endif

return nil


METHOD KeyDelete( cSubKey ) CLASS TReg32

   if !::lError

      ::nError := RegDeleteKey( ::nHandle, cSubkey )

   endif

return nil


FUNCTION IsRegistryKey( nKey, cRegKey )

   LOCAL oReg   := TReg32():New( nKey, cRegKey, .F. )
   LOCAL lExist := !oReg:lError

   oReg:Close()

RETURN lExist


FUNCTION CreateRegistryKey( nKey, cRegKey )
  
   LOCAL oReg     := TReg32():Create( nKey, cRegKey, .F. )
   LOCAL lSuccess := !oReg:lError

   oReg:Close()

RETURN lSuccess


FUNCTION GetRegistryValue( nKey, cRegKey, cRegVar, cType )
  
   LOCAL oReg     := TReg32():New( nKey, cRegKey, .F. )
   LOCAL lSuccess := !oReg:lError
   LOCAL uVal     := NIL
  
   DEFAULT cRegVar TO ''
   DEFAULT cType   TO 'C'
  
   IF lSuccess
    
      DO CASE
      CASE cType == 'N'
        uVal := 0
      CASE cType == 'D'
        uVal := CTOD( '' )
      CASE cType == 'L'
        uVal := .F.
      OTHERWISE
        uVal := ''
      ENDCASE  

      uVal := oReg:Get( cRegVar, uVal )
      IF oReg:lError
        uVal := NIL
      ENDIF

   ENDIF 

   oReg:Close()

RETURN uVal


FUNCTION SetRegistryValue( nKey, cRegKey, cRegVar, uVal )
  
   LOCAL oReg     := TReg32():New( nKey, cRegKey, .F. )
   LOCAL lSuccess := !oReg:lError

   DEFAULT cRegVar TO ''

   IF lSuccess
     oReg:Set( cRegVar, uVal )
     lSuccess := !oReg:lError
   END
   oReg:Close()

RETURN lSuccess
  

FUNCTION DeleteRegistryVar( nKey, cRegKey, cRegVar )
  
   LOCAL oReg     := TReg32():New( nKey, cRegKey, .F. )
   LOCAL lSuccess := !oReg:lError

   DEFAULT cRegVar TO ''

   IF lSuccess
     oReg:Delete( cRegVar )
     lSuccess := !oReg:lError
   END
   oReg:Close()

RETURN lSuccess
  

FUNCTION DeleteRegistryKey( nKey, cRegKey, cSubKey )

   LOCAL oReg     := TReg32():New( nKey, cRegKey, .F. )
   LOCAL lSuccess := !oReg:lError

   IF lSuccess
     oReg:KeyDelete( cSubKey )
     lSuccess := !oReg:lError
   END
   oReg:Close()

RETURN lSuccess