/*----------------------------------------------------------------------------
MINIGUI - Harbour Win32 GUI library source code

Copyright 2002-2008 Roberto Lopez <harbourminigui@gmail.com>
http://harbourminigui.googlepages.com/

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along with
   this software; see the file COPYING. If not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA (or
   visit the web site http://www.gnu.org/).

   As a special exception, you have permission for additional uses of the text
   contained in this release of Harbour Minigui.

   The exception is that, if you link the Harbour Minigui library with other
   files to produce an executable, this does not by itself cause the resulting
   executable to be covered by the GNU General Public License.
   Your use of that executable is in no way restricted on account of linking the
   Harbour-Minigui library code into it.

   Parts of this project are based upon:

   "Harbour GUI framework for Win32"
   Copyright 2001 Alexander S.Kresin <alex@belacy.belgorod.su>
   Copyright 2001 Antonio Linares <alinares@fivetech.com>
   www - http://www.harbour-project.org

   "Harbour Project"
   Copyright 1999-2008, http://www.harbour-project.org/

   "WHAT32"
   Copyright 2002 AJ Wos <andrwos@aust1.net>

   "HWGUI"
   Copyright 2001-2007 Alexander S.Kresin <alex@belacy.belgorod.su>

---------------------------------------------------------------------------*/

#include "minigui.ch"

#define TBM_SETPOS 1029
#define TBM_GETPOS 1024
#define TBS_VERT        0x0002
#define WS_CHILD    0x40000000
#define WS_TABSTOP    0x10000
#define WS_VISIBLE    0x10000000

#define TBS_AUTOTICKS           0x0001
#define TBS_TOP                 0x0004
#define TBS_LEFT                0x0004
#define TBS_BOTH                0x0008
#define TBS_NOTICKS             0x0010

*-----------------------------------------------------------------------------*
Function _DefineSlider ( ControlName, ParentFormName, x, y, w, h, lo, hi, value,;
   tooltip, scroll, change, vertical, noticks, both, top, left, HelpId, invisible, notabstop, backcolor, nId )
*-----------------------------------------------------------------------------*
   Local ParentFormHandle, Controlhandle, blInit, mVar, k, Style
   Local lDialogInMemory

   DEFAULT w         TO if( vertical, 35 + if( both, 5, 0 ), 120 )
   DEFAULT h         TO if( vertical, 120, 35 + if( both, 5, 0 ) )
   DEFAULT value     TO Int ( ( hi - lo ) / 2 )
   DEFAULT scroll    TO ""
   DEFAULT change    TO ""
   DEFAULT invisible TO FALSE
   DEFAULT notabstop TO FALSE

   if _HMG_BeginWindowActive = TRUE .or. _HMG_BeginDialogActive = TRUE
      ParentFormName := if(_HMG_BeginDialogActive,_HMG_ActiveDialogName,_HMG_ActiveFormName)
   endif
   if _HMG_FrameLevel > 0
      x    := x + _HMG_ActiveFrameCol [_HMG_FrameLevel]
      y    := y + _HMG_ActiveFrameRow [_HMG_FrameLevel]
      ParentFormName := _HMG_ActiveFrameParentFormName [_HMG_FrameLevel]
   EndIf
   lDialogInMemory := _HMG_DialogInMemory // ( GetWindowType ( ParentFormName ) == "L" )

   If .Not. _IsWindowDefined (ParentFormName) .and. .Not. lDialogInMemory
      MsgMiniGuiError("Window: "+ ParentFormName + " is not defined. Program terminated")
   Endif

   If _IsControlDefined (ControlName,ParentFormName) .and. .Not. lDialogInMemory
      MsgMiniGuiError ("Control: " + ControlName + " Of " + ParentFormName + " Already defined. Program Terminated")
   endif

   mVar := '_' + ParentFormName + '_' + ControlName
   k := _GetControlFree()

   if _HMG_BeginDialogActive = TRUE
      ParentFormHandle = _HMG_ActiveDialogHandle

      Style = WS_CHILD

      if !NoTabStop
         Style += WS_TABSTOP
      endif

      if !invisible
         Style += WS_VISIBLE
      endif

      if vertical
         Style += TBS_VERT
      endif

      if !noticks
         Style += TBS_AUTOTICKS
      else
         Style += TBS_NOTICKS
      endif

      if both
         Style += TBS_BOTH
      endif
      if top
         Style += TBS_TOP
      endif
      if left
         Style += TBS_LEFT
      endif

      if len(_HMG_aDialogTemplate) > 0        //Dialog Template

//          {{'ID',k/hwnd,class,Style,ExStyle,x,y,w,h,caption,HelpId,tooltip,font,size, bold, italic, underline, strikeout}}  --->_HMG_aDialogItems
         blInit:= {|x,y,z| InitDialogSlider(x,y,z) }
         aadd(_HMG_aDialogItems,{nId,k,"msctls_trackbar32",style,0,x,y,w,h,"",HelpId,tooltip,"",0,,,,,blInit,_HMG_BeginTabActive,.f.,_HMG_ActiveTabPage })
/* JP 62
         if _HMG_aDialogTemplate[3]   // Modal
            _HMG_aControlDeleted [k] := .T.
            Return Nil
         endif
*/
      else

         ControlHandle := GetDialogItemHandle( ParentFormHandle, nId )

         x := GetWindowCol ( Controlhandle )
         y := GetWindowRow ( Controlhandle )
         w := GetWindowWidth  ( Controlhandle )
         h := GetWindowHeight ( Controlhandle )

         SetWindowStyle ( ControlHandle,  Style )

      endif

   else

      ParentFormHandle = GetFormHandle (ParentFormName)

      ControlHandle := InitSlider ( ParentFormHandle, 0, x, y, w, h, lo, hi, vertical, noticks, both, top, left, invisible, notabstop )

   endif

   if .Not. lDialogInMemory
      If _HMG_BeginTabActive = .T.
         aAdd ( _HMG_ActiveTabCurrentPageMap , ControlHandle )
      EndIf

      SendMessage( ControlHandle , TBM_SETPOS , 1, value )

      if valtype(tooltip) != "U"
         SetToolTip ( ControlHandle , tooltip , GetFormToolTipHandle (ParentFormName) )
      endif

   endif

   Public &mVar. := k

   _HMG_aControlType [k] := "SLIDER"
   _HMG_aControlNames [k] :=  ControlName
   _HMG_aControlHandles [k] :=  ControlHandle
   _HMG_aControlParentHandles  [k] :=  ParentFormHandle
   _HMG_aControlIds  [k] := nId
   _HMG_aControlProcedures  [k] :=  scroll
   _HMG_aControlPageMap  [k] :=  {}
   _HMG_aControlValue  [k] :=  Value
   _HMG_aControlInputMask  [k] :=  ""
   _HMG_aControlLostFocusProcedure [k] :=  ""
   _HMG_aControlGotFocusProcedure  [k] :=  ""
   _HMG_aControlChangeProcedure    [k] :=  change
   _HMG_aControlDeleted [k] :=  .F.
   _HMG_aControlBkColor [k] :=  backcolor
   _HMG_aControlFontColor [k] :=  Nil
   _HMG_aControlDblClick  [k] :=  _HMG_ActiveTabButtons
   _HMG_aControlHeadClick  [k] :=  {}
   _HMG_aControlRow  [k] :=  y
   _HMG_aControlCol  [k] :=  x
   _HMG_aControlWidth   [k] :=  w
   _HMG_aControlHeight  [k] :=  h
   _HMG_aControlSpacing [k] :=  0
   _HMG_aControlContainerRow  [k] :=  iif ( _HMG_FrameLevel > 0 ,_HMG_ActiveFrameRow [_HMG_FrameLevel] , -1 )
   _HMG_aControlContainerCol  [k] :=  iif ( _HMG_FrameLevel > 0 ,_HMG_ActiveFrameCol [_HMG_FrameLevel] , -1 )
   _HMG_aControlPicture  [k] :=  ""
   _HMG_aControlContainerHandle  [k] :=  0
   _HMG_aControlFontName  [k] :=  ''
   _HMG_aControlFontSize  [k] :=  0
   _HMG_aControlFontAttributes  [k] :=  {.f.,.f.,.f.,.f.}
   _HMG_aControlToolTip  [k] :=  tooltip
   _HMG_aControlRangeMin [k] :=  Lo
   _HMG_aControlRangeMax [k] :=  Hi
   _HMG_aControlCaption  [k] :=  ''
   _HMG_aControlVisible  [k] :=  if(invisible, FALSE, TRUE)
   _HMG_aControlHelpId   [k] :=  HelpId
   _HMG_aControlFontHandle  [k] :=  iif ( _HMG_FrameLevel > 0 ,_HMG_ActiveTabName , '' )
   _HMG_aControlBrushHandle [k] :=  0
   _HMG_aControlEnabled  [k] :=  .T.
   _HMG_aControlMiscData1  [k] :=  iif ( _HMG_FrameLevel > 0 ,_HMG_ActiveFrameParentFormName [_HMG_FrameLevel] , '' )

Return Nil

*-----------------------------------------------------------------------------*
Function InitDialogSlider(ParentName,ControlHandle, k)
*-----------------------------------------------------------------------------*

   if valtype(ParentName) <> 'U'
      SendMessage( ControlHandle , TBM_SETPOS , 1 , _HMG_aControlValue [k] )
   endif
// JP 62
   if len(_HMG_aDialogTemplate) != 0 .and. _HMG_aDialogTemplate[3]   // Modal
      _HMG_aControlDeleted [k] := .T.
   endif

Return Nil