/*----------------------------------------------------------------------------
MINIGUI - Harbour Win32 GUI library source code

Copyright 2002-2008 Roberto Lopez <harbourminigui@gmail.com>
http://harbourminigui.googlepages.com/

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along with
   this software; see the file COPYING. If not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA (or
   visit the web site http://www.gnu.org/).

   As a special exception, you have permission for additional uses of the text
   contained in this release of Harbour Minigui.

   The exception is that, if you link the Harbour Minigui library with other
   files to produce an executable, this does not by itself cause the resulting
   executable to be covered by the GNU General Public License.
   Your use of that executable is in no way restricted on account of linking the
   Harbour-Minigui library code into it.

   Parts of this project are based upon:

   "Harbour GUI framework for Win32"
   Copyright 2001 Alexander S.Kresin <alex@belacy.belgorod.su>
   Copyright 2001 Antonio Linares <alinares@fivetech.com>
   www - http://www.harbour-project.org

   "Harbour Project"
   Copyright 1999-2008, http://www.harbour-project.org/

   "WHAT32"
   Copyright 2002 AJ Wos <andrwos@aust1.net>

   "HWGUI"
   Copyright 2001-2007 Alexander S.Kresin <alex@belacy.belgorod.su>

---------------------------------------------------------------------------*/

#include "minigui.ch"

*-----------------------------------------------------------------------------*
Function _DefineSpinner ( ControlName, ParentForm, x, y, w, value ,fontname,;
   fontsize, rl, rh, tooltip, change, lostfocus,;
   gotfocus, h, HelpId, invisible, notabstop , bold,;
   italic, underline, strikeout, wrap, readonly, increment , backcolor , fontcolor )
*-----------------------------------------------------------------------------*
   Local cParentForm , RetArray , mVar , k
   Local ControlHandle , FontHandle , aFont

   DEFAULT w         TO 120
   DEFAULT h         TO 24
   DEFAULT value     TO rl
   DEFAULT change    TO ""
   DEFAULT lostfocus TO ""
   DEFAULT gotfocus  TO ""
   DEFAULT invisible TO FALSE
   DEFAULT notabstop TO FALSE
   DEFAULT wrap      TO FALSE
   DEFAULT readonly  TO FALSE
   DEFAULT increment TO 1

   if ( FontHandle := GetFontHandle( FontName ) ) != 0
      aFont := GetFontParam(FontHandle)
      FontName      := aFont[1]
      FontSize      := aFont[2]
      bold          := aFont[3]
      italic        := aFont[4]
      underline     := aFont[5]
      strikeout     := aFont[6]
   endif

   if _HMG_BeginWindowActive = .T.
      ParentForm := _HMG_ActiveFormName
      if .Not. Empty (_HMG_ActiveFontName) .And. ValType(FontName) == "U"
         FontName := _HMG_ActiveFontName
      EndIf
      if .Not. Empty (_HMG_ActiveFontSize) .And. ValType(FontSize) == "U"
         FontSize := _HMG_ActiveFontSize
      EndIf
   endif
   if _HMG_FrameLevel > 0
      x 	:= x + _HMG_ActiveFrameCol [_HMG_FrameLevel]
      y 	:= y + _HMG_ActiveFrameRow [_HMG_FrameLevel]
      ParentForm := _HMG_ActiveFrameParentFormName [_HMG_FrameLevel]
   EndIf

   If .Not. _IsWindowDefined (ParentForm)
      MsgMiniGuiError("Window: "+ ParentForm + " is not defined. Program terminated")
   Endif

   If _IsControlDefined (ControlName,ParentForm)
      MsgMiniGuiError ("Control: " + ControlName + " Of " + ParentForm + " Already defined. Program Terminated")
   endif

   mVar := '_' + ParentForm + '_' + ControlName

   cParentForm := ParentForm

   ParentForm = GetFormHandle (ParentForm)

   RetArray := InitSpinner ( ParentForm, 0, x, y, w , '' , 0 , rl, rh , h, invisible, notabstop, wrap, readonly )

   ControlHandle := RetArray [1]

   If FontHandle != 0
      _SetFontHandle(ControlHandle,FontHandle)
   Else
      if valtype(fontname) == "U"
         Fontname := _HMG_DefaultFontName
      endif
      if valtype(fontsize) == "U"
         FontSize := _HMG_DefaultFontSize
      endif
      FontHandle := _SetFont (ControlHandle,FontName,FontSize,bold,italic,underline,strikeout)
   EndIf

   If _HMG_BeginTabActive = .T.
      aAdd ( _HMG_ActiveTabCurrentPageMap , RetArray )
   EndIf

   if valtype(tooltip) != "U"
      SetToolTip ( ControlHandle , tooltip , GetFormToolTipHandle (cParentForm) )
   endif

   k := _GetControlFree()

   Public &mVar. := k

   _HMG_aControlType [k] := "SPINNER"
   _HMG_aControlNames  [k] :=  ControlName
   _HMG_aControlHandles [k] :=   RetArray
   _HMG_aControlParenthandles [k] :=   ParentForm
   _HMG_aControlIds  [k] :=  0
   _HMG_aControlProcedures  [k] :=  ""
   _HMG_aControlPageMap  [k] :=  {}
   _HMG_aControlValue  [k] :=  0
   _HMG_aControlInputMask  [k] :=  ""
   _HMG_aControllostFocusProcedure  [k] :=  lostfocus
   _HMG_aControlGotFocusProcedure [k] :=   gotfocus
   _HMG_aControlChangeProcedure  [k] :=  change
   _HMG_aControlDeleted  [k] :=  .F.
   _HMG_aControlBkColor  [k] :=  backcolor
   _HMG_aControlFontColor  [k] :=  fontcolor
   _HMG_aControlDblClick  [k] :=  ""
   _HMG_aControlHeadClick  [k] :=  {}
   _HMG_aControlRow [k] :=   y
   _HMG_aControlCol  [k] :=  x
   _HMG_aControlWidth  [k] :=  w
   _HMG_aControlHeight   [k] := h
   _HMG_aControlSpacing  [k] :=  0
   _HMG_aControlContainerRow  [k] :=  iif ( _HMG_FrameLevel > 0 ,_HMG_ActiveFrameRow [_HMG_FrameLevel] , -1 )
   _HMG_aControlContainerCol  [k] :=  iif ( _HMG_FrameLevel > 0 ,_HMG_ActiveFrameCol [_HMG_FrameLevel] , -1 )
   _HMG_aControlPicture  [k] :=  ""
   _HMG_aControlContainerHandle  [k] :=  0
   _HMG_aControlFontName [k] :=   fontname
   _HMG_aControlFontSize  [k] :=  fontsize
   _HMG_aControlFontAttributes  [k] :=  {bold,italic,underline,strikeout}
   _HMG_aControlToolTip  [k] :=   tooltip
   _HMG_aControlRangeMin  [k] :=   Rl
   _HMG_aControlRangeMax  [k] :=   Rh
   _HMG_aControlCaption  [k] :=   ''
   _HMG_aControlVisible  [k] :=   if(invisible,FALSE,TRUE)
   _HMG_aControlHelpId  [k] :=   HelpId
   _HMG_aControlFontHandle  [k] :=   FontHandle
   _HMG_aControlBrushHandle  [k] :=   0
   _HMG_aControlEnabled  [k] :=   .T.
   _HMG_aControlMiscData1 [k] := {0, readonly }

   if valtype(value) == "N"
      SetSpinnerValue ( RetArray [2] , Value )
   endif

   if increment <> 1
      SetSpinnerIncrement( RetArray[2], increment )
   endif

Return Nil

#define WM_CONTEXTMENU   123
#define WM_GETDLGCODE    135
*-----------------------------------------------------------------------------*
Function OSPINEVENTS( hWnd, nMsg, wParam, lParam )      // 2006.08.13 JD
*-----------------------------------------------------------------------------*
LOCAL i, ParentForm

SWITCH nMsg

CASE WM_GETDLGCODE

   if _HMG_ExtendedNavigation == .T.
      if wParam == 0x0D					// Return key pressed
         if CheckBit( GetKeyState( VK_SHIFT ), 32768 )	// Is Shift key pressed?
            InsertShiftTab()
         else
            InsertTab()
         endif
      endif
   endif
   EXIT

CASE WM_CONTEXTMENU

   i := Ascan( _HMG_aControlHandles , {|x| IIF(VALTYPE(x)=="A", (Ascan(x, hWnd)>0), x == hWnd)} )
   ParentForm := _HMG_aControlParentHandles[i]
   i := Ascan( _HMG_aControlsContextMenu , {|x| x[1] == hWnd} )
   IF i > 0
      IF _HMG_aControlsContextMenu[i, 4] == .T.
         setfocus(wParam)
         _HMG_xControlsContextMenuID := _HMG_aControlsContextMenu[i, 3]
         TrackPopupMenu ( _HMG_aControlsContextMenu[i, 2] , LOWORD(lparam) , HIWORD(lparam) , ParentForm )
         RETURN 1
      ENDIF
   ENDIF

END

Return 0