/*----------------------------------------------------------------------------
MINIGUI - Harbour Win32 GUI library source code

Copyright 2002-2008 Roberto Lopez <harbourminigui@gmail.com>
http://harbourminigui.googlepages.com/

Modified 02.10.2005 to recognize insert/overwrite INS key status
(C) Jacek Kubica <kubica@wssk.wroc.pl>
HMG Experimental Build 10d

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along with
   this software; see the file COPYING. If not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA (or
   visit the web site http://www.gnu.org/).

   As a special exception, you have permission for additional uses of the text
   contained in this release of Harbour Minigui.

   The exception is that, if you link the Harbour Minigui library with other
   files to produce an executable, this does not by itself cause the resulting
   executable to be covered by the GNU General Public License.
   Your use of that executable is in no way restricted on account of linking the
   Harbour-Minigui library code into it.

   Parts of this project are based upon:

   "Harbour GUI framework for Win32"
   Copyright 2001 Alexander S.Kresin <alex@belacy.belgorod.su>
   Copyright 2001 Antonio Linares <alinares@fivetech.com>
   www - http://www.harbour-project.org

   "Harbour Project"
   Copyright 1999-2008, http://www.harbour-project.org/

   "WHAT32"
   Copyright 2002 AJ Wos <andrwos@aust1.net>

   "HWGUI"
   Copyright 2001-2007 Alexander S.Kresin <alex@belacy.belgorod.su>

---------------------------------------------------------------------------*/

#include "minigui.ch"

#define EM_LIMITTEXT        197
#define ES_WANTRETURN       4096
#define ES_MULTILINE        0x0004
#define ES_AUTOVSCROLL      0x0040
#define ES_AUTOHSCROLL      128
#define WS_CHILD      0x40000000
#define WS_BORDER     0x800000
#define WS_TABSTOP    0x10000
#define WS_VISIBLE    0x10000000
#define WS_HSCROLL    0x100000
#define WS_VSCROLL    0x200000
#define BS_FLAT             0x00008000
#define ES_NUMBER           8192
#define ES_RIGHT            2
#define ES_UPPERCASE        8
#define ES_LOWERCASE        16
#define ES_PASSWORD         32
#define ES_READONLY         2048
#define WS_THICKFRAME       0x40000
#define EM_GETSEL           176
#define EM_SETSEL           177
#define WM_SETTEXT          12
#define WM_CLEAR            771
#define WM_CHAR             258
#define WM_CONTEXTMENU      123

*-----------------------------------------------------------------------------*
Function _DefineTextBox ( ControlName, ParentFormName, x, y, w, h,;
   cValue, FontName, FontSize, ToolTip, nMaxLength, lUpper, lLower, lNumeric, lPassword,;
   uLostFocus, uGotFocus, uChange, uEnter, right, HelpId , readonly , bold, italic, underline,;
   strikeout , field , backcolor , fontcolor , invisible , notabstop , nId )
*-----------------------------------------------------------------------------*
   local ParentFormHandle
   local ControlHandle := 0
   local mVar
   Local FontHandle, aFont
   Local WorkArea, blInit
   Local k, Style
   Local lDialogInMemory

   // Asign STANDARD values to optional params.
   DEFAULT w          TO 120
   DEFAULT h          TO 24
   DEFAULT cValue     TO ""
   DEFAULT uChange    TO ""
   DEFAULT uGotFocus  TO ""
   DEFAULT uLostFocus TO ""
   DEFAULT nMaxLength TO 255
   DEFAULT lUpper     TO .f.
   DEFAULT lLower     TO .f.
   DEFAULT lNumeric   TO .f.
   DEFAULT lPassword  TO .f.
   DEFAULT uEnter     TO ""

   If ValType ( Field ) != 'U'
      if  at ( '>', Field ) == 0
         MsgMiniGuiError ("Control: " + ControlName + " Of " + ParentFormName + " : You must specify a fully qualified field name. Program Terminated")
      Else
         WorkArea := Left ( Field , at ( '>', Field ) - 2 )
         If Select (WorkArea) != 0
            cValue := &(Field)
         EndIf
      EndIf
   EndIf

   if ( FontHandle := GetFontHandle( FontName ) ) != 0
      aFont := GetFontParam(FontHandle)
      FontName      := aFont[1]
      FontSize      := aFont[2]
      bold          := aFont[3]
      italic        := aFont[4]
      underline     := aFont[5]
      strikeout     := aFont[6]
   endif

   if _HMG_BeginWindowActive = TRUE .or. _HMG_BeginDialogActive = TRUE
      ParentFormName := if(_HMG_BeginDialogActive,_HMG_ActiveDialogName,_HMG_ActiveFormName)
      if .Not. Empty (_HMG_ActiveFontName) .And. ValType(FontName) == "U"
         FontName := _HMG_ActiveFontName
      EndIf
      if .Not. Empty (_HMG_ActiveFontSize) .And. ValType(FontSize) == "U"
         FontSize := _HMG_ActiveFontSize
      EndIf
   endif

   if _HMG_FrameLevel > 0
      x    := x + _HMG_ActiveFrameCol [_HMG_FrameLevel]
      y    := y + _HMG_ActiveFrameRow [_HMG_FrameLevel]
      ParentFormName := _HMG_ActiveFrameParentFormName [_HMG_FrameLevel]
   EndIf
   lDialogInMemory := _HMG_DialogInMemory // ( GetWindowType ( ParentFormName ) == "L" )

   // Check if the window/form is defined.
   if ( .not. _IsWindowDefined( ParentFormName ) ) .and. .Not. lDialogInMemory
      MsgMiniGuiError( "Window: " + ParentFormName + " is not defined. Program terminated." )
   endif

   // Check if the control is already defined.
   if ( _IsControlDefined( ControlName, ParentFormName ) ) .and. .Not. lDialogInMemory
      MsgMiniGuiError( "Control: " + ControlName + " of " + ParentFormName + " already defined. Program Terminated." )
   endif

   mVar := '_' + ParentFormName + '_' + ControlName
   k := _GetControlFree()

   if _HMG_BeginDialogActive = TRUE
      ParentFormHandle = _HMG_ActiveDialogHandle

      Style = WS_CHILD + ES_AUTOHSCROLL + BS_FLAT + WS_BORDER

      if lNumeric
         Style += ES_NUMBER
      else
         if lUpper
            Style += ES_UPPERCASE
         endif
         if lLower
            Style += ES_LOWERCASE
         endif
      endif

      if lPassword
         Style += ES_PASSWORD
      endif

      if right
         Style += ES_RIGHT
      endif

      if readonly
         Style += ES_READONLY
      endif

      if !invisible
         Style += WS_VISIBLE
      endif

      if  !notabstop
         Style += WS_TABSTOP
      endif

      if len(_HMG_aDialogTemplate) > 0        //Dialog Template

//          {{'ID',k/hwnd,class,Style,ExStyle,x,y,w,h,caption,HelpId,tooltip,font,size, bold, italic, underline, strikeout}}  --->_HMG_aDialogItems
         blInit:= {|x,y,z| InitDialogTextBox(x,y,z) }
         aadd(_HMG_aDialogItems,{nId,k,"EDIT",style,0,x,y,w,h,cValue,HelpId,Tooltip,FontName,FontSize,bold,italic,underline,strikeout,blInit,_HMG_BeginTabActive,.f.,_HMG_ActiveTabPage })
/* JP 62
         if _HMG_aDialogTemplate[3]   // Modal
            _HMG_aControlDeleted [k] := .T.
            Return Nil
         endif
*/
      else

         ControlHandle := GetDialogItemHandle( ParentFormHandle, nId )

         x := GetWindowCol ( Controlhandle )
         y := GetWindowRow ( Controlhandle )
         w := GetWindowWidth  ( Controlhandle )
         h := GetWindowHeight ( Controlhandle )

         SetWindowStyle ( ControlHandle, Style )
      endif

   else

      ParentFormHandle  := GetFormHandle( ParentFormName )
       // Creates the control window
      ControlHandle := InitTextBox( ParentFormHandle, 0, x, y, w, h, '', 0, nMaxLength,;
         lUpper, lLower, .f., lPassword , right , readonly , invisible , notabstop )

   endif

   if .Not. lDialogInMemory

      If FontHandle != 0
         _SetFontHandle(ControlHandle,FontHandle)
      Else
         if valtype(fontname) == "U"
            FontName := _HMG_DefaultFontName
         endif
         if valtype(fontsize) == "U"
            FontSize := _HMG_DefaultFontSize
         endif
         FontHandle := _SetFont (ControlHandle,FontName,FontSize,bold,italic,underline,strikeout)
      EndIf

      If _HMG_BeginTabActive = .T.
         aAdd ( _HMG_ActiveTabCurrentPageMap , ControlHandle )
      EndIf

       // Add a tooltip if param has value
      if ( ValType( ToolTip ) != "U" )
         SetToolTip( ControlHandle, ToolTip, GetFormToolTipHandle( ParentFormName ) )
      endif
   endif

   Public &mVar. := k

   _HMG_aControlType [k] := if( lNumeric, "NUMTEXT", "TEXT" )
   _HMG_aControlNames  [k] :=  ControlName
   _HMG_aControlHandles  [k] :=  ControlHandle
   _HMG_aControlParenthandles  [k] :=  ParentFormHandle
   _HMG_aControlIds  [k] :=  nId
   _HMG_aControlProcedures  [k] :=  ""
   _HMG_aControlPageMap  [k] :=  Field
   _HMG_aControlValue  [k] :=  cValue
   _HMG_aControlInputMask  [k] :=  ""
   _HMG_aControlLostFocusProcedure [k] :=   uLostFocus
   _HMG_aControlGotFocusProcedure  [k] := uGotFocus
   _HMG_aControlChangeProcedure  [k] :=  uChange
   _HMG_aControlDeleted  [k] :=  .F.
   _HMG_aControlBkColor  [k] :=  backcolor
   _HMG_aControlFontColor [k] :=   fontcolor
   _HMG_aControlDblClick  [k] :=  uEnter
   _HMG_aControlHeadClick  [k] :=  {}
   _HMG_aControlRow  [k] :=  y
   _HMG_aControlCol  [k] :=  x
   _HMG_aControlWidth   [k] := w
   _HMG_aControlHeight   [k] := h
   _HMG_aControlSpacing  [k] :=  0
   _HMG_aControlContainerRow  [k] :=  iif ( _HMG_FrameLevel > 0 ,_HMG_ActiveFrameRow [_HMG_FrameLevel] , -1 )
   _HMG_aControlContainerCol  [k] :=  iif ( _HMG_FrameLevel > 0 ,_HMG_ActiveFrameCol [_HMG_FrameLevel] , -1 )
   _HMG_aControlPicture  [k] :=  ""
   _HMG_aControlContainerHandle  [k] :=  0
   _HMG_aControlFontName  [k] :=  FontName
   _HMG_aControlFontSize  [k] :=  FontSize
   _HMG_aControlFontAttributes  [k] :=  {bold,italic,underline,strikeout}
   _HMG_aControlToolTip  [k] :=   ToolTip
   _HMG_aControlRangeMin  [k] :=   0
   _HMG_aControlRangeMax  [k] :=   nMaxLength
   _HMG_aControlCaption  [k] :=   ''
   _HMG_aControlVisible  [k] :=  .Not.  invisible
   _HMG_aControlHelpId  [k] :=   HelpId
   _HMG_aControlFontHandle  [k] :=   FontHandle
   _HMG_aControlBrushHandle  [k] :=   0
   _HMG_aControlEnabled  [k] :=   .T.
   _HMG_aControlMiscData1 [k] := {0, readonly }

   if .Not. lDialogInMemory
       // With NUMERIC clause, transform numeric value into a string.
      If lNumeric
         If Valtype( cValue ) != 'C'
            cValue := AllTrim( Str( cValue ) )
         EndIf
      EndIf

       // Fill the TEXTBOX with the text given.
      If Len( cValue ) > 0
         SetWindowText( ControlHandle , cValue )
      EndIf

      If Valtype( Field ) != 'U'
         aAdd( _HMG_aFormBrowseList [ GetFormIndex ( ParentFormName ) ] , k )
      EndIf
   endif

return nil

*-----------------------------------------------------------------------------*
Function InitDialogTextBox(ParentName, ControlHandle, k)
*-----------------------------------------------------------------------------*
   Local readonly, nMaxLength, Field, cValue, lNumeric
   Field       := _HMG_aControlPageMap  [k]
   nMaxLength  := _HMG_aControlRangeMax  [k]
   readonly    := _HMG_aControlMiscData1 [k,2]
   cValue      := _HMG_aControlValue  [k]
   lNumeric    := _HMG_aControlType [k] == "NUMTEXT"

   if valtype ( readonly ) == 'L'
      SetTextEditReadOnly ( ControlHandle , readonly )
   endif
   if valtype ( nMaxLength ) != 'U'
      SendMessage( ControlHandle , EM_LIMITTEXT , nMaxLength , 0 )
   endif

   // With NUMERIC clause, transform numeric value into a string.
   if ( lNumeric )
      If Valtype(cValue) != 'C'
         cValue := AllTrim( Str( cValue ) )
      EndIf
   EndIf

   // Fill the TEXTBOX with the text given.
   if ( Len( cValue ) > 0 )
      SetWindowText ( ControlHandle , cValue )
   endif

   if valtype ( Field ) != 'U'
      aAdd ( _HMG_aFormBrowseList    [ GetFormIndex ( ParentName ) ] , k )
   EndIf
// JP 62
   if len(_HMG_aDialogTemplate) != 0 .and. _HMG_aDialogTemplate[3]   // Modal
      _HMG_aControlDeleted [k] := .T.
   ENDIF

Return Nil

*-----------------------------------------------------------------------------*
Function _DefineMaskedTextbox ( ControlName, ParentFormName, x, y, inputmask , w , value , fontname, fontsize , tooltip , lostfocus , gotfocus , change , h , enter , rightalign , HelpId , Format , bold, italic, underline, strikeout , field , backcolor , fontcolor , readonly , invisible , notabstop , nId )
*-----------------------------------------------------------------------------*
   Local i, ParentFormHandle , c , mVar , WorkArea , k , Style
   Local ControlHandle, blInit
   Local FontHandle, aFont
   Local lDialogInMemory

   HB_SYMBOL_UNUSED( RightAlign )

   If ValType ( Field ) != 'U'
      if  at ( '>', Field ) == 0
         MsgMiniGuiError ("Control: " + ControlName + " Of " + ParentFormName + " : You must specify a fully qualified field name. Program Terminated" )
      Else
         WorkArea := Left ( Field , at ( '>', Field ) - 2 )
         If Select (WorkArea) != 0
            Value := &(Field)
         EndIf
      EndIf
   EndIf

   if valtype(Format) == "U"
      Format := ""
   endif

   For i := 1 To Len (InputMask)

      c := SubStr ( InputMask , i , 1 )

      if c!='9' .and.  c!='$' .and. c!='*' .and. c!='.' .and. c!= ','  .and. c != ' ' .and. c!=''
         MsgMiniGuiError( "@...TEXTBOX: Wrong InputMask Definition" )
      EndIf

   Next i

   For i := 1 To Len (Format)

      c := SubStr ( Format , i , 1 )

      if c!='C' .and. c!='X' .and. c!= '(' .and. c!= 'E'
         MsgMiniGuiError( "@...TEXTBOX: Wrong Format Definition" )
      EndIf

   Next i

   if valtype(change) == "U"
      change := ""
   endif

   if valtype(gotfocus) == "U"
      gotfocus := ""
   endif

   if valtype(enter) == "U"
      enter := ""
   endif

   if valtype(lostfocus) == "U"
      lostfocus := ""
   endif

   if valtype(w) == "U"
      W := 120
   endif

   if valtype(h) == "U"
      h := 24
   endif

   if valtype(Value) == "U"
      Value := ""
   endif

   If .Not. Empty (Format)
      Format := '@' + AllTrim(Format)
   EndIf

   InputMask :=  Format + ' ' + InputMask

   Value := Transform ( value , InputMask )

   if ( FontHandle := GetFontHandle( FontName ) ) != 0
      aFont := GetFontParam(FontHandle)
      FontName      := aFont[1]
      FontSize      := aFont[2]
      bold          := aFont[3]
      italic        := aFont[4]
      underline     := aFont[5]
      strikeout     := aFont[6]
   endif

   if _HMG_BeginWindowActive = TRUE .or. _HMG_BeginDialogActive = TRUE
      ParentFormName := if(_HMG_BeginDialogActive,_HMG_ActiveDialogName,_HMG_ActiveFormName)
      if .Not. Empty (_HMG_ActiveFontName) .And. ValType(FontName) == "U"
         FontName := _HMG_ActiveFontName
      EndIf
      if .Not. Empty (_HMG_ActiveFontSize) .And. ValType(FontSize) == "U"
         FontSize := _HMG_ActiveFontSize
      EndIf
   endif
   if _HMG_FrameLevel > 0
      x    := x + _HMG_ActiveFrameCol [_HMG_FrameLevel]
      y    := y + _HMG_ActiveFrameRow [_HMG_FrameLevel]
      ParentFormName := _HMG_ActiveFrameParentFormName [_HMG_FrameLevel]
   EndIf
   lDialogInMemory := _HMG_DialogInMemory // ( GetWindowType ( ParentFormName ) == "L" )

   If .Not. _IsWindowDefined (ParentFormName) .and. .Not. lDialogInMemory
      MsgMiniGuiError( "Window: "+ ParentFormName + " is not defined. Program terminated" )
   Endif

   If _IsControlDefined (ControlName,ParentFormName) .and. .Not. lDialogInMemory
      MsgMiniGuiError ( "Control: " + ControlName + " Of " + ParentFormName + " Already defined. Program Terminated" )
   endif

   mVar := '_' + ParentFormName + '_' + ControlName
   k := _GetControlFree()

   if _HMG_BeginDialogActive = TRUE
      ParentFormHandle = _HMG_ActiveDialogHandle

      Style = WS_CHILD + ES_AUTOHSCROLL

      if readonly
         Style += ES_READONLY
      endif

      if invisible
         Style += WS_VISIBLE
      endif

      if  !notabstop
         Style += WS_TABSTOP
      endif

      if lDialogInMemory         //Dialog Template

//          {{'ID',k/hwnd,class,Style,ExStyle,x,y,w,h,caption,HelpId,tooltip,font,size, bold, italic, underline, strikeout}}  --->_HMG_aDialogItems
         blInit:= {|x,y,z| InitDialogMaskedTextBox(x,y,z) }
         aadd(_HMG_aDialogItems,{nId,k,"EDIT",style,0,x,y,w,h,Value,HelpId,tooltip,FontName,FontSize,bold,italic,underline,strikeout,blInit,_HMG_BeginTabActive,.f.,_HMG_ActiveTabPage })
/* JP 62
         if _HMG_aDialogTemplate[3]   // Modal
            _HMG_aControlDeleted [k] := .T.
            Return Nil
         endif
*/
      else

         ControlHandle := GetDialogItemHandle( ParentFormHandle, nId )

         x := GetWindowCol ( Controlhandle )
         y := GetWindowRow ( Controlhandle )
         w := GetWindowWidth  ( Controlhandle )
         h := GetWindowHeight ( Controlhandle )

         SetWindowStyle ( ControlHandle, Style )
      endif

   else
      ParentFormHandle = GetFormHandle (ParentFormName)
      ControlHandle := InitMaskedTextBox ( ParentFormHandle, 0, x, y, w , '' , 0  , 255 , .f. , .f. , h , .t. , readonly  , invisible , notabstop )
   endif

   if .Not. lDialogInMemory

      If FontHandle != 0
         _SetFontHandle(ControlHandle,FontHandle)
      Else
         if valtype(fontname) == "U"
            FontName := _HMG_DefaultFontName
         endif
         if valtype(fontsize) == "U"
            FontSize := _HMG_DefaultFontSize
         endif
         FontHandle := _SetFont (ControlHandle,FontName,FontSize,bold,italic,underline,strikeout)
      EndIf

      If _HMG_BeginTabActive = .T.
         aAdd ( _HMG_ActiveTabCurrentPageMap , ControlHandle )
      EndIf

      if valtype(tooltip) != "U"
         SetToolTip ( ControlHandle , tooltip , GetFormToolTipHandle (ParentFormName) )
      endif
   endif

   Public &mVar. := k

   _HMG_aControlType [k] := "MASKEDTEXT"
   _HMG_aControlNames  [k] :=  ControlName
   _HMG_aControlHandles [k] :=   ControlHandle
   _HMG_aControlParenthandles [k] :=   ParentFormHandle
   _HMG_aControlIds  [k] :=  nId
   _HMG_aControlProcedures  [k] :=  ""
   _HMG_aControlPageMap [k] :=   InputMask
   _HMG_aControlValue  [k] :=  Value
   _HMG_aControlInputMask  [k] :=  GetNumMask ( InputMask )
   _HMG_aControllostFocusProcedure  [k] :=  lostfocus
   _HMG_aControlGotFocusProcedure  [k] :=  gotfocus
   _HMG_aControlChangeProcedure  [k] :=  Change
   _HMG_aControlDeleted  [k] :=  .F.
   _HMG_aControlBkColor  [k] :=  backcolor
   _HMG_aControlFontColor  [k] :=  fontcolor
   _HMG_aControlDblClick  [k] :=  enter
   _HMG_aControlHeadClick  [k] :=  Field
   _HMG_aControlRow  [k] :=  y
   _HMG_aControlCol  [k] :=  x
   _HMG_aControlWidth  [k] :=  w
   _HMG_aControlHeight  [k] :=  h
   _HMG_aControlSpacing  [k] :=  .F.
   _HMG_aControlContainerRow  [k] :=  iif ( _HMG_FrameLevel > 0 ,_HMG_ActiveFrameRow [_HMG_FrameLevel] , -1 )
   _HMG_aControlContainerCol  [k] :=  iif ( _HMG_FrameLevel > 0 ,_HMG_ActiveFrameCol [_HMG_FrameLevel] , -1 )
   _HMG_aControlPicture   [k] := ""
   _HMG_aControlContainerHandle  [k] :=  0
   _HMG_aControlFontName  [k] :=  fontname
   _HMG_aControlFontSize  [k] :=  fontsize
   _HMG_aControlFontAttributes  [k] :=  {bold,italic,underline,strikeout}
   _HMG_aControlToolTip   [k] :=  tooltip
   _HMG_aControlRangeMin  [k] :=   0
   _HMG_aControlRangeMax  [k] :=   0
   _HMG_aControlCaption  [k] :=   ''
   _HMG_aControlVisible  [k] :=  .Not.  invisible
   _HMG_aControlHelpId  [k] :=   HelpId
   _HMG_aControlFontHandle  [k] :=   FontHandle
   _HMG_aControlBrushHandle   [k] :=  0
   _HMG_aControlEnabled  [k] :=   .T.
   _HMG_aControlMiscData1 [k] := {0 , readonly }

   if .Not. lDialogInMemory
      SetWindowText ( ControlHandle , value )

      if valtype ( Field ) != 'U'
         aAdd ( _HMG_aFormBrowseList [ GetFormIndex ( ParentFormName ) ] , k )
      EndIf
   endif

Return Nil

*-----------------------------------------------------------------------------*
Function InitDialogMaskedTextBox(ParentName, ControlHandle, k)
*-----------------------------------------------------------------------------*
   Local Field, cValue, date
   Field       := _HMG_aControlPageMap  [k]
   date        := _HMG_aControlMiscData1 [k,2]
   cValue      := _HMG_aControlValue  [k]

   if date == .F.
      SetWindowText ( ControlHandle , cValue  )
   Else
      SetWindowText ( ControlHandle , dtoc ( cValue ) )
   endif

   if valtype ( Field ) != 'U'
      aAdd ( _HMG_aFormBrowseList [ GetFormIndex ( ParentName ) ] , k )
   EndIf

    // Fill the TEXTBOX with the text given.
   if ( Len( cValue ) > 0 )
      SetWindowText ( ControlHandle , cValue )
   endif

   if valtype ( Field ) != 'U'
      aAdd ( _HMG_aFormBrowseList [ GetFormIndex ( ParentName ) ] , k )
   EndIf
// JP 62
   if len(_HMG_aDialogTemplate) != 0 .and. _HMG_aDialogTemplate[3]   // Modal
      _HMG_aControlDeleted [k] := .T.
   endif

Return Nil

Function GetNumFromText ( Text , i )
   Local x , c , s

   s := ''

   For x := 1 To Len ( Text )

      c := SubStr(Text,x,1)

      If c='0' .or. c='1' .or. c='2' .or. c='3' .or. c='4' .or. c='5' .or. c='6' .or. c='7' .or. c='8' .or. c='9' .or. c='.' .or. c='-'
         s := s + c
      EndIf

   Next x

   If Left ( AllTrim(Text) , 1 ) == '(' .OR.  Right ( AllTrim(Text) , 2 ) == 'DB'
      s := '-' + s
   EndIf

   s := Transform ( Val(s) , _HMG_aControlInputMask [i] )

Return Val(s)

Function GetNumMask ( Text )
   Local i , c , s

   s := ''

   For i := 1 To Len ( Text )

      c := SubStr(Text,i,1)

      If c='9' .or. c='.'
         s := s + c
      EndIf

      if c = '$' .or. c = '*'
         s := s+'9'
      EndIf

   Next i

Return s

*-----------------------------------------------------------------------------*
Function _DefineCharMaskTextbox ( ControlName, ParentFormName, x, y, inputmask , w , value ,;
   fontname, fontsize , tooltip , lostfocus , gotfocus , change , h , enter , rightalign , HelpId ,;
   bold, italic, underline, strikeout , field , backcolor , fontcolor , date , readonly , invisible , notabstop , nId )
*-----------------------------------------------------------------------------*
   Local ParentFormHandle , mVar , WorkArea , dateformat , k , Style
   Local ControlHandle, blInit
   Local FontHandle, aFont
   Local lDialogInMemory

   If ValType ( Field ) != 'U'
      if  at ( '>', Field ) == 0
         MsgMiniGuiError ( "Control: " + ControlName + " Of " + ParentFormName + " : You must specify a fully qualified field name. Program Terminated" )
      Else
         WorkArea := Left ( Field , at ( '>', Field ) - 2 )
         If Select (WorkArea) != 0
            Value := &(Field)
         EndIf
      EndIf
   EndIf

   if valtype(date) == "U"
      date := .F.
   endif

   if valtype(change) == "U"
      change := ""
   endif

   if valtype(gotfocus) == "U"
      gotfocus := ""
   endif

   if valtype(enter) == "U"
      enter := ""
   endif

   if valtype(lostfocus) == "U"
      lostfocus := ""
   endif

   if valtype(w) == "U"
      w := 120
   endif

   if valtype(h) == "U"
      h := 24
   endif

   if valtype(Value) == "U"
      if date == .F.
         Value := ""
      else
         Value := ctod ('  /  /  ')
      endif
   endif

   dateformat := set ( _SET_DATEFORMAT )

   if date == .t.
      if lower ( left ( dateformat , 4 ) ) == "yyyy"

         if '/' $ dateformat
            Inputmask := '9999/99/99'
         Elseif '.' $ dateformat
            Inputmask := '9999.99.99'
         Elseif '-' $ dateformat
            Inputmask := '9999-99-99'
         EndIf

      elseif lower ( right ( dateformat , 4 ) ) == "yyyy"

         if '/' $ dateformat
            Inputmask := '99/99/9999'
         Elseif '.' $ dateformat
            Inputmask := '99.99.9999'
         Elseif '-' $ dateformat
            Inputmask := '99-99-9999'
         EndIf

      else

         if '/' $ dateformat
            Inputmask := '99/99/99'
         Elseif '.' $ dateformat
            Inputmask := '99.99.99'
         Elseif '-' $ dateformat
            Inputmask := '99-99-99'
         EndIf

      endif
   endif

   if ( FontHandle := GetFontHandle( FontName ) ) != 0
      aFont := GetFontParam(FontHandle)
      FontName      := aFont[1]
      FontSize      := aFont[2]
      bold          := aFont[3]
      italic        := aFont[4]
      underline     := aFont[5]
      strikeout     := aFont[6]
   endif

   if _HMG_BeginWindowActive = TRUE .or. _HMG_BeginDialogActive = TRUE
      ParentFormName := if(_HMG_BeginDialogActive,_HMG_ActiveDialogName,_HMG_ActiveFormName)
      if .Not. Empty (_HMG_ActiveFontName) .And. ValType(FontName) == "U"
         FontName := _HMG_ActiveFontName
      EndIf
      if .Not. Empty (_HMG_ActiveFontSize) .And. ValType(FontSize) == "U"
         FontSize := _HMG_ActiveFontSize
      EndIf
   endif
   if _HMG_FrameLevel > 0
      x    := x + _HMG_ActiveFrameCol [_HMG_FrameLevel]
      y    := y + _HMG_ActiveFrameRow [_HMG_FrameLevel]
      ParentFormName := _HMG_ActiveFrameParentFormName [_HMG_FrameLevel]
   EndIf
   lDialogInMemory := _HMG_DialogInMemory // ( GetWindowType ( ParentFormName ) == "L" )

   If .Not. _IsWindowDefined (ParentFormName) .and. .Not. lDialogInMemory
      MsgMiniGuiError("Window: "+ ParentFormName + " is not defined. Program terminated" )
   Endif

   If _IsControlDefined (ControlName,ParentFormName) .and. .Not. lDialogInMemory
      MsgMiniGuiError ("Control: " + ControlName + " Of " + ParentFormName + " Already defined. Program Terminated" )
   endif

   mVar := '_' + ParentFormName + '_' + ControlName
   k := _GetControlFree()

   if _HMG_BeginDialogActive = TRUE
      ParentFormHandle = _HMG_ActiveDialogHandle

      Style = WS_CHILD + ES_AUTOHSCROLL

      if readonly
         Style += ES_READONLY
      endif

      if invisible
         Style += WS_VISIBLE
      endif

      if  !notabstop
         Style += WS_TABSTOP
      endif

      if lDialogInMemory         //Dialog Template

//          {{'ID',k/hwnd,class,Style,ExStyle,x,y,w,h,caption,HelpId,tooltip,font,size, bold, italic, underline, strikeout}}  --->_HMG_aDialogItems
         blInit:= {|x,y,z| InitDialogMaskedTextBox(x,y,z) }
         aadd(_HMG_aDialogItems,{nId,k,"EDIT",style,0,x,y,w,h,Value,HelpId,tooltip,FontName,FontSize,bold,italic,underline,strikeout,blInit,_HMG_BeginTabActive,.f.,_HMG_ActiveTabPage })
/* JP 62
         if _HMG_aDialogTemplate[3]   // Modal
            _HMG_aControlDeleted [k] := .T.
            Return Nil
         endif
*/
      else

         ControlHandle := GetDialogItemHandle( ParentFormHandle, nId )

         x := GetWindowCol ( Controlhandle )
         y := GetWindowRow ( Controlhandle )
         w := GetWindowWidth  ( Controlhandle )
         h := GetWindowHeight ( Controlhandle )


         SetWindowStyle ( ControlHandle, Style )

      endif
   else
      ParentFormHandle = GetFormHandle (ParentFormName)
      ControlHandle := InitCharMaskTextBox ( ParentFormHandle, 0, x, y, w , '' , 0 , 255 , .f. , .f. , h , rightalign , readonly , invisible , notabstop )
   endif

   if .Not. lDialogInMemory

      If FontHandle != 0
         _SetFontHandle(ControlHandle,FontHandle)
      Else
         if valtype(fontname) == "U"
            FontName := _HMG_DefaultFontName
         endif
         if valtype(fontsize) == "U"
            FontSize := _HMG_DefaultFontSize
         endif
         FontHandle := _SetFont (ControlHandle,FontName,FontSize,bold,italic,underline,strikeout)
      EndIf

      If _HMG_BeginTabActive = .T.
         aAdd ( _HMG_ActiveTabCurrentPageMap , ControlHandle )
      EndIf

      if valtype(tooltip) != "U"
         SetToolTip ( ControlHandle , tooltip , GetFormToolTipHandle (ParentFormName) )
      endif

   endif

   Public &mVar. := k

   _HMG_aControlType [k] := "CHARMASKTEXT"
   _HMG_aControlNames [k] := ControlName
   _HMG_aControlHandles [k] := ControlHandle
   _HMG_aControlParenthandles [k] := ParentFormHandle
   _HMG_aControlIds [k] := nId
   _HMG_aControlProcedures [k] := ""
   _HMG_aControlPageMap [k] := Field
   _HMG_aControlValue [k] := Value
   _HMG_aControlInputMask [k] := InputMask
   _HMG_aControllostFocusProcedure [k] := lostfocus
   _HMG_aControlGotFocusProcedure [k] := gotfocus
   _HMG_aControlChangeProcedure [k] := Change
   _HMG_aControlDeleted [k] := .F.
   _HMG_aControlBkColor [k] := backcolor
   _HMG_aControlFontColor [k] := fontcolor
   _HMG_aControlDblClick [k] := enter
   _HMG_aControlHeadClick  [k] := date
   _HMG_aControlRow [k] := y
   _HMG_aControlCol [k] := x
   _HMG_aControlWidth [k] := w
   _HMG_aControlHeight [k] := h
   _HMG_aControlSpacing [k] := 0
   _HMG_aControlContainerRow  [k] :=  iif ( _HMG_FrameLevel > 0 ,_HMG_ActiveFrameRow [_HMG_FrameLevel] , -1 )
   _HMG_aControlContainerCol  [k] :=  iif ( _HMG_FrameLevel > 0 ,_HMG_ActiveFrameCol [_HMG_FrameLevel] , -1 )
   _HMG_aControlPicture [k] := ""
   _HMG_aControlContainerHandle [k] := 0
   _HMG_aControlFontName [k] := fontname
   _HMG_aControlFontSize [k] := fontsize
   _HMG_aControlFontAttributes [k] := {bold,italic,underline,strikeout}
   _HMG_aControlToolTip  [k] := tooltip
   _HMG_aControlRangeMin [k] :=  0
   _HMG_aControlRangeMax [k] :=  0
   _HMG_aControlCaption [k] :=  ''
   _HMG_aControlVisible [k] := .Not.  invisible
   _HMG_aControlHelpId  [k] := HelpId
   _HMG_aControlFontHandle [k] :=  FontHandle
   _HMG_aControlBrushHandle  [k] := 0
   _HMG_aControlEnabled [k] :=  .T.
   _HMG_aControlMiscData1 [k] := {0 , readonly }

   if .Not. lDialogInMemory

      if date == .F.
         SetWindowText ( ControlHandle , Value  )
      Else
         SetWindowText ( ControlHandle , dtoc ( Value ) )
      endif

      if valtype ( Field ) != 'U'
         aAdd ( _HMG_aFormBrowseList [ GetFormIndex ( ParentFormName ) ] , k )
      EndIf
   endif

Return Nil

*-----------------------------------------------------------------------------*
PROCEDURE ProcessCharMask ( i , d )
*-----------------------------------------------------------------------------*
   Local InBuffer , OutBuffer := '' , icp , x , CB , CM , BadEntry := .F. , InBufferLeft , InBufferRight , Mask , OldChar , BackInbuffer
   Local pc := 0
   Local fnb := 0
   Local dc := 0
   Local pFlag := .F.
   Local ncp
   Local NegativeZero := .F.
   Local Output
   Local ol := 0

   HB_SYMBOL_UNUSED( d )

   If ValType (_HMG_aControlSpacing [i] ) == 'L'
      If _HMG_aControlSpacing [i] == .F.
         Return
      EndIf
   EndIf

   Mask := _HMG_aControlInputMask [i]

   // Store Initial CaretPos

   icp := HiWord ( SendMessage( _HMG_aControlhandles [i] , EM_GETSEL , 0 , 0 ) )

   // Get Current Content

   InBuffer := GetWindowText ( _HMG_aControlHandles [i] )

   // RL 104

   If Left ( AllTrim(InBuffer) , 1 ) == '-' .And. Val(InBuffer) == 0
      NegativeZero := .T.
   EndIf

   If Pcount() > 1

      // Point Count For Numeric InputMask

      For x := 1 To Len ( InBuffer )
         CB := SubStr (InBuffer , x , 1 )
         If CB == '.' .or. CB == ','
            pc++
         EndIf
      Next x

      // RL 89
      If left (InBuffer,1) == '.' .or. left (InBuffer,1) == ','
         pFlag := .T.
      EndIf
      //

      // Find First Non-Blank Position

      For x := 1 To Len ( InBuffer )
         CB := SubStr (InBuffer , x , 1 )
         If CB != ' '
            fnb := x
            Exit
         EndIf
      Next x

   EndIf

   BackInBuffer := InBuffer

   OldChar := SubStr ( InBuffer , icp+1 , 1 )

   If Len ( InBuffer ) < Len ( Mask )

      InBufferLeft := Left ( InBuffer , icp )

      InBufferRight := Right ( InBuffer , Len (InBuffer) - icp )

      if CharMaskTekstOK(InBufferLeft + ' ' + InBufferRight,Mask) .and. CharMaskTekstOK(InBufferLeft + InBufferRight,Mask)==.f.
         InBuffer := InBufferLeft + ' ' + InBufferRight
      else
         InBuffer := InBufferLeft +InBufferRight
      endif

   EndIf

   If Len ( InBuffer ) > Len ( Mask )

      InBufferLeft := Left ( InBuffer , icp )

      InBufferRight := Right ( InBuffer , Len (InBuffer) - icp - 1 )

      InBuffer := InBufferLeft + InBufferRight

   EndIf

   // Process Mask

   For x := 1 To Len (Mask)

      CB := SubStr (InBuffer , x , 1 )
      CM := SubStr (Mask , x , 1 )

      Do Case

      Case (CM) == 'A' .or. (CM) == '!'

         If IsAlpha ( CB ) .Or. CB == ' '

            if (CM)=="!"
               OutBuffer := OutBuffer + Upper(CB)
            else
               OutBuffer := OutBuffer + CB
            endif

         Else

            if x == icp
               BadEntry := .T.
               OutBuffer := OutBuffer + OldChar
            else
               OutBuffer := OutBuffer + ' '
            endif

         EndIf

      Case CM == '9'

         If IsDigit ( CB ) .Or. CB == ' ' .Or. ( CB == '-' .And. x == fnb .And. Pcount() > 1 )

            OutBuffer := OutBuffer + CB

         Else

            if x == icp
               BadEntry := .T.
               OutBuffer := OutBuffer + OldChar
            Else
               OutBuffer := OutBuffer + ' '
            EndIf

         EndIf

      Case CM == ' '

         If CB == ' '

            OutBuffer := OutBuffer + CB

         Else

            if x == icp
               BadEntry := .T.
               OutBuffer := OutBuffer + OldChar
            Else
               OutBuffer := OutBuffer + ' '
            EndIf

         EndIf

      OtherWise

         OutBuffer := OutBuffer + CM

         End Case

      Next x

   // Replace Content

      If ! ( BackInBuffer == OutBuffer )
         SetWindowText ( _HMG_aControlHandles [i] , OutBuffer )
      EndIf

      If pc > 1

      // RL 104

         If NegativeZero == .T.

            Output := Transform ( GetNumFromText ( GetWindowText ( _HMG_aControlhandles [i] ) , i ) , Mask )

            Output := Right (Output , ol - 1 )

            Output := '-' + Output

         // Replace Text

            SetWindowText ( _HMG_aControlhandles [i] , Output )
            SendMessage( _HMG_aControlhandles [i] , EM_SETSEL , at('.',OutBuffer) + dc , at('.',OutBuffer) + dc )

         Else

            SetWindowText ( _HMG_aControlhandles [i] , Transform ( GetNumFromText ( GetWindowText ( _HMG_aControlhandles [i] ) , i ) , Mask ) )
            SendMessage( _HMG_aControlhandles [i] , EM_SETSEL , at('.',OutBuffer) + dc , at('.',OutBuffer) + dc )

         EndIf

      Else

         If pFlag == .T.

            ncp := at ( '.' , GetWindowText ( _HMG_aControlHandles [i] ) )
            SendMessage( _HMG_aControlhandles [i] , EM_SETSEL , ncp , ncp )

         Else

         // Restore Initial CaretPos

            If BadEntry
               icp--
            EndIf

            SendMessage( _HMG_aControlhandles [i] , EM_SETSEL , icp , icp )

         // Skip Protected Characters

            For x := 1 To Len (OutBuffer)

               CB := SubStr ( OutBuffer , icp+x , 1 )
               CM := SubStr ( Mask , icp+x , 1 )

               If ( .Not. IsDigit(CB) ) .And. ( .Not. IsAlpha(CB) ) .And. ( ( .Not. CB = ' ' ) .or. ( CB == ' ' .and. CM == ' ' ) )
                  SendMessage( _HMG_aControlhandles [i] , EM_SETSEL , icp+x , icp+x )
               Else
                  Exit
               EndIf

            Next x

         EndIf

      EndIf

RETURN

*-----------------------------------------------------------------------------*
Function CharMaskTekstOK( cString, cMask )
*-----------------------------------------------------------------------------*
   Local lPassed := .f., CB, CM, x

   For x := 1 To Min( Len(cString), Len(cMask) )
      CB := SubStr( cString , x , 1 )
      CM := SubStr( cMask , x , 1 )
      Do Case
      // JK
      Case CM == 'A' .or. CM == '!'
         lPassed := ( IsAlpha ( CB ) .Or. CB == ' ' )
      Case CM == '9'
         lPassed := ( IsDigit ( CB ) .Or. CB == ' ' )
      Case CM == ' '
         lPassed := ( CB == ' ' )
      OtherWise
         lPassed := .t.
      EndCase
      If lPassed == .f.
         Exit
      EndIf
   Next

Return lPassed

*-----------------------------------------------------------------------------*
Procedure _DataTextBoxRefresh (i)
*-----------------------------------------------------------------------------*
   Local Field

   If _HMG_aControlType [i] == "MASKEDTEXT"
      Field      := _HMG_aControlHeadClick [i]
   Else
      Field      := _HMG_aControlPageMap [i]
   EndIf

   If Type ( Field ) == 'C'
      _SetValue ( '' , '' , rtrim( &(Field)) , i )
   Else
      _SetValue ( '' , '' , &(Field) , i )
   EndIf

Return

*-----------------------------------------------------------------------------*
Procedure _DataTextBoxSave ( ControlName, ParentForm )
*-----------------------------------------------------------------------------*
   Local Field , i

   i := GetControlIndex ( ControlName , ParentForm)

   If _HMG_aControlType [i] == "MASKEDTEXT"
      Field      := _HMG_aControlHeadClick [i]
   Else
      Field      := _HMG_aControlPageMap [i]
   EndIf

   &(Field) := _GetValue ( Controlname , ParentForm )

Return

*-----------------------------------------------------------------------------*
PROCEDURE ProcessNumText ( i )
*-----------------------------------------------------------------------------*
   Local InBuffer , OutBuffer := '' , icp , x , CB , BackInBuffer , BadEntry := .F. , fnb

   // Store Initial CaretPos
   icp := HiWord ( SendMessage( _HMG_aControlhandles [i] , EM_GETSEL , 0 , 0 ) )

   // Get Current Content
   InBuffer := GetWindowText ( _HMG_aControlHandles [i] )

   BackInBuffer := InBuffer

   // Find First Non-Blank Position

   For x := 1 To Len ( InBuffer )
      CB := SubStr (InBuffer , x , 1 )
      If CB != ' '
         fnb := x
         Exit
      EndIf
   Next x

   // Process Mask

   For x := 1 To Len(InBuffer)

      CB := SubStr(InBuffer , x , 1 )

      If IsDigit ( CB ) .Or. ( CB == '-' .And. x == fnb )
         OutBuffer := OutBuffer + CB
      Else
         BadEntry  := .t.
      EndIf

   Next x

   If BadEntry
      icp--
   EndIf

   // JK Replace Content

   If ! ( BackInBuffer == OutBuffer )
      SetWindowText ( _HMG_aControlHandles [i] , OutBuffer )
   EndIf

   // Restore Initial CaretPos

   SendMessage( _HMG_aControlhandles [i] , EM_SETSEL , icp , icp )

RETURN

*-----------------------------------------------------------------------------*
Function GETNumFromTextSP( Text, i )
*-----------------------------------------------------------------------------*
   Local x , c , s

   s := ''

   For x := 1 To Len ( Text )

      c := SubStr(Text,x,1)

      If c='0' .or. c='1' .or. c='2' .or. c='3' .or. c='4' .or. c='5' .or. c='6' .or. c='7' .or. c='8' .or. c='9' .or. c=',' .or. c='-' .or. c = '.'

         if c == '.'
            c :=''
         endif

         IF C == ','
            C:= '.'
         ENDIF

         s := s + c

      EndIf

   Next x

   If Left ( AllTrim(Text) , 1 ) == '(' .OR.  Right ( AllTrim(Text) , 2 ) == 'DB'
      s := '-' + s
   EndIf

   s := Transform ( Val(s) , _HMG_aControlInputMask [i] )

Return Val(s)

*-----------------------------------------------------------------------------*
Function OEDITEVENTS( hWnd, nMsg, wParam, lParam )
*-----------------------------------------------------------------------------*
Local i := ascan ( _HMG_aControlHandles , hWnd ), icp, icpe, inbuffer
Local ParentForm

SWITCH nMsg

CASE WM_CHAR

   icp :=  HiWord ( SendMessage( _HMG_aControlhandles [ i ] , EM_GETSEL , 0 , 0 ) )
   icpe := LoWord ( SendMessage( _HMG_aControlhandles [ i ] , EM_GETSEL , 0 , 0 ) )
   InBuffer := GetWindowText ( _HMG_aControlHandles [ i ] )

      // simulate overwrite mode

   if ! IsInsertActive() .and. wParam <> 13 .and. wParam <> 8 .and. substr( inBuffer, icp + 1, 1 ) <> chr( 13 )

      if IsAlpha( chr( wParam ) ) .or. IsDigit( chr( wParam ) )
         if icp <> icpe
            SendMessage( _HMG_aControlhandles [ i ] , WM_CLEAR , 0 , 0 )
            SendMessage( _HMG_aControlhandles [ i ] , EM_SETSEL , icpe , icpe )
         else
            SendMessage( _HMG_aControlhandles [ i ] , EM_SETSEL , icp , icp + 1 )
            SendMessage( _HMG_aControlhandles [ i ] , WM_CLEAR , 0 , 0 )
            SendMessage( _HMG_aControlhandles [ i ] , EM_SETSEL , icp , icp )
         endif

      else

         if wParam == 1
            SendMessage( _HMG_aControlhandles [ i ] , EM_SETSEL , 0 , -1 )
         endif

      endif
   else
      if wParam == 1
         SendMessage( _HMG_aControlhandles [ i ] , EM_SETSEL , 0 , -1 )
      endif
   endif

CASE WM_CONTEXTMENU

   ParentForm := _HMG_aControlParentHandles[i]
   i := Ascan( _HMG_aControlsContextMenu , {|x| x[1] == hWnd} )
   IF i > 0
      IF _HMG_aControlsContextMenu[i, 4] == .T.
         setfocus(wParam)
         _HMG_xControlsContextMenuID := _HMG_aControlsContextMenu[i, 3]
         TrackPopupMenu ( _HMG_aControlsContextMenu[i, 2] , LOWORD(lparam) , HIWORD(lparam) , ParentForm )
         RETURN 1
      ENDIF
   ENDIF
END

Return 0
