/*----------------------------------------------------------------------------
 MINIGUI - Harbour Win32 GUI library source code

 Copyright 2002-2008 Roberto Lopez <harbourminigui@gmail.com>
 http://harbourminigui.googlepages.com/

 This program is free software; you can redistribute it and/or modify it under 
 the terms of the GNU General Public License as published by the Free Software 
 Foundation; either version 2 of the License, or (at your option) any later 
 version. 

 This program is distributed in the hope that it will be useful, but WITHOUT 
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS 
 FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

 You should have received a copy of the GNU General Public License along with 
 this software; see the file COPYING. If not, write to the Free Software 
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA (or 
 visit the web site http://www.gnu.org/).

 As a special exception, you have permission for additional uses of the text 
 contained in this release of Harbour Minigui.

 The exception is that, if you link the Harbour Minigui library with other 
 files to produce an executable, this does not by itself cause the resulting 
 executable to be covered by the GNU General Public License.
 Your use of that executable is in no way restricted on account of linking the 
 Harbour-Minigui library code into it.

 Parts of this project are based upon:

	"Harbour GUI framework for Win32"
 	Copyright 2001 Alexander S.Kresin <alex@belacy.belgorod.su>
 	Copyright 2001 Antonio Linares <alinares@fivetech.com>
	www - http://www.harbour-project.org

	"Harbour Project"
	Copyright 1999-2008, http://www.harbour-project.org/

	"WHAT32"
	Copyright 2002 AJ Wos <andrwos@aust1.net> 

	"HWGUI"
  	Copyright 2001-2007 Alexander S.Kresin <alex@belacy.belgorod.su>

---------------------------------------------------------------------------*/
/*
 * Author: P.Chornyj <myorg63@mail.ru>
*/

#include "minigui.ch"

Function SetWndBrush( Window, nodelete, style, hatch, color, bitmap )
Local i := GetFormIndex ( Window )
Local hWnd := _HMG_aFormHandles [i] 
Local oldBrush, Brush
Local nResult := 0

DEFAULT hWnd TO _HMG_MainHandle
DEFAULT nodelete TO FALSE
DEFAULT style TO BS_SOLID
DEFAULT hatch TO HS_VERTICAL
DEFAULT color TO {255,0,255} 
DEFAULT bitmap TO "MINIGUI_EDIT_DELETE" 

IF ( IsWindowHandler( hWnd ) <> 0 )

    SWITCH style
	CASE BS_SOLID
	        Brush := CreateSolidBrush( color[1], color[2], color[3] )
        	EXIT

        CASE BS_NULL
		Brush := GetWindowBrush( hWnd )            
        	EXIT

	CASE BS_HATCHED
	        Brush := CreateHatchBrush( hatch, RGB( color[1], color[2], color[3] ) )
		EXIT

        CASE BS_PATTERN
	        Brush := CreatePatternBrush( bitmap )
    END

    oldBrush := SetWindowBrush( hWnd, Brush )

    IF nodelete == .F.
	DeleteObject( oldBrush )
    ENDIF

    nResult := iif( nodelete, oldBrush, 0 )

ENDIF

Return nResult 

/*
 * C-level
*/
#pragma BEGINDUMP

#include <windows.h>
#include <winuser.h>
#include <wingdi.h>
#include "hbapi.h"
#include "hbvm.h"
#include "hbstack.h"
#include "hbapiitm.h"

HB_FUNC ( ISWINDOWHANDLER )
{
  hb_retnl( IsWindow( (HWND) hb_parnl(1) ) );
}

HB_FUNC ( GETWINDOWBRUSH )
{
  hb_retnl( GetClassLong( (HWND) hb_parnl(1), GCL_HBRBACKGROUND ) );
}

HB_FUNC ( SETWINDOWBRUSH )
{
  hb_retnl( SetClassLong( (HWND) hb_parnl(1), GCL_HBRBACKGROUND, hb_parnl(2) ) );
}

HB_FUNC ( CREATEHATCHBRUSH )
{
  hb_retnl( (LONG) CreateHatchBrush( hb_parnl(1), (COLORREF) hb_parnl(2) ) );
}

/*
	Windows 95: Creating brushes from bitmaps or DIBs larger than 8x8 pixels is not supported. 
	If a larger bitmap is specified, only a portion of the bitmap is used.
*/
HB_FUNC ( CREATEPATTERNBRUSH )
{
  HBITMAP himage;

  himage = LoadImage( 0, hb_parc(1), IMAGE_BITMAP , 0, 0, LR_LOADFROMFILE | LR_LOADMAP3DCOLORS | LR_LOADTRANSPARENT );
  if (himage == NULL) 
  {
    himage = LoadImage( GetModuleHandle(NULL), hb_parc(1), IMAGE_BITMAP , 0, 0, LR_LOADMAP3DCOLORS | LR_LOADTRANSPARENT );
  }

  hb_retnl( (LONG) CreatePatternBrush( himage ) );
}

#pragma ENDDUMP
