/*----------------------------------------------------------------------------
MINIGUI - Harbour Win32 GUI library source code

Copyright 2002-2008 Roberto Lopez <harbourminigui@gmail.com>
http://harbourminigui.googlepages.com/

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along with
   this software; see the file COPYING. If not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA (or
   visit the web site http://www.gnu.org/).

   As a special exception, you have permission for additional uses of the text
   contained in this release of Harbour Minigui.

   The exception is that, if you link the Harbour Minigui library with other
   files to produce an executable, this does not by itself cause the resulting
   executable to be covered by the GNU General Public License.
   Your use of that executable is in no way restricted on account of linking the
   Harbour-Minigui library code into it.

   Parts of this project are based upon:

   "Harbour GUI framework for Win32"
   Copyright 2001 Alexander S.Kresin <alex@belacy.belgorod.su>
   Copyright 2001 Antonio Linares <alinares@fivetech.com>
   www - http://www.harbour-project.org

   "Harbour Project"
   Copyright 1999-2008, http://www.harbour-project.org/

   "WHAT32"
   Copyright 2002 AJ Wos <andrwos@aust1.net>

   "HWGUI"
   Copyright 2001-2007 Alexander S.Kresin <alex@belacy.belgorod.su>

---------------------------------------------------------------------------*/

#define SB_HORZ    0
#define SB_VERT    1

#define WM_PAINT   15
#define WM_CLOSE   0x0010

#define WS_CHILD   0x40000000
#define GWL_STYLE  (-16)

#include "minigui.ch"

*-----------------------------------------------------------------------------*
Function _DefineWindow ( FormName, Caption, x, y, w, h, nominimize, nomaximize, nosize, nosysmenu, nocaption, aMin, aMax,;
   InitProcedure, ReleaseProcedure, MouseDragProcedure, SizeProcedure, ClickProcedure, MouseMoveProcedure, aRGB, PaintProcedure,;
   noshow, topmost, main, icon, child, fontname, fontsize, NotifyIconName, NotifyIconTooltip, NotifyIconLeftClick,;
   GotFocus, LostFocus, VirtualHeight, VirtualWidth, scrollleft, scrollright, scrollup, scrolldown, hscrollbox, vscrollbox, helpbutton,;
   MaximizeProcedure, MinimizeProcedure, cursor, NoAutoRelease, InteractiveCloseProcedure, RestoreProcedure, MoveProcedure, DropProcedure, mdi, palette, NotifyIconDblClick )
*-----------------------------------------------------------------------------*
   Local i , htooltip , mVar , vscroll , hscroll , BrushHandle , k , FormHandle, ParentHandle

   if valtype(FormName) == "U"
      FormName := _HMG_TempWindowName
   endif

   if valtype(mdi) == "U"
      Mdi := .F.
   endif

   FormName := Alltrim(FormName)

   if Main

      i := ascan ( _HMG_aFormType , 'A' )

      if i > 0
         MsgMiniGuiError("Main Window is already defined. Program terminated" )
      Endif

      if Child == .T.
         MsgMiniGuiError("Child and Main clauses cannot be used simultaneously. Program terminated" )
      Endif

      if NoAutoRelease == .T.
         MsgMiniGuiError("NOAUTORELEASE and MAIN clauses cannot be used simultaneously. Program terminated" )
      Endif

   Else

      i := ascan ( _HMG_aFormType , 'A' )

      if i <= 0
         MsgMiniGuiError("Main Window is not defined. Program terminated" )
      Endif

      If _IsWindowDefined (FormName)
         MsgMiniGuiError("Window: "+ FormName + " is already defined. Program terminated" )
      EndIf

      If .Not. Empty (NotifyIconName)
         MsgMiniGuiError("Notification icon allowed only in Main Window. Program terminated" )
      EndIf

      If _HMG_BeginWindowMDIActive == .T.
         MsgMiniGuiError("Inside MdiWindow can be defined only MdiChild windows. Program terminated" )
      EndIf

   EndIf

   mVar := '_' + FormName

   if child == .T.
      ParentHandle := _HMG_MainHandle
   Else
      ParentHandle := 0
   endif

   if valtype(FontName) == "U"
      _HMG_ActiveFontName := ""
   Else
      _HMG_ActiveFontName := FontName
   Endif

   if valtype(FontSize) == "U"
      _HMG_ActiveFontSize := 0
   Else
      _HMG_ActiveFontSize := FontSize
   Endif

   if valtype(Caption) == "U"
      Caption := ""
   endif

   if valtype(scrollup) == "U"
      scrollup := ""
   endif
   if valtype(scrolldown) == "U"
      scrolldown := ""
   endif
   if valtype(scrollleft) == "U"
      scrollleft := ""
   endif
   if valtype(scrollright) == "U"
      scrollright := ""
   endif

   if valtype(hscrollbox) == "U"
      hscrollbox := ""
   endif
   if valtype(vscrollbox) == "U"
      vscrollbox := ""
   endif

   if valtype(InitProcedure) == "U"
      InitProcedure := ""
   endif

   if valtype(ReleaseProcedure) == "U"
      ReleaseProcedure := ""
   endif

   if valtype(MouseDragProcedure) == "U"
      MouseDragProcedure := ""
   endif

   if valtype(SizeProcedure) == "U"
      SizeProcedure := ""
   endif

   if valtype(ClickProcedure) == "U"
      ClickProcedure := ""
   endif

   if valtype(MouseMoveProcedure) == "U"
      MouseMoveProcedure := ""
   endif

   if valtype(PaintProcedure) == "U"
      PaintProcedure := ""
   endif

   if valtype(GotFocus) == "U"
      GotFocus := ""
   endif

   if valtype(LostFocus) == "U"
      LostFocus := ""
   endif

   if valtype(VirtualHeight) == "U"
      VirtualHeight := 0
      vscroll       := .f.
   Else
      If VirtualHeight <= h
         MsgMiniGuiError( "DEFINE WINDOW: Virtual Height must be greater than Window Height. Program terminated" )
      EndIf
      vscroll       := .t.
   endif

   if valtype(VirtualWidth) == "U"
      VirtualWidth  := 0
      hscroll       := .f.
   Else
      If VirtualWidth <= w
         MsgMiniGuiError( "DEFINE WINDOW: Virtual Width must be greater than Window Width. Program terminated" )
      EndIf
      hscroll       := .t.
   endif

   if Valtype ( aRGB ) == 'U'
      aRGB := { -1 , -1 , -1 }
   EndIf

   _HMG_ActiveFormName := FormName
   If mdi
      _HMG_MainClientMDIName := FormName   // JP MDI
   EndIf

   _HMG_BeginWindowActive := .T.

   UnRegisterWindow (FormName)

   // JP MDI
   if mdi
      _HMG_BeginWindowMDIActive := .T.
      BrushHandle := RegisterMDIWindow(icon,FormName, aRGB )
      Formhandle = InitMDIWindow( Caption , x, y, w, h, nominimize, nomaximize, nosize, nosysmenu, nocaption , topmost , FormName , ParentHandle , vscroll , hscroll , helpbutton )
   else
      BrushHandle := RegisterWindow(icon,FormName, aRGB )
      Formhandle = InitWindow( Caption , x, y, w, h, nominimize, nomaximize, nosize, nosysmenu, nocaption , topmost , FormName , ParentHandle , vscroll , hscroll , helpbutton , palette )
   endif

   if Valtype ( cursor ) != "U"
      SetWindowCursor( Formhandle , cursor )
   EndIf

   if Main
      _HMG_MainHandle := Formhandle
   EndIf

   if valtype(NotifyIconName) == "U"
      NotifyIconName := ""
   Else
      ShowNotifyIcon( FormHandle , .T. , LoadTrayIcon( GetInstance(), NotifyIconName ), NotifyIconTooltip )
   endif

   htooltip := InitToolTip( FormHandle , _SetToolTipBalloon() , _lSetToolTip() )

   k := ascan ( _HMG_aFormDeleted , .T. )

   if k > 0

      _HMG_MainIndex := k

      Public &mVar. := k

      _HMG_aFormNames [k] := FormName
      _HMG_aFormHandles  [k] :=  FormHandle
      _HMG_aFormActive  [k] :=  .f.
      _HMG_aFormType  [k] :=  iif ( Main == .t. , 'A' , iif( Child == .t., 'C','S' ) )
      _HMG_aFormParentHandle  [k] :=  0
      _HMG_aFormReleaseProcedure  [k] :=  ReleaseProcedure
      _HMG_aFormInitProcedure  [k] :=  InitProcedure
      _HMG_aFormToolTipHandle  [k] :=  htooltip
      _HMG_aFormContextMenuHandle  [k] :=  0
      _HMG_aFormMouseDragProcedure  [k] :=  MouseDragProcedure
      _HMG_aFormSizeProcedure  [k] :=  SizeProcedure
      _HMG_aFormClickProcedure  [k] :=  ClickProcedure
      _HMG_aFormMouseMoveProcedure  [k] :=  MouseMoveProcedure
      _HMG_aFormMoveProcedure  [k] :=  MoveProcedure
      _HMG_aFormDropProcedure  [k] :=  DropProcedure
      _HMG_aFormDeleted  [k] := .f.
      _HMG_aFormBkColor  [k] := aRGB
      _HMG_aFormPaintProcedure [k] :=   PaintProcedure
      _HMG_aFormNoShow  [k] :=  noshow
      _HMG_aFormNotifyIconName  [k] :=  NotifyIconName
      _HMG_aFormNotifyIconToolTip [k] :=     NotifyIconToolTip
      _HMG_aFormNotifyIconLeftClick    [k] :=  NotifyIconLeftClick
      _HMG_aFormNotifyIconDblClick    [k] :=  NotifyIconDblClick
      _HMG_aFormGotFocusProcedure  [k] := GotFocus
      _HMG_aFormLostFocusProcedure [k] :=     LostFocus
      _HMG_aFormReBarHandle    [k] :=  0
      _HMG_aFormNotifyMenuHandle  [k] := 0
      _HMG_aFormBrowseList    [k] :=  {}
      _HMG_aFormSplitChildList  [k] := {}
      _HMG_aFormVirtualHeight    [k] :=  VirtualHeight
      _HMG_aFormVirtualWidth    [k] :=  VirtualWidth
      _HMG_aFormFocused    [k] :=  .f.
      _HMG_aFormScrollUp    [k] :=  ScrollUp
      _HMG_aFormScrollDown    [k] :=  ScrollDown
      _HMG_aFormScrollLeft    [k] :=  ScrollLeft
      _HMG_aFormScrollRight    [k] :=  ScrollRight
      _HMG_aFormHScrollBox     [k] := HScrollBox
      _HMG_aFormVScrollBox     [k] := VScrollBox
      _HMG_aFormBrushHandle     [k] := BrushHandle
      _HMG_aFormFocusedControl  [k] := 0
      _HMG_aFormGraphTasks     [k] := {}
      _HMG_aFormMaximizeProcedure [k] :=  MaximizeProcedure
      _HMG_aFormMinimizeProcedure [k] :=  MinimizeProcedure
      _HMG_aFormRestoreProcedure [k] :=  RestoreProcedure
      _HMG_aFormAutoRelease [k] :=  .Not. NoAutoRelease
      _HMG_aFormInteractiveCloseProcedure [k] :=  InteractiveCloseProcedure
      _HMG_aFormMinMaxInfo [k] := InitMinMaxInfo ( FormHandle )
      _HMG_aFormActivateId [k] := 0

   Else

      _HMG_MainIndex := Len(_HMG_aFormNames) + 1

      Public &mVar. := Len(_HMG_aFormNames) + 1

      aAdd ( _HMG_aFormNames , FormName )
      aAdd ( _HMG_aFormHandles , FormHandle )
      aAdd ( _HMG_aFormActive , .f. )
      aAdd ( _HMG_aFormType , iif ( Main == .t. , 'A' , iif( Child == .t., 'C','S' ) ) )
      aAdd ( _HMG_aFormParentHandle , 0 )
      aAdd ( _HMG_aFormReleaseProcedure , ReleaseProcedure )
      aAdd ( _HMG_aFormInitProcedure , InitProcedure )
      aAdd ( _HMG_aFormToolTipHandle , htooltip )
      aAdd ( _HMG_aFormContextMenuHandle , 0 )
      aAdd ( _HMG_aFormMouseDragProcedure , MouseDragProcedure )
      aAdd ( _HMG_aFormSizeProcedure , SizeProcedure )
      aAdd ( _HMG_aFormClickProcedure , ClickProcedure )
      aAdd ( _HMG_aFormMouseMoveProcedure , MouseMoveProcedure )
      aAdd ( _HMG_aFormMoveProcedure , MoveProcedure )
      aAdd ( _HMG_aFormDropProcedure , DropProcedure )
      aAdd ( _HMG_aFormDeleted, .f. )
      aAdd ( _HMG_aFormBkColor, aRGB )
      aAdd ( _HMG_aFormPaintProcedure , PaintProcedure )
      aAdd ( _HMG_aFormNoShow , noshow )
      aAdd ( _HMG_aFormNotifyIconName       , NotifyIconName    )
      aAdd ( _HMG_aFormNotifyIconToolTip      , NotifyIconToolTip    )
      aAdd ( _HMG_aFormNotifyIconLeftClick   , NotifyIconLeftClick    )
      aAdd ( _HMG_aFormNotifyIconDblClick   , NotifyIconDblClick    )
      aAdd ( _HMG_aFormGotFocusProcedure      , GotFocus )
      aAdd ( _HMG_aFormLostFocusProcedure      , LostFocus )
      aAdd ( _HMG_aFormReBarHandle      , 0 )
      aAdd ( _HMG_aFormNotifyMenuHandle      , 0 )
      aAdd ( _HMG_aFormBrowseList         , {} )
      aAdd ( _HMG_aFormSplitChildList      , {} )
      aAdd ( _HMG_aFormVirtualHeight      , VirtualHeight )
      aAdd ( _HMG_aFormVirtualWidth      , VirtualWidth )
      aAdd ( _HMG_aFormFocused         , .f. )
      aAdd ( _HMG_aFormScrollUp         , ScrollUp )
      aAdd ( _HMG_aFormScrollDown         , ScrollDown )
      aAdd ( _HMG_aFormScrollLeft         , ScrollLeft )
      aAdd ( _HMG_aFormScrollRight      , ScrollRight )
      aAdd ( _HMG_aFormHScrollBox         , HScrollBox )
      aAdd ( _HMG_aFormVScrollBox         , VScrollBox )
      aAdd ( _HMG_aFormBrushHandle      , BrushHandle )
      aAdd ( _HMG_aFormFocusedControl      , 0 )
      aAdd ( _HMG_aFormGraphTasks         , {} )
      aAdd ( _HMG_aFormMaximizeProcedure      , MaximizeProcedure )
      aAdd ( _HMG_aFormMinimizeProcedure      , MinimizeProcedure )
      aAdd ( _HMG_aFormRestoreProcedure      , RestoreProcedure )
      aAdd ( _HMG_aFormAutoRelease      , .Not. NoAutoRelease )
      aAdd ( _HMG_aFormInteractiveCloseProcedure , InteractiveCloseProcedure )
      aAdd ( _HMG_aFormMinMaxInfo , InitMinMaxInfo ( FormHandle ) )
      aAdd ( _HMG_aFormActivateId , 0 )

   EndIf

   // JP MDI
   if !mdi
      InitDummy(FormHandle)
   endif

   If VirtualHeight > 0
      SetScrollRange ( Formhandle , SB_VERT , 0 , VirtualHeight - h , 1 )
   EndIf
   If VirtualWidth > 0
      SetScrollRange ( Formhandle , SB_HORZ , 0 , VirtualWidth - w , 1 )
   EndIf

   If ISARRAY ( aMin ) .And. ISARRAY ( aMax )
      IF( aMin[1]==NIL, NIL, _HMG_aFormMinMaxInfo [_HMG_MainIndex] [5] := aMin[1] )
      IF( aMin[2]==NIL, NIL, _HMG_aFormMinMaxInfo [_HMG_MainIndex] [6] := aMin[2] )
      IF( aMax[1]==NIL, NIL, _HMG_aFormMinMaxInfo [_HMG_MainIndex] [7] := aMax[1] )
      IF( aMax[2]==NIL, NIL, _HMG_aFormMinMaxInfo [_HMG_MainIndex] [8] := aMax[2] )
   EndIf

Return (FormHandle)

*-----------------------------------------------------------------------------*
Function _DefineModalWindow ( FormName, Caption, x, y, w, h, Parent, nosize, nosysmenu, nocaption, aMin, aMax,;
   InitProcedure, ReleaseProcedure, MouseDragProcedure, SizeProcedure, ClickProcedure, MouseMoveProcedure, aRGB,;
   PaintProcedure, icon, FontName, FontSize, GotFocus, LostFocus, VirtualHeight, VirtualWidth,;
   scrollleft, scrollright, scrollup, scrolldown, hscrollbox, vscrollbox, helpbutton, cursor, noshow,;
   NoAutoRelease, InteractiveCloseProcedure, MoveProcedure, DropProcedure )
*-----------------------------------------------------------------------------*
   Local i , htooltip , mVar , vscroll , hscroll , BrushHandle , k
   Local FormHandle

   if valtype(FormName) == "U"
      FormName := _HMG_TempWindowName
   endif

   i := ascan ( _HMG_aFormType , 'A' )

   if i <= 0
      MsgMiniGuiError("Main Window is not defined. Program terminated" )
   Endif

   If _IsWindowDefined (FormName)
      MsgMiniGuiError("Window: "+ FormName + " is already defined. Program terminated" )
   endif

   mVar := '_' + FormName

   if valtype(FontName) == "U"
      _HMG_ActiveFontName := ""
   Else
      _HMG_ActiveFontName := FontName
   Endif

   if valtype(FontSize) == "U"
      _HMG_ActiveFontSize := 0
   Else
      _HMG_ActiveFontSize := FontSize
   Endif

   if valtype(Caption) == "U"
      Caption := ""
   endif

   if valtype(InitProcedure) == "U"
      InitProcedure := ""
   endif

   if valtype(PaintProcedure) == "U"
      PaintProcedure := ""
   endif

   if valtype(ReleaseProcedure) == "U"
      ReleaseProcedure := ""
   endif

   if valtype(MouseDragProcedure) == "U"
      MouseDragProcedure := ""
   endif

   if valtype(SizeProcedure) == "U"
      SizeProcedure := ""
   endif

   if valtype(ClickProcedure) == "U"
      ClickProcedure := ""
   endif

   if valtype(MouseMoveProcedure) == "U"
      MouseMoveProcedure := ""
   endif

   if valtype(GotFocus) == "U"
      GotFocus := ""
   endif

   if valtype(LostFocus) == "U"
      LostFocus := ""
   endif

   if valtype(scrollup) == "U"
      scrollup := ""
   endif
   if valtype(scrolldown) == "U"
      scrolldown := ""
   endif
   if valtype(scrollleft) == "U"
      scrollleft := ""
   endif
   if valtype(scrollright) == "U"
      scrollright := ""
   endif

   if valtype(hscrollbox) == "U"
      hscrollbox := ""
   endif
   if valtype(vscrollbox) == "U"
      vscrollbox := ""
   endif

   if valtype(VirtualHeight) == "U"
      VirtualHeight := 0
      vscroll       := .f.
   Else
      If VirtualHeight <= h
         MsgMiniGuiError( "DEFINE WINDOW: Virtual Height must be greater than Window Height. Program terminated" )
      EndIf
      vscroll       := .t.
   endif

   if valtype(VirtualWidth) == "U"
      VirtualWidth  := 0
      hscroll       := .f.
   Else
      If VirtualWidth <= w
         MsgMiniGuiError( "DEFINE WINDOW: Virtual Width must be greater than Window Width. Program terminated" )
      EndIf
      hscroll       := .t.
   endif

   if Valtype ( aRGB ) == 'U'
      aRGB := { -1 , -1 , -1 }
   EndIf

   If _HMG_InplaceParentHandle <> 0
      Parent := _hmg_InplaceParentHandle
   ElseIf .Not. _HMG_BeginWindowMDIActive
      Parent := _hmg_MainHandle
   EndIf

   _HMG_ActiveFormName := FormName
   _HMG_BeginWindowActive := .T.

   UnRegisterWindow (FormName)
   BrushHandle := RegisterWindow( icon, FormName, aRGB )

   Formhandle = InitModalWindow ( Caption, x, y, w, h, Parent, nosize, nosysmenu, nocaption, FormName, vscroll, hscroll, helpbutton )

   if Valtype ( cursor ) != "U"
      SetWindowCursor( Formhandle , cursor )
   EndIf

   htooltip := InitToolTip( , _SetToolTipBalloon() , _lSetToolTip() )

   k := ascan ( _HMG_aFormDeleted , .T. )

   if k > 0

      Public &mVar. := k

      _HMG_aFormNames  [k] := FormName
      _HMG_aFormHandles   [k] :=  FormHandle
      _HMG_aFormActive   [k] :=  .f.
      _HMG_aFormType  [k] :=   "M"
      _HMG_aFormParentHandle   [k] :=  Parent
      _HMG_aFormReleaseProcedure   [k] :=  ReleaseProcedure
      _HMG_aFormInitProcedure   [k] :=  InitProcedure
      _HMG_aFormToolTipHandle   [k] :=  htooltip
      _HMG_aFormContextMenuHandle   [k] := 0
      _HMG_aFormMouseDragProcedure   [k] :=  MouseDragProcedure
      _HMG_aFormSizeProcedure   [k] :=  SizeProcedure
      _HMG_aFormClickProcedure   [k] :=  ClickProcedure
      _HMG_aFormMouseMoveProcedure   [k] :=  MouseMoveProcedure
      _HMG_aFormMoveProcedure   [k] :=  MoveProcedure
      _HMG_aFormDropProcedure   [k] :=  DropProcedure
      _HMG_aFormDeleted   [k] :=  .f.
      _HMG_aFormBkColor  [k] :=  aRGB
      _HMG_aFormPaintProcedure   [k] :=  PaintProcedure
      _HMG_aFormNoShow   [k] :=  noshow
      _HMG_aFormNotifyIconName   [k] :=  ''
      _HMG_aFormNotifyIconToolTip   [k] :=  ''
      _HMG_aFormNotifyIconLeftClick   [k] :=  ''
      _HMG_aFormNotifyIconDblClick   [k] :=  ''
      _HMG_aFormGotFocusProcedure   [k] :=  GotFocus
      _HMG_aFormLostFocusProcedure   [k] :=  LostFocus
      _HMG_aFormReBarHandle   [k] :=  0
      _HMG_aFormNotifyMenuHandle   [k] :=  0
      _HMG_aFormBrowseList     [k] :=  {}
      _HMG_aFormSplitChildList     [k] :=  {}
      _HMG_aFormVirtualHeight     [k] :=  VirtualHeight
      _HMG_aFormVirtualWidth  [k] :=     VirtualWidth
      _HMG_aFormFocused  [k] :=     .f.
      _HMG_aFormScrollUp  [k] :=     ScrollUp
      _HMG_aFormScrollDown     [k] :=  ScrollDown
      _HMG_aFormScrollLeft     [k] :=  ScrollLeft
      _HMG_aFormScrollRight  [k] :=    ScrollRight
      _HMG_aFormHScrollBox  [k] :=     HScrollBox
      _HMG_aFormVScrollBox     [k] :=  VScrollBox
      _HMG_aFormBrushHandle     [k] :=  BrushHandle
      _HMG_aFormFocusedControl   [k] := 0
      _HMG_aFormGraphTasks     [k] :=  {}
      _HMG_aFormMaximizeProcedure  [k] :=     Nil
      _HMG_aFormMinimizeProcedure  [k] :=     Nil
      _HMG_aFormRestoreProcedure  [k] :=     Nil
      _HMG_aFormAutoRelease [k] :=  .Not. NoAutoRelease
      _HMG_aFormInteractiveCloseProcedure [k] :=  InteractiveCloseProcedure
      _HMG_aFormMinMaxInfo [k] := InitMinMaxInfo ( FormHandle )
      _HMG_aFormActivateId [k] := 0

   Else

      Public &mVar. := Len(_HMG_aFormNames) + 1

      aAdd ( _HMG_aFormNames , FormName )
      aAdd ( _HMG_aFormHandles , FormHandle )
      aAdd ( _HMG_aFormActive , .f. )
      aAdd ( _HMG_aFormType , "M" )
      aAdd ( _HMG_aFormParentHandle , Parent )
      aAdd ( _HMG_aFormReleaseProcedure , ReleaseProcedure )
      aAdd ( _HMG_aFormInitProcedure , InitProcedure )
      aAdd ( _HMG_aFormToolTipHandle , htooltip )
      aAdd ( _HMG_aFormContextMenuHandle , 0 )
      aAdd ( _HMG_aFormMouseDragProcedure , MouseDragProcedure )
      aAdd ( _HMG_aFormSizeProcedure , SizeProcedure )
      aAdd ( _HMG_aFormClickProcedure , ClickProcedure )
      aAdd ( _HMG_aFormMouseMoveProcedure , MouseMoveProcedure )
      aAdd ( _HMG_aFormMoveProcedure , MoveProcedure )
      aAdd ( _HMG_aFormDropProcedure , DropProcedure )
      aAdd ( _HMG_aFormDeleted, .f. )
      aAdd ( _HMG_aFormBkColor, aRGB )
      aAdd ( _HMG_aFormPaintProcedure , PaintProcedure )
      aAdd ( _HMG_aFormNoShow , noshow )
      aAdd ( _HMG_aFormNotifyIconName       , '' )
      aAdd ( _HMG_aFormNotifyIconToolTip    , '' )
      aAdd ( _HMG_aFormNotifyIconLeftClick  , '' )
      aAdd ( _HMG_aFormNotifyIconDblClick   , '' )
      aAdd ( _HMG_aFormGotFocusProcedure    , GotFocus )
      aAdd ( _HMG_aFormLostFocusProcedure   , LostFocus )
      aAdd ( _HMG_aFormReBarHandle      , 0 )
      aAdd ( _HMG_aFormNotifyMenuHandle      , 0 )
      aAdd ( _HMG_aFormBrowseList         , {} )
      aAdd ( _HMG_aFormSplitChildList      , {} )
      aAdd ( _HMG_aFormVirtualHeight      , VirtualHeight )
      aAdd ( _HMG_aFormVirtualWidth      , VirtualWidth )
      aAdd ( _HMG_aFormFocused         , .f. )
      aAdd ( _HMG_aFormScrollUp         , ScrollUp )
      aAdd ( _HMG_aFormScrollDown         , ScrollDown )
      aAdd ( _HMG_aFormScrollLeft         , ScrollLeft )
      aAdd ( _HMG_aFormScrollRight      , ScrollRight )
      aAdd ( _HMG_aFormHScrollBox         , HScrollBox )
      aAdd ( _HMG_aFormVScrollBox         , VScrollBox )
      aAdd ( _HMG_aFormBrushHandle      , BrushHandle )
      aAdd ( _HMG_aFormFocusedControl      , 0 )
      aAdd ( _HMG_aFormGraphTasks         , {} )
      aAdd ( _HMG_aFormMaximizeProcedure      , Nil )
      aAdd ( _HMG_aFormMinimizeProcedure      , Nil )
      aAdd ( _HMG_aFormRestoreProcedure      , Nil )
      aAdd ( _HMG_aFormAutoRelease      , .Not. NoAutoRelease )
      aAdd ( _HMG_aFormInteractiveCloseProcedure , InteractiveCloseProcedure )
      aAdd ( _HMG_aFormMinMaxInfo , InitMinMaxInfo ( FormHandle ) )
      aAdd ( _HMG_aFormActivateId , 0 )

   EndIf

   InitDummy(FormHandle)

   If VirtualHeight > 0
      SetScrollRange ( Formhandle , SB_VERT , 0 , VirtualHeight - h , 1 )
   EndIf
   If VirtualWidth > 0
      SetScrollRange ( Formhandle , SB_HORZ , 0 , VirtualWidth - w , 1 )
   EndIf

   If ISARRAY ( aMin ) .And. ISARRAY ( aMax )
      IF( aMin[1]==NIL, NIL, _HMG_aFormMinMaxInfo [GetFormIndex (FormName)] [5] := aMin[1] )
      IF( aMin[2]==NIL, NIL, _HMG_aFormMinMaxInfo [GetFormIndex (FormName)] [6] := aMin[2] )
      IF( aMin[1]==NIL, NIL, _HMG_aFormMinMaxInfo [GetFormIndex (FormName)] [7] := aMax[1] )
      IF( aMin[2]==NIL, NIL, _HMG_aFormMinMaxInfo [GetFormIndex (FormName)] [8] := aMax[2] )
   EndIf

Return (FormHandle)

*-----------------------------------------------------------------------------*
Function _DefineSplitChildWindow ( FormName , w , h , break , grippertext , nocaption , title , fontname , fontsize , gotfocus , lostfocus ,;
   virtualheight , VirtualWidth , Focused , scrollleft , scrollright , scrollup , scrolldown , hscrollbox , vscrollbox , cursor )
*-----------------------------------------------------------------------------*
   Local i , htooltip , mVar , ParentForm , hscroll , BrushHandle , k
   Local FormHandle , vscroll

   if valtype(FormName) == "U"
      FormName := _HMG_TempWindowName
   endif

   i := ascan ( _HMG_aFormType , 'A' )

   if i <= 0
      MsgMiniGuiError( "Main Window is not defined. Program terminated" )
   Endif

   If _IsWindowDefined (FormName)
      MsgMiniGuiError( "Window: "+ FormName + " is already defined. Program terminated" )
   endif

   If _HMG_ActiveSplitBox == .F.
      MsgMiniGuiError( "SplitChild Windows can be defined only inside SplitBox. Program terminated" )
   EndIf

   if valtype(FontName) == "U"
      _HMG_ActiveFontName := ""
   Else
      _HMG_ActiveFontName := FontName
   Endif

   if valtype(FontSize) == "U"
      _HMG_ActiveFontSize := 0
   Else
      _HMG_ActiveFontSize := FontSize
   Endif

   if valtype(VirtualHeight) == "U"
      VirtualHeight := 0
      vscroll       := .f.
   Else
      If VirtualHeight <= h
         MsgMiniGuiError( "DEFINE WINDOW: Virtual Height must be greater than Window Height. Program terminated" )
      EndIf
      vscroll       := .t.
   endif

   if valtype(VirtualWidth) == "U"
      VirtualWidth   := 0
      hscroll       := .f.
   Else
      If VirtualWidth <= w
         MsgMiniGuiError( "DEFINE WINDOW: Virtual Width must be greater than Window Width. Program terminated" )
      EndIf
      hscroll       := .t.
   endif

   _HMG_SplitChildActive := .t.

   ParentForm := _HMG_ActiveFormName

   mVar := '_' + FormName

   _HMG_ActiveFormNameBak := _HMG_ActiveFormName

   _HMG_ActiveFormName := FormName
   _HMG_BeginWindowActive := .T.

   UnRegisterWindow (FormName)
   BrushHandle := RegisterSplitChildWindow("",FormName, {-1,-1,-1})

   i := GetFormIndex ( ParentForm )

   if i > 0

      Formhandle := InitSplitChildWindow ( w , h , FormName , nocaption , title , 0 , vscroll , hscroll )

      if Valtype ( cursor ) != "U"
         SetWindowCursor( Formhandle , cursor )
      EndIf

      If _HMG_SplitLastControl == 'TOOLBAR' .And. _HMG_ActiveSplitBoxInverted == .F.
         Break := .T.
      EndIf

      AddSplitBoxItem ( FormHandle , _HMG_aFormReBarHandle [i] , w , break , grippertext ,  ,  , _HMG_ActiveSplitBoxInverted )

      _HMG_SplitLastControl   := 'SPLITCHILD'

   EndIf

   if valtype(scrollup) == "U"
      scrollup := ""
   endif
   if valtype(scrolldown) == "U"
      scrolldown := ""
   endif
   if valtype(scrollleft) == "U"
      scrollleft := ""
   endif
   if valtype(scrollright) == "U"
      scrollright := ""
   endif

   if valtype(hscrollbox) == "U"
      hscrollbox := ""
   endif
   if valtype(vscrollbox) == "U"
      vscrollbox := ""
   endif

   htooltip := InitToolTip( FormHandle , _SetToolTipBalloon() , _lSetToolTip() )

   k := ascan ( _HMG_aFormDeleted , .T. )

   if k > 0

      Public &mVar. := k
      _HMG_ActiveSplitChildIndex := k

      _HMG_aFormNames  [k] := FormName
      _HMG_aFormHandles  [k] :=  FormHandle
      _HMG_aFormActive  [k] :=  .f.
      _HMG_aFormType  [k] :=  'X'
      _HMG_aFormParentHandle  [k] :=  GetFormHandle(ParentForm)
      _HMG_aFormReleaseProcedure  [k] :=  ""
      _HMG_aFormInitProcedure  [k] :=  ""
      _HMG_aFormToolTipHandle  [k] :=  hToolTip
      _HMG_aFormContextMenuHandle  [k] :=  0
      _HMG_aFormMouseDragProcedure  [k] :=  ""
      _HMG_aFormSizeProcedure  [k] :=  ""
      _HMG_aFormClickProcedure  [k] :=  ""
      _HMG_aFormMouseMoveProcedure  [k] :=  ""
      _HMG_aFormMoveProcedure  [k] :=  ""
      _HMG_aFormDropProcedure  [k] :=  ""
      _HMG_aFormDeleted  [k] := .f.
      _HMG_aFormBkColor  [k] := Nil
      _HMG_aFormPaintProcedure  [k] :=  ""
      _HMG_aFormNoShow   [k] := .f.
      _HMG_aFormNotifyIconName  [k] :=  ""
      _HMG_aFormNotifyIconToolTip    [k] :=  ""
      _HMG_aFormNotifyIconLeftClick    [k] :=  ""
      _HMG_aFormNotifyIconDblClick    [k] :=  ""
      _HMG_aFormGotFocusProcedure  [k] := gotfocus
      _HMG_aFormLostFocusProcedure [k] :=     lostfocus
      _HMG_aFormReBarHandle    [k] :=  0
      _HMG_aFormNotifyMenuHandle [k] :=     0
      _HMG_aFormBrowseList    [k] :=     {}
      _HMG_aFormSplitChildList [k] :=     {}
      _HMG_aFormVirtualHeight    [k] :=     VirtualHeight
      _HMG_aFormVirtualWidth    [k] :=     VirtualWidth
      _HMG_aFormFocused    [k] :=     Focused
      _HMG_aFormScrollUp    [k] :=     ScrollUp
      _HMG_aFormScrollDown    [k] :=     ScrollDown
      _HMG_aFormScrollLeft    [k] :=     ScrollLeft
      _HMG_aFormScrollRight    [k] :=     ScrollRight
      _HMG_aFormHScrollBox    [k] :=     HScrollBox
      _HMG_aFormVScrollBox    [k] :=     VScrollBox
      _HMG_aFormBrushHandle    [k] :=     BrushHandle
      _HMG_aFormFocusedControl [k] :=     0
      _HMG_aFormGraphTasks    [k] :=     {}
      _HMG_aFormMaximizeProcedure [k] :=      Nil
      _HMG_aFormMinimizeProcedure [k] :=      Nil
      _HMG_aFormRestoreProcedure [k] :=      Nil
      _HMG_aFormAutoRelease [k] :=  .T.
      _HMG_aFormInteractiveCloseProcedure [k] :=  ""
      _HMG_aFormMinMaxInfo [k] := InitMinMaxInfo ( FormHandle )
      _HMG_aFormActivateId [k] := 0

   Else

      Public &mVar. := Len(_HMG_aFormNames) + 1
      _HMG_ActiveSplitChildIndex := Len(_HMG_aFormNames) + 1

      aAdd ( _HMG_aFormNames , FormName )
      aAdd ( _HMG_aFormHandles , FormHandle )
      aAdd ( _HMG_aFormActive , .f. )
      aAdd ( _HMG_aFormType , 'X' )
      aAdd ( _HMG_aFormParentHandle , GetFormHandle(ParentForm) )
      aAdd ( _HMG_aFormReleaseProcedure , "" )
      aAdd ( _HMG_aFormInitProcedure , "" )
      aAdd ( _HMG_aFormToolTipHandle , hToolTip )
      aAdd ( _HMG_aFormContextMenuHandle , 0 )
      aAdd ( _HMG_aFormMouseDragProcedure , "" )
      aAdd ( _HMG_aFormSizeProcedure , "" )
      aAdd ( _HMG_aFormClickProcedure , "" )
      aAdd ( _HMG_aFormMouseMoveProcedure , "" )
      aAdd ( _HMG_aFormMoveProcedure , "" )
      aAdd ( _HMG_aFormDropProcedure , "" )
      aAdd ( _HMG_aFormDeleted, .f. )
      aAdd ( _HMG_aFormBkColor, Nil )
      aAdd ( _HMG_aFormPaintProcedure , "" )
      aAdd ( _HMG_aFormNoShow , .f. )
      aAdd ( _HMG_aFormNotifyIconName       , "" )
      aAdd ( _HMG_aFormNotifyIconToolTip    , "" )
      aAdd ( _HMG_aFormNotifyIconLeftClick  , "" )
      aAdd ( _HMG_aFormNotifyIconDblClick   , "" )
      aAdd ( _HMG_aFormGotFocusProcedure    , gotfocus )
      aAdd ( _HMG_aFormLostFocusProcedure   , lostfocus )
      aAdd ( _HMG_aFormReBarHandle      , 0 )
      aAdd ( _HMG_aFormNotifyMenuHandle      , 0 )
      aAdd ( _HMG_aFormBrowseList         , {} )
      aAdd ( _HMG_aFormSplitChildList      , {} )
      aAdd ( _HMG_aFormVirtualHeight      , VirtualHeight )
      aAdd ( _HMG_aFormVirtualWidth      , VirtualWidth )
      aAdd ( _HMG_aFormFocused         , Focused )
      aAdd ( _HMG_aFormScrollUp         , ScrollUp )
      aAdd ( _HMG_aFormScrollDown         , ScrollDown )
      aAdd ( _HMG_aFormScrollLeft         , ScrollLeft )
      aAdd ( _HMG_aFormScrollRight      , ScrollRight )
      aAdd ( _HMG_aFormHScrollBox         , HScrollBox )
      aAdd ( _HMG_aFormVScrollBox         , VScrollBox )
      aAdd ( _HMG_aFormBrushHandle      , BrushHandle )
      aAdd ( _HMG_aFormFocusedControl      , 0 )
      aAdd ( _HMG_aFormGraphTasks         , {} )
      aAdd ( _HMG_aFormMaximizeProcedure      , Nil )
      aAdd ( _HMG_aFormMinimizeProcedure      , Nil )
      aAdd ( _HMG_aFormRestoreProcedure      , Nil )
      aAdd ( _HMG_aFormAutoRelease      , .T. )
      aAdd ( _HMG_aFormInteractiveCloseProcedure , "" )
      aAdd ( _HMG_aFormMinMaxInfo , {} )
      aAdd ( _HMG_aFormActivateId , 0 )

   EndIf

   InitDummy(FormHandle)

   aAdd ( _HMG_aFormSplitChildList [ i ] , _HMG_ActiveSplitChildIndex )

   If VirtualHeight > 0
      SetScrollRange ( Formhandle , SB_VERT , 0 , VirtualHeight - h , 1 )
   EndIf
   If VirtualWidth > 0
      SetScrollRange ( Formhandle , SB_HORZ , 0 , VirtualWidth - w , 1 )
   EndIf

Return (FormHandle)

*-----------------------------------------------------------------------------*
Function _SetWindowSizePos ( FormName , row , col , width , height )
*-----------------------------------------------------------------------------*
   Local actpos := {0,0,0,0}
   Local h := GetFormHandle( FormName )

   GetWindowRect( h, actpos )

   col   := IFNIL( col, actpos[1], col )
   row   := IFNIL( row, actpos[2], row )
   width := IFNIL( width, actpos[3] - actpos[1], width )
   height:= IFNIL( height, actpos[4] - actpos[2], height )

   MoveWindow ( h , col , row , width , height , .t. )

Return Nil

*-----------------------------------------------------------------------------*
Function GetFormIndex (FormName)
*-----------------------------------------------------------------------------*
   Local mVar

   mVar := '_' + FormName

Return ( &mVar )

*-----------------------------------------------------------------------------*
Function _SetNotifyIconName ( FormName , IconName )
*-----------------------------------------------------------------------------*
   Local i := GetFormIndex ( FormName )

   _HMG_aFormNotifyIconName [i] := IconName

Return ChangeNotifyIcon( _HMG_aFormHandles [i], LoadTrayIcon( GetInstance(), IconName ), _HMG_aFormNotifyIconTooltip [i] )

*-----------------------------------------------------------------------------*
Function _SetNotifyIconTooltip ( FormName , TooltipText )
*-----------------------------------------------------------------------------*
   Local i := GetFormIndex ( FormName )

   _HMG_aFormNotifyIconTooltip [i] := TooltipText

Return ChangeNotifyIcon( _HMG_aFormHandles [i], LoadTrayIcon( GetInstance(), _HMG_aFormNotifyIconName [i] ), TooltipText )

*-----------------------------------------------------------------------------*
Function _GetNotifyIconName ( FormName )
*-----------------------------------------------------------------------------*
   Local i := GetFormIndex ( FormName )

Return ( _HMG_aFormNotifyIconName [i] ) 

*-----------------------------------------------------------------------------*
Function _GetNotifyIconTooltip ( FormName )
*-----------------------------------------------------------------------------*
   Local i := GetFormIndex ( FormName )

Return ( _HMG_aFormNotifyIconTooltip [i] ) 

*-----------------------------------------------------------------------------*
Function _DefineSplitBox ( ParentForm, bottom, inverted )
*-----------------------------------------------------------------------------*
   Local i, cParentForm, Controlhandle

   if _HMG_BeginWindowActive = .T.
      ParentForm := _HMG_ActiveFormName
   endif
   if _HMG_FrameLevel > 0
      MsgMiniGuiError("SPLITBOX cannot be defined inside Tab control. Program terminated" )

   EndIf

   If .Not. _IsWindowDefined (ParentForm)
      MsgMiniGuiError("Window: "+ ParentForm + " is not defined. Program terminated" )
   Endif

   If _HMG_SplitChildActive == .T.
      MsgMiniGuiError("SplitBox cannot be defined inside SplitChild Windows. Program terminated" )
   EndIf

   If _HMG_ActiveSplitBox == .T.
      MsgMiniGuiError("SplitBox controls cannot be nested. Program terminated" )
   EndIf

   _HMG_ActiveSplitBoxInverted := Inverted

   _HMG_ActiveSplitBox := .T.

   _HMG_ActiveSplitBoxParentFormName := ParentForm

   cParentForm := ParentForm

   ParentForm = GetFormHandle (ParentForm)

   ControlHandle := InitSplitBox ( ParentForm, bottom , inverted )

   i := GetFormIndex ( cParentForm )

   if i > 0
      _HMG_aFormReBarHandle [i] := ControlHandle
   EndIf

Return Nil

*-----------------------------------------------------------------------------*
Function _EndSplitBox ()
*-----------------------------------------------------------------------------*

   _HMG_SplitLastControl   := 'TOOLBAR'

   _HMG_ActiveSplitBox := .F.

Return Nil

*-----------------------------------------------------------------------------*
Function _EndSplitChildWindow ()
*-----------------------------------------------------------------------------*

   _HMG_ActiveFormName := _HMG_ActiveFormNameBak
   _HMG_SplitChildActive := .f.
   _HMG_aFormActive [ _HMG_ActiveSplitChildIndex ] := .t.

Return Nil

*-----------------------------------------------------------------------------*
Function _EndWindow ()
*-----------------------------------------------------------------------------*

   Do Case
   case _HMG_SplitChildActive == .t.
      _EndSplitChildWindow ()
   case _HMG_MdiChildActive == .t.   // JP MDI
      _EndMdiChildWindow ()
   otherwise
      _HMG_BeginWindowActive := .F.
      _HMG_ActiveFormName := ""
   Endcase

Return Nil

*-----------------------------------------------------------------------------*
Function InputBox ( cInputPrompt , cDialogCaption , cDefaultValue , nTimeout , cTimeoutValue , lMultiLine )
*-----------------------------------------------------------------------------*

   Local RetVal := '' , mo := If( ValType (lMultiLine) == 'L' .and. lMultiLine == .T., 150, 0 )

   DEFAULT cInputPrompt   TO ""
   DEFAULT cDialogCaption   TO ""
   DEFAULT cDefaultValue   TO ""

   DEFINE WINDOW _InputBox;
      AT 0,0;
      WIDTH 350;
      HEIGHT 115 + mo + GetTitleHeight();
      TITLE cDialogCaption;
      MODAL;
      NOSIZE;
      FONT 'Arial';
      SIZE 10

      ON KEY ESCAPE ACTION ( _HMG_DialogCancelled := .T. , _InputBox.Release )

      @ 07,10 LABEL _Label;
         VALUE cInputPrompt;
         WIDTH 280
// JK
      If ValType (lMultiLine) == 'L' .and. lMultiLine == .T.
         @ 30,10 EDITBOX _TextBox;
            VALUE cDefaultValue;
            HEIGHT 26 + mo;
            WIDTH 320
      else
         @ 30,10 TEXTBOX _TextBox;
            VALUE cDefaultValue;
            HEIGHT 26 + mo;
            WIDTH 320;
            ON ENTER ( _HMG_DialogCancelled := .F. , RetVal := _InputBox._TextBox.Value , _InputBox.Release )

      endif
      //
      @ 67+mo,120 BUTTON _Ok;
         CAPTION _HMG_MESSAGE [6];
         ACTION ( _HMG_DialogCancelled := .F. , RetVal := _InputBox._TextBox.Value , _InputBox.Release )

      @ 67+mo,230 BUTTON _Cancel;
         CAPTION _HMG_MESSAGE [7];
         ACTION   ( _HMG_DialogCancelled := .T. , _InputBox.Release )

      If ValType (nTimeout) != 'U'

         If ValType (cTimeoutValue) != 'U'

            DEFINE TIMER _InputBox;
               INTERVAL nTimeout;
               ACTION  ( RetVal := cTimeoutValue , _InputBox.Release )

         Else

            DEFINE TIMER _InputBox;
               INTERVAL nTimeout;
               ACTION _InputBox.Release

         EndIf

      EndIf

   END WINDOW

   _InputBox._TextBox.SetFocus

   CENTER WINDOW _InputBox

   ACTIVATE WINDOW _InputBox

Return ( RetVal )

*-----------------------------------------------------------------------------*
Function _SetWindowRgn(name,col,row,w,h,lx)
*-----------------------------------------------------------------------------*

   c_SetWindowRgn( GetFormHandle( name ), col, row, w, h, lx )

Return Nil
*-----------------------------------------------------------------------------*
Function _SetPolyWindowRgn(name,apoints,lx)
*-----------------------------------------------------------------------------*
   local apx:={}, apy:={}

   aeval(apoints,{|x| aadd(apx,x[1]), aadd(apy,x[2])})

   c_SetPolyWindowRgn( GetFormHandle( name ), apx, apy, lx )

Return Nil

*-----------------------------------------------------------------------------*
Function _ActivateWindow ( aForm, lNoWait )
*-----------------------------------------------------------------------------*
   Local i , z , MainFound := .F.
   local nForm := len( aForm )
   Local FormName
   Local VisibleModalCount := 0
   Local VisibleModalName := ''
   Local TmpId

   If _HMG_ThisEventType == 'WINDOW_RELEASE'
      MsgMiniGuiError("ACTIVATE WINDOW: activate windows within an ON RELEASE window procedure is not allowed. Program terminated" )
   EndIf

   If _HMG_BeginWindowActive = .T.
      MsgMiniGUIError("ACTIVATE WINDOW: DEFINE WINDOW Structure is not closed. Program terminated" )
   Endif

   If _HMG_ThisEventType == 'WINDOW_GOTFOCUS'
      MsgMiniGUIError("ACTIVATE WINDOW / Activate(): Not allowed in window's GOTFOCUS event procedure. Program terminated" )
   Endif

   If _HMG_ThisEventType == 'WINDOW_LOSTFOCUS'
      MsgMiniGUIError("ACTIVATE WINDOW / Activate(): Not allowed in window's LOSTFOCUS event procedure. Program terminated" )
   Endif

   * Look For Main Window

   For z := 1 to nForm
      FormName := aForm [z]
      i := GetFormIndex ( FormName )
      If _hmg_aFormType [i] == 'A'
         MainFound := .T.
         Exit
      EndIf
   Next z

   // Main Check

   If _HMG_MainActive == .F.
      If MainFound == .F.
         MsgMiniGUIError("ACTIVATE WINDOW: Main Window must be activated in the first ACTIVATE WINDOW command. Program terminated" )
      EndIf
   Else
      If MainFound == .T.
         MsgMiniGUIError("ACTIVATE WINDOW: Main Window already active. Program terminated" )
      EndIf
   EndIf

   // Set Main Active Public Flag

   If MainFound == .T.
      _HMG_MainActive := .T.
   EndIf

   If nForm > 1

      If _HMG_IsModalActive
         MsgMiniGUIError("Multiple Activation can not be used when a modal window is active. Program terminated" )
      Endif

      TmpId := _GenActivateId( nForm )

      For z := 1 to nForm

         FormName := aForm [z]

         If .Not. _IsWindowDefined ( Formname )
            MsgMiniGUIError("Window: "+ FormName + " is not defined. Program terminated" )
         Endif

         If _IsWindowActive ( FormName )
            MsgMiniGUIError("Window: "+ FormName + " already active. Program terminated" )
         Endif

         i := GetFormIndex ( FormName )

         * Only One Visible Modal is Allowed

         If _HMG_aFormType [i] == "M" .and. _HMG_aFormNoShow [i] == .F.
            VisibleModalCount++
            VisibleModalName := _hmg_aformnames [i]
            if VisibleModalCount > 1
               MsgMiniGUIError("ACTIVATE WINDOW: Only one initially visible modal window allowed. Program terminated" )
            EndIf
         Endif

         * Set Activate Id

         _HMG_aFormActivateId [i] := TmpId

         * If NOSHOW Is Not Specified, Show The Window.

         If _HMG_aFormNoShow [i] == .F.
            ShowWindow( _HMG_aFormHandles [i] )
         EndIf

         _SetActivationFlag(i)
         _ProcessInitProcedure(i)
         _RefreshDataControls(i)

      Next z

      * If Specified, Execute Show Method For Visible Modal
      * If Not, Process Focus For Last Window In The List

      If VisibleModalCount == 1
         _ShowWindow ( VisibleModalName )
      Else
         If _SetFocusedSplitChild(i) == .f.
            _SetActivationFocus(i)
         Endif
      EndIf

   Else

      FormName := aForm [1]

      If .Not. _IsWindowDefined ( Formname )
         MsgMiniGUIError("Window: "+ FormName + " is not defined. Program terminated" )
      Endif

      If _IsWindowActive ( FormName )
         MsgMiniGUIError("Window: "+ FormName + " already active. Program terminated" )
      Endif

      i := GetFormIndex ( FormName )
      // JP MDI Background
      if _HMG_BeginWindowMDIActive
         IF _HMG_aFormBkColor [i] [1] != -1
            SetWindowBackground( _HMG_MainClientMDIHandle, PaintBkGnd( _HMG_MainClientMDIHandle, _HMG_aFormBkColor [i] ) )
         endif
      endif
      // JP end
      if _HMG_aFormType [i] == "M"

         _ShowWindow ( _HMG_aFormNames [i], .F. )

         _SetActivationFlag(i)
         _ProcessInitProcedure(i)
         _RefreshDataControls(i)

      Else
         If _HMG_IsModalActive
            MsgMiniGUIError("Non Modal Window " + Formname + " can't be activated when a modal window is active. Program terminated" )
         endif

         If _HMG_aFormNoShow [i] == .F.
            ShowWindow( GetFormHandle(FormName) )
         EndIf

         _SetActivationFlag(i)
         _ProcessInitProcedure(i)
         _RefreshDataControls(i)

         If _SetFocusedSplitChild(i) == .f.
            _SetActivationFocus(i)
         endif

      Endif

   Endif

   If ValType( lNoWait ) != "L"
      lNoWait := .F.
   EndIf

   If .Not. lNoWait
      * Start The Message Loop
      _DoMessageLoop()
   Endif

Return Nil

*-----------------------------------------------------------------------------*
Function _ActivateAllWindows
*-----------------------------------------------------------------------------*
   Local i
   Local FormCount := Len(_HMG_aFormNames)
   Local aForm := {}
   Local MainName := ''

   * If Already Active Windows Abort Command

   If ascan ( _hmg_aFormActive , .t. ) > 0
      MsgMiniGuiError("ACTIVATE WINDOW ALL: this command should be used at application startup only. Program terminated" )
   EndIf

   * Force NoShow And NoAutoRelease Styles For Non Main Windows
   * ( Force AutoRelease And Visible For Main )

   For i := 1 To FormCount

      If _HMG_aFormDeleted [i] == .F.

         If _hmg_aFormType [i] != 'X'

            if _hmg_aFormType [i] == 'A'
               _hmg_aFormNoShow [i] := .F.
               _hmg_aFormAutoRelease [i] := .T.
               MainName := _HMG_aFormNames [i]
            ELse
               _hmg_aFormNoShow [i] := .T.
               _hmg_aFormAutoRelease [i] := .F.
               aadd ( aForm , _HMG_aFormNames [i] )
            EndIf

         EndIf

      EndIf

   Next i

   aadd ( aForm , MainName )

   * Check For Error And Call Activate Window Command

   if Len ( aForm ) > 0

      If Empty ( MainName )
         MsgMiniGuiError("ACTIVATE WINDOW ALL: Main Window is not defined. Program terminated" )
      EndIf

      _ActivateWindow ( aForm )

   Else
      MsgMiniGuiError("ACTIVATE WINDOW ALL: no windows is defined. Program terminated" )
   EndIf

Return Nil

*-----------------------------------------------------------------------------*
Procedure _RefreshDataControls(i)
*-----------------------------------------------------------------------------*
   Local x, w, v, ControlIndex, SplitIndex

   If Len ( _HMG_aFormGraphTasks [i] ) > 0
      SendMessage( _HMG_aFormHandles [i] , WM_PAINT , 0 , 0 )
   EndIf

   For x := 1 To Len ( _HMG_aFormBrowseList [i] )
      ControlIndex := _HMG_aFormBrowseList [i] [x]
      v := _HMG_aControlValue [ ControlIndex ]
      _Refresh ( ControlIndex )
      If _HMG_aControlType [ ControlIndex ] == 'COMBO' .Or. _HMG_aControlType [ ControlIndex ] == 'BROWSE'
         _SetValue ( '', '', v , ControlIndex )
      EndIf
   Next x

   If Len ( _HMG_aFormSplitChildList [i] ) > 0

      For x := 1 To Len ( _HMG_aFormSplitChildList [i] )
         SplitIndex := _HMG_aFormSplitChildList [i] [x]
         For w := 1 To Len ( _HMG_aFormBrowseList [SplitIndex] )
            ControlIndex := _HMG_aFormBrowseList [SplitIndex] [w]
            v := _HMG_aControlValue [ ControlIndex ]
            _Refresh ( ControlIndex )
            If _HMG_aControlType [ ControlIndex ] == 'COMBO' .Or. _HMG_aControlType [ ControlIndex ] == 'BROWSE'
               _SetValue ( '', '', v , ControlIndex )
            EndIf
         Next w
      Next x

   EndIf

Return
*-----------------------------------------------------------------------------*
Procedure _SetActivationFlag(i)
*-----------------------------------------------------------------------------*
   Local x

   _HMG_aFormActive [i] = .t.

   For x := 1 To Len ( _HMG_aFormSplitChildList [ i ] )
      _HMG_aFormActive [ _HMG_aFormSplitChildList [i] [x] ] := .t.
   Next x

   If valtype( _HMG_aFormDropProcedure [i] ) == 'B'
      DragAcceptFiles( _HMG_aFormHandles [i], .T. )
   EndIf

Return
*-----------------------------------------------------------------------------*
Procedure _ProcessInitProcedure(i)
*-----------------------------------------------------------------------------*
   If valtype(_HMG_aFormInitProcedure [i]) == 'B'
      ProcessMessages()
      _PushEventInfo()
      _HMG_ThisEventType := 'WINDOW_INIT'
      _HMG_ThisFormIndex := i
      _HMG_ThisType := 'W'
      _HMG_ThisIndex := i
      _HMG_ThisFormName :=  _HMG_aFormNames [ _HMG_ThisFormIndex ]
      _HMG_ThisControlName :=  ""
      Eval( _HMG_aFormInitProcedure [i] )
      _PopEventInfo()
   EndIf
   if _HMG_AutoAdjust .and. _HMG_MainClientMDIHandle == 0
      _Autoadjust( _HMG_aFormHandles [i] )
   endif

Return
*-----------------------------------------------------------------------------*
Function _SetFocusedSplitChild(i)
*-----------------------------------------------------------------------------*
   Local x, SplitFocusFlag := .f.

   if Len ( _HMG_aFormSplitChildList [ i ] ) > 0

      For x := 1 To Len ( _HMG_aFormSplitChildList [ i ] )
         If _HMG_aFormFocused [ _HMG_aFormSplitChildList [i] [x] ] == .t.
            setfocus ( _HMG_aFormhandles [ _HMG_aFormSplitChildList [i] [x] ] )
            SplitFocusFlag := .T.
         EndIf
      Next x

   EndIf

Return SplitFocusFlag
*-----------------------------------------------------------------------------*
Procedure _SetActivationFocus(i)
*-----------------------------------------------------------------------------*
   Local x, FocusDefined := .f., Sp := GetFocus()

   For x := 1 To Len (_HMG_aControlHandles)
      If _HMG_aControlParenthandles [x] == _HMG_aFormhandles [i]
         If ValType ( _HMG_aControlhandles [x] ) == 'N'
            If _HMG_aControlhandles [x] == Sp
               FocusDefined := .T.
               Exit
            EndIf
         ElseIf ValType ( _HMG_aControlhandles [x] ) == 'A'
            If _HMG_aControlhandles [x] [1] == Sp
               FocusDefined := .T.
               Exit
            EndIf
         EndIf
      EndIf
   Next x
   If FocusDefined == .F.
      setfocus( GetNextDlgTabItem ( _HMG_aFormHandles [i] , 0 , 0 ) )
   EndIf

Return
*-----------------------------------------------------------------------------*
Function _GenActivateId(nForm)
*-----------------------------------------------------------------------------*
   Local TmpStr
   Local TmpId

   Do While .T.
      TmpId := Int ( Seconds() * 100 )
      TmpStr := '_HMG_ACTIVATE_' + alltrim( str ( TmpId ) )
      If ! __MVEXIST ( TmpStr )
         Exit
         End If
      EndDo

      __MVPUBLIC ( TmpStr )
      __MVPUT ( TmpStr , nForm )

Return TmpId
*-----------------------------------------------------------------------------*
Procedure _hmg_OnHideFocusManagement(i)
*-----------------------------------------------------------------------------*
   Local x, z, FormCount := Len ( _HMG_aFormHandles )

   If _HMG_aFormParentHandle [i] == 0

      * Non Modal

      if _hmg_IsModalActive == .F.
         For z := 1 to FormCount
            if _HMG_aFormDeleted [z] == .f.
               EnableWindow ( _HMG_aFormhandles [z] )
            EndIf
         Next z

      EndIf

   Else

      * Modal

      x := Ascan ( _HMG_aFormHandles , _HMG_aFormParenthandle [i] )
      if x > 0

         if _HMG_aFormType [x] == "M"

            * Modal Parent

            _HMG_IsModalActive := .T.
            _hmg_activemodalhandle := _HMG_aFormParenthandle [i]

            EnableWindow ( _HMG_aFormParenthandle [i] )

            SetFocus( _HMG_aFormParenthandle [i])

         Else

            * Non Modal Parent

            _HMG_IsModalActive := .F.
            _hmg_activemodalhandle := 0

            For z := 1 to FormCount
               If _HMG_aFormDeleted [z] == .f.
                  EnableWindow ( _HMG_aFormhandles [z] )
               EndIf
            Next z

            SetFocus( _HMG_aFormParenthandle [i] )

         Endif

      Else

         * Missing Parent

         _HMG_IsModalActive := .F.
         _hmg_activemodalhandle := 0

         For z := 1 to FormCount
            If _HMG_aFormDeleted [z] == .f.
               EnableWindow ( _HMG_aFormhandles [z] )
            EndIf
         Next z

         SetFocus( _HMG_MainHandle )

      EndIf

   EndIf

Return
*-----------------------------------------------------------------------------*
Function _DoControlEventProcedure ( bBlock , i , cEventType , nParam )
*-----------------------------------------------------------------------------*
   Local lRetVal

   Default cEventType := '' , nParam := 0

   if valtype( bBlock ) == 'B'

      _PushEventInfo()
      _HMG_ThisFormIndex := ascan ( _HMG_aFormHandles , _HMG_aControlParentHandles[i] )
      _HMG_ThisType := 'C'
      _HMG_ThisIndex := i
      _HMG_ThisFormName := _HMG_aFormNames [ _HMG_ThisFormIndex ]
      _HMG_ThisControlName := _HMG_aControlNames [ _HMG_ThisIndex ]
      If _HMG_BeginWindowActive == .F. .or. cEventType # 'CONTROL_ONCHANGE' .or. _HMG_MainClientMDIHandle != 0
         Eval( bBlock, nParam )
      EndIf
      _PopEventInfo()
      lRetVal := .T.

   Else
      lRetVal := .F.
   EndIf

Return lRetVal
*-----------------------------------------------------------------------------*
Function _DoWindowEventProcedure ( bBlock , i , cEventType )
*-----------------------------------------------------------------------------*
   Local lRetVal := .F.

   Default cEventType := ''

   if valtype( bBlock ) == 'B'

      _PushEventInfo()
      _HMG_ThisFormIndex := i
      _HMG_ThisEventType := cEventType
      _HMG_ThisType := 'W'
      _HMG_ThisIndex := i
      _HMG_ThisFormName := _HMG_aFormNames [ _HMG_ThisFormIndex ]
      _HMG_ThisControlName :=  ""
      lRetVal := Eval( bBlock )
      _PopEventInfo()

   EndIf

Return lRetVal
*-----------------------------------------------------------------------------*
Function _GetGridCellData (i)
*-----------------------------------------------------------------------------*
   Local ThisItemRowIndex
   Local ThisItemColIndex
   Local ThisItemCellRow
   Local ThisItemCellCol
   Local ThisItemCellWidth
   Local ThisItemCellHeight
   Local r
   Local xs
   Local xd
   Local aCellData

   r := ListView_HitTest ( _HMG_aControlHandles [i] , GetCursorRow() - GetWindowRow ( _HMG_aControlHandles [i] ) , GetCursorCol() - GetWindowCol ( _HMG_aControlHandles [i] ) )
   If r [2] == 1
      ListView_Scroll( _HMG_aControlHandles [i] , -10000  , 0 )
      r := ListView_HitTest ( _HMG_aControlHandles [i] , GetCursorRow() - GetWindowRow ( _HMG_aControlHandles [i] ) , GetCursorCol() - GetWindowCol ( _HMG_aControlHandles [i] ) )
   Else
      r := LISTVIEW_GETSUBITEMRECT ( _HMG_aControlHandles [i]  , r[1] - 1 , r[2] - 1 )

            *   CellCol                  CellWidth
      xs := ( ( _HMG_aControlCol [i] + r [2] ) + ( r[3] )) - ( _HMG_aControlCol [i] + _HMG_aControlWidth [i] )

      If ListViewGetItemCount( _HMG_aControlHandles [i] ) > ListViewGetCountPerPage( _HMG_aControlHandles [i] )
         xd := 20
      Else
         xd := 0
      EndIf

      If xs > -xd
         ListView_Scroll( _HMG_aControlHandles [i] , xs + xd , 0 )
      Else
         If r [2] < 0
            ListView_Scroll( _HMG_aControlHandles [i] , r[2] , 0 )
         EndIf
      EndIf

      r := ListView_HitTest ( _HMG_aControlHandles [i] , GetCursorRow() - GetWindowRow ( _HMG_aControlHandles [i] ) , GetCursorCol() - GetWindowCol ( _HMG_aControlHandles [i] ) )

   EndIf

   ThisItemRowIndex := r[1]
   ThisItemColIndex := r[2]

   If r [2] == 1
      r := ListView_GetItemRect ( _HMG_aControlHandles [i] , r[1] - 1 )
   Else
      r := ListView_GetSubItemRect ( _HMG_aControlHandles [i] , r[1] - 1 , r[2] - 1 )
   EndIf

   ThisItemCellRow := _HMG_aControlRow [i] + r [1]
   ThisItemCellCol := _HMG_aControlCol [i] + r [2]
   ThisItemCellWidth := r[3]
   ThisItemCellHeight := r[4]

   aCellData := { ThisItemRowIndex , ThisItemColIndex , ThisItemCellRow , ThisItemCellCol , ThisItemCellWidth , ThisItemCellHeight }

Return aCellData

*-----------------------------------------------------------------------------*
Function _SetToolTipBalloon ( lNewBalloon )
*-----------------------------------------------------------------------------*
   Static lBalloon := .F.
   Local lOldBalloon := lBalloon

   If lNewBalloon <> Nil
      lBalloon := lNewBalloon
   Endif

return lOldBalloon

/*
   31/10/2005
   Enable/Disable ToolTip Messages
*/
*-----------------------------------------------------------------------------*
Function _lSetToolTip( lNewValue )
*-----------------------------------------------------------------------------*
   Static lToolTip := .T.
   Local lOldValue := lToolTip

   If lNewValue <> Nil
      lToolTip := lNewValue
   Endif

return lOldValue

*-----------------------------------------------------------------------------*
Function IsThemed()
*-----------------------------------------------------------------------------*
   local uResult := .F.

   If IsXPThemeActive() .AND. IsAppXPThemed()
      if GetDllVersion("comctl32.dll")[1] >= 6
         uResult := .T.
      EndIf
   EndIf

return uResult

*-----------------------------------------------------------------------------*
Function IsAppXPThemed()
*-----------------------------------------------------------------------------*
   local uResult := .F.

   If _HMG_IsXPorLater

      InitUxTheme()

      uResult := IsAppThemed()

      EndUxTheme()

   EndIf

return uResult

*-----------------------------------------------------------------------------*
Function IsXPThemeActive()
*-----------------------------------------------------------------------------*
   local uResult := .F.

   If _HMG_IsXPorLater

      InitUxTheme()

      uResult := IsThemeActive()

      EndUxTheme()

   EndIf

return uResult

*-----------------------------------------------------------------------------*
Procedure VirtualChildControlFocusProcess( nControlHandle , nWindowHandle )
*-----------------------------------------------------------------------------*
   Local x
   Local nWindowVirtualWidth
   Local nWindowVirtualHeight
   Local nWindowHeight			:= 0
   Local nWindowWidth			:= 0
   Local nControlHeight			:= 0
   Local nControlWidth			:= 0
   Local nControlRow			:= 0
   Local nControlCol			:= 0
   Local nHorizontalScrollBoxPos
   Local nVerticalScrollBoxPos
   Local nHorizontalScrollBarRangeMax
   Local nVerticalScrollBarRangeMax
   Local nVisibleAreaFromRow
   Local nVisibleAreaFromCol
   Local nVisibleAreaToRow
   Local nVisibleAreaToCol
   Local nNewScrollBarPos

   IF _HMG_AutoScroll == .F. .or. _HMG_AutoAdjust == .T.
      Return
   ENDIF

	* Get Window Width / Height / Virtual Width / Virtual Height

   For x := 1 To Len ( _HMG_aFormHandles )

      If _HMG_aFormHandles [x] == nWindowHandle

         nWindowVirtualHeight	:= _HMG_aFormVirtualHeight [x]
         nWindowVirtualWidth	:= _HMG_aFormVirtualWidth [x]

         If 	nWindowVirtualHeight == 0 .And. nWindowVirtualWidth == 0
            Return
         Else
            nWindowHeight	:= GetWindowHeight ( nWindowHandle )
            nWindowWidth	:= GetWindowWidth ( nWindowHandle )
            Exit
         EndIf

      EndIf

   Next x

	* Get Control Row / Col / Width / Height

   For x := 1 To Len ( _HMG_aControlHandles )

      If ISNUMBER ( _HMG_aControlHandles [x] ) .And. ISNUMBER ( nControlHandle )

         If _HMG_aControlHandles [x] == nControlHandle

            nControlHeight	:= _HMG_aControlHeight [ x ]
            nControlWidth	:= _HMG_aControlWidth [ x ]
            nControlRow	:= _HMG_aControlRow [ x ]
            nControlCol	:= _HMG_aControlCol [ x ]

            Exit

         EndIf


      ElseIf	ISARRAY ( _HMG_aControlHandles [x] ) .And. ISNUMBER ( nControlHandle )

         If aScan ( _HMG_aControlHandles [x] , nControlHandle ) > 0

            nControlHeight	:= _HMG_aControlHeight [ x ]
            nControlWidth	:= _HMG_aControlWidth [ x ]
            nControlRow	:= _HMG_aControlRow [ x ]
            nControlCol	:= _HMG_aControlCol [ x ]

            Exit

         EndIf

      EndIf

   Next x

	* Get hScrollBox Position / vScrollBox Position

   nHorizontalScrollBoxPos	:= GetScrollPos ( nWindowHandle , SB_HORZ )
   nVerticalScrollBoxPos	:= GetScrollPos ( nWindowHandle , SB_VERT )

	* Get hScrollBar Maximun Range / vScrollBar Maximun Range

   nHorizontalScrollBarRangeMax	:= GetScrollRangeMax( nWindowHandle , SB_HORZ )
   nVerticalScrollBarRangeMax	:= GetScrollRangeMax( nWindowHandle , SB_VERT )

        * Calculate Current Visible Area

   nVisibleAreaFromRow	:= nVerticalScrollBoxPos
   nVisibleAreaFromCol	:= nHorizontalScrollBoxPos

   nVisibleAreaToRow	:= nVisibleAreaFromRow + nWindowHeight - 50
   nVisibleAreaToCol	:= nVisibleAreaFromCol + nWindowWidth - 10

	* Determine if Control Getting the Focus is out of Visible
	* Area. If So, scroll The Window.

	* Control is too Low To be Visible

   If nControlRow + nControlHeight > nVisibleAreaToRow

      nNewScrollBarPos := nControlRow + nControlHeight - nWindowHeight + 100

      If nNewScrollBarPos > nVerticalScrollBarRangeMax
         nNewScrollBarPos := nVerticalScrollBarRangeMax
      EndIf

      _HMG_SETVSCROLLVALUE( nWindowHandle , nNewScrollBarPos )

   Else

		* Control is too High To be Visible

      If nControlRow + nControlHeight < nVisibleAreaFromRow

         nNewScrollBarPos := nControlRow - nWindowHeight - 100

         If nNewScrollBarPos < 0
            nNewScrollBarPos := 0				
         EndIf

         _HMG_SETVSCROLLVALUE( nWindowHandle , nNewScrollBarPos )

      EndIf

   EndIf

	* Control Is Too Right To Be Visible

   If nControlCol + nControlWidth > nVisibleAreaToCol

      nNewScrollBarPos := nControlCol + nControlWidth - nWindowWidth + 100

      If nNewScrollBarPos > nHorizontalScrollBarRangeMax
         nNewScrollBarPos := nHorizontalScrollBarRangeMax
      EndIf

      _HMG_SETHSCROLLVALUE( nWindowHandle , nNewScrollBarPos )

   Else

		* Control Is Too Left To Be Visible

      If nControlCol + nControlWidth < nVisibleAreaFromCol

         nNewScrollBarPos := nControlCol - nWindowWidth - 100

         If nNewScrollBarPos < 0
            nNewScrollBarPos := 0				
         EndIf

         _HMG_SETHSCROLLVALUE( nWindowHandle , nNewScrollBarPos )


      EndIf

   EndIf

Return

*-----------------------------------------------------------------------------*
Function GetWindowType ( FormName )
*-----------------------------------------------------------------------------*
   Local mVar , i

   mVar := '_' + FormName

   i := &mVar
   If i == 0
      Return ''
   EndIf

Return ( _HMG_aFormtype [ &mVar ] )
*-----------------------------------------------------------------------------*
Function _IsWindowActive ( FormName )
*-----------------------------------------------------------------------------*
   Local mVar , i

   mVar := '_' + FormName

   If Type ( mVar ) <> 'U'
      i := &mVar
      If i == 0
         Return .F.
      EndIf
      Return ( _HMG_aFormActive [ &mVar ] )
   EndIf

Return .F.
*-----------------------------------------------------------------------------*
Function _IsWindowDefined ( FormName )
*-----------------------------------------------------------------------------*
   Local mVar , i

   mVar := '_' + FormName

   If Type ( mVar ) <> 'U'
      i := &mVar
      If i == 0
         Return .F.
      EndIf
      Return ( .Not. ( _HMG_aFormDeleted [ &mVar ] ) )
   EndIf

Return .F.
*-----------------------------------------------------------------------------*
Function GetFormName (FormName)
*-----------------------------------------------------------------------------*
   Local mVar , i
   if _HMG_BeginWindowMDIActive
      if GetActiveMdiHandle() == 0
         Return _HMG_MainClientMDIName
      else
         Return _GetWindowProperty ( GetActiveMdiHandle(), "PROP_FORMNAME" )
      endif
   endif

   mVar := '_' + FormName

   i := &mVar
   If i == 0
      Return ''
   EndIf

Return ( _HMG_aFormNames [ &mVar ] )
*-----------------------------------------------------------------------------*
Function GetFormToolTipHandle (FormName)
*-----------------------------------------------------------------------------*
   Local mVar , i
   if _HMG_BeginWindowMDIActive
      FormName := GetFormName (FormName)
   endif

   mVar := '_' + FormName

   i := &mVar
   If i == 0
      Return 0
   EndIf

Return ( _HMG_aFormToolTipHandle [ &mVar ] )
*-----------------------------------------------------------------------------*
Function GetFormHandle (FormName)
*-----------------------------------------------------------------------------*
   Local mVar , i

   mVar := '_' + FormName

   i := &mVar
   if i == 0
      Return 0
   endif

Return ( _HMG_aFormHandles [ &mVar ] )

*-----------------------------------------------------------------------------*
Function ReleaseAllWindows ()
*-----------------------------------------------------------------------------*
   Local i , FormCount , ControlCount

   If _HMG_ThisEventType == 'WINDOW_RELEASE'
      MsgMiniGuiError("Release a window in its own ON RELEASE procedure or release the Main Window in any ON RELEASE procedure is not allowed. Program terminated" )
   EndIf

   FormCount := Len (_HMG_aFormHandles)

   For i = 1 to FormCount
      if  _HMG_aFormActive [i] == .t.

         _DoWindowEventProcedure ( _HMG_aFormReleaseProcedure [i] , i , 'WINDOW_RELEASE' )

         if .Not. Empty ( _HMG_aFormNotifyIconName [i] )
            _HMG_aFormNotifyIconName [i] := ''
            ShowNotifyIcon ( _HMG_aFormhandles [i] , .F., NIL, NIL )
         EndIf

      Endif

   Next i

   If IsExtendedMenuStyleActive()
      // Release OwnerDraw Main Menu
      _OnDestroyMenu ( GetMenu( _HMG_MainHandle ) )
   EndIf

   ControlCount := Len ( _HMG_aControlHandles )

   For i := 1 To ControlCount
      If _HMG_aControlType [i] == 'HOTKEY'
         ReleaseHotKey ( _HMG_aControlParentHandles [i] , _HMG_aControlIds [i] )
      ElseIf _HMG_aControlType [i] == 'FONT'
         _EraseFontDef (i)
      EndIf
   Next i

   dbcloseall()

   ExitProcess(0)

Return Nil

*-----------------------------------------------------------------------------*
Function _ReleaseWindow ( FormName )
*-----------------------------------------------------------------------------*
   Local FormCount , b , i , x

   b := _HMG_InteractiveClose
   _HMG_InteractiveClose := 1   // Interactive Close event is disabled

   If .Not. _IsWindowDefined (Formname)
      MsgMiniGuiError("Window: "+ FormName + " is not defined. Program terminated" )
   Endif

   If .Not. _IsWindowActive (Formname)
      MsgMiniGuiError("Window: "+ FormName + " is not active. Program terminated" )
   Endif

   If _HMG_ThisEventType == 'WINDOW_RELEASE'
      If GetFormIndex (FormName) == _HMG_ThisIndex
         MsgMiniGuiError("Release a window in its own ON RELEASE procedure or release the Main Window in any ON RELEASE procedure is not allowed. Program terminated" )
      EndIf
   EndIf

   * If the window to release is the main application window, release all
   * windows command will be executed

   If GetWindowType (FormName) == 'A'

      If _HMG_ThisEventType == 'WINDOW_RELEASE'
         MsgMiniGuiError("Release a window in its own ON RELEASE procedure or release the Main Window in any ON RELEASE procedure is not allowed. Program terminated" )
      Else
         ReleaseAllWindows()
      EndIf

   EndIf

   i := GetFormIndex ( Formname )

   * Release Window

   if _hmg_aformtype [i] == 'M' .and. _hmg_activemodalhandle <> _hmg_aformhandles [i]

      EnableWindow ( _hmg_aformhandles [i] )
      PostMessage( _hmg_aformhandles [i], WM_CLOSE, 0, 1 )

   Else

      FormCount := Len (_HMG_aFormHandles)

      For x := 1 To FormCount
         If _hmg_aFormParentHandle [x] == _hmg_aformhandles [i]
            _hmg_aFormParentHandle [x] := _hmg_MainHandle
         EndIf
      Next x

      EnableWindow ( _hmg_aformhandles [i] )
      PostMessage( _hmg_aformhandles [i], WM_CLOSE, 0, 1 )

   EndIf

   _HMG_InteractiveClose := b

Return Nil

*-----------------------------------------------------------------------------*
Function _ShowWindow ( FormName, lProcessMessages )
*-----------------------------------------------------------------------------*
   Local i , FormCount , x
   Local ActiveWindowHandle

   Default lProcessMessages := .T. // GF 31/01/2008

   i := GetFormIndex ( FormName )

   if _HMG_aFormType [i] == "M"

      // Find Parent

      If _HMG_IsModalActive
         _hmg_aFormParentHandle [i] := _hmg_activemodalhandle
      Else

         ActiveWindowHandle := _hmg_UserWindowHandle
         if ascan ( _hmg_aFormHandles , ActiveWindowHandle ) == 0
            ActiveWindowHandle := _hmg_MainHandle
         endif
         _hmg_aFormParentHandle [i] := ActiveWindowHandle

      EndIf

      FormCount := Len ( _HMG_aFormHandles )

      for x := 1 to FormCount
         if x <> i .and. _HMG_aFormType [x] != 'X'
            DisableWindow ( _HMG_aFormHandles[x] )
         endif
      next x

      if Len ( _HMG_aFormSplitChildList [i] ) > 0
         For x := 1 To Len ( _HMG_aFormSplitChildList [i] )
            EnableWindow ( _HMG_aFormhandles [ _HMG_aFormSplitChildList [i] [x] ] )
         Next x
      EndIf

      _HMG_IsModalActive := .T.
      _hmg_activemodalhandle := _HMG_aFormHandles[i]
      EnableWindow ( _HMG_aFormHandles[i] )

      If _SetFocusedSplitChild(i) == .f.
         _SetActivationFocus(i)
      EndIf

   EndIf

   ShowWindow ( _HMG_aFormHandles [i] )

   If lProcessMessages // GF 31/01/2008
      * Do Process Messages
      ProcessMessages()
   Endif

Return Nil

*-----------------------------------------------------------------------------*
Function _HideWindow ( FormName )
*-----------------------------------------------------------------------------*
   Local i

   i := GetFormIndex (FormName)

   if i > 0

      If IsWindowVisible ( _hmg_aformhandles [i] )

         if _HMG_aFormType [i] == 'M'
            if _hmg_activemodalhandle <> _hmg_aformhandles [i]
               MsgMiniGuiError("Non top modal windows can't be hide. Program terminated" )
            EndIf
         EndIf

         HideWindow ( _HMG_aFormHandles [i] )
         _hmg_OnHideFocusManagement (i)

      EndIf

   EndIf

Return Nil

*-----------------------------------------------------------------------------*
Function _CenterWindow ( FormName )
*-----------------------------------------------------------------------------*
   C_Center( GetFormHandle(FormName) )
Return Nil

// (JK) HMG Experimental 1.1. Build 14
*-----------------------------------------------------------------------------*
Function _IsChild( hNowActive, hWnd )
*-----------------------------------------------------------------------------*
   Local i := ascan(_HMG_aControlHandles, hWnd)
Return ( _HMG_aControlParenthandles [i] == hNowActive )

*-----------------------------------------------------------------------------*
Function _IsChildOfActiveWindow( hWnd )
*-----------------------------------------------------------------------------*
   LOCAL hNowActive := GetActiveWindow()
   LOCAL lRet := _IsChild(hNowActive, hWnd)
   LOCAL hParent

   Do While lRet
      hParent := _GetParent(hWnd)
      If hNowActive <> hParent
         If AND(GetWindowLong(hParent, GWL_STYLE), WS_CHILD) > 0
            hWnd := hParent
         Else
            lRet := .F.
         EndIf
      Else
         Exit
      EndIf
   EndDo

Return lRet

*-----------------------------------------------------------------------------*
Function _GetParent( hWnd )
*-----------------------------------------------------------------------------*
   Local i := ascan(_HMG_aControlHandles, hWnd), ParenthWnd
   if i > 0
      ParenthWnd := _HMG_aControlParenthandles[i]
   endif
Return ParenthWnd