/*----------------------------------------------------------------------------
MINIGUI - Harbour Win32 GUI library source code

Copyright 2002-2008 Roberto Lopez <harbourminigui@gmail.com>
http://harbourminigui.googlepages.com/

MDI window source code
(C)2005 Janusz Pora <januszpora@onet.eu>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along with
   this software; see the file COPYING. If not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA (or
   visit the web site http://www.gnu.org/).

   As a special exception, you have permission for additional uses of the text
   contained in this release of Harbour Minigui.

   The exception is that, if you link the Harbour Minigui library with other
   files to produce an executable, this does not by itself cause the resulting
   executable to be covered by the GNU General Public License.
   Your use of that executable is in no way restricted on account of linking the
   Harbour-Minigui library code into it.

   Parts of this project are based upon:

   "Harbour GUI framework for Win32"
   Copyright 2001 Alexander S.Kresin <alex@belacy.belgorod.su>
   Copyright 2001 Antonio Linares <alinares@fivetech.com>
   www - http://www.harbour-project.org

   "Harbour Project"
   Copyright 1999-2008, http://www.harbour-project.org/

   "WHAT32"
   Copyright 2002 AJ Wos <andrwos@aust1.net>

   "HWGUI"
   Copyright 2001-2007 Alexander S.Kresin <alex@belacy.belgorod.su>

---------------------------------------------------------------------------*/

#define WM_MOVE      3
#define WM_SIZE      5
#define WM_CLOSE        0x0010
#define WM_COMMAND      0x0111
#define WM_DESTROY      0x0002
#define WM_NOTIFY   78
#define WM_CREATE   1
#define WM_QUIT      18
#define WM_ACTIVATE   6
#define WM_SETFOCUS   7
#define WM_CHILDACTIVATE                0x0022

#define WM_KEYDOWN                      0x0100
#define WM_KEYUP                        0x0101
#define WM_CHAR                         0x0102

#define WM_WINDOWPOSCHANGING            0x0046
#define WM_WINDOWPOSCHANGED             0x0047

#define SIZE_MAXIMIZED   2
#define SIZEFULLSCREEN   2
#define SIZE_MAXSHOW   3
#define SIZE_MINIMIZED   1
#define SIZEICONIC   1
#define SIZE_RESTORED   0


#define WM_MDICREATE                    0x0220
#define WM_MDIDESTROY                   0x0221
#define WM_MDIACTIVATE                  0x0222
#define WM_MDIRESTORE                   0x0223
#define WM_MDINEXT                      0x0224
#define WM_MDIMAXIMIZE                  0x0225
#define WM_MDITILE                      0x0226
#define WM_MDICASCADE                   0x0227
#define WM_MDIICONARRANGE               0x0228
#define WM_MDIGETACTIVE                 0x0229

#define MDITILE_VERTICAL       0x0000
#define MDITILE_HORIZONTAL     0x0001
#define MDITILE_SKIPDISABLED   0x0002


#define CW_USEDEFAULT       0x80000000
#define DLGC_WANTALLKEYS   4
#define WM_GETDLGCODE    135

#include "minigui.ch"
#include "error.ch"

Memvar mVar
#ifdef _TSBROWSE_
   Memvar _TSB_aControlhWnd, _TSB_aControlObjects, _TSB_aClientMDIhWnd
#endif

*-----------------------------------------------------------------------------*
Function MdiEvents ( hWnd, nMsg, wParam, lParam )
*-----------------------------------------------------------------------------*
   Local i, x, ControlCount
#ifdef _TSBROWSE_
   Local oGet
#endif

   do case

#ifdef _TSBROWSE_
   ***********************************************************************
   case nMsg == WM_KEYDOWN  .OR. nMsg == WM_KEYUP
   ***********************************************************************

      IF  Type('_TSB_aControlhWnd') == 'A' .and. Len(_TSB_aControlhWnd) > 0
         oGet := GetObjectByClientMDI(hWnd)
         IF ValType(oGet) == 'O'
            oGet:HandleEvent ( nMsg, wParam, lParam )
         ENDIF
      ENDIF

#endif
   ***********************************************************************
   case nMsg == WM_MDIACTIVATE
   ***********************************************************************

      if _HMG_MdiChildActive == .f.

         i := aScan ( _HMG_aFormHandles , hWnd )
         if i > 0
            _DoWindowEventProcedure ( _HMG_aFormClickProcedure [i] , i )
         endif

      endif

   ***********************************************************************
   case nMsg == WM_GETDLGCODE
   ***********************************************************************

      Return DLGC_WANTALLKEYS

   ***********************************************************************
   case nMsg == WM_WINDOWPOSCHANGED .or. nMsg ==  WM_WINDOWPOSCHANGING
   ***********************************************************************

      Return 0

   ***********************************************************************
   case nMsg == WM_MOVE
   ***********************************************************************

      if _HMG_MdiChildActive == .f.

         i := aScan ( _HMG_aFormhandles , hWnd )

         if i > 0
            _DoWindowEventProcedure ( _HMG_aFormMoveProcedure  [i] , i )
         EndIf

      endif

   ***********************************************************************
   case nMsg == WM_SIZE
   ***********************************************************************

      if _HMG_MdiChildActive == .f.

         i := aScan ( _HMG_aFormHandles , hWnd )
         if i > 0

            If wParam == SIZE_MAXIMIZED

               _DoWindowEventProcedure ( _HMG_aFormMaximizeProcedure [i] , i )

            ElseIf wParam == SIZE_MINIMIZED

               _DoWindowEventProcedure ( _HMG_aFormMinimizeProcedure [i] , i )

            ElseIf wParam == SIZE_RESTORED .AND. !IsWindowSized( hWnd )

               _DoWindowEventProcedure ( _HMG_aFormRestoreProcedure [i] , i )

            Else

               _DoWindowEventProcedure ( _HMG_aFormSizeProcedure [i] , i )

            EndIf

         endif

      endif

   ***********************************************************************
   case nMsg == WM_CLOSE
   ***********************************************************************

      If GetEscapeState() < 0
         Return (1)
      EndIf

      Return _MdiChildClose(hWnd)

   ***********************************************************************
   case nMsg == WM_DESTROY
   ***********************************************************************

      ControlCount := Len (_HMG_aControlHandles)

      i := Ascan ( _HMG_aFormhandles , hWnd )

      if i > 0
         * Remove Child Window Properties

         _RemoveProperty( hWnd, "PROP_CFILE" )
         _RemoveProperty( hWnd, "PROP_FORMNAME" )
         _RemoveProperty( hWnd, "PROP_MODIFIED" )

          * Remove Child Controls

         For x:=1 To ControlCount
            if _HMG_aControlParentHandles [x] == hWnd
               _EraseControl(x,i)
            EndIf
         Next x

         * Update Form Index Variable

         mVar := '_' + _HMG_aFormNames [i]
         if type ( mVar ) != 'U'
            __MVPUT ( mVar , 0 )
         EndIf

         _HMG_aFormDeleted       [i]   := .T.
         _HMG_aFormhandles       [i]   := 0
         _HMG_aFormNames         [i]   := ""
         _HMG_aFormActive        [i]   := .f.
         _HMG_aFormType          [i]   := ""
         _HMG_aFormParenthandle      [i]   := 0
         _HMG_aFormInitProcedure      [i]   := ""
         _HMG_aFormReleaseProcedure   [i]   := ""
         _HMG_aFormToolTipHandle      [i]   := 0
         _HMG_aFormContextMenuHandle    [i]   := 0
         _HMG_aFormMouseDragProcedure   [i]   := ""
         _HMG_aFormSizeProcedure    [i]   := ""
         _HMG_aFormClickProcedure     [i]   := ""
         _HMG_aFormMouseMoveProcedure   [i]   := ""
         _HMG_aFormMoveProcedure   [i]   := ""
         _HMG_aFormDropProcedure   [i]   := ""
         _HMG_aFormBkColor      [i]   := Nil
         _HMG_aFormPaintProcedure   [i]   := ""
         _HMG_aFormNoShow      [i]   := .F.
         _HMG_aFormNotifyIconName   [i] := ''
         _HMG_aFormNotifyIconToolTip   [i] := ''
         _HMG_aFormNotifyIconLeftClick   [i] := ''
         _HMG_aFormNotifyIconDblClick   [i] := ''
         _HMG_aFormReBarHandle      [i] := 0
         _HMG_aFormNotifyMenuHandle   [i] := 0
         _HMG_aFormBrowseList      [i] := {}
         _HMG_aFormSplitChildList   [i] := {}
         _HMG_aFormVirtualHeight      [i] := 0
         _HMG_aFormGotFocusProcedure   [i] := ""
         _HMG_aFormLostFocusProcedure   [i] := ""
         _HMG_aFormVirtualWidth      [i] := 0
         _HMG_aFormFocused      [i] := .f.
         _HMG_aFormScrollUp      [i] := ""
         _HMG_aFormScrollDown      [i] := ""
         _HMG_aFormScrollLeft      [i] := ""
         _HMG_aFormScrollRight     [i] := ""
         _HMG_aFormHScrollBox      [i] := ""
         _HMG_aFormVScrollBox      [i] := ""
         _HMG_aFormBrushHandle      [i] := 0
         _HMG_aFormFocusedControl   [i] := 0
         _HMG_aFormGraphTasks      [i] := {}
         _HMG_aFormMaximizeProcedure   [i] := Nil
         _HMG_aFormMinimizeProcedure   [i] := Nil
         _HMG_aFormRestoreProcedure   [i] := Nil
         _HMG_aFormAutoRelease      [i] := .F.
         _HMG_aFormInteractiveCloseProcedure [i] := ""
         _HMG_aFormMinMaxInfo [i] := {}
         _HMG_aFormActivateId [i] := 0

         _HMG_InteractiveCloseStarted := .F.

      Endif

   otherwise

      If nMsg != WM_CREATE .and. nMsg != WM_CLOSE .and. nMsg != WM_DESTROY
         Events ( hWnd, nMsg, wParam, lParam )
      Endif

   endcase

Return (0)

*-----------------------------------------------------------------------------*
Function _DefineChildMDIWindow ( FormName, x, y, w, h, nominimize, nomaximize, nocaption, novscroll, nohscroll, title, FontName, FontSize,;
   initprocedure, ReleaseProcedure, ClickProcedure, GotFocus, LostFocus, SizeProcedure, maximizeprocedure, minimizeprocedure, focused, cursor, InteractiveCloseProcedure )
*-----------------------------------------------------------------------------*
   Local i , htooltip , mVar , ParentForm , k , aRGB := { -1 , -1 , -1 }
   Local FormHandle , ChildIndex

   if valtype(FormName) == "U"
      FormName := _HMG_TempWindowName
   endif

   ChildIndex := _HMG_ActiveMDIChildIndex + 1

   FormName += '_'+alltrim(str(ChildIndex))

   i := ascan ( _HMG_aFormType , 'A' )

   if i <= 0
      MsgMiniGuiError("Main Window Not Defined. Program Terminated" )
   Endif

   If _IsWindowDefined (FormName)
      MsgMiniGuiError("Window: "+ FormName + " already defined. Program Terminated" )
   endif

   If _HMG_BeginWindowMDIActive == .F.
      MsgMiniGuiError("MdiChild Windows Can be Defined Only Inside MdiWindow. Program terminated" )
   EndIf

   if valtype(x) == "U"
      x := CW_USEDEFAULT
   Endif
   if valtype(y) == "U"
      y := CW_USEDEFAULT
   Endif
   if valtype(w) == "U"
      w := CW_USEDEFAULT
   Endif
   if valtype(h) == "U"
      h := CW_USEDEFAULT
   Endif

   if valtype(FontName) == "U"
      _HMG_ActiveFontName := ""
   Else
      _HMG_ActiveFontName := FontName
   Endif

   if valtype(FontSize) == "U"
      _HMG_ActiveFontSize := 0
   Else
      _HMG_ActiveFontSize := FontSize
   Endif

   _HMG_MdiChildActive := .t.

   ParentForm := _HMG_MainClientMDIName

   mVar := '_' + FormName

   _HMG_ActiveFormName := FormName
   _HMG_BeginWindowActive := .T.

   i := GetFormIndex ( ParentForm )
   if i > 0

      Formhandle := InitMdiChildWindow (_HMG_MainClientMDIHandle,Title, x, y, w , h, nominimize, nomaximize, nocaption, novscroll, nohscroll)

      _SetWindowProperty( Formhandle, "PROP_CFILE", if(Title=="","No Title",Title) )
      _SetWindowProperty( Formhandle, "PROP_FORMNAME", FormName )
      _SetWindowProperty( Formhandle, "PROP_MODIFIED" , .f.)

      // JP MDI Background
      aRGB := _HMG_aFormBkColor [i]
      // End
      if Valtype ( cursor ) != "U"
         SetWindowCursor( Formhandle , cursor )
      EndIf

   EndIf

   htooltip := InitToolTip( FormHandle , _SetToolTipBalloon() , _lSetToolTip() )  //JP MDI ToolTip
   //JP MDI Background
   IF aRGB[1] != -1
      SetWindowBackground( Formhandle, PaintBkGnd( Formhandle, aRGB ) )
   endif
   //JP End
   k := ascan ( _HMG_aFormDeleted , .T. )
   _HMG_ActiveMDIChildIndex := ChildIndex

   if k > 0

      Public &mVar. := k

      _HMG_aFormNames  [k] :=  FormName
      _HMG_aFormHandles  [k] :=  FormHandle
      _HMG_aFormActive  [k] :=  .f.
      _HMG_aFormType  [k] :=  'Y'
      _HMG_aFormParentHandle  [k] :=  GetFormHandle(ParentForm)
      _HMG_aFormReleaseProcedure  [k] :=  ReleaseProcedure
      _HMG_aFormInitProcedure  [k] :=  initprocedure
      _HMG_aFormToolTipHandle  [k] :=  hToolTip
      _HMG_aFormContextMenuHandle  [k] :=  0
      _HMG_aFormMouseDragProcedure  [k] :=  ""
      _HMG_aFormSizeProcedure  [k] :=  SizeProcedure
      _HMG_aFormClickProcedure  [k] :=  ClickProcedure
      _HMG_aFormMouseMoveProcedure  [k] :=  ""
      _HMG_aFormMoveProcedure  [k] :=  ""
      _HMG_aFormDropProcedure  [k] :=  ""
      _HMG_aFormDeleted  [k] := .f.
      _HMG_aFormBkColor  [k] := aRGB		//JP MDI Background
      _HMG_aFormPaintProcedure  [k] :=  ""
      _HMG_aFormNoShow   [k] := .f.
      _HMG_aFormNotifyIconName  [k] :=  ""
      _HMG_aFormNotifyIconToolTip    [k] :=  ""
      _HMG_aFormNotifyIconLeftClick  [k] :=  ""
      _HMG_aFormNotifyIconDblClick  [k] :=  ""
      _HMG_aFormGotFocusProcedure  [k] := gotfocus
      _HMG_aFormLostFocusProcedure [k] := lostfocus
      _HMG_aFormReBarHandle    [k] :=  0
      _HMG_aFormNotifyMenuHandle [k] :=     0
      _HMG_aFormBrowseList    [k] :=     {}
      _HMG_aFormSplitChildList [k] :=     {}
      _HMG_aFormVirtualHeight    [k] := 0
      _HMG_aFormVirtualWidth    [k] := 0
      _HMG_aFormFocused    [k] :=  Focused
      _HMG_aFormScrollUp    [k] := ""
      _HMG_aFormScrollDown    [k] := ""
      _HMG_aFormScrollLeft    [k] := ""
      _HMG_aFormScrollRight   [k] := ""
      _HMG_aFormHScrollBox    [k] := ""
      _HMG_aFormVScrollBox    [k] := ""
      _HMG_aFormBrushHandle    [k] := 0
      _HMG_aFormFocusedControl [k] := 0
      _HMG_aFormGraphTasks    [k] :=  {}
      _HMG_aFormMaximizeProcedure [k] := maximizeprocedure
      _HMG_aFormMinimizeProcedure [k] := minimizeprocedure
      _HMG_aFormRestoreProcedure   [k] := ""
      _HMG_aFormAutoRelease [k] :=  .T.
      _HMG_aFormInteractiveCloseProcedure [k] :=  InteractiveCloseProcedure
      _HMG_aFormMinMaxInfo [k] := {}
      _HMG_aFormActivateId [k] := 0

   Else

      Public &mVar. := Len(_HMG_aFormNames) + 1

      aAdd ( _HMG_aFormNames , FormName )
      aAdd ( _HMG_aFormHandles , FormHandle )
      aAdd ( _HMG_aFormActive , .f. )
      aAdd ( _HMG_aFormType , 'Y' )
      aAdd ( _HMG_aFormParentHandle , GetFormHandle(ParentForm) )
      aAdd ( _HMG_aFormReleaseProcedure , ReleaseProcedure  )
      aAdd ( _HMG_aFormInitProcedure , initprocedure  )
      aAdd ( _HMG_aFormToolTipHandle , hToolTip )
      aAdd ( _HMG_aFormContextMenuHandle , 0 )
      aAdd ( _HMG_aFormMouseDragProcedure , "" )
      aAdd ( _HMG_aFormSizeProcedure , SizeProcedure )
      aAdd ( _HMG_aFormClickProcedure , ClickProcedure )
      aAdd ( _HMG_aFormMouseMoveProcedure , "" )
      aAdd ( _HMG_aFormMoveProcedure , "" )
      aAdd ( _HMG_aFormDropProcedure , "" )
      aAdd ( _HMG_aFormDeleted, .f. )
      aAdd ( _HMG_aFormBkColor, Nil )
      aAdd ( _HMG_aFormPaintProcedure , "" )
      aAdd ( _HMG_aFormNoShow , .f. )
      aAdd ( _HMG_aFormNotifyIconName , "" )
      aAdd ( _HMG_aFormNotifyIconToolTip , "" )
      aAdd ( _HMG_aFormNotifyIconLeftClick , "" )
      aAdd ( _HMG_aFormNotifyIconDblClick  , "" )
      aAdd ( _HMG_aFormGotFocusProcedure   , gotfocus )
      aAdd ( _HMG_aFormLostFocusProcedure  , lostfocus )
      aAdd ( _HMG_aFormReBarHandle       , 0 )
      aAdd ( _HMG_aFormNotifyMenuHandle  , 0 )
      aAdd ( _HMG_aFormBrowseList        , {} )
      aAdd ( _HMG_aFormSplitChildList    , {} )
      aAdd ( _HMG_aFormVirtualHeight     , 0 )
      aAdd ( _HMG_aFormVirtualWidth      , 0 )
      aAdd ( _HMG_aFormFocused           , Focused )
      aAdd ( _HMG_aFormScrollUp          , "" )
      aAdd ( _HMG_aFormScrollDown        , "" )
      aAdd ( _HMG_aFormScrollLeft        , "" )
      aAdd ( _HMG_aFormScrollRight       , "" )
      aAdd ( _HMG_aFormHScrollBox        , "" )
      aAdd ( _HMG_aFormVScrollBox        , "" )
      aAdd ( _HMG_aFormBrushHandle       , 0 )
      aAdd ( _HMG_aFormFocusedControl    , 0 )
      aAdd ( _HMG_aFormGraphTasks        , {} )
      aAdd ( _HMG_aFormMaximizeProcedure , maximizeprocedure )
      aAdd ( _HMG_aFormMinimizeProcedure , minimizeprocedure )
      aAdd ( _HMG_aFormRestoreProcedure  ,  "" )
      aAdd ( _HMG_aFormAutoRelease       , .T. )
      aAdd ( _HMG_aFormInteractiveCloseProcedure , InteractiveCloseProcedure )
      aAdd ( _HMG_aFormMinMaxInfo , {} )
      aAdd ( _HMG_aFormActivateId , 0 )

   EndIf

Return (FormHandle)

*-----------------------------------------------------------------------------*
Function _EndMdiChildWindow ()
*-----------------------------------------------------------------------------*
   _ActivateMdiWindow ( _HMG_ActiveFormName )
   _HMG_ActiveFormName := _HMG_ActiveFormNameBak
   _HMG_MdiChildActive := .f.

Return Nil

*-----------------------------------------------------------------------------*
Function _ActivateMdiWindow ( FormName )
*-----------------------------------------------------------------------------*
   Local i

   If .Not. _IsWindowDefined (Formname)
      MsgMiniGUIError("Window: "+ FormName + " is not defined. Program terminated" )
   Endif

   If _IsWindowActive (FormName)
      MsgMiniGUIError("Window: "+ FormName + " already active. Program terminated" )
   Endif

   i := GetFormIndex ( FormName )

   if _HMG_aFormType [i] == "M"

      _ShowWIndow ( _HMG_aFormNames [i] )

      _SetActivationFlag(i)
      _ProcessInitProcedure(i)
      _RefreshDataControls(i)

   Else

      If _HMG_IsModalActive
         MsgMiniGUIError("Non Modal WIndows can't be activated when a modal window is active. " + Formname +" Program Terminated" )
      endif

      If _HMG_aFormNoShow [i] == .F.
         ShowWindow( GetFormHandle(FormName) )
      EndIf

      _SetActivationFlag(i)
      _ProcessInitProcedure(i)
      _RefreshDataControls(i)

      If _SetFocusedSplitChild(i) == .f.
         _SetActivationFocus(i)
      endif

   Endif

Return Nil

*-----------------------------------------------------------------------------*
Function _MdiChildClose ( hWnd )
*-----------------------------------------------------------------------------*
   Local i, xRetVal

   i := Ascan ( _HMG_aFormhandles , hWnd )

   if i > 0

         * Process Interactive Close Event / Setting

      If ValType ( _HMG_aFormInteractiveCloseProcedure [i] ) == 'B'
         xRetVal := _DoWindowEventProcedure ( _HMG_aFormInteractiveCloseProcedure [i] , i , 'WINDOW_ONINTERACTIVECLOSE' )
         If ValType (xRetVal) == 'L'
            If !xRetVal
               return 1
            EndIf
         EndIf
      EndIf

      Do Case
      Case _HMG_InteractiveClose == 0
         MsgStop ( _HMG_MESSAGE [3] )
         return 1
      Case _HMG_InteractiveClose == 2
         If ! MsgYesNo ( _HMG_MESSAGE [1] , _HMG_MESSAGE [2] )
            return 1
         EndIf
      Case _HMG_InteractiveClose == 3
         if _HMG_aFormType [i] == 'A'
            If ! MsgYesNo ( _HMG_MESSAGE [1] , _HMG_MESSAGE [2] )
               return 1
            EndIf
         EndIf
      EndCase

      if valtype( _HMG_aFormReleaseProcedure [i] )=='B'
         _HMG_InteractiveCloseStarted := .T.
         _DoWindowEventProcedure ( _HMG_aFormReleaseProcedure [i] , i , 'WINDOW_RELEASE' )
      EndIf

   EndIf

Return 0

*-----------------------------------------------------------------------------*
Function ActivateMdiChildWindow ( ChildName )
*-----------------------------------------------------------------------------*
   Local n

   For n := 1 to len(_HMG_aFormHandles)
      if _HMG_aFormType [n] ==  'Y'
         if ChildName == _GetWindowProperty(_HMG_aFormHandles [n], "PROP_FORMNAME")
            _MdiWindowsActivate(_HMG_aFormHandles [n])
         endif
      endif
   Next

Return Nil

*-----------------------------------------------------------------------------*
Function _CloseActiveMdi()
*-----------------------------------------------------------------------------*
   Local ChildHandle := GetActiveMdiHandle()

   if _MdiChildClose(ChildHandle) == 0
      DestroyActiveMdi(ChildHandle)
   endif

Return Nil

*-----------------------------------------------------------------------------*
Function _MdiChildCloseAll()
*-----------------------------------------------------------------------------*
   Local ChildHandle, n

   For n := 1 to len(_HMG_aFormHandles)
      if _HMG_aFormType  [n] == 'Y'
         ChildHandle := _HMG_aFormHandles [n]
         if _MdiChildClose(ChildHandle) == 0
            DestroyActiveMdi(ChildHandle)
         endif
      endif
   Next

Return Nil

*-----------------------------------------------------------------------------*
Function _MdiChildRestoreAll()
*-----------------------------------------------------------------------------*
   Local n

   For n := 1 to len(_HMG_aFormHandles)
      if _HMG_aFormType [n] == 'Y'
         _MdiWindowsRestore( _HMG_aFormHandles [n] )
      endif
   Next

Return Nil

*-----------------------------------------------------------------------------*
Function GetActiveMdiHandle()
*-----------------------------------------------------------------------------*
Return if(_HMG_MainClientMDIHandle > 0, SendMessage(_HMG_MainClientMDIHandle, WM_MDIGETACTIVE, 0, 0), 0)

*-----------------------------------------------------------------------------*
Function DestroyActiveMdi(hwndCln)
*-----------------------------------------------------------------------------*
   SendMessage(_HMG_MainClientMDIHandle, WM_MDIDESTROY, hwndCln, 0)
Return Nil

*-----------------------------------------------------------------------------*
Function _MdiWindowsTile(lVert)
*-----------------------------------------------------------------------------*
   LOCAL Style := if( lVert, MDITILE_VERTICAL, MDITILE_HORIZONTAL )
   SendMessage(_HMG_MainClientMDIHandle, WM_MDITILE, Style, 0)
Return Nil

*-----------------------------------------------------------------------------*
Function _MdiWindowsCascade()
*-----------------------------------------------------------------------------*
   SendMessage(_HMG_MainClientMDIHandle, WM_MDICASCADE, 0, 0)
Return Nil

*-----------------------------------------------------------------------------*
Function _MdiWindowsIcons()
*-----------------------------------------------------------------------------*
   SendMessage(_HMG_MainClientMDIHandle, WM_MDIICONARRANGE, 0, 0)
Return Nil

*-----------------------------------------------------------------------------*
Function _MdiWindowsRestore(childhwnd)
*-----------------------------------------------------------------------------*
   SendMessage(_HMG_MainClientMDIHandle, WM_MDIRESTORE, childhwnd, 0)
Return Nil

*-----------------------------------------------------------------------------*
Function _MdiWindowsActivate(childhwnd)
*-----------------------------------------------------------------------------*
   if _HMG_MainClientMDIHandle > 0
      SendMessage(_HMG_MainClientMDIHandle, WM_MDIACTIVATE, childhwnd, 0)
   endif
Return Nil

#ifdef _TSBROWSE_
*-----------------------------------------------------------------------------*
FUNCTION GetObjectByClientMDI( hWnd )
*-----------------------------------------------------------------------------*
   LOCAL oWnd := 0, nPos
   IF ( nPos := AScan(_TSB_aClientMDIhWnd, hWnd ) ) > 0
      oWnd := _TSB_aControlObjects[ nPos ]
   ENDIF
RETURN oWnd

#endif