#include "MiniGUI.ch"
#include "Stock.ch"


Declare window wStock


/******
*
*       CallsTable()
*
*       Manipulation table for procedures/functions
*
*/

Procedure CallsTable
Memvar aOptions
Local aLangStrings := GetLangStrings( GET_CALLSTABLE_LANG, aOptions[ OPTIONS_LANGFILE ] )

Define window wConsole            ;
       At 132, 235                ;
       Width 380                  ;
       Height 350                 ;
       Title aLangStrings[ 1, 2 ] ;
       Icon 'STOCK'               ;
       Modal                      ;
       On init BuildList()

   @ 5, 5 EditBox edtConsole              ;
          Height ( wConsole.Height - 40 ) ;
          Width ( wConsole.Width - 20 )   ;
          ReadOnly

End Window

CenterInside( 'wStock', 'wConsole' )
DisableCloseButton( GetFormHandle( 'wConsole' ) )

Define window wCallsTable         ;
       At 132, 235                ;
       Width 500                  ;
       Height 350                 ;
       Title aLangStrings[ 1, 2 ] ;
       Icon 'STOCK'               ;
       Modal

   @ 5, 5 Grid grdList                                          ;
          Width 480                                             ;
          Height 310                                            ;
          Headers { aLangStrings[ 3, 2 ], aLangStrings[ 4, 2 ], ;
                    aLangStrings[ 5, 2 ], aLangStrings[ 6, 2 ]  ;
                  }                                             ;
          Widths { 130, 100, 130, 130 }                        ;
          Font 'Tahoma' Size 10

   On key Escape of wCallsTable Action wCallsTable.Release
   On key Alt+X  of wCallsTable Action ReleaseAllWindows()
       
End Window
CenterInside( 'wStock', 'wCallsTable' )

Activate window wConsole
Activate window wCallsTable

Return

****** End of CallsTable ******


/*****
*
*       BuildList()
*
*       Array of shared procedures/functions
*
*/

Static Procedure BuildList
Local nCount := wStock.grdContent.ItemCount, ;
      Cycle                                , ;
      cFile  := ''                         , ;
      cName                                , ;
      cType                                , ;
      aList  := {}

wCallsTable.grdList.DeleteAllItems
wCallsTable.grdList.DisableUpdate

For Cycle := 1 to nCount

    cName := RTrim( wStock.grdContent.Item( Cycle )[ 1 ] )
    cType := wStock.grdContent.Item( Cycle )[ 2 ]
    
    If !Empty( Left( cName, 1 ) )
       cFile := cName
    Endif
    
    If ( ( Upper( cType ) == 'PROCEDURE' ) .or. ;
         ( Upper( cType ) == 'FUNCTION'  ) .or. ;
         ( Upper( cType ) == 'HB_FUNC'   )      ;
       )
       AAdd( aList, { LTrim( cName ), cType, cFile, '' } )
    Endif

Next

// Fill a list

aList := FillList( aList )

AEval( aList, { | elem | wCallsTable.grdList.AddItem( elem ) } )

wCallsTable.grdList.ColumnsAutoFit
wCallsTable.grdList.EnableUpdate

wConsole.edtConsole.Value := ( wConsole.edtConsole.Value + 'Finished' + CRLF )
wConsole.Release

Return

****** End of BuildList ******


/******
*
*       FillList( aList ) --> aList
*
*       Fill a list
*
*/

Static Function FillList( aList )
Memvar aOptions
Local aFiles   := {}, ;
      aNewList := {}, ;
      aTmp          , ;
      nLen          , ;
      nLen1         , ;
      Cycle         , ;
      Cycle1        , ;
      oFile         , ;
      cString := '' , ;
      cMsg          , ;
      cName         , ;
      nPos    := 1

// Sort by function & path

ASort( aList, , , { | x, y | Upper( x[ 3 ] + x[ 1 ] ) < Upper( y[ 3 ] + y[ 1 ] ) } )
aTmp := AClone( aList )

// Build list of all program files

nLen := Len( aList )

For Cycle := 1 to nLen

  If !( aList[ Cycle, 3 ] == cString )
     cString := aList[ Cycle, 3 ]
     AAdd( aFiles, cString )
  Endif
  
Next

// Functions search

nLen  := Len( aFiles )
nLen1 := Len( aList )

For Cycle := 1 to nLen

   oFile := TFileRead() : New( aOptions[ OPTIONS_GETPATH ] + '\' + aFiles[ Cycle ] )
   
   oFile : Open()
   If !oFile : Error()

      // Check proceedures/functions entry
      // for each string of program file

      wConsole.edtConsole.Value := ( wConsole.edtConsole.Value + aFiles[ Cycle ] )
      cMsg := GetProperty( 'wConsole', 'edtConsole', 'Value' )
      
      Do while oFile : MoreToRead()
      
         cString := oFile : ReadLine()

         SetProperty( 'wConsole', 'edtConsole', 'Value', ( cMsg + ' - ' + LTrim( Str( nPos ) ) ) )
         DoMethod( 'wConsole', 'Show' )
         Do Events
         nPos ++
         
         For Cycle1 := 1 to nLen1
           
           // Search is not make in files with determined function
           
           If !( Upper( aList[ Cycle1, 3 ] ) == Upper( aFiles[ Cycle ] ) )
           
              cName := Upper( aList[ Cycle1, 1 ] )
           
              If ( ( ( cName + '(' ) $ Upper( cString ) ) .or. ;
                   ( ( cName + ' (' ) $ Upper( cString ) )     ;
                 )
           
                 If !Empty( nPos := AScan( aTmp, { | elem | Upper( elem[ 1 ] ) == cName } ) )
              
                    If Empty( aTmp[ nPos, 4 ] )
                       aTmp[ nPos, 4 ] := aFiles[ Cycle ]
                    Else
                       aTmp := ASize( aTmp, ( Len( aTmp ) + 1 ) )
                       AIns( aTmp, ( nPos + 1 ) )
                       aTmp[ nPos + 1 ] := { '', '', '', aFiles[ Cycle ] }
                    Endif
                 
                 Endif
              
              Endif
              
           Endif
           
         Next
         
      Enddo

      oFile : Close()

      cMsg += CRLF
      SetProperty( 'wConsole', 'edtConsole', 'Value', cMsg )
     
   Endif
   
Next

// Table compacting

nLen := Len( aTmp )

For Cycle := 1 to nLen

  If ( Cycle == 1 )
     AAdd( aNewList, { aTmp[ Cycle, 1 ], aTmp[ Cycle, 2 ], aTmp[ Cycle, 3 ], aTmp[ Cycle, 4 ] } )
  Else
  
     If !( Upper( aTmp[ Cycle, 4 ] ) == Upper( ATail( aNewList )[ 4 ] ) )
        AAdd( aNewList, { aTmp[ Cycle, 1 ], aTmp[ Cycle, 2 ], aTmp[ Cycle, 3 ], aTmp[ Cycle, 4 ] } )
     Endif
     
  Endif
  
Next

Return aNewList

****** End of FillList ******
