/*
 * Author: P.Chornyj <myorg63@mail.ru>
*/

#include <windows.h>

#if !defined( INVALID_FILE_ATTRIBUTES )
   #define INVALID_FILE_ATTRIBUTES  ( (DWORD) - 1 )
#endif
#if !defined( FILE_ATTRIBUTE_DEVICE )
   #define FILE_ATTRIBUTE_DEVICE 0x00000040
#endif

char  *_getenv( const char * );
char  *xstrcpy( char *, const char *, ... );
BOOL  fsFileExists( const char * );
BOOL  strEmpty( const char *, DWORD );
int   WaitRun( char *, WORD );

#define MPMB_NOLOCFILE     -101
#define MPMB_NOLOCREAD     -102
#define MPMB_NOBUILDFILE   -103

int PASCAL WinMain( HINSTANCE inst, HINSTANCE previnst, LPSTR cmdline, int cmdshow )
{
   DWORD    dwResult;
   char     *szTmpFolder = _getenv( ( const char * ) "TEMP" );
   char     *szLocFile;
   HANDLE   handle;
   DWORD    dwSize, dwNumberOfBytesRead;
   char     cBuildFile[MAX_PATH] = { 0 };

   szLocFile = xstrcpy( NULL, szTmpFolder, ( const char * ) "\\", ( const char * ) "_MpmBuild.txt", NULL );

   if( !fsFileExists(szLocFile) )
   {
      dwResult = MPMB_NOLOCFILE;
      goto end;
   }

   handle = CreateFile( szLocFile, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, (DWORD) NULL, NULL );

   if( handle == 0 )
   {
      dwResult = MPMB_NOLOCREAD;
      goto end;
   }

   dwSize = GetFileSize( handle, NULL );
   if( dwSize == 0xFFFFFFFF )
   {
      dwResult = MPMB_NOLOCREAD;
      CloseHandle( handle );
      goto end;
   }

   ReadFile( handle, cBuildFile, (dwSize <= MAX_PATH) ? dwSize : MAX_PATH, &dwNumberOfBytesRead, NULL );
   CloseHandle( handle );

   if( strEmpty(cBuildFile, dwNumberOfBytesRead) )
   {
      dwResult = MPMB_NOLOCREAD;
      goto end;
   }

   if( !fsFileExists(cBuildFile) )
   {
      dwResult = MPMB_NOBUILDFILE;
      goto end;
   }

   DeleteFile( szLocFile );

   dwResult = WaitRun( cBuildFile, 0 );

end:
   if( szTmpFolder )
   {
      free( ( void * ) szTmpFolder );
   }

   if( szLocFile )
   {
      free( ( void * ) szLocFile );
   }

   return dwResult;
}

/*
 */
char *_getenv( const char *pszName )
{
   DWORD size = GetEnvironmentVariableA( pszName, NULL, 0 );
   char  *pszBuffer = NULL;

   if( size != 0 )
   {
      pszBuffer = ( char * ) malloc( size );
      GetEnvironmentVariableA( pszName, pszBuffer, size );
   }

   return pszBuffer;
}

/*
 */
BOOL fsFileExists( const char *szFileName )
{
   DWORD dwAttr;
   BOOL  bExist;

   dwAttr = GetFileAttributesA( szFileName );

   bExist = ( dwAttr != INVALID_FILE_ATTRIBUTES ) && ( dwAttr & (FILE_ATTRIBUTE_DIRECTORY | FILE_ATTRIBUTE_DEVICE) ) == 0;

   return bExist;
}

/*
 */
BOOL strEmpty( const char *szText, ULONG ulLen )
{
   while( ulLen-- )
   {
      char  c = szText[ulLen];

      if( !isspace(c) )
      {
         return FALSE;
      }
   }

   return TRUE;
}

/*
 */
char *xstrcpy( char *szDest, const char *szSrc, ... )
{
   char     *szResult;
   va_list  va;

   if( szDest == NULL )
   {
      const char  *szSrcPtr = szSrc;
      ULONG       ulSize = 1;

      va_start( va, szSrc );
      while( szSrcPtr )
      {
         ulSize += strlen( szSrcPtr );
         szSrcPtr = va_arg( va, char * );
      }

      va_end( va );
      szDest = ( char * ) malloc( ulSize );
   }

   szResult = szDest;

   va_start( va, szSrc );
   while( szSrc )
   {
      while( *szSrc )
      {
         *szDest++ = *szSrc++;
      }

      szSrc = va_arg( va, char * );
   }

   *szDest = '\0';
   va_end( va );

   return szResult;
}

/*
 */
int WaitRun( char *cFile, WORD wShow )
{
   DWORD                dwExitCode;
   STARTUPINFO          stInfo = { 0 };
   PROCESS_INFORMATION  prInfo;
   BOOL                 bResult;

   stInfo.cb = sizeof( stInfo );
   stInfo.dwFlags = STARTF_USESHOWWINDOW;
   stInfo.wShowWindow = wShow;

   bResult = CreateProcess( NULL, cFile, NULL, NULL, TRUE, CREATE_NEW_CONSOLE | NORMAL_PRIORITY_CLASS, NULL, NULL, &stInfo, &prInfo );
   if( !bResult )
   {
      return( DWORD ) - 1L;
   }

   WaitForSingleObject( prInfo.hProcess, INFINITE );
   GetExitCodeProcess( prInfo.hProcess, &dwExitCode );

   CloseHandle( prInfo.hThread );
   CloseHandle( prInfo.hProcess );

   return dwExitCode;
}
